C                   FILE BONDER                   Last update: 23 April 1993
C                   -----------                                T.Stephen
C
C
C
C  S BONDL(ICMPD)              CALCULATES THE TRANSLATIONS OF ATOMS
C  S CALCD(I,J)                CALCULATES BOND DISTANCE XATOM I TO YATOM J
C                              TO BE STORED IN COMMON /DIST/
C  S PLACE(D,ER,DX,DY,DZ,...)  STORES BOND DISTANCE IN COMMON /DIST/
C  S ANGCHK(II,IANGLIM)        ELIMINATES APPROPRIATE BONDS AROUND ATOM II
C  S ANGLIM(ANGLEX)            ELIMINATES BONDS SEPARATED BY SMALL ANGLES
C                              FROM COMMON /DIST/
C  S SHIFT(M,N)                SHIFTS BONDS IN COMMON /DIST/
C  S INIT                      INITIALISES COMMON /DIST/
C  S WRLENG(IXATOM)            PRINTS BONDS IN /DIST/ AROUND IXATOM
C  S WRANG(IXATOM)             PRINTS ANGLES AMONG ABOVE BONDS
C  S WRTERM(IXATOM,IDFLAG,NT)  PRINTS BONDS AND ANGLES TO THE TERMINAL
C  S DISTAN(ALENG,X,Y,Z,I,J)   CORRECTS BOND LENGTHS FOR THERMAL MOTION (/DIST/)
C  S THECOI(W,X)               ISOTROPIC CONTRIBUTION TO ABOVE CORRECTION
C  S THECOA(W,L,LS,X,Y,Z)      ANISOTROPIC CONTRIBUTION TO ABOVE CORRECTION
C  S RNGERD                    READS INSTRUCION FOR CALCULATING RANGES
C  S BONDRD                    READS INSTRUCTION FOR DIRECTING THE CALCULATION
C                              OF BOND LENGTHS
C  F ICHECK(IXATOM)            CHECK CONDITONS FOR CALCULATING BOND LENGTHS
C  S INDRITE                   WRITES COMMON /BONDS/ TO THE BOND FILE
C  S INDCELL                   WRITES THE CELL PART TO THE BOND FILE
C  F IVLCHK(II,SUM)            CHECKS THE BOND VALENCE FOR BONDS AROUND ATOM II
C                              STORED IN /DIST/. THE VALENCE SUM IS RETURNED
C                              IN SUM.
C  S DISBON(II,JWRITE)         COPIES INFORMATION ABOUT ATOM II FROM /DIST/
C                              TO /BOND/
C  S ERANAL                    ANALYSES FOR POSSIBLE ERRORS IN STORED DATA
C  S INDATOM                   WRITES COMMON /IDENT/ TO THE BOND FILE
C  S STATUS(ICODE)             PRINTS STATUS FILE FOR BOND FILE
C  S DUMCON                    DUMMY ATOMS STORED IN /DUMMY/ ARE TRANSFERED
C                              TO /XATOM/
C  F IDISO(II)                 SETS THE DISORDER FLAG IDISO
C  S DIAGNO(II,SUM,NT)         THIS SUBROUTINE COMPILES A DIAGNOSTIC OUTPUT
C  S CLASSO(ICMPD)             SETS ICMPD=3 IF COMPOUND IS ORGANIC
c  s bondhelp                  information about bonds (honestly !)
C
C
C     THE FOLLOWING EXTERNALS ARE CALLED:
C
C  CHEMLB:  BOST, BONRNG, CHEMNUM, CLASS, FINDEL
C  DISTLB:  DUMMYH, ERRLN, RANGES, WKBOND, XLENG
C  MAINLB:  CALCY
C  MATHLB:  ANGLE, HXWORD, INCH, INTEGER, LIMIT, SCAN, SECOND, SQART, VMINV
C           UPCASE,READYN
C  IOLIB:   IFILE,HFILEN
C  SPELIB:  NALTNO
C  SYMLIB:  GETSTL
C
C
*************************************************************************
C
C
      SUBROUTINE BONDL(ICMPD)
C
C  *****SUBROUTINE*****         BONDL
C                               -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C  MODIFIED 82.6.30 BY OLIVER SLUPECKI
C  MODIFIED 83.9.22 BY I D BROWN
C  MODIFIED 86.11.25 BY LIZ BROWN
C
C
C   THIS SUBROUTINE FIRST CALCULATES THE RANGES IN WHICH THE Y
C   ATOMS LIE BY CREATING A REGION ONE MAXIMUM BOND LENGTH AROUND
C   THE VOLUME OF THE CELL CONTAINING THEN XATOMS.   A CALL TO
C   CALCY FILLS THE YATOM ARRAY WITH ALL THE ATOMS IN THIS REGION.
C   THEN, FOR EACH XATOM (INDEX=I) IT DETERMINES THE MAXIMUM BONDING
C   REGION AND, FOR EVERY YATOM (INDEX=J) IN THIS REGION IT CALCULATES
C   THE DISTANCE BY A CALL TO CALCD.   CALCD DETERMINES THE DISTANCE
C   AND IF IT IS IN RANGE, STORES IT.   AFTER LOOPING THROUGH ALL
C   THE ATOMS IN THE Y-ARRAY IT CALLS VARIOUS CHECKING AND OUTPUT
C   ROUTINES.
C   NOTE ON TERMINALS:
C   IF NTERM=6, THE PROGRAM OPERATES INTERACTIVELY. IN THIS CASE 
C   NOUT SHOULD BE SET TO THE UNIT FOR WRITING A FILE WHICH CAN BE
C   SENT TO THE PRINTER.  IF NTERM=0, ALL OUTPUT IS WRITTEN TO NOUT
C   WHICH SHOULD BY DEFAULT BE SET TO 6 BUT MAY HAVE OTHER VALUES.
C
C   CALLS  ANGLIM, BONCHK, CALCD, CALCY, CHEMNUM, CLASS, INIT, RANGES,
C          RNGERD, STATUS, WRANG, WRLENG, WRTERM, DIAGNO, READYN, WAIT
C
C   USES   ICHECK, LIMIT, SECOND, IDISO, IVLCHK, HFILEN
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP,NSTAT,NRANG
      include 'comatom.for'
      include 'combond.for'  
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/RCELL/RS(3),RAL,RBE,RGA,RVOL,RM(3,3)
C*      COMMON/RCELL/RS(3),RANGLE(3),RVOL,RM(3,3)
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      DIMENSION XM(3),XN(3),XMAX(3),XMIN(3)
*
C
C   ASSIGN CHEMICAL PERIOD AND GROUP AND TEST IF COMPOUND IS A METAL
C   (ICMPD = 1 FOR METAL, 2 FOR INORG, 3 FOR ORGANIC). IF INDEX IS NOT
C   SPECIFIED THEN SKIP PAST THIS TEST...
C
C   ICMPD .LT. 0  FOR ERRORS
C
C**********************************************************************
c   idflag.eq.1  display diagnostics
c   iwflag.eq.1  display distances on screen
c   ibwflg.eq.1  save bonds in /bonds/
      IDFLAG=0
      ISAVE=0
      iwflag=0
      IBWFLG=0
      IANGLIM=0
      IFFLAG=0
      IRFLAG=0
      GFORT='LIST'
C
C   **ITERACTIVE SESSION**
C   ----------------------
C
      IF(NTERM.NE.0) THEN
	WRITE (NTERM,350)
  350   FORMAT(/////,'         MOLECULAR GEOMETRY ',/,
     2           '         ------------------ ',/,/,
     3'  You are now in a module that will calculate the molecular',/,
     4' geometry. The bond lengths and angles will be calculated',/,
     5' and may be shown on the screen.  A fuller version is written',/,
     6' to the file DUMP.LIS for printing. The bond vectors can also',/,
     7' be saved for further calculations.',/)
  360   write(nout,380)
  380   format(///' ',
     +4X,'1. Help'/,
     +5X,'2. Calculate the bond lengths'/,
     +5X,/,
     +5X,'20. Return to menu level 3'//,
     +5X,' Enter your choice : ')
	ihcr = 0
	read(nin,390,err=360)ihcr
  390   format(I5)
	if(ihcr.le.0.or.ihcr.gt.20) go to 360
	if(ihcr.eq.1) call bondhelp
	if(ihcr.eq.2) go to 400
	if(ihcr.eq.20) go to 990
	go to 360 
  400   WRITE(NTERM,410)
  410   FORMAT(/,1X,'Do you want a diagnostic printout? (Y/N/E)')
	CALL READYN(*420,*425,*6,*400)
  420   IDFLAG = 1
  425   CONTINUE
	IF(NWRITE.NE.0) THEN
	  HNAME=HFILEN(NWRITE)
	  WRITE(NTERM,440)HNAME
  440     FORMAT(/,1X,'A full listing of the molecular',
     +                ' geometry is saved on ',A8,'.')
  510     NND=NWRITE
	  ISAVE=1
	ELSE
  520     WRITE(NTERM,530)
  530     FORMAT(/,1X,'Do you want a full listing of the molecular',
     +                ' geometry saved ? (Y/N/E)')
	  CALL READYN(*535,*570,*6,*520)
  535     ISAVE=1
	  WRITE(6,540)
  540     FORMAT(/,1X,'What file should it be written to?')
	  READ(NIN,550) HNAME
  550     FORMAT(A80)
	  CALL UPCASE(HNAME)
	  IF(HNAME.EQ.' '.OR.HNAME(1:1).EQ.'*') HNAME = 'OUTPUT'
  560     NUNIT=0
	  GSTAT='NEW'
	  NND=IFILE(HNAME,GFORT,GSTAT,NUNIT)
	  IF(NND.LE.0) THEN
	    HNAME='OUTPUT'
	    GO TO 560
	    ENDIF
	  WRITE(NND,9006)
	  ENDIF
  570   CONTINUE
  620   WRITE(NTERM,630)
  630   FORMAT(/,1X,'Do you want to save these bonds for further',
     +              ' use? (Y/N/E)')
	CALL READYN(*640,*660,*6,*620)
  640   IBWFLG=1
	nbon=0
  660   write(nterm,1660)
 1660   format(' Do you want the results shown on the screen Y/N/E ')
	iwflag=0
	call readyn(*670,*680,*6,*660)
  670   iwflag=1
  680   CONTINUE
	ENDIF
      IF(NTERM.EQ.0) WRITE(NOUT,9006)
 9006 FORMAT('1')
      N=3
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9002) SECOND(N)
 9002 FORMAT(' ++DEBUG++BONDL++9002++',F6.2,' SET BOND RANGES')
      ICMPD = 0
C
      CALL CHEMNUM
      CALL CLASS(ICMPD)
      IF(ICMPD.EQ.1.AND.NSTAT.NE.0) CALL STATUS(4)
      IF(ICMPD.NE.2.AND.NINDEX.NE.0) GO TO 900
C
C   SET RANGES FOR DISTANCE CALCULATION
C   -----------------------------------
C
      CALL RNGERD
C   FIND THE LONGEST BOND LENGTH REQUIRED
      RMAX=-10.
      DO 105 J=1,3
	XM(J)=-10.
  105   XN(J)=10.
      DO 110 I=1,NJ
	DO 110 J=1,NJ
  110     IF(TOP(I,J).GT.RMAX) RMAX=TOP(I,J)
C   FIND THE RANGE OCCUPIED BY X ATOMS
      DO 130 I=1,NX
	DO 120 J=1,3
	  IF(XX(J,I).GT.2.) GO TO 130
	  IF(XX(J,I).GT.XM(J)) XM(J)=XX(J,I)
  120     IF(XX(J,I).LT.XN(J)) XN(J)=XX(J,I)
  130   CONTINUE
C   SET THE RANGE FOR YATOMS
      DO 140 I=1,3
	XMAX(I)=XM(I)+RS(I)*RMAX
  140   XMIN(I)=XN(I)-RS(I)*RMAX
      N=3
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9003) SECOND(N)
 9003 FORMAT(' ++DEBUG++BONDL++9003++',F6.2,' START CALCULATING YATOMS')
      CALL CALCY(XMAX,XMIN)
      IF(NY.EQ.MY) THEN
	ICMPD = -1
	GOTO 6
	ENDIF
C
C   BEGIN OUTER LOOP I
C   ------------------
C
      N=3
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9004) SECOND(N)
 9004 FORMAT(' ++DEBUG++BONDL++9004++',F6.2,' CALCULATE DISTANCES')
      DO 5 I=1,NX
	CALL INIT
C   DO NOT CALCULATE DISTANCES AROUND DUMMY ATOMS
	if(npos(i).lt.0.or.igrp(i).lt.0) go to 5
	DO 10 K=1,3
   10     IF(XX(K,I).GT.8.) GOTO 5
C
C   GET RANGES FOR A GIVEN X-ATOM,AND THEN DEFINE THE LIMITS OF EACH
C   DIMENSION.
C
	RMAX = 0
	RMIN = 10.
	DO 20 J=1,NX
	  CALL RANGES(I,J,DMAX,DMIN)
	  IF(NDEBUG.NE.0) WRITE(NDEBUG,9011) I,J,DMAX,DMIN
 9011     FORMAT(' ***DEBUG***BONDL***9011***    FOR BONDS',2I3,
     +      ' DMAX =',F5.2,' DMIN =',F5.2)
	  IF(RMAX.GE.DMAX) GO TO 15
	  RMAX = DMAX
   15     IF(RMIN.LE.DMIN) GO TO 20
	  RMIN = DMIN
   20     CONTINUE
	IF(NDEBUG.NE.0) WRITE(NDEBUG,9010) I,RMAX,RMIN
 9010   FORMAT(' ***DEBUG***BONDL***9010***    FOR ATOM',I3,' RMAX =',
     +    F5.2, ' RMIN =',F5.2)
	IF(RMAX.LT..01)GO TO 5
	DO 25 K=1,3
	  XMIN(K)=XX(K,I)-RS(K)*RMAX
   25     XMAX(K)=XX(K,I)+RS(K)*RMAX
C
C  BEGIN INNER LOOP WITH J...
C  --------------------------
C
	DO 60 J=1,NY
	  IF(LIMIT(YX(1,J),XMAX,XMIN).EQ.0) GO TO 60
	  CALL CALCD(I,J)
   60     CONTINUE
C  
C   PRINT AND ANALYZE ENVIRONMENT OF ATOM I
C
	IF(NDEBUG.NE.0) WRITE(NDEBUG,9020)I,IFANG,ANGLX,NB,
     +    (IDIST(1,M),M=1,NB)
 9020   FORMAT(' ***DEBUG***BONDL***9020***    FOR ATOM ',I2,
     +  ' IFANG= ',I2,' ANGLX= ',F3.0,' NB= ',I2,
     +  ' IDIST(1,M)= ',10I3,/,'        ',15I3)
	CALL ANGCHK(I,IANGLIM)
	ICN(I)=NB+NH(I)+ND(I)
	IF(NTERM.EQ.0) THEN
	  IF(NINDEX.EQ.0) THEN
	    CALL WRLENG(I,IANGLIM)
	    IF(NOAN.EQ.0) CALL WRANG(I)
	    ENDIF
	  NT=NOUT
	  CALL DIAGNO(I,SUM,NT,IANGLIM)
	  ENDIF
	IF(NTERM.NE.0) THEN
	  NT=NTERM
	  IF(ISAVE.NE.0) THEN
	    NTNOUT=NOUT
	    NOUT = NND
	     CALL WRLENG(I,IANGLIM)
	    IF(NOAN.EQ.0) CALL WRANG(I)
	    IF(IDFLAG.NE.O.OR.NINDEX.NE.0) THEN
	      CALL DIAGNO(I,SUM,NOUT,IANGLIM)
	      ENDIF
	    NOUT=NTNOUT
	    ENDIF
	  IF(IDFLAG.NE.0) CALL DIAGNO(I,SUM,NT,IANGLIM)
	  if(iwflag.eq.1) then
	    CALL WRTERM(I,IDFLAG,NT)
	    CALL WAIT
	    endif
    4     CONTINUE
	  ENDIF
	IF(IBWFLG.NE.0.OR.NINDEX.NE.0) CALL DISBON(I,JWRITE)
C
C  PRINT ERROR FLAGS AFTER CALCULATIONS FOR ATOM I
C
      IF (NDEBUG.NE.0) WRITE(NDEBUG,9030) I,IFFLAG,IRFLAG
 9030 FORMAT(' ***DEBUG***BONDL***9030***    FOR ATOM ',I2,
     +         ' IFFLAG= ',I2,'IRFLAG= ',I2)
    5   CONTINUE
C********* Call Classo to identify organic compounds **********
      CALL CLASSO(ICMPD)
      N=3
    6 IF(NDEBUG.NE.0) WRITE(NDEBUG,9005) SECOND(N),nbon
 9005 FORMAT(' ++DEBUG++BONDL++9005++',F6.2,'Sec.  ',
     +       I4,' distances stored')
      RETURN
  900 IF(ICMPD.NE.1) GO TO 920
      WRITE(NOUT,9001)
 9001 FORMAT('    This Compound Is a Metal, No Distances Will Be ',
     +  'Calculated')
      GO TO 990
  920 IF(ICMPD.NE.2) GO TO 930
      WRITE(NOUT,9201)
 9201 FORMAT('    This Compound Is Inorganic, No Distances Will Be ',
     +  'Calculated')
      GO TO 990
  930 WRITE(NOUT,9301)
 9301 FORMAT('    This Compound Is Not Inorganic and Is Not a Metal,',
     +  ' No Distances Will Be Calculated')
  990 RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE CALCD(I,J)
C                =====
C
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C  REWRITTEN 83.9.20 BY I D BROWN
C  MODIFIED 85.5.6 BY DAVID KNIGHT
C
C  CALLS  BOST, ERRLN, GETSTL, PLACE, RANGES, VMINV
C
C  USES   XLENG
C
C  CALCULATES THE DISTANCE FROM XATOM I TO YATOM J
C  STORES THIS DISTANCE AS BOND IN COMMON /DIST/ IF
C  IT LIES WITHIN THE BOND WINDOW FOR THE ION PAIR I,J
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION ITRAN(3)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      include 'comatom.for'
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      DIMENSION ZX(3),XT(3),DX(3),KCENT(3),DY(3)
C
      CALL VMINV(DY,YX(1,J),XX(1,I))
      DIST=XLENG(DY(1),DY(2),DY(3),0)
      CALL RANGES(I,IYNUM(J),DMAX,DMIN)
      IF(DIST.LT.DMIN) THEN
C     DISTANCE IS TOO SHORT
C     ---------------------
	JJ=IYNUM(J)
	IF(OCCU(I)+OCCU(JJ).LE.1.1) THEN
C       DISORDER: IGNORE SHORT DISTANCE
C       -------------------------------
	  MINBON=MINBON+1
	  RETURN
	ELSE
	  CALL ERRLN(ERRR,DIST,DY(1),DY(2),DY(3),I,J)
	  IF(DIST.LT.0.01.OR.DIST+3.*ERRR.LT.DMIN) THEN
C         DISTANCE+ERROR IS STILL TOO SMALL (OR ATOM IS ITSELF)
C         ---------------------------------
	    MINBON=MINBON+1
	    IF(GELEM(I).NE.GELEM(JJ).OR.GDENT(I).NE.GDENT(JJ)
     +        .OR.DIST.GT.0.01) IFFLAG=IFFLAG+1
	    RETURN
	    ENDIF
	  ENDIF
	ENDIF
      IF(DIST.GT.DMAX) RETURN
C
C   CALCULATE ERRORS AND STORE BONDS
C   --------------------------------
C
      ERRL=0
      IF(NOBE.EQ.0) CALL ERRLN(ERRL,DIST,DY(1),DY(2),DY(3),I,J)
      CALL GETSTL(ITRAN,IYNUM(J),YX(1,J),IYSYM(J))
      CALL BOST(I,IYNUM(J),ST,EST,ERRL,DIST)
      CALL PLACE(DIST,ERRL,DY(1),DY(2),DY(3),IYNUM(J),IYSYM(J),
     +  ITRAN,ST,I)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE PLACE(DIST,ERRL,DX,DY,DZ,JATOM,JSYM,ITRANS,ST,I)
C                =====
C
C   WRITTEN 82.6.22 BY OLIVER SLUPECKI
C   MODIFIED 85.5.3 BY DAVID KNIGHT
C
C   CALLS SHIFT
C
C
C   THIS ROUTINE INSERTS THE BOND INFORMATION RECEIVED INTO THE BUFFERED
C   VALUES IN /DIST/.  IT IS INSERTED ACCORDING TO BOND LENGTH;
C   THE LONGER BONDS ARE SHIFTED DOWN
C
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
C
      DIMENSION ITRANS(3)
C
      DO 15 J=1,30
	IF(XDIST(1,J).GT.DIST) GOTO 20
   15 CONTINUE
      RETURN
   20 IF(J.EQ.30) GOTO 30
      KEND=NB-J+1
      K=0
   25 IF(K.EQ.KEND) GOTO 30
      M=NB-K+1
      N=NB-K
      K=K+1
      IF(M.GT.30) GOTO 25
      CALL SHIFT(M,N)
      GOTO 25
   30 IF(NB.LT.30) GOTO 35
      WRITE(NOUT,100)
      GOTO 40
   35 NB=NB+1
   40 XDIST(1,J)=DIST
      XDIST(2,J)=ERRL
      XDIST(3,J)=ST
      XDIST(4,J)=DX
      XDIST(5,J)=DY
      XDIST(6,J)=DZ
      IDIST(1,J)=JATOM
      IDIST(2,J)=JSYM
      IDIST(3,J)=ITRANS(1)
      IDIST(4,J)=ITRANS(2)
      IDIST(5,J)=ITRANS(3)
      RETURN
  100 FORMAT(' *ERROR*PLACE*  MORE THAN 30 BONDS GENERATED')
      END
C
*************************************************************************
C
      SUBROUTINE ANGCHK(II,IANGLIM)
C
C   *****SUBROUTINE*****       ANGCHK
C                              ------
C
C   WRITTEN 86.11.12 BY LIZ BROWN
C
C
C   THIS SUBROUTINE ELIMINATES BONDS AROUND ATOM II THAT ARE 
C   CLOSER THAN ANGLX DEGREES TO A SHORTER BOND WHEN APPROPRIATE
C   FLAGS ARE SET 
C   
C   IANGLIM=1 IF THE ELIMINATION ROUTINE (ANGLIM) HAS BEEN CALLED
C
C   CALLS ANGLIM
C
C   USES ICHECK 
C        MATHLB:INTEGER
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      include 'comatom.for'
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
*
      IANGLIM = 0
      IF(ICHECK(II).NE.0) THEN
	CALL ANGLIM(ANGLX)
	IANGLIM = 1
	ENDIF
C
C     IF BONDS ARE BEING CALCULATED
C     -----------------------------
C
      IF(IBND.NE.0)THEN
C
C       IF THE ATOM IS N(-2) CALL ANGLIM TO REMOVE N...H DISTANCE IN 
C       HYDRAZINE.
C       ------------------------------------------------------------
C
	JOX = INTEGER(OXID(II),0)
	IF(GELEM(II).EQ.'N'.AND.JOX.LT.0.AND.JOX.GT.-3) GO TO 100
C
C       IF ANY ATOM IS C(+2) THEN CALL ANGLIM ....
C       ------------------------------------------
C
	IF(GELEM(II).EQ.'C'.AND.JOX.EQ.2) GO TO 100
	DO 10 J=1,NB
	  IYN = IDIST(1,J)
   10     IF(GELEM(IYN).EQ.'C'.AND.INTEGER(OXID(IYN),0).EQ.2) GO TO 100
C
C       IF A LOW VALENCE MAIN GROUP CATION IS BONDED TO ANOTHER LOW
C       VALENCE MAIN GROUP CATION THEN CALL ANGLIM ....
C       -----------------------------------------------------------
C
	IF((NZA(2,II)-10-INTEGER(OXID(II),0)).GT.0.AND.
     +    OXID(II).GT.0) THEN
	  IYN = IDIST(1,J)    
	  DO 20 J=1,NB
	    IF((NZA(2,IYN)-10-INTEGER(OXID(IYN),0)).GT.0.
     +        AND.OXID(IYN).GT.0.) GO TO 100
   20       CONTINUE
	  ENDIF
	GO TO 200
  100   CALL ANGLIM(45.)
	IANGLIM=1
	ENDIF
  200 RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE ANGLIM(ANGLX)
C                ======
C
C  WRITTEN 82.6.28 BY OLIVER SLUPECKI
C  MODIFIED 85.5.8 BY DAVID KNIGHT
C
C  CALLS  ANGLE, SHIFT
C
C  THIS ROUTINE SCANS DOWN BUFFERED VALUES IN /DIST/.
C  AN BOND IS ELIMINATED FROM THE BUFFER IF IT
C  SUBTENDS AN ANGLE EQUAL TO OR LESS THAN A CRITICAL
C  ANGLE FROM A BOND ABOVE IT IN THE LIST.
C
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
C
      I=2
   15 IF(I.GT.NB)RETURN
      KEND = I-1
      DO 20 K=1,KEND
C
C  FOR DEBUGGING PRINT ANGLES:
	IF(NDEBUG.NE.0) WRITE(NDEBUG,111)IDIST(1,I),IDIST(1,K)
     +    ,ANGLE(XDIST(4,I),XDIST(4,K),AM),ANGLX
  111   FORMAT(' ***DEBUG***ANGLIM***111***',2I4,2F9.3)
C
	IF(ANGLE(XDIST(4,I),XDIST(4,K),AM).LE.ANGLX)GOTO 30
   20   CONTINUE
      I = I+1
      GOTO 15
   30 J=I
   35 IF(J.EQ.NB)GOTO 40
      M=J
      J=J+1
      CALL SHIFT(M,J)
      GOTO 35
   40 XDIST(1,J)=100.
      NB=NB-1
      GOTO 15
      END
C
C
*************************************************************************
C
C
      SUBROUTINE SHIFT(M,N)
C                =====
C
C  WRITTEN 82.6.22 BY OLIVER SLUPECKI
C  MODIFIED 85.5.3 BY DAVID KNIGHT
C
C  THIS ROUTINE COPIES VALUES IN /DIST/ FROM
C  ONE "ROW" TO ANOTHER
C
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
C
      IF(M.GT.30) WRITE(NOUT,100) M,N
  100 FORMAT(' ****SHIFT****    M=',I3,'   N=',I3,
     +'   Writing Outside Arrays in /DIST/')
      XDIST(1,M)=XDIST(1,N)
      DO 10 I=1,5
	XDIST(I,M)=XDIST(I,N)
	IDIST(I,M)=IDIST(I,N)
   10   CONTINUE
      XDIST(6,M)=XDIST(6,N)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE INIT
C                ====
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  SETS INITIAL VALUES FOR COMMON /DIST/
C
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      DO 15 I=1,30
	XDIST(1,I)=100.
C
   15 CONTINUE
      MINBON=-1
      NB=0
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE WRLENG(IXATOM,IANGLIM)
C                ======
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C  MODIFIED 85.5.9 BY DAVID KNIGHT
C  MODIFIED 90.11.27 BY SEAN HEALY
C
C  CALLS  BOST, DISTAN
C  USES   HXWORD, ICHECK, INTEGER, SQART
C
C  THIS ROUTINE PRINTS THE BONDS IN COMMON /DIST/ TO IXATOM AND THE
C  RELATED INFO OF THE BONDED ATOMS (NAME, IDENTIFIER, SYMMETRY OPERATOR)
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      common/files/nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      COMMON/BOSTT/NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      include 'comatom.for'
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
C
C      IF(NTERM.EQ.0) NOUT=NND
      IF(GDENT(IXATOM)(2:2).EQ.' ') THEN
	WRITE(NOUT,100) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  100   FORMAT(///,'  Environment of ',A2,2X,A3,
     +    '        Occupation Number =',F5.2,/,2X,20('-'))
      ELSE IF(GDENT(IXATOM)(3:3).EQ.' ') THEN
	WRITE(NOUT,101) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  101   FORMAT(///,'  Environment of ',A2,2X,A3,
     +    '        Occupation Number =',F5.2,/,2X,21('-'))
      ELSE
	WRITE(NOUT,102) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  102   FORMAT(///,'  Environment of ',A2,2X,A3,
     +    '        Occupation Number =',F5.2,/,2X,22('-'))
	ENDIF
      IF(IFANG.EQ.1.OR.IFANG.EQ.0) GOTO 20
      IF (ICHECK(IXATOM).EQ.0) GOTO 20
   15 WRITE(NOUT,105)ANGLX
  105 FORMAT('  Bonds Less Than ',F6.3,' Degrees From a Shorter',
     +' Bond Are Not Listed')
C
   20 IF(NB.GT.0) GOTO 30
      WRITE(NOUT,110)
  110 FORMAT(//,10X,25H' No Atoms in This Range')
      RETURN
   30 CONTINUE
C
C  PRINT HEADING FOR TABLE.
C
      WRITE(NOUT,120)
  120 FORMAT(/,22X,'Distance',7X,'Min   Max   Rid   Unc ',5X,
     +'Direction Cosines',9X,'Sym Trans(*12)     Occupancy',
     +'       Valence',/)
C
C  PRINT TABLE.
C
      ESTRS=0.0
      STRS=0.0
      DO 25 I=1,NB
	K=IDIST(1,I)
	COSA=A*XDIST(4,I)/XDIST(1,I)
	COSB=B*XDIST(5,I)/XDIST(1,I)
	COSC=C*XDIST(6,I)/XDIST(1,I)
	IF(JFIX.EQ.1) GO TO 28


	HLENG = HXWORD(XDIST(1,I),XDIST(2,I))
	HOCC = HXWORD(OCCU(K),SOCCU(K))
	WRITE(NOUT,131) GELEM(IXATOM),GDENT(IXATOM),GELEM(K),GDENT(K),
     +    HLENG,COSA,COSB,COSC,(IDIST(L,I),L=2,5),HOCC
  131   FORMAT(/,1X,A2,A3,' - ',A2,A3,7X,A11,31X,3F6.3,9X,I2,2X,
     +    3I3,5X,A10)
	GO TO 29
   28   LERR=INTEGER(XDIST(2,I),3)
	LSOCC=INTEGER(SOCCU(K),2)
	WRITE(NOUT,130)GELEM(IXATOM),GDENT(IXATOM),GELEM(K),GDENT(K),
     +    XDIST(1,I),LERR,COSA,COSB,COSC,(IDIST(L,I),L=2,5),OCCU(K),
     +    LSOCC
  130   FORMAT(/,1X,A2,A3,' - ',A2,A3,7X,F5.3,'(',I2,')',33X,3F6.3,9X,
     +    I2,2X,3I3,3X,F6.2,'(',I2,')')
C
C  SUBROUTINE DISTAN WILL CALCULATE AND PRINT MIN,MAX,RID,AND UNC
C  LENGTHS.
C
   29   CALL DISTAN(XDIST(1,I),COSA,COSB,COSC,IXATOM,IDIST(1,I),
     +    IDIST(2,I))
C
C   A CALL TO BOST CALCULATES THE VALENCE AND ITS ERROR.THESE
C   ARE PRINTED, AND ACCUMULATED AS SUMS.
C
	IF(NBOST.EQ.0)  GO TO 25
	CALL BOST(IXATOM,K,STR,ESTR,XDIST(2,I),XDIST(1,I))
	ESTRS=ESTRS+ESTR*ESTR
	STRS=STRS+STR
	IF(JFIX.EQ.1) GO TO 24
	HTEMP = HXWORD(STR,ESTR)
	WRITE(NOUT,140) HTEMP
  140   FORMAT('+',121X,A11)
	GO TO 25
   24   CONTINUE
	WRITE(NOUT,141) STR,INTEGER(ESTR,3)
  141   FORMAT('+',122X,F6.3,'(',I2,')')
   25   CONTINUE
      IF(ABS(STRS).LT.0.0001) GOTO 160
      ESTRSQ=SQART(ESTRS)
      IF(JFIX.EQ.1) GO TO 152
      HTEMP = HXWORD(STRS,ESTRSQ)
      WRITE(NOUT,150) HTEMP
  150 FORMAT('0',107X,'Valence Sum = ',A11)
      GO TO 154
  152 WRITE(NOUT,153) STRS,INTEGER(ESTRSQ,3)
  153 FORMAT('0',107X,'Valence Sum = ',F6.3,'(',I2,')')
  154 CONTINUE
      VALENCE=ABS(OXID(IXATOM))
      DIFF=ABS(OXID(IXATOM))-ABS(STRS)
      if(abs(estrsq).lt.0.0001) then
	dse=0.
      else
	DSE=DIFF/ESTRSQ
	endif
      IF(ABS(VALENCE).GT.0.0001) WRITE(NOUT,155) DIFF,DSE,VALENCE
  155 FORMAT('0',21X,'Difference = ',F5.3,14X,'Difference/Standard ',
     +  'Error = ',F7.2,11X,'Theoretical Valence =   ',F5.3)
      IF(IANGLIM.NE.0) WRITE(NOUT,9401)
 9401 FORMAT('    Bonds subtending an angle less than a critical ',
     +  'angle to a shorter bond have been eliminated.')
  160 RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE WRANG(IXATOM)
C                =====
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C  MODIFIED 85.5.10 BY DAVID KNIGHT
C
C  USES ANGLE, HXWORD, INTEGER, SQART
C
C  SUBROUTINE TO PRINT THE BOND ANGLES AMONG THE BONDS IN THE
C  ENVIRONMENT OF THE CURRENT ATOM (IXATOM).
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION HANG(12),ANGL(12),GNAME(12),GD(12),IDEL(12),NUM(12)
      DIMENSION DX1(3),DX2(3),DY1(3),DY2(3),DDX0(3),DDX1(3),DDX2(3)
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/ECELL/DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      include 'comatom.for'
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
C
C      IF(NTERM.EQ.0) NOUT=NND
      J = NB
      IF(J.GT.12) J=12
      IF(J.LE.1) RETURN
      WRITE(NOUT,100) GELEM(IXATOM),GDENT(IXATOM)
  100 FORMAT(/,' Angle in Degrees Subtended at   ' ,A2,A3,' by ')
C
      DO 10 I=1,J
	K=IDIST(1,I)
	GNAME(I)=GELEM(K)
	GD(I)=GDENT(K)
	NUM(I)=K
   10   CONTINUE
      J1=J-1
      WRITE(NOUT,110) (GNAME(I),GD(I),I=1,J1)
  110 FORMAT(/,12X,11(A2,A3,6X))
C
      DO 60 I=2,J
	INI = NUM(I)
	I1=I-1
	DO 50 K=1,I1
	  INK = NUM(K)
	  ANGL(K) = ANGLE(XDIST(4,I),XDIST(4,K),AM(1,1))
	  IF(IERRA.EQ.0) GOTO 50
	  ERR = 0.
	  DO 20 II=1,3
	    DX1(II)=XDIST(II+3,I)
	    DY1(II)=XDIST(II+3,I)
	    DX2(II)=XDIST(II+3,K)
   20       DY2(II)=XDIST(II+3,K)
	  DO 30 II=1,3
	    DY1(II) = DX1(II) + DXX(II,INI)
	    DEL = ANGL(K) - ANGLE(DY1(1),DX2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY1(II) = DX1(II) - DXX(II,INI)
	    DEL = ANGL(K) - ANGLE(DY1(1),DX2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY2(II) = DX2(II) + DXX(II,INK)
	    DEL = ANGL(K) - ANGLE(DX1(1),DY2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY2(II) = DX2(II) - DXX(II,INK)
	    DEL = ANGL(K) - ANGLE(DX1(1),DY2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY1(II) = DX1(II) + DXX(II,IXATOM)
	    DY2(II) = DX2(II) + DXX(II,IXATOM)
	    DEL = ANGL(K) - ANGLE(DY1(1),DY2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY1(II) = DX1(II) - DXX(II,IXATOM)
	    DY2(II) = DX2(II) - DXX(II,IXATOM)
	    DEL = ANGL(K) - ANGLE(DY1(1),DY2(1),AM(1,1))
	    ERR = ERR + DEL*DEL
	    DY1(II)=XDIST(II+3,I)
	    DY2(II)=XDIST(II+3,K)
   30       CONTINUE
	  DO 40 II=1,6
	    DEL = ANGL(K) - ANGLE(XDIST(4,I),XDIST(4,K),DM(1,II))
	    ERR = ERR + DEL*DEL*2.
   40       CONTINUE
	  DELAN = SQART(ERR/2.)
	  IF(NDEBUG.NE.0) WRITE(NDEBUG,120) I,K,DELAN
  120     FORMAT(' ***DEBUG***WRANG***120*** I=',I3,', K=',I3,
     +      ', DELAN=',F8.2)
	  IDEL(K) = INTEGER(DELAN,1)
	  IF(JFIX.EQ.0) HANG(K) = HXWORD(ANGL(K),DELAN)
   50     CONTINUE
	IF(IERRA.EQ.0) THEN
	  WRITE(NOUT,130) GNAME(I),GD(I),(ANGL(K),K=1,I1)
  130     FORMAT(/,' ',A2,A3,3X,11(F7.1,4X))
	ELSE IF(JFIX.EQ.0) THEN
	  WRITE(NOUT,140) GNAME(I),GD(I),(HANG(K),K=1,I1)
  140     FORMAT(/,' ',A2,A3,3X,11(A10,1X))
	ELSE
	  WRITE(NOUT,150) GNAME(I),GD(I),(ANGL(K),IDEL(K),K=1,I1)
  150     FORMAT(/,' ',A2,A3,2X,11(F6.1,'(',I2,')',1X))
	  ENDIF
   60   CONTINUE
      RETURN
      END
C
C
*************************************************************************
C
      SUBROUTINE WRTERM(IXATOM,IDFLAG,NT)
C                ======
C
C  WRITTEN 86.11.21 BY LIZ BROWN
C
C  CALLS  
C  USES   HXWORD, INTEGER, SQART, ANGLE
C
C  THIS ROUTINE PRINTS THE BONDS IN COMMON /DIST/ TO IXATOM AND THE
C  RELATED INFO OF THE BONDED ATOMS (NAME, IDENTIFIER, SYMMETRY OPERATOR)
C  AND PRINTS THE BOND ANGLES AMONG THE BONDS IN THE ENVIRONMENT 
C  OF THE CURRENT ATOM (IXATOM).
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION HANG(12),ANGL(12),GNAME(12),GD(12),IDEL(12),NUM(12)
      DIMENSION DX1(3),DX2(3),DY1(3),DY2(3),DDX0(3),DDX1(3),DDX2(3)
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      include 'comatom.for'
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/ECELL/DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
C
C
      NOUT = NT
      J=NB
      jj=nb
      if(jj.gt.12) jj=12
      IF(J.GT.8) J=8
      IF(J.LT.1) RETURN
C
C  PRINT HEADING FOR TABLE
C  -----------------------
C
C
      IF(IDFLAG.EQ.0) THEN
	IF(GDENT(IXATOM)(2:2).EQ.' ') THEN
	  WRITE(NOUT,100) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  100     FORMAT(/,'  Environment of ',A2,2X,A3,
     +      '        Occupation Number =',F5.2,/,2X,20('-'))
	ELSE IF(GDENT(IXATOM)(3:3).EQ.' ') THEN
	  WRITE(NOUT,101) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  101     FORMAT(/,'  Environment of ',A2,2X,A3,
     +      '        Occupation Number =',F5.2,/,2X,21('-'))
	ELSE
	  WRITE(NOUT,102) GELEM(IXATOM),GDENT(IXATOM),OCCU(IXATOM)
  102     FORMAT(/,'  Environment of ',A2,2X,A3,
     +      '        Occupation Number =',F5.2,/,2X,22('-'))
	  ENDIF
	ENDIF
      DO 10 I=1,JJ
	K=IDIST(1,I)
	GNAME(I)=GELEM(K)
	GD(I)=GDENT(K)
   10   CONTINUE
      WRITE(HFMT,110) 8*(8-J)+1,J
  110 FORMAT('(5X,',I2,'X,',I1,'(A2,A3,3X),'' Dist/Val '')')
      WRITE(NOUT,HFMT)(GNAME(I),GD(I),I=1,J)
C
C  PRINT TABLE.
C
      DO 60 I=1,JJ
	K=IDIST(1,I)
	HLENG = HXWORD(XDIST(1,I),XDIST(2,I))
	WRITE(NOUT,120) GELEM(K),GDENT(K),
     +    (IDIST(L,I),L=2,5),OCCU(K),hleng
  120   FORMAT(1X,A2,A3,I2,'(',3I3,')','Oc=',F4.2,42X,A10)
C
	DO 50 KK=1,J
	  ANGL(KK)=ANGLE(XDIST(4,I),XDIST(4,KK),AM(1,1))
   50     CONTINUE
	IF(J.EQ.1) ANGL(1)=0
	WRITE(HHFMT,130) 8*(8-J)+1,J
  130   FORMAT('(1X,',I2,'X,',I1,'(F7.1,1X),5X,F6.3)')
	WRITE(NOUT,HHFMT)(ANGL(KK),KK=1,J),xdist(3,i)
   60   CONTINUE
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE DISTAN(ALENG,X,Y,Z,I,KATOM,KSYM)
C                ======
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  CALLS THECOA, THECOI
C
C  THIS ROUTINE TESTS FOR ISOTROPIC, ANISOTROPIC OR ABSENT TEMPERATURE
C  FACTORS. FORMULAE ARE TAKEN FROM BUSING AND LEVY,ACTA CRYST.(1964),
C  VOLUME 14,PAGE 142.
C
C  CALCULATES AND PRINTS THE THERMAL MOTION CORRECTIONS FOR THE DISTANCE
C  -ALENG- DUE TO THE TEMPERATURE FACTORS FOUND IN COMMON /TEMP/ OR
C  THE ONES GIVEN AS -X-, -Y-, -Z-
C
***********************************************************************
*                                                                     *
*  I'M NOT CONVINCED THAT THIS ROUTINE IS DOES SOMETHING MEANING FULL *
*                                                                     *
***********************************************************************
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      include 'comatom.for'
C
      TISO1=U(1,1,I)
      TANI1=ABS(U(2,2,I))+ABS(U(3,3,I))+ABS(U(1,2,I))+ABS(U(1,3,I)) +
     +      ABS(U(2,3,I))
      TISO2=U(1,1,KATOM)
      TANI2=ABS(U(2,2,KATOM))+ABS(U(3,3,KATOM))+ABS(U(1,2,KATOM)) +
     +      ABS(U(1,3,KATOM))+ABS(U(2,3,KATOM))
      IF(ABS(TISO1)+TANI1.LT.0.0001) RETURN
      IF(ABS(TISO2)+TANI2.LT.0.0001) RETURN
      IF(TANI1.LT.0.0001) CALL THECOI(WJ,U(1,1,I))
      IF(TANI1.GT.0.0001) CALL THECOA(WJ,I,1,X,Y,Z)
      IF(TANI2.LT.0.0001) CALL THECOI(WI,U(1,1,KATOM))
      IF(TANI2.GT.0.0001) CALL THECOA(WI,KATOM,KSYM,X,Y,Z)
C
C CALCULATE THE MINIMUM,MAXIMUM,RIDING,AND UNCORRELATED CORRECTED
C DISTANCES,IN TERMS OF WI,WJ(SEE BUSING+LEVY).
C
      D1=ALENG + ((WJ-WI)**2)/(2.0*ALENG)
      D2=ALENG + ((WJ+WI)**2)/(2.0*ALENG)
      D3=ALENG + (WI**2-WJ**2)/(2.0*ALENG)
      D4=ALENG + (WI**2+WJ**2)/(2.0*ALENG)
      WRITE(NOUT,200) D1,D2,D3,D4
  200 FORMAT('+',34X,4F6.3)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE THECOI(W,X)
C                ======
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  USES SQART
C
C  ROUTINE TO CALCULATE THE CONTRIBUTION OF AN ISOTROPIC
C  TEMP. FACTOR TO THE THERMAL MOTION CORRECTION
C
      IF(X.LE.0.0) X=0.0
      W=SQART(X)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE THECOA(W,L,LS,X,Y,Z)
C                ======
C
C  WRITTEN 1970 BY I.D.BROWN
C
C  USES SQART
C
C  ROUTINE TO CALCULATE THE CONTRIBUTION TO THE CORRECTIONS IN AN
C  ANISOTROPIC BOND. CALLED BY DIST.
C
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(9),ICELL
C*      COMMON/CELL/ A,B,C,AL,BE,AGA,VOL,AM(9)
      COMMON/RCELL/AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
C*      COMMON/RCELL/ AS,BS,CS,RALPHA,RBETA,RGAMMA,RVOL,RM(3,3)
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
C*      COMMON/SYM/NSYM,MSYM,S(3,3,48),TT(3,48)
      include 'comatom.for'
      DIMENSION BB(3,3),BR(3,3),BT(3,3),BU(3)
C
C  ****WARNING**** THE SYMETRY TRANSFORMATION DOES NOT WORK FOR
C   TRIGONAL SPACE GROUPS
C
      BB(1,1)=U(1,1,L)*AS*AS
      BB(2,2)=U(2,2,L)*BS*BS
      BB(3,3)=U(3,3,L)*CS*CS
      BB(2,1)=U(1,2,L)*AS*BS
      BB(1,2)=BB(2,1)
      BB(3,1)=U(1,3,L)*AS*CS
      BB(1,3)=BB(3,1)
      BB(3,2)=U(2,3,L)*BS*CS
      BB(2,3)=BB(3,2)
      DO 70 I=1,3
	DO 70 J=1,3
	  BR(I,J)=0.0
   70     BT(I,J)=0.0
      DO 80 IB=1,3
	DO 80 I=1,3
	  DO 80 J=1,3
   80       BR(I,IB)=BB(I,J)*S(J,IB,LS)+BR(I,IB)
      DO 90 IC=1,3
	DO 90 IB=1,3
	  DO 90 I=1,3
   90       BT(IC,IB)=BR(I,IB)*S(I,IC,LS)+BT(IC,IB)
      DO 150 I=1,3
  150   BU(I)=AM(3*(I-1)+1)*X/A+AM(3*(I-1)+2)*Y/B+AM(3*(I-1)+3)*Z/C
      UU=BU(1)*X/A+BU(2)*Y/B+BU(3)*Z/C
      TT=0.0
      SS=0.0
      DO 100 I=1,3
	DO 100 J=1,3
	  TT=BT(I,J)*AM(3*(I-1)+J)+TT
  100     SS=BT(I,J)*BU(I)*BU(J)+SS
      W=TT-SS/UU
      IF(W.LE.0.0) W=0.0
      W=SQART(W)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE RNGERD
C                ======
C
C   WRITTEN 81.7.20 BY I D BROWN
C   MODIFIED 82.7.29 BY OLIVER SLUPECKI
C   MODIFIED 85.5.16 BY DAVID KNIGHT
C
C   READS IN INSTRUCTIONS FOR CALCULATING BOND LENGTHS
C
C   CALLS  BORNG, FINDEL, SCAN, WAIT
C
C   USES   INTEGER
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP,NSTAT,NRANG
      include 'comatom.for'
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/GRANGE/GEL(20)
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     +              HFORM(2),HCOMM(7),GRFACT,GRAD
C
      DIMENSION GNA(2),PARAM(2),IX(2),XXX(2)
      DATA GSTAR,GBL/'*','        '/
C
C   INITIALISE ARRAYS
C   THE COMMENTED VARIABLES ARE SET IN 'RESETB'...
C
*     NJ=0
      IER=0
      IALL=0
      DO 5 I=1,20
*       GEL(I) = GBL
*       IOX(I) = 0
	DO 5 J=1,20
	  BOT(I,J) = DDMIN
    5     TOP(I,J) = DDMAX
      DO 7 I=1,NX
	CALL FINDEL(GELEM(I),INTEGER(OXID(I),0),III,IER)
	IF(IER.EQ.0) GO TO 7
	WRITE(NOUT,9011)
	GO TO 9
    7   CONTINUE
    9 CONTINUE
C
C   CALL BONRNG TO SET DISTANCE RANGES FOR BONDS
C
      IF(NINDEX.NE.0.OR.IBND.EQ.1) CALL BONRNG
C
C   READ RANGES FROM THE FILE NRANG
C
      IF(NRANG.EQ.0) GOTO 110
C
C   READ NEXT LINE
C
      WRITE(6,1010)
 1010 FORMAT(' ','ENTER A RANGE IN THE FORM "RN atom1 ox1 atom2 ox2 ',
     +  'max min" (TYPE "END" TO EXIT)',/,' (IF YOU TYPE IN THE ATOMS,',
     +  ' DO NOT DEFAULT THE OXIDATION NUMBERS)',/)
      GOTO 25
   20 WRITE(6,1020)
 1020 FORMAT(' ','ENTER ANOTHER RANGE:',/)
   25 READ(NRANG,1000,END=10000) HLINE
 1000 FORMAT(BZ,A80)
10000 WRITE(NOUT,1001) HLINE
 1001 FORMAT(10X,A80)
      II=1
      CALL SCAN(GKID,N,X,EX,II,HLINE,IALL)
C
C   READ RNGE CARD
C
      IF(GKID(1:2).NE.'RN') GO TO 100
      DO 30 I=1,2
	CALL SCAN(GNA(I),N,X,EX,II,HLINE,IALL)
	CALL SCAN(GA,IX(I),XXX(I),EX,II,HLINE,IALL)
	IF(IX(I).EQ.0.AND.XXX(I).NE.0.0) IX(I)=INTEGER(XXX(I),0)
	IF(GA.EQ.GSTAR) IX(I)=0
   30   CONTINUE
      DO 40 I=1,2
	PARAM(I)=-1.
	CALL SCAN(GA,N,PARAM(I),EX,II,HLINE,IALL)
	IF(GA.EQ.GSTAR) PARAM(I) = -1.
	IF(II.EQ.99) GO TO 42
   40   CONTINUE
   42 IF(PARAM(1).LT.0) PARAM(1) = DDMAX
      IF(PARAM(2).LT.0) PARAM(2) = DDMIN
C
C   IF(GNA(1) = '*') SET GLOBAL RANGE
C
      IF(GNA(1).NE.GSTAR) GO TO 60
      DO 50 I=1,20
	DO 50 J=1,20
	  TOP(I,J) = PARAM(1)
	  BOT(I,J) = PARAM(2)
   50     CONTINUE
      DDMAX = PARAM(1)
      DDMIN = PARAM(2)
      GO TO 20
C
C   IF(GNA(2) = '*') SET RANGE FOR CENTRAL ATOMS
C
   60 IF(GNA(2).NE.GSTAR) GO TO 80
      CALL FINDEL(GNA(1),IX(1),I,IER)
      IF(IER.EQ.1) GO TO 901
      IF(IER.GT.1) GO TO 20
      DO 65 J=1,20
	TOP(I,J) = PARAM(1)
   65   BOT(I,J) = PARAM(2)
      GO TO 20
C
C   IF GNA(2) NOT EQ TO '*' SET RANGE FOR SPECIFIC BONDS
C
   80 CALL FINDEL(GNA(1),IX(1),I,IER)
      IF(IER.EQ.1) GO TO 901
      CALL FINDEL(GNA(2),IX(2),J,IER)
      IF(IER.EQ.1) GO TO 901
      IF(IER.GT.1) GO TO 20
      TOP(I,J) = PARAM(1)
      BOT(I,J) = PARAM(2)
      GO TO 20
  100 CONTINUE
C
C   READ END CARD
C
      IF(GKID(1:2).NE.'EN') GOTO 900
C
C  PRINT BOND RANGES (MAXIMUM AND MINIMUM DISTANCES)
C
  110 CONTINUE
      WRITE(NOUT,350)
      WRITE(NOUT,360) (GEL(I),I=1,NJ)
      DO 330 I=1,NJ
  330   WRITE(NOUT,370) GEL(I),IOX(I),(TOP(I,J),J=1,NJ)
      WRITE(NOUT,380)
      WRITE(NOUT,360) (GEL(I),I=1,NJ)
      DO 340 I=1,NJ
  340   WRITE(NOUT,370) GEL(I),IOX(I),(BOT(I,J),J=1,NJ)
  350 FORMAT(///,' Maximum Bond Lengths:')
  360 FORMAT(9X,20(A2,3X))
  370 FORMAT(2X,A2,I2,1X,20F5.2)
  380 FORMAT(/,' Minimum Bond Lengths:')
      if(nterm.ne.0) then
  400   write(nterm,1400)
 1400   format(' Do you want to change any of these values (Y/N/E) ')
	call readyn(*402,*404,*999,*400)
  402   write(nterm,1404)
 1404   format('  Enter the element symbols and oxidation states of',
     1    ' the two atoms',/,
     2    ' followed by the maximum and minimum distances.  Blank',
     2    ' to exit',/,
     3    ' Example: Ba,2,O,-2,3.02,1.00')
  405   gel1 = '   '
	read(nin,1405,err=402) hline
 1405   format(A80)
	ii=1
	call scan(gel1,n,x,ex,ii,hline,iall)
	call scan(g,iox1,x,ex,ii,hline,iall)
	call scan(gel2,n,x,ex,ii,hline,iall)
	call scan(g,iox2,x,ex,ii,hline,iall)
	call scan(g,n,dmax,ex,ii,hline,iall)
	call scan(g,n,dmin,ex,ii,hline,iall)
	if(gel1.eq.'  ') go to 110
	write(nout,11405) gel1,iox1,gel2,iox2,dmax,dmin
11405   format(20X,2(A2,I3,1X),2F10.3)
	call upcase(gel1)
	call upcase(gel2)
	do 403 i=1,nj
	  gell = gel(i)
	  call upcase(gell)
	  if(gel1.eq.gell.and.(iox1.eq.iox(i).or.iox1.eq.0)) i1=i
	  if(gel2.eq.gell.and.(iox2.eq.iox(i).or.iox2.eq.0)) i2=i
  403     continue
	top(i1,i2) = dmax
	top(i2,i1) = dmax
	bot(i1,i2) = dmin
	bot(i2,i1) = dmin
	write(nterm,1403)
 1403   format(' Enter next range, blank to exit')
	go to 405
	endif
  404 continue
      WRITE(NOUT,390) GENT,HFORM(1)
  390 FORMAT('0    Entry No.=',A10,/'       Formula =',A74)
C
C  PRINT COMMENTS INDICATING IFANG STATUS
C
      IF(IFANG.NE.0 .AND. IFANG.NE.8) GOTO 410
      IF(IFANG.EQ.0)WRITE (NOUT,500)
      IF(IFANG.EQ.8)WRITE (NOUT,510) ANGLX
      IF (NTERM.NE.0) CALL WAIT
      RETURN
  410 WRITE(NOUT,520)
      IF(NINDEX.NE.0) WRITE(NOUT,530) ANGLX,GSTAR
      RETURN
  500 FORMAT(/'  All Interatomic Distances Within Range Are Listed ')
  510 FORMAT(/'  Bonds Less Than ',F6.2,
     +' Degrees From a Shorter Bond Are Not Listed  ')
  520 FORMAT(/'  All Interatomic Distances Within Range Are Listed ',
     +'  Except As Noted Below  ')
  530 FORMAT('  Bonds Less Than ',F6.2,' Degrees From a Shorter Bond',
     +' Are Not Listed:  Indicated By: ',A1,/  )
  900 WRITE(6,9001) GKID
 9001 FORMAT(' **RNGERD** ',3X,A8,' Is Not a Recognised Identifier.',
     +' This Card Ignored.')
      GO TO 20
  901 WRITE(NOUT,9011)
 9011 FORMAT(' ***ERROR**' ,'  More Than 20 Elements in This ',
     +'Compound. There May Be Problems in Setting Bond Ranges')
      GO TO 20
  999 return
      END
C
C
*************************************************************************
C
C
      SUBROUTINE BONDRD
C                ======
C
C   WRITTEN 81.7.20 BY I D BROWN
C   MODIFIED 82.7.29 BY OLIVER SLUPECKI
C   MODIFIED 85.5.16 BY DAVID KNIGHT
C
C   READS IN INSTRUCTIONS FROM INPUT FILE,
C   SETS FLAGS FOR BOND CALCULATIONS AND BONDFILE OUTPUT
C
C   CALLS SCAN, UPCASE
C   USES IFILE
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION PARAM(2),IX(2)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP,NSTAT,NRANG
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
C
C   INITIALIZE SWITCHES, VMIND AND VMAXD ARE DEFAULT VALUES
C
      NOBE= 0
      NOAN= 0
      IERRA= 0
C   JRFLAG=1 CAUSES ALL ENTRIES TO BE WRITTEN TO BOND FILE
C         =0 ONLY ERROR FREE ENTRIES WRITTEN
      JRFLAG=0
      JWRITE = 0
      IFANG=0
      ANGLX=55.
      NINDEX=0
      IRANG=0
      NRANG=0
      IBND=0
      JFIX = 0
      VMIND = 1.2
      VMAXD = 0.038
      VMIN = VMIND
      VMAX = VMAXD
      DDMAX = 3.4
      DDMIN = 0.01
      IALL = 0
*
*  TEST WHETHER A LIST IS TO BE READ OR NOT...
*
      IF(NBOND.EQ.0) GO TO 220
*
   20 WRITE(NOUT,1010)
 1010 FORMAT(' ','ENTER A DISTANCE OPTION:',/)
      READ(NBOND,1000,END=10000) HLINE
 1000 FORMAT(BZ,A80)
10000 WRITE(NOUT,1001) HLINE
 1001 FORMAT(10X,A80)
      II=1
      CALL SCAN(GKID,N,X,EX,II,HLINE,IALL)
      CALL UPCASE(GKID)
C
C   READ FIX CARD
C
      IF(GKID(1:2).NE.'FI') GO TO 200
      JFIX = 1
      GO TO 20
  200 CONTINUE
C
C   READ NOBE,NOAN,ERRA CARD
C
      IF(GKID(1:3).NE.'NOB') GOTO 201
      NOBE =  1
      GOTO 20
  201 IF(GKID(1:3).NE.'NOA') GOTO 202
      NOAN =  1
      GOTO 20
  202 IF(GKID(1:2).NE.'ER') GOTO 203
      IERRA =  1
      GOTO 20
C
C   READ RANGE (OR RNGE) CARD
C
  203 IF(GKID(1:2).NE.'RN'.AND.GKID(1:3).NE.'RAN') GOTO 204
      GFILE = ' '
      HFILE = ' '
      IALL = -1
      MM = 1
   30 CALL SCAN(GFILE,N,X,EX,II,HLINE,IALL)
      CALL UPCASE(GFILE)
      HFILE(MM:MM+7) = GFILE
      MM = MM + 8
      IF(II.GE.99) GOTO 40
      IF(IALL.LT.0) GOTO 30
   40 IALL = 0
      IF(HFILE.EQ.' '.OR.HFILE.EQ.'*') HFILE = 'INPUT'
      nunit=0
      gfort=' '
      gstat='old'
      nnr=ifile(hfile,gfort,gstat,nunit)
      if(nnr.le.0) return
      nrang=nnr
      GOTO 20
C
C   READ BOND CARD
C
  204 IF(GKID(1:2).NE.'BO'.OR.GKID(1:5).EQ.'BONDF') GOTO 205
      IBND = 1
      IF(II.EQ.99) GO TO 20
      CALL SCAN(GA,N,VMIN,EX,II,HLINE,IALL)
      CALL UPCASE(GA)
      IF(GA(1:1).EQ.'*') VMIN = VMIND
      IF(II.EQ.99) GO TO 20
      CALL SCAN(GA,N,VMAX,EX,II,HLINE,IALL)
      CALL UPCASE(GA)
      IF(GA(1:1).EQ.'*') VMAX=VMAXD
      GOTO 20
C
C   READ ANGL CARD
C
  205 IF(GKID(1:2).NE.'AN') GOTO 207
      IFANG = 8
      ANGLX = 55.
      IF(II.GE.99) GOTO 20
      CALL SCAN(GA,N,ANGLEX,EX,II,HLINE,IALL)
      CALL UPCASE(GA)
      IF(GA(1:1).EQ.'*') GOTO 206
      ANGLX = ANGLEX
      IF(II.GE.99) GOTO 20
  206 CALL SCAN(GA,L,X,EX,II,HLINE,IALL)
      CALL UPCASE(GA)
      IF(GA(1:1).EQ.'*') GOTO 20
      IFANG = L
      GOTO 20
C
C   READ BONDFILE CARD
C
  207 IF(GKID.NE.'BONDFILE') GO TO 208
      GFILE = ' '
      HFILE = ' '
      IALL = -1
      MM = 1
   50 CALL SCAN(GFILE,N,X,EX,II,HLINE,IALL)
      CALL UPCASE(GFILE)
      HFILE(MM:MM+7) = GFILE
      MM = MM + 8
      IF(II.GE.99) GOTO 60
      IF(IALL.LT.0) GOTO 50
   60 IALL = 0
      IF(HFILE.EQ.' '.OR.HFILE.EQ.'*') HFILE = 'OUTPUT'
      nunit=0
      gfort=' '
      gstat=' '
      nni=ifile(hfile,gfort,gstat,nunit)
      if(nni.le.0) return
      nindex=nni
      NOAN =  1
      IF(II.NE.99) THEN
	CALL SCAN(GA,JRFLAG,X,EX,II,HLINE,IALL)
	CALL UPCASE(GA)
	ENDIF
      IF(IFANG.EQ.0) THEN
C     USE ANGLIM IF OXIDATION STATES ARE ZERO
	IFANG=2
	ANGLX=55.
      ENDIF
      GO TO 20
C
C   READ WRITEALL (OR RITE) CARD
C
  208 IF(GKID(1:2).NE.'RI'.AND.GKID(1:2).NE.'WR') GO TO 209
      JWRITE = 1
      GO TO 20
  209 CONTINUE
C
C   READ STATUS CARD
C
      IF(GKID(1:2).EQ.'ST') THEN
	GFILE = ' '
	HFILE = ' '
	IALL = -1
	MM = 1
   70   CALL SCAN(GFILE,N,X,EX,II,HLINE,IALL)
	CALL UPCASE(GFILE)
	HFILE(MM:MM+7) = GFILE
	MM = MM + 8
	IF(II.GE.99) GOTO 80
	IF(IALL.LT.0) GOTO 70
   80   IALL = 0
	IF(HFILE.EQ.' '.OR.HFILE.EQ.'*') HFILE = 'OUTPUT'
	nunit=0
	gfort=' '
	gstat=' '
	nns=ifile(hfile,gfort,gstat,nunit)
	if(nns.le.0) return
	nstat=nns
	GO TO 20
	ENDIF
C
C   READ END CARD
C
  210 IF(GKID(1:2).NE.'EN') GO TO 900
  220 IF(NDEBUG.NE.0) WRITE(NDEBUG,1900) IFANG,NINDEX,NRANG,IBND,
     +                                   JFIX,JWRITE,VMAX,VMIN
 1900 FORMAT(' ***DEBUG***BONDRD***1900*** IFANG=',I2,', NINDEX=',I2,
     +       ', NRANG=',I2,',IBND=',I2,',JFIX=',I2,', JWRITE=',I2,
     +       ', VMAX=',F6.3,', VMIN=',F6.3)
      RETURN
  900 WRITE(NOUT,9001) GKID
 9001 FORMAT(' **BONDRD**',3X,A8,' Is Not a Recognised Identifier,',
     + ' This Card Ignored')
      GO TO 20
      END
C
C
*************************************************************************
C
C
      FUNCTION ICHECK(IXATOM)
C              ======
C
C   WRITTEN 82.8.11 BY OLIVER SLUPECKI
C   MODIFIED 85.5.10 BY DAVID KNIGHT
C
C   CALLS RANGES
C
C   USES  INTEGER
C
C   THIS SUBROUTINE DETERMINES IF A CENTRAL ATOM OR
C   NEIGHBOURING ATOM HAS AN OXIDATION STATE DEFAULT OF
C   0, IF THE MAXIMUM RANGE HAS BEEN SET TO 3.4 BY DEFAULT
C   OR IF THE SUM OF TWO OCCUPATION NUMBERS ARE .LT. 1.98.
C   IF ANY OF THESE CONDITIONS ARE TRUE ICHECK=1,OTHERWISE 0.
C   COMBINATIONS OF THESE CRITERIA CAN BE SPECIFIED WITH IFANG ON
C   AN ANGL INSTRUCTION CARD.
C
c    ifang  oxid=0  max d=3.4  occu  any other
c      0
c      1      x         x        x
c      2      x
c      3                x
c      4                         x
c      5      x         x
c      6      x                  x
c      7                x        x
c      8      x         x        x        x
c
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      include 'comatom.for'
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
C
      ICHECK = 0
      IF(NB.EQ.0) RETURN
      IF(IFANG.EQ.0) RETURN
      IF(IFANG.EQ.8) GO TO 15
      DDMAX1 = DDMAX + 0.00001
      DDMAX2 = DDMAX - 0.00001
      IF(IFANG.EQ.3 .OR. IFANG.EQ.4 .OR. IFANG.EQ.7) GOTO 5
      IF(INTEGER(OXID(IXATOM),1).EQ.0) GOTO 15
    5 DO 10 N=1,NB
      J=IDIST(1,N)
      IF(IFANG.EQ.3 .OR. IFANG.EQ.4 .OR. IFANG.EQ.7) GOTO 6
	 IF(OXID(J).EQ.0.0) GOTO 15
    6 IF(IFANG.EQ.2 .OR. IFANG.EQ.4 .OR. IFANG.EQ.6) GOTO 7
	 CALL RANGES(IXATOM,J,DMAXI,DMINI)
      IF(DMAXI.LT.DDMAX1  .AND.  DMAXI.GT.DDMAX2) GOTO 15
    7 IF(IFANG.EQ.2 .OR. IFANG.EQ.3 .OR. IFANG.EQ.5) GOTO 10
      IF((OCCU(IXATOM) + OCCU(J)).LE.1.98) GOTO 15
   10 CONTINUE
      RETURN
   15 ICHECK=1
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE INDRITE
C                =======
C
C   WRITTEN 83.1.30 BY I D BROWN
C   MODIFIED 85.5.13 BY DAVID KNIGHT
C
C   CONTROLS THE WRITING OF THE BOND FILE
C
C   CALLS DUMMYH, INDATOM, INDCELL, STATUS
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP,NSTAT
      include 'comatom.for'
      include 'combond.for'
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
C
C   WRITE BONDS FORMED BY DUMMY H ATOMS
C
      CALL DUMMYH
C   TEST IF ANY ATOM HAS ZERO COORDINATION NUMBER
      DO 20 I=1,NX
	 IF(IYFLAG(I).NE.0) GO TO 20
	 IFFLAG=IFFLAG+1
	 WRITE(NOUT,2001) I,GELEM(I),GDENT(I)
 2001    FORMAT('0',15X,'No Bonds Found to Atom ',I2,1X,A2,A3)
   20    CONTINUE
      DO 22 I =1,NBON
	IF(ABS(XBON(3,I)).GT.5.0) WRITE(NOUT,1022) I,XBON(3,I)
 1022   FORMAT(' *ERROR*INDRITE*1022* bond valence of bond XBON(3,',i3,
     +  ') = ',F6.2)
   22   CONTINUE
C
c   test if sum of all oxidation numbers is zero
C
      SUMOX=0
      DO 21 I=1,NX
	SUMOX=SUMOX+ABS(OXID(I))
   21   CONTINUE
      IF(SUMOX.LE.0.001) THEN
	IRFLAG=IRFLAG+1
	WRITE(NOUT,2010)
 2010   FORMAT('0',15X,'Sum of oxidation numbers is zero')
	ENDIF
C   TEST IF BONDS ARE TO BE PRINTED OR WRITTEN TO FILE (ERROR FREE)
C   ---------------------------------------------------------------
      IRF=IRFLAG+IFFLAG
      IF(IRF.GE.1.AND.JRFLAG.EQ.0) GO TO 500
      IF(NSTAT.NE.0.AND.IRF.EQ.0) CALL STATUS(5)
      IF(NSTAT.NE.0.AND.IRF.NE.0) CALL STATUS(12)
      CALL INDCELL
C
C  WRITE BONDS, REMOVING ANY RELATED BY SYMMETRY
C
      DO 26 J=1,NBON
	I=J-1
	IF(I.GT.0) THEN
	  IF(IBON(1,I).EQ.IBON(1,J).AND.JBON(1,I).EQ.JBON(1,J).AND.
     +      ABS(XBON(1,I)-XBON(1,J)).LT.0.0001) GO TO 26
	  ENDIF
	WRITE(NINDEX,1000) IBON(1,J),(JBON(K,J),K=1,5),IFBON(3,J),
     +    INTEGER(XBON(1,J),3),(INTEGER(XBON(K,J),4),K=4,6),
     +    INTEGER(XBON(2,J),3),IABS(INTEGER(XBON(3,J),2))
 1000   FORMAT('D',3I2,3I3,I1,I4,3I5,2I3,37X)
   26   CONTINUE
      CALL INDATOM
      RETURN
C
C   ERROR IN ENTRY - ANALYSE ERROR
C
  500 CALL ERANAL
      WRITE(NOUT,5001) (IBON(1,J),(JBON(K,J),K=1,5),IFBON(3,J),
     +  INTEGER(XBON(1,J),3),(INTEGER(XBON(K,J),4),K=4,6),
     +  INTEGER(XBON(2,J),3),IABS(INTEGER(XBON(3,J),2)),J=1,NBON)
 5001 FORMAT(10X,3I2,3I3,I1,I4,3I5,2I3,37X)
   30 RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE INDCELL
C                =======
C
C   WRITTEN 81.12.31 BY I D BROWN
C   MODIFIED 85.5.13 BY DAVID KNIGHT
C   MODIFIED 90.10.1 BY SEAN HEALY
C
C   THIS SUBROUTINE WRITES THE CELL PART OF A BOND FILE
C
C   USES INTEGER
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION IENTR(8),GKOM(6),IA(3),IAL(3)
      COMMON/CELL/A(3),AL(3),VOL,AM(3,3),ICELL
C*      COMMON/CELL/A(3),AL(3)
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     +              HFORM(2),HCOMM(7),GRFACT,GRAD
c
c PROGRAMMERS NOTE ---
c changed to STATUSS for Unix system
c
      COMMON/STATUSS/LENT,JENT
      DATA GKOM/'SNS ','NDS ','EDS ','SNP ','NDP ','EDP '/
C
      WRITE(NINDEX,1000) JENT
 1000 FORMAT('@E',I6)
      WRITE(NINDEX,1020) HCODN(1)
 1020 FORMAT('Q',A16,57X)
      WRITE(NINDEX,1030) HFORM(1)
 1030    FORMAT('F',A74)
      GRAD = 'X'
      DO 40 I=1,7
	DO 40 J=1,6
	  IF(HCOMM(1)(1:4).EQ.GKOM(J)) GRAD=GKOM(J)(1:1)
   40     CONTINUE
      WRITE(NINDEX,1040) GRAD,GRFACT
 1040 FORMAT('R',A1,A4,69X)
      DO 50 I=1,3
	 IA(I) = INTEGER(A(I),3)
	 IAL(I) = INTEGER(AL(I),4)
   50    CONTINUE
      WRITE(NINDEX,1001) (IA(I),I=1,3),(IAL(I),I=1,3)
 1001 FORMAT('C',6I5,44X)
      WRITE(NINDEX,1110) HALL
 1110 FORMAT('S',A24,50X)
      RETURN
      END
C
C
*************************************************************************
C
C
C
      FUNCTION IVLCHK(II,SUM)
C              ======
C
C   WRITTEN 86.11.14 BY LIZ BROWN
C
C   CHECKS THE BOND VALENCE FOR BONDS AROUND ATOM II STORED IN /DIST/.
C   THE VALENCE SUM IS RETURNED IN SUM
C  
C
C   IYFLAG IS SET TO 1 IF BOND IS FOUND TO ATOM(I)
C   IFFLAG GIVES FATAL ERROR COUNT
C   IRFLAG GIVES WARNING COUNT
C   IVLCHK = 1  WARNING
C          = 2  FATAL ERROR
C
C   CALLS 
C
C   USES  
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      include 'comatom.for'
      include 'combond.for'
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/DUMMY/NXH,XNUMH(10)
      SUM=0.
      NBFLAG=0 
      IVLCHK=0

      IF(NDEBUG.NE.0) WRITE(NDEBUG,1000) II,NZA(2,II),II,
     +   INTEGER(OXID(II),0)
 1000 FORMAT(' **DEBUG**CHKBON**1000**',
     +   ' NZA(2,'I2,')=',I3,', INTEGER(OXID(',I2,'),0)=',I3)
      IF(NB.EQ.0) GO TO 60
      IYFLAG(II) = 1
      DO 50 I=1,NB
	iyn=idist(1,I)
C
C   IF THE BOND IS TO H THEN BOND VALENCE MUST BE LESS THAN 0.8
C   -----------------------------------------------------------
C
       IF((GELEM(II).EQ.'H '.OR.GELEM(II).EQ.'D '.OR.GELEM(IYN).
     +   EQ.'H '.OR.GELEM(IYN).EQ.'D ').AND.ABS(XDIST(3,I)).GT.0.8)
     +   XDIST(3,I) = 0.8*XDIST(3,I)/ABS(XDIST(3,I))
C
C   DO NOT ADD IN BOND VALENCES FOR ANION-ANION AND CATION-CATION BONDS
C   -------------------------------------------------------------------
C
	IF(OXID(II).LT.-0.001.AND.OXID(IYN).LT.-0.001) GO TO 10
	IF(OXID(II).GT.0.001.AND.OXID(IYN).GT.0.001) GO TO 10
	SUM = SUM + XDIST(3,I)
   10   CONTINUE
   50   CONTINUE
   60 CONTINUE
C
C     ADD CONTRIBUTIONS FROM BONDS TO DUMMY ATOMS
C     -------------------------------------------
C
      IF(NH(II).NE.0) THEN
	IYFLAG(II)=1
	SUM = SUM - NH(II)*0.8
	ENDIF
      IF(ND(II).NE.0) THEN
	IYFLAG(II)=1
	SUM = SUM - ND(II)*0.8
	ENDIF
      VALNCE(II)=SUM
C
C     TEST OUTPUT FOR ACCEPTABILITY
C     -----------------------------
C
      DIFF = OXID(II) - SUM
      DLIM = 0.25 + (1.1-OCCU(II))*ABS(OXID(II))
      ADIFF = ABS(DIFF)
      IF(ADIFF.GE.1.5) THEN
C
C     SUM MAY HAVE WRONG SIGN
C     -----------------------
C
	DIF=OXID(II)+SUM
	ADIF=ABS(DIF)
	IF(ADIF.LT.DLIM)THEN
C
C       AGREEMENT IS OK WITH REVERSED SIGN
C       ---------------------------------- 
C
	  SUM=-SUM
	  DIFF=-DIF
	  ADIFF=ADIF
	  ENDIF
	ENDIF
      IF(ADIFF.GE.DLIM)THEN
C
C     DIFFERENCE IS NOT ACCEPTABLE UNLESS
C     -----------------------------------
C
       IF(ADIFF.LT.0.5.AND.(GELEM(II).EQ.'H '.OR.GELEM(II).EQ.'D '))
     +   GO TO 500

C
C       ATOM IS HYDROGEN
C       ----------------
C
	IF((GELEM(II).EQ.'O '.OR.GELEM(II).EQ.'F '.OR.GELEM(II).EQ.'CL'
     +   .OR.GELEM(II).EQ.'S ').AND.(DIFF.GE.-0.7.AND.DIFF.LE.0.3))THEN
C
C       ATOM IS H BOND ACCEPTOR
C       -----------------------
C
	  DO 100 I=1,NX
	    DO 100 L=1,3
  100         IF(XX(L,I).GT.8.) GO TO 500
	  IF(NXH.NE.0) GO TO 500
C
C
C         AND THERE ARE DUMMY ATOMS     (N.B.  Dummy atoms are stored
C         -------------------------            in two possible ways.)
C
	  ENDIF
	IF(GELEM(II).EQ.'C '.AND.INTEGER(OXID(II),0).EQ.2) THEN
C 
C         ATOM IS C+2 AND OXID=3 IS ACCEPTABLE
C         ------------------------------------
C
	  IF(ABS(3.0-SUM).LT.DLIM) GO TO 500
	  ENDIF
C
C       HAVING EXHAUSTED EXCUSES, THE ATOM IS IN ERROR
C       ----------------------------------------------
C
	IF(ADIFF.GT.0.8) THEN
	  IFFLAG=IFFLAG+1
	  IVLCHK = 2
	ELSE
	  IRFLAG=IRFLAG+1
	  IVLCHK = 1
	  ENDIF
	ENDIF
  500   CONTINUE
      RETURN
      END
C
C
*************************************************************************

C
C
      SUBROUTINE DISBON(II,JWRITE)
C                ======
C
C   WRITTEN 86.11.14 BY LIZ BROWN
C
C
C     THIS SUBROUTINE COPIES THE INFORMATION ABOUT THE BONDS FORMED
C     BY ATOM II IN /DIST/ TO THE NEXT AVAILABLE LOCATIONS IN /BOND/
C     If JWRITE=1 all bonds are copied
C     If JWRITE=0 bonds are only copied if atom II is a cation, has 
C                 OXID=0, is N or peroxy O
C  
C     IWRITE FLAGS WHETHER THE BONDS ARE TO BE WRITTEN TO THE BOND
C     FILE.  THIS OCCURS FOR CATIONS, ATOMS WITH ODID=0, N AND
C     PEROXY O, UNLESS JWRITE = 1 IN WHICH CASE ALL BONDS ARE WRITTEN.
C  
C
C   CALLS DISTLB:WKBOND
C
C   USES IDISO 
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      common/files/nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
C
C
C   IWRITE FLAGS WHETHER BONDS ARE TO BE WRITTEN TO BOND FILE
C   THIS OCCURS FOR CATIONS, ATOMS WITH OXID=0, N AND PEROXY O
C
      IWRITE = 1
      IF(JWRITE.EQ.1) GO TO 10
      nz = nza(2,ii)
      if(nz.gt.20) nz = 3
      IF(INTEGER(OXID(II),0).LE.(NZ-18)) IWRITE = 0
      IF(GELEM(II).EQ.'N') IWRITE = 1
   10 IF(NDEBUG.NE.0) WRITE(NDEBUG,15) IWRITE,JWRITE
   15 FORMAT(' **DEBUG**DISBON**15**IWRITE=',I2,'JWRITE=',I2)
      IF(NB.EQ.0) GO TO 60
      DO 50 I=1,NB
	IF(IWRITE.EQ.0) GO TO 500
	NBON = NBON + 1
	IF(MAXDIM(NBON,MBON,'**DISBON**50**').EQ.0) THEN
	  DO 20 J=1,5
	    JBON(J,NBON)=IDIST(J,I)
   20       CONTINUE  
	  DO 30 J=1,6
	    XBON(J,NBON)=XDIST(J,I)
   30       CONTINUE   
	  IBON(1,NBON)=II
	  IBON(2,NBON)=1
	  DO 40 L=3,5
	    IBON(L,NBON)=0
   40       CONTINUE
	  IFBON(3,NBON)=IDISO(II)
	  ENDIF
   50   CONTINUE
C
C   ADD BONDS TO DUMMY H STORED IN /XATOM/
C   --------------------------------------
C
   60 CONTINUE
      IF(NH(II).NE.0) THEN
	MJ = NH(II)
	DO 80 I=1,NX
	  IF(XX(1,I).GT.8..AND.GELEM(I).EQ.'H ') THEN
	    DO 70 J=1,MJ
	      NBON = NBON + 1
	      IF(MAXDIM(NBON,MBON,'**DISBON**70**').EQ.0) THEN
		CALL WKBOND(II,I)
		ENDIF
   70         CONTINUE
	    ENDIF
   80     CONTINUE
	ENDIF
      IF(ND(II).NE.0) THEN
	MJ = ND(II)
	DO 100 I=1,NX
	  IF(XX(1,I).GT.8..AND.GELEM(I).EQ.'D ') THEN
	    DO 90 J=1,MJ
	      NBON = NBON + 1
	      IF(MAXDIM(NBON,MBON,'**DISBON**90**').EQ.0) THEN
		CALL WKBOND(II,I)
		ENDIF
   90         CONTINUE
	    ENDIF
  100     CONTINUE
	ENDIF
  500 RETURN
C
C
      END
C
C
*************************************************************************
C
C
C
      SUBROUTINE ERANAL
C                ======
C
C   WRITTEN 84.1.17 BY I.D.BROWN
C   MODIFIED 85.5.14 BY DAVID KNIGHT
C
C   THIS SUBROUTINE ATTEMPTS TO ANALYSE ERRORS FOUND IN ENTRIES
C
C   CALLS  STATUS
C
C   USES   INTEGER
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NFILE(7),NSTAT
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     +              HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      include 'comatom.for'
      include 'combond.for'
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/DUMMY/NXH,XNUMH(10)
      COMMON/GDUMMY/GDH(10)
c **programmers note**  VSUM could be replaced by VALNCE in XATOM
      COMMON/VALAT/VSUM(1000)
C
C   IPFLAG: FLAGS FOR UNBONDED ANIONS
C   ---------------------------------
      IPFLAG = 0
      WRITE(NOUT,1100)
 1100 FORMAT('0',10X,'Error Analysis')
      IER=0
C
C   GIVE ERROR COUNT
C   ----------------
      WRITE(NOUT,1140) IFFLAG,IRFLAG
 1140 FORMAT(15X,I3,' Fatal Errors, ',I3,' Warnings')
C
C   WRITE R FACTOR
C   --------------
      WRITE(NOUT,1150) GRFACT
 1150 FORMAT(15X,'R Factor = ',A4)
C
C   PRINT COMMENTS
C   --------------
      DO 160 I=1,KCOMM
	WRITE(NOUT,1160) HCOMM(I)
 1160   FORMAT(15X,A80)
  160   CONTINUE
C
C   CHECK SPACE GROUP
C   -----------------
      IMON=0
      ITRIG=0
      DO 180 I=1,24
	IF(HALL(I:I).EQ.'X') IMON=-1
	IF(HALL(I:I).EQ.'Y') IMON=1
	IF(HALL(I:I).EQ.'*') ITRIG=-10
	IF(HALL(I:I).EQ.'R') ITRIG=10
	IF(HALL(I:I).EQ.'2') ITRIG=ITRIG-2
	IF(HALL(I:I).EQ.'3') ITRIG=ITRIG+3
	IF(HALL(I:I).EQ.'4') ITRIG=ITRIG-8
	IF(HALL(I:I).EQ.'6') ITRIG=ITRIG+6
	IF(HALL(I:I).EQ.'(') GOTO 190
  180   CONTINUE
  190 IF(ITRIG.GT.0) THEN
C
C     SPACE GROUP IS TRIGONAL, RHOMBOHEDRAL OR HEXAGONAL
C     --------------------------------------------------
	IF(ABS(2.094-AGAMMA).GT.0.01) THEN
	  IER=1
	  IF(NSTAT.NE.0) CALL STATUS(8)
	  WRITE(NOUT,1180)
 1180     FORMAT(15X,'Gamma Should Be 120 Degrees')
	  ENDIF
	ENDIF
      IF(ITRIG.EQ.-2) THEN
C
C     SPACE GROUP IS MONOCLINIC
C     -------------------------
	IF(IMON.NE.0.AND.ABS(1.571-AGAMMA).GT.0.01) THEN
	  WRITE(NOUT,1181)
	  IER=1
	  IF(NSTAT.NE.0) CALL STATUS(9)
 1181     FORMAT(15X,'Gamma Should Be 90 Degrees')
	  ENDIF
	IF(IMON.LE.0.AND.ABS(1.571-BETA).GT.0.01) THEN
	  WRITE(NOUT,1182)
	  IF(NSTAT.NE.0) CALL STATUS(10)
	  IER=1
 1182     FORMAT(15X,'Beta Should Be 90 Degrees')
	  ENDIF
	IF(IMON.GE.0.AND.ABS(1.571-ALPHA).GT.0.01) THEN
	  WRITE(NOUT,1183)
	  IF(NSTAT.NE.0) CALL STATUS(11)
	  IER=1
 1183     FORMAT(15X,'Alpha Should Be 90 Degrees')
	  ENDIF
	ENDIF
      IF(IER.EQ.1) RETURN
C
C   CHECK ASSIGNMENT OF DUMMY H ATOMS
C   ---------------------------------
      YH=0
      XH=0
      DO 220 I=1,NX
	IF(XX(1,I).GT.8.0.AND.(GELEM(I).EQ.'H '.OR.GELEM(I).EQ.
     +'D ')) THEN
C
C       THERE ARE DUMMY H ATOMS
C       -----------------------
	  XH=XH+NPOS(I)*OCCU(I)
	  ENDIF
	YH=YH+(NH(I)+ND(I))*OCCU(I)*NPOS(I)
	IF(IYFLAG(I).EQ.0.AND.OXID(I).LT.0.0.AND.NZA(2,I).EQ.17) THEN
C
C       ATOM IS AN HALIDE WITH NO BOND
C       ------------------------------
	  IPFLAG = 1
	  ENDIF
  220   CONTINUE
      IF (NXH.NE.0.AND.XH.LT.0.01) THEN
	DO 230 L=1,NXH
  230     IF(GDH(L).EQ.'H '.OR.GDH(L).EQ.'D ') XH=XH+XNUMH(L)
	ENDIF
      IF(IPFLAG.GT.0.AND.XH.GT.0.01) CALL STATUS(13)
      NNH=INTEGER(YH,0)
      NXH=INTEGER(XH,0)
      IF(NDEBUG.NE.0) WRITE(NDEBUG,1221) XH,YH
 1221 FORMAT(' ++DEBUG++ERANAL++1221++',F5.1,' DUMMY H ATOMS GIVEN',
     + F5.1,' ASSIGNED')
      NDIF=NXH-NNH
      DIFF=XH-YH
      IF(DIFF.LT.-0.1) THEN
C
C     MORE DUMMY H ASSIGNED THAN GIVEN
C     --------------------------------
	CALL STATUS(15)
	WRITE(NOUT,1225) ABS(DIFF)
 1225   FORMAT(15X,F5.1,' Too Many Dummy H Assigned')
	GOTO 275
	ENDIF
      IF(XH.GT.0.1.AND.ABS(DIFF).GT.0.1) THEN
C
C     DUMMY H ARE IMPROPERLY ASSIGNED
C     -------------------------------
	CALL STATUS(16)
	WRITE(NOUT,1220) DIFF
 1220   FORMAT(15X,F5.1,' Missing H Atoms Assignments.  Possible ',
     +   'Assignments Are:')
	DO 270 I=1,NX
	  NN=0
	  DIF=Valnce(I)-OXID(I)
  250     IF(OXID(I).LT.-0.5.AND.DIF.GT.0.5) THEN
C
C         PROBABLE H ASSIGNMENT MISSING
C         -----------------------------
	    DIF=DIF-0.8
	    NN=NN+1
	    GO TO 250
	    ENDIF
	  IF(NN.GT.0) THEN
	    WRITE(NOUT,1250) NN,I,GELEM(I),GDENT(I)
 1250       FORMAT(20X,I2,' to ',I3,1X,A2,A3)
	    IER=1
	    ENDIF
  270     CONTINUE
	ENDIF
  275 IF(IER.EQ.1) GO TO 900
C
C   CHECK ENVIRONMENT OF P, S OR CL ATOMS
C   -------------------------------------
      DO 320 I=1,NX
	IF((GELEM(I).EQ.'CL'.AND.INTEGER(OXID(I),0).EQ.7).OR.
     +     (GELEM(I).EQ.'S '.AND.INTEGER(OXID(I),0).EQ.6).OR.
     +     (GELEM(I).EQ.'P '.AND.INTEGER(OXID(I),0).EQ.5)) THEN
C
C         ATOM IS P5, S6 OR CL7
C         ---------------------
	  IF(ICN(I).NE.4) THEN
	    WRITE(NOUT,1320) GELEM(I),GDENT(I),ICN(I)
 1320       FORMAT(15X,A2,A3,' Has Coordination Number',I3)
	    ENDIF
	  ENDIF
  320   CONTINUE
C
C   CHECK FOR ATOMS IN SAME PLACE WITH TOTAL OCCUPATION>1
C   ------------------------------------------------------
      DO 420 I=1,NX
	DO 415 J=I+1,NX
      IF (JTEST(XX(1,I),XX(1,J)).EQ.0) THEN
C    WE HAVE ATOMS ON SAME POSTION
	IF(OCCU(I)+OCCU(J).GT.1.1) THEN
	WRITE (NOUT,1420)GELEM(I),GDENT(I),GELEM(J),GDENT(J),
     +   (occu(i)+occu(j))
 1420   FORMAT('0',15X,'Atoms ',2(A2,A6),' are in same place with ',
     +  'total occupation equal to ',f5.2)
	ENDIF
      ENDIF
  415   CONTINUE
  420  CONTINUE
C
C   CHECK FOR BONDS THAT ARE IMPOSSIBLY SHORT
C   -----------------------------------------
      DO 520 I=1,NBON
	II=IBON(1,I)
	IJ=JBON(1,I)
	STR=XBON(3,I)
	OXIDII=ABS(OXID(II))
	OXIDIJ=ABS(OXID(IJ))
	IF((STR.GT.OXIDII.AND.OXIDII.GT.0.02).OR.
     +           (STR.GT.OXIDIJ.AND.OXIDIJ.GT.0.02)) THEN
C
C       BOND HAS AN IMPOSSIBLE VALENCE
C       ------------------------------
	  WRITE(NOUT,1520) II,GELEM(II),GDENT(II),IJ,GELEM(IJ),GDENT(IJ)
     +     ,XBON(1,I),STR
 1520     FORMAT(15X,'The Bond Between',I4,1X,A2,A3,' and',I4,1X,A2,A3,
     +     'Is Too Short at',F6.3,'A.  The Valence =',F5.2)
	  ENDIF
  520   CONTINUE
C
C   UPDATE STATUS
C   -------------
  900 IF(IFFLAG.NE.0) CALL STATUS(7)
      IF(IRFLAG.NE.0) CALL STATUS(6)
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE INDATOM
C                =======
C
C   WRITTEN 82.1.22 BY I D BROWN
C   COMMON BLOCKS CHANGED 85.5.14 BY DAVID KNIGHT
C    ... and again 13 feb 89 by Chris Carr
C   WRITES THE ATOM PART OF A BOND FILE
C
C   CALLS  NALTNO
C
C   USES   INTEGER
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      include 'comatom.for'
      DIMENSION IALT(3)
C
C   WRITE ATOMS
C
      DO 200 I=1,NX
	IOCCU = INTEGER(OCCU(I),2)
	IOXID = INTEGER(OXID(I),2)
	IF(IALTNO(I).EQ.0) CALL NALTNO(XX(1,I),NPOS(I),IALTNO(I),IALS,
     +    IALC,IALT)
	WRITE(NINDEX,1200) I,GELEM(I),GDENT(I),IOXID,NPOS(I),IOCCU,
     +    ICN(I),IALTNO(I)
 1200   FORMAT('G',I2,A2,A3,I4,2I3,2I2,53X)
  200   CONTINUE
      WRITE(NINDEX,1201)
 1201 FORMAT('Z')
      RETURN
      END
C
C
*************************************************************************
C
C
      SUBROUTINE STATUS(ICODE)
C                ======
C
C   WRITTEN 84.2.15 BY I D BROWN
C   MODIFIED 84.11.16 TO PRINT NEW FORMAT
C   COMMON BLOCKS CHANGED 85.5.14 BY DAVID KNIGHT
C    .. and again by Chris Carr  13 feb 89
C   MODIFIED 90.10.1 BY SEAN HEALY
C   THIS ROUTINE PRINTS STATUS FILE
C
C   USES  INCH
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILEB/NFILE(7),NSTAT
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
c
c PROGRAMMERS NOTE---
c changed to STATUSS (extra S)
c
      COMMON/STATUSS/LENT,JENT,ISTAT(30)
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     +              HFORM(2),HCOMM(7),GRFACT,GRAD
      DIMENSION HHFORM(16)
      DATA HHFORM/
     + '(1X,I6,1X,''D'',3X,''2'',5X,''No Space Group'')',
     + '(1X,I6,1X,''D'',3X,''2'',5X,''No Cell Constants'')',
     + '(1X,I6,1X,''D'',3X,''2'',5X,''No Coordinates'')',
     + '(1X,I6,1X,''M   2'')',
     + '(1X,I6,1X,''K   2'')',
     + '(1X,I6,1X,''F   2'')',
     + '(1X,I6,1X,''*   2 '')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Gamma Should Be 120'')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Gamma Should Be 90'')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Beta Should Be 90'')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Alpha Should Be 90'')',
     + '(1X,I6,1X,''F'',3X,''2'',5X,''W'')',
     + '(1X,I6,1X,''A P 2  23'')',
     + '(1X,I6,1X,''A P 2   2'')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Remove Assignment of H'')',
     + '(1X,I6,1X,''AE'',2X,''2'',5X,''Assign H'')'/
C
C   PRINT OUT TOTALS AT END OF RUN
C
      IF(ICODE.GT.30) THEN
	WRITE(NSTAT,31)
   31   FORMAT(7X,/,' Status Summary',/)
	WRITE(NSTAT,32) ISTAT(5)
   32   FORMAT(11X,I3,' K Ok')
	IST=ISTAT(6)+ISTAT(7)+ISTAT(20)
	WRITE(NSTAT,33) IST
   33   FORMAT(11X,I3,' *,F Status Unassigned')
	WRITE(NSTAT,34) ISTAT(12)
   34   FORMAT(11X,I3,' F Flagged')
	IST=ISTAT(8)+ISTAT(9)+ISTAT(10)+ISTAT(11)+ISTAT(15)+ISTAT(16)
	WRITE(NSTAT,35) IST
   35   FORMAT(11X,I3,' E Errors')
	IST=ISTAT(13)+ISTAT(14)
	WRITE(NSTAT,39) IST
   39   FORMAT(11X,I3,' P Problems')
	IST=ISTAT(1)+ISTAT(2)+ISTAT(3)
	WRITE(NSTAT,36) IST
   36   FORMAT(11X,I3,' D Missing Data')
	WRITE(NSTAT,37) ISTAT(4)
   37   FORMAT(11X,I3,' M Metals')
	WRITE(NSTAT,38) ISTAT(30)
   38   FORMAT(11X,I3,' X Missing')
	RETURN
	ENDIF
C
C   INITIALISE LENT ON FIRST RUN THROUGH
C
      IF(LENT.EQ.0) LENT=JENT-1
C
C   DO NOT OVERWRITE EXISTING ENTRY
C
      IF(JENT.EQ.LENT) RETURN
C
C   MISSING ENTRIES
C
  120 LENT=LENT+1
      IF(JENT.LE.LENT) GO TO 200
C  PROGRAMMERS' NOTE ********
C  THE FOLLOWING CODE WILL GENERATE STATUS ENTRIES FOR MISSING ENTRY NUMBERS
C  -------------------------------------------------------------------------
C
C     WRITE(NSTAT,1120) LENT
C1120 FORMAT(1X,I6,1X,'X   2')
C     ISTAT(30)=ISTAT(30)+1
      GO TO 120
C
C   SIMPLE CODES
C
  200 IF(ICODE.LE.16) THEN
C       org: ISTAT(ICODE)=ISTAT(ICODE)+1
	ISTAT(ICODE)=ISTAT(ICODE)
	WRITE(NSTAT,HHFORM(ICODE)) JENT
	ENDIF
      IF(ICODE.EQ.20) THEN
C       org: ISTAT(20)=ISTAT(20)+1
	ISTAT(20)=ISTAT(20)
	WRITE(NSTAT,20) JENT,HMSYM
   20   FORMAT(1X,I6,' *Space Group ',A24)
	ENDIF
      LENT=JENT
      RETURN
      END

C
*************************************************************************

C
      SUBROUTINE DUMCON
C
C     WRITTEN 86.22.19 BY LIZ BROWN  - oh really ??
C
C     IF DUMMY ATOMS ARE STORED IN /DUMMY/ THEY ARE TRANSFERED
C     TO /XATOM/
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      include 'comatom.for'
      COMMON/DUMMY/NXH,XNUMH(10)
      COMMON/GDUMMY/GDH(10)
      COMMON/SYMA/MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30),
     +            MULTSP(30),SP(3,3,30),TP(3,30)
      DIMENSION GDENTAR(10)
      DATA (GDENTAR(I),I=1,10)/'DUM1','DUM2','DUM3','DUM4','DUM5',
     +             'DUM6','DUM7','DUM8','DUM9','DUM10'/
C
      IF(NXH.NE.0)THEN
	DO 10 I=1,NXH
C
C         ADD DUMMY ATOMS TO /XATOM/ ARRAY
C         --------------------------------
C
	  NX=NX+1
	  IF(MAXDIM(NX,MX,'**DUMCON**10**').EQ.0) THEN
	    GELEM(NX)=GDH(I)
	    GDENT(NX)=GDENTAR(I)
	    DO 20 J=1,3
   20         XX(J,NX) = 9.0
	    OCCU(NX) = XNUMH(I)
	    NPOS(NX) = MULTSP(1)
	    CALL CHEMNO(GELEM(NX),NZA(1,NX),NZA(2,NX))
	    OXID(NX) = NZA(2,NX)
	    IF(OXID(NX).GT.11) OXID(NX) = OXID(NX) - 10
	    ENDIF
   10     CONTINUE
	ENDIF
      NHX=0
      RETURN
      END
C
C
*************************************************************************
C
C
      FUNCTION IDISO(II)
C
C     WRITTEN 86.11.14 BY LIZ BROWN
C
C     THIS FUNCTION SETS THE DISORDER FLAG IDISO
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      include 'comatom.for'
      IDISO = 0
      DO 10 I=1,NB
	IF(OCCU(II).LT.0.99) IDISO = 1
	IYN = IDIST(1,I)
	IF(OCCU(IYN).LT.0.99) IDISO = 1
   10   CONTINUE
      RETURN
      END
C
*************************************************************************
C
      SUBROUTINE DIAGNO(II,SUM,NT,IANGLIM)
C
C     WRITTEN 86.11.26 BY LIZ BROWN
C
C     THIS SUBROUTINE COMPILES A DIAGNOSTIC OUTPUT
C
C
C     CALLS
C     USES IVLCHK
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP,NSTAT,NRANG
      include 'comatom.for'     
      include 'combond.for'
C
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,
     +             DDMIN,RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/RCELL/AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
C*      COMMON/RCELL/RS(3),RANGLE(3),RVOL,RM(3,3)
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      DIMENSION XM(3),XN(3),XMAX(3),XMIN(3)
C
C
C   CALCULATE COORDINATION NUMBERS
C
      NOUT=NT
      NVLCHK=IVLCHK(II,SUM)
      ICN(II)=NB+NH(II)+ND(II)
      IF(ICN(II).EQ.0) RETURN
      NHD = NH(II) + ND(II)
      DIFF=ABS(OXID(II))-ABS(SUM)
      GG=' '
      IF(IDISO(II).NE.0) GG = 'D'
      GANGLIM=' '
      IF(IANGLIM.NE.0) GANGLIM='*'
      GWARNING=' '
      IF(NVLCHK.EQ.1) GWARNING='WARNING'
      IF(NVLCHK.EQ.2) GWARNING='ERROR'
      IF(NVLCHK.NE.0) THEN
	IF(NHD.NE.0) THEN
	  WRITE(NOUT,1010) II,GELEM(II),GDENT(II),NPOS(II),OCCU(II),
     +      ICN(II),GANGLIM,NHD,GG,OXID(II),SUM,DIFF,GWARNING
 1010     FORMAT(/,1X,I2,'.',1X,A2,A3,1X'Mult=',I3,', Occ=',F4.2,
     +      ', CN=',I2,1X,A1,1X,I2,'H',1X,A1,1X,'Oxid=',F5.2,
     +      ', V Sum=',F5.2,', Diff=',F5.2,1X,A7)
	  IYFLAG(II) = 1
	ELSE
	  WRITE(NOUT,1020) II,GELEM(II),GDENT(II),NPOS(II),OCCU(II),
     +      ICN(II),GANGLIM,GG,OXID(II),SUM,DIFF,GWARNING
 1020     FORMAT(/,1X,I2,'.',1X,A2,A3,1X,'Mult=',I3,', Occ=',F4.2,
     +      ', CN=',I2,1X,A1,5X,A1,1X,'Oxid=',F5.2,', V Sum=',
     +      F5.2,', Diff=',F5.2,1X,A7)
	  ENDIF
      ELSE
	IF(NHD.NE.0) THEN
	  IYFLAG(II) = 1
	  WRITE(NOUT,1030) II,GELEM(II),GDENT(II),NPOS(II),OCCU(II),
     +      ICN(II),GANGLIM,NHD,GG,OXID(II),SUM,DIFF,GWARNING
 1030     FORMAT(/,1X,I2,'.',1X,A2,A3,1X,'Npos=',I3,', Occ=',F4.2,
     +      ', CN=',I2,1X,A1,1X,I2,'H',1X,A1,1X,'Oxid=',F5.2,
     +      ', V Sum=',F5.2,', Diff=',F5.2,1X,A7)
	ELSE
	  WRITE(NOUT,1040) II,GELEM(II),GDENT(II),NPOS(II),OCCU(II),
     +      ICN(II),GANGLIM,GG,OXID(II),SUM,DIFF,GWARNING
 1040     FORMAT(/,1X,I2,'.',1X,A2,A3,1X,'Npos=',I3,', Occ=',F4.2,
     +      ', CN=',I2,1X,A1,5X,A1,1X,'Oxid=',F5.2,', V Sum=',
     +      F5.2,', Diff=',F5.2,1X,A7)
	  ENDIF
	IF(NTERM.NE.0) THEN
	  IF(NOUT.EQ.6) THEN
	    WRITE(NOUT,1045)
 1045       FORMAT(1X,'-------------------------------------------',
     +                '-------------------------------------')
	    ENDIF
	  ENDIF
	ENDIF
      IF(MINBON.NE.0) THEN
	  IF(NTERM.EQ.0) WRITE(NOUT,1050) MINBON
 1050     FORMAT('+',91X,I2,' Distances Below Min')
	  IF(NTERM.NE.0) WRITE(NOUT,1055) MINBON
 1055     FORMAT(3X,I2,' Distances Below Min')
	ENDIF
      RETURN
  900 IF(NBON.EQ.MBON+1) WRITE(NOUT,1900) MBON
 1900 FORMAT(/,' **ERROR**DIAGNO*1900** More Than ',I4,' Bonds,',
     +       ' Remainder Not Stored',/)
      RETURN
      END
C
C
*************************************************************************
C
      SUBROUTINE CLASSO(ICMPD)
C
C
C     SETS ICMPD=3 IF COMPOUND IS ORGANIC
C
C
      implicit character*8 (g)
      INCLUDE 'comatom.for'
      INCLUDE 'combond.for'
C
C
      DO 10 I=1,NBON
	IF(GELEM(IBON(1,I)).EQ.'C'.AND.
     +     GELEM(JBON(1,I)).EQ.'C') ICMPD=3
	IF(GELEM(IBON(1,I)).EQ.'C'.AND.
     +     GELEM(JBON(1,I)).EQ.'H') ICMPD=3
	IF(GELEM(IBON(1,I)).EQ.'H'.AND.
     +     GELEM(JBON(1,I)).EQ.'C') ICMPD=3
   10 CONTINUE
      RETURN
      END
c
*************************************************************************
c
	subroutine bondhelp
c
c       written by Lucien Jamieson - April 27, 1989
c
c       Used as a help screen for Bonder.
c
       common /files/nin,nout
1000   write(nout,1100)
1100   format(/,'                              Bond Calculations '//,
     +5X,'   This module calculates the bond lengths in a crystal '/,
     +5X,'based on the criteria proposed by Altermatt and Brown '/,
     +5X,'(Acta Cryst. (1985) B41 240).   These only apply to '/,
     +5X,'inorganic compounds.'/,
     +5X,'   The results will be viewed directly on the screen, but ')
       write(nout,1200)
1200   format(' ',
     +4X,'in any case a fuller listing of distances, angles, '/,
     +5X,'direction cosines and valences is written to the file '/,
     +5X,'DUMP.LIS.'/,
     +5X,'   The bonds generated may be stored, overwritting any '/,
     +5X,'bonds previously stored by the program.   An optional ')
       write(nout,1300)
1300   format(' ',
     +4X,'diagnostic feature will warn if the bond valence sums '/,
     +5X,'differ too greatly from the central atom oxidation state.'/,
     +5X,' '///////)
       call pageflip(*9999,*1000,*9999)
9999   return
       end
   
