C     FILE CHEMLB                                Last update: 28 July 1992
C          ------                                             C.Brodie
C
C     THIS FILE CONTAINS:
C
C  S OXIDE(GATOM,XOXID,NA)       CALCULATES OXIDATION STATES FOR COMMON /IDENT/
C  F OXUM(OXID,N)                SUMS UP OXID(1) TO OXID(N)
C  F NCOORD(G1,G2,OX1,OX2)       PREDICTS COORDINATION NUMBER OF G2 AROUND G1
C  F ICOOR(NP,NG)                RETURNS ESTIMATED COORDINATION NUMBER
C  S CHEMNUM                     CALLS =CHEMNO= FOR ALL ELEMENTS IN C. /IDENT/
C  S CHEMNO(GELEM,NP,NG)         GETS GROUP AND PERIOD FOR ELEMENT =GELEM=
c  F VALSUM(I)                   calculates the valence sum around atom i
C  S BOST(IX,JY,ST,EST,ERR,DIST) CALCULATES BOND STRENGTH AND ERROR FROM THE
C                                BOND LENGTH
C  F IBOST(GEL,GEM,OXM,OXL)      FINDS ADDRESS IN PBOST OF BOND TYPE
C  F DBOST(G1,G2,OX1,OX2,VAL)    CALLS ABOST AND RETURNS BOND LENGTH
C  S ABOST(G1,G2,IO1,IO2,VAL,BL) CALCULATES A BOND LENGTH FROM ITS STRENGTH
C  F COST(G1,G2,VAL)             ESTIMATES A CONTACT LENGTH FROM CONTACT VALENCE
C  S LDVAL(G1,G2,OX1,OX2,IER)    LOADS ONE VALENCE CURVE INTO BOST ARRAY
C  S CALVAL(G1,G2,IO1,IO2,IF,P1,P2,P3)   CALCULATES THE VALENCE PARAMETERS
C  S FINDEL(GN,IX,II,IER)        FINDS ELEMENT =GN= IN COMMON /GRANGE/
C  S BONRNG                      SETS RANGES FOR DISTANCES THAT REPRESENT BONDS
C  F IFNUMB(GELEM)               RETURNS F-NUMBER FOR RARE EARTHS AND ACTINIDES
C  F PICWGT(NP,NG,NF)            RETURNS ATOMIC WEIGHT FOR PERIOD/GROUP/F-NUM.
C  F VOLGET(GELEM)               RETURNS THEORETICAL VOLUME OF ANION =GELEM=
C  F ELMWGT(GELEM)               RETURNS ATOMIC WEIGHT FOR ELEMENT =GELEM=
C  S QSDERE(SNV,DRAD,DELE)       RETURNS VILLARS' QSD'S FOR TERNARY CPDS
C  F VILLEC(GELEM)               RETURNS VILLARS' ELECTRONEGATIVITY
C  F VILRAD(GELEM)               RETURNS VILLARS' PSEUDOPOT. RADIUS 
C  F RADIUS(GELEM,OXID,ICN)      RETURNS RADIUS FOR ANION =GELEM=
C  F ACID(NP,NG,OXID)            RETURNS LEWIS ACID STRENGTH FOR A CATION
C  F ACID1(G1,G2,OX1,OX2)        SAME AS ACID (DIFFERENT PARAMETERS)
C  F BASE(NP,NG,OXID,STR,NC)     RETURNS LEWIS BASE STRENGTH FOR AN ANION
C  F CONTAC(NP1,NP2)             RETURNS THEORETICAL CONTACT DISTANCE BASED ON
C                                THE PERIOD NUMBER OF THE PARTNERS ONLY
C  S CLASS(I)                    RETURNS I=1/2/3 FOR METAL/INORGANIC/ORGANIC
C                                COMPOUND STORED IN COMMON /IDENT/ (UNFINISHED)
C  F MTRANS(I)                   RETURNS 1/-1 FOR ATOM I A TRANSITION METAL/
C                                MAIN GROUP ELEMENT
c           Formula Manipulations
c           ---------------------
C  S FORMUL(HFORM,GELEM,IFORM,N) COPIES THE FORMULA GELEM(N),IFORM(N) TO HFORM
C    FORMLA(HFORM,GELEM,XFORM,N) COPIES THE FORMULA GELEM(N),XFORM(N) TO HFORM
C  F HFORMC(GELEM,NEL,N)          AS FORMUL BUT SUMS OVER EACH ELEMENT
C  F F000                        CALCULATES FZERO, THE SUM OF THE ATOMIC NUMBERS
C                                IN THE UNIT CELL
c  s decodefm(hfor)              decodes formula into /xatom/
c  S FORMAN(HFOR,GEL,XEL,NEL)    DECODES FORMULA IN HFOR INTO (GEL,XEL)NEL
C  S REDFOR(GEL,XEL,NEL)         REDUCES FORMULA TO ONE OCCURANCE OF EACH ELEM.
C  S SORFOR(GFORM,XFORM,OXID,N)  SORTS BY ATOMIC NUMBER
C  F IATOMNO(GELEM)              RETURNS THE ATOMIC NUMBER
C  F LCOORD(G1,G2,OX1,OX2,ACSTR,BASTR) CALCULATES BEST INTEGER COORDINATION
C                                      NUMBER
C  F RCOORD(GELMNT,PCN,OX1)      REFINES COORDINATION NUMBER RETURNED FROM
C                                PCOORD
C  F PCOORD(G1,G2,OX1,OX2)       ESTIMATES PAULING COORDINATION NUMBER
C
C     EXTERNALS CALLED FROM THIS FILE:
C
C  FOR-EXP
C
C  MATHLB: INTEGER, SORTY, SQART, UPCASE, elog
C
C
***********************************************************************
C
C
C
      SUBROUTINE OXIDE(GATOM,XOXID,NA)
C
C     CALLS CHEMNO,SORTY,INTEGER,OXNUM
C
C     SUBROUTINE TO CALCULATE OXIDATION STATES OF ELEMENTS GIVEN IN COMMON
C     /IDENT/
C
C     FLAN,FLMG AND FLTM ARE PARAMETERS WHOSE VALUE DETERMINES WHICH
C     ATOMS SHOULD HAVE THEIR INITIAL OXIDATION NUMBERS REDUCED. THEY
C     REFER TO ANIONS,MAIN GROUP ELEMENTS AND TRANSITION METALS
C     RESPECTIVELY.
C     IAN, IMG, ITM ARE INTEGERS THAT GIVE THE ATOM NUMBERS IN
C     ASCENDING ORDER OF FLAN, FLMG, AND FLTM.
C     THE ROUTINE ASSIGNS A MAXIMUM POSITVE OXIDATION STATE TO
C     EACH ATOM AND THEN REDUCES THESE IN ORDER UNTIL THE SUM (OXUM)
C     OF OXIDATION STATES IS ZERO.
C
C     STOPS AFTER =MCOUNT= TRIALS
C
C     ERROR-TEST AFTER RETURN: ERROR IF (ABS(OXUM(XOXID,NA)) .GT. 0.49)
C
C     WRITTEN BY I.D. BROWN / D. ALTERMATT
C---- MCMASTER UNIVERSITY, MARCH 1985
C
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      include 'comatom.for'
      common /dens/   zform,dens,ddens
C
      DIMENSION NP(100), NG(100), FLAN(100), FLMG(100), FLTM(100),
     1          IAN(100), IMG(100), ITM(100), GATOM(100), XOXID(100)
C
       MCOUNT=30
       KMAX=100
       IF(NX.LE.0) THEN
	 WRITE(NOUT,2000)
 2000    FORMAT(//,' **ERROR**OXIDE**2000** NO ELEMENTS STORED YET',//)
	 RETURN
	 ENDIF
       IF(ZFORM.LT.1.0) ZFORM=1.0
       K=0
       DO 50 I=1,NX
	 K=K+1
	 IF (K.GT.KMAX) GOTO 900
	 GATOM(K)=GELEM(I)
	 IF(NZA(1,I).LE.0.OR.NZA(2,I).LE.0)
     1     CALL CHEMNO (GELEM(I), NZA(1,I),NZA(2,I))
	 NP(K)=NZA(1,I)
	 NG(K)=NZA(2,I)
	 if(ng(k).ge.20) ng(k) = 3
	 OXID(I)=0.0000
	 NPO=NPOS(I)*INTEGER((OCCU(I)/ZFORM),0)
	 IF(NPO.LE.0) NPO=INTEGER(OCCU(I),0)
	 IF(NDEBUG.NE.0) WRITE(NDEBUG,2010) GATOM(K),NP(K),NG(K)
	 KK=K
	 DO 40 J=2,NPO
	   K=K+1
	   IF (K.GT.KMAX) GOTO 900
	   GATOM(K)=GATOM(KK)
	   NP(K)=NP(KK)
	   NG(K)=NG(KK)
	   IF(NDEBUG.NE.0) WRITE(NDEBUG,2010) GATOM(K),NP(K),NG(K)
 2010      FORMAT(' ++DEBUG++OXIDE++2010++ ELEMENT ',A2,', PERIOD'
     1       ,I3,', GROUP',I3)
 40        CONTINUE
 50      CONTINUE
       NA=K
       IF(NA.GT.MCOUNT) MCOUNT=NA
       IF(NDEBUG.NE.0) WRITE(NDEBUG,2030) MCOUNT
 2030  FORMAT(' ++DEBUG++OXIDE++2030++ MCOUNT =',I4)
C
C
C      SET INITIAL OXIDATION STATE AND THE ANION, MAIN GROUP AND
C      TRANSITION METAL FLAGS
       DO 60 I=1,NA
	  XOXID(I)=NG(I)
	  IF (NG(I).GE.12) XOXID(I)=NG(I)-10
	  IF (NG(I).GT.8.AND.NG(I).LT.12) XOXID(I)=16-NG(I)
	  if(ng(i).eq.3.and.np(i).ge.7) xoxid(i) = 7
	  FLAN(I)=3.*(NG(I)-10)-4.*NP(I)
	  IF (FLAN(I).LT.0) FLAN(I)=0
	  FLMG(I)=0
	  IF (NG(I).GT.12) FLMG(I)=0.7*NP(I)+NG(I)-16.4
	  FLTM(I)=0
	  IF(NG(I).GT.2.AND.NG(I).LT.13) FLTM(I)=XOXID(I)+0.25*NG(I)-2
 60    CONTINUE
C
C       REDUCE OXIDATION STATE OF THE ATOM WITH THE LARGEST VALUE OF
C            FLAN, FLMG OR FLTM
C        IF WTAN (CALCULATED FROM FLAN) IS LARGEST THE ATOM IS AN ANION
C        IF FLTM IS LARGEST REDUCE TRANSITION METAL OXIDATION STATE BY 1
C        IF FLMG IS LARGEST THE MAIN GROUP ELEMENT IS IN A LOWER
C             OXIDATION STATE.
C
       NCOUNT=0
 100   CALL SORTY (NA,FLAN,IAN)
       NCOUNT=NCOUNT+1
       MAN=IAN(1)
C      MAN IS THE ATOM WITH THE LARGEST VALUE OF FLAN
       WTAN=OXUM (XOXID,NA)+8*FLAN(MAN)-18
       IOXUM=INTEGER(OXUM(XOXID,NA),0)
       IF(NDEBUG.NE.0) THEN
	 DO 200 I=1,NA
	   WRITE(NDEBUG,2020) GATOM(I),XOXID(I),FLAN(I),FLMG(I),
     1       FLTM(I),I,IAN(I)
 2020      FORMAT(' ELEMENT ',A2,', OXID',F5.1,', FLAN',F5.1,', FLMG',
     1       F5.1,', FLTM',F5.1,', I =',I4,', IAN(I) =',I4)
  200      CONTINUE
	 ENDIF
       CALL SORTY (NA,FLMG,IMG)
       MMG=IMG(1)
       CALL SORTY (NA,FLTM,ITM)
       MTM=ITM(1)
       IF(NDEBUG.NE.0) WRITE(NDEBUG,2040) MAN,WTAN,MMG,MTM,NCOUNT
 2040  FORMAT(' ++DEBUG++OXIDE++2040++ MAN =',I3,', WTAN =',F6.2,
     1   ', MMG =',I3,', MTM =',I3,', NCOUNT =',I4,/,' ++DEBUG++',
     2   'OXIDE++2080++ UPDATED:')
       IF (WTAN.GT.FLMG(MMG)) THEN
	  IF(WTAN.GT.FLTM(MTM).AND.IOXUM.GE.8) THEN
	     XOXID(MAN)=XOXID(MAN)-8
	     FLAN(MAN)=0
	     FLMG(MAN)=0
	     GTP='MAN'
	     IF(NDEBUG.NE.0) WRITE(NDEBUG,2080) GTP,GATOM(MAN),XOXID
     1         (MAN),FLAN(MAN),FLMG(MAN),FLTM(MAN)
 2080        FORMAT(' CODE: ',A3,', ELEMENT: ',A2,', OXID:',F5.1,', ',
     1         'FLAN:',F5.1,', FLMG:',F5.1,', FLMT:',F5.1)
	  ELSE
	     XOXID(MTM)=XOXID(MTM)-1
	     FLTM(MTM)=FLTM(MTM)-1
	     GTP='MTM'
	     IF(NDEBUG.NE.0) WRITE(NDEBUG,2080) GTP,GATOM(MTM),XOXID
     1         (MTM),FLAN(MTM),FLMG(MTM),FLTM(MTM)
	     ENDIF
       ELSE
	  IF (FLMG(MMG).GT.FLTM(MTM).AND.IOXUM.GE.2) THEN
	     XOXID(MMG)=XOXID(MMG)-2
	     FLMG(MMG)=FLMG(MMG)-2
	     GTP='MMG'
	     IF(NDEBUG.NE.0) WRITE(NDEBUG,2080) GTP,GATOM(MMG),XOXID
     1         (MMG),FLAN(MMG),FLMG(MMG),FLTM(MMG)
	  ELSE
	     XOXID(MTM)=XOXID(MTM)-1
	     FLTM(MTM)=FLTM(MTM)-1
	     GTP='MTM'
	     IF(NDEBUG.NE.0) WRITE(NDEBUG,2080) GTP,GATOM(MTM),XOXID
     1         (MTM),FLAN(MTM),FLMG(MTM),FLTM(MTM)
	     ENDIF
	ENDIF
      IF (NCOUNT.GE.MCOUNT) GOTO 800
      IF (NDEBUG.NE.0) WRITE(NDEBUG,2060) OXUM(XOXID,NA)
 2060 FORMAT(' ++DEBUG++OXIDE++2060++ OXUM(XOXID,NA) = ',F6.2,/)
      IOXUM=INTEGER(OXUM(XOXID,NA),0)
      IF (IOXUM.GT.0) GOTO 100
C
C     OXUM IS EITHER O OR LESS THAN 0. RETURN
C
 800  IF(NDEBUG.NE.0) WRITE(NDEBUG,2050) OXUM(XOXID,NA)
 2050 FORMAT(' ++DEBUG++OXIDE++2050++ OXUM(XOXID,NA) AT EXIT: ',F6.2,/)
      RETURN
 900  WRITE(NOUT,2070) KMAX
 2070 FORMAT(//,' **ERROR**OXIDE**2070** MORE THAN ',I4,' ATOMS ',
     1  'PER FORMULA UNIT',//)
      XOXID(1)=1000.
      RETURN
      END
C
***********************************************************************
C
      FUNCTION OXUM (OXID,N)
C
C     RETRUNS SUM OF ASSIGNED OXIDATION NUMBERS
C
C     WRITTEN BY I.D. BROWN
C     MCMASTER UNIVERSITY, MARCH 1985
C
      DIMENSION OXID(N)
      OXUM=0
      DO 100 I=1,N
 100  OXUM=OXUM+OXID(I)
      RETURN
      END
C
C
C
C
      FUNCTION NCOORD(G1,G2,OX1,OX2)
C
C   written 86.11.12 by I.D.Brown
c
c   predicts the coordination number of g1 when surrounded by g2
c
      implicit character*8 (g)
      implicit character*80 (h)
      dimension ang(12)
      data ang/2,2,1.72,1.62,1.41,1.41,1.25,1.15,1.10,1.05,1,1/
c
      ncoord=0
      do 100 i=12,1,-1
	bval=ox1/i
	bond=dbost(g1,g2,ox1,ox2,bval)
	cval=ang(i)*bval/2
	conmin=cost(g1,g2,cval)
	contact=ang(i)*bond
	if(contact.gt.conmin) then
	  ncoord=i
	  return
	  endif
  100   continue
      return
      end
C
***********************************************************************
C
      function icoor(np,ng)
c     ---------------------
c
c  written 86.6.9 by I.D.Brown
c
c   Estimates  likely coordination number for element given
c
      icoor= 6
      i=ng-np
      if(i.ge.11) icoor=4
      if(i.le.-2) icoor=5-i
      if(np.eq.1) icoor=2
      return
      end
C
C
***********************************************************************
C
      SUBROUTINE CHEMNUM
C                =======
C
C   WRITTEN 83.3.31 BY I D BROWN
C
C   ASSIGNS PERIOD AND GROUP NUMBERS TO ATOMS COMMON /IDENT/
C
C   CALLS CHEMNO
C
      IMPLICIT CHARACTER*8 (G)
      include 'comatom.for'
C
      DO 100 II=1,NX
	NZA(1,II)=0
	NZA(2,II)=0
	CALL CHEMNO(GELEM(II),NZA(1,II),NZA(2,II))
  100 CONTINUE
      RETURN
      END
c
c
***********************************************************************
c
c
c
      subroutine chemno(gelm,nzp,nzg)
c
c     assigns period and group number to an element name in gelem
c     RE and actinides are assigned to groups 21-34
      implicit character*8 (g)
      common /files/ nin,nout,ndebug
      dimension gzq(18,7),gzr(14,2)
      data gzq/'H ',16*'  ','HE','LI','BE',10*'  ','B ','C ','N ','O '
     1     ,'F ','NE','NA','MG',10*'  ','AL','SI','P ','S ','CL','AR',
     2     'K ','CA','SC','TI','V ','CR','MN','FE','CO','NI','CU','ZN',
     3     'GA','GE','AS','SE','BR','KR','RB','SR','Y ','ZR','NB','MO',
     4     'TC','RU','RH','PD','AG','CD','IN','SN','SB','TE','I ','XE',
     5     'CS','BA','LA','HF','TA','W ','RE','OS','IR','PT','AU','HG',
     6     'TL','PB','BI','PO','AT','RN','FR','RA','AC',15*'  '/
      data gzr/'CE','PR','ND','PM','SM','EU','GD','TB','DY','HO','ER',
     1     'TM','YB','LU','TH','PA','U ','NP','PU','AM','CM','BK',
     2     'CF','ES','FM','MD','NO','LR'/
      data ghd/'D '/
      nzp=0
      nzg=0
c
c     case of deuterium
c
      gelem=gelm
      call upcase(gelem)
      if (gelem.ne.ghd) goto 30
      nzp=1
      nzg=1
      goto 95
30    continue
c
c     main group element or transition metal
c
      do 50 i=1,7
	do 40 j=1,18
	  if (gelem.ne.gzq(j,i)) goto 40
	  nzp=i
	  nzg=j
	  goto 95
40        continue
50      continue
c
c     rare earth or actinide
c
      do 70 i=1,2
	do 60 j=1,14
	  if(gelem.ne.gzr(j,i)) goto 60
	  nzg=j+20
	  nzp=i+5
	  goto 95
60        continue
70      continue
      write (nout,2000) gelem
2000  format (/,1X,'**ERROR**CHEMNO**2000** ',A2,' IS AN UNRECOGNI',
     1       'ZED ELEMENT NAME')
      goto 100
95    if (ndebug.ne.0) write (ndebug,2010) gelem,nzp,nzg
2010  format (/,1X,'++DEBUG++CHEMNO++2010++ ELEMENT ',A2,
     1  ' BELONGS TO PERIOD ',I2,' AND GROUP ',I2)
100   continue
      return
      end
C
C
***********************************************************************
c
c
c
      FUNCTION VALSUM(i)
c
c   written 91.10.4 by I.D.Brown
c
c   calculates the valence sum around atom i in the xatom list.
c   Bonds must have been previously calculated
c
c    calls SPANNER:expand, jyat
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2             netind(20),ifbond(4,600)
      include 'combond.for'
      dimension ky(5),ly(5)
c
c
c   set bond limits and calculate bond valences for EXPAND
c   ------------------------------------------------------
      bondlim(1) = 0.001
      bondlim(2) = 5.0
      if(abs(xbon(3,1)).lt.0.001) then
	do 120 i=1,nbon
	  call bost(ibon(1,i),jbon(1,i),xbon(3,i),estr,
     1         xbon(2,i),xbon(1,i))
  120     continue
	endif
c
c   Expand the coordination shell around atom i
c   -------------------------------------------
      iat = i
      itype = 3
      call expand(itype)
c
c   Calculate valence sum
c   ---------------------
      call jyat(1,ky)
      valsum = 0
      do 150 ii=2,nya
	call jyat(ii,ly)
	call bost(ky(1),ly(1),str,estr,err,
     1             dstnce(ky(1),ky(2),ky(3),ly(1),ly(2),ly(3)))
	valsum = valsum + str
  150   continue
      return
      end
c
C
***********************************************************************
C
      SUBROUTINE BOST(IX,JY,STR,ESTR,ERR,DIST)
C
C  *****SUBROUTINE*****          BOST
C                                ----
C
C  WRITTEN 1973 BY I.D.BROWN
C  REWRITTEN JULY 82 BY DAVID R. MOSSCROP / 84.6.12  D. ALTERMATT
c  ibost extracted 91.7.31 by I.D.Brown
C
c  This routine calculates the valence of a bond and its error based
c  on the bond  distance and the parameters stored in /bost/.  If no
c  suitable parameters are given, a call to LDVAL will generate them
c
C  CALLS ibost, SQART
C
      IMPLICIT CHARACTER*8 (G)
C
C programmer's note:  /bost/ is changed to /bostt/ for
c                     Unix system
      COMMON/BOSTT/NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON/GBOST/GBOST(2,30)
      include 'comatom.for'
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
C
C  SEARCH FOR BOND STRENGTH INFO FOR ATOMS IX AND JY IN /bost/
C
      STR=0.0
      ESTR=0.0
      n = ibost(gelem(ix),gelem(jy),oxid(ix),oxid(jy))
      if(n.le.0) return
C
C  CALCULATE THE VALENCE AND ITS ERROR
C
   30 IF(IFBOST(N).NE.1)GO TO 350
       STR=PBOST(3,N)*(PBOST(1,N)/DIST)**PBOST(2,N)
       ESTR=ERR*PBOST(2,N)*PBOST(3,N)*((PBOST(1,N)/DIST)**(PBOST(2,N)
     1     +1.))/PBOST(1,N)
       GO TO 400
  350 IF(IFBOST(N).NE.2) GO TO 370
       STR = EXP((PBOST(1,N)-DIST)/PBOST(2,N))
       ESTR = ERR*STR/PBOST(2,N)
       GO TO 400
  370 IF(IFBOST(N).NE.3) GO TO 450
      IF(DIST.LE.PBOST(1,N)) THEN
	STR=ABS(OXID(IX))
      ELSE
	STR=PBOST(2,N)/(DIST-PBOST(1,N)) - PBOST(3,N)
	ENDIF
       IF(STR.LT.0.) THEN
	 STR=0.
	 RETURN
	 ENDIF
       ESTR=ERR*STR/(DIST-PBOST(1,N))
  400 CONTINUE
      IF(OCCU(JY).GT.OCCU(IX)) GO TO 440
      STR = STR*OCCU(JY)/OCCU(IX)
      ESTR=STR*SQART( (ESTR/STR)**2 + (SOCCU(JY)/OCCU(JY))**2
     1  +(SOCCU(IX)/OCCU(IX))**2 )
  440 CONTINUE
      IF(oxid(ix).lt.0..AND.OXID(JY).GT.0.) STR = -STR
      RETURN
  450 WRITE(NOUT,100)IFBOST(N)
  100 FORMAT(' ++ERROR++BOST: UNRECOGNIZED IFBOST FLAG: IF = ',I3)
      RETURN
      END
C
C
***********************************************************************
c
      FUNCTION IBOST(gel,gem,oxl,oxm)
c
c   written 91.7.31 by I.D.Brown
c
c  Returns the address in PBOST of the valence parameters for the bond
c  between gel/oxl and gem/oxm
c  If the bost array is full, ibost = -2
c  If no valence parameters could be found or determined, ibost = -1
c
c   calls: integer, ldval, upcase
C
      implicit character*8 (g)
      implicit character*80 (h)
c
      COMMON/BOSTT/NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON/GBOST/GBOST(2,30)
c
      ibost = 0
      gell = gel
      gemm = gem
      call upcase(gell)
      call upcase(gemm)
      IOXID=INTEGER(oxl,0)
      JOXID=INTEGER(oxm,0)
      DO 10 I=1,NBOST
	if(gell.eq.gbost(1,i).and.ioxid.eq.ioxbost(1,i).and.
     1      gemm.eq.gbost(2,i).and.joxid.eq.ioxbost(2,i)) then
	  ibost = i
	  go to 30
	else if(gemm.eq.gbost(1,i).and.joxid.eq.ioxbost(1,i).and.
     1      gell.eq.gbost(2,i).and.ioxid.eq.ioxbost(2,i)) then
	  ibost = i
	  go to 30
	  endif
   10   CONTINUE
      CALL LDVAL(gell,gemm,oxl,oxm,IER)
      ibost = -ier
      if(ier.eq.0) ibost = nbost
   30 RETURN
      end
c
***********************************************************************
C
      FUNCTION DBOST(G1,G2,OX1,OX2,VAL)
C
C     CALLS ABOST, INTEGER
C
C     RETURNS THE BOND LENGTH CORRESPONDING TO THE VALENCE 'VAL'
C
C     WRITTEN BY I.D. BROWN
C     MCMASTER UNIVERSITY, MARCH 1985
C
      IMPLICIT CHARACTER *8 (G)
C
      if(val.lt.0.005) then
	dbost=0.0
	return
	endif
      IOX1=INTEGER(OX1,0)
      IOX2=INTEGER(OX2,0)
      CALL ABOST(G1,G2,IOX1,IOX2,VAL,DIST)
      DBOST=DIST
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE ABOST(GEL,GEM,IOXL,IOXM,SM,BLENG)
C
C   *****SUBROUTINE*****     ABOST
C                            -----
C
C   WRITTEN 82.1.8 BY I D BROWN
C   REWRITTEN JULY 82 BY DAVID R. MOSSCROP / 84.6.12  D. ALTERMATT
c   91.7.31 ibost extracted by I.D.Brown
C
C   CALCULATES A BOND LENGTH GIVEN THE BOND VALENCE
C
      IMPLICIT CHARACTER*8 (G)
C
C programmer's note:  /bost/ is changed to /bostt/ for
c                     Unix system
      COMMON/BOSTT/NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON/GBOST/GBOST(2,30)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
C
C   FIND THE VALENCE PARAMETERS REQUIRED
C
      BLENG=0.
      n = ibost(gel,gem,float(ioxl),float(ioxm))
      if(n.le.0) return
C
C   CALCULATE BOND LENGTH
C
   30 IF(IFBOST(N).NE.1) GO TO 350
       BLENG = PBOST(1,N)*(SM/PBOST(3,N))**(-1./PBOST(2,N))
       RETURN
  350 IF(IFBOST(N).NE.2) GO TO 370
       BLENG = PBOST(1,N) - PBOST(2,N)*ELOG(SM)
       RETURN
  370 IF(IFBOST(N).NE.3) GO TO 450
       BLENG = PBOST(2,N)/(SM+PBOST(3,N)) + PBOST(1,N)
       RETURN
  450 WRITE(NOUT,100) IFBOST(N),GBOST(1,N),GBOST(2,N)
  100 FORMAT(' ++ERROR++ABOST: UNRECOGNISED IFBOST FLAG: IF = ',I3,
     +  ' FOR BOND',2(1X,A2))
      RETURN
      END
C
C
***********************************************************************
c
c
      function cost(g1,g2,val)
c
c   written 1986.9.10 by I.D.Brown
c
c   this function returns an estimate of the length of a contact given
c   the contact valence.
c
c   while under development, this program will only give distances appropriate
c   for O-O contacts, whatever atoms are given.
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      dimension p(5),gelm(11),rad(11)
      data gelm/'F ','CL','BR','I ','O ','S ','SE','TE',
     +                                          'N ','P ','C '/
      data rad /1.21,1.50,1.78,1.85,1.31,1.65,1.71,1.85,
     +                                          1.35,1.50,1.28/
c
      cost=0.
      p(1)=0.
      p(2)=10
      do 100 i=1,11
	if(gelm(i).eq.g1) p(1)=p(1)+rad(i)
	if(gelm(i).eq.g2) p(1)=p(1)+rad(i)
  100   continue
      if(abs(val).gt.0.0001) cost=p(1)*(val**(-1/p(2)))
      return
      end
c
c
***********************************************************************
C
C
      SUBROUTINE LDVAL(G1,G2,OX1,OX2,IER)
C
C   WRITTEN 85.3.14 BY I.D.BROWN
C
C   CALLS CALVAL AND LOADS THE VALENCE PARAMETERS FOR ONE PAIR OF ATOMS
C   INTO THE BOST ARRAYS
C
C   IER IS RETURNED AS 0 IF ALL HAS GONE WELL
C   IER=1 IF CALVAL IS UNABLE TO CALCULATE THE PARAMETERS
C      =2 IF THE BOST ARRAYS ARE ALREADY FULL
C
C    CALLS CALVAL AND INTEGER
C
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug
c programmer's note:  /bost/ is changed to /bostt/ for Unix
c                     system
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),
     1                pbost(5,30)
      common /gbost/  gbost(2,30)
C
      IER=0
      IF(NBOST.EQ.MBOST) THEN
	IER=2
	RETURN
	ENDIF
C
      IOX1=INTEGER(OX1,0)
      IOX2=INTEGER(OX2,0)
      NBOST=NBOST+1
      CALL CALVAL(G1,G2,IOX1,IOX2,IFBOST(NBOST),PBOST(1,NBOST),
     +            PBOST(2,NBOST),PBOST(3,NBOST))
      IF(IFBOST(NBOST).EQ.0) THEN
	IER=1
	NBOST=NBOST-1
	RETURN
	ENDIF
      GBOST(1,NBOST)=G1
      GBOST(2,NBOST)=G2
      IOXBOST(1,NBOST)=IOX1
      IOXBOST(2,NBOST)=IOX2
      if(ndebug.ne.0) write(ndebug,1000) g1,iox1,g2,iox2,ifbost(nbost),
     +      (pbost(j,nbost),j=1,3)
 1000 format(' *ldval* Valence parameters for ',A2,I3,1X,A2,2I3,3F10.3)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE CALVAL(GEL,GEM,IOXL,IOXM,IFB,PB1,PB2,PB3)
C
C    ****SUBROUTINE*****     CALVAL
C                            ------
C   CALLS CHEMNO
C
C   WRITTEN 83.3.31 BY I D BROWN
C   REVISED 84.10.26 BY I D BROWN
C   CALCULATES VALENCE PARAMETERS FROM A TABLE
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON/FILES/NIN,NOUT,NDEBUG
      DIMENSION DIST1(18,7),GNAME(12),DIST2(12),DCOR(10),GRE(14)
      DATA GNAME/'H','C ','N ',
     +     'O ','F ','P ','S ','CL','SE','BR','TE','I '/
      DATA DIST2/-0.32,0.0,0.09,0.00,-0.11,0.30,0.49,0.38,0.60,0.54,
     +  0.80,0.81/
      DATA DIST1/ 0.870,  17*1.0,
     2 1.466,1.378,  10*1.0,
     2             1.371,1.390,1.432,1.43 ,1.43 ,1.43,
     3 1.802,1.693,   10*1.0,
     3             1.651,1.624,1.617,1.624,1.632,1.63 ,
     4 2.126,1.967,1.849,1.814,1.804,1.794,1.790,1.760,1.685,1.794,
     4 1.895,2.084,1.718,1.748,1.760,1.782,1.822,1.94 ,
     5 2.267,2.117,2.014,1.928,1.912,1.907,1.91 ,1.91 ,1.810,1.900,
     5 2.060,2.284,1.902,1.925,1.928,1.926,1.955,1.95 ,
     6 2.438,2.280,2.170,1.925,1.922,1.917,1.91 ,1.91 ,1.860,1.900,
     6 2.060,2.352,2.06 ,2.042,2.02 ,2.09 ,2.09 ,2.09 ,
     7 2.5  ,2.4  ,2.07 , 15*2.20/
      DATA GRE/'CE','PR','ND','PM','SM','EU','GD','TB','DY','HO',
     + 'ER','TM','YB','LU'/
      DATA FCOR/0.016/
      DATA DCOR/ .020,.040,.060,.030,-.005,.033,-.007,.140,.220,.380/
      IFB = 2
      PB2 = 0.37
      PB3 = 1.0
c
c  case for boron and boranes
c  --------------------------
      if(gel.eq.'B'.and.gem.eq.'B') then
	pb1=1.315
	pb2=0.50
	go to 300
      else if((gel.eq.'B'.and.gem.eq.'H').or.
     +        (gel.eq.'H'.and.gem.eq.'B')) then
	pb1=1.05
	go to 300
      endif
c
c  case when one atom has zero oxidation state (no calculation)
c  --------------------------------------------
      IF(IOXL.EQ.0.OR.IOXM.EQ.0) GO TO 500
c
c  determine which is an anion
c  ---------------------------
C
C   GXNAM = ANION NAME
C   GXNAL = CATION NAME
C
      if(ioxl.lt.0) then
	GXNAM = GEL
	GXNAL = GEM
	IOXNAL = IOXM
      else
	GXNAM = GEM
	GXNAL = GEL
	IOXNAL = IOXL
	endif
      call upcase(gxnam)
      call upcase(gxnal)
c
c  Case for both atoms anions (value calculated only when both are in low
c  --------------------------                 oxidation states)
      if(ioxl.lt.0.and.ioxm.lt.0) then
	call chemno(gxnal,npm,ngm)
	call chemno(gxnam,npl,ngl)
	if((ngm-ioxm).ne.18.and.(ngl-ioxl).ne.18) then
	  do 212 i=1,11
	    if(gxnal.eq.gname(i)) then
	      do 210 j=1,11
		if(gxnam.eq.gname(j)) then
		  radius = (2.86+dist2(i)+dist2(j))/2.
		  go to 300
		  endif
  210           continue
	      endif
  212       continue
	  endif
	  go to 500
	endif
c  
c  Case for one anion and one cation
c  ---------------------------------
      if((ioxl*ioxm).lt.0) then
c
c     Determine the anion
c     -------------------
	DO 250 I=1,11
	  IF(GXNAM.EQ.GNAME(I)) THEN
	    CALL CHEMNO(GXNAL,NZP,NZG)
c
c       The cation is H
c       ---------------
	    if(nzp.eq.1) then
	      ifb=3
	      pb1=dist2(i)+0.60
	      pb2=0.41
	      pb3=0.14
	      go to 300
	      endif
c
c       Other cations
c       -------------
	    DMULT = 1.0
	    PCORR = 0.
	    DCORR = 0.
	    FCORR = 0.
	    NDEL = NZG - IOXNAL
	    IF(NZG.LE.12.AND.NDEL.GT.0) THEN
	      DMULT = 0.8
	      IF(NDEL.LE.10) DCORR = DCOR(NDEL)
	      ENDIF
	    IF(NZG.EQ.3.AND.NZP.EQ.6) THEN
	      DO 220 J=1,14
  220           IF(GXNAL.EQ.GRE(J)) FCORR = J*FCOR
	      ENDIF
	    IF(NDEL.GT.10) PCORR = 0.0175*(NZP-2)
	    PB1=DIST1(NZG,NZP)+DMULT*DIST2(I)-DCORR-FCORR+PCORR
	    GO TO 300
	    ENDIF
  250     CONTINUE
	  go to 500
	endif
c
c  case for both cations
c  ---------------------
      if(ioxl.gt.0.and.ioxm.gt.0) then
	go to 500
	endif
c
c  exit if parameters not determined
c  ---------------------------------
  500 IFB=0
      RETURN
c
c  normal exit
c  -----------
  300 IF(NDEBUG.NE.0) WRITE(NDEBUG,1300) GXNAL,GXNAM,IFB,PB1,PB2
 1300 FORMAT(1X,'VALENCE PARAMETERS FROM CALVAL ',2(2X,A2),I3,
     +  F6.3,F5.2)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE FINDEL(GNA,IX,II,IER)
C
C   *****SUBROUTINE*****     FINDEL
C                            ------
C
C   WRITTEN 81.7.23 BY I D BROWN
C  MODIFIED 82.7.12 BY OLIVER SLUPECKI
C
C   FINDS THE ELEMENT GNA IN THE ARRAY GEL.  IF IT IS NOT FOUND  IT
C   ADDS THE NAME GNA INTO GEL.  NJ IS E SIZE OF GEL.  II IS
C   THE POSITION OF NAME GNA IN GEL. IF IER.GT.0 GEL IS FULL
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,DDMIN,
     1             RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/GRANGE/GEL(20)
C
      IF(NJ.EQ.0) GO TO 30
      DO 20 I=1,NJ
	 II = I
	 IF(GNA.EQ.GEL(I).AND.IX.EQ.IOX(I))RETURN
   20    CONTINUE
   30 NJ = NJ+1
      IF(NJ.GT.20) GO TO 900
      II = NJ
      GEL(NJ) = GNA
      IOX(NJ) = IX
      RETURN
  900 IER = IER + 1
      NJ = NJ - 1
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
***********************************************************************
C
C
      SUBROUTINE BONRNG
C                ======
C
C   WRITTEN 82.1.5 BY I D BROWN
C   MODIFIED 82.7.12 BY OLIVER SLUPECKI
C   MODIFIED 85.5.8 BY DAVID KNIGHT
C
C   SETS RANGES FOR DISTANCES THAT REPRESENT BONDS
C   (REF. BROWN & ALTERMATT ACTA CRYST A (1985))
C
C   CALLS  ABOST, CHEMNO, FINDEL
C
C   USES   INTEGER, MTRANS
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      include 'comatom.for'
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,DDMIN,
     +             RMAX,RMIN,VMAX,VMIN,ANGLX
      COMMON/GRANGE/GEL(20)
      COMMON/SWITCH/JFIX,ERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL
      INTEGER ERRA
      DIMENSION OC(20)
C
C   IRFLAG COUNTS THE NON-FATAL ERRORS, IFFLAG THE FATAL ERRORS
C
      IRFLAG=0
      IFFLAG=0
      IF(HALL.EQ.' ') IFFLAG=IFFLAG+1
C
C   THE MINIMUM BOND VALENCE CALCULATED IS VMIN*(OXID(IX))
C
C   SET SPECIAL LIMITS FOR CATION-CATION BONDS IF IFTYP=1
C
C   DNN = MAX DIST FOR N-N BONDS, DOO = MAX DIST FOR O-O BONDS
C   DNCME = MAX DIST FOR BONDS FROM C OR N TO METALS
C   SMIN = VALENCE FOR SHORTEST BOND
C   SMAX = VALENCE FOR LONGEST BOND
C
      VMAX = 0.038
      VMIN = 1.2
      DNN = 2.0
      DOO = 2.0
      DNCME = 2.8
      IFTYP = 1
C
C   SET INITIAL VALUES OF TOP, BOT
C
      DO 50 I=1,20
	DO 50 J=1,20
	  TOP(I,J) = 0.
   50     BOT(I,J) = 1.0
C
C   LOAD ELEMENTS FROM GELEM INTO GEL
C
*     NJ=0
      DO 100 I=1,NX
	IER=0
	IOXID = INTEGER(OXID(I),0)
	CALL FINDEL(GELEM(I),IOXID,II,IER)
	IF(IER.NE.0) GO TO 300
	OC(II) = OCCU(I)
  100   CONTINUE
C
C   SET LIMITS IN EACH ELEMENT OF ARRAY TOP AND BOT
C
  300 DO 410 I=1,NJ
	CALL CHEMNO(GEL(I),NPI,NZI)
  302   CONTINUE
	DO 410 J=1,NJ 
	  SMIN = 0.
	  SMAX = 0.5
	  CALL CHEMNO(GEL(J),NPJ,NZJ)
c Reset group for rare earths
	  if (nzj.ge.20) nzj=nzj-20
c
c DEAL FIRST WITH BORANES
c -----------------------
	  if(gel(i).eq.'B'.and.gel(j).eq.'B') then
	    top(i,j) = 2.22
	    bot(i,j) = 1.00
	    go to 410
	  else if((gel(i).eq.'B'.and.gel(j).eq.'H').or.
     +            (gel(i).eq.'H'.and.gel(j).eq.'B')) then
	    top(i,j) = 1.45
	    bot(i,j) = 0.80
	    go to 410
	  endif
	  IF(IOX(I).EQ.0.OR.IOX(J).EQ.0) THEN
C CASE D: AT LEAST ONE ATOMS HAS OX = 0
C         -----------------------------
	    SMAX = 0.5
	    SMIN = 2.0
	    TOP(I,J)=3.4
	    BOT(I,J)=1.0
	    GO TO 380
	  ELSE
C         BOTH ATOMS HAVE NON ZERO OXIDATION STATE
C         ----------------------------------------
C           N.B. IFANG IS SET TO 2 IN BONDRD SO THAT ANGLIM IS CALLED IN
C           THIS CASE
	    IF(IOX(I).LT.0.OR.IOX(J).LT.0) THEN
C           AT LEAST ONE ATOM IS AN ANION
C           -----------------------------
	      IF(IOX(I).GT.0.OR.IOX(J).GT.0) THEN
C CASE A:     ONE ATOM IS A CATION THE OTHER AN ANION
C             ----------------------------------------
		IF(IOX(I).GT.0) THEN
		  KI=I
		  KJ=J
		  NZK=IOX(I)
		  IF(NZI.GT.11) NZK=NZI-10
		ELSE
		  KI=J
		  KJ=I
		  NZK=IOX(J)
		  IF(NZJ.GT.11) NZK=NZJ-10
		  ENDIF
		SMAX = VMAX*IOX(KI)
		SMIN = (1.+0.1*NZK)*IABS(IOX(KJ))
C    SPECIAL CASES:  H, C.N, C.O, MO ETC
		IF(NZI.EQ.5.OR.NZI.EQ.6
     1             .or.nzj.eq.5.or.nzj.eq.6) SMAX=0.1
		IF(GEL(KI).EQ.'C '.AND.(GEL(KJ).EQ.'N '.OR.
     +           GEL(KJ).EQ.'O ')) SMIN = 3.0*VMIN
		IF(GEL(KI).EQ.'H ') SMIN=2.0
		TOP(I,J) = 3.4
		BOT(I,J) = 1.0
		GO TO 380
	      ELSE
C CASE B:     BOTH ATOMS ARE ANIONS
C             ---------------------
		SMIN = 0.
		IF((NZI-18-IOX(I)).LT.0.AND.(NZJ-18-IOX(J)).LT.0) THEN
C CASE B1:      BOTH ATOMS HAVE SUBNORMAL OXIDATION STATES
C               ------------------------------------------
		  SMAX = 0.5
		  SMIN = 3.0
		  TOP(I,J) = 2.5
		  BOT(I,J) = 1.0
		  GO TO 380
		  ENDIF
		ENDIF
	    ELSE
C           BOTH ATOMS ARE CATIONS
C           ----------------------
	      SMIN = 0.
	      IF(MTRANS(NZI).GT.0.OR.MTRANS(NZJ).GT.0) THEN
C CASE C2:    ONE ATOM IS A TRANSITION ELEMENT
C             --------------------------------
		IF(MTRANS(NZI).GT.0.AND.MTRANS(NZJ).GT.0) THEN
C CASE C2A:     BOTH ATOMS ARE TRANSITION ELEMENTS
C               ----------------------------------
		  IF((NZI-IOX(I)).GT.0.AND.(NZJ-IOX(J)).GT.0) THEN
C                 BOTH ATOMS HAVE SUBNORMAL OXIDATION STATES
C                 -------------------------------------------
		    SMAX = 0.15*AMAX0(IOX(I),IOX(J))
		    SMIN = 2.00*AMIN0(IOX(I),IOX(J))
		    TOP(I,J) = 2.0 + 0.075*NPI + 0.075*NPJ
		    BOT(I,J) = 1.0
		    GO TO 380
		    ENDIF
		ELSE
C               ONE ATOM IS A TRANSITION ELEMENT, ONE A MAIN GROUP
C               --------------------------------------------------
		  IOXM=10
		  IF(MTRANS(NZI).GT.0) THEN
C                 J IS THE MAIN GROUP ELEMENT
C                 ---------------------------
		    IF(NZJ-IOX(J)-10.GT.0) THEN
C                   AND IT HAS LOWER OXIDATION STATE
C                   --------------------------------
		      IOXM=IOX(J)
		      NPM=NPJ
		      NPT=NPI
		      ENDIF
		  ELSE
C                 I IS THE MAIN GROUP ELEMENT
C                 ---------------------------
		    IF(NZI-IOX(I)-10.GT.0) THEN
C                   AND IT HAS LOWER OXIDATION STATE
C                   --------------------------------
		      IOXM=IOX(I)
		      NPM=NPI
		      NPT=NPJ
		      ENDIF
		    ENDIF
		  IF(IOXM.LT.10) THEN
		    SMAX=VMAX*IOXM
		    SMIN=1.5
		    TOP(I,J)=1.4 + 0.1*NPM + 0.2*NPT
		    BOT(I,J)=1.0
		    ENDIF
		  ENDIF
		IF((GEL(I).EQ.'C '.AND.IOX(I).EQ.2).OR.
     +           (GEL(J).EQ.'C '.AND.IOX(J).EQ.2)) THEN
C CASE C2B      ONE ATOM IS C(+2)
C               -----------------
C                 N.B. IN THIS CASE ANGLIM IS CALLED IN BONCHK
		  IOXM=IOX(I)
		  IF(GEL(I).EQ.'C ') IOXM=IOX(J)
		  SMAX = VMAX*IOXM
		  SMIN = VMIN*3.0
		  TOP(I,J) = 3.0
		  BOT(I,J) = 1.0
		  IF(GEL(J).EQ.'C ') SMAX = VMAX*IOX(I)
		  ENDIF
	      ELSE
C CASE C3     BOTH ATOMS ARE MAIN GROUP ELEMENTS
C             ----------------------------------
		IF((NZI-10-IOX(I)).GT.0.AND.(NZJ-10-IOX(J)).GT.0) THEN
C CASE C3A      BOTH ATOMS ARE IN LOWER OXIDATION STATES
C               ----------------------------------------
C                 N.B. IF ONE ATOM IS C(+2) ANGLIM IS CALLED IN BONCHK
		  SMAX = 0.15*AMAX0(IOX(I),IOX(J))
		  SMIN = 2.0*AMIN0(IOX(I),IOX(J))
		  TOP(I,J) = 0.9+0.2*NPI+0.2*NPJ
		  BOT(I,J) = 1.0
		else if(gel(i).eq.'C '.and.gel(j).eq.'C ') then
C CASE C3B      BOTH ATOMS ARE CARBON (organic)
C               -------------------------------
		  smax=0.8
		  smin=3.5
		  top(i,j)=1.7
		  bot(i,j)=1.0
		else if((gel(i).eq.'C '.and.gel(j).eq.'H ').or.
     +                  (gel(i).eq.'H '.and.gel(j).eq.'C ')) then
c               ONE ATOM IS C, THE OTHER H
C               --------------------------
		  smax=0.7
		  smin=1.3
		  top(i,j)=1.3
		  bot(i,j)=0.7
		  ENDIF
		ENDIF
	      ENDIF
	    ENDIF
C SET BOND LIMITS ACCORDING TO THE BOND VALENCES SMAX, SMIN
  380     CONTINUE
	  SMAX = ABS(SMAX)
	  SMIN = ABS(SMIN)
	  IF(NDEBUG.NE.0) WRITE(NDEBUG,1380) GEL(I),IOX(I),GEL(J),
     +     IOX(J),SMIN,SMAX
 1380     FORMAT(' **DEBUG**BONRNG**1380**  BOND BETWEEN ATOM ',A2,I2,
     +     ' AND ',A2,I2,', SMIN =',F6.3,' AND SMAX =',F6.3)
	  IF(TOP(I,J).LT.0.01) GO TO 410
	    IF(SMIN.GT.0.001) THEN
	      CALL ABOST(GEL(I),GEL(J),IOX(I),IOX(J),SMAX,TX)
	      CALL ABOST(GEL(I),GEL(J),IOX(I),IOX(J),SMIN,BX)
	      IF(TX.GT.0.01)THEN
		TOP(I,J)=TX
		BOT(I,J)=BX
		ENDIF
	      IF(BOT(I,J).LT.0.001) BOT(I,J)=0.001
	    ENDIF
c
c  SET A REALISTIC LOW LIMIT FOR BONDS TO H ATOMS
c
	 if(gel(i).eq.'H '.or.gel(j).eq.'H ') bot(i,j)=0.6
  410    CONTINUE
      RETURN
      END
c
c
***********************************************************************
c
c
      function ifnumb(gelem)
c
c     returns the 'F-number' for rare earths and actinides
c     (element name =gelem=).
c
c     written by D. Altermatt
c     McMaster University, Nov. 1984
c
      character*8 gelem,gel(14,2)
c
      data gel/'CE','PR','ND','PM','SM','EU','GD','TB','DY','HO',
     1         'ER','TM','YB','LU','TH','PA','U ','NP','PU','AM',
     2         'CM','BK','CF','ES','FM','MD','NO','LR'/
c
      ifnumb=0
      do 10 i=1,2
	do 10 j=1,14
10        if(gelem.eq.gel(j,i)) ifnumb=j
      return
      end
c
c
***********************************************************************
c
c
      function picwgt(i,j,k)
c
c     returns the atomic weight of the element with group number j,
c     period number i and 'F-number' k
c
c     written by D. Altermatt
c     McMaster University, Nov. 1984
c
      dimension wgtm(18,7),wgtr(14,2)
      data wgtm/1.01,16*0.0,4.0,6.94,9.01,10*0.0,10.81,12.01,14.01,
     1         16.0,19.0,20.18,22.99,24.31,10*0.0,26.98,28.09,30.97,
     2         32.06,35.45,39.95,39.10,40.08,44.96,47.90,50.94,52.0,
     3         54.94,55.85,58.93,58.71,63.55,65.38,69.72,72.59,74.92,
     4         78.96,79.90,83.80,85.47,87.62,88.91,91.22,92.91,95.94,
     5         98.91,101.07,102.91,106.4,107.87,112.4,114.82,118.69,
     6         121.75,127.6,126.9,131.3,132.91,137.34,138.91,178.49,
     7         180.95,183.85,186.2,190.2,192.22,195.09,196.97,200.59,
     8         204.37,207.2,208.98,210,210,222,223,226.03,227,15*0.0/
      data wgtr/140.12,140.91,144.24,147,150.4,151.96,157.25,158.93,
     1         162.5,164.93,167.26,168.93,173.04,174.97,232.04,231.04
     2        ,238.03,237.05,244,243,247,247,252,254,257,257,255,256/
c
      picwgt=0.0
      if(i.gt.7.or.j.gt.18) goto 99
      if(k.eq.0) then
	picwgt=wgtm(j,i)
      else
	if(i.lt.6.or.k.gt.14) goto 99
	n=i-5
	picwgt=wgtr(k,n)
	endif
99    return
      end
c
c
***********************************************************************
c
c
      function volget(gel)
c
c     returns the theoretical volume of anion =gel=
c
c     written by D. Altermatt
c     McMaster University, Nov. 1984
c
      implicit character*8 (g)
      dimension g1(10),vol(10)
      data g1/'O ','F ','CL','BR','I ','N ','S ','TE','SE','  '/
      data vol/9.0,9.41,24.83,31.53,44.60,13.03,26.09,45.21,
     1         32.51,0.0/
c
      volget=0.0
      do 10 i=1,10
	if(gel.ne.g1(i)) goto 10
	volget=vol(i)*2.4
10      continue
      return
      end
C
C
***********************************************************************
C
C
      FUNCTION ELMWGT(GNAM)
C              ======
C
C   WRITTEN 85.6.5 BY DAVID KNIGHT
C
C   THIS FUNCTION RETURNS THE ATOMIC WEIGHT OF GNAM
C
C   CALLS CHEMNO
C   USES PICWGT,IFNUMB
C
      IMPLICIT CHARACTER*8 (G)
      CALL CHEMNO(GNAM,NP,NG)
      ELMWGT = PICWGT(NP,NG,IFNUMB(GNAM))
      RETURN
      END
C
c
***********************************************************************
c
      subroutine qsdere(sumnv,delrad,delel)
c
c   written 88.1.22 by I.D.Brown
c
c   determines the quantum structural parameters for a ternary compound
c   according to the method of Villars (86 Phys Rev Letters 57, 3085)
c
c   sumnv = valence electron count
c   delrad= difference in pseudopotential radii
c   delel = difference in electronegativities
c
c   calls CHEMLB: villec, vilrad, chemno, forman
c   calls MATHLB: sorty, wait
c
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/ nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      common/form/ nform(3),mform,form(24,3)
      common/gform/ gform(24,3)
      common/htext/ gcryst(2),gtwin(5),htext(10,5),hdata(4,19)
c  the dimensions of 'is' are 'mform'
      dimension is(24)
c
c  sort formula into decreasing element concentration
c  --------------------------------------------------
      if(nform(1).eq.0) then
	if(hdata(1,5)(1:5).eq.'     ') then
	  write(nout,1000)
 1000     format(' *error*qsdere*1000* Formula not given, calculation',
     +     ' impossible.')
	  return
	  endif
	call forman(hdata(1,5),gform,form,nform)
	call redfor(gform,form,nform)
	endif
      if(ndebug.ne.0) write(ndebug,1100) (gform(i,1),form(i,1),
     +        i=1,nform(1))
 1100 format(' *debug*qsdere*1100* Formula found is ',24(1X,A2,F6.2))
      call sorty(nform,form,is)
      if(nform(1).ge.3) then
	n=nform(1)-3
      else
	write(nout,1200)
 1200   format(' *error*qsdere*1200* fewer than 3 elements. ',
     + ' Calculation impossible')
	if(nterm.ne.0) call wait
	return
	endif
      sum=0.
      do 100 i=1,3
  100   sum=sum+form(is(i),1)
      fa=form(is(3),1)/sum
      fb=form(is(2),1)/sum
      fc=form(is(1),1)/sum
      ga=gform(is(3),1)
      gb=gform(is(2),1)
      gc=gform(is(1),1)
c
c  calculate electronegativity difference
c  --------------------------------------
      elc=villec(gc)
      elb=villec(gb)
      ela=villec(ga)
      delel=2*(fa*(2*ela-elb-elc)+
     +                  fb*(elb-elc))
c
c  calculated radius difference
c  ----------------------------
      radc=vilrad(gc)
      radb=vilrad(gb)
      rada=vilrad(ga)
      delrad=2*(fa*(2*rada-radb-radc)+
     +                  fb*(radb-radc))
c
c  calculate electron number
c  -------------------------
      call chemno(gc,np,nvc)
      call chemno(gb,np,nvb)
      call chemno(ga,np,nva)
      if(nvc.gt.12) nvc=nvc-10
      if(nvb.gt.12) nvb=nvb-10
      if(nva.gt.12) nva=nva-10
      sumnv=fa*nva+fb*nvb+fc*nvc
      if(ndebug.ne.0) write(ndebug,1300) ga,fa,gb,fb,gc,fc,nva,nvb,nvc,
     +  rada,radb,radc,ela,elb,elc
 1300 format(' *debug*qsdere*1300* ',3(A2,1X,F4.1,1X),3I3,6F5.2)
      return
      end
c
c
c
***********************************************************************
C
C
      function villec(gel)
c
c   written 88.1.20 by I.D.Brown
c
c   returns Villars electronegativity for element gel
c
c   calss mathlb: upcase
c
      implicit character*8 (g)
      dimension gdat(95)
      dimension vdat(95)
      data gdat/'H','HE','LI','BE','B','C','N','O','F','NE',
     1 'NA','MG','AL','SI','P','S','CL','AR',
     2 'K','CA','SC','TI','V','CR','MN','FE','CO','NI','CU','ZN',
     3          'GA','GE','AS','SE','BR','KR',
     4 'RB','SR','Y','ZR','NB','MO','TC','RU','RH','PD','AG','CD',
     5          'IN','SN','SB','TE','I','XE',
     6 'CS','BA','LA','CE','PR','ND','PM','SM','EU','GD','TB','DY',
     7   'HO','ER','TM','YB','LU','HF','TA','W','RE','OS','IR','PT',
     8 'AU','HG','TL','PB','BI','PO','AT','RN',
     9 'FR','RA','AC','TH','PA','U','NP','PU','AM'/
      data vdat/2.10,0,0.90,1.45,1.90,2.37,2.85,3.32,3.78,0,
     1 0.89,1.31,1.64,1.98,2.32,2.65,2.98,0,
     2 0.80,1.17,1.50,1.86,2.22,2.00,2.04,1.67,1.72,1.76,1.08,1.44,
     3           1.70,1.99,2.27,2.54,2.83,0,
     4 0.80,1.13,1.41,1.70,2.03,1.94,2.18,1.97,1.99,2.08,1.07,1.40,
     5           1.63,1.88,2.14,2.38,2.76,0,
     6 0.77,1.08,1.35,1.1 ,1.1 ,1.2 ,1.15,1.2 ,1.15,1.1 ,1.2 ,1.15,
     7   1.2 ,1.2 ,1.2 ,1.1 ,1.2 ,1.73,1.94,1.79,2.06,1.85,1.87,1.91,
     8 1.19,1.49,1.69,1.92,2.14,2.40,2.64,0,
     9 0.70,0.90,1.10,1.3 ,1.5 ,1.7 ,1.3 ,1.3 ,1.3 /
      gl = gel
      call upcase(gl)
      villec=0.
      do 100 i=1,95
	if(gl.eq.gdat(i)) then
	  villec=vdat(i)
	  return
	  endif
  100   continue
      return
      end
c
c
***********************************************************************
c
      function vilrad(gel)
c
c   written 88.1.20 by I.D.Brown
c
c   returns Villars psuedopotential radius
c
c   calls mathlb: upcase
c
      implicit character*8 (g)
      dimension gdat(95)
      dimension vdat(95)
      data gdat/'H','HE','LI','BE','B','C','N','O','F','NE',
     1 'NA','MG','AL','SI','P','S','CL','AR',
     2 'K','CA','SC','TI','V','CR','MN','FE','CO','NI','CU','ZN',
     3          'GA','GE','AS','SE','BR','KR',
     4 'RB','SR','Y','ZR','NB','MO','TC','RU','RH','PD','AG','CD',
     5          'IN','SN','SB','TE','I','XE',
     6 'CS','BA','LA','CE','PR','ND','PM','SM','EU','GD','TB','DY',
     7   'HO','ER','TM','YB','LU','HF','TA','W','RE','OS','IR','PT',
     8 'AU','HG','TL','PB','BI','PO','AT','RN',
     9 'FR','RA','AC','TH','PA','U','NP','PU','AM'/
      data vdat/1.25,0,1.61,1.08,0.795,0.64,0.54,0.465,0.405,0,
     1 2.65,2.03,1.675,1.42,1.24,1.10,1.01,0,
     2 3.69,3.00,2.75,2.58,2.43,2.44,2.22,2.11,2.02,2.18,2.04,1.88,
     3           1.695,1.56,1.415,1.285,1.20,0,
     4 4.10,3.21,2.94,2.825,2.76,2.72,2.65,2.605,2.52,2.45,2.375,2.215,
     5           2.05,1.88,1.765,1.67,1.585,0,
     6 4.31,3.402,3.08,4.50,4.48,3.99,3.99,4.14,3.94,3.91,3.89,3.67,
     7   3.65,3.63,3.60,3.59,3.37,2.91,2.79,2.735,2.68,2.65,2.628,2.70,
     8 2.66,2.41,2.235,2.09,1.997,1.90,1.83,0,
     9 4.37,3.53,3.12,4.98,4.96,4.72,4.93,4.91,4.89/
      vilrad=0.
      gl=gel
      call upcase(gl)
      do 100 i=1,95
	if(gdat(i).eq.gl) then
	  vilrad=vdat(i)
	  return
	  endif
  100   continue
      return
      end
C
C
***********************************************************************
C
      FUNCTION RADIUS(GELM,OXID,IC)
C              ======
C
C   WRITTEN 85.6.10 BY DAVID KNIGHT
C
c   This function returns the atomic radius of GELM(OXID) with coordination
c   number IC.   The radius are based on Oxygen=1.40.  The anion radii are
c   listed, the cation radii are calculated based on the bond valence.
c   "True" radii will have values about 0.6A greater for cations and 0.6A
c   less for anions.  Non-bonded radii can be found by adding 0.62 to these
c   "true" radii.
C
c 
C   UNKNOWN CATIONS WILL GET A RADIUS OF 1.0 ANGSTROMS.
C   UNKNOWN ANIONS WILL GET A RADIUS OF 1.9 ANGSTROMS.
C
C   CALLS DBOST, LDVAL
C
      IMPLICIT CHARACTER*8 (G)
      GELO = 'O '
      OXO = -2.0
      ROXID = 1.40
      RADIUS = 1.9
      icn=ic
      IF(ICN.LT.1.OR.ICN.GT.100) then
	call chemno(gelm,np,ng)
	ICN = icoor(np,ng)
	endif
      IF(OXID.GE.0.) THEN
10      RADIUS = DBOST(GELM,GELO,OXID,OXO,(OXID/ICN))
	IF(RADIUS.LT.0.1) THEN
	  IER = 0
	  CALL LDVAL(GELM,GELO,OXID,OXO,IER)
	  IF(IER.NE.0) THEN
	    RADIUS = 1.0
	    RETURN
	  ELSE
	    radius=dbost(gelm,gelo,oxid,oxo,(oxid/icn))
	    ENDIF
	  ENDIF
	RADIUS = RADIUS - ROXID
	IF(GELM.EQ.'H '.AND.RADIUS.LT.0.1) RADIUS = 0.1
	if(radius.lt.0.01) radius=1.0
      ELSEIF(OXID.LT.0.) THEN
	IF(GELM.EQ.'C ') RADIUS = 1.40
	IF(GELM.EQ.'O ') RADIUS = ROXID
	IF(GELM.EQ.'N ') RADIUS = 1.49
	IF(GELM.EQ.'P ') RADIUS = 1.70
	IF(GELM.EQ.'S ') RADIUS = 1.89
	IF(GELM.EQ.'SE') RADIUS = 2.00
	IF(GELM.EQ.'TE') RADIUS = 2.20
	IF(GELM.EQ.'F ') RADIUS = 1.29
	IF(GELM.EQ.'CL') RADIUS = 1.78
	IF(GELM.EQ.'BR') RADIUS = 2.03
	IF(GELM.EQ.'I ') RADIUS = 2.20
	ENDIF
      RETURN
      END
c
c
***********************************************************************
c
c
      function acid(np,ng,oxid)
c
c     calls integer
c
c     returns the lewis acid strength for bond length estimation
c     purposes
c
c     written by I.D.Brown
c     McMaster University, Dec. 1984
c     Lewis acid strengths taken from Acta Cryst (1988) B44 545
c     Rare earths and actinides are in groups 21-34 and are treated
c     identically.
c
      dimension ac(18,7)
      data ac/.8,17*1.0,  
     2 .205, .501,    10*1.0,     .87, 1.35, 1.67, 2.1, 2.6, 1.0,
     3 .156, .334,   10*1.0,      .57, 1.00, 1.25, 1.5, 1.75, 1.0,  
     4 .126, .274, .49, .67, 1.08, 1.50, 1.33, 2.22, .51, .34,
     4                 .45, .402, .65, .89,  1.13, 1.5, 1.6, 1.0,
     5 .124, .233, .43, .60, .833, 1.23, 1.51, 1.97, .50, .46,
     5                 .20, .326, .50, .68, .83,   1.0, 1.2, 1.0,
     6 .113, .195, .35, .60, .832, 1.07, 1.51, 1.97, .50, .46, 
     6                 .20, .360, .49, .70, .64, 3*1.0,  
     7 .070, .18,  .33, 15*1.0/
c
c
      if(ng.lt.20) then
	acid=ac(ng,np)
	if(ng.le.8) then
	  j=ng-integer(oxid,0)
	  if(j.gt.0) then
	    do 100 i=1,j
100           acid=acid*.75
	    endif
	  endif
      else
	acid = 0.354 + 0.0066*(ng-20)
	endif
      return
      end
c
***********************************************************************
c
      function acid1(g1,g2,ox1,ox2)
c
c
c   written 1987.7.1 by D.Lawrence
c
c   similar to acid but with a different algorithm
c
c   calls pcoord amd rcoord
c
      pcn=pcoord(g1,g2,ox1,ox2)
      cn=rcoord(g1,pcn,ox1)
      acid1=ox1/cn
      return
      end
c
c
c
c
      function base(np,ng,oxid,str,nc)
c
c     returns the lewis base strength for element -np,ng- in
c     oxidation state -oxid-, assuming that it already has
c     formed -nc- bonds with a total strength -str-
c
c     written by I.D. Brown
c     McMaster University, Jan. 1985
c     updated June 1987 by D.Lawrence
c
      common /files/ nin,nout,ndebug,nread,nwrite,nspgr
      if (nc.ge.2*np) then
	base=oxid-str
	return
	end if
      base=(oxid-str)/(2*np-nc)
      return
      end
c
c
***********************************************************************
c
c
      function contac(np1,np2)
c
c     returns a proposal for a contact distance between an element
c     of period np1 and an element of period np2
c
c     written by D. Altermatt
c     McMaster University, Dec. 1984
c
      nn=1.5
      contac=2*nn+np1/5+np2/5
      return
      end
c
c
***********************************************************************
c
c
      SUBROUTINE CLASS(ICMPD)
C
C   *****SUBROUTINE*****    CLASS
C                           -----
C
C   WRITTEN 83.2.9 BY I D BROWN
C   SETS ICMPD = 1 FOR METALS
C              = 2 FOR INORGANIC COMPOUNDS
C              = 3 FOR ORGANIC COMPOUNDS (NOT YET WRITTEN)
C   USES INTEGER
C
      implicit character *8 (g)
C
      include 'comatom.for'
C
      METAL = 0
      INORG = 0
      ICMPD = 2
      INFLAG= 0
      IHFLAG= 0
      DO 50 I=1,NX
	nzz = nza(2,i)
	if(nzz.ge.20) nzz = 3
C
C   TEST FOR ELEMENT TO LEFT OF ZINTL LINE
C
	IF((nzz-NZA(1,I)).LE.10) METAL = 1
C
C   TEST FOR HALOGEN, S OR O
C
	TEMP1 = NZA(1,I)/2
	TEMP2 = nzz
	IF((TEMP2-TEMP1).GT.14.2) INORG = 1
C   TEST FOR CN
	IF(GELEM(I).EQ.'C '.AND.INTEGER(OXID(I),0).EQ.2) INORG = 1
C
C   TEST FOR H
C
	IF(GELEM(I).EQ.'H '.OR.GELEM(I).EQ.'D ') INORG=1
   50 CONTINUE
      IF(METAL.EQ.1.AND.INORG.EQ.0) ICMPD = 1
      RETURN
      END
c
c
***********************************************************************
c
c
      FUNCTION MTRANS(I)
C     ------------------
C
C   WRITTEN 84.7.19 BY I.D.BROWN
C   RETURNS THE VALUE +1 IF THE ATOM WHOSE GROUP IS I IS A TRANSITION
C   METAL AND -1 IF IT IS A MAIN GROUP ELEMENT
c   Group 12 is regarded as main group
C   USED IN BONRNG
C
      MTRANS=-1
      IF(I.LT.12.AND.I.GT.2) MTRANS=1
      RETURN
      END
C
c
***********************************************************************
c
      subroutine formul(hform,gelem,iform,n)
c
c   Written 85.10.4 by I.D.Brown /July 1986 by N.Heinig
c
c   converts the formula given by gelem(n) and iform(n) to hform
c
c   calls integer, upcase, length
c
      implicit character*8(g)
      implicit character*80(h)
      common/files/ nin,nout,ndebug
      dimension gelem(n),gnum(10),galph(26)
      dimension iform(n), form(n)
      data gdot/'.'/
      data galph /'a','b','c','d','e','f','g','h','i','j','k',
     *     'l','m','n','o','p','q','r','s','t','u','v','w','x',
     *     'y','z' /
c
      iflag = 1
      goto 1
c
c  Entry point for formulae with real values
c
      entry formla(hform,gelem,form,n)
      iflag = 0
c
    1 do 10 i=1,80
   10   hform(i:i)=' '
      k=0
      do 100 i=1,n
	gel=gelem(i)
	if(gel(2:2).eq.',') gel(2:2)=' '
	k=k+1
	hform(k:k)=gel(1:1)
	k=k+1
	if(gel(2:2).ne.' ') then
c
c  Convert second letter of element name to lower case.
c
	  if(gel(2:2).ge.'A' .and. gel(2:2).le.'Z')then
	    do 110 j=1,26
	      galphu = galph(j)
	      call upcase( galphu)
	      if(gel(2:2) .eq. galphu) then
		gel(2:2) = galph(j)
		goto 111
		endif
  110         continue
	    endif   
  111     hform(k:k)=gel(2:2)
	  k=k+1
	  endif
c
c  Check if too many atoms are stored.
c
	istart=0
	if(iflag .eq. 1) then
	  ifor=iform(i)
	  if(ifor.gt.999) then
	    write(nout,1000)
 1000       format(' *error*formul*1000* Too many atoms in formula')
	    ifor=999
	    endif
	elseif(iflag .eq. 0)then
	  for = form(i)
	  if(for.gt.999.) then
	    write(nout,1000)
	    for = 999.
	    endif
	  endif
c
c  Store real values in hform.
c
	if(iflag.eq.0) then
	  if (for .eq. 1.) then
	    goto 101
	  else
	    write(htemp, 1010) for
 1010       format(F7.2)
	    read(htemp, 1011) gfor    
 1011       format(A7)
	    call upcase(gfor)
	    leng = length(gfor)
	    hform(k:k+leng) = gfor(1:leng)
	    k = k+leng
	    goto 101
	    endif
	else
c
c  Store integer values in hform
c 
	  if(ifor .eq. 1) then
	    goto 101
	  else
	    write(htemp, 1110) ifor
 1110       format(I5)
	    read(htemp,1111) gfor
 1111       format(A5)
	    call upcase(gfor)
	    leng = length(gfor)
	    hform(k:k+leng) = gfor(1:leng)
	    k = k + leng
	    endif
	  endif
  101   if(k .gt. 80) return
  100   continue
      return
      end
C
C
***********************************************************************
c
c
c
      function hformc(gel,nel,n)
c
c   written 86.11.28 by I.D.Brown
c
c   Returns the formula of the group of n atoms whose element names are
c   given in gel(n) and whose frequency is given in nel(n).  The list may
c   contain the same element several times and typically nel=1.
c
c   calls FORMUL
C
      implicit character*80 (h)
      implicit character*8  (g)
      dimension gel(n),nel(n),gfc(10),nfc(10)
      nf=0
      do 255 k=1,10
  255   nfc(k)=0
      do 280 i=1,n
	do 260 k=1,nf
	  if(gel(i).eq.gfc(k)) then
	    nfc(k)=nfc(k)+nel(i)
	    go to 280
	    endif
  260     continue
	  nf=nf+1
	  if(nf.le.10) then
	    gfc(nf)=gel(i)
	    nfc(nf)=nel(i)
	    endif
  280   continue
      if(nf.gt.10) nf=10
      call formul(hformc,gfc,nfc,nf) 
      return
      end
C
C
***********************************************************************
C
      function F000(dummy)
C
C WRITTEN JULY 2, 1986 BY NINA HEINIG
C
C SUMS THE ATOMIC NUMBERS IN THE UNIT CELL TO PRODUCE FZERO,
C USING ZFORM AND FORM.
C
C CALLS IATOMNO
C
      IMPLICIT CHARACTER*8 (G)
      include 'comatom.for'
      common /dens/   zform,dens,ddens,cdens,fwgt,waveln,absorb,
     1                  fzero,tmprtr
      common /form/ nform(3), mform, form(24,3)
      common /gform/ gform(24,3)

      f000 = 0.
      do 17 i=1,nform(1)
	n=iatomno(gform(i,1))
	f000=f000+n*form(i,1)*zform
   17   continue
      return
      end
C
c
***********************************************************************
c
      SUBROUTINE DECODEFM(hfor)
c
c   written 91.11.28 by I.D.Brown
c
c    decodes the formula in hfor and stores the atoms in the XATOM
c     array.  The atoms are not stored as dummy atoms, i.e., all
c     occupation numbers are 1.0 and every atom in the formula unit
c     appears as a separate atom in the array
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      include 'comatom.for'
      common /files/ nin,nout,ndebug
      dimension gel(24), gatom(50)
      dimension xel(24), xoxid(50)
c
      nel = 0
      do 100 i=1,24
	gel(i) = '           '
	xel(i) = 0.0
  100   continue
      call forman(hfor,gel,xel,nel)
c
c   Store atoms 
c   -----------
      ii = 0
      do 220 i=1,nel
	iel = integer(xel(i),0)
	do 230 j = 1,iel
	  ii = ii + 1
	  if(ii.gt.mx) then
	    write(nout,1220) mx
 1220       format(' *warning*modnet*1220*  More than',I5,' atoms ',
     1      'generated.  Remainder ignored')
	    call wait
	    ii = ii - 1
	    go to 240
	    endif
	  gelem(ii) = gel(i)
	  gdent(ii) = gint(j)
	  call upcase(gdent(ii))
	  occu(ii) = 1.
	  npos(ii) = 1.
  230     continue
  220   continue
  240 nx = ii
c
c   Calculate the oxidation states
c   ==============================
      call chemnum
      call oxide(gatom,xoxid,na)
      if(nx.eq.na) then
	do 250 i=1,nx
  250     oxid(i) = xoxid(i)
      else
	do 270 i =1,nx
	  do 260 j = 1,na
	    if(gelem(i).eq.gatom(j))then
	       oxid(i) = xoxid(j)
	       endif
  260       continue
  270     continue
	endif
  280 write(nout,1280)
 1280 format(' The following oxidation states have been assigned')
      linc = 0
      do 290 i = 1,nx
	llinc = linc + 1
	write(nout,1290) i,gelem(i),gdent(i),oxid(i)
 1290   format(5X,I3,1X,A2,A6,F10.2)
	if(linc.ge.20) then
	  call wait
	  linc = 0
	  endif
  290   continue
      if(linc.ne.0) call wait
  300 write(nout,1300)
 1300 format(' Do you wish to change any oxidation states (Y/N/E) ')
      call readyn(*310,*900,*900,*300)
c     start implicit loop ii
  310   continue
	write(nout,1310)
 1310   format(' Enter the number of the atom to change, ',
     1            'blank to exit ')
	read(nin,1320,err=310) ii  
	if(ii.eq.0) go to 280
 1320   format(I3)
  330   write(nout,1330) gelem(ii),gdent(ii),oxid(ii)
 1330   format(' Enter new oxidation state for atom ',A2,A6,
     1  ' The old oxidation state is ',F6.2)
  340   read(nin,1340,err=330) oxid(ii)
 1340   format(F7.0)
	go to 310
c       end implicit loop ii
  900 return
      end
c
C
***********************************************************************
c
      subroutine forman(hfor,gel,xel,nel)
c
c   written 88.2.11 by I.D.Brown
c   converts chemical formula in hfor into a machine readable format
c   in gel(nel),xel(nel)
c
c   It reads one character at a time
c    MODE indicates the nature of the last character read
c    LETTER counts the number of contiguous letters being read
c    NUMBER counts the number of contiguous digits (or decimal points)
c            read
c    NEOL counts the number of contiguous closing parentheses read
c    LEVEL records the parenthesis level at which the reading is
c            currently occuring
c    LEL(NEL) records the level at which a given element was read
c
      implicit character*80 (h)
      implicit character*8 (g)
      common/files/nin,nout,ndebug
      dimension gel(24),xel(24),lel(24)
c
      nel=0
      number=0
      letter=0
      level=0
      neol=0
      mode=0
      i=0
      do 50 j=1,24
	gel(j)='  '
	xel(j)=0.
   50   continue
c
c   read in next character and set the mode
c
  100 i=i+1
      if(i.ge.80) go to 900
      if(mode.eq.4) neol=neol+1
      if(hfor(i:i).eq.'(') then
	mode=1
	level=level+1
	go to 200
      else if((hfor(i:i).ge.'A'.and.hfor(i:i).le.'Z').or.
     +        (hfor(i:i).ge.'a'.and.hfor(i:i).le.'z')) then
	mode=2
	letter=letter+1
	go to 200
      else if((hfor(i:i).ge.'0'.and.hfor(i:i).le.'9').or.
     +         hfor(i:i).eq.'.') then
	mode=3
	number=number+1
	go to 200
      else if(hfor(i:i).eq.')') then
	mode=4
	go to 200
      else if(hfor(i:i).eq.' ') then
	mode=5
	go to 200
      else
	mode=0
	if(ndebug.ne.0) write(ndebug,1150) hfor(i:i)
 1150   format(' *debug*forman*1150* Unrecognised character is ',A1)
	go to 200
	endif
c
c   take appropriate action according to mode and the flag settings
c
  200 continue
      if(neol.ne.0) then
	if(mode.eq.3) go to 100
	go to 500
      else if(letter.ne.0) then
	if(mode.eq.2) go to 100
	go to 300
      else if(number.ne.0) then
	if(mode.eq.3) go to 100
	go to 400
      else
	if(mode.eq.3.or.mode.eq.4) write(ndebug,1200) hfor(i:i+4)
 1200     format(' *debug*forman*1200* Syntax error at ',A5)
	go to 100
	endif
c
c   store letter
c
  300 continue
      nel=nel+1
      if(nel.gt.24) then
	nel=24
	go to 900
	endif
      is=i-letter
      gel(nel)=hfor(is:i-1)
      xel(nel)=1.0
      lel(nel)=level
      letter=0
      go to 100
c
c   store number
c
  400 continue
      is=i-number
      call efnum(x,ex,hfor(is:i-1),number,0)
      number=0
      xel(nel)=x
      go to 100
c
c    close level
c
  500 continue
      xmult=1.0
      if(number.ne.0) then
	is=i-number
	call efnum(x,ex,hfor(is:i-1),number,0)
	xmult=x
	number=0
	endif
  530 continue
      if(level.le.0) then
	write(nout,1530)
 1530   format(' *error*forman*1530* Unbalanced parentheses')
	endif
      do 550 j=1,nel
	if(lel(j).eq.level) then
	  xel(j)=xmult*xel(j)
	  lel(j)=lel(j)-1
	  endif
  550   continue
      level=level-1
      neol=neol-1
      if(neol.ne.0) go to 530
      go to 100
  900 return
      end
c
c
***********************************************************************
c
      subroutine redfor(gel,xel,nel)
c
c   written 88.2.15 by I.D.Brown
c
c   reduces formula by combining multiple occurances of the same element
c
      implicit character*8 (g)
      implicit character*80 (h)
      dimension gel(nel),xel(nel)
c
      i=0
  100 i=i+1
	if(i.ge.nel) return
	j=i
  200   j=j+1
	  if(j.gt.nel) go to 100
	  if(gel(i).eq.gel(j)) then
	    xel(i)=xel(i)+xel(j)
	    do 300 k=j,nel-1
	      gel(k)=gel(k+1)
	      xel(k)=xel(k+1)
  300         continue
	    nel=nel-1
	    endif
	  go to 200
c         end of j loop
c       end of i loop
      end
C
C
***********************************************************************
C
       subroutine sorfor(gform, xform, oxide, n)
c
c  written 89.6.20 by Lucien Jameison
c
c  The arrays gform(n),xform(n),oxide(n) are sorted in order of
c  increasing Lewis acid strength of the element gform[oxide].
c  xform contains the number of atoms of gform in the formula unit.
c
       common /files/ nin, nout
       include 'comatom.for'
c
       dimension gform(n), xform(n), oxide(n), xacid(100)
c
       do 10 j=1,nn
	 mn = 100
	 nn = n
	 if(n.gt.mn) then
	    nn = mn
	    write(nout,1000) nn
1000        format('*** Warning sorfor * 1000 * Only the first ',I3,/,
     +5X,'atoms of formulae used')
	    endif
	 xacid(j)=acid(nza(1,i),nza(2,i),oxide(i))
  10     continue
       do 150 j=1,n
	 do 140 i=1,n
	   iflag = 0
	   if(xacid(i).gt.xacid(i+1)) then
	     iflag = 1
	     oxidt=oxide(i+1)
	     gformt=gform(i+1)
	     xformt=xform(i+1)
	     oxide(i+1)=oxide(i)
	     gform(i+1)=gform(i)
	     xform(i+1)=xform(i)
	     oxide(i)=oxidt
	     gform(i)=gformt
	     xform(i)=xformt
	     endif
140        continue 
       if(iflag.eq.0) return
150      continue   
       return 
       end
C
C
***********************************************************************
C
      FUNCTION IATOMNO(GELEM)
C
C WRITTEN JULY 2, 1986 BY NINA HEINIG
C
C CALCULATES THE ATOMIC NUMBER, GIVEN THE ELEMENT SYMBOL
C
C CALLS UPCASE
C
      IMPLICIT CHARACTER*8 (G)
      DIMENSION GELM(103)
      DATA GELM / 'H ','HE','LI','BE','B ','C ','N ', 'O ',
     *    'F ','NE','NA','MG','AL','SI','P ','S ','CL','AR',
     *    'K ','CA','SC','TI','V ','CR','MN','FE','CO','NI',
     *    'CU','ZN','GA','GE','AS','SE','BR','KR','RB','SR',
     *    'Y ','ZR','NB','MO','TC','RU','RH','PD','AG','CD',
     *    'IN','SN','SB','TE','I ','XE','CS','BA','LA','CE',
     *    'PR','ND','PM','SM','EU','GD','TB','DY','HO','ER',
     *    'TM','YB','LU','HF','TA','W ','RE','OS','IR','PT',
     *    'AU','HG','TL','PB','BI','PO','AT','RN','FR','RA',
     *    'AC','TH','PA','U ','NP','PU','AM','CM','BK','CF',
     *    'ES','FM','MD','NO','LR' /
C
      IATOMNO = 0
      GELEMT = GELEM
      CALL UPCASE(GELEMT)
      DO 17  N = 1,103
	IF(GELEMT .EQ. GELM(N))THEN
	  IATOMNO = N
	  RETURN
	  ENDIF
  17    CONTINUE
      IF(GELEMT .EQ. 'D ') THEN
	N = 1
      ELSEIF(GELEMT .EQ. 'LW') THEN
	N = 103
      ELSE
	N = 0
	ENDIF
      IATOMNO = N
      RETURN   
      END

***********************************************************************


      FUNCTION LCOORD(G1,G2,OX1,OX2,ACSTR,BASTR)

c
c       written May 1987 by D.Lawrence
c
c      Function to determine best integer coordination from RCOORD
c
      IMPLICIT CHARACTER*8 (G)
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE
c programmer's note:  /bost/ is changed to /bostt/ for
c                     Unix system
      COMMON /BOSTT/ NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      PCN=PCOORD(G1,G2,OX1,OX2)
      CN=RCOORD(G1,PCN,OX1)
      LCN=INT(CN+0.15)
      MCN=INT(CN+1.15)
      IF (LCN.EQ.5) LCN=4
      IF (MCN.EQ.5) MCN=6
      IF (MCN.GE.PCN+1.0) THEN
	 LCOORD=LCN
	ELSE
	 IF (ACSTR.GT.BASTR) THEN
	    LCOORD=MCN
	   ELSE
	    LCOORD=LCN
	   ENDIF
	ENDIF
      IF (NDEBUG.NE.0) WRITE (NDEBUG,1000) PCN,CN,LCOORD
1000  FORMAT (' +++LCOORD 1000+++ PCN,RCN,LCN are as follows ',
     + 2F6.1,I2)
      RETURN
      END

***********************************************************************

      FUNCTION RCOORD(GELMNT,PCN,OX1)

C                   Written 87.6.5 by D.Lawrence
C              applies a correction factor to the Pauling coordination
C              number to get the refined coordination number.

      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H) 
      CALL CHEMNO(GELMNT,NP,NG)
      X=1.00

C   Alkali metals

      IF (NG.EQ.1) THEN
	X=0.73

c   Alkaline earth metals

      ELSE IF (NG.EQ.2) THEN
	X=0.87-0.0058*NG**2*NP

c  Main group elements

      ELSE IF (NG.GE.13.AND.NG.LE.17) THEN
	X=0.87-0.0058*((NG-10)**2)*(NP-5)-0.06*(NG-10-OX1)*(7-NP)
      ELSE IF ((NG.GE.3.AND.NG.LE.9).AND.(OX1-2.0.LT.0.000001)) THEN
	X=0.85

c  noble metals

      ELSE IF (NG.EQ.10.AND.(NP.EQ.5.OR.NP.EQ.6).AND.OX1.LT.2.5) THEN
	X=0.60

c  coinage metals

      ELSE IF (NG.EQ.11) THEN
	 X=0.80
	 IF (OX1.LT.1.5) X=0.40
	 IF (OX1.GT.2.5) X=0.53
C      ELSE IF (NG.EQ.11.AND.(OX1-1.0.LT.0.000001)) THEN
C       X=0.40
C      ELSE IF (NG.EQ.11.AND.OX1.GT.1.0) THEN
C       X=0.80

c  mercury group

      ELSE IF (NG.EQ.12) THEN
	X=0.70
      ENDIF 
      RCOORD=X*PCN
      RETURN
      END       

***********************************************************************


      FUNCTION PCOORD(G1,G2,OX1,OX2)
c
c     written May 1987 by D.Lawrence
c   Routine to calculate the coordination number of a given cation
c   based on ionic radii.
c
      IMPLICIT CHARACTER*8 (G)
c programmer's note:  /bost/ is changed to /bostt/ for
c                     Unix system
      COMMON /BOSTT/ NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      ASSCN=6.0
      RADOX=RADIUS(G2,OX2,ASSCN)
      BONDSTR=OX1/ASSCN
      BONDLEN=DBOST(G1,G2,OX1,OX2,BONDSTR)
      RAD=BONDLEN-RADOX
      RATIO=RAD/RADOX
      PCOORD=(RATIO+0.14)/0.096
      RETURN
      END
c
c
c
