C
C                     FILE ICSDLB              Last update: 30 July 1992
C                     ===========                           C.Brodie
C
C   THIS FILE CONTAINS:
C
C   S RDICSD(NENTRY,JENT)      CONTROL ROUTINE TO READ ENTRIES FROM ICSD
C   S ICSDIX(NENTRY,JENT)      READS BIBLIOGRAPHIC DATA
C   S ATOMRED(LRS,LRB,NF,NFL)  READS ATOM SECTIONS
C   S CELLRED(LRS,LRB,NF,NFL)  READS CELL SECTION
C   S CHEKRED(LRB,NF,LRS)      READS CHECK FLAGS OF RECORD TYPE 20
C   S ENTFIND(HN,NENTRY,JENT,IZ)     LOCATES ENTRY =NENTRY= IN FILE
C   S ENTRED(HN,JENT)          READS ENTRY HEADERS
C   S ICSDSK(NENT,JENT)        DUMMY
C   S RECRED(...)              READS A LOGICAL RECORD FROM ICSD FILE
C   S REFRED(LRB,NF,NFL,JCODN) READS BIBLIOGRAPHIC REFERENCES
C   S SYMRED(LRB,NF,NFL)       READS SYMMETRY SECTIONS
C   S SYMREDH(LRB,NF,NFL,ISY)  READS SPACE GROUP SYMBOLS
C   S TEMPRED(LRB,NF,NFL,LRS,LRT)    READS TEMPERATURE FACTORS
C   S TITLRED(JTIT,LRB,NFL)    READS TITLE SECTION
C   S CRYSTN(NENTRY,JENT)      READS ICSD ENTRIES IN CAN/SND FORMAT
C   S SKPFIZ(NENT,IERR)        LOCATES FIRST ENTRY IN ICSD (CD-ROM) FILE 
C   S RDFIZ(IERR)              READS ICSD (CD-ROM) FILE 
C   S TFCONV(U)                CONVERTS TEMPERATURE FACTORS 
C
C
C     EXTERNALS CALLED FROM THIS FILE:
C
C   MAINLB: PARAM
C   MATHLB: EFNUM, NUM, SECOND, maxdim, upcase
C
C
C
C
C
C -----------------------------------------------------------------
        SUBROUTINE RDICSD (NENTRY,JENT)
C -----------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 13, By D Clarkson.
C
C -----------------------------------------------------------------
C
C Purpose : This is a subroutine to read an ICSD file.
C
C Subroutines called : ATOMRED     RECRED     SYMREDH
C                      CELLRED     REFRED     TEMPRED
C                      CHEKRED     SYMRED     TITLRED
C                      ENTFIND
C
C Input   : NENTRY ; The entry required to be found and
C                    interpreted.
C           JENT   ; The current entry during search.
C
C           NENTRY=0    will read the next entry in the ICSD file.
C           NENTRY=XXX  will read entry XXX ( if found ).
C
C Output  : If the entry is FOUND ; all information contained in
C                                   the entry is read.
C
C           Otherwise, an appropriate message is issued.
C
C -----------------------------------------------------------------
       IMPLICIT CHARACTER *8 (G)
       IMPLICIT CHARACTER *80 (H)
       CHARACTER *150 LRB
       CHARACTER *130 LRBUF
       CHARACTER *1   CAT,CZ,CE,CBL
       DIMENSION LRTYP(250),LRSZ(250),NFL(18)
C
       COMMON /CELL/   A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
       COMMON /CHECK/  ICHECK(22)
       COMMON /DENS/   ZFORM,DENS,DDENS
       COMMON /ECELL/  DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
       COMMON /ENTRY/  NLR,NTYP1,NTYP2,NBYTS,NTST1,NTST2,NTST3,
     *                 NTST4,MLR
       COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD
       include 'combond.for'
       COMMON /RCELL/  AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
       COMMON /SYM/    NSYM,MSYM,S(3,3,48),T(3,48)
       COMMON /GSYM/   GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
       COMMON /TITLE/  JAUTH,KCOMM
       COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     *                 HFORM(2),HCOMM(7),GRFACT
       DATA CAT,CZ,CE,CBL/'@','Z','E',' '/
C
C  Initialization
C
       ICELL=0
       IFORMF=0
       IRFF=0
       ISYMF=0
       IZEND=0
       JAUTH=0
       JCODN=0
       JTITLF=0
       KCOMM=0
       MLR=250
       MSYM=48
       MX=100
       NSYM=0
       NX=0
C
C RDICSD will rewind if it reaches the end of an archive file and
C it is searching for a particular entry, but does not find
C that entry. If it finds that entry, then it will not rewind.
C It will rewind only once.
C If it is not searching for a particular entry, then it will not rewind.
C
       IF (JENT.EQ.-1) JENT = 0
       IF (NDEBUG.NE.0) WRITE(NDEBUG,1100) NENTRY,JENT
 1100  FORMAT('0','++++DEBUG++++RDICSD++++SEARCH FOR NENTRY ',I6,
     *       ', JENT = ',I6,/)
       IF (NENTRY.EQ.0) IZEND=2
C
C Reading the first line of the file.
C
       IF (JENT.NE.0) GO TO 100
       READ (NREAD,1001,END=961) HN
1001   FORMAT (A80)
       IF (HN(1:1).EQ.CAT.AND.HN(2:2).EQ.CE) THEN
         JENT=JENT+1
         GO TO 101
         ENDIF
       IF (HN(1:1).NE.CAT.OR.HN(2:2).NE.CZ) GO TO 901
       JENT=JENT+1
       WRITE (NOUT,1011) (HN(I:I),I=3,80)
1011   FORMAT (' ',5X,'Z-FILE OPENED ',78A1)
C
C Test if the next line is the end of the file.
C
100    READ (NREAD,1001,END=961) HN
101    IF (HN(1:1).EQ.CAT.AND.HN(2:2).EQ.CZ.AND.HN(3:3).EQ.CE) GO TO 961
C
C Test if the next line starts an entry file.
C
       IF (HN(1:1).NE.CAT.OR.HN(2:2).NE.CE) THEN
         WRITE (NOUT,1021) NENTRY,JENT,(HN(I:I),I=1,80)
1021     FORMAT (' ',1X,'CARD FOLLOWING Z-ENTRY DOES NOT START AN'
     *    ,' ENTRY FILE, NENTRY--',I6,', JENT--',I6,', AT--100',
     *    /,1X,80A1)
         GO TO 100
         ENDIF
       IF (NDEBUG.NE.0) WRITE(NDEBUG,1110) HN
 1110  FORMAT(1X,'++++DEBUG++++RDICSD++++ENTRY HEADER: ',A80)
C
C  Locate the required entry in the ICSD file
C  JENT=-1 signals that the entry was not found.
C
       CALL ENTFIND (HN,NENTRY,JENT,IZEND)
       IF (JENT.LE.0) THEN
         WRITE (NOUT,1031) NENTRY
1031     FORMAT (' ',5X,'END OF Z-FILE REACHED BEFORE',I6,
     *                     ' FOUND')
         JENT= -1
         RETURN
         ENDIF
     
       WRITE (GENT,'(I8)') JENT
       call upcase(gent)
       WRITE (NOUT,1041) GENT
1041   FORMAT (' ',5X,'ENTRY NUMBER ',A8,' FOUND')
       DO 10 I=1,22
         ICHECK(I)=0
10       CONTINUE
C
C To read the logical records from the entry.
C
       IF (NLR.GT.MLR) GO TO 911
       READ (NREAD,1051,END=999)(LRTYP(J),LRSZ(J),J=1,NLR)
1051   FORMAT (16(I2,I3))
       READ (NREAD,1001,END=999) HN
       IP=1
       DO 200 J=1,NLR
         CALL RECRED (LRSZ(J),IP,HN,NF,NFL,LRBUF,LRB,LDUMMY)
         IF (NDEBUG.NE.0) WRITE(NDEBUG,1061) SECOND(3)
1061       FORMAT (1X,'*** DEBUG *** TIME ',F7.3,' ***')
         WRITE (NOUT,1066) LRTYP(J),(LRB(I:I),I=3,3+NF-1)
         WRITE (NOUT,1067) (LRB(I:I),I=3+NF,LRSZ(J))
1066     FORMAT (5X,I2,5X,20A1)
1067     FORMAT (37X,95A1)
C
C Read the title of the entry.
C
C LRTYP = Logical record type
C
C LRTYP(1) is the NAME of the entry.
C
         IF (LRTYP(J).NE.1) GO TO 510
           HNAME=LRBUF
C
C LRTYP(2) is the TRIVIAL NAME of the entry.
C
510      IF (LRTYP(J).NE.2) GO TO 520
           IF (KCOMM.GE.7) GO TO 200
           KCOMM=KCOMM+1
           HCOMM(KCOMM)=LRBUF
           GO TO 200
C
C LRTYP(3) is the MINERAL NAME.
C
520      IF (LRTYP(J).NE.3) GO TO 530
           IF (KCOMM.GE.7) GO TO 200
           KCOMM=KCOMM+1
           HCOMM(KCOMM)=LRBUF
           GO TO 200
C
C LRTYP(4) is the CHEMICAL FORMULA.
C
530      IF (LRTYP(J).NE.4) GO TO 540
           IF (IFORMF.GE.2) GO TO 200
           IFORMF=IFORMF+1
           HFORM(IFORMF)(1:74)=LRBUF(1:74)
           IF (IFORMF.LT.2) THEN
             IFORMF=IFORMF+1
             HFORM(IFORMF)(1:74)=LRBUF(75: )
           ENDIF
           GO TO 200
C
C LRTYP(5) is the TITLE of the reference.
C
540      IF (LRTYP(J).NE.5) GO TO 550
           IF (JTITLF.GE.2) GO TO 200
           CALL TITLRED (JTITLF,LRBUF,LRSZ(J)-2)
           GO TO 200
C
C LRTYP(6) contains BIBLIOGRAPHIC DATA about the reference.
C
550      IF (LRTYP(J).NE.6) GO TO 560
           IF (JCODN.GE.2) GO TO 200
           CALL REFRED (LRBUF,NF,NFL,JCODN)
           GO TO 200
C
C LRTYP(7) lists the AUTHORS, LAST NAMES and INITIALS.
C
560      IF (LRTYP(J).NE.7) GO TO 570
           IF (JAUTH.GE.10) GO TO 200
           JAUTH=JAUTH+1
           HAUTH(JAUTH)=LRBUF
           GO TO 200
C
C LRTYP(8) reads the CELL CONSTANTS.
C
570      IF (LRTYP(J).NE.8) GO TO 580
           CALL CELLRED (LRSZT,LRBUF,NF,NFL)
           IF (LRSZT.NE.LRSZ(J)) GO TO 941
           IF (ICELL.NE.1) GO TO 942
           GO TO 200
C
C LRTYP(9) reads the SYMMETRY DESCRIPTORS.
C
580      IF (LRTYP(J).NE.9) GO TO 590
           IF (NF.GT.1) GO TO 983
           CALL SYMREDH (LRBUF,NF,NFL,ISYMF)
           GO TO 200
C
C LRTYP(10) reads the SYMMETRY DESCRIPTORS.
C
590      IF (LRTYP(J).NE.10) GO TO 600
           IF (NSYM.GE.MSYM) GO TO 971
           CALL SYMRED (LRBUF,NF,NFL)
           GO TO 200
C
C LRTYP(11) reads COMMENTS.
C
600      IF (LRTYP(J).NE.11) GO TO 610
           IF (KCOMM.GE.7) GO TO 200
           KCOMM=KCOMM+1
           HCOMM(KCOMM)=LRBUF
           GO TO 200
C
C LRTYP(12) reads the ATOMIC DESCRIPTORS.
C
610      IF (LRTYP(J).NE.12) GO TO 620
           CALL ATOMRED (LRSZT,LRBUF,NF,NFL)
           IF (LRSZT.NE.LRSZ(J)) GO TO 951
           IF (NX.GT.MX) GO TO 921
           GO TO 200
C
C LRTYP(13) reads the TEMPERATURE FACTORS.
C
620      IF (LRTYP(J).NE.13) GO TO 630
           CALL TEMPRED (LRBUF,NF,NFL,LRSZT,LRTYP(J))
           IF (LRSZT.NE.LRSZ(J)) GO TO 992
           GO TO 200
C
C LRTYP(14) reads the TEMPERATURE FACTORS.
C
630      IF (LRTYP(J).NE.14) GO TO 640
           CALL TEMPRED (LRBUF,NF,NFL,LRSZT,LRTYP(J))
           IF (LRSZT.NE.LRSZ(J)) GO TO 993
           GO TO 200
C
C LRTYP(15) reads the TEMPERATURE FACTORS.
C
640      IF (LRTYP(J).NE.15) GO TO 650
           CALL TEMPRED (LRBUF,NF,NFL,LRSZT,LRTYP(J))
           IF (LRSZT.NE.LRSZ(J)) GO TO 994
           GO TO 200
C
C LRTYP(16) reads COMMENTS.
C
650      IF (LRTYP(J).NE.16) GO TO 660
           IF (KCOMM.GE.7) GO TO 200
           KCOMM=KCOMM+1
           HCOMM(KCOMM)=LRBUF
           GO TO 200
C
C LRTYP(17) reads the CAS number.
C
660      IF (LRTYP(J).NE.17) GO TO 670
           IF (KCOMM.GE.7) GO TO 200
           KCOMM=KCOMM+1
           HCOMM(KCOMM)=LRBUF
           GO TO 200
C
C LRTYP(18) reads the R FACTOR.
C
670      IF (LRTYP(J).NE.18) GO TO 680
           IF (IRFF.GE.1) GO TO 200
           IRFF=IRFF+1
           GRFACT=LRBUF(1:LRSZ(J)-2)
           GO TO 200
C
C LRTYP(20) describes the QUALITY of the DATA.
C
680      IF (LRTYP(J).NE.20) GO TO 200
           CALL CHEKRED (LRBUF,NF,LRSZT)
           IF (LRSZT.NE.LRSZ(J)) GO TO 995
200      CONTINUE
         WRITE (NOUT,1093) NTST1,NTST2,NTST3,NTST4
1093     FORMAT ('0',10X,'FORMAT TEST STATUS        ',I1,/,
     *           11X,'PLAUSIBILITY TEST STATUS ',I2,/,
     *           11X,'BOND TEST STATUS         ',I2,/,
     *           11X,'FORMULA TEST STATUS      ',I2,//)
         GO TO 999
C
C These statements are reached due to errors at some point in
C the program before.
C
C When a statement number is given it refers to the subroutine
C from which the error statement originates.
C
901    WRITE (NOUT,9001)(HN(I:I),I=1,80),NENTRY
9001   FORMAT (' ',10X,'---ERROR---  FILE IS NOT A Z-FILE ',/,
     *         10X,' LINE IS ',80A1,/,' SEARCHING FOR ENTRY ',
     *         'NUMBER ',I6)
       JENT= -2
       RETURN
911    WRITE (NOUT,9011) NLR,MLR,NENTRY
9011   FORMAT (' ',10X,'---ERROR--- NUMBER OF LOGICAL RECORDS, ',I4,
     *  ', IS GREATER THAN MAXIMUM, ',I4,'. NENTRY IS ',I6,/,10X,
     *  ' FROM STATEMENT NUMBER 1041')
       NENTRY = JENT
       JENT = -3
       RETURN
921    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
9021   FORMAT (' ',10X,'---ERROR---  RECORD TYPE IS ',I4,10X,
     *  'RECORD SIZE IS ',I4)
       WRITE (NOUT,9031) MX
9031   FORMAT (11X,'MORE THAN ',I4,' ATOMS IN ENTRY ',/,11X,
     *    'FROM STATEMENT NUMBER 610 ')
       NENTRY = JENT
       JENT = -3
       RETURN
941    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9041) LRSZT
9041   FORMAT(' ',10X,'RECORD LENGTH READ IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 570 ')
       NENTRY = JENT
       JENT = -3
       RETURN
942    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9042)
9042   FORMAT (' ',10X,'CELL CONSTANTS NOT READ CORRECTLY',/,10X,
     *  ' FROM STATEMENT NUMBER 570')
       NENTRY = JENT
       JENT = -3
       RETURN
951    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9051) LRSZT
9051   FORMAT(' ',10X,'RECORD LENGTH IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 610 ')
       NENTRY = JENT
       JENT = -3
       RETURN
961    IF (IZEND.EQ.1) WRITE (NOUT,9061) NENTRY
9061   FORMAT ('0',10X,'END OF Z-FILE REACHED BEFORE',I6,' FOUND',/,
     *   10X,'  FROM STATEMENT NUMBER 100 ',/)
       IF (IZEND.GE.2) WRITE (NOUT,9065)
9065   FORMAT ('0',10X,'END OF Z-FILE',/)
       REWIND NREAD
       IZEND=IZEND+1
       IF (IZEND.LE.1) THEN
          READ (NREAD,1001,END=961) HN
          IF (HN(1:1).NE.CAT.OR.HN(2:2).NE.CZ) GOTO 901
          GOTO 100
          ENDIF
       JENT= -1
       RETURN
971    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9071) MSYM
9071   FORMAT (' ',10X,'NUMBER OF SYMMETRY RECORDS EXCEEDS ',I4,/,10X,
     *   ' FROM STATEMENT NUMBER 590')
       NENTRY = JENT
       JENT = -3
       RETURN
983    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9083)
9083   FORMAT (' ',10X,'NUMBER OF FIELDS EXCEEDS 1.',/,10X,
     *  ' PROGRAM CANNOT INTERPRET THIS RECORD ')
       NENTRY = JENT
       JENT = -3
       RETURN
992    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9092) LRSZT
9092   FORMAT (' ',10X,'RECORD LENGTH IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 620 ')
       NENTRY = JENT
       JENT = -3
       RETURN
993    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9093) LRSZT
9093   FORMAT (' ',10X,'RECORD LENGTH IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 630 ')
       NENTRY = JENT
       JENT = -3
       RETURN
994    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9094) LRSZT
9094   FORMAT (' ',10X,'RECORD LENGTH IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 640 ')
       NENTRY = JENT
       JENT = -3
       RETURN
995    WRITE (NOUT,9021) LRTYP(J),LRSZ(J)
       WRITE (NOUT,9095) LRSZT
9095   FORMAT (' ',10X,'RECORD LENGTH IN ENTRY IS ',I4,/,10X,
     *  ' FROM STATEMENT NUMBER 680 ')
       NENTRY = JENT
       JENT = -3
999    RETURN
       END

************************************************************************
C ----------------------------------------------------------------------
       SUBROUTINE ICSDIX (NENTRY,JENT)
C ----------------------------------------------------------------------
C
C WRITTEN 84 07 11, BY M JACKA
C
C Revised 85 06 13, By D Clarkson
C
C ----------------------------------------------------------------------
C
C Purpose : Prepares bibliographic data about entries from an ICSD file,
C
C           including : Entry number
C                       Title of the entry
C                       Chemical formula
C                       Title of reference
C                       Bibliography of source
C                       Authors' names
C
C Subroutines called  : ENTRED
C                       RECRED
C
C Input   : NENTRY ; the entry required to be located and
C                    interpreted.
C
C           JENT   ; The current entry in the search.
C
C           NENTRY=0    will read the next entry in the file.
C           NENTRY=XXX  will read entry XXX ( if found ).
C
C Output  : Bibliographic data from the entry is read and interpreted.
C
C ----------------------------------------------------------------------
       IMPLICIT CHARACTER *8 (G)
       IMPLICIT CHARACTER *80 (H)
       CHARACTER *150 LRB
       CHARACTER *130 LRBUF
       CHARACTER *1   CAT,CZ,CE
       DIMENSION LRTYP(100),LRSZ(100),NFL(18)
C
       COMMON /ENTRY/  NLR,NTYP1,NTYP2,NBYTS,NTST1,NTST2,NTST3,
     *                 NTST4,MLR
       COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD
       COMMON /TITLE/  JAUTH,KCOMM
       COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     *                 HFORM(2),HCOMM(7),GRFACT,GRAD
       DATA CAT,CZ,CE/'@','Z','E'/
C
       IFORMF=0
       JAUTH=0
       JCODN=0
       JTITLF=0
       LINE=1
       MLR=100
C
C Reading first line of the file
C
       IF (JENT.NE.0) GO TO 100
       READ (NREAD,1001,END=999) HN
1001   FORMAT (A80)
       IF (HN(1:1).NE.CAT.OR.HN(2:2).NE.CZ) GO TO 901
       JENT=JENT+1
       WRITE (NOUT,1011) (HN(I:I),I=3,80)
1011   FORMAT (' ',5X,' Z-FILE OPENED ',78A1)
C
C Test if the next line is the end of the Z file
C
100    READ (NREAD,1001,END=999) HN
       IF (HN(1:1).EQ.CAT.AND.HN(2:2).EQ.CZ.AND.HN(3:3).EQ.CE)
     *   GO TO 911
C
C Test if the next line starts an entry file
C
       IF (HN(1:1).NE.CAT.OR.HN(2:2).NE.CE) THEN
         WRITE (NOUT,1021) (HN(I:I),I=1,80)
1021     FORMAT (' ',' LINE AFTER Z-ENTRY DOES NOT START AN ENTRY ',
     *    ' FILE ',/,' LINE IS ',80A1,/,' TEST NEXT LINE FOR ',
     *    ' ENTRY START ')
         GO TO 100
         ENDIF
C
C  Search for the required entry.
C  JENT=-1 signals that the entry was not found.
C
       CALL ENTFIND (HN,NENTRY,JENT,IZEND)
       IF ( JENT.LE.0 ) THEN
         WRITE (NOUT,1031) NENTRY
1031     FORMAT (' ',5X,'END OF Z-FILE REACHED BEFORE',I6,' FOUND')
         JENT=-1
         RETURN
       ENDIF
C
C  Process all logical records in the entry
C
       IF (NLR.GT.MLR) GO TO 921
110    READ (NREAD,1041,END=999) (LRTYP(J),LRSZ(J),J=1,NLR)
1041   FORMAT(16(I2,I3))
       READ (NREAD,1001,END=999) HN
       IP=1
       DO 200 J=1,NLR
         CALL RECRED (LRSZ(J),IP,HN,NF,NFL,LRBUF,LRB,LINE)
C
C LRTYP = Logical record type
C
C LRTYP(1) is the TITLE of the entry
C
         IF (LRTYP(J).NE.1) GO TO 310
         WRITE (NOUT,4001) JENT,(LRB(I:I),I=3,LRSZ(J))
4001     FORMAT (1X,/,/,1X,I6,9X,100A1)
         GO TO 200
C
C LRTYP(4) is the CHEMICAL FORMULA
C
310      IF (LRTYP(J).NE.4) GO TO 320
         WRITE (NOUT,4002) (LRB(I:I),I=3,LRSZ(J))
4002     FORMAT (16X,100A1)
         GO TO 200
C
C LRTYP(5) is the TITLE of the REFERENCE
C
320      IF (LRTYP(J).NE.5) GO TO 330
         WRITE (NOUT,4003) (LRB(I:I),I=3,LRSZ(J))
4003     FORMAT (16X,100A1)
         GO TO 200
C
C LRTYP(6) contains BIBLIOGRAPHIC DATA about the reference
C
330      IF (LRTYP(J).NE.6) GO TO 340
         IF (JCODN.GE.2) GO TO 340
         CALL REFRED (LRBUF,NF,NFL,JCODN)
         WRITE (NOUT,4004) ((HCODN(K)(L:L),L=1,26),K=1,JCODN)
4004     FORMAT (16X,26A1)
         GO TO 200
C
C LRTYP(7) lists the AUTHORS, SURNAMES and INITIALS
C
340      IF (LRTYP(J).NE.7) GO TO 350
         WRITE (NOUT,4005) (LRB(I:I),I=3,LRSZ(J))
4005     FORMAT (16X,100A1)
         GOTO 200
C
C  Exit the loop only after the required record types have been
C  checked for.
C
350      IF (LRTYP(J).GT.7) GOTO 800
200    CONTINUE
       GO TO 800
C
C These statements are reached due to errors in the program before
C
901    WRITE (NOUT,9001) (HN(I:I),I=1,80)
9001   FORMAT (' ',10X,' -- ERROR -- FILE IS NOT A Z-FILE ',/,
     *   1X,' FIRST LINE IS ',80A1)
       JENT= -2
       RETURN
911    WRITE (NOUT,9011)
9011   FORMAT (1X,' END OF Z-FILE REACHED ',/,1X,' GO TO ',
     *   ' SUBROUTINE RDICSD FOR A SPECIFIC ENTRY ')
       JENT= -1
       RETURN
921    WRITE (NOUT,9021) NLR,JENT
9021   FORMAT (' ',10X,' -- ERROR --  NUMBER OF LOGICAL RECORDS ',I3,
     *   ' IS GREATER THAN MAXIMUM ALLOWED. JENT IS ',I6,/,10X,
     *   'FROM  STATEMENT NUMBER 1031')
       GO TO 110
800    ISKIP=NTYP2-LINE
C
C  Skip over the remainder of the entry.
C
       DO 10 I=1,ISKIP
         READ (NREAD,1001,END=999) HN
10     CONTINUE
       JCODN=0
       LINE=1
999    RETURN
       END

************************************************************************

C --------------------------------------------------------------------
       SUBROUTINE ATOMRED (LRSZT,LRBUF,NF,NFL)
C --------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C REVISED 85 05 13, By D Clarkson.
C
C --------------------------------------------------------------------
C
C Purpose : ATOMRED will read the atomic descriptors.
C
C Subroutines called   : EFNUM
C
C Functions called     : NUM
C
C Input   : LRBUF      ; holds the atomic desciptors in packed format.
C
C Output  : XX,DXX     ; Fractional atom coordinates and errors.
C           U,DU       ; Temperature factors and errors.
C           OCCU,SOCCU ; Site occupation number and error.
C           NH,ND      ; Number of attached H and D atoms respec.
C           NPOS       ; Number of equivalent positions in unit cell.
C           OXID       ; Oxidation number.
C           GELEM      ; Element name.
C           GDENT      ; Atom identifier.
C           GWYCK      ; Wykcoff letter.
C
C NOTE   :  All output is via common blocks.
C
C --------------------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8 (G)
       CHARACTER *60 LRBUF
       CHARACTER *100 TTYP
       DIMENSION NFL(NF)
       include 'comatom.for'
       COMMON /DUMMY/  NXH,XNUMH(10)
       COMMON /GDUMMY/ GDH(10)
       NX=NX+1
       PI=3.1416
       TCONVB=2.0*PI**2
       K=1
       GELEM(NX)=LRBUF(K:K+NFL(1)-1)
       K=K+NFL(1)
       GDENT(NX)=LRBUF(K:K+NFL(2)-1)
       K=K+NFL(2)
       CALL EFNUM (OXID(NX),DUM,LRBUF(K:K),NFL(3),0)
       K=K+NFL(3)
       NPOS(NX)=NUM(LRBUF(K:K),NFL(4))
       K=K+NFL(4)
       GWYCK(NX)=LRBUF(K:K)
       K=K+NFL(5)
       IF (NF.LE.5) THEN
         DO 110 I=1,3
           XX(I,NX)=9.0
           DXX(I,NX)=0.0
110        CONTINUE
         GO TO 140
         ENDIF
       IF (NFL(6).EQ.0.AND.NFL(8).EQ.0.AND.NFL(10).EQ.0) THEN
         DO 170 I=1,3
           XX(I,NX)=9.0
           DXX(I,NX)=0.0
170        CONTINUE
         GO TO 160
         ENDIF
       IXX1=NFL(6)
       IF (NF.LE.8) THEN
         IXX2=NFL(7)
         IXX3=NFL(8)
         GO TO 210
         ENDIF
       IDXX1=NFL(7)
       IXX2=NFL(8)
       IF (NF.LE.9) THEN
         IXX3=NFL(9)
         GO TO 220
         ENDIF
       IDXX2=NFL(9)
       IXX3=NFL(10)
       IF (NF.LE.10) GO TO 230
       IDXX3=NFL(11)
       GO TO 100
210    IDXX1=0
220    IDXX2=0
230    IDXX3=0
100    CALL EFNUM (XX(1,NX),DXX(1,NX),LRBUF(K:K),IXX1,IDXX1)
       K=K+IXX1+IDXX1
       CALL EFNUM (XX(2,NX),DXX(2,NX),LRBUF(K:K),IXX2,IDXX2)
       K=K+IXX2+IDXX2
       CALL EFNUM (XX(3,NX),DXX(3,NX),LRBUF(K:K),IXX3,IDXX3)
       K=K+IXX3+IDXX3
160    IF (NF.LE.11) GO TO 140
       IF (NFL(12).NE.0) THEN
         TTYP(NX:NX)=LRBUF(K:K)
         K=K+1
         ENDIF
       IF (NF.LE.12) GO TO 140
       IF (NF.LE.13) THEN
         IDU11=0
         GO TO 130
         ENDIF
       IDU11=NFL(14)
130    CALL EFNUM (U(1,1,NX),DU(1,1,NX),LRBUF(K:K),NFL(13),IDU11)
       IF (TTYP(NX:NX).EQ.'B') THEN
         U(1,1,NX)=U(1,1,NX)/TCONVB
         DU(1,1,NX)=DU(1,1,NX)/TCONVB
         ENDIF
       K=K+NFL(13)+IDU11
       IF (NF.LE.14) GO TO 140
       IF (NF.LE.15) THEN
         NFL16=0
         GO TO 150
         ENDIF
       NFL16=NFL(16)
150    CALL EFNUM (OCCU(NX),SOCCU(NX),LRBUF(K:K),NFL(15),NFL16)
       IF (OCCU(NX).LT.0.000001) OCCU(NX)=1.0
C
C  MAKE ADJUSTMENTS FOR DUMMY ATOMS
C
       IF(XX(1,NX).GT.8.0) THEN
         IF(NXH.EQ.0) GOTO 245
         DO 240 J=1,NXH
           IF(GELEM(NX).EQ.GDH(J)) THEN
             XNUMH(J) = XNUMH(J) + NPOS(NX)*OCCU(NX)
             GOTO 250
             ENDIF
  240      CONTINUE
         IF(NXH.EQ.10) THEN
           WRITE(NOUT,260)
  260      FORMAT(' *****WARNING***** ATTEMPT TO STORE MORE THAN 10 ',
     +       'DUMMY ATOMS.  REMAINDER IGNORED')
           GOTO 250
           ENDIF
  245    NXH = NXH + 1
         GDH(NXH) = GELEM(NX)
         XNUMH(NXH) = NPOS(NX)*OCCU(NX)
  250    CONTINUE
         DO 252 I=1,3
           XX(I,NX)=0.0
           DXX(I,NX)=0.0
  252    CONTINUE
         OCCU(NX)=0.0
         SOCCU(NX)=0.0
         U(1,1,NX)=0.0
         DU(1,1,NX)=0.0
         NX = NX - 1
         ENDIF
       K=K+NFL(15)+NFL16
       IF (NF.LE.16) GO TO 140
       NH(NX)=NUM(LRBUF(K:K),NFL(17))
       K=K+NFL(17)
       IF (LRBUF(K:K).EQ.'D') THEN
         ND(NX)=NH(NX)
         NH(NX)=0
         ENDIF
       K=K+NFL(18)
140    LRSZT=NF+K+1
       RETURN
       END

************************************************************************

C ------------------------------------------------------------
       SUBROUTINE CELLRED (LRSZT,LRBUF,NF,NFL)
C ------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ------------------------------------------------------------
C
C Purpose : CELLRED will read the unit cell constants.
C
C Subroutines called : EFNUM
C                      PARAM
C
C Functions called   : NUM
C
C Input  : LRBUF ; Buffer holding cell info in packed format.
C
C Output : A,B,C,DA,DB,DC      ; Unit cell edges and errors.
C          ALPHA,BETA,AGAMMA   ; Unit cell angles.
C          DALPHA,DBETA,DGAMMA ; Errors in angles.
C          DENS,DDENS          ; Density and error.
C          ZFORM               ; Number of formula units.
C          ICELL               ; cell data stored/not stored.
C
C ------------------------------------------------------------
       CHARACTER *50 LRBUF
       DIMENSION NFL(NF)
       COMMON /CELL/  A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
       COMMON /DENS/  ZFORM,DENS,DDENS
       COMMON /ECELL/ DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
       COMMON /RCELL/ AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
       K=1
       PI=3.1416
       CNVRAD=PI/180
       CALL EFNUM (A,DA,LRBUF(K:K),NFL(1),NFL(2))
       K=K+NFL(1)+NFL(2)
       CALL EFNUM (B,DB,LRBUF(K:K),NFL(3),NFL(4))
       K=K+NFL(3)+NFL(4)
       CALL EFNUM (C,DC,LRBUF(K:K),NFL(5),NFL(6))
       K=K+NFL(5)+NFL(6)
       CALL EFNUM (ALPHA,DALPHA,LRBUF(K:K),NFL(7),NFL(8))
       K=K+NFL(7)+NFL(8)
       CALL EFNUM (BETA,DBETA,LRBUF(K:K),NFL(9),NFL(10))
       K=K+NFL(9)+NFL(10)
       CALL EFNUM (AGAMMA,DGAMMA,LRBUF(K:K),NFL(11),NFL(12))
C
C  Convert angles and their errors into Radians.
C
       ALPHA=ALPHA*CNVRAD
       DALPHA=DALPHA*CNVRAD
       BETA=BETA*CNVRAD
       DBETA=DBETA*CNVRAD
       AGAMMA=AGAMMA*CNVRAD
       DGAMMA=DGAMMA*CNVRAD
C
       K=K+NFL(11)+NFL(12)
       IF (NF.EQ.12) GO TO 101
       ZFORM=NUM(LRBUF(K:K),NFL(13))
       K=K+NFL(13)
       IF (NF.EQ.13) GO TO 101
       IF (NF.EQ.14) THEN
         NFL15=0
       ELSE
         NFL15=NFL(15)
         ENDIF
       CALL EFNUM (DENS,DDENS,LRBUF(K:K),NFL(14),NFL15)
       K=K+NFL(14)+NFL15
101    CALL PARAM
       LRSZT=NF+K+1
       ICELL=1
       RETURN
       END

************************************************************************

C ----------------------------------------------------------------------
       SUBROUTINE CHEKRED (LRBUF,NF,LRSZT)
C ----------------------------------------------------------------------
C
C WRITTEN 84 07 27, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : CHEKRED will interpret the check flags of record type 20.
C
C Input   : LRBUF ; buffer that holds the check flags in packed format.
C
C Output  : The check flags are interpreted.
C
C ----------------------------------------------------------------------
       CHARACTER *22 LRBUF
       COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
       K=1
       WRITE(NOUT,1000)
1000   FORMAT('0')
30     IF ( LRBUF(K:K+1).EQ.'11' ) THEN
         WRITE(NOUT,1001) '1','1'
       ELSE IF ( LRBUF(K:K+1).EQ.'21' ) THEN
         WRITE(NOUT,1002) '2','1'
       ELSE IF ( LRBUF(K:K+1).EQ.'22' ) THEN
         WRITE(NOUT,1003) '2','2'
       ELSE IF ( LRBUF(K:K+1).EQ.'23' ) THEN
         WRITE(NOUT,1004) '2','3'
       ELSE IF ( LRBUF(K:K+1).EQ.'51' ) THEN
         WRITE(NOUT,1005) '5','1'
       ELSE IF ( LRBUF(K:K+1).EQ.'52' ) THEN
         WRITE(NOUT,1006) '5','2'
       ELSE IF ( LRBUF(K:K+1).EQ.'53' ) THEN
         WRITE(NOUT,1007) '5','3'
       ELSE IF ( LRBUF(K:K+1).EQ.'54' ) THEN
         WRITE(NOUT,1008) '5','4'
       ELSE IF ( LRBUF(K:K+1).EQ.'55' ) THEN
         WRITE(NOUT,1009) '5','5'
       ELSE IF ( LRBUF(K:K+1).EQ.'56' ) THEN
         WRITE(NOUT,1010) '5','6'
       ELSE IF ( LRBUF(K:K+1).EQ.'70' ) THEN
         WRITE(NOUT,1011) '7','0'
       ELSE IF ( LRBUF(K:K+1).EQ.'71' ) THEN
         WRITE(NOUT,1012) '7','1'
       ELSE IF ( LRBUF(K:K+1).EQ.'72' ) THEN
         WRITE(NOUT,1013) '7','2'
       ELSE IF ( LRBUF(K:K+1).EQ.'73' ) THEN
         WRITE(NOUT,1014) '7','3'
       ELSE IF ( LRBUF(K:K+1).EQ.'74' ) THEN
         WRITE(NOUT,1015) '7','4'
       ELSE IF ( LRBUF(K:K+1).EQ.'75' ) THEN
         WRITE(NOUT,1016) '7','5'
       ELSE IF ( LRBUF(K:K+1).EQ.'76' ) THEN
         WRITE(NOUT,1017) '7','6'
       ELSE IF ( LRBUF(K:K+1).EQ.'77' ) THEN
         WRITE(NOUT,1018) '7','7'
       ENDIF
       K=K+2
       IF ( K.LE.NF ) GO TO 30
C
1001   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   QUESTIONABLE ',
     *            'STRUCTURE')
1002   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   TOLERABLE ',
     *            'DIFFERENCE IN FORMULA CHECK')
1003   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   TOLERABLE ',
     *            'DIFFERENCE IN CHARGE CHECK')
1004   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   UNUSUAL ',
     *            'BUT TOLERABLE DENSITY')
1005   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   R-FACTOR ',
     *            'NOT GIVEN IN PAPER')
1006   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   UNREASONABLE ',
     *            'TEMPERATURE FACTORS IN PAPER')
1007   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   TEMPERATURE ',
     *            'FACTORS MISSING IN PAPER')
1008   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   UNUSUAL ',
     *            'OCCUPATION NUMBERS IN PAPER')
1009   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   UNUSUAL ',
     *            'LATTICE CONSTANTS IN PAPER')
1010   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   NON-STANDARD ',
     *            'TEMPERATURE FACTORS GIVEN IN PAPER')
1011   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   MODULATED ',
     *            'OR UNUSUAL TYPE OF STRUCTURE')
1012   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   IMPORTANT ',
     *            'DATA MISSING FROM PAPER')
1013   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   COORDINATES ',
     *              'DETERMINED BUT NOT PUBLISHED')
1014   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   COORDINATES ',
     *            'NOT DETERMINED')
1015   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   ERROR IN ',
     *            'COORDINATES IN PAPER UNCORRECTED')
1016   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   ERROR IN ',
     *            'COORDINATES IN PAPER CORRECTED')
1017   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   COORDINATES ',
     *            'IN PAPER PROBABLY CORRECT BUT DISTANCES ',
     *            'DO NOT AGREE')
1018   FORMAT (5X,'CHECK FLAGS ',2(A1,1X),'   NO COORDINATES ',
     *            'IN DATABASE AS ENTRY IS PROBABLY IN THE ',
     *            'METALS OR ORGANIC FILE')
       LRSZT=2+K-1+NF
       RETURN
       END

************************************************************************

C ----------------------------------------------------------------------
       SUBROUTINE ENTFIND (HN,NENTRY,JENT,IZEND)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : ENTFIND is a linear search routine that will look up the
C           entry with number *NENTRY*.
C
C           JENT = Current entry number.
C           JENT = -1 ; signals that the end-of-file has been reached.
C
C Functions called : NUM
C
C Input  : HN     ; buffer holding the entry descriptor.
C          NENTRY ; the entry number being sought.
C
C Output : JENT   ; the entry number found,
C                   = -1 if end-of-file before entry is found.
C          IZEND  ; =  2 if the entry is found.
C
C ----------------------------------------------------------------------
       implicit character *8 (g)
       IMPLICIT CHARACTER *80 (H)
       CHARACTER *1 CAT,CZ,CE
       COMMON /ENTRY/  NLR,NTYP1,NTYP2,NBYTS,NTST1,NTST2,NTST3,
     *                 NTST4,MLR
       COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD
       DATA CAT,CZ,CE /'@','Z','E'/
670    CALL ENTRED ( HN,JENT )
       IF (JENT.EQ.NENTRY.OR.NENTRY.EQ.0) THEN
         NBYTS=NUM(HN(17:17),5)
         NTST1=NUM(HN(22:22),1)
         NTST2=NUM(HN(23:23),2)
         NTST3=NUM(HN(25:25),2)
         NTST4=NUM(HN(27:27),2)
         IF (JENT.EQ.NENTRY) IZEND=2
         RETURN
         ENDIF
       ISKIP=NTYP1+NTYP2+1
       DO 680 I=1,ISKIP
680      READ (NREAD,6081,END=900) HN
6081     FORMAT (A80)
       IF (HN(1:1).EQ.CAT.AND.HN(2:2).EQ.CE) GO TO 670
       IF (HN(1:1).EQ.CAT.AND.HN(2:2).EQ.CZ.AND.HN(3:3).EQ.CE) GO TO 900
       RETURN
900    JENT= -1
       RETURN
       END

************************************************************************

C ----------------------------------------------------------------------
       SUBROUTINE ENTRED (HN,JENT)
C ----------------------------------------------------------------------
C
C WRITTEN 84 07 11, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : ENTRED interprets the information contained on the header
C           of the entry. ie. the @E line.
C
C Functions called : NUM
C
C Input   : HN    ; buffer holding the header line.
C
C Output  : JENT  ; the number of the entry.
C           NLR   ; the number of logical records.
C           NTYP1 ; the number of index entries.
C           NTYP2 ; the number of data entries.
C
C ----------------------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       COMMON /ENTRY/  NLR,NTYP1,NTYP2,NBYTS,NTST1,NTST2,NTST3,
     *                 NTST4,MLR
       JENT=NUM(HN(3:3),6)
       NLR=NUM(HN(9:9),3)
       NTYP1=NUM(HN(12:12),2)
       NTYP2=NUM(HN(14:14),3)
       RETURN
       END

************************************************************************

C ----------------------------------------------------------------------
       SUBROUTINE ICSDSK(NENT,JENT)
C ----------------------------------------------------------------------
C
C WRITTEN 84 07 11, BY M JACKA
C
C ---------------------------------------------------------------------
C
C THIS SUBROUTINE IS NOT USED IN RDICSD
C IT MAY BE CALLED ELSEWHERE
C
C ----------------------------------------------------------------------
       RETURN
       END
C ----------------------------------------------------------------------
       SUBROUTINE RECRED (LRSZ,IP,HN,NF,NFL,LRBUF,LRB,LINE)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : RECRED will read one logical record. If necessary, a new
C           physical record will be read.
C           The record pointer (IP) is moved accordingly.
C
C Functions called : NUM
C
C Input   : HN     ; buffer holding a physical record.
C           IP     ; pointer for the physical record. range = 1..80
C
C Output  : NF     ; number of fields in the logical record.
C           NFL    ; array holding the length of each field.
C           LRBUF  ; buffer to hold data part of logical record.
C           LRB    ; buffer to hold entire logical record.
C           LINE   ; number of physical records read.
C
C ----------------------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       CHARACTER *150 LRB
       CHARACTER *130 LRBUF
       DIMENSION NFL(18)
       COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD
C
C  Clear the buffers and field length array
C
       DO 10 I=1,18
         NFL(I)=0
10       CONTINUE
       DO 20 I=1,150
         LRB(I:I)=' '
20       CONTINUE
       DO 30 I=1,130
         LRBUF(I:I)=' '
30       CONTINUE
C
C Interpreting the individual logical record.
C pointer will move to beginning of the next line if it runs over.
C
       LRSZM=LRSZ
       IF (LRSZM.GT.150) LRSZ=150
       DO 200 J=1,LRSZ
C
C  Read another physical record if required.
C
         IF (IP.GT.80) THEN
           READ (NREAD,1011,END=989) HN
1011       FORMAT (A80)
           IP=1
           LINE=LINE+1
           ENDIF
C
C Set up a buffer array (LRB) to hold the LRSZ elements of the
C logical record.
C
         LRB(J:J)=HN(IP:IP)
         IP=IP+1
200      CONTINUE
C
C Determine the number of fields in the logical record
C
       NF=NUM(LRB(1:1),2)
       IF (NF.EQ.0) GO TO 215
       DO 210 I=1,NF
         J=I+2
         NFL(I)=NUM(LRB(J:J),1)
210      CONTINUE
C
C Take the cell intelligence of the logical record from LRB
C and put it into LRBUF
C
215    LRBUF=LRB(3+NF: )
       IF (LRSZM.NE.LRSZ) THEN
          IP=IP+LRSZM-LRSZ
225       IF (IP.GT.80) THEN
            READ (NREAD,1011,END=989) HN
            IP=IP-80
            LINE=LINE+1
            GO TO 225
            ENDIF
         ENDIF
       RETURN
989    WRITE (NOUT,9089) NREAD
9089   FORMAT (10X,' -- ERROR -- FROM SUBROUTINE RECRED, FORMAT ',
     *  'STATEMENT NUMBER 9089',/,10X,' END OF FILE IS FOUND ',
     *  'PREMATURELY',/,10X,' INPUT FILE IS ',I3)
       RETURN
       END
C ----------------------------------------------------------------------
       SUBROUTINE REFRED (LRBUF,NF,NFL,JCODN)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : REFRED will read the bibliographic reference.
C
C Input   : LRBUF  ; buffer holding the reference in packed form.
C
C Output  : HCODN  ; literature references.
C
C ----------------------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8 (G)
       CHARACTER *130 LRBUF
       DIMENSION NFL(NF)
       COMMON /TITLE/   JAUTH,KCOMM
       COMMON /GTITLE/  GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     *                  HFORM(2),HCOMM(7),GRFACT,GRAD
       IP=1
       JCODN=JCODN+1
       HCODN(JCODN)=' '
       HCODN(JCODN)(1:2)=LRBUF(IP:IP+NFL(1)-1)
       IP=NFL(1)+1
       HCODN(JCODN)(3:8)=LRBUF(IP:IP+NFL(2)-1)
       IP=IP+NFL(2)
       HCODN(JCODN)(9+4-NFL(3):12)=LRBUF(IP:IP+NFL(3)-1)
       IP=IP+NFL(3)
       HCODN(JCODN)(13+4-NFL(4):16)=LRBUF(IP:IP+NFL(4)-1)
       IP=IP+NFL(4)
       HCODN(JCODN)(17+4-NFL(5):20)=LRBUF(IP:IP+NFL(5)-1)
       RETURN
       END
C ----------------------------------------------------------------------
       SUBROUTINE SYMRED (LRBUF,NF,NFL)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 26, BY M JACKA
C
C Revised 85 05 15, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : Decodes individual symmetry cards. symmetry
C           decoded from alphanumeric to a symmetry
C           matrix(S) and a translation matrix(T)
C
C Input   : LRBUF ; buffer holding symmetry data.
C
C Output  : S     ; Symmetry matrix.
C           T     ; Translation matrix.
C
C ----------------------------------------------------------------------
       CHARACTER *50 LRBUF
       DIMENSION NFL(18)
       COMMON /SYM/  NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
       NSYM=NSYM+1
       IA=0
       DO 10 I=1,NF
         SIGN=1.0
         DO 20 J=IA+1,IA+NFL(I)
           IF (LRBUF(J:J).EQ.'-') THEN
             SIGN= -1.0
             GO TO 20
             ENDIF
           IF (LRBUF(J:J).EQ.'1') THEN
             T(I,NSYM)=0.5
             GO TO 20
             ENDIF
           IF (LRBUF(J:J).EQ.'X') THEN
             S(I,1,NSYM)= SIGN
             GO TO 20
             ENDIF
           IF (LRBUF(J:J).EQ.'Y') THEN
             S(I,2,NSYM)= SIGN
             GO TO 20
             ENDIF
           IF (LRBUF(J:J).EQ.'Z') THEN
             S(I,3,NSYM)= SIGN
             GO TO 20
             ENDIF
20         CONTINUE
         IA=IA+NFL(I)
10       CONTINUE
       RETURN
       END
C --------------------------------------------------------
       SUBROUTINE SYMREDH (LRBUF,NF,NFL,ISYMF)
C --------------------------------------------------------
C
C WRITTEN 84 06 19, BY M JACKA
C
C Revised 85 05 15, By D Clarkson.
C
C --------------------------------------------------------
C
C Purpose : Identifies and interprets the Hermann-Mauguin
C           symbol or Hall symbol.
C
C Input   : LRBUF ; buffer holding the above symbols.
C
C Output  : HMSYM ; Hermann-Mauguin space group symbol.
C           HALL  ; Hall space group symbol.
C
C --------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8 (G)
       CHARACTER *24 LRBUF
       DIMENSION NFL(NF)
       COMMON /GSYM/  GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
       NFL1=NFL(1)
       IF (ISYMF.GE.1) GO TO 110
       GLAT=LRBUF(1:1)
       IF (GLAT.EQ.'R') THEN
         IF (LRBUF(NFL1:NFL1).EQ.'R'.OR.LRBUF(NFL1:NFL1).EQ.' ')
     1       GLAT='P'
         IF (LRBUF(NFL1:NFL1).EQ.'H') GLAT='H'
         ENDIF
       HMSYM=LRBUF(1:NFL1)
       ISYMF=1
       GO TO 140
C
C  Second space group card reads the Hall symbol
C  LTCODE is kept from the HM symbol
C
110    HALL=LRBUF(1:NFL1)
140    RETURN
       END
C ----------------------------------------------------------------------
       SUBROUTINE TEMPRED (LRBUF,NF,NFL,LRSZT,LRTYPJ)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 20, BY M JACKA
C
C Revised 85 05 15, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : Identifies temperature factors by GELEM and GDENT,
C           corresponding to data from ATOMRED.
C           Reads and converts temperature factors to U form.
C
C Subroutines called : EFNUM
C
C Input   : LRBUF ; buffer holding temperature factors in packed format.
C
C Output  : U     ; temperature factors for xatoms.
C           DU    ; errors in temperature factors.
C
C ----------------------------------------------------------------------
       IMPLICIT CHARACTER *8 (G)
       CHARACTER *60 LRBUF
       DIMENSION NFL(14)
       COMMON /FILES/   NIN,NOUT,NDEBUG,NREAD
       include 'comatom.for'
       COMMON /RCELL/   AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
C
       PI=3.1416
       TCONVB=2.0*PI**2
       K=1
       GELEMT=LRBUF(K:K+NFL(1)-1)
       K=K+NFL(1)
       GDENTT=LRBUF(K:K+NFL(2)-1)
       K=K+NFL(2)
       DO 10 I=1,NX
         IF (GELEMT.EQ.GELEM(I).AND.GDENTT.EQ.GDENT(I)) THEN
           NTX=I
           GO TO 40
           ENDIF
10       CONTINUE
       GO TO 901
40     CALL EFNUM (U(1,1,NTX),DU(1,1,NTX),LRBUF(K:K),NFL(3),NFL(4))
       K=K+NFL(3)+NFL(4)
       CALL EFNUM (U(2,2,NTX),DU(2,2,NTX),LRBUF(K:K),NFL(5),NFL(6))
       K=K+NFL(5)+NFL(6)
       CALL EFNUM (U(3,3,NTX),DU(3,3,NTX),LRBUF(K:K),NFL(7),NFL(8))
       K=K+NFL(7)+NFL(8)
       CALL EFNUM (U(1,2,NTX),DU(1,2,NTX),LRBUF(K:K),NFL(9),NFL(10))
       K=K+NFL(9)+NFL(10)
       CALL EFNUM (U(1,3,NTX),DU(1,3,NTX),LRBUF(K:K),NFL(11),NFL(12))
       K=K+NFL(11)+NFL(12)
       CALL EFNUM (U(2,3,NTX),DU(2,3,NTX),LRBUF(K:K),NFL(13),NFL(14))
       K=K+NFL(13)+NFL(14)
       IF (LRTYPJ.EQ.15) GO TO 150
       IF (LRTYPJ.EQ.14) GO TO 140
       IF (ABS(AS).LT.0.000001.OR.ABS(BS).LT.0.000001.OR.
     *     ABS(CS).LT.0.000001) THEN
         WRITE(NOUT,9021) LRTYPJ
9021     FORMAT (1X,' RECIPROCAL CELL CONSTANTS NOT CALCULATED ',I5)
         GO TO 150
         ENDIF
       PI2= 2.0*PI**2
       U(1,1,NTX)=U(1,1,NTX)/(PI2*AS**2)
       DU(1,1,NTX)=DU(1,1,NTX)/(PI2*AS**2)
       U(2,2,NTX)=U(2,2,NTX)/(PI2*BS**2)
       DU(2,2,NTX)=DU(2,2,NTX)/(PI2*BS**2)
       U(3,3,NTX)=U(3,3,NTX)/(PI2*CS**2)
       DU(3,3,NTX)=DU(3,3,NTX)/(PI2*CS**2)
       U(1,2,NTX)=U(1,2,NTX)/(PI2*AS*BS)
       DU(1,2,NTX)=DU(1,2,NTX)/(PI2*AS*BS)
       U(1,3,NTX)=U(1,3,NTX)/(PI2*AS*CS)
       DU(1,3,NTX)=DU(1,3,NTX)/(PI2*AS*CS)
       U(2,3,NTX)=U(2,3,NTX)/(PI2*BS*CS)
       DU(2,3,NTX)=DU(2,3,NTX)/(PI2*BS*CS)
       GO TO 150
140    U(1,1,NTX)=U(1,1,NTX)/TCONVB
       DU(1,1,NTX)=DU(1,1,NTX)/TCONVB
       U(2,2,NTX)=U(2,2,NTX)/TCONVB
       DU(2,2,NTX)=DU(2,2,NTX)/TCONVB
       U(3,3,NTX)=U(3,3,NTX)/TCONVB
       DU(3,3,NTX)=DU(3,3,NTX)/TCONVB
       U(1,2,NTX)=U(1,2,NTX)/TCONVB
       DU(1,2,NTX)=DU(1,2,NTX)/TCONVB
       U(1,3,NTX)=U(1,3,NTX)/TCONVB
       DU(1,3,NTX)=DU(1,3,NTX)/TCONVB
       U(2,3,NTX)=U(2,3,NTX)/TCONVB
       DU(2,3,NTX)=DU(2,3,NTX)/TCONVB
150    U(2,1,NTX)=U(1,2,NTX)
       DU(2,1,NTX)=DU(1,2,NTX)
       U(3,1,NTX)=U(1,3,NTX)
       DU(3,1,NTX)=DU(1,3,NTX)
       U(3,2,NTX)=U(2,3,NTX)
       DU(3,2,NTX)=DU(2,3,NTX)
       LRSZT=NF+K+1
       RETURN
901    WRITE (NOUT,9001) GELEMT,GDENTT
9001   FORMAT (' ',10X,' -- ERROR -- GELEM(TEMPRED) ',A2,'  ',
     *    'GDENT(TEMPRED)',A3,/,10X,'ATOMIC COORDINATES NOT READ IN',
     *    'YET')
999    RETURN
       END
C -----------------------------------------------------------------------
       SUBROUTINE TITLRED (JTITLF,LRBUF,NFL)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 22, BY M JACKA
C
C Revised 85 05 15, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : Reads the title of the reference.
C
C Input   : LRBUF ; buffer holding the title of the reference.
C
C Output  : HTITL ; title of the reference.
C
C ----------------------------------------------------------------------
       implicit character *8 (g)
       IMPLICIT CHARACTER *80 (H)
       CHARACTER *130 LRBUF
       COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     *                 HFORM(2),HCOMM(7),GRFACT,GRAD
       JTITLF=JTITLF+1
       HTITLE(JTITLF)=LRBUF(1:80)
C
C  If the title is longer than 80 characters then we place the
C  remainder in the next HTITLE array location.
C
       IF (NFL.GT.80) THEN
         JTITLF=JTITLF+1
         IF (JTITLF.GE.3) RETURN
         HTITLE(JTITLF)=LRBUF(81: )
         ENDIF
       RETURN
       END
c
c
c   The following routine is used for reading icsd data from Ottawa
c
      subroutine crystn(nentry,jent)
c
c   written 86.1.9 by I D Brown
c
c    calls MATHLB:upcase,efnum,maxdim,num,param
c
c   nentry is the entry number being sought
c   jent is the entry number found (=-1 if EOF reached first)
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  rs(3),ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      include 'comatom.for'
      common /dens/   zform,dens,ddens,cdens,fwgt,waveln,absorb,
     1                  fzero,tmprtr
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                  hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /dummy/  nxh,xnumh(10)
      common /gdummy/ gdh(10)
      common /htext/  gcryst(2),gtwin(5),htext(10,5),hdata(4,19)
      common/cond/ condit(6,8),nstd,mstd,cstd(6,4)
      dimension nbuf(20)
      data gblank/'        '/
c
c
      tconvb=8.0*3.1416**2
      icell=0
      jauth=0
      kcomm=0
      jent=0
      nx=0
      linc=0
c
c   find the entry 'nentry'
c   -----------------------
   20 read(nread,10000,end=40) lent,itype,irec,nind,hline
      if(ndebug.ne.0) then 
        write(ndebug,1001) nentry, lent
 1001   format(' *debug*crystn*1001* nentry =',I8,', lent =',I8)
        endif
      if(lent.eq.nentry.or.nentry.eq.0) then
        jent=lent
        write(gent,'(i8)') jent
        call upcase(gent)
        go to 100
        endif
      go to 20
   40 write(nout,10020) nentry
10020 format(' End of Crystin file reached before entry ',I6,' found')
      jent=-1
      return
c
c   read one record from the entry
c   ------------------------------
   50 do 55 i=1,20
   55   nbuf(i)=0
      read(nread,10000,end=9000) lent,itype,irec,nind,hline
10000 format(1x,i6,i2,i3,i3,a80)
      write(nout,10001) lent,itype,irec,nind,hline
10001 format(1x,i6,i2,2i3,a80)
      linc=linc+1
      if(linc.ge.20.and.nout.eq.6.and.nterm.ne.0) then
        linc=0
        endif
      if(jent.eq.0) then
        jent=lent
        write(gent,'(i8)') jent
        call upcase(gent)
        endif
      if(jent.ne.lent) then
        backspace(nread,err=9000)
        go to 9000
        endif
c
c   read name
c   ---------
  100 if(itype.ne.1) go to 200
      hname=hline
      go to 50
c
c   read trivial name
c   ----------------
  200 if(itype.ne.2) go to 300
      kcomm=kcomm+1
      if(maxdim(kcomm,7,'*error*crystn*200*').ne.0) go to 50
      hcomm(kcomm)=hline
      go to 50
c
c   read mineral name
c   -----------------
  300 if(itype.ne.3) go to 400
      kcomm=kcomm+1
      if(maxdim(kcomm,7,'*error*crystn*300*').ne.0) go to 50
      hcomm(kcomm)=hline
      go to 50
c
c   read formula
c   ------------
  400 if(itype.ne.4) go to 500
      if(irec.le.3) hdata(irec+1,5)=hline
      go to 50
c
c   read title
c   ----------
  500 if(itype.ne.5) go to 600
      if(irec.le.1) then
        htitle(irec+1)=hline
        endif
      go to 50
c
c   read reference
c   --------------
  600 if(itype.ne.6) go to 700
      if(irec.le.1) then
        hcodn(irec+1)=hline(nind+1:)
        endif
      go to 50
c
c   read authors
c   ------------
  700 if(itype.ne.7) go to 800
      if(irec.le.9) then
        hauth(irec+1)=hline
        jauth=irec+1
        endif
      go to 50
c
c   read unit cell
c   --------------
  800 if(itype.ne.8) go to 900
      do 810 i=1,20
        nbuf(i)=0
        if(i.le.nind) nbuf(i)=num(hline(i:i),1)
  810   continue
      icount=1+nind
      call efnum(a,da,hline(icount:),nbuf(1),nbuf(2))
      icount=icount+nbuf(1)+nbuf(2)
      call efnum(b,db,hline(icount:),nbuf(3),nbuf(4))
      icount=icount+nbuf(3)+nbuf(4)
      call efnum(c,dc,hline(icount:),nbuf(5),nbuf(6))
      icount=icount+nbuf(5)+nbuf(6)
      call efnum(alpha,dalpha,hline(icount:),nbuf(7),nbuf(8))
      icount=icount+nbuf(7)+nbuf(8)
      call efnum(beta,dbeta,hline(icount:),nbuf(9),nbuf(10))
      icount=icount+nbuf(9)+nbuf(10)
      call efnum(agamma,dgamma,hline(icount:),nbuf(11),nbuf(12))
      icount=icount+nbuf(11)+nbuf(12)
      call efnum(zform,dummy,hline(icount:),nbuf(13),0)
      icount=icount+nbuf(13)
      call efnum(dens,ddens,hline(icount:),nbuf(14),nbuf(15))
      degrad=3.141592654/180.0
      alpha =degrad*alpha
      dalpha=degrad*dalpha
      beta  =degrad*beta
      dbeta =degrad*dbeta
      agamma=degrad*agamma
      dgamma=degrad*dgamma
      if(ndebug.ne.0) write(ndebug,18000) alpha,dalpha,beta,dbeta,
     +               agamma,dgamma
18000 format(' *debug*crystn*18000* cell = ',6f8.3)
      icell=1
      call param
      go to 50
c
c   read space group symbol
c   -----------------------
  900 if(itype.ne.9) go to 1000
      icount=1+nind
      call upcase(hline(icount:))
      if(irec.eq.0) hmsym=hline(icount:)
      if(irec.eq.1) hall=hline(icount:)
      go to 50
c
c   read symmetry
c   -------------
 1000 if(itype.ne.10) go to 1100
      write(nout,11001)
11001 format(' crystn cannot yet read symmetry operators')
      go to 50
c
c   read ?
c   ------
 1100 if(itype.ne.11) go to 1200
      go to 50
c
c   read atom positions
c   -------------------
 1200 if(itype.ne.12) go to 1300
      do 1210 i=1,20
        nbuf(i)=0
        if(i.le.nind) nbuf(i)=num(hline(i:i),1)
 1210   continue
      nx=nx+1
      if(maxdim(nx,mx,'*error*crystn*1200*').ne.0) go to 50
      icount=1+nind
      gelem(nx)=hline(icount:icount+1)
      icount=icount+nbuf(1)
      gdent(nx)=gblank
      if(nbuf(2).ne.0) gdent(nx)=hline(icount:icount+nbuf(2)-1)
      icount=icount+nbuf(2)
      call efnum(oxid(nx),dum,hline(icount:),nbuf(3),0)
      icount=icount+nbuf(3)
      npos(nx)=num(hline(icount:),nbuf(4))
      icount=icount+nbuf(4)
      if(nbuf(5).ne.0) gwyck(nx)=hline(icount:icount+nbuf(5)-1)
      icount=icount+nbuf(5)
      if(nbuf(6).ne.0) go to 1215
c   we have a dummy atom with coordinates not 9 9 9
      xx(1,nx)=9.0
      icount=icount+nbuf(6)+nbuf(7)+nbuf(8)+nbuf(9)+nbuf(10)+nbuf(11)
 1215 do 1220 j=1,3
        k=4+2*j
        call efnum(xx(j,nx),dxx(j,nx),hline(icount:),
     +              nbuf(k),nbuf(k+1))
        icount=icount+nbuf(k)+nbuf(k+1)
 1220  continue
      gtemp=hline(icount:icount+nbuf(12)-1)
      icount=icount+nbuf(12)
      call efnum(u(1,1,nx),du(1,1,nx),hline(icount:),nbuf(13),nbuf(14))
      icount =icount+nbuf(13)+nbuf(14)
      if(gtemp.eq.'B') then
        u(1,1,nx)=u(1,1,nx)/tconvb
        du(1,1,nx)=du(1,1,nx)/tconvb
        endif
      call efnum(occu(nx),soccu(nx),hline(icount:),nbuf(15),nbuf(16))
      icount=icount+nbuf(15)+nbuf(16)
      nh(nx)=num(hline(icount:),nbuf(17))
      icount=icount+nbuf(17)
      if(hline(icount:icount).eq.'D') then
        nd(nx)=nh(nx)
        nh(nx)=0
        endif
      if(occu(nx).lt.0.0001) occu(nx)=1.0
      if(xx(1,nx).gt.8.0) then
c       we have a dummy atom
        do 1240 j=1,nxh
          if(gelem(nx).eq.gdh(j)) then
            xnumh(j)=xnumh(j)+npos(nx)*occu(nx)
            go to 1250
            endif
 1240     continue
        nxh=nxh+1
        if(maxdim(nxh,10,'*ERROR* more that 10 dummy elements')
     1      .eq.1) go to 1250
        gdh(nxh)=gelem(nx)
        xnumh(nxh)=npos(nx)*occu(nx)
 1250   continue
        do 1255 j=1,3
          xx(j,nx)=0.0
          dxx(j,nx)=0.0
 1255     continue
        occu(nx)=0.0
        soccu(nx)=0.0
        u(1,1,nx)=0.0
        du(1,1,nx)=0.0
        nx=nx-1
        endif
      go to 50
c
c   read anisotropic atomic dispalcement parameters
c   -----------------------------------------------
 1300 if(itype.lt.13.or.itype.gt.15) go to 1600
      do 1320 i=1,20
        nbuf(i)=0
        if(i.le.nind) nbuf(i)=num(hline(i:i),1)
 1320   continue
      icount=1+nind
      glm=hline(icount:icount+nbuf(1)-1)
      icount=icount+nbuf(1)
      gdnt=hline(icount:icount+nbuf(2)-1)
      icount=icount+nbuf(2)
      do 1330 i=1,nx
        if(glm.eq.gelem(i).and.gdnt.eq.gdent(i)) then
          l=i
          go to 1338
          endif
 1330   continue
      write(nout,11330) glm,gdnt
11330 format(' Error: ADP''s read before atom coordinates for ',A2,A6)
 1338 call efnum(u(1,1,l),du(1,1,l),hline(icount:),nbuf(3),nbuf(4))
      icount=icount+nbuf(3)+nbuf(4)
      call efnum(u(2,2,l),du(2,2,l),hline(icount:),nbuf(5),nbuf(6))
      icount=icount+nbuf(5)+nbuf(6)
      call efnum(u(3,3,l),du(3,3,l),hline(icount:),nbuf(7),nbuf(8))
      icount=icount+nbuf(7)+nbuf(8)
      if(nind.le.8) go to 1340
      call efnum(u(1,2,l),du(1,2,l),hline(icount:),nbuf(9),nbuf(10))
      icount=icount+nbuf(9)+nbuf(10)
      if(nind.le.10) go to 1340
      call efnum(u(1,3,l),du(1,3,l),hline(icount:),nbuf(11),nbuf(12))
      icount=icount+nbuf(11)+nbuf(12)
      if(nind.le.12) go to 1340
      call efnum(u(2,3,l),du(2,3,l),hline(icount:),nbuf(13),nbuf(14))
      icount=icount+nbuf(13)+nbuf(14)
 1340 continue
      u(2,1,l)=u(1,2,l)
      du(2,1,l)=du(1,2,l)
      u(3,1,l)=u(1,3,l)
      du(3,1,l)=du(1,3,l)
      u(3,2,l)=u(2,3,l)
      du(3,2,l)=du(2,3,l)
c   case for betas
c   --------------
      if(itype.eq.13) then
        if(icell.ne.1) then
          write(nout,11340)
11340     format(' Atomic displacement parameter not converted ',
     +           'to U form')
        else
          pi2=2*(3.14153**2)
          do 1350 i=1,3
            do 1350 j=1,3
              u(i,j,l)=u(i,j,l)/(pi2*rs(i)*rs(j))
              du(i,j,l)=du(i,j,l)/(pi2*rs(i)*rs(j))
 1350         continue
          endif
        endif
c   case for B's
c   ------------
      if(itype.eq.14) then
        pi2=8*(3.14153**2)
        do 1360 i=1,3
          do 1360 j=1,3
            u(i,j,l)=u(i,j,l)/pi2
            du(i,j,l)=du(i,j,l)/pi2
 1360       continue
        endif
      go to 50
c
c   read remark
c   -----------
 1600 if(itype.ne.16) go to 1700
      kcomm=kcomm+1
      if(maxdim(kcomm,7,'*error*crystn*1600*').ne.0) go to 50
      hcomm(kcomm)=hline
      go to 50
c
c   read ?
c   ------
 1700 if(itype.ne.17) go to 1800
      go to 50
c
c   read r factor
c   -------------
 1800 if(itype.ne.18) go to 1900
      call efnum(condit(1,7),dummy,hline(1:),5,0)
      grfact=hline
      go to 50
c
c   we have an error
c   ----------------
 1900 write (nout,19001) itype
19001 format(' ',i3,' is an unrecognised record type')
      go to 50
 9000 return
      end

***********************************************************************

      subroutine skpfiz(nent,ierr)  

c
c  This subroutine finds the location of the first entry in an ICSD       
c  (CD-ROM) file, then reads that entry in using rdfiz.
c
c  Written by C.Brodie    14 July 1992
c  
      implicit character*8 (g)
      implicit character*80 (h)
      common /gtitle/ gent,hname,hcodn(2),htitl(2),hauth(10),hform(2),
     1                   hcomm(7),grfact,grad
      common /href/ hinfo(2,2),gfilen(5)
      common /files/ nin,nout,ndebug,nread
      if (ndebug.ne.0) write (ndebug,1940) nread
 1940 format ('*DEBUG* skpfiz *1940*',I4)
      do 1902 i=1,nent
 1901       read (nread,19002,end=1904) irec,jrec,hrec
            if (irec.ne.0) go to 1901
            gent=hrec(1:6)
            ierr=0
 1902 continue
19002 format (I4,I1,3X,A72)
      call rdfiz(ierr)
      return
 1904 write (nout,19003) gfilen(nread)
      ierr=+1
19003 format ('End of file on',A20)
      return
      end


********************************************************************

      subroutine rdfiz(ierr)

c
c  This subroutine reads in data from an ICSD (CD-ROM) file, assuming
c  that the beginning of the desired entry has already been found.  
c
c  Written by C.Brodie    14 July 1992
c
c  Subroutines called: 
c  This file:  tfconv
c  Mainlb:  param
c
c  Each line of data is read in as the record label (broken into 
c  irec and jrec) and a character string containing the data to be
c  stored.  Character data within the string is extracted by character
c  number, and numerical data is read from the string as if the string
c  were an internal file, using an internal read (occasionally, both
c  numeric and character data are read in this way using a format 
c  statement, for ease and efficiency).
c
      implicit character*8 (g)
      implicit character*80 (h)
      include 'comatom.for'
      common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      common /cell/ a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/ da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /gsym/ glat,gcent,hmsym,hall,gsysp(2,30)
      common /dens/ zform,dens,ddens,cdens,fwgt,waveln,absorb,fzero,
     1                 tmprtr
      common /gtitle/ gent,hname,hcodn(2),htitl(2),hauth(10),hform(2),
     1                  hcomm(7),grfact,grad
      common /title/ jauth,kcomm
      common /href/ hinfo(2,2),gfilen(5)
      common /htext/ gcryst(2),gtwin(5),htext(10,5),hdata(4,19)
      common /rcell/ as,bs,ds,ral,rbe,rga,rvol,rm(3,3)
C
C  Local variables used in reading of data.
C
      gtype=' '
      ktitl=0
      degrad=3.141592654/180.0
      ierr=-1
C
C  Begin reading at first line of data after the collection code.
C
      if (ndebug.ne.0) write (ndebug,449) nread
  449 format ('*DEBUG* rdfiz *449*',I4)
      write (nwrite,2000)   
 2000 format ('ICSD file read in from CD-ROM'//)       
  450 read (nread,1000,end=910) irec,jrec,hrec
 1000 format (I4,I1,3X,A72)
C
C  irec=990 indicates the end of an entry.
C
      if (irec.eq.990) go to 900
      if (irec.eq.10) then
              hname=hrec(1:72)
              write (nwrite,2010) hname
 2010 format (A73)
      else if (irec.eq.40) then
              hform(1)=hrec(1:72)
              hdata(1,5)=hrec(1:72)
              write (nwrite,2010) hdata(1,5)
      else if (irec.eq.50) then
              ktitl=ktitl+1
              if (ktitl.le.2) then
                htitl(ktitl)=hrec(1:72)
                write (nwrite,2010) htitl(ktitl)
                endif
      else if (irec.eq.60) then
              hcodn(1)=hrec(1:72)
              write (nwrite,2010) hcodn(1)
      else if (irec.eq.70) then
              jauth=jauth+1
              if (maxdim(jauth,7,'author').ne.0) go to 450
              hauth(jauth)=hrec(1:72)
              write (nwrite,2010) hauth(jauth)
      else if (irec.eq.80) then
                read (hrec(1:66),1010) a,da,b,db,c,dc,alpha,dalpha
 1010 format (3(F9.5,E8.0),F7.3,E8.0)
                if (da.lt.0) da=0.0
                if (db.lt.0) db=0.0
                if (dc.lt.0) dc=0.0
                if (dalpha.lt.0) dalpha=0.0
      else if (irec.eq.81) then
              read (hrec(1:47),1020) beta,dbeta,agamma,dgamma,zform,
     1                  dens,ddens
 1020 format (2(F7.3,E8.0),F3.0,F6.3,E8.0)
                if (dbeta.lt.0) dbeta=0.0
                if (dgamma.lt.0) dgamma=0.0
                if (ddens.lt.0) ddens=0.0
              write (nwrite,2020) a,b,c,alpha,beta,agamma,dens
              write (nwrite,2020) da,db,dc,dalpha,dbeta,dgamma,ddens
 2020 format (3F7.3,4F7.2)
C
C  All angles are converted from degrees to radians.
C
              alpha=alpha*degrad
              beta=beta*degrad
              agamma=agamma*degrad
              dalpha=dalpha*degrad
              dbeta=dbeta*degrad
              dgamma=dgamma*degrad
              call param
              icell=1
              if (ndebug.ne.0) then
                write (ndebug,2030) am
                write (ndebug,2030) rm
                do 50 i=1,6
                   write (ndebug,2030) (dm(j,i),j=1,9)
 2030 format (9F8.4)
   50           continue
                endif
      else if (irec.eq.90) then
              hmsym=hrec(1:72)
      else if (irec.eq.120) then
              nx=nx+1
              if (maxdim(nx,mx,'atom').ne.0) go to 450
              read (hrec(1:63),1030) gelem(nx),gdent(nx),oxid(nx),
     1                  npos(nx),gwyck(nx),xx(1,nx),dxx(1,nx),xx(2,nx),
     1                  dxx(2,nx),xx(3,nx),dxx(3,nx)
 1030 format (A2,A3,F6.3,I3,A1,3(F8.5,E8.0))
              do 60 i=1,3
                if (dxx(i,nx).lt.0) dxx(i,nx)=0.0
   60           continue
              write (nwrite,2040) gelem(nx),gdent(nx),oxid(nx),npos(nx),
     1                  gwyck(nx),(xx(i,nx),i=1,3)
 2040 format (A2,1X,A3,1X,F6.3,1X,I3,1X,A1,1X,3(F8.5,1X))
              write (nwrite,2050) (dxx(i,nx),i=1,3)
 2050 format (20X,3(F8.5,1X))
      else if (irec.eq.121) then
              read (hrec(1:36),1040) gtempf,u(1,1,nx),du(1,1,nx),
     1                  occu(nx),soccu(nx)
 1040 format (A1,E11.4,E9.0,F7.4,E8.0)
              if (du(1,1,nx).lt.0) du(1,1,nx)=0.0
C
C  Temperature factors are converted if necessary.
C
              if (gtempf.eq.'B') then
                      call tfconv(u(1,1,nx))
                      call tfconv(du(1,1,nx))
              endif
C  Number of hydrogen or deuterium atoms is determined and read.
C
              gtype=hrec(38:38)
              if (gtype.eq.'H') then
                      read (hrec(37:37),1045) nh(nx)
              else if (gtype.eq.'D') then
                      read (hrec(37:37),1045) nd(nx)
              endif
 1045 format (I1)
C
C  Temperature factor matrices are matched with the correct atoms, 
C  read, and converted if necessary.
C
      else if (irec.eq.130.or.irec.eq.140.or.irec.eq.150) then
              loc=0
              read (hrec(1:5),1047) gatom,gnum
 1047 format (A2,A3)
              call upcase (gatom)
              call upcase (gelem(loc))
              do 10 loc=1,nx
                 if (gatom.eq.gelem(loc).and.gnum.eq.gdent(loc)) then 
                      go to 500
                 endif
   10         continue
  500         read (hrec(6:65),1050) u(1,1,loc),du(1,1,loc),
     1                  u(2,2,loc),du(2,2,loc),u(3,3,loc),du(3,3,loc)
 1050 format (3(E11.4,E9.0))
      else if (irec.eq.131.or.irec.eq.141.or.irec.eq.151) then
              read (hrec(1:60),1060) u(1,2,loc),du(1,2,loc),
     1                  u(1,3,loc),du(1,3,loc),u(2,3,loc),du(2,3,loc)
 1060 format (3(E11.4,E9.0))
              u(2,1,loc)=u(1,2,loc)
              du(2,1,loc)=du(1,2,loc)
              u(3,1,loc)=u(1,3,loc)
              du(3,1,loc)=du(1,3,loc)
              u(3,2,loc)=du(3,2,loc)
              do 70 i=1,3
                do 80 j=1,3
                  if (du(i,j,loc).lt.0) du(i,j,loc)=0.0
   80             continue
   70           continue
              if (irec.eq.131) then
                      call tfconv(u(1,1,loc))
                      call tfconv(du(1,1,loc))
              endif
              if (irec.eq.141) then
                      pi8=8*(3.141592654**2)
                      do 20 i=1,3
                         do 30 j=1,3
                            u(i,j,loc)=u(i,j,loc)/pi8
                            du(i,j,loc)=du(i,j,loc)/pi8
   30                    continue
   20                 continue
              endif
              write (nwrite,2060) gelem(loc),gdent(loc),
     1                  ((u(i,j,loc),i=1,3),j=1,3)
 2060 format (A2,1X,A3,1X,9(E8.5,1X))
              write (nwrite,2070) ((du(i,j,loc),i=1,3),j=1,3)  
 2070 format (7X,9(E8.5,1X))
      else if (irec.eq.180) then
              grfact=hrec(6:14)
              write (nwrite,2080) grfact
 2080 format ('R factor= ',A7)
              endif
      go to 450
  900 ierr=0
      return
  910 write (nout,1070) gfilen(nread)
      ierr=+1
 1070 format ('End of file on',A20) 
      return
      end

****************************************************************************

      subroutine tfconv(u)
c
c  Subroutine to convert temperature factors.
c
c  Written by C.Brodie   14 July 1992
c
      common /rcell/ rs(3)
      common /cell/ a,b,c,alpha,beta,gamma,vol,am(3,3),icell
      common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      dimension u(3,3)
      pi2=2*(3.141592654**2)
      if (icell.ne.1) then
              write (nout,2000)
2000  format ('Atomic displacement parameter not converted to U form',
     1             ' since cell not given')
      else
              do 40 i=1,3
                 do 50 j=1,3
                    u(i,j)=u(i,j)/(pi2*rs(i)*rs(j))
50               continue                   
40            continue                
      endif
      return
      end









