c
c                    FILE IOLIB            Last update: 26 April 1993
c                    ----------                          T. Stephen
c
c  ******************************************************************
c    MACHINE DEPENDENT CODE FLAGGED AS
c     'machine dependent'
c  ******************************************************************
c
c  f ifile(hf,gcc,gst,nun)  opens file HF on unit NUN with print control GCC
c                           and status GST
c  s loadvl(hvalence,nn)  loads bond valence parameters
c  f hfilen(i)           gets the name of the file open as unit i
c  f iclofi(nunit)       closes unit NUNIT checking its status
c  f iexfil(hname1,hname2)  checks the existence of file HNAME1 in directory
c  f iclosu(nunit,hfile) closes a unit/file opened by IFILE
c  s chemgr              produces input files for the CHEMGRAPH-system
c  s atomgr              produces input files for the program ATOMS
c  s page(gcomd)         prints the checked data to the output stream
c  s paper(nprint)       prints a short structural paper
c  f nline(ip,if,nfile)  starts a new line on nfile if (ip+if).gt.90
c  s pub(nprint)         prints atom information to 'nprint' for publication
c  s table1(nprint)      prints structural information to 'nprint'
c  s table3(nprint)      prints bond information to 'nprint'
c  s rval(nval)          reads valence parameters from unit 'nval'
c  s rdbonf(ier)         reads a bond file entry from unit 'nread'
c  s fdbonf(gen,ken)     locates entry 'gen' on the bondfile (unit 'nread')
c  s qmenu               menu for quattro output
c  s qpro                quattro output creation
c  s qhelp               help for quattro output stuff
c  s capsuplo            draws pseudoplot-projection of coordinationsphere around ict
c  s drawli              slave of capsuplo for line-drawing.
c
c      externals called from this file:
c
c  for-open
c
c  CHEMLB: abost, acid, calval, contac
c  DISTLB: conxyz, indepb, bonfit
c  MAINLB: param, sghall
c  MATHLB: hxword, integer, second, maxdim, hleft, upcase, length, inerr
c  SCFSLB: ueq
c  SPELIB: spost, spesor, spmult, spsym
c  SYMLIB: getstl, invstl, mltstl, sittrn, grmult
c  SPANNER:atomsl, bondsl, expand, jyat
c  PLOTLIB: ortho
c
c********* BEWARE:  Machine dependent code in IFILE
c                   flagged 'machine dependent code'
c
c********* BEWARE:  Some routines in this file are interactive !!
c
c********* BEWARE:  There is non-standard Fortran 77
c                   code in RVAL (use of ICHAR)
C
C
C
C
      FUNCTION IFILE(HFILE,GFORT,GSTAT,nunit)
C     -------- ------------------------------
C
C   WRITTEN 85.6.10 BY I D BROWN AND DAVID KNIGHT
C   Revised by D. Altermatt to work on VAX, N500 and Cyber
C                           (see flag 'machine' in STRUMO)
C
c   Calls iclifi, iexfil, length, upcase
C
c   This function is the systems file manager.
c
c   INPUT:
c
c     HFILE:  filename (including filetype if necessary) of max. 40
c             characters (rest ignored), '*' or blank. If no filename
c             specified, initialisation of input and output stream, only.
c             System default filenames are: INPUT, OUTPUT, COMMAND
c             (= input) and DEBUG (= dump file)
c
c     GFORT:  'LIST', 'FORTRAN', 'PRINT' or blank (= print file) for
c             the print (carriage) control flag of the file
c
c     GSTAT:  'OLD', 'NEW', 'UNKNOWN' or blank (= unknown) for the
c             expected file status in the directory
c             ****** 'OLD' files are opened as READONLY files
c
c     NUNIT:  positive integer less than 100 or zero (= take next free one)
c             for unit on which HFILE is to be opened
c
c
c   RETURNS:
c
c     IFILE = -1:   an error occurred (see message on output stream)
c
c              0:   if HFILE was '*' or blank and the default streams
c                   were initialized properly
c
c              N:   file with given attributes successfully opened on
c                   unit N. If NUNIT was given 0 or NUNIT was already
c                   occupied by another file, N is found as
c                   MAX(NFUNIT)+1
c
C
C   There are two default file names:
C                                          File on:
C   Name    Unit (if not given)  VAX     N500      Cyber    UNIX     PC
C
C   INPUT   nin      (5)          5    TERMINAL   $INPUT   /dev/tty   5
C   OUTPUT  nout     (6)          6    TERMINAL   $OUTPUT  /dev/tty   6
C
C
C***** BEWARE: Do not close any files during execution except by using
C              function ICLOSU !!
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
c
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR,machine,nterm
      COMMON/MFILE/NFILE,MFILE,nfunit(20)
      COMMON/GFILE/HFNAME(20)
      mfile=20
c
c     Set error flag
c
      ifile = -1
C
c     Default call (initialisation of system)
c
	IF(NFILE.EQ.0) THEN
	HFNAME(1) = 'INPUT'
	HFNAME(2) = 'OUTPUT'
	if(nin.le.0.or.nin.gt.99) nin=-5
	if(nout.le.0.or.nout.gt.99) nout=-6
	if(machine.eq.1) then
	  gmach='VAX'
	  hfiles='TAPE5'
	  nfile=1
	  iunit=iabs(nin)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,status='unknown',err=999)
	  hfiles='TAPE6'
	  nfile=2
	  iunit=iabs(nout)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,status='unknown',err=999)
	else if(machine.eq.2) then
	  gmach='ND500'
	  hfiles='TERMINAL'
	  nfile=1
	  iunit=iabs(nin)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,file=hfiles,status='unknown',err=999)
	  nfile=2
	  iunit=iabs(nout)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,file=hfiles,status='unknown',err=999)
	else if(machine.eq.3) then
	  gmach='CYBER'
	  hfiles='$INPUT'
	  nfile=1
	  iunit=iabs(nin)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,file=hfiles,status='unknown',err=999)
	  hfiles='$OUTPUT'
	  nfile=2
	  iunit=iabs(nout)
	  if(iclofi(iunit).ne.0) goto 999
	  open(iunit,file=hfiles,status='unknown',err=999)
	else if(machine.eq.4) then
	  gmach='UNIX'
	  hfiles='/dev/tty'
	  nfile=1
	  iunit=iabs(nin)
	  if(iclofi(iunit).gt.0) goto 999
	  open(iunit,file=hfiles,status='old',err=999)
	  hfiles='/dev/tty'
	  nfile=2
	  iunit=iabs(nout)
	  if(iclofi(iunit).ne.0) goto 999
	  open(iunit,file=hfiles,status='old',err=999)        
	else if(machine.eq.5) then
	  gmach='PC'
*          hfiles='/dev/tty'
	  nfile=1
	  nin=5
	  nout=6
	  iunit=iabs(nin)
*          if(iclofi(iunit).gt.0) goto 999
*          open(iunit,file=hfiles,status='old',err=999)
*          hfiles='/dev/tty'
	  nfile=2
	  iunit=iabs(nout)
*          if(iclofi(iunit).ne.0) goto 999
*          open(iunit,file=hfiles,status='old',err=999)        
	else
	  gmach='UNKNOWN'
	  nfile=1
	  goto 999
	  endif
	if(nout.lt.0) then
	  nout=iabs(nout)
	  write(nout,2000)
2000      format(//,' ***** CAUTION: Output stream defined by IFILE:',
     1      ' NOUT = 6 *****',/)
	else if(nin.lt.0) then
	  nin=iabs(nin)
	  write(nout,2010)
2010      format(//,' ***** CAUTION: Input stream defined by IFILE:',
     1      ' NIN = 5 *****',/)
	  endif
	nfunit(1)=nin
	nfunit(2)=nout
	ENDIF
      IF(HFILE.EQ.' '.OR.HFILE.EQ.'*') THEN
c
c     No filename given, so return IFILE = 0
c
	IFILE = 0
	RETURN
	ENDIF
c
c     Check input
c
      if (machine.eq.1) call upcase(hfile)
      if (machine.eq.2) call upcase(hfile)
      if (machine.eq.3) call upcase(hfile)
      if (machine.eq.4) call lowcase(hfile)
      if (machine.eq.5) call upcase(hfile)

      call upcase(gfort)
      call upcase(gstat)
	
      if(nunit.lt.0.or.nunit.gt.99) nunit=0
      hfile(41:80)=' '
      IF(HFILE.EQ.'COMMAND') HFILE='INPUT'
      IF(GSTAT.EQ.' ') GSTAT = 'UNKNOWN'
c
c     Check if HFILE is already in the list
c
      DO 10 I=1,NFILE
	IF(HFILE.EQ.HFNAME(I)) THEN
	  IFILE = nfunit(i)
	  IF(NDEBUG.gt.0) WRITE(NDEBUG,2100) HFILE,IFILE
2100      FORMAT(' **DEBUG**IFILE**2100** HFILE = ',A40,
     +      ' already opened on unit ',I4,/)
	  RETURN
	  ENDIF
   10   CONTINUE
c
c     Check unit number NUNIT and update if necessary
c     default the system file DEBUG to unit 9
c
      iknown=0
      ihigh=0
      if(hfile.eq.'DEBUG'.and.nunit.eq.0) nunit=9
      do 20 i=1,nfile
	if(nfunit(i).gt.ihigh) ihigh=nfunit(i)
20      if(nfunit(i).eq.nunit) iknown=1
      ihigh=ihigh+1
      nold=nunit
      if(nunit.eq.0) iknown=1
      if(iknown.ne.0) then
	if(nunit.gt.0) write(nout,2050) nunit,ihigh
2050    format(//,' *****CAUTION: IO-UNIT specified (',I4,') not',
     1    ' equal to IO-UNIT opened (',I4,') *****',//)
	nunit=ihigh
	endif
      if(ndebug.gt.0) write(ndebug,2110) nold,nunit
2110  format(' **DEBUG**IFILE**2110** IO-UNIT specified = ',I4,
     1  ', IO-UNIT found to open = ',I4)
c
c     Still room for an additional one?
c
      IF(NFILE.GE.MFILE) THEN
	WRITE(NOUT,2060) MFILE,hfile
2060    FORMAT(//,' *****ERROR**IFILE** Only a maximum of ',I3,
     +    ' filenames allowed',/,21X,'File ',A40,' not opened',//)
	if(nterm.ne.0) then
  180     write(nterm,2061)
 2061     format(' Do you want to try opening the file anyway (Y/N/E)')
	  call readyn(*200,*190,*190,*180)
	  endif
  190   return
	endif
c
c     System filename DEBUG found
c
  200 if(hfile.eq.'DEBUG') then
	nfile=nfile+1
	hfname(nfile)=hfile
	nfunit(nfile)=nunit
	if(machine.eq.1) then
	  hfile='DUMP.LIS'
	  gfort='FORTRAN'
	  gstat='NEW'
	else if(machine.eq.2) then
	  hfile='DUMP:LIS'
	  gfort='PRINT'
	  gstat='UNKNOWN'
	else if(machine.eq.3) then
	  hfile='DUMPLIS'
	  gstat='UNKNOWN'
	else if(machine.eq.4) then
	  hfile='dump.list'
	  gfort='FORTRAN'
	  gstat='NEW'
	else if(machine.eq.5) then
	  hfile='dump.lst'
	  gfort='FORTRAN'
	  gstat='NEW'
	  endif
      else
	nfile=nfile+1
	hfname(nfile)=hfile
	nfunit(nfile)=nunit
	endif
c
c     Now open this file
c
      if(gstat.eq.'OLD') then
	if(iexfil(hfile,hfile1).ne.0) then
	  write(nout,2070) hfile
2070      format(//,' *****ERROR**IFILE** File ',A40,/,
     1      21X,'does not exist in your directory!! File not opened',//)
	  nfile=nfile-1
	  return
	else if(ndebug.gt.0) then
	  write(ndebug,2120) hfile,hfile1
2120      format(' **DEBUG**IFILE**2120** File ',A40,' found ',/,
     1      24X,'in directory as ',A80)
	  endif
	endif
	gmach='UNKNOWN'
      if(machine.eq.1) then
	gmach='VAX'
	if(gstat.eq.'OLD') then
	  if(iclofi(nunit).gt.0) goto 995
********************************************************************
*     machine dependent code -- OPEN statements not accepted by    *
*     compiler other than of VMS 4.x                               *
********************************************************************
*          open(nunit,file=hfile,status=gstat,shared,readonly,err=990)
	else
	  if(iclofi(nunit).gt.0) goto 995
	  if(gfort.ne.'FORTRAN') gfort='LIST'
c this line is not recognized by UNIX f77  (commented out)
	  open(nunit,file=hfile,status=gstat,carriagecontrol=gfort,
     1      err=990)
	  endif
      else if(machine.eq.2) then
	gmach='ND500'
	if(gstat.eq.'OLD') then
	  if(iclofi(nunit).gt.0) goto 995
	  gfort='R'
	  open(nunit,file=hfile,status=gstat,access=gfort,err=990)
	else
c
c     A safeguard. The Norsk will bomb if you try to open a second
c     version of an existing file
c
	  if(iexfil(hfile,hfile1).eq.0) gstat='UNKNOWN'
	  if(gfort.eq.'FORTRAN') gfort='PRINT'
	  if(gfort.ne.'PRINT') gfort='RW'
	  if(iclofi(nunit).gt.0) goto 995
	  open(nunit,file=hfile,access=gfort,status=gstat,err=990)
	  endif
      else if(machine.eq.3) then
	gmach='CYBER'
	if(gstat.eq.'OLD') then
	  if(iclofi(nunit).gt.0) goto 995
	  open(nunit,file=hfile,status=gstat,err=990)
	else
c
c     A safeguard. The Cyber will bomb if you try to open a second
c     version of an existing file
c
	  if(iexfil(hfile,hfile1).eq.0) gstat='UNKNOWN'
	  if(iclofi(nunit).gt.0) goto 995
	  open(nunit,file=hfile,status=gstat,err=990)
	  endif
      else if(machine.eq.4) then
	gmach='UNIX'
	if(gstat.eq.'OLD') then
	   if(iclofi(nunit).gt.0) goto 995
	   open(nunit, file=hfile, status=gstat, err=990)
	else
	   if(iexfil(hfile, hfile1).eq.0) gstat='UNKNOWN'
	   if(iclofi(nunit).gt.0) goto 995
	   open(nunit,file=hfile,status=gstat,err=990)
	endif
      else if (machine.eq.5) then
	gmach='PC'
	if(gstat.eq.'OLD') then
	   if(iclofi(nunit).gt.0) goto 995
	   open(nunit, file=hfile, status=gstat, err=990)
	else
	   if(iexfil(hfile, hfile1).eq.0) gstat='UNKNOWN'
	   if(iclofi(nunit).gt.0) goto 995
	   open(nunit,file=hfile,status=gstat,err=990)
	endif
      else
	write(nout,2080) machine
2080    format(//,' *****ERROR**IFILE** Illegal machine flag ',I3,
     1    ', file not opened',//)
	nfile=nfile-1
	return
	endif
      if(hfname(nfile).eq.'DEBUG') write(nout,2030) hfile
2030  format(//,' A file called ',A8,' has been created in your ',
     1  'DIRECTORY',//)
      if(ndebug.gt.0) write(ndebug,2130) hfile,gfort,gstat,nunit,gmach
2130  format(' **DEBUG**IFILE**2130** File ',A40,' opened with',/,
     1  ' print control ',A8,', status ',A8,' on unit ',I4,
     2  ', machine ',A8)
      ifile=nunit
      return
c
c     Error cases
c
990   write(nout,2200) hfile,gfort,gstat,nunit,gmach
2200  format(//,' *****ERROR**IFILE** Cannot open file ',A40,' with',/,
     1  ' print control ',A8,', status ',A8,' on unit ',I4,
     2  ', machine ',A8,//)
      nfile=nfile-1
      return
995   write(nout,2210) nunit,hfile
2210  format(//,' *****ERROR**IFILE** Error in checking status of unit '
     1  ,I4,/,21X,'File ',A40,' not opened *****',//)
      nfile=nfile-1
      return
999   WRITE(NOUT,2220) hfname(nfile),hfile,iunit,gmach
2220  format(//,' *****ERROR**IFILE** Cannot open system file ',A8,
     1  ' as ',A8,/,21X,'on unit ',I4,', machine ',A8,' *****',//)
      nfile=0
      return
      END
c
c
*************************************************************************
c
      subroutine loadvl(hvalence,nn)
c
c   subroutine to load bond valence parameters
c  
      implicit character*8 (g)
      implicit character*80 (h)
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),
     1                pbost(5,30)
      common /gbost/ gbost(2,30)
      common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      nbost=0
      nunit=0
      gfort=' '
      gstat='OLD'
      nn=ifile(hvalence,gfort,gstat,nunit)
      if(nn.ne.0) then
	rewind nn
	call rval(nn)
	write(nout,1250) ((gbost(j,i),ioxbost(j,i),j=1,2),ifbost(i),
     1      (pbost(j,i),j=1,5),i=1,nbost)
 1250   format(' Bond valence parameters loaded',/,
     1         ' ------------------------------',/,
     2   30(3X,2(A2,'(',I3,')'),2X,I1,2X,5F6.3,/))
	call wait
	endif
      return
      end

***********************************************************************
c

      function hfilen(i)
c
c   returns the name of the file that is opened as unit i
c
c   written 1986.6.5 by I.D.Brown
c
      implicit character*80 (h)
      common/gfile/hfname(20)
      common/mfile/nfile,mfile,nfunit(20)
c
      hfilen=' '
      do 100 k=1,nfile
	if(nfunit(k).eq.i) then
	  hfilen=hfname(k)
	  if(hfilen.eq.'DEBUG') then
	    if (machine.eq.1) then
	      hfilen='DUMP.LIS'
	    else if(machine.eq.2) then
	      hfilen='DUMP:LIS'
	    else if(machine.eq.3) then
	      hfilen='DUMPLIS'
	    else if(machine.eq.4) then
	      hfilen='dump.list'
	    else if(machine.eq.5) then
	      hfilen='dump.lst'
	      endif
	    endif
	  return
	  endif
  100   continue
      return
      end
c
c
c

      function iclofi(iunit)
c
c     Written by D. Altermatt, Uni Lausanne, May 26, 1986
c
c     Checks status of unit IUNIT and closes it if opened
c
c     Returns ICLOFI = 0 if IUNIT closed properly
c                      1 if an error occurred
c
      logical lopen
c
      iclofi = 1
      if(iunit.le.0) return
      inquire(iunit,opened=lopen,err=99)
      if(lopen) close (iunit)
      iclofi = 0
99    return
      end
c
c
c
c
      function iexfil(hfile,hname)
c
c     Written by D. Altermatt, Uni Lausanne, May 26, 1986
c
c     Checks the existance of file HFILE in the current directory,
c     changes HNAME to the name found in the directory
c
c     Returns IEXFIL = 0 if HFILE exists
c                      1 if HFILE does not exist: Error
c
      logical lexist
      character*80 hfile,hname
c
      iexfil = 1
      if(hfile.eq.' '.or.hfile.eq.'*') return
      inquire(file=hfile,name=hname,exist=lexist,err=99)
      if(lexist) iexfil = 0
99    return
      end
c
c
c
c
      function iclosu(nunit,hfile)
c
c     Written by D. Altermatt, Uni Lausanne, May 27,1986
c
c     Properly closes a unit/file opened by function IFILE.
c
c     INPUT:
c       NUNIT:   unit nr. in NFUNIT to be closed or 0
c
c       HFILE:   file name (max. 40 char.) in HFNAME to be closed of blank
c
c     RETURNS:
c       ICLOSU = 0  if file properly closed, say:
c                   if NUNIT given and HFILE blank: NUNIT found in NFUNIT
c                   if HFILE given and NUNIT 0    : HFILE found in HFNAME
c                   if both given                 : match in NFUNIT(I) and
c                                                   HFNAME(I)
c
c                1  error: no match or both dummy
c
c     Calls: upcase
c
      implicit character*80 (h)
      implicit character*8 (g)
c
      common /mfile/ nfile,mfile,nfunit(20)
      common /gfile/ hfname(20)
c
      hfile(41:80)=' '
      call upcase(hfile)
      if(nunit.le.0.or.nunit.gt.99) nunit=0
      do 10 i=1,nfile
	if(hfname(i).eq.hfile) then
	  if(nfunit(i).eq.nunit.or.nunit.eq.0) then
	    j=i
	    goto 20
	  else
	    iclosu=1
	    return
	    endif
	else if(nfunit(i).eq.nunit) then
	  if(hfname(i).eq.hfile.or.hfile.eq.' ') then
	    j=i
	    goto 20
	  else
	    iclosu=1
	    return
	    endif
	  endif
10      continue
      iclosu=1
      return
20    nfile=nfile-1
      close (nfunit(j))
      do 30 i=j,nfile
	hfname(i)=hfname(i+1)
30      nfunit(i)=nfunit(i+1)
      iclosu=0
      return
      end
c
c
c
c
      subroutine chemgr
C     -----------------
c
c     calls bonfit, calcy, getstl, ifile, indepb, inerr, invstl, mltstl,
c           readyn, sittrn
c
c     prepares an input file for the CHEMGRAPH-system on the SSC-VAX
c
c     started by D. Altermatt
c     McMaster University, feb. 1985
c     but actually written by I.D.Brown 1985.3.19
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nnwrite,nspgp,machine
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /dens/   zform,dens,ddens
c PROGRAMMER'S NOTE --- /bost/ has been modified to /bostt/ for
c                       UNIX system.
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),
     1                pbost(5,30)
      common /gbost/  gbost(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /range/  nj,iox(20),top(20,20),bot(20,20),ddmax,
     1                ddmin,rmax,rmin,vmax,vmin,anglx
      common /grange/ gel(20)
      common /val/    xm(3,3),pmap(3),dmax,rrmax,jt,nz1
      common /gval/   gor(3),gem(20),gdm(20)
      common /gmenu/  hmenu(20,3)
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     +                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c
      dimension jbond(12),l0(3),kl(3),kl1(3),kl2(3),kl3(3),il1(3)
      dimension iyl(3),nt(3,1000),xmax(3),xmin(3),jyatom(5),jjy(3)
      dimension xy(3)
      data hblank(1:10)/'          '/
      data l0/0,0,0/
c
      rad=57.2957795131
      if(machine.eq.1) then
	gdtype='.cda'
	gctype='.cco'
      else if(machine.eq.2) then
	gdtype=':cda'
	gctype=':cco'
      else if(machine.eq.4) then
	gdtype='.cda'
	gctype='.cco'
      else if(machine.eq.5) then
	gdtype='.cda'
	gctype='.cco'
	endif
c
c   prepare files
c
      hfile(1:10)=hblank(1:10)
      hcile(1:10)=hblank(1:10)
   20 if(machine.eq.1) write(nout,2000)
      if(machine.eq.2) write(nout,2002)
      if(machine.eq.4) write(nout,2000)
      if(machine.eq.5) write(nout,2000)
 2000 format(/,' Two CHEMGRAF files will be written, a data ',
     + 'file with the extension .CDA',/,
     + ' and a command file with the extension .CCO.',/,
     + ' When in CHEMGRAF type ''READ OBEY nnn.CCO'' ',/,
     + ' where ''nnn'' is the file name you will be prompted',/,
     + ' for. This will in turn read in the data file.',/,
     + ' Do not forget to NOMINATE your terminal',///,
     + ' What is the ',
     + 'root name for the CHEMGRAF files (max. of 6 characters)? ',$)
 2002 format(/,' Two CHEMGRAF files will be written, a data ',
     + 'file with the extension :CDA',/,
     + ' and a command file with the extension :CCO.',/,
     + ' When in CHEMGRAF type ''READ OBEY nnn.CCO'',',/,
     + ' where ''nnn'' is the filename you will be prompted for',/,
     + ' this will in turn read in the data file.',/,
     + ///,'$What is the ',
     + 'root name for the CHEMGRAF files (max. of 6 characters)? ')
      read(nin,2001,err=20) gfile
 2001 format(A6)
      if(gfile.eq.' '.or.gfile(1:1).eq.' ') call inerr(*20)
      do 40 i=1,6
	if(gfile(i:i).eq.' ') go to 50
	hfile(i:i)=gfile(i:i)
	hcile(i:i)=gfile(i:i)
	j=i
   40   continue
   50 continue
      hfile(j+1:j+4)=gdtype(1:4)
      hcile(j+1:j+4)=gctype(1:4)
c
c   write data file (hfile)
c
      nunit=0
      gfort='LIST'
      gstat='NEW'
      nnw = ifile(hfile,gfort,gstat,nunit)
      if(nnw.le.0) return
      nwrite=nnw
      write(nwrite,1000) a,b,c,rad*alpha,rad*beta,rad*agamma
 1000 format(10X,6F10.4)
c
c   iflag and nflag are used to eliminate dummy atoms
c
      iflag=0
      nflag=0
      do 60 i=1,nx
	iflag=iflag+iabs(ialtno(i))
	nflag=nflag+npos(i)
   60   continue
   52 write(nout,1052)
 1052 format(//,' Menu 2.18: Preparing output for Chemgraf',/,
     1          ' ----------------------------------------',/,
     2 '  1. Plot asymmetric unit only',/,
     3 '  2. Expand coordinates to fill a given volume',/,
     4 '  3. Expand bond network to fill one cell',/,
     5 ///////,
     + ' 20. Return to menu 2',//,
     1 ' Type in selection ')
      read(nin,1053) instru
 1053 format(I1)
      if(instru.eq.20) return
      if(instru.ge.4.or.instru.le.0) go to 52
      if(instru.eq.1) go to 55
      if(instru.eq.2) go to 132
      if(instru.eq.3) go to 300
c
c    prepare data file for asymmetric unit only
c
   55 i10 = 1
      if(ind1.lt.nbon.and.nbon.gt.0) call indepb(i10,i10,nbon)
      do 100 i=1,nx
	if(iflag.eq.0.or.ialtno(i).gt.0) then
	  if(nflag.eq.0.or.npos(i).gt.0) then
	    if(nbon.gt.0) then
	      k=0
	      do 120 j=1,12
  120           jbond(j)=0
	      do 130 j=1,nbon
		if(inde1(j).eq.j) then
		  if(ibon(1,j).eq.i) then
		    k=k+1
		    jbond(k)=jbon(1,j)
		  else if(jbon(1,j).eq.i) then
		    k=k+1
		    jbond(k)=ibon(1,j)
		    endif
		  endif
  130           continue
	      endif
	    write(nwrite,1001) gelem(i),gdent(i),(xx(j,i),j=1,3),
     +          (jbond(j),j=1,12)
 1001       format(A2,A6,3F8.5,12I4)
	    endif
	  endif
  100   continue
	go to 800
c
c   prepare data file for expanded unit
c
  132 write(nout,1132)
 1132 format(//,' Enter the limits of the region to be filled',/,
     +      ' xmin xmax ymin ymax zmin zmax',/)
      read(nin,*) (xmin(i),xmax(i),i=1,3)
      do 133 i=1,3
  133   if(xmax(i).lt.xmin(i)) call inerr(*52)
      write(nout,1133)
 1133 format(//,' Enter the minimum and maximum valences of the ',/,
     + 'bonds to be plotted',/,
     + ' vmin vmax',/)
      read(nin,*) (bondlim(i),i=1,2)
      call calcy(xmax,xmin)
      do 140 i=1,ny
  140   call getstl(nt(1,i),iynum(i),yx(1,i),iysym(i))
      do 180 i=1,ny
	if(iflag.ne.0.and.ialtno(iynum(i)).le.0) go to 180
	if(nflag.ne.0.and.npos(iynum(i)).le.0) go to 180
	nbond=0
	do 144 j=1,12
  144     jbond(j)=0
	if(nbon.le.0) go to 175
	do 170 j=1,nbon
	  call bonfit(j,iynum(i),iys,iyl,k,ks,kl)
	  if(k.ne.0) then
	    if(xbon(3,j).ge.bondlim(1).and.xbon(3,j).le.bondlim(2))then
	      n=1
	      call invstl(n,is1,il1,iys,iyl,iysym(i),nt(1,i))
	      if(n.eq.0) write(nout,1144) iys
 1144         format(//,' *error*chemgr*1144* symmetry op.',I3,
     +               ' not found',//)
	      n=1
	      call mltstl(ks1,kl1,is1,il1,ks,kl,n)
	      n=0
  146         n=n+1
	      call sittrn(n,ks3,kl3,ialtno(k),ks1,kl1,
     +               ialtno(iynum(i)),iysym(i),nt(1,i))
	      if(n.le.0) go to 170
	      if(ndebug.ne.0) write(ndebug,9144)i,j,iysym(i),
     +          (nt(l,i),l=1,3),ks3,(kl3(l),l=1,3),ks1,(kl1(l),l=1,3),
     +          ks,(kl(l),l=1,3),is1,(il1(l),l=1,3),iys,(iyl(l),l=1,3)
 9144         format(' *debug*chemgr*9144* ',2I3,1X,24I3)
	      do 160 l=1,ny
		if(iynum(l).eq.k.and.iysym(l).eq.ks3) then
		  do 150 ll=1,3
  150               if(kl3(ll).ne.nt(ll,l)) go to 160
		  nbond=nbond+1
		  jbond(nbond)=l
		  endif
  160           continue
	      go to 146
	      endif
	    endif
  170     continue
c
c    remove duplicate bonds
c
	if(nbond.gt.1) then
	  do 174 j=1,nbond
	    jj=j+1
	    do 173 l=jj,nbond
	      if(jbond(j).eq.jbond(l))then
		nbond=nbond-1
		do 172 ll=l,nbond
  172             jbond(ll)=jbond(ll+1)
		jbond(nbond+1)=0
		endif
  173         continue
  174       continue
	  endif
  175   write(nwrite,1175) gelem(iynum(i)),i,gdent(iynum(i)),
     +     (yx(j,i),j=1,3),(jbond(j),j=1,12)
 1175   format(A2,2X,I3,A1,3F8.5,12I4)
  180   continue
      go to 800
c
c   prepare atom file for expanded network
c
  300 if(nbon.le.0) then
	write(nout,1301)
 1301   format(/,' You have given me no bonds so I am unable to ',
     +                                'expand the bond network')
	go to 52
	endif
      call atomsl(iat)
      call bondsl(bondlim,iconfl)
      write(nout,1300) gelem(iat),gdent(iat),(bondlim(k),k=1,2)
 1300 format(' Calculating the network around ',A2,A6,' with bonds ',
     1 'between ',F5.2,' and ',F5.2,' v.u.')
      call expand(1)
      do 350 i=1,nya
	do 310 j=1,12
  310     jbond(j)=0
	k=0
	do 320 j=1,nbo
	  if(ibond(1,j).eq.i) then
	    k=k+1
	    if(maxdim(k,12,'*chemgr*320* jbond').ne.0) go to 330
	    jbond(k)=ibond(2,j)
	  else if(ibond(2,j).eq.i) then
	    k=k+1
	    if(maxdim(k,12,'*chemgr*321* jbond').ne.0) go to 330
	    jbond(k)=ibond(1,j)
	    endif
  320     continue
  330   call jyat(i,jyatom)
	do 340 k=1,3
  340     jjy(k)=jyatom(2+k)/12+5
	call symopl(xy,s(1,1,jyatom(2)),t(1,jyatom(2)),jyatom(3),
     +                xx(1,jyatom(1)))
	write(nwrite,1350) gelem(jyatom(1)),gdent(jyatom(1))(1:1),
     +    jyatom(2),(jjy(k),k=1,3),(xy(k),k=1,3),
     +    (jbond(k),k=1,12)
 1350   format(A2,A1,I2,3I1,3F8.5,12I4)
  350   continue
c
c   write command file
c
  800 hparf='EXEC:CHEMX.PTN'
      write(nout,2900) hparf(1:14)
 2900 format(' What is the name of the parameter file you ',
     +                         'wish to use?',/,
     + ' (Default is ',A20,')')
      read(nin,2901) hparff
 2901 format(A20)
      if(hparff(1:4).ne.'    ') hparf=hparff
      nunit=0
      gfort='list'
      gstat='new'
      mmw=ifile(hcile,gfort,gstat,nunit)
      if(mmw.le.0) return
      mwrite=mmw
      write(mwrite,3000) hparf(1:20),hfile(1:10)
 3000 format('DEF JOU',/,
     +       'RE PA',/,
     +       A20,/,
     +       'RE F',/,
     +       '/OPEN ',A10,/,
     +       '/REAL 75 (10X,6F10.4,5X)',/,
     +       '/READ /A- /B- /C- /AL /BE /GA',/,
     +       '/INTEGER 80 (A2,2X,I4,24X,12I4)',/,
     +       '/REAL 80 (8X,3F8.5,48X)',/,
     +       '/READ /EL /SER /X /Y /Z /CON 12',/,
     +       '/FIN',/,/,
     +       'DEF NOJOU',/,
     +       'SET PACK')
      nltt=nlt
      if(nltt.lt.1) nltt=1
*  Programmers note: It may be necessary to fill array tu(,) here
      do 215 l=1,nltt
	do 220 i=1,nsym
	  if(i.eq.1.and.l.eq.1) go to 220
	  write(mwrite,3005) ((s(j,k,i),k=1,3),(t(j,i)+tu(j,l)),j=1,3)
 3005     format('/M ',3(3F4.0,F8.5))
  220     continue
  215   continue
      if(gcent.eq.'C'.or.gcent.eq.'c') then
	write(mwrite,3007)
 3007   format('/CEN',/,
     +         '/FI',/)
      else
	write(mwrite,3006)
 3006   format('/NOC',/,
     +         '/FI',/)
	endif
      write(nout,4000) hcile,hfile
 4000 format(/,' The CHEMGRAF files ',A10,' and ',A10,' have been',
     +   ' written',//)
      write(nout,4010)
 4010 format(/,' Hit <RETURN> to continue: ')
      read(nin,2001,err=900) gdum
  900 return
      end
C
C
c**********************************************************************
c
c
      SUBROUTINE ATOMGR
c     -----------------
c
c   calls calcy, ifile, ierr    
c
c   prepares an '.str' input file for the modelling program 'Atoms'
c   
c     adapted from Atoms 'datfile.doc' by C.Brodie  
c     McMaster University, August 1992  
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c   system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      common /range/  nj,iox(20),top(20,20),bot(20,20),ddmax,
     1                ddmin,rmax,rmin,vmax,vmin,anglx
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30),xyz(100,3)
      include 'comatom.for'
      dimension ifcr(2),itlim(3,2),kpr(6),gsar(10,76),rphi(3)
      dimension icocon(4),iohkl(100,3)
      dimension fod(100),cp(6),sill(3)
      dimension icolat(5),gtit(1,12),hdim(3),xmin(3),xmax(3)
      data hblank(1:10)/'          '/
c
c
c
c
c  prepare file
c
      hfile(1:10)=hblank(1:10)
  70  write (nwrite,7990)
7990  format(/'Subroutine ATOMS has been called'/)
      if (machine.ne.5) then
	write (nout,8000) 
8000    format(/'An ATOMS file cannot be written on this machine'/)
	return
	endif
      write (nout,8010)
8010  format(/'An ATOMS file will be written with the extension .str',
     1      /'Enter the rootname for this file (max. of 6 characters:')
      read (nin,8020,err=70) gfile
8020  format(A6)
      if (gfile.eq.' '.or.gfile(1:1).eq.' ') call inerr(*70)
      do 80 i=1,6
	if(gfile(i:i).eq.' ') go to 90
	hfile(i:i)=gfile(i:i)
	j=i
  80    continue
  90  continue  
      hfile(j+1:j+4)='.str'
      nunit=16
      gfort=' '
      gstat='unknown'
      nnw=ifile(hfile,gfort,gstat,nunit)
      if(nnw.le.0) return
c
c  Prompt for block dimensions and call calcy to fill this block
c
      write(nout,9850) cp
9850  format(/'The unit cell is  a= ',F10.5,'  b= ',F10.5,'  c= ',F10.5,
     1        /,18X,'alpha= ',F10.5,'  beta= ',F10.5,'  gamma= ',F10.5)
      write(nout,9860) hmsym
9860  format(/'The Hermann-Maugin space group is ',A70)
      write(nout,9870)
9870  format(/'Give the maximum and minimum dimensions of the volume',
     1        /'that are to be plotted in fractions of the unit cell.'/)  
      hdim(1)='x(min) and x(max) '
      hdim(2)='y(min) and y(max) '
      hdim(3)='z(min) and z(max) '
      do 160 i=1,3
	write (nout,9880) hdim(i)
9880    format(A18,'= ')    
	read (nin,*) xmin(i),xmax(i)
 160    continue
      call calcy(xmax,xmin)
c
c  write data file (hfile) 
c
      JDUM=0
      XDUM=0.0
      deg=180.0/3.1415927
      WRITE(nnw,9801,ERR=900) hname
9801  FORMAT(A60)
      isl=0
      iper=1
      jcel=4
      kfout=1
      icnt=0
      idispl=0
      inorth=0
      ictr=0
      iwson=0
      iwdon=0
      WRITE(nnw,9802,ERR=900) JCEL,KFOUT,ICNT,IDISPL,INORTH, 
     1 ISL,IPER,ICTR,IWSON,IWDON
9802  FORMAT(20I3)
      isck=0
      isext=0
      ibck=0
      iplte=0
      isyst=1
      indd=0
      inrot=0
      ifreez=0
      iorf=0
      ifopt=1
      WRITE(nnw,9802,ERR=900) ISCK,ISEXT,IBCK,IPLTE,ISYST,
     1 INDD,INROT,IFREEZ,IORF,IFOPT
      iaxes=0
      irh=0
      istsc=1
      iver=3
      ifade=1
      iback=1
      iptsp=0
      WRITE(nnw,9802,ERR=900) IAXES,IRH,ISTSC,IVER,IFADE,
     1 IBACK,IPTSP,JDUM,JDUM,JDUM
      icolsb=1
      icolsf=14
      icolpb=2
      icolpf=1
      icolxs=15
      icolxp=3
      icoldb=15
      icoldf=15
      WRITE(nnw,9802,ERR=900) ICOLSB,ICOLSF,ICOLPB,ICOLPF,
     1 ICOLXS,ICOLXP,ICOLDB,ICOLDF
      ifcr(2)=2
      isep=0
      nu=0
      icart=0
      iprbr=0
      nssa=2
      icolxd=15
      WRITE(nnw,9802,ERR=900) IFCR(2),ISEP,NU,ICART,IPRBR,
     1 NSSA,ICOLXD
      nss=1
      ncon=1       
      nform=6
      npol=0
      nboco=0
      WRITE(nnw,9820,ERR=900) NSS,NCON,nform,-1,NPOL,
     1 NBOCO,ny
9820  FORMAT(15I5)
      iuv=0
      do 92 i=1,3
	do 92 j=1,2
	  itlim(i,j)=0
92      continue
      WRITE (nnw,9802,ERR=900) IUV,((ITLIM(I,J),J=1,2),I=1,3)
      do 55 j=1,6
	kpr(j)=0
55      continue
      WRITE (nnw,9802,ERR=900) (KPR(J),J=1,6)
      dp=50.00000
      blinc=.00000
      fadfac=.00000
      blshade=.00000
      scdfix=1.00000
      scpfix=1.00000
      axsc=.50000
      sdspac=.00000
      WRITE(nnw,9803,ERR=900) DP,BLINC,FADFAC,BLSHADE,
     1 SCDFIX,SCPFIX,AXSC,SDSPAC
9803  FORMAT(8F10.5)
      frd=.45000
      dashlen=.20000
      scsfix=1.00000
      xlwidc=.01000
      xlwida=.01000
      xlwidb=.01000
      xlwidp=.01000
      WRITE(nnw,9803,ERR=900) FRD,DASHLEN,XDUM,SCSFIX,
     1 XLWIDC,XLWIDA,XLWIDB,XLWIDP
      do 57 i=1,3
	rphi(i)=.00000
57      continue
      WRITE(nnw,9803,ERR=900) RPHI
      cp(1)=a
      cp(2)=b
      cp(3)=c
      cp(4)=alpha*deg
      cp(5)=beta*deg
      cp(6)=agamma*deg
      WRITE(nnw,9803,ERR=900) CP
      WRITE(nnw,9804,ERR=900)
9804  FORMAT('X,Y,Z;                                   ')
      kclas=1
      icocon(1)=1
      icocon(2)=1
      icocon(3)=0
      icocon(4)=2
      WRITE(nnw,9802,ERR=900) KCLAS,(ICOCON(J),J=1,4)
      WRITE(nnw,9810,ERR=900)
9810  FORMAT(12X,'P P1        ')
      fod(1)=a
      fod(2)=a
      fod(3)=b
      fod(4)=b
      fod(5)=c
      fod(6)=c
      DO 150 I=1,nform
      do 61 j=1,3
	iohkl(i,j)=0
61      continue
      iohkl(1,1)=-1
      iohkl(2,1)=1
      iohkl(3,2)=-1
      iohkl(4,2)=1
      iohkl(5,3)=-1
      iohkl(6,3)=1
      WRITE(nnw,9821,ERR=900) I,(IOHKL(I,J),J=1,3),FOD(I)
9821  FORMAT(4I7,2X,F10.5)
150   continue  
      if (ny.lt.my) gelem(iynum(ny+1))=gelem(iynum(ny))
      icolor=1
      DO 170 I=1,ny
	ityp=mod(iynum(i),14)+1
	size=.200
	icolat(1)=0
	icolat(3)=0
	icolat(4)=11
	if(gelem(iynum(i)).ne.gelem(iynum(i+1))) icolor=icolor+1
	icolat(2)=icolor
	icolat(5)=icolor
	WRITE(nnw,9822,ERR=900) I,gelem(iynum(i)),gdent(iynum(i)),ITYP,
     1           (yx(j,i),J=1,3),SIZE,(ICOLAT(J),J=1,5)
9822    FORMAT(I5,2X,A2,A4,I3,2X,3F10.5,F7.3,2X,8I3)
170     continue  
      irima=1
      irimb=1
      irimp=1
      ishada=0
      ishadb=0
      ishadp=0
      istpair=0
      ncza=8
      nczb=4
      ishin=0
      WRITE(nnw,9802,ERR=900)IRIMA,IRIMB,IRIMP,ISHADA,ISHADB,
     1 ISHADP,ISTPAIR,NCZA,NCZB,ISHIN
      icoar=0
      ievbk=8
      ipal=0
      WRITE(nnw,9802,ERR=900) ICOAR,IEVBK,IPAL
      strot= 3.00000
      blang=110.00000
      sill(1)=.81653
      sill(2)=.40821
      sill(3)=.40821
      WRITE(nnw,9803,ERR=900) STROT,BLANG,SILL
      write(nout,9890) hfile 
9890  format(/'An ATOMS file has been written to the file ',A20)
      GO TO 999

c** Errors
 900  WRITE(*,9812)   
9812  FORMAT(/3X,'** ERROR ** Atomgr  9812 error in writing file')
 999  RETURN
      END
c
c
c**********************************************************************
C
C
      SUBROUTINE PAGE(GCOMD)
C
C  *****SUBROUTINE****                PAGE
C                                     ----
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C  MODIFIED JUNE 1985 BY DAVID KNIGHT / JULY 1986 BY NINA HEINIG / 
C  MAY 1989 BY EILEEN MURGATROYD / SEPTEMBER 1990 BY SEAN HEALY
C
C  CALLS CHEMLB: FORMLA(in FORMUL), f000, PICWGT
C  CALLS CALCY, SYMXYZ
C  USES HXWORD, IFNUMB, INTEGER
C
C  PRINTS THE TITLE PAGE WITH CHECKED INPUT DATA
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DIMENSION ICOUNT(100),GNAM(100),ANUM(100), UU(6), DUU(6)
      DIMENSION XOX(100)
      DIMENSION HUU(6)
      INTEGER NUM(100)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
c Programmer's Note:  /bost/ has been changed to /bostt/ for
c                     Unix system
      COMMON/BOSTT/NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON/GBOST/GBOST(2,30)
      COMMON/CELL/A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON/DENS/ZFORM,DENS,DDENS,CDENS,FWGHT,WAVELN,ABSORB,
     +            FZERO,TMPRTR
      COMMON/DUMMY/NXH,XNUMH(10),XCHARG(10)
      COMMON/GDUMMY/GDH(10)
      include 'comatom.for'
      COMMON/ECELL/DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
      COMMON/RCELL/AS,BS,CS,RALPHA,RBETA,RGAMMA,RVOL,RM(3,3)
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL
      COMMON/SYMA/MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30),MULTSP(30)
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),HFORM(2),
     +              HCOMM(7),GRFACT
      COMMON/FORM/  NFORM(3),MFORM,FORM(24,3)
      COMMON/HTEXT/GCRYST(2),GTWIN(5),HTXT(10,5),HDATA(4,19)
      COMMON/GFORM/ GFORM(24,3)
      DIMENSION XMAX(3),XMIN(3)
      DOUBLE PRECISION PI/ 3.141592653 /,CONVRT
C
      LATOM = 0
      LLATOM= 0
C
C   PRINT THE TITLE
C   ---------------
      WRITE(NPAGE,1000) HNAME
 1000 FORMAT('1',//,' ',A79,/)
C
C   PRINT JOURNAL TITLE AND AUTHORS NAMES
C
      WRITE(NPAGE,1010) (HTITLE(J),J=1,2),(HAUTH(J),J=1,JAUTH)
 1010 FORMAT(2(' ',A80,/),/,10(' ',A80,/))
C
C   PRINT REFERENCE 
C   ---------------
      DO 6 I=1,2
	IALL = 0
	IF(HCODN(I)(3:3).NE.' ') THEN
	  II = 1
	  CALL SCAN(G,N,X,EX,II,HCODN(I),IALL)
	  IF(II.GE.99) GOTO 5
	  CALL SCAN(G,NREF1,X,EX,II,HCODN(I),IALL)
	  IF(II.GE.99) GOTO 5
	  CALL SCAN(G,NREF2,X,EX,II,HCODN(I),IALL)
	  IF(II.GE.99) GOTO 5
	  CALL SCAN(G,NREF3,X,EX,II,HCODN(I),IALL)
    5     WRITE(NPAGE,1030) HCODN(I)(1:2),HCODN(I)(3:8),NREF1,NREF2,
     +      NREF3
	  ENDIF
    6   CONTINUE
 1030 FORMAT(' Reference  -  ','19',A2,1X,A6,I5,I5,'-',I4)
   10 WRITE(NPAGE,1025)
 1025 FORMAT('0')
C
C   PRINT COMMENTS
C   --------------
      JJ=MIN0(KCOMM,7)
      IF(KCOMM.NE.0) WRITE(NPAGE,1040) (HCOMM(I),I=1,JJ)
 1040 FORMAT(/,7(' ',A80,/))
C
C   PRINT R FACTOR
C   --------------
      WRITE(NPAGE,1050) GRFACT
 1050 FORMAT(' R = ',A8)
C
C   CALCULATE FORMULA
C   -----------------
      DO 20 I=1,3
	XMAX(I)=0.999995
   20   XMIN(I)=-0.000005
      IF(GLAT.EQ.'A'.OR.GLAT.EQ.'F') XMAX(2)=XMAX(2)-0.5
      IF(GLAT.EQ.'B'.OR.GLAT.EQ.'F') XMAX(3)=XMAX(3)-0.5
      IF(GLAT.EQ.'C') XMAX(1)=XMAX(1)-0.5
      CALL CALCY(XMAX,XMIN)
      DO 30 I=1,100
	ANUM(I)=0.0
   30   ICOUNT(I)=0
      DO 40 I=1,NY
	JJ=IYNUM(I)
   40   ICOUNT(JJ)=ICOUNT(JJ)+1
      DO 50 I=1,NX
	IF(ICOUNT(I).EQ.0) GO TO 50
	IF(GLAT.EQ.'A'.OR.GLAT.EQ.'B'.OR.GLAT.EQ.'C') ICOUNT(I)=
     +    2*ICOUNT(I)
	IF(GLAT.EQ.'F') ICOUNT(I)=4*ICOUNT(I)
	IF(ICOUNT(I).NE.NPOS(I).AND.NPOS(I).NE.0) THEN
	  WRITE(NPAGE,1060) GELEM(I),GDENT(I),NPOS(I),ICOUNT(I)
 1060     FORMAT(' ***WARNING*** Atom Denoted :      ',2A4,' Has an ',
     +      'Input Multiplicity of ',I5,' But a Calculated ',
     +      'Multiplicity of     ',I5)
	  ENDIF
	IF(IALTNO(I).EQ.0) GOTO 50
	IF(MULTSP(IALTNO(I)).NE.ICOUNT(I)) THEN
	  WRITE(NPAGE,1070) GELEM(I),GDENT(I),IALTNO(I),
     +      MULTSP(IALTNO(I)),ICOUNT(I)
 1070     FORMAT(' ***WARNING*** The Multiplicity of Atom Denoted :',
     +      '      ',2A4,' of Special Position ',I3,' Is ',I5,' But',
     +      ' the Number of Atoms Calculated ',/,' in the Unit Cell',
     +      ' Is',I5)
	  ENDIF
   50   CONTINUE
      LATOM=1
      GNAM(1)=GELEM(1)
      XOX(1)=OXID(1)
      DO 70 I=2,NX
	DO 60 J=1,LATOM
	  IF(GELEM(I).EQ.GNAM(J).AND.ABS(OXID(I)-XOX(J)).LT.0.001)
     +       GOTO 70
   60     CONTINUE
	LATOM=LATOM+1
	GNAM(LATOM)=GELEM(I)
	XOX(LATOM)=OXID(I)
   70   CONTINUE
      DO 80 I=1,LATOM
	DO 80 J=1,NX
	  IF(GNAM(I).EQ.GELEM(J).AND.ABS(OXID(J)-XOX(I)).LT.0.001)
     +      ANUM(I)=ANUM(I)+FLOAT(ICOUNT(J))
     +      *OCCU(J)
   80     CONTINUE
C
C   ADD H ATOMS FOR WHICH COORDINATES HAVE NOT BEEN GIVEN
C
      LAT1=LATOM+1
      LATOM=LATOM+NXH
      J=0
      DO 90 I=LAT1,LATOM
	J=J+1
	GNAM(I)=GDH(J)
	ANUM(I)=XNUMH(J)
	XOX(I)=1.0
   90   CONTINUE
      INTEG = 1
      DO 100 I=1,LATOM
	NUM(I)=INTEGER(ANUM(I),0)
	AA=FLOAT(NUM(I))
	DIFF=ABS(ANUM(I)-AA)
	IF(DIFF.GT.0.01) INTEG= 0
  100   CONTINUE
      LLATOM=MIN0(13,LATOM)
      IF(INTEG.EQ.0) WRITE(NPAGE,1080) (GNAM(I),ANUM(I),I=1,LLATOM)
 1080 FORMAT(/' Contents of Unit Cell:    ',6(A2,F5.2,1X))
      IF(INTEG.EQ.1) WRITE(NPAGE,1090) (GNAM(I),NUM(I),I=1,LLATOM)
 1090 FORMAT(/' Contents of Unit Cell:    ',6(A2,I5,1X))
C
      Z = ZFORM
      KZ=INTEGER(Z,0)
      DO 110 I=1,LATOM
  110   ANUM(I)=ANUM(I)/FLOAT(KZ)
      CALL SORFOR(GNUM,ANUJ,XOX,LATOM)
      CALL FORMLA(HFRML, GNAM, ANUM, LATOM)
      WRITE(NPAGE,1100) HFRML
 1100 FORMAT(/,' Contents of Formula Unit: ', A80)
      IF ((HDATA(1,5)(1:2)).NE.'  ') THEN
	WRITE (NPAGE,1110) (HDATA(I,5),I=1,4)
      ELSEIF(HFORM(1)(1:1).NE.' '.OR.HFORM(1)(2:2).NE.' ') THEN
	WRITE(NPAGE,1110) (HFORM(I),I=1,2)
 1110   FORMAT(/,' Formula:                  ',(T30,A80))
	ENDIF
c
c    calculate the total charge
c
      tcharg=0.0
      do 111 i=1,nx
	tcharg=tcharg+(oxid(i)*npos(i)*occu(i)/z)
  111 continue
      if(nxh.ne.0)then
	do 112 i=1,nxh
	tcharg=tcharg+(xcharg(i)/z)
  112   continue
      endif
      write(npage,1111)tcharg
 1111 format(/,' Total charge:              ',f5.3/)
c
	WRITE(NPAGE,1120) HMSYM,HALL
 1120 FORMAT(' ','Space Group:   Hermann-Mauguin= ',A24,//,
     +  16X,'Hall           = ',A24/)
C
C   CALCULATE AND PRINT FORMULA WEIGHT
C   ----------------------------------
      FWGHT=0.0
      IF(ZFORM.NE.0.) GO TO 120
      WRITE(NPAGE,1130)
 1130 FORMAT(//,'Z Less Than 1, Formula Weight Not Calculated')
      GO TO 140
  120 CONTINUE
C
C   CALCULATE THE FORMULA WEIGHT
C   ----------------------------
      DO 130 I=1,LATOM
	FWGHT=FWGHT+ANUM(I)*ELMWGT(GNAM(I))
  130   CONTINUE
      WRITE(NPAGE,1140) FWGHT
 1140 FORMAT(//,' Formula Weight = ',F8.2)
  140 CONTINUE
C
C   CALCULATE FZERO
C   ---------------
      fnul=f000(dummy)
      IF(NDEBUG.NE. 0) WRITE(NDEBUG,1143) FNUL,FZERO,ZFORM
 1143 FORMAT(' ***DEBUG**PAGE*1143: F(000)= ',F9.2,'FZERO=',F9.2,
     *      'ZFORM=',F7.2)
      WRITE(NPAGE,1145) FNUL
 1145 FORMAT(/,' F(000) =',6X,F9.2)
      IF(abs(fzero).gt.0.1.and.abs(fnul-fzero).gt.0.9) 
     +   WRITE(NPAGE,1147) fnul,fzero
 1147 FORMAT(' *ERROR*PAGE*1147* DIFF. BETWEEN CALCULATED ',F7.2,
     +  ' & STORED ',F7.2,' F(000)')
C
C   PRINT UNIT CELL (CONVERTING RADIANS TO DEGREES)
C   -----------------------------------------------
      CONVRT = 180.00 / PI
      DAL= CONVRT*DALPHA
      DBE= CONVRT*DBETA
      DGA= CONVRT*DGAMMA
      AL=  CONVRT*ALPHA
      BE=  CONVRT*BETA
      AGA= CONVRT*AGAMMA
      WRITE(NPAGE,1150) HXWORD(A,DA),HXWORD(B,DB),HXWORD(C,DC),
     +  HXWORD(AL,DAL),HXWORD(BE,DBE),HXWORD(AGA,DGA)
 1150 FORMAT(//' a = ',A14,'  b = ',A14,'  c = ',A14,' (Angstroms)',//,
     +' Alpha = ',A14,'  Beta = ',A14,'  Gamma = ',A14,' (Degrees)')
      EDENS=CDENS
      CDENS=Z*FWGHT*1.660/VOL
      WRITE(NPAGE,1160) VOL,HXWORD(DENS,DDENS),KZ,CDENS,EDENS
 1160 FORMAT(/,' U = ',F8.2,'A**3',5X,'Dm = ',A14,5X,'Z = ',
     +  I5,5X,'Dx = ',F7.2,'(',F7.2,' given',')',/)
C
C   PRINT ATOMIC COORDINATES
C   ------------------------
      WRITE(NPAGE,1170)
 1170 FORMAT(/,' Atomic Positions',//,10X,'Oxid   Alt  Mult',7X,'x',
     +  12X,'y',12X,'z',8X,'Occupation')
      DO 160 I=1,NX
	WRITE(NPAGE,1180) GELEM(I),GDENT(I),OXID(I),IALTNO(I),NPOS(I),
     +    GWYCK(I),HXWORD(XX(1,I),DXX(1,I)),HXWORD(XX(2,I),DXX(2,I)),
     +    HXWORD(XX(3,I),DXX(3,I)),HXWORD(OCCU(I),SOCCU(I))
 1180   FORMAT('0',A2,A6,F5.2,2X,I3,2X,I3,A1,3X,3A13,3X,A10)
  160   CONTINUE
c
c  PRINT TEMPERATURE FACTORS
c  -------------------------
      WRITE(NPAGE,1175)
 1175 FORMAT(/, ' Temperature Factors',//, 10X, 3X, 'U(1,1)',6X,
     +    'U(2,2)',6X,'U(3,3)',6X,'U(1,2)',6X,'U(1,3)',6X,'U(2,3)') 
      DO 165 I=1,NX
	IF((ABS(U(2,2,I))+ABS(U(3,3,I))+ABS(U(1,2,I))+ABS(U(1,3,I))+
     +    ABS(U(2,3,I))) .LT. 0.0001) GOTO 155
	DO 167  J=1,3
	  UU(J) = U(J,J,I)
	  DUU(J) = DU(J,J,I)
  167     CONTINUE
	UU(4)  = U(1,2,I)
	DUU(4) = DU(1,2,I)
	UU(5)  = U(1,3,I)
	DUU(5) = DU(1,3,I)
	UU(6)  = U(2,3,I)
	DUU(6) = DU(2,3,I)
	DO 169 J=1,6
	  HUU(J) = HXWORD(UU(J),DUU(J))
  169     CONTINUE
	WRITE(NPAGE,1185) GELEM(I),GDENT(I),(HUU(J), J=1,6) 
	GOTO 165
  155   WRITE(NPAGE,1185) GELEM(I),GDENT(I),HXWORD(U(1,1,I),DU(1,1,I))
 1185   FORMAT('0',A2,A6,3X,6(A14))
  165   CONTINUE
C
C   PRINT SYMMETRY MATRIX
C   ---------------------
      WRITE(NPAGE,1190)
 1190 FORMAT('0','    Symmetry Matrix',/)
      NOUTSAVE = NOUT
      NOUT = NPAGE
      CALL SYMXYZ(NSYM,S,T)
      NOUT = NOUTSAVE
C
C   PRINT BOND VALENCE PARAMETERS
C
      IF(NBOST.EQ.0) GOTO 170
      WRITE(NPAGE,1200)((GBOST(I,J),I=1,2),IFBOST(J),(PBOST(I,J),I=1,3),
     +  J=1,NBOST)
 1200 FORMAT(///,'0',' Bond Strength Parameters in the Formula    ',
     +  ' S = C(R/A)**(-B)   ',/,5X, 'If Key = 2 Then the Expression ',
     +  'Is S = EXP((R-A)/B)',//,13X,'Key',9X,'A',9X,'B',9X,'C',/,
     +   100(/,3X,A2,3X,A2,4X,I1,6X,F7.3,2F10.3))
      GOTO 180
  170 WRITE(NPAGE,1210)
 1210 FORMAT(////'  No Bond Strength Parameters Given')
  180 RETURN
      END
C
C
C
C
      subroutine paper(nprint)
c
c    written 85.6.24 by I.D.Brown / 86.7.11 by N.Heinig
c    Prints a short structural paper using data stored in common blocks
c
c    Uses integer, length, nline
c
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /dens/   zform,dens,ddens,cdens,fwgt,waveln,absorb,
     1                  fzero,tmprtr
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                  hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /href/  hinfo(2,2), gfilen(5)
      common /gcond/ gcrad,girad,gdiff, gabs, grefn, gstat
      common /cond/  condit(6,8), nstd,mstd,cstd(6,4)
      common /htext/ gcryst(2), gtwin(5),htxt(10,5), hdata(4,19)
      common /form/ nform(3), mform, form(24,3)
      common /gform/ gform(24,3)
c
      data gline,glline,gllline,gpage/'('' '')','(''0'')','(''-'')',
     1       '(''1'')'/
c
c  Titles and Authors
c
      write(nprint,1000) htitle
 1000 format('1',////,5X,A75,//,5X,A75,///)
      isp=10
      do 20 i=1,jauth
	lauth=length(hauth(i))
	if(i.ne.jauth) write(hfmt,1020) isp,lauth
	if(i.eq.jauth) write(hfmt,1025) isp,lauth
 1020   format('(1H+,',I3,'X,A',I3,',1H,)')
 1025   format('(1H+,',I3,'X,A',I3,')')
	write(nprint,hfmt) hauth(i)
	isp=isp+lauth+3
	if(isp.gt.80) then
	  isp=10
	  write(nprint,glline)
	  endif
   20   continue
c
c  Address of Institute
c
      if(machine.eq.1) write(nprint,1030)
      if(machine.eq.2) write(nprint,1031)
      if(machine.eq.4) write(nprint,1030)
 1030 format('0',10X,'Institute for Materials Research, McMaster ',
     + 'University,',/,/,11X,'Hamilton, Ontario, Canada L8S 4M1')
 1031 format('0',10X,'Institut de Cristallographie, Universite de ',
     + 'Lausanne,',//,11X,'BSP - Dorigny, CH-1015 Lausanne')
c
c  Abstract
c
      ipo = 10
      write(nprint,1100)
 1100 format('0',/,/,/,11X,'Abstract',/,11X, 8('~'))
c
c  Common chemical name or synonym
c
      assign 1110 to ifmat 
      lname = length(hdata(1,2)(10:80))
      if(lname .ne. 0) then
	write(hfmt,1110) ipo, lname
 1110   format('(1H0,//,1X,',I3,'X, A',I3,' )')
	write(nprint, hfmt) hdata(1,2)(10:80)
	ipo = ipo + lname
	do 26  i=2,4
	  lname = length(hdata(i,2)(10:80))
	  if(lname .ne. 0) then
	    ipo = nline(ipo, lname, nprint)
	    write(hfmt, 1112) ipo,lname
 1112       format('(1X,',I3,'X, A',I3,')')
	    write(nprint,hfmt) hdata(i,2)(10:80)
	    ipo = ipo + lname
	    else
	    goto 27
	    endif  
   26     continue
   27   write(hfmt, 1114) ipo
 1114   format('(1H+,' , I3,'X, 2H; )')
	write(nprint, hfmt)
	ipo = ipo + 2
	assign 1112 to ifmat 
	endif
c
c  IUPAC name
c
      lname = length(hdata(1,1)(10:80))
      if(lname .ne. 0) then
	ipo = nline(ipo, lname,nprint)
	write(hfmt,ifmat) ipo, lname
	write(nprint, hfmt) hdata(1,2)(10:80)
	ipo = ipo + lname
	do 29  i=2,4
	  lname = length(hdata(i,1)(10:80))
	  if(lname .ne. 0) then
	    ipo = nline(ipo, lname, nprint)
	    write(hfmt, 1112) ipo,lname
	    write(nprint,hfmt) hdata(i,1)(10:80)
	    ipo = ipo + lname
	    else
	    goto 30
	    endif  
   29     continue
   30   write(hfmt, 1114) ipo
	write(nprint, hfmt)
	ipo = ipo + 2
	assign 1112 to ifmat 
	endif
c
c  Formula
c
      lform=length(hdata(1,5)(1:71))
      if(lform .ne. 0) then
	ipo = nline(ipo, lform,nprint)
	write(hfmt,ifmat) ipo,lform
	write(nprint,hfmt) hdata(1,5)(9:80)
	ipo=ipo+lform
	do 70 i=2,4
	  lform = length(hdata(i,5)(9:80))
	  if(lform .ne. 0) then
	    ipo = nline(ipo,lform,nprint)
	    write(hfmt,1112) ipo,lform
	    write(nprint, hfmt) hdata(i,5)(9:80)
	    ipo = ipo + lform
	    else
	    goto 80
	    endif
   70     continue
   80   write(hfmt,1114) ipo
	write(nprint,hfmt)
	ipo = ipo + 2 
      endif
c
c  Chemical Abstracts refcode
c
      do 23  i=1,2
	lcas = length( hinfo(i,1))
	if( lcas .ne. 0) then
	  ipo = nline(ipo,lcas+22,nprint)
	  write(hfmt,1027) ipo,lcas
 1027     format('(1H+,',I3,'X,21HCAS registry number: ,A',I3,',1H,)')
	  write(nprint, hfmt) hinfo(i,1)
	  ipo = ipo + lcas + 22
	  endif
   23   continue         
c
c  Other Parameters; a,b,c, hmsym ,hall, alpha,beta,gamma,vol,density
c
      con=57.2957795
      ha=hxword(a,da)
      la=length(ha)
      hb=hxword(b,db)
      lb=length(hb)
      hc=hxword(c,dc)
      lc=length(hc)
      hal=hxword(con*alpha,con*dalpha)
      lal=length(hal)
      hbe=hxword(con*beta,con*dbeta)
      lbe=length(hbe)
      hga=hxword(con*agamma,con*dgamma)
      lga=length(hga)
      iz=integer(zform,0)
      hd=hxword(dens,ddens)
      ld=length(hd)
c
c  formula weight
c
      if(fwgt .gt. 0.0001) then
	ipo=nline(ipo,16,nprint)
	write(hfmt,1120) ipo
 1120   format('(1H+,',I3,'X,6HF.W. =,F8.2,2H, )')
	write(nprint,hfmt) fwgt
	ipo=ipo+16
	endif
c
c  Hermann Mauguin and Hall space group symbol
c
      lhmsym=length(hmsym)
      if(lhmsym .ne. 0) then
	ipo=nline(ipo,2+lhmsym,nprint)
	write(hfmt,1130) ipo,lhmsym
 1130   format('(1H+,',I3,'X,A',I3,',2H, )')
	write(nprint,hfmt) hmsym
	ipo=ipo+2+lhmsym
	endif
      if(hall(1:2).ne.'  ') then
	lhall=length(hall)
	ipo=nline(ipo,18+lhall,nprint)
	write(hfmt,1140) ipo,lhall
 1140   format('(1H+,',I3,'X,15H(Hall symbol = ,A',I3,',3H), )')
	write(nprint,hfmt) hall
	ipo=ipo+18+lhall
	endif
c
c  a, b, c, alpha, beta, gamma
c
      ipo=nline(ipo,6+la,nprint)
      write(hfmt,1150) ipo,la
 1150 format('(1H+,',I3,'X,4Ha = ,A',I3,',2H, )')
      write(nprint,hfmt) ha
      ipo=ipo+6+la
      ipo=nline(ipo,6+lb,nprint)
      write(hfmt,1160) ipo,lb
 1160 format('(1H+,',I3,'X,4Hb = ,A',I3,',2H, )')
      write(nprint,hfmt) hb
      ipo=ipo+6+lb
      ipo=nline(ipo,6+lc,nprint)
      write(hfmt,1170) ipo,lc
 1170 format('(1H+,',I3,'X,4Hc = ,A',I3,',2H, )')
      write(nprint,hfmt) hc
      ipo=ipo+6+lc
      if(con*alpha.ne.90.000.and.con*dalpha.ge.0.0001) then
	ipo=nline(ipo,11+lal,nprint)
	write(hfmt,1180) ipo,lal
 1180   format('(1H+,',I3,'X,9H$alpha = ,A',I3,',2H, )')
	write(nprint,hfmt) hal
	ipo=ipo+11+lal
	endif
      if(con*beta.ne.90.000.and.con*dbeta.ge.0.0001) then
	ipo=nline(ipo,10+lbe,nprint)
	write(hfmt,1190) ipo,lbe
 1190   format('(1H+,',I3,'X,8H$beta = ,A',I3,',2H, )')
	write(nprint,hfmt) hbe
	ipo=ipo+10+lbe
	endif
      if(con*agamma.ne.90.000.and.con*dgamma.ge.0.0001) then
	ipo=nline(ipo,11+lga,nprint)
	write(hfmt,1200) ipo,lga
 1200   format('(1H+,',I3,'X,9H$gamma = ,A',I3,',2H, )')
	write(nprint,hfmt) hga
	ipo=ipo+11+lga
	endif
c
c  volume and z factor
c
      ipo=nline(ipo,25,nprint)
      write(hfmt,1210) ipo
 1210 format('(1H+,',I3,'X,8Hvolume =,F8.2,9H A$(3$), )')
      write(nprint,hfmt) vol
      ipo=ipo+25
      ipo=nline(ipo,8,nprint)
      write(hfmt,1220) ipo
 1220 format('(1H+,',I3,'X,3HZ =,I3,2H, )')
      write(nprint,hfmt) iz
      ipo=ipo+8
c
c  density
c
      if(dens.ge.0.01) then
	ipo=nline(ipo,7+ld,nprint)
	write(hfmt,1230) ipo,ld
 1230   format('(1H+,',I3,'X,5HDm = ,A',I3,',2H, )')
	write(nprint,hfmt) hd
	ipo=ipo+7+ld
	endif
      if(cdens .gt. 0.001) then
	ipo=nline(ipo,12,nprint)
	write(hfmt,1240) ipo
 1240   format('(1H+,',I3,'X,4HDx =,F6.2,2H, )')
	write(nprint,hfmt) cdens
	ipo=ipo+12
	endif
c
c  radiation type and wavelength
c
      if(girad .ne. ' ') then
	call upcase(girad)
	ipo=nline(ipo,39,nprint)
	if(girad .eq. 'X') then
	  hcomt = 'MoK$alpha radiation, $lambda ='
	elseif(girad .eq. 'N') then
	  hcomt = ' neutron radiation,  $lambda ='
	elseif(girad .eq. 'E') then
	  hcomt = 'electron radiation,  $lambda ='
	elseif(girad .eq. 'S') then
	  hcomt = 'synchrotron radiation,$lambda='
	else
	  hcomt = 'unidentified radiation,       '
	  endif
	write(hfmt,1250) ipo, hcomt
 1250   format('(1H+,',I3,'X, 30H',A30,',F7.4,',
     +    '2H, )')
	write(nprint,hfmt) condit(1,2)
	ipo=ipo+39
	endif
c
c  etc
c
      ipo=nline(ipo,42,nprint)
      write(hfmt,1260) ipo
 1260 format('(1H+,',I3,'X,24Habsorption coefficient =,F7.2,',
     +  '11H mm$(-1$), )')
      write(nprint,hfmt) condit(1,4)
      ipo=ipo+42
      ffzero=f000(dummy)
      if (abs(ffzero).gt.0.01) then
	fzero = ffzero
	ipo=nline(ipo,17,nprint)
	write(hfmt,1270) ipo
 1270   format('(1H+,',I3,'X,8HF(000) =,F7.2,2H, )')
	write(nprint,hfmt) fzero
	ipo=ipo+17
	endif
      ipo=nline(ipo,13,nprint)
      if(condit(2,2).lt.0.001) condit(2,2)=279.
      write(hfmt,1280) ipo
 1280 format('(1H+,',I3,'X,3HT =,I4,4H K, )')
      write(nprint,hfmt) integer(condit(2,2),0)
      ipo=ipo+11
      ipo=nline(ipo,18,nprint)
      write(hfmt,1290) ipo
 1290 format('(1H+,',I3,'X,10HR factor =,F6.3,1H.)')
      write(nprint,hfmt) condit(1,7)
      ipo=ipo+18
      if(condit(2,7) .ne. 0) then
	ipo = nline(ipo, 13, nprint)
	write(hfmt,1295) ipo
 1295   format('(1H+,',I3,'X,5HwR = ,F6.3,1H.)')
	write(nprint, hfmt) condit(2,7)
	ipo = ipo + 13
	endif
      if(condit(1,5) .ne. 0) then
	ipo = nline( ipo, 43, nprint)
	write(hfmt, 1297) ipo
 1297   format('(1H+,',I3,'X,37HThe number of measured reflections is
     *        , F6.0 )')
	write(nprint,hfmt) condit(1,5)
	ipo = ipo + 43
	endif
c
c  Text of the abstract
c
      labs = length(htxt(1,1))
      if(labs .ne. 0) then
	ht = htxt(1,1)(10:80)
	write(nprint,1300) ht
 1300   format(/, 11X, A80,/)
	do 370  i=2,10
	  labs = length(htxt(i,1))
	  if(labs .ne. 0) then
	    ht = htxt(i,1)(10:80)
	    write(nprint, 1320) ht
 1320       format(11X,A80,/)
	    endif
  370     continue
	endif
c
c  Text of the Introduction
c
      lint = length(htxt(1,2))
      if(lint .ne. 0) then
	write(nprint, 1400)
	ht = htxt(1,2)(10:80)
 1400   format(///,11x,'Introduction',/,11X,12('~')) 
	write(nprint,1300) ht
	do 470 i=2,10
	  lint = length(htxt(i,2))
	  ht = htxt(i,2)(10:80)
	  if(lint .ne. 0) write(nprint,1320) ht
  470     continue
	endif
c
c  Text of the Experiment
c
      write(nprint, 1500)
 1500 format(///,11X,'Experiment',/,11X,10('~'),//)
      lexpt = length(htxt(1,3))
      if(lexpt .ne. 0) then
	write(nprint,1300) htxt(1,3)(10:80)
	do 500 i=2,10
	  lexpt = length(htxt(i,3))
	  if(lexpt .ne. 0) write(nprint,1320) htxt(i,3)(10:80)
  500     continue 
	write(nprint, 1510)
 1510   format(/,1X)  
	endif
c
      ipo = 10
      lsrce = length(hdata(1,3)(9:80))
      if(lsrce .ne. 0) then
	write(hfmt, 1520) ipo
 1520   format('(1H+, ',I3,'X, 16HThe source is:  ,A71)')
	write(nprint, hfmt) hdata(1,3)(9:80)
	ipo = ipo + 16 + lsrce
	do 505 i=2,4
	  lsrce = length(hdata(i,3)(9:80))
	  if(lsrce .ne. 0) then
	    ipo = nline(ipo, lsrce, nprint)
	    write(hfmt, 1530) ipo
 1530       format('(1H+,',I3,'X, A71)')
	    write(nprint, hfmt) hdata(i,3)(9:80)
	    ipo = ipo + lsrce
	    else
	    goto 510
	    endif
  505     continue
  510   write(hfmt, 1114) ipo
	write(nprint,hfmt)
	ipo = ipo + 2
	endif
      lsrce = length(hdata(1,11)(9:80))
      if(lsrce .ne. 0) then
	write(hfmt, 1540) ipo
 1540   format('(1H+, ',I3,'X,37HThe structure solution was solved by:   
     *           ,A71)')
	write(nprint, hfmt) hdata(1,11)(9:80)
	ipo = ipo + 37 + lsrce
	do 515 i=2,4
	  lsrce = length(hdata(i,11)(9:80))
	  if(lsrce .ne. 0) then
	    ipo = nline(ipo, lsrce, nprint)
	    write(hfmt, 1530) ipo
	    write(nprint, hfmt) hdata(i,11)(9:80)
	    ipo = ipo + lsrce
	    else
	    goto 520
	    endif
  515     continue
  520   write(hfmt, 1114) ipo
	write(nprint,hfmt)
	ipo = ipo + 2
	endif
c
c  Text of the Description
c
      ldes = length(htxt(1,4))
      if(ldes .ne. 0) then
	write(nprint, 1600)
 1600   format(///,11X, 'Description',/,11X,11('~'))
	hd = htxt(1,4)(10:80)
	write(nprint,1300) hd
	do 610  i=2,10
	  ldes = length(htxt(i,4))
	  if(ldes .ne. 0) then
	    hd = htxt(i,4)(10:80)
	    write(nprint, 1320) hd
	    endif
  610     continue
	ipo = 10
	ldiso = length(hdata(1,19)(9:80))
	if(ldiso .ne. 0) then
	  hcomt = 'kind of disorder :  '
	  ipo = nline(ipo,20, nprint) 
	  write(hfmt, 1620) ipo, hcomt
 1620     format('(1H+, ',I3,'X, 20H',A20,')' )
	  write(nprint, hfmt) 
	  ipo = ipo + 20
	  ipo = nline(ipo, ldiso,nprint)
	  write(hfmt, 1530) ipo
	  write(nprint, hfmt) hdata(1,19)(9:80)
	  ipo = ipo + ldiso
	  do 630 ij=2,4
	    labs = length(hdata(ij,ncount)(9:80))
	    if(labs .ne. 0) then
	      ipo = nline(ipo, labs, nprint)
	      write(hfmt, 1530) ipo
	      write(nprint, hfmt) hdata(ij,ncount)(9:80)
	      ipo = ipo + labs
	    else
	      goto 640
	      endif
  630       continue
  640     write(hfmt, 1114) ipo
	  write(nprint,hfmt)
	  ipo = ipo + 2
	  endif
	endif
      return
      end
C
C
C
C
      FUNCTION NLINE(IPO,IFD,NFILE)
C
C   WRITTEN 85.7.9 BY I. D. BROWN
C
C   STARTS A NEW LINE ON NFILE IF (IPO+IFD).GT.90
C   IPO IS POINTER
C   IFD IS LENGTH OF FIELD TO BE WRITTEN
C
      NLINE=IPO
      IF(NLINE+IFD.GT.90) THEN
	NLINE = 10
	WRITE(NFILE,1000)
 1000   FORMAT(/,1X)
	ENDIF
      RETURN
      END
C
C
C
C
C
      SUBROUTINE PUB(NPRINT)
C
C   *****SUBROUTINE*****         PUB
C                                ---
C
C   WRITTEN 81.6.9 BY I D BROWN / 86.7.11 BY N.HEINIG
C
C   USES HWORD, UEQ, TABLE1, TABLE3
C
C   PRINTS ATOMIC, THERMAL COORDINATES, STRUCTURE DATA AND BOND INFORMATION
C   IN A FORM SUITABLE FOR PUBLICATION
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      include 'comatom.for'
      include 'combond.for'
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),HFORM(2),
     +              HCOMM(7),GRFACT,GRAD
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                  multsp(30),sp(3,3,30),tp(3,30)
      common /href/  hinfo(2,2), gfilen(5)
      common /gcond/ gcrad,girad,gdiff, gabs, grefn, gstat
      common /cond/  condit(6,8), nstd,mstd,cstd(6,4)
      common /htext/ gcryst(2), gtwin(5), htxt(10,5), hdata(4,19)
      common /form/ nform(3), mform, form(24,3)
      common /gform/ gform(24,3)
c
      DIMENSION HU(3,3)
      CALL PAGE(GCOND)
C
C   PRINT STRUCTURE DETERMINATIONS
C
      CALL TABLE1(NPRINT) 
C
C   PRINT ATOMIC COORDINATES
C
      WRITE(NPRINT,1070) HNAME
 1070 FORMAT('1',////,10X,A80,//,25X,'Table 2: Atomic Coordinates'
     *      ,/,25X, 27('~'))
      WRITE(NPRINT,1071)
 1071 FORMAT(1X,//,25X,20X, '4',23X,'3',12X,'3',
     +  /,25X,'Atomic Positions X10   (H Atom Pos',
     +  'itions X10  )','  Ueq = 10 *((U11 + U22 + U33)/3)',
     +  ///,25X,'Atom',7X,'x',11X,'y',11X,'z',9X,'Ueq'/)
      DO 100 I=1,NX
	N=4
	IF(GELEM(I).EQ.'H ') N=3
	HX = HWORD(xx(1,I),DXX(1,I),N)
	HY = HWORD(xx(2,I),DXX(2,I),N)
	HZ = HWORD(xx(3,I),DXX(3,I),N)
	CALL UEQ(UU,DUU,I)
	HUU = HWORD(UU,DUU,N)
  100   WRITE(NPRINT,1100) GELEM(I),GDENT(I),HX,HY,HZ,HUU
 1100   FORMAT('0',24X,A2,A3,2X,4(A10,2X))
C
C   PRINT BOND LENGTHS 
C
      CALL TABLE3(NPRINT)
C
C   PRINT TEMPERATURE FACTORS
C
      WRITE(NPRINT,1125) HNAME
 1125 FORMAT('1',////,10X,A80,//,25X,'Table S1: Temperature Factors'
     *      ,/,25x,29('~'))
      WRITE(NPRINT,1150)
 1150 FORMAT(1X,//,25X,23X,'4',28X,'3',
     +    /,25X,'Temperature Factors X10     (Hydrogen Param',
     +  'eters X10  )',///,25X,'Atom',6X,'U11',9X,'U22',9X,'U33',9X,
     +  'U12',9X,'U13',9X,'U23'/)
      DO 130 I=1,NX
	N = 4
	IF(GELEM(I).EQ.'H ') N=3
	IF((ABS(U(2,2,I)) + ABS(U(3,3,I)) + ABS(U(1,2,I))
     +    + ABS(U(1,3,I)) + ABS(U(2,3,I))).GT.0.00001) GO TO 110
	HU(1,1) = HWORD(U(1,1,I),DU(1,1,I),N)
	WRITE(NPRINT,1200) GELEM(I),GDENT(I),HU(1,1)
	GO TO 130
  110   DO 120 J=1,3
	  DO 120 K=J,3
  120       HU(J,K) = HWORD(U(J,K,I),DU(J,K,I),N)
	WRITE(NPRINT,1200) GELEM(I),GDENT(I),(HU(J,J),J=1,3),
     +    HU(1,2),HU(1,3),HU(2,3)
 1200   FORMAT('0',24X,A2,A3,2X,6(A10,2X))
  130   CONTINUE
      RETURN
      END
C
C
C
C
C
      SUBROUTINE TABLE1( NPRINT )
C
C WRITTEN 86.7.9 BY NINA HEINIG
C
C PRINTS OUT STRUCTURE DETERMINATION DATA AND OTHER MISCELLANEOUS INFO.
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      include 'comatom.for'
      include 'combond.for'
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),HFORM(2),
     +              HCOMM(7),GRFACT,GRAD
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                  multsp(30),sp(3,3,30),tp(3,30)
      common /href/  hinfo(2,2), gfilen(5)
      common /gcond/ gcrad,girad,gdiff, gabs, grefn, gstat
      common /cond/  condit(6,8), nstd,mstd,cstd(6,4)
      common /htext/ gcryst(2), gtwin(5), htxt(10,5), hdata(4,19)
      common /form/ nform(3), mform, form(24,3)
      common /gform/ gform(24,3)
C
      DIMENSION ICONDT(6)
C
      WRITE(NPRINT, 1017) HNAME
 1017 FORMAT('1',////10X,A80,//25X,'Table 1:  Structure Determination'
     *     ,/,25X, 33('~'),//)
C
C   KIND OF DIFFRACTOMETER
C
      IF( GDIFF .EQ. 'E') THEN
	HCOMT = 'Enraf-Nonius'
      ELSEIF( GDIFF .EQ. 'G') THEN
	HCOMT = 'GE-XRD'
      ELSEIF( GDIFF .EQ. 'H') THEN
	HCOMT = 'Hilger-Watts 4 circle'
      ELSEIF( GDIFF .EQ. 'P') THEN
	HCOMT = 'Picker'
      ELSEIF( GDIFF .EQ. 'S') THEN
	HCOMT = 'Nicolet(Syntex)'
      ELSEIF( GDIFF .EQ. 'W') THEN
	HCOMT = 'Philips PW1100'
      ELSE
	HCOMT = 'Unspecified'
	ENDIF
      WRITE(NPRINT, 1027) HCOMT
 1027 FORMAT(1X, 23X,'Type of diffractometer used:',18X,A30,/)
C
C UNIT CELL MEASUREMENTS
C
      WRITE(NPRINT,1021) 
 1021 FORMAT(1X,23X,'Measurement of the unit cell:',/)
      WRITE(NPRINT, 1081) CONDIT(5,1)
 1081 FORMAT(1X,25X,'No. of reflections used:',22X,
     *      6X,F6.0,/)
      IF(CONDIT(3,1) .NE. 0) THEN
	WRITE(NPRINT, 1137) CONDIT(3,1), CONDIT(4,1)
 1137   FORMAT(1X,25X,'Angular range of these reflections (degrees):'
     *        ,6X,F6.1,
     *        '< 2*THETA <',F6.1,/'+',25X,57X,'_         _',/)
	ENDIF
      WRITE(NPRINT,1757) CONDIT(1,1)
 1757 FORMAT(1X,25X,'Wavelength (A):',
     *       39X,F8.4,/)
C
C INTENSITIES MEASURED
C
      WRITE(NPRINT, 1031) 
 1031 FORMAT(1X,23X,'Intensities measured:',/)
      DO 17 I=1,6
	ICONDT(I) = INTEGER(CONDIT(I,3),0)
   17   CONTINUE
      WRITE(NPRINT, 1037) ICONDT, CONDIT(4,2)
 1037 FORMAT(1X,25X,'Reflections used have Miller indices:',//1X,
     *      25X,  I4,'<h<',I4,', ',I4,'<k<',I4,
     *      ', ',I4,'<l<', I4,/'+',29X,'_ _',10X,'_ _',
     *      10X,'_ _',//
     *      1X,25X,'and have SIN(THETA)/LAMBDA less than: ',
     *      13X,F8.1,/)
      WRITE(NPRINT, 1057) CONDIT(5,2)
 1057 FORMAT(1X,25X,'Number of measured reflections:',22X,
     *         F6.0,/)
      WRITE(NPRINT, 1067) CONDIT(6,7)
 1067 FORMAT(1X,25X,'Number of unobserved reflections:',20X,
     *         F6.0,/)
      WRITE(NPRINT, 1077) CONDIT(1,8)
 1077 FORMAT(1X,25X,'Number of reflections used in refinement:',12X,
     *         F6.0,/)
      WRITE(NPRINT, 1247) CONDIT(5,7),CONDIT(4,7)
 1247 FORMAT(1X,25X,'Number of reflections with I(obs) >',
     *       F8.4,'*SIGMA(I(obs)):',5X,F8.2,/'+',34X,25X,'_',/)
      WRITE(NPRINT, 1079) CONDIT(2,5)
 1079 FORMAT(1X,25X,'Number of symmetry independent reflections:',
     *         10X,F6.0,/)
      WRITE(NPRINT, 1085) CONDIT(3,5)
 1085 FORMAT(1X,25X,'Agreement index between equivalent reflections:',
     *       4X, F8.3,/)
      IF(CONDIT(5,6) .LT. 90000000)THEN
	WRITE(NPRINT ,1207) CONDIT(5,6)
 1207   FORMAT(1X,25X,'Reflections with I > ',F8.4,'*SIGMA(I(obs))',
     *        ' contribute to the refinement',/)
      ELSE
	WRITE(NPRINT, 1217) CONDIT(6,6)
 1217   FORMAT(1X,25X,'All reflections with I >',F8.2,
     *         ' are used in the refinement',/)
	ENDIF
C
C  ABSORPTION CORRECTION
C
      WRITE(NPRINT, 1083)
 1083 FORMAT(1X,23X,'Absorption correction: ',/)
      IF( GABS .NE. ' ') THEN
	GA = GABS
	CALL UPCASE(GA)
	IF( GA .EQ. 'A') THEN
	  HCOMT = '    Analytical'
	ELSEIF( GA .EQ. 'E') THEN
	  HCOMT = 'Semi-empirical'
	ELSEIF( GA .EQ. 'G') THEN
	  HCOMT = '      Gaussian'
	ELSEIF( GA .EQ. 'M') THEN
	  HCOMT = '  Monte Carlo'
	ELSEIF( GA .EQ. 'N') THEN
	  HCOMT = 'No correction'
	ENDIF
	LABS = LENGTH(HDATA(8,1)(9:80))
	LABS2 = LENGTH(HDATA(8,2)(9:80))
	IF( LABS .GT. 35 .OR. LABS2 .NE. 0) THEN
	  WRITE(NPRINT,1087) HCOMT
 1087     FORMAT(26X,'Type of absorption correction made:',10X,A14,/)
	  DO 87 I =1,4  
	    IF(LENGTH(HDATA(8,I)) .NE. 0) THEN
	      WRITE( NPRINT,1787) HDATA(8,I)(9:80)
 1787         FORMAT(1X,25X,45X,A65,/)
	      ENDIF
   87       CONTINUE
	ELSE
	  WRITE(NPRINT,1797) HCOMT,HDATA(8,1)(9:80)
 1797     FORMAT(26X,'Type of absorption correction made:',10X,A14,1X,
     *           A40,/)
	  ENDIF
	WRITE( NPRINT, 1097) CONDIT(1,4)
 1097   FORMAT(1X,25X,30X,24X,3X,'-1',/
     *        1X,25X,'Linear absorption coefficient:', 16X,
     *        F8.1,' mm ',/)
	WRITE(NPRINT, 1099) CONDIT(3,4),CONDIT(2,4)
 1099   FORMAT(1X,25X,'Maximum absorption correction applied: ',12X,
     *        F8.4,//1X,25X,'Minimum absorption correction applied: '
     *        ,12X,F8.4,/)
	ENDIF
      IF(GREFN .EQ. 'F')THEN
	HCOMT = 'Structure factors'
	WRITE(NPRINT, 1177) HCOMT
 1177   FORMAT(1X,25X,'Refinement based on:', 22X, A30,/)
	WRITE(NPRINT, 1727)
 1727   FORMAT(1X,25X,'R = &||F | - |F || / &|F |' ,/,
     *            26X,8X, '0', 6X, 'C', 8X, '0', //,
     *            26X,18X, '2',9X,'2',1X,'0.5',/,
     *            26X,'Rw = (& w(|F | -|F |) / & wF )',/,
     *            26X,12X, '0',5X,'C',9X,'0',/)
      ELSEIF(GREFN .EQ. 'S') THEN
	HCOMT = 'Square of the structure factor'
	WRITE(NPRINT, 1177) HCOMT
	WRITE(NPRINT, 1737) 
 1737   FORMAT(1X,25X,8X,'2',4X,'2',6X,'2',/
     *        26X,'R = & |F  - F | / &F ',/
     *        26X,8X,'0',4X,'C',6X,'0',//
     *        26X,10X,'2',4X,'2',1X,'2',7X,'4',1X,'0.5',/
     *        26X,'Rw = ( w(F  - F )  / & wF )',/,
     *        26X,10X,'0',4X,'C',9X,'0',/)
       ELSEIF(GREFN .EQ. 'I') THEN
	HCOMT = 'Intensities'
	WRITE(NPRINT,1177) HCOMT
	WRITE(NPRINT,1747)
 1747   FORMAT(1X,25X,'R = &||I | - |I || / &|I |',/,
     *            26X,8X,     '0',6X,'C',8X,  '0',//,
     *            26X,18X, '2',9X,'2',1X,'0.5',/
     *            26X,'Rw = (& w(|I | -|I |) / & wI )',/
     *            26X,12X, '0',5X,'C',9X,'0',/)
	ENDIF
      do 580  i= 1,8
	if(i .eq. 2) then
	  ncount = 4
	  hcomt = 'The material is prepared by: '
	elseif( i .eq. 3) then
C
C EXTINCTION PARAMETERS
C
	  write(nprint, 1707)
 1707     format(1X,23X, 'Extinction correction:',/)
	  IF( GCRYST(1) .NE. '   ') THEN
	    IF(GCRYST(1) .EQ. 'BAC') THEN
	      HCOMT = ' Becker and Coppens'
	    ELSEIF(GCRYST(1) .EQ. 'LAR') THEN
	      HCOMT = '             Larson'
	    ELSEIF(GCRYST(1) .EQ. 'NAT') THEN
	      HCOMT = 'Nelmes and Thornley'
	      ENDIF
	    WRITE(NPRINT, 1167) HCOMT
 1167       FORMAT(1X,25X,'Extinction correction used:',13X,
     *            A19,/)
	  ENDIF
	  ncount = 10
	  hcomt = 'The extinction correction is: '
C
C REFINEMENT OF PARAMETERS
C
	elseif( i .eq. 4) then
	  write(nprint, 1717)
 1717     format(1X,23X, 'Refinement of parameters:',/)
	elseif( i .eq. 5) then
	  ncount = 12
	  hcomt = 'Form factors are found:      '
	elseif( i .eq. 6) then
	  ncount = 13 
	  hcomt = 'Weighting scheme used:       '
	elseif( i .eq. 7) then
	  ncount = 14
	  hcomt = 'Hydrogen atom refinement:    '
	elseif( i .eq. 8) then
	  ncount = 15
	  hcomt = 'Programs used:               '
	  endif
	labs = length(hdata(ncount,1)(9:80))
	if(labs .ne. 0) then
	  write(nprint,1560) hcomt, hdata(ncount,1)(9:80)
 1560     format(1X,25X,A29,A71,/) 
	  do 560 ij=2,4
	    labs = length(hdata(ncount,ij)(9:80))
	    if(labs .ne. 0) then
	      write(nprint, 1570) hdata(ncount,ij)(9:80)
 1570         format(1X,25X,29X,A71,/)             
	    else
	      goto 570
	      endif
  560       continue
  570     endif
  580   continue
      IF(CONDIT(2,8) .NE. 0) THEN
	NCOND = INTEGER(CONDIT(2,8),0)
	WRITE(NPRINT, 1107) NCOND
 1107   FORMAT(1X,25X,'Number of parameters refined:',24X,I6,/)
	ENDIF
      WRITE(NPRINT,1187) CONDIT(1,6),CONDIT(2,6)
 1187 FORMAT(1X,25X,'Maximum shift/error in final round of refinement:'
     *       ,2X,F8.4,//           
     *       1X,25X,'Average shift/error in final round of refinement:'
     *       ,2X,F8.4,/)
      IF (GIRAD .EQ. 'X') THEN
	HCOMT = '(eA  )'
	ELSE
	HCOMT = '         '
	ENDIF
      WRITE(NPRINT, 1197) CONDIT(4,6),HCOMT,CONDIT(3,6),HCOMT
 1197 FORMAT(1X, 25X, 27X, 24X, 11X, '-3',/
     *       1X,25X,'Maximum Difference Density:',24X,F8.4,A9,/
     *       1X, 25X, 35X, 27X, '-3',/
     *       1X,25X,'Minimum Difference Density:',24X,F8.4,A9,/)
      IF(GSTAT .NE. ' ') THEN
	GA = GSTAT
	CALL UPCASE(GA)
	IF(GA .EQ. 'F') HCOMT='Structure factor'
	IF(GA .EQ. 'S') HCOMT='Square of the structure factor'
	IF(GA .EQ. 'I') HCOMT='Intensities'
	WRITE(NPRINT, 1227) HCOMT
 1227   FORMAT(1X,25X,'Form of structure factors in agreement indices:',
     *         5X, A40,/)
	IF (CONDIT(3,7) .GT. 0.01) THEN
	  WRITE(NPRINT, 1237) CONDIT(3,7)
 1237     FORMAT(1X,25X,'Goodness of fit:',35X, F8.4,/)
	  ENDIF
	ENDIF
      RETURN
      END
C
C
C
C

C
C
      SUBROUTINE TABLE3(NPRINT)
C
C WRITTEN 86.7.7 BY NINA HEINIG
C
C PRINTS OUT BOND LENGTH AND BOND TYPE
C CALLS HXWORD
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      include 'comatom.for'
      include 'combond.for'
C
C  BOND
C
      WRITE(NPRINT, 1107) HNAME
 1107 FORMAT('1',////,10X,A80,//,25X,'Table3: Bonds',/,25X,
     *       13('~'),/) 
      WRITE(NPRINT, 1117) 
 1117 FORMAT(1X,//,25X,4X,'Bond',12X,'Length',8X,/)
      DO 17 I=1,NBON
	IB = IBON(1,I)
	JB = JBON(1,I)
	WRITE(NPRINT,1127) GELEM(IB),GDENT(IB),GELEM(JB),GDENT(JB)
 1127   FORMAT(1X,/,24X,A2,A6,'-',A2,A6)
C
C BOND LENGTH AND ERROR     
C
	WRITE(NPRINT,1137) HXWORD(XBON(1,I),XBON(2,I))
 1137   FORMAT('+',41X, A14)
   17   CONTINUE
      RETURN 
      END 
C
C
C
C
C
C
C
      subroutine rval(nval)
c
c---- reads in valence parameters from the valence file 'nval'
c     and stores them in common /bost/
c
c     calls integer
c
c     common /bost/ may already be partly filled
c
c     note:  /bost/ is now /bostt/
c          (they'll all be changed someday)
c
c     if(nbon.ne.0)  parameters for the existing bonds are stored
c
c     if(nbon.eq.0)  parameters for all possible bonds are stored
c
c     in general the program will not find parameters for all given
c     bonds, the rest has to be altered by 'calval'
c
c     pointers:
c       gp(n)    contains the element symbols in alphabetic order
c       ip(n,1)  contains the corresponding atom (ion) numbers
c       ip(n,2)  contains the oxidation states of these atoms (ions)
c
c**** caution: program uses 'ICHAR' after statement 290, 300 AND 310
c
c     written by Daniel Altermatt
c---- McMaster University, June 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      dimension gp(100),ip(100,2),hline(10)
c
      common /files/  nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),pbost(5,30)
      common /gbost/  gbost(2,30)
c
c---- ikor makes  ichar('0')-ikor=0
c
      ikor=ichar('0')
      do 10 i=1,100
10      gp(i)=' '
      n1=0
      n2=nbost
      ieof=0
c
c---- if(nbon.ne.0) fill up common /bost/
c
********************************************************************
** WARNING: THERE IS AN ERROR IN THIS SECTION WHEN NBON.NE.0 THAT **
** LEADS TO LOOPING WITHIN THE PROGRAM                            **
** UNTIL THIS IS FIXED THE SECTION IS BYPASSED                    **
********************************************************************
      GO TO 130
********************************************************************
   15 if(nbon.eq.0) goto 130
      do 50 i=1,nbon
	if(nbost.eq.0) goto 40
	do 30 j=1,nbost
	if(gelem(ibon(1,i)).ne.gbost(1,j).or.integer(oxid(ibon(1,i)),0)
     1    .ne.ioxbost(1,j)) goto 20
	if(gelem(jbon(1,i)).ne.gbost(2,j).or.integer(oxid(jbon(1,i)),0)
     1    .ne.ioxbost(2,j)) goto 30
	goto 50
20      if(gelem(ibon(1,i)).ne.gbost(2,j).or.integer(oxid(ibon(1,i)),0)
     1    .ne.ioxbost(2,j)) goto 30
	if(gelem(jbon(1,i)).ne.gbost(1,j).or.integer(oxid(jbon(1,i)),0)
     1    .ne.ioxbost(1,j)) goto 30
	goto 50
30      continue
40      nbost=nbost+1
	if(nbost.gt.mbost) then
	  write(nout,2000)
2000      format(1X,'***** error RVAL: overflow in common /bost/,',
     1      ' remainder ignored',/)
	  goto 60
	  endif
	gbost(1,nbost)=gelem(ibon(1,i))
	gbost(2,nbost)=gelem(jbon(1,i))
	ioxbost(1,nbost)=integer(oxid(ibon(1,i)),0)
	ioxbost(2,nbost)=integer(oxid(jbon(1,i)),0)
50      continue
c
c---- set up pointers for common /bost/
c
60    if(n2.eq.0) n2=1
      do 120 i=n2,nbost
	j=1
	if(ioxbost(1,i).lt.ioxbost(2,i)) j=2
	if(n1.ne.0) goto 70
	  n1=1
	  gp(n1)=gbost(j,i)
	  ip(n1,1)=i
	  ip(n1,2)=ioxbost(j,i)
	  goto 120
70      do 110 n=1,n1
	  if(gbost(j,i).ge.gp(n)) goto 100
80        nn=n+1
	  n1=n1+1
	  do 90 k=n1,nn,-1
	    gp(k)=gp(k-1)
	    ip(k,1)=ip(k-1,1)
90          ip(k,2)=ip(k-1,2)
	  gp(n)=gbost(j,i)
	  ip(n,1)=i
	  ip(n,2)=ioxbost(j,i)
	  goto 120
100       if(gbost(j,i).gt.gp(n)) goto 110
	  if(ioxbost(j,i).gt.ip(n,2)) goto 110
	  if(ioxbost(j,i).lt.ip(n,2)) goto 80
	  j1=j+1
	  if(j1.eq.3) j1=1
	  j2=1
	  if(ioxbost(j,i).eq.ioxbost(j2,ip(n,1))) j2=2
	  if(gbost(j1,i).gt.gbost(j2,ip(n,1))) goto 110
	  if(gbost(j1,i).lt.gbost(j2,ip(n,1)).or.ioxbost(j1,i).le.
     1      ioxbost(j2,ip(n,1))) goto 80
110       continue
	n1=n1+1
	gp(n1)=gbost(j,i)
	ip(n,1)=i
	ip(n,2)=ioxbost(j,i)
120     continue
      if(n1.eq.nbost) goto 220
      write(nout,2010) n1,nbost
2010  format(1X,'***** error RVAL: sorting of /bostt/ failed,',
     1  ' gp(',I3,'), nbost=',I3,/)
      return
c
c---- set pointers for common /ident/
c
130   if(nx.ne.0) goto 140
      write(nout,2020)
2020  format(1X,'***** error RVAL: no atoms or bonds found',/)
      return
c
c---- gp is filled alphabetically; each element is further
c     sorted by oxidation number, lowest to highest
c
140   do 200 i=1,nx
	if(n1.ne.0) goto 150
	n1=1
	gp(n1)=gelem(i)
	ip(n1,1)=i
	ip(n1,2)=integer(oxid(i),0)
	goto 200
150     do 190 n=1,n1
	  if(gelem(i).ge.gp(n)) goto 180
160       n1=n1+1
	  if(n1.gt.100) goto 210
	  nn=n+1
	  do 170 k=n1,nn,-1
	    gp(k)=gp(k-1)
	    ip(k,1)=ip(k-1,1)
170         ip(k,2)=ip(k-1,2)
	  gp(n)=gelem(i)
	  ip(n,1)=i
	  ip(n,2)=integer(oxid(i),0)
	  goto 200
180       if(gelem(i).eq.gp(n).and.integer(oxid(i),0).lt.ip(n,2))
     1      goto 160
190       continue
	n1=n1+1
	if(n1.gt.100) goto 210
	gp(n1)=gelem(i)
	ip(n1,1)=i
	ip(n1,2)=integer(oxid(i),0)
200     continue
      if(n1.eq.nx) then
	do 205 i = 1,n1
205       call upcase(gp(i))
	goto 220
	endif
210   write(nout,2030) n1,nx
2030  format(1X,'***** error RVAL: sorting of /ident/ failed,',
     1  ' gp(',I3,'), nx=',I3,/)
      return
c
c---- read in blocks of 10 lines of 'nval' until we reach the element
c     symbol gp(n) in column(1:2) of hline
c
220   n=1
230   if(ieof.ne.0) return
      do 240 i=1,10
	read(nval,1000,end=999) hline(i)
1000    format(A80)
	call upcase(hline(i))
240     continue
250   i1=10
      if(ieof.ne.0) i1=i-1
260   if(n.gt.n1) return
c
c---- compare first element name on tenth line read in from valence
c     file with element for which the search is being made to see if
c     the block of ten lines may contain the required information;
c     if not, read another ten lines
c
      if(hline(i1)(1:2).lt.gp(n)(1:2)) goto 230
c
c---- loop through pointer gp(n) and compare content with hline
c
      do 340 i=1,i1
c
c------ look in the hline array for match of element name and
c       oxidation number for which the search is being made
c
	if(hline(i)(1:2).lt.gp(n)(1:2)) goto 340
	if(hline(i)(1:2).eq.gp(n)(1:2)) goto 290
	n=n+1
	if(n2.eq.0) goto 260
	n=n-1
	nbost=nbost-1
	do 280 i1=ip(n,1),nbost
	  do 270 j=1,5
	    pbost(j,i1)=pbost(j,i1+1)
	    if(j.gt.2) goto 270
	    gbost(j,i1)=gbost(j,i1+1)
	    ioxbost(j,i1)=ioxbost(j,i1+1)
270         continue
	  ifbost(i1)=ifbost(i1+1)
280       continue
	goto 260
c
c---- hit of first element, check oxidation state and possible
c     partner in bond...
c
290     is=1
	if(hline(i)(3:3).eq.'-') is=-1
	ns=ichar(hline(i)(4:4))-ikor
	ns=ns*is
	if(n2.eq.0) goto 310
c
c---- ...for filled /bostt/: pick up defined pairs
c
300     if(ip(n,2).ne.ns) goto 340
	j=1
	if(ip(n,2).ne.ioxbost(2,ip(n,1))) j=2
	if(gbost(j,ip(n,1))(1:2).ne.hline(i)(6:7)) goto 340
	is=1
	if(hline(i)(9:9).eq.'-') is=-1
	ns=ichar(hline(i)(10:10))-ikor
	ns=ns*is
	if(ioxbost(j,ip(n,1)).ne.ns) goto 340
	read(hline(i),5000)
     1    ifbost(ip(n,1)),(pbost(j,ip(n,1)),j=1,3)
5000    format(BZ,10X,I2,3F7.3)
	n=n+1
	if(n.gt.n1) return
	goto 300
c
c---- ...for /ident/: search for any pairs possible
c
310     if(ip(n,2).ne.ns) goto 340
c
c---- having found a match (with hline(i)'s first element and oxi-
c     dation number), search through gp to find a similar match
c     with the second element and oxidation number of hline(i)
c
	is=1
	if(hline(i)(9:9).eq.'-') is=-1
	ns=ichar(hline(i)(10:10))-ikor
	ns=ns*is
	do 330 l=1,n1
	  if(gp(l).ne.hline(i)(6:7)) goto 330
	  if(ip(l,2).ne.ns) goto 330
	  nbost=nbost+1
	  if(maxdim(nbost,mbost,' *rval*320* /bost/').ne.0) return
 320      read(hline(i),5010)
     1      (gbost(l1,nbost),ioxbost(l1,nbost),l1=1,2),ifbost(nbost),
     2      (pbost(l2,nbost),l2=1,3)
5010      format(BZ,A2,I2,1X,A2,I3,I2,3F7.3)
	  if(gbost(1,nbost).eq.gbost(1,nbost-1).and.
     +       gbost(2,nbost).eq.gbost(2,nbost-1).and.
     +       ioxbost(1,nbost).eq.ioxbost(1,nbost-1).and.
     +       ioxbost(2,nbost).eq.ioxbost(2,nbost-1)) nbost=nbost-1
	  goto 340
330       continue
340     continue
      goto 230
999   ieof=1
      goto 250
c
c---- thanks boys, that's it !!
c
      end
C
C
C
C
      SUBROUTINE RDBONF(IER)
C     -----------------
C
C   WRITTEN 83.11.24 BY I D BROWN / 84.6.13 D. ALTERMATT
C
C   CALLS: MAINLB: param, sghall
C          MATHLB: integer, second
C          SPELIB: spost, spesor, spmult, spsym
C          SYMLIB: grmult
c
C
C
C   READS ONE ENTRY FROM THE BOND FILE STARTING AT THE CURRENT LINE
C   IER=0 EXCEPT IF THE ENTRY CONTAINS ASTERISKS OR MORE THAN THE
C   MAXIMUM ALLOWED NUMBER OF ATOMS OR BONDS IN WHICH CASE IER=1
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      include 'comatom.for'
      include 'combond.for'
      COMMON/CELL/A,B,C,AL,BE,AGA,VOL,AM(3,3),ICELL
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     +       HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON/TITLE/JAUTH,KCOMM
      DIMENSION ISM(5),CENT(3,7)
      DATA GA,GB,GC,GD,GF,GG,GQ,GR,GS,GZ,GAT/'A','B','C','D','F','G',
     +     'Q','R','S','Z','@'/
      DATA CENT/0.,0.,0., 0.,.5,.5, .5,0.,.5, .5,.5,0., .5,.5,.5,
     1     .33333,.66667,.66667, .66667,.33333,.33333/
C
      IER=0
      NX=0
      nbon=0
    5 READ(NREAD,1000,END=900) GKEY,HLINE
 1000 FORMAT(A1,A79)
      IF(GKEY.EQ.GAT) then
	gent(1:6)=hline(2:7)
	GO TO 5
	endif
C
C   Z INDICATES END OF ENTRY
C
      IF(GKEY.EQ.GZ) THEN
	IF(NX.EQ.0.OR.nbon.EQ.0) IER=1
	WRITE(NOUT,1005) GENT
	WRITE(NOUT,1006) HCODN(1)
	WRITE(NOUT,1007) HFORM(1)
	WRITE(NOUT,1008) GRFACT
	WRITE(NOUT,1009) A,B,C,AL,BE,AGA
	WRITE(NOUT,1010) HALL
	DO 8 I=1,nbon
    8     WRITE(NOUT,1011) (IBON(K,I),K=1,5),(JBON(J,I),J=1,5),
     1      IFBON(3,I),(XBON(J,I),J=1,6)
	DO 9 I=1,NX
    9     WRITE(NOUT,1012) I,GELEM(I),GDENT(I),OXID(I),NPOS(I),
     +     OCCU(I),ICN(I),IALTNO(I)
	IF(NDEBUG.NE.0) WRITE(NDEBUG,1013) SECOND(3)
 1005   FORMAT('    ',A8)
 1006   FORMAT('    ',A16)
 1007   FORMAT('    ',A74)
 1008   FORMAT('    ',A8)
 1009   FORMAT('    ',6F7.4)
 1010   FORMAT('    ',A15)
 1011   FORMAT('    ',I3,'(',I2,3I3,')',2X,I3,'(',I2,3I3,')',I2,
     1   F7.3,F6.3,F6.2,1X,3F7.4)
 1012   FORMAT('    ',I3,A2,A3,F5.2,I5,F5.2,2I4)
 1013   FORMAT(' ++DEBUG++RDBONF++1012++ COMPLETED AT ',F6.2)
	RETURN
      ENDIF
C
C   FIRST CHECK ON SPACE GROUP SYMBOL
C
      IF(GKEY.EQ.GS) THEN
C     ------------------
	HALL=HLINE
	call sghall
	call grmult(ier)
	ndeb=ndebug
	ndebug=0
	nbug=0
	call spost(ier,nbug)
	call spesor
	call spmult
	call spsym
	ndebug=ndeb
	GOTO 5
	ENDIF
C
C   REPLACE ASTERISKS BY 9'S AND FLAG
C
      DO 7 I=1,79
	IF(HLINE(I:I).EQ.'*') THEN
	  HLINE(I:I)='9'
	  IER=1
	  ENDIF
    7   CONTINUE
      IF(GKEY.EQ.GB.OR.GKEY.EQ.GD) THEN
C    -------------------
	nbon=nbon+1
	IF(nbon.GT.mbon) THEN
	  IER=1
	  nbon=mbon
	ELSE
	  IF(GKEY.EQ.GB)THEN
	    READ(HLINE,1020)
     +        IBON(1,nbon),JBON(1,nbon),ISM,GDIS,XBON(1,nbon),
     +         (XBON(J,nbon),J=4,6),XBON(2,nbon),XBON(3,nbon)
 1020       FORMAT(2I2,I2,4I1,A1,F4.3,3F5.4,F3.3,F3.2)
	    JBON(2,nbon)=ISM(1)
	    DO 10 JJ=1,3
	      TT=12.*T(JJ,ISM(1))
	      TT=TT+12.*(CENT(JJ,ISM(2))+ISM(JJ+2)-2.0)
	      IT=INTEGER(TT,0)
	      JBON(JJ+2,nbon)=MOD(IT,12)
   10         CONTINUE
	    IFBON(3,nbon)=0
	    IF(GDIS.EQ.'D') IFBON(3,nbon)=1
	  ELSE IF(GKEY.EQ.GD) THEN
	    READ(HLINE,1021)
     1        IBON(1,nbon),(JBON(I,nbon),I=1,5),IFBON(3,nbon),
     2        XBON(1,nbon),(XBON(I,nbon),I=4,6),XBON(2,nbon),
     3        XBON(3,nbon)
 1021       FORMAT(3I2,3I3,I1,F4.3,3F5.4,F3.3,F3.2)
	    ENDIF
	  IBON(2,nbon)=1
	  DO 22 K=1,3
   22       IBON(K+2,nbon)=0
	  GO TO 5
	  ENDIF
	ENDIF
      IF(GKEY.EQ.GA.OR.GKEY.EQ.GG) THEN
C     ------------------
	NX=NX+1
	IF(NX.GT.MX) THEN
	  NX=MX
	  IER=1
	ELSE
	  IF(GKEY.EQ.GA) READ(HLINE,1030)
     +      GELEM(NX),GDENT(NX),OXID(NX),NPOS(NX),OCCU(NX),ICN(NX)
 1030     FORMAT(2X,A2,A3,F4.2,I3,F3.2,I2)
	  IF(GKEY.EQ.GG) READ(HLINE,1031)
     +      GELEM(NX),GDENT(NX),OXID(NX),NPOS(NX),OCCU(NX),ICN(NX),
     +      IALTNO(NX)
 1031     FORMAT(2X,A2,A3,F4.2,I3,F3.2,2I2)
	  GWYCK(NX)=' '
	  ENDIF
	GO TO 5
	ENDIF
      IF(GKEY.EQ.GR) THEN
C     ------------------
	READ(HLINE,1040)
     +    GRAD,GRFACT
 1040   FORMAT(A1,A4)
	GO TO 5
	ENDIF
      IF(GKEY.EQ.GF) THEN
C     ------------------
	HFORM(1)=HLINE
	GO TO 5
	ENDIF
      IF(GKEY.EQ.GQ) THEN
C     ------------------
	READ(HLINE,1060) HCODN(1)
 1060   FORMAT(A16)
	GO TO 5
	ENDIF
      IF(GKEY.EQ.GC) THEN
C     ------------------
	ICELL=1
	READ(HLINE,1070)
     +    A,B,C,AL,BE,AGA
 1070   FORMAT(3F5.3,3F5.4)
	call param
	GO TO 5
	ENDIF
      WRITE(NOUT,2000) GKEY
 2000 FORMAT(1X,A1,' IS AN UNRECOGNISED RECORD TYPE')
      GO TO 5
  900 WRITE(NOUT,1900)
 1900 FORMAT(' ++RDBONF++1900++  END OF FILE REACHED BEFORE END OF ',
     + 'ENTRY')
      IER=-1
      RETURN
      END
C
C
C
C
      SUBROUTINE FDBONF(GEN1,KENT)
C     -----------------
C
C   WRITTEN 83.11.24 BY I D BROWN / 84.6.13 D. ALTERMATT /89.7.1 S. URQUHART
C
C   THIS SUBROUTINE POSITIONS THE BONDFILE READY TO READ ENTRY GEN1.
C   IF GEN1='0' OR 'BLANK' THE NEXT ENTRY IS FOUND
C   IF THE FIND IS SUCCESFUL KENT=0.
C   IF END OF FILE IS REACHED BEFORE GEN1 THEN KENT=-1
C
c   icount is the counter of entries read in sucessfully. If there is an
c   eof it is set to the last complete entry. Nent and iflag are used to set
c   this appropriately in this subroutime. Icount is a feature of strumo(r)
c   summer `89 version and is not complet e through all the file reading files
c
c   calls MATHLB: hleft
c
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      COMMON/GTITLE/GENT
      COMMON/COUNTER/ICOUNT
      KENT=0
      NENT=0
      IFLAG=0
C
C   LOOK FOR FIRST ENTRY
C
   10 READ(NREAD,1000,END=900) g,HLINE
 1000 FORMAT(A2,A78)
      IF(g(1:2).NE.'@E') GO TO 10
      NENT=1
      gen2=hleft(hline,6)
      if(ndebug.ne.0) write(ndebug,1001) gen1,gen2,g,hline(1:8)
 1001 format(' *debug*fdbonf*1001* ',2('*',A8),'*',A2,A8,'*')
      IF(gen2(1:6).EQ.GEN1(1:6).OR.GEN1(1:1).EQ.'0'.OR.
     1  GEN1(1:6).EQ.'      '.or.gen1(1:1).eq.'*') THEN
	GENT=HLINE(1:8)
	RETURN
      ELSE
C     READ FILE TO LOOK FOR SUBSEQUENT ENTRIES
   20   READ(NREAD,1020,END=900) g
 1020   FORMAT(A1)
	IF(g(1:1).EQ.'Z') THEN
	  READ(NREAD,1000,END=900) g,HLINE
	  gen2=hleft(hline,6)
	  if(ndebug.ne.0) write(ndebug,1002) gen1,gen2,g,hline(1:8)
 1002     format(' *debug*fdbonf*1002* ',2('*',A8),'*',A2,A8,'*')
	  IF(g(1:2).EQ.'@E'.AND.gen2(1:6).EQ.GEN1(1:6))THEN
	    GENT=HLINE(1:8)
	    IF (IFLAG.EQ.1) ICOUNT=NENT
	    RETURN
	  ELSE IF (G(1:2).EQ.'@E') THEN
	    IFLAG=1
	    NENT=NENT+1
	    GO TO 20
	  ENDIF
	ELSE
	  GO TO 20
	ENDIF
      ENDIF
  900 WRITE(NOUT,1900) GEN1
 1900 FORMAT(' ++FDBOND++  END OF FILE REACHED BEFORE ENTRY ',A8,
     +  ' FOUND')
      IF (IFLAG.EQ.1) ICOUNT=NENT-1
      KENT=-1
      RETURN
      END
c
c------------------------------------------------------------------
c  Quattro pro compatible file maker:
c------------------------------------------------------------------
c
      subroutine qmenu
c
c     Written 3/93 by T. Stephen
c
c     For the Quattro file option in Strumor, menu level 3
c     option 15.
c
c     calls qpro,qhelp,wait

      implicit character*8 (g)
      implicit character*80 (h)
      common/files/nin,nout,ndebug
      common/qblock/iq(60)
      common/hqblock/hfile
      include 'comatom.for'
      include 'combond.for'

      if (ibon(2,1).eq.0) goto 35
      hfile='stoq.prn'
 10   write(nout,1000)hfile(1:16)
 1000 format(/,'                Quattro Output',//,
     1 'choose:',//,'1: Help',/
     2 '2: Change filename (Currently  ',a10,')',/
     3 '3: Send all bonds to quattro file',/
     4 '4: Send bonds around selected atoms to quattro file',/
     5 '5: Send bonds around single atom to quattro file',/,
     6 '6: Send only the bonds around the cations to quattro file',/,
     7 '7: Send only the bonds around the anions to quattro file',//,
       
     8 /////////,'20: Return to menu level 3',/)
 20   read(nin,1100,err=15) in
 1100 format(i6)
c help
      if (in.eq.1) call qhelp
c new filename
      if (in.eq.2) then
	write(nout,1020)
 1020   format(//,'Enter filename',/)
	read(nin,1110,err=25)hfile
 1110   format(a80)
	write(nout,1130)hfile
 1130   format('O.K., will attempt to make next save to ',a40)
	call wait
      endif
c print all
      if (in.eq.3) then
	do 30 i=1,nx
 30       iq(i)=1
	call qpro
      endif
c print choice
      if (in.eq.4) then
	write(nout,1040)
 1040   format(/////////////////////////////)
	do 40 i=1,nx
 110      write(nout,1050)gelem(i)(1:2),gdent(i)(1:4)
 1050     format('Include bonds around ',a2,a4,' in Quattro file? ')
	  write(nout,1140)
 1140     format('(y)es, (n)o or (e)xit=no to remaining atoms') 
	  iq(i)=0
	  call readyn(*50,*40,*100,*110)
 50       iq(i)=1
 40     continue
 100    call qpro
      endif
c print only one
      if (in.eq.5) then
	write(nout,1040)
	do 60 i=1,aint(real(nx)/2)
	  write(nout,1060)2*i-1,gelem(2*i-1)(1:2),gdent(2*i-1)(1:4),
     +                    2*i,gelem(2*i)(1:2),gdent(2*i)(1:4)
 1060     format(i2,'. (',a2,a4,')',25x,i2,'. (',a2,a4,')')
 60     continue
	if (mod(nx,2).eq.1) 
     +    write(nout,1120)nx,gelem(nx)(1:2),gdent(nx)(1:4)
 1120   format(i2,'. (',a2,a4,')')
	write(nout,1070)
 1070   format('Which atom''s bonds would you like to use? ',/,
     +   '(identify by number)')
	read(nin,1100,err=15) j  
	if (j.ge.1.and.j.le.nx) then
	  iq(j)=1
	  call qpro
	else
	  write(nout,1080)
 1080     format('Not a valid input!')
	  call wait
	endif
      endif
c print cations
      if (in.eq.6) then
	do 80 i=1,nx
 80       if (oxid(i).gt.0) iq(i)=1
	call qpro
      endif
c print anions
      if (in.eq.7) then
	do 90 i=1,nx
 90       if (oxid(i).lt.0) iq(i)=1
	call qpro
      endif
c see ya
      if (in.eq.20) return
      do 70 i=1,nx                    
 70     iq(i)=0
      goto 10

c errors
 15   write(nout,1010)
 1010 format('Expects integer input')
      call wait
      goto 10
 25   write(nout,1030)
 1030 format('Need a filename!')
      call wait
      goto 10
 35   write(nout,1090)
 1090 format('You must calculate and store the bonds before ',
     +       'using this routine.')
      call wait
      return
      end

      
***********************************************
cQuattro Output Routines....
c
c     called from strumor, menu level 3, #15
c     calls:  wait, expand, iyat

      subroutine qpro
c     ---------------
c
c     Written 3/93 by T. Stephen
c    
c     Should send bondlengths and some other stuff to a file which 
c     can be retrieved by the Quattro Pro spreadsheet by selecting
c     'Tools' then 'Import' then 'Comma and "" Delimited File' and
c     the file name.
c
c     calls expand,wait,jyat
c
      implicit character*8 (g)
      implicit character*80 (h)
      implicit character*249 (z)

      include 'comatom.for'
      include 'combond.for'
      common /files/ nin,nout,ndebug
      common /distt/nb,mb,idist(5,30),xdist(6,30),minbond
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1              nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2              netind(20),ifbond(4,600)
      common/qblock/iq(60)
      common/hqblock/hfile
      dimension jjat(5)

c   initialize variables
      maxnb=0
      bondlim(1)=0.0001
      bondlim(2)=5
      hfort='FORTRAN'
      hstat='unknown'
      nunit=77
c   open file
      nn=iclofi(nunit)
      if (nn.ne.0) then
	write(nout,1000)nn
 1000   format('Error number IO-',i2,' has gothcha')
      endif
      nnw=nunit
      nn=ifile(hfile,hfort,hstat,nunit)

c   title
      write(nnw,1010)
 1010 format('"Lengths,","Valences","and","Errors","by","Strumo(r)"')

c   the real mess
      do 10 i=1,nx
	if (iq(i).ge.1) then
	  iat=i
	  call expand(3)
	  write(nnw,1200)gelem(i),gdent(i)
 1200     format(////,'"Environment","of","',a2,a6,'"')
	  write(nnw,1210)
 1210     format('"","Sym+Trans","Length","Error","Valence",',
     +     '"x.Comp","y.Comp","z.Comp"')
	  do 20 j=1,nbo
	    call jyat(ibond(2,j),jjat)
 20         write(nnw,1220)gelem(jjat(1)),
     +        gdent(jjat(1)),(jjat(jjj),jjj=2,5),
     +        (xbond(jjj,j), jjj=1,6)
 1220       format('"',a2,a6,'","',4(i3),'",',6(f8.6,','))
	endif
 10   continue
      write(nout,1119)hfile
 1119 format('O.K., bonds written to ',A80,'.',/,
     1  'To retrieve from Quattro, select ''Tools'', ''Import'' ',/,
     2  '''Comma and "" Delimited file'', and the filename')
      if (iclosu(0,hfile).ne.0) write(1020)
 1020 format('Problem closing file')
      call wait
      return
      end

      subroutine qhelp
c     ----------------
c
c     Written by T. Stephen 3/93
c
      common/files/nin,nout,ndebug
      write(nout,1000)
1000  format(///////////////////////////////)
      write(nout,1010)
1010  format('Quattro help:',//,'This option allows you to save ',
     +'output that ',/,'can be read by the Quattro speadsheet.',//)
      write(nout,1020)
1020  format('Warning: ',/,
     +'Saving overwrites, so change filename between saves.',//)
      write(nout,1030)
1030  format('After saving the data, it can be retrieved by Quattro ',
     +'as follows:',/)
      write(nout,1040)
1040  format('1. With Quattro loaded, select ''Tools'' from the menu',
     +' bar.',/,'2. Select ''Import'' from the menu that pops up.',/,
     +'3. Choose ''Import Comma and "" Delimited file'' from the next',
     +' menu.',/,'4. Go to the proper directory.',/,
     +'5. Select the file name you saved it to (def.=STOQ.PRN)')
      call wait
      return
      end
c****************************************************************************
c
c
      subroutine capsuplo(ict,xxbond,iibond)
c
c     written 05/Jan/93   by Martin Kunz
c
c     this routine draws a bond-stick pseudoplot of a coordination-
c     polyhedron with a given central atom (ict). The nearest neighbour bonds 
c     are passed in xxbond (<-xbond) and iibond (<-ibond). iibond(3,j)
c     represents the address of bond j in /span/. xxbond and iibond are
c     obtained through subroutine allcosphr in spanner.for
c
c     calls mathlib: inerr
c           iolib  : drawli
c           plotlib: ortho
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      common /cell/   a(3),al(3),vol,am(3,3),icell
      include 'comatom.for'
      include 'combond.for'
      common /span/  nya,mya,iyatom(5,600),iterm(600),nbo,mbo,
     1               ibond(2,600),xbond(6,600),iat,bondlim(2),
     2               netind(20),ifbond(4,600)
      common /oatom/ nyo,myo,oyx(3,1000)
c
      dimension gfbnd(600),gplot(24,70),gtext(24)
      dimension gax(3),mt(3,20),ispac(3)
      dimension hlab(20),bbten(3,3),rotm(3,3)
      dimension xxbond(6,20),iibond(3,20)
c
c       the dimensions in the next two statements are 'my'

      data (gax(i),i=1,3)/'X       ','Y       ','Z       '/
      data gplus,gzero,gminus/'+       ','o       ','-       ' /
c---------------------------------------------------------------------------   
c set start parameters; set central atom to origin:      
c---------------------------------------------------------------------------
      ny=0
      do 100 i=1,3
  100  yx(i,1)=0.0
c----------------------------------------------------------------------------
c  setup plot array 'gplot':      
c----------------------------------------------------------------------------
      do 105 i=1,24
       do 105 j=1,70
  105  gplot(i,j)=' '
      ny=ny+1
c-----------------------------------------------------------------------------
c  transformation of /span/-data into /yatom/ array:      
c-----------------------------------------------------------------------------
      do 110 i=1,icn(ict)
       ny=ny+1
       do 120 j=1,3
120     yx(j,ny)=xxbond((j+3),i)
c----------------------------------------------------------------------------
c    transforms bond-label from /span/ into a displayable character.       
c----------------------------------------------------------------------------
       gfbnd(i)=gint(iibond(3,i))
c--------------------------------------------------------------------------
c  filling of plot label-strings       
c--------------------------------------------------------------------------
       hlab(1)(1:2)=gelem(iyatom(1,iibond(1,i)))(1:2)
       hlab(1)(3:6)=gdent(iyatom(1,iibond(1,i)))(1:4)
       hlab(ny)(1:2)=gelem(iyatom(1,iibond(2,i)))(1:2)
       hlab(ny)(3:6)=gdent(iyatom(1,iibond(2,i)))(1:4)
       hlab(ny)(7:7)='('
       hlab(ny)(8:10)=gfbnd(i)(6:8)
       hlab(ny)(11:11)=')'
  110 continue
c---------------------------------------------------------------------------
c  transformation of atoms in /yatom/ into a orthogonal coordinate     
c  system. Orthogonal coordinates are stored in array /oatom/.
c---------------------------------------------------------------------------
      call ortho(bbten,rotm)
      nyo=ny
  135 write(nout,1000)
 1000 format(//,'Enter the PROJECTION axis (X, Y or Z):',//,
     1       'Note: X, Y, Z refer to orthogonal axes, where',/,
     2       'Z is parallel c and Y is perpendicular to a.')
      read(nin,1080) gaxis
      if((gaxis.eq.'x').or.(gaxis.eq.'X')) then
	naxis=1
      else if((gaxis.eq.'y').or.(gaxis.eq.'Y')) then
	naxis=2
      else if((gaxis.eq.'z').or.(gaxis.eq.'Z')) then
	naxis=3
      else 
	call inerr(*135)
      endif
  150 write(nout,1020)
 1020 format(//,'Enter the HORIZONTAL axis (X, Y or Z):',//,
     1       'Note: X, Y, Z refer to orthogonal axes, where',/,
     2       'Z is parallel c and Y is perpendicular to a.')
      read(nin,1080) gaxis
      if((gaxis.eq.'x').or.(gaxis.eq.'X')) then
	iacros=1
      else if((gaxis.eq.'y').or.(gaxis.eq.'Y')) then
	iacros=2
      else if((gaxis.eq.'z').or.(gaxis.eq.'Z')) then
	iacros=3
      else 
	call inerr(*150)
      endif
      if(naxis.eq.iacros) then
       call inerr(*150)
      endif
      idown=6-naxis-iacros
c---------------------------------------------------------------------------
c determine xmax und xmin      
c---------------------------------------------------------------------------
      do 160 i=1,nyo
       do 170 j=1,3
	if(i.eq.1) then
	 xmax=oyx(j,i)
	 xmin=oyx(j,i)
	endif
	 if(oyx(j,i).gt.xmax) then
	  xmax=oyx(j,i)
	 endif
	 if(oyx(j,i).lt.xmin) then
	  xmin=oyx(j,i)
	 endif
  170  continue
  160 continue
c----------------------------------------------------------------------------
c  determination of the molecule/screen ratio:      
c----------------------------------------------------------------------------
      delx=xmax-xmin
      ispac(idown)=integer((delx/18.),3)
      ispac(iacros)=integer((delx/36.),3)
      do 190 i=1,nyo
c----------------------------------------------------------------------------
c  setup of the screen coordinates:       
c----------------------------------------------------------------------------
       mt(idown,i)=integer(oyx(idown,i),3)
       mt(iacros,i)=integer(oyx(iacros,i),3)
       mt(idown,i)=mt(idown,i)/ispac(idown)
       mt(iacros,i)=mt(iacros,i)/ispac(iacros)
       mt(idown,i)=mt(idown,i)+12
       mt(iacros,i)=mt(iacros,i)+30
  190 continue
c----------------------------------------------------------------------------
c  setup of plot frame into plot-array 'gplot':    
c----------------------------------------------------------------------------
       do 200 i=1,70
  200  gplot(24,i)='-'
      gplot(24,2)='<'
      gplot(24,70)='>' 
      gplot(24,30)='O'
      do 210 i=1,24
  210  gplot(i,1)='|'
      gplot(1,1)=''
      gplot(12,1)='O'
      gplot(24,1)=''
c----------------------------------------------------------------------------
c   positioning of atoms within plotarray:      
c---------------------------------------------------------------------------
      do 220 i=1,nyo
       gplot(mt(idown,i),mt(iacros,i))=''
c---------------------------------------------------------------------------
c   determination of 'bond-character' for bond plot:      
c---------------------------------------------------------------------------
       if(i.eq.1) go to 220
       if(oyx(naxis,i).lt.0) then 
	gchar=gminus
       else if (oyx(naxis,i).gt.0) then
	gchar=gplus
       else if (oyx(naxis,i).eq.0) then
	gchar=gzero
       endif
c----------------------------------------------------------------------------
c   draws pseudoline for bonds into plot array:      
c---------------------------------------------------------------------------
       call drawli(mt(1,1),mt(1,i),gplot,gchar,iacros,idown)
  220 continue
c----------------------------------------------------------------------------
c   positioning of atom-labels within plot-array:  
c----------------------------------------------------------------------------
      do 225 i=1,nyo
       iflag=0
       do 213 j=1,8 
c----------------------------------------------------------------------------
c   tries to avoid overwriting of previously written atom-labels
c----------------------------------------------------------------------------
	if((gplot(mt(idown,i),(mt(iacros,i)+j)).ne.(' ')).and.   
     1     (gplot(mt(idown,i),(mt(iacros,i)+j)).ne.('+')).and.
     2     (gplot(mt(idown,i),(mt(iacros,i)+j)).ne.('-')).and.
     3     (gplot(mt(idown,i),(mt(iacros,i)+j)).ne.('o'))) then
	 iflag=1
	endif
  213  continue      
       if(iflag.eq.1) then
	mt(idown,i)=mt(idown,i)+1
       endif
       do 215 j=1,11
  215   gplot(mt(idown,i),(mt(iacros,i)+j))=hlab(i)(j:j)
  225 continue
c----------------------------------------------------------------------------
c    writes plot-labels into plot-array:      
c---------------------------------------------------------------------------
      do 230 i=1,24
  230  gtext(i)=' ' 
      gtext(1)='Across: '
      gtext(1)(8:8)=gax(iacros)(1:1)
      gtext(2)='Vertical'
      gtext(3)='Axis:   '
      gtext(3)(8:8)=gax(idown)(1:1)
      gtext(4)='Axis    '
      gtext(5)='origin:O'
      gtext(6)='Bonds:  '
      gtext(7)='below   '
      gtext(8)='plane:  '
      gtext(9)='--------'
      gtext(10)='above   '
      gtext(11)='plane:  '
      gtext(12)='++++++++'
      gtext(13)='in plane'  
      gtext(14)='oooooooo'              
      gtext(15)='        '
      gtext(16)='Numbers '
      gtext(17)='in par. '
      gtext(18)='=bond-  '
      gtext(19)='   label'
      gtext(20)='Hit     '
      gtext(21)='<RETURN>'
      gtext(22)='to cont.'
c----------------------------------------------------------------------------
c   writes plot-array 'gplot' on screen:      
c----------------------------------------------------------------------------
      do 240 i=1,23
       write(nout,1030) (gplot(i,j),j=1,70),gtext(i)
 1030  format(70A1,1X,A8)
  240 continue   
      if(machine.eq.1) assign 1040 to nformat
      if(machine.eq.2) assign 1050 to nformat
      if(machine.eq.3) assign 1060 to nformat
      if(machine.eq.4) assign 1070 to nformat
      if(machine.eq.5) assign 1070 to nformat
      write(nout,nformat) (gplot(24,j),j=1,70),gtext(24)(1:7)
 1040 format('$',70A1,1X,A7)
 1050 format(/,'$',70A1,1X,A7)
 1060 format('+',70A1,1X,A7)
 1070 format(70A1,1X,A7)
      read(nin,1080,err=999) gdum
 1080 format(A1)
  999 return
      end
c************************************************************************
c
c
      subroutine drawli(mt,nt,gplot,gchar,iacros,idown)
c
c     calls integer
c
c     draws a line of =gchar= from position =mt= to position =nt=
c     into =gplot=
c
c     written by D. Altermatt
c     McMaster university, feb. 1985
c
      implicit character*8 (g)
c
      common /files/ nin,nout,ndebug,nread,nwrite,nspgr,
     1               machine,nterm
      dimension gplot(24,70),mt(3),nt(3),gax(3)
      data gax/'x','y','z'/
c
      n1=mt(iacros)
      n2=nt(iacros)
      n3=mt(idown)
      n4=nt(idown)
      id1=iabs(n2-n1)
      id2=iabs(n4-n3)
      idir1=1
      if(n2.lt.n1) idir1=-1
      idir2=1
      if(n4.lt.n3) idir2=-1
      idir=idir1*idir2
      if(ndebug.ne.0) write(ndebug,2000) n1,n3,n2,n4,idir1,gax(iacros)
     1    ,idir2,gax(idown),gchar,gchar,gchar
2000  format(' ++drawli++3000++ (',I3,',',I3,') to (',I3,',',I3,'), ',
     1    'directions:',I3,'*',A1,',',I3,'*',A1,', type:',3A1)
      a=id1
      b=id2
      if(id1.eq.id2.and.id2.eq.0) goto 40
      if(id2.gt.id1) goto 20
      do 10 i=n1,n2,idir1
	j=i-n1
	n5=n1+j
	n6=n3+idir*integer((j*b/a),0)
	if(gplot(n6,n5).eq.' ') gplot(n6,n5)=gchar
10      continue
      goto 40
20    do 30 i=n3,n4,idir2
	j=i-n3
	n5=n3+j
	n6=n1+idir*integer((j*a/b),0)
	if(gplot(n5,n6).eq.' ') gplot(n5,n6)=gchar
30      continue
40    return
      end
