C
C                        FILE MAINLB      Last update:   29 Nov 90
C                                                        S. Healy
c
C     THIS FILE CONTAINS THE FOLLOWING ROUTINES:
C
C  S RESETA              RESETS MOST COMMON BLOCK CONTENTS FOR A NEW RUN
C  S RESETB              RESETS THE ADDITIONAL COMMON BLOCKS OF =CRYST=
C  S RESETC              RESETS THE ADDITIONAL COMMON BLOCKS OF =SCFSLB=
C  S PARAM               COMPUTES METRIC TENSOR AND INVERSE FROM LATTICE CONST.
C  S MATRIX(....)        SLAVE OF PARAM
C  S SYMD(S,T,NSYM,MSYM) DOUBLES NUMBER OF SYM.OPS. DUE TO CENTRE OF SYMMETRY
C  S SPACRD              CONVERTS HERMANN-MAUGUIN TO HALL SYMBOL SEARCHING
C                        THROUGH SPACE GROUP FILE ON UNIT 'NSPGP'
C  S CENT(GLAT)          PRINTS CENTRING TRANSLATIONS IN TABLE FORM
C  S SGHALL              DERIVES SYMMETRY OPERATORS FROM HALL SYMBOL
C  S SYMRD               READS ALPHANUMERIC SYMMETRY OPERATORS
C  S SYMXYZ(NSYM,S,T)    PRINTS THE SYM.OPS. IN (X,Y,Z)-FORM
C *S LOSYM               TRANSFORMS SPECIFIED VOLUME OF CRYSTAL TO S.G. P1
C  S CDNX                CHECKS JOURNAL CODEN
C  S CALCY(XMAX,XMIN)    FILLS COMMON /YATOM/ WITH ALL ATOMS IN COMMON /INDEX/
C                        WITHIN THE LIMITS (XMAX(3),XMIN(3))
C  S CALYAT(I,XMIN,XMAX) AS ABOVE, BUT ONLY FOR ATOM =I=
C  S CALAT(I,XXX,XMIN,XMAX) AS CALCY BUT FOR ATOM AT XXX
C  S STORE(XS,IS,IA,L)   STORES COORDINATES IN COMMON /YATOM/
C  S REMOTE(X,IS,DISA,DISIG) FINDS POINT X FURTHEST FROM SYM. IMAGES OF X
c  S UEQ(UU,DUU,I)       UU(DUU) IS ISOTROPIC DISPLACEMENT PARAM OF ATOM I
C  S TEMPER              TEST IF ADP IS ANISOTROPIC AND CALLS =DIAG=
C  F ZEDSQ(X,U)          RETURNS SQUARE OF RMS OF ADP(U) IN X DIRECTION
C
C  * CONTAINS INTERACTIVE CODE
C
C    EXTERNALS CALLED FROM THIS FILE:
C
C  FOR-COS, FOR-SIN
C
C  MATHLB: ARCOS, INTEGER, JTEST, MXV, SQART, VPLUSV,diag,gint,dist,upcase
c
c
c
c
c
c
      subroutine reseta
c
c     resets common blocks for a new run
c
c     written from previous programs by D. Altermatt
c     McMaster University, July 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /dens/   zform,dens,ddens
c PROGRAMMER'S NOTE:  /bost/ has been changed to /bostt/
c                     for Unix system.
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),
     1                pbost(5,30)
      common /gbost/  gbost(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /val/    xm(3,3),pmap(3),dmax,rrmax,jt,nz1
      common /gval/   gor(3),gem(20),gdm(20)
      common /gmenu/  hmenu(20,3)
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600)
      common/gcom/    gcom(20,10)
      common/complex/ nc,ncom(10),xcom(3,20,10)
c
c set array sizes
c  
      msym=48
      msp=48
      mbost=30
      my=1000
      mya=600
      mbo=600
c
c  reset variable parameters
c
      icell=0
      da=0.
      db=0.
      dc=0.
      as=0.
      nsym=0
      nsp = 0
      hform(1)='          '
      do 10 i=1,mx
      ialtno(i)=0
      gelem(i)='  '
      oxid(i)=0
      gdent(i)='        '
      gwyck(i)=' '
      glat=' '
      gcent=' '
      if (i.gt.80) goto 10
      hmsym(i:i)=' '
      hall(i:i)=' '
      if (i.gt.30) goto 10
      gsysp(1,i)=' '
      gsysp(2,i)=' '
10    continue
      hname=hall
      nx=0
      do 50 i=1,mx
        oxid(i)=0.
        occu(i)=0.
        soccu(i)=0.
        nh(i)=0
        nd(i)=0
        do 50 j=1,3
          xx(j,i)=0.
50    continue
c 
c     reset variables in the /temp/ common block
c
      do 11 i=1,3
        do 11 j=1,3
          do 11 k=1,100
            u(i,j,k)=0.0
            du(i,j,k)=0.0
   11       continue    
      zform=1.
      dens=0.
      nbost=0
      ny=0
      nbon=0
      ncon=0
      ind1=0
      ind2=0
      do 20 i=1,8
20      gent(i:i)=' '
      grfact=gent
      grad=gent
      jauth=0
      do 30 i=1,10
        if(i.gt.2) goto 35
        hcodn(i)=hall
        htitle(i)=hall
        hform(i)=hall
35      if(i.gt.7) goto 40
        hcomm(i)=hall
40      hauth(i)=hall
30      continue
      nya=0
      nbo=0
      nc=0
      do 51 i=1,10
        ncom(i)=0
        do 49 j=1,20
          gcom(j,i)=' '
          do 48 k=1,3
  48        xcom(k,j,i)=0.0
  49      continue
  51    continue
      return
      end
C
C
C
C
      SUBROUTINE RESETB
C
C   *****SUBROUTINE*****         RESETB
C                                ------
C
C   WRITTEN 85.5.27 BY DAVID KNIGHT
C   CALLED FROM THE CRYST CONTROL PROGRAMME
C
C   RESETS THE VALUES OF VARIABLES IN THE TEMP, RANGE,
C   SWITCH, DIST, INDEX, AND DUMMY COMMON BLOCKS
C
      IMPLICIT CHARACTER *8 (G)
C
      COMMON/RANGE/NJ,IOX(20),TOP(20,20),BOT(20,20),DDMAX,DDMIN,
     +             RMAX,RMIN,VMAX,VMIN,ANGLX
      common/grange/gel(20)
      COMMON/SWITCH/JFIX,IERRA,NOBE,NOAN,IFANG,IRANG,IBND
      COMMON/DISTT/NB,MB,IDIST(5,30),XDIST(6,30),MINBON
      COMMON/INDEX/JWRITE,JRFLAG,IRFLAG,IFFLAG,IYFLAG(100)
      COMMON/DUMMY/NXH,XNUMH(10)
      COMMON/GDUMMY/GDH(10)
      COMMON/IZFLAG/ IZFLAG
C
C   RESET VARIABLES IN THE /RANGE/ COMMON BLOCK
C
      IZFLAG=0
      NJ=0
      DDMAX=3.4
      DDMIN=0.01
      RMAX=10.0
      RMIN=0.0
      do 15 i=1,20
        gel(i)='  '
        iox(i)=0
        do 14 j=1,20
          top(i,j)=ddmax
          bot(i,j)=ddmin
   14     continue
   15   continue 
C
C   RESET VARIABLES IN THE /DIST/ COMMON BLOCK
C
      NB=0
      MB=30
C
C   RESET VARIABLES IN THE /INDEX/ COMMON BLOCK
C
      JWRITE=0
      IRFLAG=0
      IFFLAG=0
      DO 20 I=1,mx
   20   IYFLAG(I)=0
C
C   RESET VARIABLES IN THE /DUMMY/ COMMON BLOCKS
C
      NXH=0
      DO 40 I=1,10
        XNUMH(I)=0
        GDH(I)='        '
   40   CONTINUE
      RETURN
      END
c
c
c
c
c
       SUBROUTINE RESETC
C
C  *****SUBROUTINE****  RESETC
C                       ------
C  WRITTEN JUNE 13,1986 BY NINA HEINIG
C
C  RESETS THE COMMON BLOCKS INTRODUCED IN SCFSLB TO 
C  ZERO OR BLANK
C
       IMPLICIT CHARACTER*8 (G)
       IMPLICIT CHARACTER*80 (H)
C 
       COMMON /HREF/ HINFO(2,2), GFILEN(5),hist(40)
       COMMON /GCOND/ GCRAD,GIRAD,GDIFF,GABS,GREFN,GSTAT
       COMMON /COND/  CONDIT(6,8), NSTD, MSTD, CSTD(6,4)
       COMMON /HTEXT/ GCRYST(2),GTWIN(5),HTEXT(10,5),HDATA(4,19)
       COMMON /GFORM/ GFORM(24,3)
       COMMON /FORM/ NFORM(3), MFORM, FORM(24,3)
       COMMON /CRYST/ SCALE, ESCALE, EXTE(6,2), ABST,ABSTE,TVOL(5),
     *        ETVOL(5)
       data gblank/'        '/
C
       NSTD = 0
       mstd = 4
       SCALE = 0.
       ESCALE = 0.
       ABST = 0.
       ABSTE = 0.
       mform = 24
C
C  RESET 80 CHARACTER ARRAYS TO BLANK
C
       DO 30  I = 1,4
         DO 20 J =1,19
           DO 10 IJ = 1,80
             HDATA(i,J)(IJ:IJ) = ' '
             IF(J .GT. 10) GOTO 10
             HTEXT(j,I)(IJ:IJ) = ' '
             IF(I .EQ. 4) HTEXT(j,I+1)(IJ:IJ) = ' '
             IF(J .GT. 2) GOTO 10
             IF(I .GT. 2) GOTO 10
             HINFO(I,J)(IJ:IJ) = ' '
   10        CONTINUE
   20      CONTINUE
   30    CONTINUE
       do 35 i=1,80
         do 32 j=1,40
   32      hist(j)(i:i)=' '
   35    continue
C
C  RESET 8 CHARACTER ARRAYS TO BLANK (AND FORM TO ZERO)
C
       GCRAD = gblank
       GIRAD = gblank
       GDIFF = gblank
       GABS  = gblank
       GREFN = gblank
       GSTAT = gblank
       DO 50  J =1,5
         GTWIN(J) = gblank
         GFILEN(J)= gblank
         IF( J .GT. 3) GO TO 50
         DO 40  IJ=1,24
           GFORM(IJ,J)= gblank
           FORM(IJ,J) = 0.
   40      CONTINUE
   50    CONTINUE          
       do 60 j=1,2
         gcryst(j)=gblank
   60    continue
C
C  RESET THE NUMERICAL ARRAYS TO ZERO
C
       DO 80  I=1,6
         DO 70  J=1,8      
           CONDIT(I,J) = 0.   
           IF (J .GT. 4) GOTO 70
           CSTD(I,J) = 0.
           IF (J .GT. 2) GOTO 70
           EXTE(I,J) = 0.
   70      CONTINUE
         IF(I .GT. 5) GOTO 80
         TVOL(I) = 0.
         ETVOL(I) = 0.
         IF(I .GT. 3) GOTO 80
         NFORM(I) = 0
   80    CONTINUE
       RETURN
       END 

c
c
c
c
c
C
      SUBROUTINE PARAM
C
C *****SUBROUTINE*****          PARAM
C                               -----
C
C WRITTEN 80.6.1 BY DOUG THIERS
C REVISED 81.3.26 BY I D BROWN
C
C  CALLS MATRIX, ARCOS,sqart
C
C  SUBROUTINE TO COMPUTE METRIC TENSOR AND INVERSE LENGTHS FROM
C  THE LATTICE PARAMETERS;ALSO THE ERRORS IN THESE.
C  PARAM IS CALLED WHENEVER LATTICE PARAMETERS ARE READ
C
      common/files/nin,nout,ndebug
      COMMON/CELL/ A,B,C,AL,BE,AGA,VOL,AM(3,3),ICELL
      COMMON/ECELL/ DA,DB,DC,DAL,DBE,DGA,DM(9,6)
      COMMON/RCELL/ AS,BS,CS,RAL,RBE,RGA,RVOL,RM(3,3)
      CALL MATRIX(A,B,C,AL,BE,AGA,AM)
      COS2A=COS(AL)**2
      COS2B=COS(BE)**2
      COS2G=COS(AGA)**2
      COSA=COS(AL)
      COSB=COS(BE)
      COSG=COS(AGA)
      SINA=SIN(AL)
      SINB=SIN(BE)
      SING=SIN(AGA)
      ARG= 1. - COS2A - COS2B - COS2G + 2.*COSA*COSB*COSG
      VOL=A*B*C*(sqart(arg))
      if(vol.lt.0.0001) then
        write(nout,1000)
 1000   format(' **ERROR**PARAM** Unit cell has zero volume')
        call wait
        return
        endif
      AS=B*C*SINA/VOL
      BS=A*C*SINB/VOL
      CS=B*A*SING/VOL
C
C  CALCULATE INVERSE CELL ANGLES
C
      RGA = ARCOS((COSA*COSB-COSG)/(SINA*SINB))
      RBE = ARCOS((COSA*COSG-COSB)/(SINA*SING))
      RAL = ARCOS((COSB*COSG-COSA)/(SINB*SING))
      CALL MATRIX(AS,BS,CS,RAL,RBE,RGA,RM)
      CALL MATRIX(AS,BS,CS,RAL,RBE,RGA,RM)
C   ***THIS STATEMENT WAS REPEATED TO OVERCOME A BUG IN THE CDC***
C
C  CALCULATE ERRORS IN METRIC TENSOR
C
      CALL MATRIX(A+DA,B,C,AL,BE,AGA,DM(1,1))
      CALL MATRIX(A,B+DB,C,AL,BE,AGA,DM(1,2))
      CALL MATRIX(A,B,C+DC,AL,BE,AGA,DM(1,3))
      CALL MATRIX(A,B,C,AL+DAL,BE,AGA,DM(1,4))
      CALL MATRIX(A,B,C,AL,BE+DBE,AGA,DM(1,5))
      CALL MATRIX(A,B,C,AL,BE,AGA+DGA,DM(1,6))
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      SUBROUTINE MATRIX(A,B,C,AL,BE,AGA,X)
C
C  *****SUBROUTINE*****             MATRIX
C                                   ------
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  THIS SUBROUTINE CREATES THE METRIC TENSOR OF THE UNIT CELL
C
      DIMENSION X(9)
      X(1)=A**2
      X(2)=A*B*COS(AGA)
      X(4)=X(2)
      X(3)=A*C*COS(BE)
      X(7)=X(3)
      X(5)=B**2
      X(6)=B*C*COS(AL)
      X(8)=X(6)
      X(9)=C**2
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      SUBROUTINE SYMD(S,T,NSYM,MSYM)
C
C  *****SUBROUTINE*****           SYMD
C                                 ----
C
C WRITTEN 80.6.1 BY DOUG THIERS
C
C SUBROUTINE TO DOUBLE THE NUMBER OF SYM.OP.S FOR CENTROSYMETRIC S.G.S
C
      DIMENSION S(3,3,MSYM),T(3,MSYM)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      DO 5 I=1,NSYM
      IF(NSYM+I.LE.MSYM) GOTO 10
      WRITE(NOUT,100)
  100 FORMAT(40H ERROR IN SUBROUTINE:SYMD.MORE SYM OP.S
     1   ,40HCALCULATED THAN ALLOWED(48).                             )
      RETURN
   10 DO 15 J=1,3
      DO 25 K=1,3
   25 S(J,K,NSYM+I)=-S(J,K,I)
   15 T(J,NSYM+I)=T(J,I)
    5 CONTINUE
      NSYM=2*NSYM
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      SUBROUTINE SPACRD
C
C  *****SUBROUTINE*****           SPACRD
C                                 ------
C
C WRITTEN 80.6.1 BY DOUG THIERS / 84.6.11 D. ALTERMATT
C
C THIS PROGRAM SEARCHES THROUGH THE SPACE GROUP FILE TO
C CONVERT A KNOWN HERMANN-MAUGIN SYMBOL INTO THE CORRESPONDING
C HALL SYMBOL
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      INTEGER TEST
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      REWIND NSPGP
      call upcase(hmsym)
    1 READ(NSPGP,100,END=10000) HMS,HHALL
  100 FORMAT(BZ,5X,A24,A15)
      GOTO 40
10000    WRITE(NOUT,130)hmsym,hfilen(nspgp)
  130    FORMAT('*ERROR*spacrd*130* space group',A8,' not found on file',
     1   A10,' before symmetry operators read') 
         RETURN
   40 TEST = 1
      DO 3 M=1,24
         M1=M
         IF(HMSYM(M:M).NE.' ') GOTO 4
    3 CONTINUE
    4 M2=M1+23
      IF(HMSYM(M1:M2).NE.HMS(1:24)) TEST=0
         IF(TEST.NE.1) GOTO 1
      IF(HALL(1:8).NE.'        ') RETURN
      HALL=HHALL
      WRITE(NOUT,1610) HALL,HMSYM
 1610 FORMAT(' HALL SPACE GROUP SYMBOL FOUND = ',A15,' FOUND FOR ',
     1       A24,/)
      RETURN
      END
C
C
C
C
      SUBROUTINE CENT(GLAT)
C  *****SUBROUTINE*****           CENT
C                                 ----
C
C WRITTEN 80.6.1 BY DOUG THIERS / 84.6.12 D. ALTERMATT
C
C AS OF JUNE, 1985 THIS SUBROUTINE MAY NOT BE CALLED FROM ANYWHERE
C
C PRINTS CENTERING TRANSLATIONS IN TABLE FORM
C
      IMPLICIT CHARACTER*8 (G)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      WRITE(NOUT,100)
  100 FORMAT(/,'0 LATTICE CENTRING TRANSLATIONS  ',/)
      IF(GLAT.EQ.'P') WRITE(NOUT,110)
  110 FORMAT(30H  P - NO TRANSLATIONS                           )
      IF(GLAT.EQ.'A'.OR.GLAT.EQ.'F') WRITE(NOUT,120)
      IF(GLAT.EQ.'B'.OR.GLAT.EQ.'F') WRITE(NOUT,130)
      IF(GLAT.EQ.'C'.OR.GLAT.EQ.'F') WRITE(NOUT,140)
      IF(GLAT.EQ.'I') WRITE(NOUT,150)
      IF(GLAT.EQ.'H') WRITE(NOUT,160)
      RETURN
  120 FORMAT(55H  A - COORDINATES TRANSLATED BY  (   0  , +1/2 , +1/2 ))
  130 FORMAT(55H  B - COORDINATES TRANSLATED BY  ( +1/2 ,   0  , +1/2 ))
  140 FORMAT(55H  C - COORDINATES TRANSLATED BY  ( +1/2 , +1/2 ,   0  ))
  150 FORMAT(55H  I - COORDINATES TRANSLATED BY  ( +1/2 , +1/2 , +1/2 ))
  160 FORMAT(55H  R - COORDINATES TRANSLATED BY  ( +1/3 , +2/3 , +2/3 ),
     1/ ,    55H                         AND BY  ( +2/3 , +1/3 , +1/3 ))
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      SUBROUTINE SGHALL
*
*
* WRITTEN BY DAVID R. MOSSCROP (7901822)
* MAY-JUNE ; 1982.
*
* FROM AN ARTICLE AND PROGRAM: NEW SPACE GROUP NAMES.
*      BY HALL 1981  REF:ACTA CRYST .A37, 517-523.
*
* CALLS SYMD, SYMXYZ
*
* THIS PROCEDURE USES THE HALL SPACE GROUP NAME TO GENERATE
* THE CORRESPONDING MATRIX TRANSFORMATIONS.
C
C   MODIFIED 84.2.14 BY I D BROWN TO ALLOW ORIGIN SHIFT
C   UPDATED BY D. ALTERMATT
*
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      INTEGER OLD
      INTEGER ROT(12,20)
      INTEGER SMAT(16,50)
      INTEGER SVEC(3)
      DIMENSION ISHIFT(3),GCHR(30)
      INTEGER TAB(3,9)
      INTEGER VEC(3,8)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /SYM/ NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON /GSYM/ GLAT,GCENT,HMSYM,HALL
*
* VARIABLE DICTIONARY:
*
* GCENT IS CENTROSYMMETRIC FLAG (A OR C)
* GLAT IS LATTICE FLAG (P,R,A,B,C,I,F).
* IAXS IS CURRENT AXIS POINTER TO TAB.
* ISGN IS MATRIX SIGN FLAG.
* IROT IS CURRENT ORDER POINTER TO TAB.
* JAXS IS PREVIOUS AXIS POINTER TO TAB.
* JROT IS PREVIOUS ORDER POINTER TO TAB.
* NIN IS INPUT DEVICE NUMBER.
* NOUT IS OUTPUT DEVICE NUMBER.
* NSYM IS NUMBER OF EQUIVALENT POSITIONS.
* NGEN IS NUMBER OF GENERATED SEITZ MATRICES.
* NROT IS CURRENT ROTATION ORDER (1,2,3,4,6).
* NEW IS POSITION OF MATRIX PRODUCT.
* OLD IS MATRIX COUNT BEFORE INNER LOOP.
* ROT(12,20) IS TABLE OF ROTATION MATRICES.
* SMAT(16,50) IS SEITZ MATRIX ARRAY.
* SVEC(3) IS TEMPORARY T-VECTOR ARRAY.
* ISHIFT IS THE ORIGIN SHIFT
* TAB(3,9) POINTS TO ROT(AXIS,ORDER).
* VEC(3,8) IS THE T-VETORS FOR A,B,C,N,U,V,W,D.
*
******************************************************************
* LOADING INITIAL DATA
*
      DATA ROT/
     1 1,0,0,0, 0,1,0,0, 0,0,1,0,
     2 1,0,0,0, 0,-1,0,0, 0,0,-1,0,
     3 -1,0,0,0, 0,1,0,0, 0,0,-1,0,
     4 -1,0,0,0, 0,-1,0,0, 0,0,1,0,
     5 1,0,0,0, 0,0,1,0, 0,-1,-1,0,
     6 -1,0,-1,0, 0,1,0,0, 1,0,0,0,
     7 0,1,0,0, -1,-1,0,0, 0,0,1,0,
     8 1,0,0,0, 0,0,1,0, 0,-1,0,0,
     9 0,0,-1,0, 0,1,0,0, 1,0,0,0,
     1 0,1,0,0, -1,0,0,0, 0,0,1,0,
     2 1,0,0,0, 0,1,1,0, 0,-1,0,0,
     3 0,0,-1,0, 0,1,0,0, 1,0,1,0,
     4 1,1,0,0, -1,0,0,0, 0,0,1,0,
     5 0,0,1,0, 1,0,0,0, 0,1,0,0,
     6 -1,0,0,0, 0,0,-1,0, 0,-1,0,0,
     7 0,0,-1,0, 0,-1,0,0, -1,0,0,0,
     8 0,-1,0,0, -1,0,0,0, 0,0,-1,0,
     9 -1,0,0,0, 0,0,1,0, 0,1,0,0,
     1 0,0,1,0, 0,-1,0,0, 1,0,0,0,  0,1,0,0, 1,0,0,0, 0,0,-1,0/
*
      DATA GCHR/'1','2','3','4','5','6','P','R','A','B','C','I','F',
     1'N','U','V','W','D',' ','-','X','Y','Z','*','''','"','(','0','H',
     2'H'/
*
      DATA TAB/1,1,1,2,3,4,5,6,7,8,9,10,0,0,0,11,12,13,
     1  14,14,14,15,16,17,18,19,20/
*
      DATA VEC/6,0,0, 0,6,0, 0,0,6, 6,6,6,
     1  3,0,0, 0,3,0, 0,0,3, 3,3,3/
*
* VARIABLE INITIALIZATION.
*
      JCENT=0
      GCENT=GCHR(9)
      GLAT=' '
      NGEN=1
      NSYM=1
      ISGN=12
      NLT=1
*
      NROT=0
      LLTCODE=0
      IROT=0
      JROT=0
      IAXS=0
      JAXS=0
*
******************************************************************
* THE PROGRAM: SGHALL
*
* ARRAY INITIALIZATION.
*
*
* ZERO VECTOR ACCUMULATOR.
*
      DO 1 I=1,3
      SVEC(I)=0
      TU(I,NLT)=0.0
    1 CONTINUE
*
* SET THE (4,4) ELEMENT TO 1.0.
*
      DO 2 I=1,50
      SMAT(16,I)=12
      SMAT(4,I)=0
      SMAT(8,I)=0
      SMAT(12,I)=0
    2 CONTINUE
*
* SET ORIGIN SHIFT TO ZERO
*
      DO 30 I=1,3
   30   ISHIFT(I)=0
*
* SET IDENTITY ROTATIONS.
*
      DO 3 I=1,12
      SMAT(I,1)=ROT(I,1)*12
    3 CONTINUE
*
* SET IDENTITY TRANSLATIONS.
*
      DO 4 I=13,16
      SMAT(I,1)=ROT(I-10,1)*12
    4 CONTINUE
*
* CHECKING ELEMENT BY ELEMENT AND SET VARIABLES AS NEEDED.
*
      DO 66 N=1,15
      DO 6 M=1,29
      IF (HALL(N:N).EQ.GCHR(M)) GO TO 606
    6 CONTINUE
*
* ERROR CONDITION CHARACTER NOT IDENTIFIED.
*
      WRITE(NOUT,1670) HALL(N:N),N
 1670 FORMAT(' ++ERROR++SGHALL++1670++ ILLEGAL CHARACTER: ',A1,
     1       ' IN COLUMN ',I1)
*
* SKIP INACTIVE BLANKS.
*
  606 IF ((M.EQ.19).AND.(NROT.EQ.0)) GO TO 66
C
C  BLANK INITIATES CONSTRUCTION OF SYMMETRY GENERATOR
C
      IF ( M.EQ.19 ) GO TO 1011
C
C PARENTHESIS INITIATES READING OF ORIGIN SHIFT
C
      IF(M.EQ.27) GO TO 40
*
*
* SET GCENT AND GLAT, DEPENDENT ON FIRST ELEMENT(S).
*
      IF ( LLTCODE .NE. 0) GO TO 1000
*
* FIRST NEGATIVE SIGN INDICATES CENTRE.
*
      IF (M .NE. 20) GO TO 1001
      JCENT=1
      GCENT=GCHR(11)
      GO TO 66
*
* ALLOWED CHARACTERS: P,R,A,B,C,I,F TO SET LTCODE.
*
 1001 IF ((M .GE. 7).AND.(M .LE.13)) GO TO 2000
      WRITE(NOUT,1600) GCHR(M)
 1600 FORMAT(' ++ERROR++SGHALL++1600++ UNRECOGNISED LATTICE TYPE ',
     1 A1)
      GO TO 150
 2000 CONTINUE
      LLTCODE=M-6
      GLAT=GCHR(M)
*
* SPACE GROUPS WITH NAMES 'R' TYPE ARE ASSIGNED
* GLAT 'H' FOR A HEXAGONAL SETTING. THIS IS A
* LOCAL CONVENTION.
*
      IF ( M .EQ. 8 ) GLAT=GCHR(30)
*
* ADD CENTERING TRANSLATIONS TO =TU=
*
      IF(GLAT.EQ.'P') GOTO 660
      IF(GLAT.EQ.'I') THEN
        NLT=NLT+1
        DO 60 I=1,3
   60   TU(I,NLT)=0.5
        GOTO 660
        ENDIF
      IF(GLAT.EQ.'A'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.0
        TU(2,NLT)=0.5
        TU(3,NLT)=0.5
        IF(GLAT.EQ.'A') GOTO 660
        ENDIF
      IF(GLAT.EQ.'B'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.5
        TU(2,NLT)=0.0
        TU(3,NLT)=0.5
        IF(GLAT.EQ.'B') GOTO 660
        ENDIF
      IF(GLAT.EQ.'C'.OR.GLAT.EQ.'F') THEN
        NLT=NLT+1
        TU(1,NLT)=0.5
        TU(2,NLT)=0.5
        TU(3,NLT)=0.0
        GOTO 660
        ENDIF
      IF(GLAT.EQ.'H') THEN
        NLT=NLT+1
        TU(1,NLT)=0.3333333
        TU(2,NLT)=0.6666667
        TU(3,NLT)=0.6666667
        NLT=NLT+1
        TU(1,NLT)=0.6666667
        TU(2,NLT)=0.3333333
        TU(3,NLT)=0.3333333
        ENDIF
  660 IF(NDEBUG.NE.0) WRITE(NDEBUG,2020) ((TU(I1,I2),I1=1,3),I2=1,NLT)
 2020 FORMAT(/,' ++DEBUG++SGHALL++2020++  TU(3,NLT)',4(/,26X,3F5.2))
      GOTO 66
*
* IF CHARACTER IS A NEGATIVE SIGN, SET ISGN TO NEGATIVE,
* OTHERWISE POSITIVE. THE LATER NEGATIVE SIGNS INDICATE
* INVERSIONS.
*
 1000 IF (M.NE.20) GO TO 999
      ISGN=-12
      GO TO 66
*
* ROTATION MUST FOLLOW LATTICE TYPE.
*
*
* IF ROTATION AXIS ALREADY SET GO TO 1003.
*
  999 IF (NROT .NE. 0) GO TO 1003
      IF (M .LE.6) GO TO 2011
      WRITE(NOUT,2010) GCHR(M)
 2010 FORMAT(' ++ERROR++SGHALL++2010++ ILLEGAL ROTATION CHARACTER '
     1 ,A1)
      GO TO 150
 2011 CONTINUE
*
* SET ROTATION AXIS IN NROT.
*
      IF ((M .EQ. 1) .AND. (ISGN .LT.0)) NSYM=NSYM*2
      NROT=M
      IROT=M
      IF ((M.NE.1).OR.(ISGN.LT.0)) NGEN=NGEN+1
      IF(NGEN.LE.MSYM) GO TO 1005
      WRITE(NOUT,1501)
 1501 FORMAT(' ++ERROR++SGHALL++1501++ NGEN GREATER THAN MSYM.')
      NSYM=0
      RETURN
*
*********** SELECT AXIS EXPLICILY **************
*
* IF CHARACTER DOES NOT SET AXIS GO TO 1005.
*
 1003 IF (M .LE. 20) GO TO 1009
      IAXS=M-20
      GO TO 1004
*
* SET AXIS IMPLICITLY.
*
*
*  IF DEFAULT AXIS NOT REQUIRED GO TO 1004.
*
 1005 CONTINUE
*
*  FIRST AXIS ASSUMED Z.
*
      IF ((NGEN .NE.2).AND.(IROT .NE.1)) GO TO 1006
      IAXS=3
      GO TO 1004
*
* SECOND AXIS ASSUMED X IF NROT=2 AND PREVIOUS AXIS WAS 2 OR 4.
*
 1006 IF ((NROT .NE.2) .OR. (JROT .NE.2)) GO TO 1007
      IAXS=1
      GO TO 1004
*
* AXIS AFTER 3,6 ASSUMED AS X-Y.
*
 1007 IF (( NROT .NE.2) .OR.(JROT .NE.3))  GO TO 1008
      IAXS=5
      GO TO 1004
 1008 IF (NROT .EQ.3) GO TO 2500
      WRITE(NOUT,1639)
 1639 FORMAT(' ++ERROR++SGHALL++1639++ UNABLE TO ASSIGN IMPLICIT ',
     1 'AXIS.')
      GO TO 150
 2500 IAXS=4
*
*
* FOR NON-PRINCIPAL AXES.
*
 1004 IF (IAXS .LE.3) GO TO 66
      IROT=IAXS+3
      IAXS=JAXS
      IF((IAXS.EQ.0).AND.(IROT.EQ.7))IAXS=1
      GO TO 66
*
********** SPECIFY TRANSLATION ***********
*
*
* ONLY FOR X,Y,Z DIRECTIONS.
*
 1009 IF(M .GT. 6) GO TO 1010
      IF (IROT .GT.M) GO TO 2510
      WRITE(NOUT,1660) GCHR(M)
 1660 FORMAT(' ++ERROR++SGHALL++1660++ ILLEGAL SCREW AXIS SYMBOL ',
     1 A1)
      GO TO 150
 2510 CONTINUE
*
* CALCULATE TRANSLATIONS IN 12 THS.
*
      SVEC(IAXS)=SVEC(IAXS)+M*12/NROT
*
* TEST FOR A,B,C,N,U,V,W,D.
*
 1010 IF(((M.LE.8).OR.(M.GE.12)).AND.((M.LE.13).OR.(M.GE.19)))GOTO1011
      IF (M .GE. 12) GO TO 1111
      I=M-8
      GO TO 1112
 1111 I=M-10
 1112 DO 7 J=1,3
      SVEC(J)=SVEC(J)+VEC(J,I)
    7 CONTINUE
      GO TO 66
*
* CONSTRUCT GENERATOR WHEN BLANK IS REACHED.
*
 1011 IF (M .NE.19) GO TO 66
*
* POINT TO ROTATION MATIX.
*
      K=TAB(IAXS,IROT)
      DO 8 I=1,12
      SMAT(I,NGEN)=ROT(I,K)*ISGN
    8 CONTINUE
      DO 9 I=13,15
*
* DEPOSIT TRANSLATON BASE 12.
*
      SMAT(I,NGEN)=MOD(SVEC(I-12)+120,12)
    9 CONTINUE
*
* CALCULATE THE NUMBER OF EQUIVALENT POSITIONS.
*
      NSYM=NSYM*NROT
      IF (NROT .GT. 3) NROT=NROT/2
*
* RESET ORDER FLAGS AND AXIS FLAGS.
*
      JROT=NROT
      NROT=0
      IROT=0
      JAXS=IAXS
      IAXS=0
      ISGN=12
*
* ZERO VECTOR ACCUMULATOR.
*
      DO 10 I=1,3
      SVEC(I)=0
   10 CONTINUE
*
      IF(HALL(N:N+1).EQ.'  ') GOTO 666
      GO TO 66
C
C   READ ORIGIN SHIFT
C
   40 IAX=0
      NN=N
   41 ISIGN=1
      IAX=IAX+1
      IF(IAX.GT.3) GO TO 666
   42 NN=NN+1
      DO 45 M=1,29
   45 IF(HALL(NN:NN).EQ.GCHR(M)) GO TO 47
      WRITE(NOUT,1670) HALL(NN:NN),NN
      GO TO 150
   47 IF(M.EQ.20) THEN
        ISIGN=-1
        GO TO 42
        ENDIF
      IF(M.EQ.28) THEN
        ISHIFT(IAX)=0
        GO TO 41
        ENDIF
      IF(M.LE.6) THEN
        ISHIFT(IAX)=ISIGN*M
        GO TO 41
        ENDIF
      IF(M.EQ.29) GO TO 666
      WRITE(NOUT,200) HALL(NN:NN),NN
      NSYM=0
      RETURN
   66 CONTINUE
  666 CONTINUE
      IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,1666) GLAT,NROT,ISHIFT
        DO 667 I=1,NGEN
          WRITE(NDEBUG,1667) (SMAT(J,I),J=1,16)
  667     CONTINUE
 1666   FORMAT(' ++DEBUG++SGHALL++1666++ LATCODE=',A1,' NROT=',I3,
     1   ' ISHIFT=',3I4)
 1667   FORMAT(10X,16I4)
        ENDIF
*
* END OF INPUT DATA (HALL SPACE GROUP NAME) EVALUATION LOOPS.
*
* CHECK FOR ERROR CONDITION.
*
      IF ((NROT .LE.0).AND.(LLTCODE .NE.0)) GO TO 151
  150 CONTINUE
*
* ERROR CONDITION.
*
      WRITE(NOUT,200) HALL(N:N),N,NROT,LLTCODE,GLAT,GCENT
  200 FORMAT (' ++DEBUG++SGHALL++200++  ILLEGAL CONSTRUCTION AT',
     1       ' CHARACTER ',A1,' IN COL.',I3,/,25X,'NROT =',I3,
     1       ', LLTCODE =',I3,', GLAT = ',A1,', GCENT = ',A1,/)
      NSYM=0
      RETURN
  151 CONTINUE
*
*
*GENERATE MATRICES BY PROGRESSIVE BINARY PRODUCTS.
*
*
*
* SKIP GENERATOR IF P1 OR -P1.
*
      IF (NGEN .LE.1) GO TO 117
      M=2
  723 IF(M.LE.NGEN) GO TO 273
      GO TO 105
*
* SAVE CURRENT MATRIX COUNT.
*
  273 OLD=NGEN
      N=M
  724 IF(N.LE.NGEN) GO TO 274
      GO TO 11
*
* POINT TO GENERATED MATRIX IN SMAT.
*
  274 NEW=NGEN+1
*
      DO 13 I=1,3
      DO 131 J=1,13,4
  131 SMAT(I+J-1,NEW)=((SMAT(I,M)*SMAT(J,N))+(SMAT(I+4,M)*SMAT(J+1,N))+
     1   (SMAT(I+8,M)*SMAT(J+2,N))+(SMAT(I+12,M)*SMAT(J+3,N)))/12
      SMAT(I+12,NEW)=MOD(SMAT(I+12,NEW)+120,12)
   13 CONTINUE
*
* IS MATRIX UNIQUE?
*
      LFLAG=1
      DO 14 L=1,NGEN
      DO 103 I=1,12
  103 IF (SMAT(I,L).NE.SMAT(I,NEW)) GO TO 1674
      IF (LLTCODE .NE. 1) GO TO 251
      DO 15 II=13,16
      IF (SMAT(II,L) .EQ. SMAT(II,NEW)) GO TO 15
*
* ERROR CONDITION FOUND.
*
      WRITE (NOUT,300)
  300 FORMAT('++SGHALL++ERROR: MATRICES RELATED BY PURE TRANSLATION.')
      NSYM=0
      RETURN
   15 CONTINUE
*
      GO TO 251
*
 1674 DO 16 J=1,12
   16 IF (SMAT(J,L) .NE. SMAT(J,NEW)) GO TO 20
*
* ERROR CONDITION NOT AVOIDED ( IE NOT SKIPPED OVER).
*
      WRITE(NOUT,400)
  400 FORMAT('++SGHALL++ERROR: SOME MATRICES RELATED BY INVERSION.')
      NSYM=0
      RETURN
*
   20 CONTINUE
      LFLAG=LFLAG+1
   14 CONTINUE
  251 IF (LFLAG .GT. NGEN) NGEN=NEW
      N=N+1
      GO TO 724
   11 IF ((NGEN.EQ.OLD).AND.(NGEN.GE. NSYM)) GO TO 105
      M=M+1
      GO TO 723
  105 CONTINUE
*
*
* SORT ON ZERO PATTERN.
*
      DO 17 N=2,NGEN
      DO 18 L=N,NGEN
      DO 181 K=1,12
*
* EXIST ON ZERO MISMATCH.
*
  181 IF((SMAT(K,N-1).EQ.0).AND.(SMAT(K,L).NE.0))GO TO 18
      GO TO 1171
   18 CONTINUE
      GO TO 17
*
*
* EXCHANGE SORTED ELEMENTS.
*
 1171 DO 19 K=1,16
      I=SMAT(K,L)
      SMAT(K,L)=SMAT(K,N)
      SMAT(K,N)=I
   19 CONTINUE
*
   17 CONTINUE
  117 CONTINUE
*
* ERROR CHECK OF NGEN RELATIVE TO NSYM.
*
      IF (NGEN .LE. NSYM) GO TO 1013
      WRITE(NOUT,500) NGEN,NSYM
  500  FORMAT('++SGHALL++ERROR: ',I5,' MATRICES GENERATED,',I5,
     1       ' EXPECTED')
      NSYM=0
*
* THE FOLLOWING LOOPS ENTERS THE VALUES OF SMAT(16,48),
* WHICH IS THE ARRAY USED IN THIS ROUTINE TO STORE THE
* ROTATION AND TRANSLATION MATRICES, INTO TWO COMPARABLE
* ARRAYS S(3,3,48) AND T(3,48) WHICH ARE USED IN OTHER
* ROUTIES EXTERNAL TO SGHALL.
*
 1013 DO 7775 M=1,48
      DO 7775 J=1,3
      DO 7775 I=1,3
      K=4*(J-1)+I
*
* REAL DIVISION BY ZERO IS NEEDED TO CONVERT THE 12 THS
* USED IN SGHALL INTO ROTATIONS AND TRANSLATIONS
* UNDERSTANDABLE TO ROUTINE WHICH USE S(3,3,48) AND T(3,48).
* THE VALUES IN S AND T ARE NOW 1 OR DECIMAL FRACTIONS.
*
      S(I,J,M)=SMAT(K,M)/12.0
      T(I,M)=SMAT(I+12,M)/12.0
 7775 CONTINUE
C
C  ADD ORIGIN SHIFT
C
      DO 7780 M=1,48
        DO 7777 I=1,3
          IF(S(I,I,M).LT.-0.001) THEN
            T(I,M)=T(I,M)+ISHIFT(I)/6.0
            IF(T(I,M).GT.0.99) T(I,M)=T(I,M)-1.0
            IF(T(I,M).LT.-0.01) T(I,M)=T(I,M)+1.0
            ENDIF
 7777     CONTINUE
 7780   CONTINUE
      IF(GCENT.EQ.GCHR(11)) CALL SYMD(S,T,NSYM,MSYM)
      IF(NDEBUG.NE.0) THEN
        NOUT1=NOUT
        NOUT=NDEBUG
        CALL SYMXYZ (NSYM,S,T)
        NOUT=NOUT1
        ENDIF
      RETURN
      END
C
C
C
C
       SUBROUTINE SYMRD(K,LFAIL,II,HLINE,S,T)
C
C  *****SUBROUTINE*****            SYMRD
C                                  -----
C
C WRITTEN 80.6.1 BY DOUG THIERS
C Modified 85.6.4 by David Clarkson.
C  CALLS VAL
C
C  ROUTINE TO DECODE INDIVIDUAL SYMMETRY CARDS.THE SYMMETRY IS DECODED
C  FROM ALPHANUMERIC TO A SYMMETRY MATRIX AND TRANSLATION VECTOR.
C
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8  (G)
       CHARACTER *1 BLNK,COMMA,PLUS,MINUS,XX,YY,ZZ,SLASH
       DIMENSION S(3,3,48),T(3,48)
       COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
       COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
       DATA BLNK,COMMA,PLUS,MINUS/' ',',','+','-'/
       DATA XX,YY,ZZ,SLASH/'X','Y','Z','/'/
       DO 100 I=1,3
         T(I,K)=0.0
         DO 100 J=1,3
100        S(J,I,K)=0.0
       J=1
       SIGN=1.0
       IMAX=80
5      CONTINUE
       IF(HLINE(II:II).EQ.BLNK) GOTO 999
       IF(HLINE(II:II).EQ.COMMA)GOTO 46
       IF(HLINE(II:II).EQ.PLUS) GOTO 99
       IF(HLINE(II:II).NE.MINUS)GOTO 10
       SIGN=-1.0
       GOTO 999
10     IF(HLINE(II:II).NE.XX) GOTO 20
       S(J,1,K)=SIGN
       IF(HLINE(II+1:II+1).EQ.BLNK) GOTO 46
       GOTO 99
20     IF(HLINE(II:II).NE.YY) GOTO 30
       S(J,2,K)=SIGN
       IF(HLINE(II+1:II+1).EQ.BLNK) GOTO 46
       GOTO 99
30     IF(HLINE(II:II).NE.ZZ) GOTO 40
       S(J,3,K)=SIGN
       IF(HLINE(II+1:II+1).EQ.BLNK) GOTO 46
       GOTO 99
40     I1=II+1
       IF(HLINE(I1:I1).NE.SLASH) GOTO 80
       I2=II+2
       ARG1=FLOAT(NUM(HLINE(II:II),1))
       ARG2=FLOAT(NUM(HLINE(I2:I2),1))
       TEMP=ARG1/ARG2
       T(J,K)=SIGN*TEMP
       II=II+2
       IF(HLINE(II+1:II+1).EQ.BLNK) GOTO 46
       GOTO 99
80     WRITE(NOUT,110) HLINE
110    FORMAT(' +++ ERROR +++ SYMRD+++ BAD SYNTAX IN SYMMETRY CARD.',
     1        /,' ',A80)
       LFAIL=1
       GOTO 9999
46     J=J+1
       IF(J.GT.3) GOTO 9999
99     SIGN=1.0
999    II=II+1
       IF(II-IMAX) 5,5,9999
9999   RETURN
       END
C
C
C
C
      SUBROUTINE SYMXYZ(NSYM,S,T)
*
* WRITTEN BY DAVID R. MOSSCROP (7901822)
* MAY-JUNE ; 1982.
C 84.6.12 D. ALTERMATT
C 3 JUNE 1992  C.BRODIE
* FROM AN ARTICLE AND PROGRAM: NEW SPACE GROUP NAMES.
*      BY HALL 1982  REF:ACTA CRYST .A36, 517-523.
*
* CALLS INTEGER
*
* THIS SUBROUTINE PRINTS OUT THE SPACE GROUP TRANSFORMATIONS
* IN FAMILIAR X,Y,Z COORDINATES.
*
*
* TRANS IS A ONE DIMENSIONAL ARRAY OF 30 ELEMENTS
* IN TRANS ONE TRANSFORMATION IS STORED BEFORE
* BEING PRINTED OUT IN XYZ FORM.
*
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
*
C      INTEGER TRANS(30),INUM(7),CHR(26)
C      INTEGER NSMAT(16,48)
C      INTEGER BLNK,COMA,OVER,PLUS,SUBT
      DIMENSION S(3,3,NSYM), T(3,NSYM)
      DIMENSION GTRANS(30), GINUM(7), GCHR(26)
      DIMENSION NSMAT(16,48)
      COMMON /FILES/ NIN,NOUT
      COMMON /GSYM/ GLAT,GCENT,HMSYM,HALL
      DATA GBLNK,GCOMA,GPLUS,GSUBT,GOVER/' ',',','+','-','/'/
      DATA GINUM/'1','2','3','5','3','4','6'/
      DATA GCHR/'1','2','3','4','5','6','P','R','A','B','C','I','F',
     1  'N','U','V','W','D',' ','-','X','Y','Z','*','''','"'/
*
* IN THIS ROUTINE NGEN IS EQUAL TO NSYM.
*
      IF (NSYM .GT. 0) GO TO 1021
      WRITE(NOUT,*)' NOTHING TO PRINT.'
      RETURN
*
* EXTERNALLY THE ROTATIONS AND TRANSLATIONS ARE STORED AND
* MANIPULATED IN ARRAYS S(3,3,48) AND T(3,48) RESPECTIVELY.
* IN SYMXYZ ( THIS ROUTINE ) THE ARRAY NSMAT(16,48) SERVES
* AS STORAGE FOR BOTH THE ROTATIONS AND TRANSLATIONS.
*
 1021 DO 7775 M=1,NSYM
      DO 7775 J=1,3
      DO 7775 I=1,3
      K=4*(J-1)+I
      NSMAT(K,M)=INTEGER((S(I,J,M)*12),0)
*
* MULTIPLICATION BY 12 IS NECESSARY TO CHANGE THE DECIMAL
* FRACTIONS OF S(3,3,48) AND T(3,48) INTO THE 12 THS
* WHICH THIS ROUTINE REQUIRES.
* 12 THS ARE USED BECAUSE THE TRANSLATION FRACTIONS
* (1/4, 1/3, 3/4, 1/2, 1/6, 5/6 ETC) ALL ARE NICE, IE EVEN
* MULTIPLIES OF 12.
*
      NSMAT(I+12,M)=INTEGER((T(I,M)*12),0)
 7775 CONTINUE
*
* PRINT HALL SPACE GROUP SYMBOL.
*
      WRITE(NOUT,100)   HALL
  100 FORMAT(' HALL SPACE GROUP SYMBOL = ',A15)
*
      WRITE(NOUT,105)
  105 FORMAT('    ')
      WRITE(NOUT,110)
  110 FORMAT(' SYMMETRY OPERATORS ')
      DO 20 M=1,NSYM
*
* PRESET ARRAY TRANS TO BLANKS.
*
      DO 21 I=1,30
      GTRANS(I)=GBLNK
   21 CONTINUE
      N=1
*
* EXTRACT TRANSLATIONAL ELEMENTS.
*
      DO 22 L=1,3
      J=NSMAT(L+12,M)
*
* SPECIAL POSITION WITH COORDINATE(S) '0'
*
      NSUM=IABS(NSMAT(4*L-3,M))+IABS(NSMAT(4*L-2,M))
     1     +IABS(NSMAT(4*L-1,M))+IABS(J)
      IF(NSUM.EQ.0) THEN
         GTRANS(N)='0'
         N=N+1
         ENDIF
      IF (J.LE.0) GO TO 1017
*
* SET THE NUMERATOR AS ONE (1).
*
      IF (J.GE.8) GO TO 1014
      GTRANS(N)=GINUM(1)
      GO TO 1015
*
* SET NUMERATOR AS 2, 3 OR 5.
*
 1014 GTRANS(N)=GINUM(J-6)
*
* INSERT SLASH INTO FRACTION.
*
 1015 GTRANS(N+1)=GOVER
*
* POINT TO DENOMINATOR CODE.
*
      I=IABS(6-J)
      IF (J.NE.6) GO TO 1016
*
* SET DENOMINATOR AS TWO (2)
*
      GTRANS(N+2)=GINUM(2)
      GO TO 1020
*
* SET DENOMINTATOR AS 3, 4 OR 6
 1016 GTRANS(N+2)=GINUM(I+3)
 1020 N=N+3
 1017 DO 23 K=1,3
      J=L+(K-1)*4
      J=NSMAT(J,M)
      IF (J.EQ.0) GO TO 23
      IF (J.LE.0) GO TO 1018
*
* INSERT PLUS SIGN.
*
      GTRANS(N)=GPLUS
      GO TO 1019
*
* INSERT NEGATIVE SIGN.
*
 1018 GTRANS(N)=GSUBT
*
* INSERT X, Y, OR Z.
*
 1019 GTRANS(N+1)=GCHR(K+20)
      N=N+2
   23 CONTINUE
*
* SEPARATE WITH COMMA.
*
      GTRANS(N)=GCOMA
      N=N+1
   22 CONTINUE
*
* REMOVE THE LAST COMMA FROM THE OUTPUT.
*
      GTRANS(N-1)=GBLNK
*
* LIST SYMMETRY AS X,Y,Z.
*
      WRITE(NOUT,700)M,GTRANS
  700 FORMAT(3X,I3,5X,30A1)
   20 CONTINUE
      RETURN
      END
C
C
c
c
      subroutine losym
c
c   Requests the user for a volume of crystallographic space and turns
c   the contents of this space into a new unit cell in space group P1
c
c   written 1986.9.5 by I.D.Brown
c
c   calls calcy, readyn, gint, wait
c
      implicit character*8 (g)
      implicit character*80 (h)
      common /files/ nin,nout,ndebug,nread,nwrite,nspgr,machine
      common /cell/ a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/ da,db,dc,dalpha,dbeta,dgamma
      include 'comatom.for'
      common /sym/ nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/ mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1        multsp(30),sp(3,3,30),tp(3,30)
      common /gsym/ glat,gcent,hmsym,hall,gsysp(2,30)
c
c  dimensions of oxidt,nzat,gelt,and gidt are MX
c
      dimension xmin(3),xmax(3),oxidt(300),nzat(2,300),xrange(3)
      dimension gelt(300),gidt(300)
c
      conv=57.29578
  190 write(nout,1190)
 1190 format(/,' This routine will prompt you for a new unit cell ',
     + '(to be expressed',/,
     + ' in terms of the current unit cell).  It then transforms the '
     + 'space group',/,
     + ' to P1 in the new cell',/,/,
     + ' Do you wish to continue (Y/N/E)?  ',$)
      call readyn(*200,*900,*900,*190)
  200 write(nout,1200)
 1200 format(/,' Enter the region you wish to choose as the new unit',
     +       ' cell (asymmetric unit)',/,
     + ' xmin,xmax,ymin,ymax,zmin,zmax; in units of current unit cell')
        read(nin,*,err=200) (xmin(i),xmax(i),i=1,3)
  205 write(nout,1205)
 1205 format(/,' Have you entered these correctly (Y/N/E)?  ',$)
      call readyn(*207,*200,*900,*205)
  207 do 210 i=1,3
  210   xmax(i)=xmax(i)-0.00001
      call calcy(xmax,xmin)
      if(ny.gt.mx) then
  220   write(nout,1220) ny,mx
 1220   format(' This operation generated ',I4,' atoms but only ',I4,
     +         ' can be stored. ',/,
     +         ' Do you wish to reset the limits (Y/N/E)  ',$)
        call readyn(*200,*900,*900,*220)
        endif
c
c  copy array yx to xx
c
      ntemp=nx
      nx=ny
      do 310 i=1,ny
        do 310 j=1,3
          xx(j,i)=yx(j,i)
          dxx(j,i)=0.
  310     continue
c
c   copy identifiers to temporary storage
c
      do 340 i=1,ntemp
        gelt(i)=gelem(i)
        gidt(i)=gdent(i)
        oxidt(i)=oxid(i)
        do 330 j=1,2
  330     nzat(j,i)=nza(j,i)
  340   continue
c
c  set up new x identifiers
c
      do 360 i=1,ny
        npos(i)=1
        icn(i)=0
        nh(i)=0
        nd(i)=0
        ialtno(i)=1
        gwyck(i)='a'
        occu(i)=1.
        soccu(i)=0.
        gelem(i)=gelt(iynum(i))
        gtemp=gint(i)
        gdent(i)(1:3)=gtemp(6:8)
        gdent(i)(4:6)=gidt(iynum(i))(1:3)
        oxid(i)=oxidt(iynum(i))
        do 350 j=1,2
  350     nza(j,i)=nzat(j,iynum(i))
  360   continue
c
c   reset the sym and y arrays
c
      ny=0
      nsym=1
      nsp=0
      if(nlt.gt.0) nlt=1
      glat='P'
      gcent='A'
      hmsym='P1'
      hall='P 1'
      gsysp(1,1)='a'
      gsysp(2,1)='1'
c
c   determine the new unit cell and transform the coordinates
c
      do 420 i=1,3
  420    xrange(i)=xmax(i)-xmin(i)
      a=a*xrange(1)
      b=b*xrange(2)
      c=c*xrange(3)
      da=da*xrange(1)
      db=db*xrange(2)
      dc=dc*xrange(3)
      call param
      do 440 i=1,nx
        do 430 j=1,3
          xx(j,i)=(xx(j,i)-xmin(j))/xrange(j)
  430     continue
  440   continue
      write(nout,1500) a,b,c,conv*alpha,conv*beta,conv*agamma
 1500 format(/,' The new cell is:',/,
     + '       a = ',F7.3,/,
     + '       b = ',F7.3,/,
     + '       c = ',F7.3,/,
     + '   alpha = ',F7.3,/,
     + '    beta = ',F7.3,/,
     + '   gamma = ',F7.3,/,/,
     + ' The new space group is P1',/,
     + ' Don''t forget to recalculate the bonds !!')
      call wait
  900 return
      end
C
C
       SUBROUTINE CDNX (KK)
C
C  *****SUBROUTINE*****           CDNX
C                                 ----
C
C WRITTEN 80.6.1 BY DOUG THIERS
C Modified 85.6.4 by David Clarkson
C
C  MODULE TO CHECK THE 6-CHAR CODEN STORED IN ICODN(2-7,KK)
C
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8  (G)
       CHARACTER *36 JTART
       CHARACTER *34 JTOP
       DIMENSION INUM(6)
       COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     1                 HFORM(2),HCOMM(7),GRFACT,GRAD
       COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
       DATA JTART /'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890'/
       DATA JTOP  /'ABCDEFGHIJKLMNOPQRSTUVWXYZ23456789'/
       DO 100 I=3,7
         L=0
         DO 200 J=1,36
           IF(HCODN(KK)(I:I).EQ.JTART(J:J)) L=J
200      CONTINUE
         IF(L.EQ.0) WRITE(NOUT,9000) HCODN(KK),(' ',JJ=1,I-1),'^'
         INUM(I-2)=L
100    CONTINUE
       L=0
       DO 300 I=1,34
         IF(JTOP(I:I).EQ.HCODN(KK)(8:8)) L=I
300    CONTINUE
       IF (L.EQ.0) WRITE(NOUT,9000) HCODN(KK),(' ',JJ=1,7),'^'
       IF (L.EQ.34) L=0
       INUM(6)=L
       ICHECK=MOD (INUM(1)*11+INUM(2)*7+INUM(3)*5+INUM(4)*3+INUM(5),34)
       IF (ICHECK.EQ.INUM(6)) RETURN
       IF (ICHECK.EQ.0) ICHECK=34
       WRITE(NOUT,9001) JTOP(ICHECK:ICHECK),HCODN(KK)
       RETURN
9000   FORMAT (' +++ ERROR +++ CDNX +++ INVALID CHARACTER IN CODEN',
     1         /,' ',A80,/,' ',50A1)
9001   FORMAT (' +++ ERROR +++ CDNX +++ CALCULATED CHECK DIGIT IS',
     1         ' DIFFERENT FROM CURRENT VALUE.',/,' CALCULATED ',
     2         'CHECK DIGIT IS ',A1,/,' ',A80)
       END
C
C
C
C
      SUBROUTINE CALCY(XMAX,XMIN)
C
C   ******SUBROUTINE*******    CALCY
C                              -----
C
C   WRITTEN 83.9.6 BY I D BROWN
C   REWRITTEN BY D. ALTERMATT
C   MCMASTER UNIVERSITY, DEC. 1984
C
C   CALLS STORE,MXV,VPLUSV,INTEGER
C
C   THIS SUBROUTINE CALCULATES ALL THE COORDINATES IN THE REGION OF
C   THE UNIT CELL BETWEEN XMIN AND XMAX, TESTING TO SEE IF ANY ATOMS ARE
C   IN SPECIAL POSITIONS.
C
      IMPLICIT CHARACTER*8 (G)
C
      DIMENSION XMAX(3),XMIN(3),XT(3),XS(3),XR(3),XL(3),L(3),LS(3)
C
      COMMON/FILES/NIN,NOUT,NDEBUG
      include 'comatom.for'
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/GSYM/GLAT
C
      IF(NDEBUG.NE.0) WRITE(NDEBUG,1001) XMIN,XMAX
 1001 FORMAT(' ++DEBUG++CALCY++1001++ XMIN=',3F7.4,'; XMAX=',3F7.4)
C
C   ZERO YATOM ARRAY
C
      NY=0
C
C  LOOP THROUGH THE X ATOMS
C
      DO 500 I=1,NX
C    SKIP DUMMY ATOMS
        IF(XX(1,I).GT.8.0.OR.NPOS(I).LT.0) GO TO 500
        if(igrp(i).lt.0) go to 500
        LL=NY
C
C   LOOP THROUGH THE SYMMETRY OPERATORS
C
        DO 160 J=1,NSYM
          call symop(xs,s(1,1,j),t(1,j),xx(1,i))
C
C   LOOP THROUGH ALL CENTERING OPERATIONS AND ADJACENT CELLS
C
        DO 100 M=1,NLT
          DO 5 K=1,3
            XR(K)=XS(K)+TU(K,M)+3.0
            XL(K)=0.0
    5       XT(K)=-20.0
C
C   FIRST SET LIMITS
C
          DO 10 K=1,3
   20       XR(K)=XR(K)-1.0
            IF(XR(K).GT.XMAX(K)) GO TO 20
            IF(XT(K).LT.-19) XT(K)=XR(K)
            IF(XR(K).LT.XMIN(K)) THEN
              XR(K)=XR(K)+1.0
              XL(K)=XR(K)
              IF(XL(K).GT.(XT(K)+0.1)) GOTO 100
              XR(K)=XT(K)
              GOTO 10
              ENDIF
            GOTO 20
   10       CONTINUE
          CALL STORE(XR,J,I,LL)
          IF(NY.GT.MY) GOTO 165
C
C   NOW LOOP THROUGH THE THREE COORDINATES SO THAT WE GET ALL POSITIONS
C
          DO 30 K=1,3
            L(K)=INTEGER((XT(K)-XL(K)),0)
   30       LS(K)=L(K)
          K=3
   40     L1=LS(K)
          YL=L(K)-LS(K)
          XR(K)=XT(K)-YL
          DO 70 N=1,L1
            XR(K)=XR(K)-1.0
            CALL STORE(XR,J,I,LL)
            IF(NY.GT.MY) GOTO 165
            IF(K.NE.3) THEN
              LS(K)=LS(K)-1
              K=3
              GOTO 40
              ENDIF
   70       CONTINUE
          IF(LS(1).EQ.0.AND.LS(2).EQ.0) GOTO 100
          XR(3)=XT(3)
          K=2
          IF(LS(2).EQ.0) THEN
            LS(2)=L(2)
            XR(2)=XT(2)
            K=1
            ENDIF
          GOTO 40
  100     CONTINUE
  160   CONTINUE
  165 IF(NDEBUG.NE.0) WRITE(NDEBUG,1601) NY,I
 1601   FORMAT(/,' ++DEBUG++CALCY++1601++ ',I4,' ATOMS STORED IN ',
     +    'Y ARRAY AFTER PROCESSING ATOM ',I3)
        IF(NY.GT.MY) NY=MY
  500 CONTINUE
      IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,2000)
 2000   FORMAT(/,' ++DEBUG++CALCY++2000++ 4*(IYNUM, IYSYM, YX(3))')
        WRITE(NDEBUG,2010) (IYNUM(I),IYSYM(I),(YX(J,I),
     +  J=1,3),I=1,NY)
 2010   FORMAT(4(I5,I4,3F6.3))
        ENDIF
  900 RETURN
      END
C
C
C
C
      subroutine calyat(i,xmin,xmax)
c
c     adds all possible postions of atom I within xmin/xmax to yatom array
c     written 87.6.29 ib I.D.Brown as a shell for calat
c
      include 'comatom.for'
      dimension xmax(3),xmin(3)
c
      IF(NDEBUG.NE.0) WRITE(NDEBUG,1001) XMIN,XMAX
 1001 FORMAT(' ++DEBUG++CALYAT++1001++ XMIN=',3F7.4,'; XMAX=',3F7.4)
      DO 300 K=1,3
        XMAX(K)=XMAX(K)+0.0001
  300   XMIN(K)=XMIN(K)-0.0001
      nys=ny+1
      call calat(i,xx(1,i),xmax,xmin)
C
C   END-OF-PROGRAM, DEBUG MESSAGES
C
  165 IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,1601) NY,I
 1601   FORMAT(/,' ++DEBUG++CALYAT++1601++ ',I4,' ATOMS STORED IN ',
     +    'Y ARRAY AFTER PROCESSING ATOM ',I3)
        IF(NY.GT.MY) NY=MY
        WRITE(NDEBUG,2000)
 2000   FORMAT(/,' ++DEBUG++CALYAT++2000++ 3*(IYNUM, IYSYM, YX(3))')
        WRITE(NDEBUG,2010) (IYNUM(K),IYSYM(K),(YX(J,K),
     +  J=1,3),K=NYS,NY)
 2010   FORMAT(3(I5,I4,3F6.3))
        ENDIF
      DO 310 K=1,3
        XMAX(K)=XMAX(K)-0.0001
  310   XMIN(K)=XMIN(K)+0.0001
  900 RETURN
      end
c
c
c
      SUBROUTINE CALAT(I,xxx,XMIN,XMAX)
C
C     CALLS STORE,VPLUSV,MXV,INTEGER
C
C     ADDS ALL POSSIBLE POSITIONS OF ATOM xxx WITHIN XMIN/XMAX
C     TO COMMON /YATOM/
c     NB I is the address of xxx in /xatom/ and should be set negative
c     if it does not refer to an atom in /xatom/
C
C     WRITTEN BY D. ALTERMATT
C     MCMASTER UNIVERSITY, DEC. 1984
C
      IMPLICIT CHARACTER*8 (G)
C
      DIMENSION XMAX(3),XMIN(3),XT(3),XS(3),XR(3),LOOPV(48),XL(3)
      DIMENSION L(3),LS(3)
      COMMON/FILES/NIN,NOUT,NDEBUG
      include 'comatom.for'
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/SYMA/MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30),
     1       MULTSP(30),SP(3,3,30),TP(3,30)
      COMMON/GSYM/GLAT
C
C
C   GET THE SPECIAL POSITION NUMBER, IF KNOWN, AND LOAD LOOPING VECTOR
C
      if(i.gt.0) then
        NS=IALTNO(I)
      else
        ns=0
        endif
      IF(NS.LT.1.OR.NS.GT.NSP) THEN
        DO 50 J=1,NSYM
   50   LOOPV(J)=J
      ELSE
        DO 60 J=1,NSYM
   60   LOOPV(J)=ISPTAB(J,NS)
        ENDIF
C
C   LOOP THROUGH THE SYMMETRY OPERATORS
C
      ll=ny
      DO 160 J=1,NSYM
        J1=J-1
          DO 170 JJ=1,J1
  170     IF(LOOPV(J).EQ.LOOPV(JJ)) GOTO 160
        CALL MXV(XT,S(1,1,J),XXx)
        CALL VPLUSV(XS,T(1,J),XT)
C
C   LOOP THROUGH ALL CENTERING OPERATIONS AND ADJACENT CELLS
C
        DO 100 M=1,NLT
          DO 5 K=1,3
            XR(K)=XS(K)+TU(K,M)+3.0
            XL(K)=0.0
    5       XT(K)=-20.0
C
C   FIRST SET LIMITS
C
          DO 10 K=1,3
   20       XR(K)=XR(K)-1.0
            IF(XR(K).GT.XMAX(K)) GO TO 20
            IF(XT(K).LT.-19) XT(K)=XR(K)
            IF(XR(K).LT.XMIN(K)) THEN
              XR(K)=XR(K)+1.0
              XL(K)=XR(K)
              IF(XL(K).GT.(XT(K)+0.1)) GOTO 100
              XR(K)=XT(K)
              GOTO 10
              ENDIF
            GOTO 20
   10       CONTINUE
          CALL STORE(XR,J,I,LL)
          IF(maxdim(ny,my,'++calat++10++ ny').ne.0) return
C
C   NOW LOOP THROUGH THE THREE COORDINATES SO THAT WE GET ALL POSITIONS
C
          DO 30 K=1,3
            L(K)=INTEGER((XT(K)-XL(K)),0)
   30       LS(K)=L(K)
          K=3
   40     L1=LS(K)
          YL=L(K)-LS(K)
          XR(K)=XT(K)-YL
          DO 70 N=1,L1
            XR(K)=XR(K)-1.0
            CALL STORE(XR,J,I,LL)
            IF(maxdim(ny,my,'++calat++40++ ny').ne.0) return
            IF(K.NE.3) THEN
              LS(K)=LS(K)-1
              K=3
              GOTO 40
              ENDIF
   70       CONTINUE
          IF(LS(1).EQ.0.AND.LS(2).EQ.0) GOTO 100
          XR(3)=XT(3)
          K=2
          IF(LS(2).EQ.0) THEN
            LS(2)=L(2)
            XR(2)=XT(2)
            K=1
            ENDIF
          GOTO 40
  100     CONTINUE
  160   CONTINUE
      return
      END
C
C
C
C
      SUBROUTINE STORE(XS,IS,IA,L)
C
C   *****SUBROUTINE*****    STORE
C                           -----
C
C   WRITTEN 83.9.8 BY I D BROWN
C
C   CALLS JTEST
C
C   STORES YATOMS
C
      DIMENSION XS(3)
      COMMON/FILES/NIN,NOUT
      include 'comatom.for'
      IF(NY.GE.MY) GO TO 50
C
C   IF COORDINATES DUPLICATE THOSE ALREADY THERE DO NOT STORE
C
      DO 5 J=L+1,NY
    5   IF(JTEST(XS,YX(1,J)).EQ.0) GO TO 49
      NY = NY+1
      DO 10 I=1,3
   10   YX(I,NY) = XS(I)
      IYSYM(NY) = IS
      IYNUM(NY) = IA
   49 RETURN
   50 IF(NY.GE.MY) WRITE(NOUT,1050) MY
 1050 FORMAT('  ++++WARNING+++STORE+++  MORE THAN ',I4,' Y ATOMS.',
     + '   REMAINDER IGNORED')
      NY=NY+1
      RETURN
      END
C
*
C
C
      subroutine remote(isp,ndsx,x,is,dsx)
c
c   Finds the place in which an atom is furthest from any symmetry
c   images of itself.
c   written 87.6.29 by I.D.Brown
c
c   x are the coordinates of the remote point, dismin is the shortest
c   distance and ismin is the symmetry operator relating the nearest
c   atom to x
c
c   this routine calls calcat, sorty, maxdim, mean
c
      common/files/ nin,nout,ndebug
      common/cell/a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common/sym/nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      dimension isame(3,48),jsame(48)
      common/syma/mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     +            multsp(30),sp(3,3,30),tp(3,30)
      include 'comatom.for'
      dimension x(3),jy(100),xmax(3),xmin(3),disx(100),dx(3)
c   mdis is the size of the jy and disx arrays
      dimension ifix(3),idone(3),dsx(20),is(20)
c
c   select initial position
c  nfix counts the number of coordinates that are fixed by symmetry
c   because the point selected lies on a special position
c   ifix(3) is 1 for the coordinates that are so fixed
c  ndone counts the number of distances that have already been fitted
c   idone(3) is 1 for cooordinates that have finished refining
c  nsame counts the number of coordinates that are not to be refined
c   isame(3) is 1 for coordinates in the nearest image related only by 
c     translation
c
      mdis= 100
      dmax=10.
      nround=1
      mround=50
      ndone=1
      nfix=0
      do 90 k=1,3
        ifix(k)=0
        idone(k)=0
        x(k)=0.07*k
        do 80 kk=1,3
   80     if(abs(sp(k,kk,isp)).gt.0.01) go to 90
        x(k)=tp(k,isp)
        ifix(k)=1
        nfix=nfix+1
   90   continue
c
c   identify symmetry elements that involve only translations
c
      do 120 i=1,nsym
        jsame(i)=0
        do 110 k=1,3
          isame(k,i)=0
          k1=k+1
          k2=k+2
          if(k1.gt.3) k1=k1-3
          if(k2.gt.3) k2=k2-3
          if(abs(s(k,k,i)-1.).le.0.01.and.abs(s(k,k1,i)).le.0.01.and.
     +            abs(s(k,k2,i)).le.0.01) then
            isame(k,i)=1
            if(abs(t(k,i)).gt.0.01) jsame(i)=1
            endif
  110     continue
  120   continue
      if(ndebug.ne.0) write(ndebug,1120)
     +                   (jsame(i),(isame(k,i),k=1,3),i=1,nsym)
 1120 format(' ++debug++remote++1120++ ',18(I3,1X,3I2))
c
c   calculate the y array around x  (N.B. this will destroy anything
c   already stored in the y atom array
c
  200 xmax(1)=x(1)+dmax/a
      xmin(1)=x(1)-dmax/a
      xmax(2)=x(2)+dmax/b
      xmin(2)=x(2)-dmax/b
      xmax(3)=x(3)+dmax/c
      xmin(3)=x(3)-dmax/c
      ny=0
      ii=-1
      nys=ny+1
      call calat(ii,x,xmin,xmax)
  165 IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,1601) NY,I
 1601   FORMAT(/,' ++DEBUG++remote++1601++ ',I4,' ATOMS STORED IN ',
     +    'Y ARRAY AFTER PROCESSING ATOM ',I3)
        write(ndebug,1200) ny,ii,x,xmin,xmax
 1200   format(' ++debug++remote++1200++ ',2I5,9F7.4)
        WRITE(NDEBUG,2000)
 2000   FORMAT(/,' ++DEBUG++remote++2000++ 3*(IYNUM, IYSYM, YX(3))')
        WRITE(NDEBUG,2010) (IYNUM(K),IYSYM(K),(YX(J,K),
     +  J=1,3),K=NYS,NY)
 2010   FORMAT(3(I5,I4,3F6.3))
        ENDIF
c  if ny is too large, reduce dmax and try again
      if(ny.gt.mdis) then
        dmax=dmax-1.
        go to 200
        endif
c
c   calculate distances to neighbouring atoms and sort
c
      if(ny.le.2) go to 310
      do 240 j=1,ny
        call vminv(dx,yx(1,j),x)
        disx(j)=dist(dx,am)
  240   continue
      if(maxdim(ny,100,'++remote++240++ ny').ne.0) go to 250
  250 call sorty(ny,disx,jy)
C
C   find the shortest distance and fix distances to be used in calculation
c   the following distances are ignored in determining the shortest
c   distance to be used in the calculation:
c      distance between the atom and itself (ndone)
c      any distances already fitted (ndone)
c      any distances generated by glide planes and screww axes (ignore)
c
      ignore=0
  255 continue
      nymax=ny-ndone-ignore
      jmin=jy(nymax)
      isym=iysym(jmin)
      nsame=0
      if(jsame(isym).ne.0) then
        ignore=ignore+1
        go to 255
        endif
      do 258 k=1,3
  258   if(isame(k,isym).ne.0) nsame=nsame+1
      nym=nymax-3+nsame
      if(nym.le.0) nym=1
c
c   test for glides and screws among the short distances
c
      kn=0
      k=0
c     start of implicit j loop
        j=nymax+1
  259   j=j-1
        if(j.le.0) go to 257
        k=k+1
        ky=jy(j)
        is(k)=iysym(ky)
        if(jsame(is(k)).eq.0) kn=kn+1
        if(kn.lt.(4-nsame)) go to 259
  257   nym=j
c       end of implicit j loop
      if(ndebug.ne.0) then
        write(ndebug,1250) (jy(j),j=nymax,nym,-1)
        write(ndebug,1251) (disx(j),j=1,ny)
 1250   format(' ++debug++remote++1250++ The shortest distances are ',
     +           10I3)
 1251   format(' ++debug++remote++1251++ ',4(/,4X,25F5.2))
        write(ndebug,1252) nymax,nym,nfix,ifix,ndone,idone,nsame,
     +     (isame(k,isym),k=1,3)
 1252   format(' ++debug++remote++1252++ ',2I3,2X,3(I4,1X,3I3,2X))
        endif
c
c   average the shortest distances
c
      disa=0.
      disig=0.
      ndis=0.
      jk=0
      do 260 j=nymax,nym,-1
        jj=jy(j)
        jk=jk+1  
        if(jsame(is(jk)).eq.0) call mean(disx(jj),disa,disig,ndis)
  260   continue
      if(ndebug.ne.0) write(ndebug,1260) nround,disa,disig,disx(jmin)
 1260 format(' ++debug++remote++1260++ after round=',I3,', disa='
     +  ,F5.2,', disig=',F5.2,', disx(min)=',F5.2)
c
c   test for convergence or exit if not converging
c
      if(nround.gt.mround) go to 400
      nround=nround+1
      if(abs(disa-disx(jmin)).lt.0.01) then
        if((idone(1).ne.0.or.ifix(1).ne.0).and.
     +     (idone(2).ne.0.or.ifix(2).ne.0).and.
     +     (idone(3).ne.0.or.ifix(3).ne.0)) go to 400
        do 265 k=1,3
          if(ifix(k).eq.0.and.isame(k,isym).eq.0.and.idone(k).eq.0)
     +                                            then
            idone(k)=1
            ndone=ndone+1
            endif
  265     continue
        ndone=ndone+1
        go to 200
        endif
c
c   select new X by moving away from the nearest symmetry image by an
c   amount equal to half the difference between the average distance and
c   the shortest distance, then repeat calculation. 
c
      if(abs(disx(jmin)).lt.0.001) then
        write(nout,1280) jmin
 1280   format(' ++error++remote++1280++ Improper calculation of ',
     +  'yatom ',I4)
        return
        endif
      dk=(disa-disx(jmin))/(2*disx(jmin))
      do 280 k=1,3
        if(ifix(k).eq.0.and.idone(k).eq.0.and.isame(k,isym).eq.0) then
          delx=dk*(x(k)-yx(k,jmin))
          x(k)=x(k)+delx
          endif
  280   continue
      go to 200
c
c   deal with the case where only one or no images are found
c
  310 if(ny.le.1) then
        write(nout,1310) dmax, (x(k),k=1,3)
 1310   format(' ++remote++1310++ No other atoms found within ',
     +   F5.2,'A of ',3F6.3)
        return
      else
        call vminv(dx,yx(1,2),x)
        disx(1)=dist(dx,am)
        if(disx(1).gt.4..or.jsame(iysym(2)).ne.0) then
          write(nout,1320) disx(1)
 1320     format(' ++remote++1320++ Only one other atom at a distance',
     +      F6.2,'A')
          return
        else
          if(abs(disx(1)).lt.0.001) then
            write(nout,1280) jmin
            return
            endif
          do 330 k=1,3
            if(ifix(k).eq.0.and.idone(k).eq.0.and.
     +                    isame(k,iysym(2)).eq.0) then
              delx=(x(k)-yx(k,1))*(4.-disx(1))/(2*disx(1))
              x(k)=x(k)+delx
              endif
  330       continue
            go to 200
          endif
        endif
      go to 200
c
c  Exit routine
c
  400 continue
          k=0
          do 463 j=ny-1,nym,-1
            k=k+1
            jk=jy(j)
            is(k)=iysym(jk)
            dsx(k)=disx(jk)
  463       continue
          ndsx=k
          return
      end
C
C
C
*                UEQ
*
      SUBROUTINE UEQ(UU,DUU,I)
C
C   written c. 1985 by D.Altermatt
c
c  returns the isotropic atomic displacement parameter of atom i in uu(duu)
c
      COMMON/CELL/A,B,C,AL,BE,AGA,V,AM(3,3),ICELL
      COMMON/ECELL/DA,DB,DC,DAL,DBE,DGA
      include 'comatom.for'
      UU=U(1,1,I)
      DUU=DU(1,1,I)
      IF (ICELL.EQ.0) RETURN
      IF (U(2,2,I).LT.1.0E-7) RETURN
      DR = 0.01745329
      ALD = AL*DR
      BED = BE*DR
      AGD = AGA*DR
      UU = (U(1,1,I) + U(2,2,I) + U(3,3,I) + 2.0*(
     1      U(1,2,I)*COS(AGD) + U(1,3,I)*COS(BED) +
     1      U(2,3,I)*COS(ALD)))/3.0
      SDUJJ = DU(1,1,I) + DU(2,2,I) + DU(3,3,I)
      IF (SDUJJ.LT.1.0E-7) RETURN
      DUGA = (COS(AGD)*DU(1,2,I))**2
      UGA = (DGA*SIN(AGD)*U(1,2,I))**2
      DUBE = (COS(BED)*DU(1,3,I))**2
      UBE = (DBE*SIN(BED)*U(1,3,I))**2
      DUAL = (COS(ALD)*DU(2,3,I))**2
      UAL = (DAL*SIN(ALD)*U(2,3,I))**2
      DUU = (SDUJJ + 2.0*(SQRT(DUGA+UGA)+SQRT(DUBE+UBE)+
     1                    SQRT(DUAL+UAL)))/3.0
      RETURN
      END
     
C
C
C
C
C
      SUBROUTINE TEMPER
C                ======
C
C  WRITTEN 80.6.1 BY DOUG THIERS
c  revised by incorporating part of diag 86.4.25 by I.D.Brown
C
C  diagonalises the anisotropic atomic displacement parameters
C
c  calls MATHLB: diag
c
      implicit character*8 (g)
      implicit character*80 (h)
      COMMON/CELL/CELL(6),VOL,AM(3,3),ICELL
      COMMON/RCELL/RCELL(6),RVOL,RM(3,3)
      COMMON/ECELL/ DA,DB,DC,DAL,DBE,DGA,DM(3,3,6)
      include 'comatom.for'
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      DIMENSION E(3),BO(3),QP(3,3),DCR(3),DCD(3),
     +    DCRJ(3),DCDJ(3),RMS(3),B(3,3)
C
C
      PI=3.1415926
      PS=9.8696044
      IF(NX.EQ.0) RETURN
      ICOUNT = 0
  100 FORMAT('1',10X,'DIAGONALISED TEMPERATURE FACTORS')
      D=0.00001
C
C   loop through all atoms
c
      DO 10 K=1,NX
        IF(ABS(U(1,2,K)).LT.D.AND.ABS(U(1,3,K)).LT.D
     +  .AND.ABS(U(2,3,K)).LT.D.AND.ABS(U(2,2,K)).LT.D
     +  .AND.ABS(U(3,3,K)).LT.D)GOTO 10
        IF(ICOUNT.EQ.0) WRITE(NTEMP,100)
        ICOUNT = ICOUNT + 1
c
c   convert U's into betas
c
        DO 140 I=1,3
  140     B(I,I) = U(I,I,K)*2.*PS*RCELL(I)*RCELL(I)
        B(1,2) = U(1,2,K)*2.*PS*RCELL(1)*RCELL(2)
        B(1,3) = U(1,3,K)*2.*PS*RCELL(1)*RCELL(3)
        B(2,3) = U(2,3,K)*2.*PS*RCELL(2)*RCELL(3)
        B(2,1)=B(1,2)
        B(3,1)=B(1,3)
        B(3,2)=B(2,3)
        WRITE(NTEMP,224)
  224   FORMAT('0')
        WRITE(NTEMP,225) GELEM(K),GDENT(K)
  225   FORMAT(' ',A2,A6,24X,'INPUT TEMPERATURE FACTOR (BETA) ',
     +  'MATRIX',/)
        WRITE(NTEMP,240) B
  240   FORMAT(' ',30X,3F12.7)
        WRITE(NTEMP,250)
  250   FORMAT ('0 RMS(A)',3X,'DIRECTION COSINES REAL',3X,
     1  'DIRECTION COSINES RECIP',7X,'ANGLES REAL',10X,
     2  'ANGLES RECIP',/)
c
c   diagonalise beta matrix
c
        CALL DIAG(b,bo,qp,rm,ier)
        if(ndebug.ne.0) then
          write(ndebug,10330) ier,(bo(i),(qp(j,i),j=1,3),i=1,3)
10330     format(' *debug*temper*10330* ',I2,1X,3(F6.2,1X,3F6.2,2X))
          endif
  330   if(ier.lt.0) then
c
c   matrix is non positive definite
c
          WRITE(NTEMP,340) GELEM(i),gdent(i)
  340     FORMAT('0','atomic displacement parameter of ',A2,A6,
     +    'is not positive definite')
          go to 10
        else if(ier.eq.1) then
c
c   matrix is isotropic
c
  354     RMS1=SQART(bo(1)/(2.*ps))
          WRITE(NTEMP,360) GELEM(K),gdent(K),BO(1),RMS1
  360     FORMAT('0',A2,A6,' has an isotropic atomic displacement ',
     +     'parameter B = ',F8.5,5X,'rms disp =',F8.5,' A','.')
          go to 10
          endif
        if(ier.eq.2) then
c
c   matrix has cylindrical symmetry
c
  352      WRITE(NTEMP,355) GELEM(K),gdent(K)
  355      format('0',A2,A6,' has a cylindrically symmetric atomic ',
     +      'displacement factor',/,7X,'Only the direction cosines ',
     +      'of the unique principal axis are given')
           endif
c
c   calculate direction cosines and angles with real and reciprocal axes
c      start with direction cosines to reiprocal axes (DCR)
c
        do 510 l=1,3
          if(ier.eq.2.and.l.eq.2) go to 999
          DO 437 I=1,3
  437       DCR(I)=0.
          DO 438 I=1,3
            DO 438 J=1,3
              DCR(I)=DCR(I)+(QP(J,L)*RM(I,J)/RCELL(I))
              DR=DCR(I)
              IF(DR.LE.1.00000) GO TO 438
              IF(DR.GT.1.1) GO TO 438
              DCR(I)=1.0000000
  438         CONTINUE
C
C     EXPRESS THIS SAME LTH UNIT EIGENVECTOR IN TERMS OF THE DIRECT CELL
C                  BASIS VECTORS. THE COMPONENTS HERE ARE E(I),I=1,2,3.
C
          DO 439 I=1,3
  439       E(I)=0.0
          DO 440 I=1,3
            DO 440 J=1,3
  440         E(I)=E(I)+QP(J,L)*RM(I,J)
c
c   now calculate direction cosines to real cell (DCD)
c
          DO 441 I=1,3
  441       DCD(I)=0.
          DO 442 I=1,3
            DO 442 J=1,3
              DCD(I)=DCD(I)+(E(J)*AM(I,J)/CELL(I))
              DD=DCD(I)
              IF(DD.LE.1.00000) GO TO 442
              IF(DD.GT.1.1) GO TO 442
              DCD(I)=1.000000
  442         CONTINUE
          DO 443 I=1,3
            DCRJ(I) = 361.
            DCDJ(I) = 361.
            IF(ABS(DCD(I)).LE.1.0) DCD1=SQART(1.0-(DCD(I))**2)
            IF(ABS(DCR(I)).LE.1.0) DCR1=SQART(1.0-(DCR(I))**2)
            IF(ABS(DCD(I)).LE.1.0) DCDJ(I)=ATAN2(DCD1,DCD(I))*180./PI
  443       IF(ABS(DCR(I)).LE.1.0) DCRJ(I)=ATAN2(DCR1,DCR(I))*180./PI
          IF(BO(L)/(2.0*PS).LT.0.0) then
            ier=-1
            goto 330
            endif
          RMS(L)=SQART(BO(L)/(2.0*PS))
          WRITE(NTEMP,610) RMS(L),DCD,DCR,DCDJ,DCRJ
  610     FORMAT (' ',F6.3,2(2X,3F8.4),2(3X,3F6.0))
          GO TO 510
  999     RMS(L)=SQART(BO(L)/(2.0*PS))
          WRITE(NTEMP,611) RMS(L),BO(L)
  611     FORMAT(' ',F9.5,F10.5)
  510     CONTINUE
   10   CONTINUE
      RETURN
      END
c
c
c
      function zedsq(x,u)
c
c    written 87.10.16 by I.D.Brown
c
c   This function returns the mean square value of the atomic displacement
c   factor U(3,3) in the direction of the vector X(3)
c
c   calls MATHLB: dist, mxv
c
      common /cell/ a,b,c,al,be,aga,vol,am(3,3),icell
      dimension x(3),y(3),u(3,3),aa(3)
c
      aa(1)=a
      aa(2)=b
      aa(3)=c
      xl=dist(x,am)
      if(abs(xl).lt.0.001) then
        zedsq=0.
        return
        endif
      call mxv(y,am,x)
      z=0.
      do 100 i=1,3
        do 90 j=1,3
          z=z+y(i)*y(j)*u(i,j)/(aa(i)*aa(j))
   90     continue
  100   continue
      zedsq=z/(xl*xl)
      return
      end
c
c
