C
C             File  MAPLIB                       Last update: April 1993
C             ------------                                    T. Stephen
C
C    THIS FILE CONTAINS THE FOLLOWING ROUTINES:
C
C  S VALMAP(iat)               MAIN SUBROUTINE TO CALCULATE VALENCE MAPS
C  S VALINT(nor,maxv,maxh)     INTERACTIVE CONDITIONS SETUP
C  S FINDAT(.......)           SETS FLAGS FOR CALCULATION
C  S VSUM(VAL,DMAX,AC,IF)      CALCULATES VALENCE SUM AROUND AN ATOM
C  S ORIENT(GOR,KOR)           SETS UP ORIENTATIONS FOR VALENCE MAP
C  F DISTNC(X,Y)               RETURNS THE DISTANCE X(3)-Y(3)
C  F VALEN(D,I,J)              RETURNS THE VALENCE OF BOND I-J
C  F FUNCT(S,J,PMAP)           RETURNS THE VALENCE SUM FUNCTION FOR PLOTc_BTING
C  S MAPHELP                   HELP FILE
C
C    EXTERNALS CALLED FROM THIS FILE:
C
C  FOR-EXP, FOR-SQRT, for-atan, for-tan, for-abs
C  Chemlb:  ldval, radius, bonrng, chemno
C  Iolib:   ifile
C  Mainlb:  calcy, inerr
C  Mathlb:  integer, readyn, upcase, wait, vmimv, dist, pageflip
c  W77plot or Dudplot:  bluetored, box, charplo, line, sixteenc, video
C
C
C *****  Interactive input-output in VALINT *****
C
C
***********************************************************************
C
C
      SUBROUTINE VALMAP(iat)
C     ---------- ------
c   (*** iat is no longer used. replaced by nterm ***)
C
C   WRITTEN 81/4/27 BY I.D.BROWN
C   Updated 86/3/5  by D.Altermatt
c   Updated 93/4    by T.Stephen
C
C   Calls:  calcy, (distnc,) findat, funct, integer, orient, valen,
C           valint, vsum, wait, vminv, dist, chemno, bluetored, box,
c           charplo, line, sixteenc, video
C
C   THIS IS THE SUBROUTINE THAT CALCULATES VALENCE MAPS
C
C   IAT is the input-output flag
C       if IAT = 1: interactive program is calling
C       else      : batch job
C
C   MODIFIED FROM THE PROGRAM BONDGRP WRITTEN BY K WALTERSSON
C   (SEE ACTA CRYST A34 901 (1978))
C
c   In any case VALMAP expects the common blocks /*cell/, /*ident/ and
c   /xatom/ to be filled already. All atoms, including the ones to search
c   for, have to be given in /*ident/, at least as dummys.
c
c   In case of a batch run, the following iAformation in /*val/ must be
c   supplied befor calling VALMAP:
c
c   xpm(n,m)   M=1: lower, 2: upper limit and 3: increment in lattice
c              fractions for N=1: x, 2: y and 3: z direction of crystall
c
c   pmap(n)    depends on JT (see label 200, subroutine VALINT)
c              N=1: oxidation state of atom to search for
c                2: exponent of factor for specified function
c                3: scale factor used to ensure that integer output has
c                      sufficient accuracy. (default=1, 10 is recommended)
c
c   valnce(n)  may contain partial valences for atomic positions in
c              /*ident/ or must be set to 100.0 else for N=1 to NX
c
c   dmax       (will be calculated)
c
c   valim(2)   (will be calculated)
c
c   jt         function type key (see label 200, subroutine VALINT)
c
c   nz1        number of atomic positions in GEM/GDM to be excluded
c              from /*ident/-list for calculations. If nz1 = 0, the
c              first encountered element name in /gident/ with all
c              possible oxidation states will be taken.
c
c   igrid(n)   number of grid points in N=1: vertical, 2: horizontal
c              and 3: section (page to page) direction of plot
c              (see MAXH(orizontal) and MAXV(ertical))
c
c   gro(n)     labels (x, y or z) for plotting axes in N=1: vertical,
c              2: horizontal and 3: section direction of plot
c
c   gem/gdm    element names and identifiers according to list in /gident/
c              for the atomic positions to be excluded from calculations
c              (see subroutine FINDAT)
c   iflag      =0 for excluded atoms, else =1
c   jflag      =1 for atoms in the bond environment around xa,ya
c
      IMPLICIT CHARACTER*8 (G)
      implicit character *80 (h)
C
c   system common blocks
c
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,machine,nterm
      COMMON /CELL/   A,B,C,AL,BE,AGA,VOL,AM(3,3),ICELL
      COMMON /RCELL/  RA(6),RVOL,RM(3,3)
      COMMON /GSYM/   GLAT
      include 'comatom.for'
c PROGRAMMER'S NOTE:  /bost/ has been changed to /bostt/ for
c                     Unix system.      
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),pbost(5,30)
      common /gbost/  gbost(2,30)
      COMMON /VAL/    XPM(3,3),PMAP(3),DMAX,valim(2),JT,
     1                NZ1,igrid(3)
      COMMON /GVAL/   GOR(3),GEM(20),GDM(20),ivga,ref,skew
      common /gtitle/ gent,hname,hcodn(2),htitl(2),hauth(10),
     1                hform(2),hcom(7),grfact,grad
c
      DIMENSION GTYPE(6),xxx(3)
      DIMENSION JYN(984),KOR(3),X(3),AMIN(3),AMAX(3),ACORR(3)
      DIMENSION IFLAG(984),VS(984),xmax(3),xmin(3),jflag(984),vls(984)
      DATA GTYPE/'VALENCE','DENSITY','ENERGY ','S      ','T      ',
     +           'BESTFIT'/
c
      nwri=nwrite
      nwrite=nout
      istart=0
c
c     Max. number of grid point (horizontal and vertical) for paper output
c
      maxh=60
      maxv=50
c
c     Get conditions for map plotting in case of interactive job
c     (else its assumed they're already given...)
c
	ivga=0
	write(nout,1000)
1000    format(///'          Welcome to the Valence Sum Map routine',/,
     +            '          --------------------------------------',//,
     + '    This routine allows you to calculate the valence map for ',
     +             'the compound ',/,
     + ' you are studying.  It plots the valence that the missing',
     +            ' atom would have if',/,
     + ' it were placed at an arbitrary point in the crystal.',
     +           '  You will be',/,
     + ' prompted for the name of the atom to be omitted.'///////)
  90    write (nout,1100)
1100    format(' '/,
     +5X,'1. Help'/,
     +5X,'2. Calculate the valence sum map'/,
     +/,
     +/,
     +5X,'20. Return to menu level 3'//,
     +5X,' Enter your choice : ')
	ihcr = 0
	read(nin,1110,err = 90) ihcr
1110    format(I5)
	if(ihcr.le.0.or.ihcr.gt.21) go to 90
	if(ihcr.eq.1) call maphelp
	if(ihcr.eq.2) go to 28
	if(ihcr.eq.20) go to 990
	go to 90
c
C
C   INITIALISE PARAMETERS
C   ---------------------
28    if(nterm.ne.0) call valint(kor,maxv,maxh,*900,istart)
      dd=1.0
      do 20 i=1,nx
	if(npos(i).le.0) goto 20
	do 25 j=i,nx
	  if(npos(j).le.0) goto 25
	  xd=valen(dd,i,j)
25        continue
20      continue
c
c     Fill selected volume with atoms
c     -------------------------------
      do 10 i=1,3
	xmax(i)=xpm(i,2)+1.5*dmax*ra(i)
10      xmin(i)=xpm(i,1)-1.5*dmax*ra(i)
      ny=0
      call calcy(xmax,xmin)
      if(ny.le.0) then
	write(nout,2000)
2000    format(///,' ***ERROR**VALMAP** No atoms found within',
     1    ' specified limits',/,20X,'Valence map calculations',
     2    ' aborted',///)
	if(iat.eq.1) call wait
	return
	endif
      if(iat.eq.1.and.nwrite.ne.nout) then
	write(nout,2005) gtype(jt)
2005    format(///,' Calculating ',A7,' map -- this takes a few',
     1    ' seconds!!',///)
	endif
      WRITE(nwrite,2010)
 2010 FORMAT('1',///,' ***** Parameter Listing for Valence Map *****',
     1  ///)
C
C   FLAG excluded ATOMS (IE ATOMS THAT ARE TO BE OMITTED DURING THE
C   CALCULATION HAVE IFLAG=0, OTHERS HAVE IFLAG=1)
C   ----------------------------------------------------------------
c   entry point for repeat calculation
c
      CALL FINDAT(IFLAG,GELEM,GDENT,npos,NX,GEM,GDM,NZ1)
      IF(NDEBUG.NE.0) WRITE(NDEBUG,2020) (IFLAG(i),i=1,nx)
2020  FORMAT(' ++DEBUG++VALMAP++2020++ ',100I1)
C
C   II IS THE NUMBER OF THE FIRST MISSING ATOM
C   ------------------------------------------
      II=1
      oxo=pmap(1)
      if(gdm(1).eq.'ALL') oxo=0.0
      DO 30 I=1,NX
	if(iflag(i).eq.1) goto 30
	IF(GELEM(I).ne.GEM(1)) GO TO 30
	if(abs(oxo).gt.0.001.and.oxid(i).ne.pmap(1)) goto 30
	if(abs(pmap(1)).lt.0.01) pmap(1)=oxid(i)
	II=I
	goto 40
30      continue
C
C  CALCULATE THE PARTIAL VALENCE SUMS  AT EACH ATOM AND STORE IN VALNCE
C  --------------------------------------------------------------------
40    DO 50 I=1,3
   50   ACORR(I) = DMAX*RA(I)*1.2
      IF(NDEBUG.NE.0) WRITE(NDEBUG,2030) ACORR
2030  FORMAT(' ++DEBUG++VALMAP++2030++ ACORR',3F8.4)
      CALL VSUM(VALNCE,DMAX,ACORR,IFLAG)
      DO 60 I=1,NX
	IF(IFLAG(I).EQ.0) GO TO 60
	VALNCE(I) = VALNCE(I) - ABS(OXID(I))
   60   CONTINUE
c
c  determine acid strength of atom sought (bases negative)
c  -------------------------------------------------------
      do 65 i=1,nx
	if(iflag(i).eq.0) then
	  call chemno(gelem(i),np,ng)
	  astren=oxid(i)/icoor(np,ng)
	  go to 67
	  endif
   65   continue
   67 continue
C
C     List paramters for map calculations
C     -----------------------------------
      write(nwrite,2040) gtype(jt),gem(1),gdm(1),(gor(kor(i)),
     1  xpm(i,1),xpm(i,2),xpm(i,3),i=1,3),(gor(i),i=1,3)
2040  format(' Calculating ',A7,' map for ',A2,1X,A6,
     1  ' within the limits',//,
     2  3(1X,A1,' from ',F6.3,' to ',F6.3,' in steps of ',F6.3,
     2  ' of the unit cell',/),/,
     3  ' where ',A1,' goes down the page, ',A1,' across the page',/,
     4  ' and ',A1,' from page to page',//)
      if(jt.eq.2) then
	write(nwrite,2042) pmap(1),pmap(2)
2042    format('     D = (V /',F6.3,')**(-',F6.1,')',//)
      else if(jt.eq.3) then
	write(nwrite,2044) pmap(2),pmap(1)
2044    format('     E = ',F6.1,' * (V /',F6.3,')',//)
      else if(jt.eq.4) then
	write(nwrite,2046) pmap(1)
2046    format('     S = (V -',F6.3,')**2',//)
      else if(jt.eq.5) then
	write(nwrite,2048) pmap(1),pmap(2)
2048    format('     T = ((V -',F6.3,') + 1)**(2 -',F6.1,')',//)
      else if(jt.eq.6) then
	write(nwrite,2049) pmap(1)
2049    format('     U = (V -',F6.3,')**2 + ',
     1         '&(weight*(V(j) - OX(j))**2)/#atoms',//)
	endif
      write(nwrite,2055) dmax
2055  format(' Valence sums calculated for each grid point using',/,
     1  ' a sphere of enclosure of radius ',F6.3,' Angstroms.'//)
      if(iat.eq.1.and.nwrite.eq.nout) call wait
      write(nwrite,2050)
2050  format(/,' Atomic parameters used:',//,
     1  '  Nr.  Elem.   Ident.   Ox.State   Part.Valence   Status',
     2  '       Coordinates',/)
      linc=0
      do 70 i=1,nx
	linc=linc+1
	gstat=' '
	if(iflag(i).eq.0) gstat='excluded'
	if(npos(i).eq.0) gstat='unknown'
	if(npos(i).lt.0) gstat='dummy'
	write(nwrite,2060) i,gelem(i),gdent(i),oxid(i),valnce(i),gstat,
     1    (xx(j,i),j=1,3)
2060    format(1X,I3,3X,A2,5X,A6,4X,F6.3,7X,F7.3,6X,A8,3(1X,F6.3))
	if(linc.eq.20.or.i.eq.nx) then
	  if(iat.eq.1.and.nwrite.eq.nout) then
	    linc=0
	    call wait
	    if(i.ne.nx) write(nwrite,2050)
	    endif
	  endif
70      continue
      write(nwrite,2070)
2070  format(//,' Valence Curves stored for calculations:',//,
     1  ' Elem.1   Ox.State1   Elem.2   Ox.State2   Equ.Type   ',
     2  'Parameters',/)
      linc=0
      do 80 i=1,nbost
	linc=linc+1
	write(nwrite,2080) (gbost(j,i),ioxbost(j,i),j=1,2),ifbost(i),
     1    (pbost(j,i),j=1,3)
2080    format(3X,A2,6X,I6,7X,A2,6X,I6,9X,I2,6X,F7.4,2X,F5.2,2X,F5.2)
	if(linc.gt.20.or.i.eq.nbost) then
	  if(iat.eq.1.and.nwrite.eq.nout) then
	    linc=0
	    call wait
	    if(i.ne.nbost) write(nwrite,2070)
	    endif
	  endif
80      continue
C
C   In batch runs ARRANGE THE ORIENTATION OF THE AXES FOR THE MAP.
C   GOR(I) CONTAINS THE NAME OF THE VERTICAL AXIS, HORIZONTAL AXIS
C   AND SECTION AXIS RESPECTIVELY
C   KOR(I) IS SET TO CONTAIN 1,2 OR 3 CORRESPONDING TO X,Y AND Z
C   (A,B AND C AXIS) IN GOR(I)
C   ----------------------------------------------------------------
      if(iat.ne.1) CALL ORIENT(GOR,KOR)
C
C   SELECT THE LAYER ZA AT WHICH THE CURRENT CALCULATION IS MADE
C   ------------------------------------------------------------
      NZZA=0
      ZA=XPM(kor(3),1) - XPM(kor(3),3)
      i=video(ivga)
      if (i.eq.0.and.ivga.ne.0) then
	i=video(0)
	write(nout,119)
119     format('Your monitor can not handle the selected video mode.')
	call wait
	goto 900
      end if
      if (ivga.eq.1) iiii=sixteenc(1)
      if (ivga.ge.2) iiii=bluetored(1)
  120 NZZA = NZZA + 1
      ZA = ZA + XPM(kor(3),3)
      IF(NZZA.GT.igrid(3).OR.ZA.GT.XPM(kor(3),2)) GO TO 900
      if (ivga.eq.0) WRITE(nwrite,2100) GTYPE(JT),GOR(3),ZA,pmap(3)
2100  FORMAT('1','************** ',A7,' MAP',4X,A1,' =',F6.3,
     + ' All values multiplied by ',F6.1)
c      frac1=maxh
      frac1=igrid(1)
      frac2=igrid(2)
      frac=frac1/frac2
      if(frac.lt.4.2) then
	nfact=10**integer(frac,0)
      else
	nfact=10**4
      endif
      xfact=nfact
      nform=integer(frac,0)+1
      valim(1)=nfact*9.9999-1.0
      valim(2)=-nfact*0.99999+1.0
      DO 140 I=1,igrid(2)
140     JYN(I) = integer((xfact*(XPM(kor(2),1)+(I-1)*XPM(kor(2),3))),0)
      if (ivga.eq.0) then
	write(hfmt,2110) nfact,igrid(2),nform
2110    format('(4H ',I2,'*,A1,3H = ',I2,'I',I1,')')
	WRITE(nwrite,hfmt) GOR(2),(JYN(I),I=1,igrid(2))
	write(hfmt,2120) nfact
2120    format('(4H ',I2,'*,A1)')
	write(nwrite,hfmt) gor(1)
      endif
C
C   SELECT POINT XA, YA ON THE LAYER CURRENTLY BEING CALCULATED
C   -----------------------------------------------------------
      nxa = 0
      XA = XPM(kor(1),1) - XPM(kor(1),3)
      if (ivga.ne.0) i=clearscreen(1)
      linecount=0
  220 NXA = NXA + 1
      XA = XA + XPM(kor(1),3)
      IF(NXA.GT.igrid(1).OR.XA.GT.XPM(kor(1),2)) then
	if(iat.eq.1.and.nwrite.eq.nout) then
 2004     if (ivga.eq.0) then
	    write(nout,2002)
 2002       format('Hit Return to continue')
	    read(nin,2003)gdum
	  else
c       opening blurb
	    write(nout,2007)gor(1),gor(2),gor(3),za
2007        format('Down: ',a1,' axis.  Across: ',a1,' axis.  ',a1,
     +       ' position =',f6.3,' Unit cell fractions.',/,'Cross ', 
     +       'section of unit cell in yellow.  Labels are tenths ',
     +       'of unit cell.')
c        plot unit cell edges
	    i0ac=-1*xpm(kor(2),1)/xpm(kor(2),3)*ref+39
	    i0dn=-1*xpm(kor(1),1)/xpm(kor(1),3)*ref+64
	    i1ac=(1-xpm(kor(2),1))/xpm(kor(2),3)*ref+39
	    i1dn=(1-xpm(kor(1),1))/xpm(kor(1),3)*ref+64
	    if (xpm(kor(2),2)-xpm(kor(2),1).le.0.01) goto 2073
	    if (skew.le.0) then
	      i0act=i0ac+int(-1*skew)
	      i1act=i1ac+int(-1*skew)
	      i0acb=i0ac+int(skew*(1-(xpm(kor(2),2)-xpm(kor(2),1))))
	      i1acb=i1ac+int(skew*(1-(xpm(kor(2),2)-xpm(kor(2),1))))
	    else
	      i0act=i0ac
	      i1act=i1ac
	      i0acb=int(i0ac+skew/(xpm(kor(2),2)-xpm(kor(2),1)))
	      i1acb=int(i1ac+skew/(xpm(kor(2),2)-xpm(kor(2),1)))
	    endif
2073        i=line(i0act,i0dn,i1act,i0dn,14)
	    i=line(i0act,i0dn,i0acb,i1dn,14) 
	    i=line(i1acb,i1dn,i1act,i0dn,14) 
	    i=line(i1acb,i1dn,i0acb,i1dn,14)
c       write out scale (tenths of unit cell)
	    do 2081 i=0,10
	      inear=(i*i1act+(10-i)*i0act)/10
	      ix=line(inear,i0dn-2,inear,i0dn-1,14)
	      write(gbuf,788)i
2081          ix=charplo(10,10,inear-15,i0dn-13,3,gbuf,14)
	    do 2082 i=1,10
	      inear=(i*i1dn+(10-i)*i0dn)/10
	      icmpd=(i*i0acb+(10-i)*i0act)/10-1
	      ix=line(icmpd,inear,icmpd-1,inear,14)
	      write(gbuf,788)i
2082          ix=charplo(10,10,icmpd-25,inear-5,3,gbuf,14)
788           format(i2)
	  endif
c      colour scale
	  if (ivga.eq.1) then
c           sixteen colour scale
	    do 778 i=1,12
	      ico=i
	      if (i.eq.7) ico=15
	      if (i.ge.8) ico=i-1
	      colval=2.0*tan((i-7.5)/4.4567)
	      if (jt.eq.2.or.jt.eq.5) colval=colval/4+1
	      if (jt.eq.3) colval=colval+pmap(2)
	      if (jt.eq.1) colval=colval+pmap(1)
	      write(gbuf,783,err=2004)colval
	      ixth=160*mod((i-1),4)
	      iyth=maxv+64+21*mod((i-1)/4,4)
	      ix=charplo(21,10,ixth,iyth,6,gbuf,ico)
	      gbuf='-'
	      ix=charplo(21,20,ixth+60,iyth,1,gbuf,ico)
	      colval=2.0*tan((i-6.5)/4.4567)
	      if (jt.eq.1) colval=colval+pmap(1)
	      if (jt.eq.3) colval=colval+pmap(2)
	      if (jt.eq.2.or.jt.eq.5) colval=colval/4+1
	      write(gbuf,783,err=2004)colval
	      ix=charplo(21,10,ixth+80,iyth,6,gbuf,ico)
783           format(f6.2)
778         continue
	  else if (ivga.ge.2) then
c           127 colour "continuous" red to blue scale.
	    kfct=1
	    if (ivga.eq.3) kfct=2
	    do 912 i=16,143
	      ix=charplo(38,kfct*10,kfct*5*(i-16),maxv+64,1,'*',i)
	      if (mod(i,6).eq.0) then
		colval=2.0*tan((i-79.5)/25.735)
		if (jt.eq.2.or.jt.eq.5) colval=colval/4+1
		if (jt.eq.3) colval=colval+pmap(2)
		if (jt.eq.1) colval=colval+pmap(1)
		write(gbuf,783,err=2004)colval
		ix=charplo(kfct*15,kfct*5,kfct*5*(i-19),
     +               maxv+99,6,gbuf,i)
	      endif
912         continue
	  endif
	  if (ivga.ne.0) then
	    write(nout,2023)hname
2023        format('V. map for: ',A67)
	    do 2024 i=1,5000000
2024            ix=i+1+i*2
	    hdum='Hit  enterfor  next page.     ''E''  then enter'//
     +           'to   quit.'
	    do 2031 i=1,11
2031          ix=charplo(maxv/32,6,0,48+i*maxv/32,5,
     +                   hdum((i-1)*5+1:i*5+1),10)
	    read(nin,2003)gdum             
 2003       format(a1)
	    if (gdum(1:1).eq.'e'.or.gdum(1:1).eq.'E') goto 900
	  endif
	endif
	goto 120
      endif
      NYA = 0
      YA = XPM(kor(2),1) - XPM(kor(2),3)
  230 NYA = NYA + 1
      YA = YA + XPM(kor(2),3)
      IF(NYA.GT.igrid(2).or.ya.gt.xpm(kor(2),2)) GO TO 420
C
C   SET LIMITS AROUND XA, YA
C   ------------------------
      i1=kor(1)
      i2=kor(2)
      i3=kor(3)
      x(i1)=xa
      x(i2)=ya
      x(i3)=za
      DO 260 I=1,3
	AMIN(I) = X(I) - ACORR(I)
  260   AMAX(I) = X(I) + ACORR(I)
*     IF(NDEBUG.NE.0) WRITE(NDEBUG,1260) AMAX,AMIN
*1260 FORMAT(' ++DEBUG++VALMAP++1260++ AMAX',3F8.4,' AMIN',3F8.4)
C
C   LOOP THROUGH THE Y ATOMS AND TRANSLATE TO FIND THE YATOM NEAR
C   XA,YA
C   --------------------------------------------------------------
      vt=0.
      SS = 0.0001
      do 280 i=1,nx
	jflag(i)=0
	vls(i)=abs(valnce(i))
  280   continue
      DO 400 I=1,NY
	JJ = IYNUM(I)
	jflag(jj)=0
	IF(IFLAG(JJ).EQ.0) GO TO 400
	do 300 j=1,3
 300      if(yx(j,i).lt.amin(j).or.yx(j,i).gt.amax(j)) goto 400
	call vminv(xxx,x,yx(1,i))
	dd=dist(xxx,am)
	if(dd.gt.dmax) goto 400
	if(dd.lt.0.01) goto 400
	jflag(jj)=1
	vv=valen(dd,ii,jj)
	ss=ss+vv
C   PROGRAMMERS NOTE:
C    At this point we should calculate all the points related by symmetry
C    to x and, treating this as an excluded atom, calculate their contri-
C    bution to the partial valences and the repulsion term between symmtery
C    related excluded atoms
C   END OF PROGRAMMERS NOTE:
	if(jt.eq.6.and.astren*oxid(jj).lt.0.)then
	  vls(jj) = vls(jj)-vv
	  endif
  400   CONTINUE
C
C   CALCULATE THE FUNCTION TO BE PLOTTED
C   ------------------------------------
      if(jt.eq.6) then
	nsum=0
	do 410 i=1,nx
	  if(iflag(i).gt.0.and.jflag(i).gt.0.and.npos(i).gt.0
     +       .and.igrp(i).ge.0.and.(oxid(i)/pmap(1)).lt.0) then
	    vt=vt+npos(i)*(vls(i))**2
	    nsum=nsum+npos(i)
	    endif
  410     continue
	vs(nya)=pmap(3)*((vt+(ss-pmap(1))**2)/(nsum+1))**0.5
      else
	VS(NYA) = FUNCT(SS,JT,PMAP)
	endif
      if(vs(nya).gt.valim(1).and.ivga.eq.0) vs(nya)=valim(1)
      if(vs(nya).lt.valim(2).and.ivga.eq.0) vs(nya)=valim(2)
      GO TO 230
  420 CONTINUE
C
C   WRITE ONE LINE OF THE MAP
C   -------------------------
      JXA = integer(xfact*XA,0)
      if (ivga.eq.0) then
	write(hfmt,2200) igrid(2),nform
2200    format('(I5,3X,',I3,'I',I2,')')
	WRITE(nwrite,hfmt) JXA,(integer(VS(I),0),I=1,igrid(2))
	if((maxv/igrid(1)).ge.2) write(nwrite,2210)
2210    format(' ')
      else if (ivga.ge.1) then
	iii=0
	iref=int(ref)
	do 777 i=1,igrid(2)*iref,iref
	  iii=iii+1
c         parallelogram form:
	  if (skew.ge.0) then
	    iskew=int(skew*(linecount/(igrid(1)*ref)))
	  else
	    iskew=int(skew*(linecount/(igrid(1)*ref)-1))
	  endif
c         set proper colour dependent on mapping function
	  col=atan((vs(iii)-pmap(1))/2)
	  if (jt.eq.3) col=atan((vs(iii)-pmap(2))/2)
	  if (jt.ge.4) col=atan(vs(iii)/2)
	  if (jt.eq.2.or.jt.eq.5) col=atan(2*(vs(iii)-1))
	  if (ivga.eq.1) then
	    icol=int(4.4567*col+7.5)
	    if (icol.eq.7) icol=16
	    if (icol.ge.7) icol=icol-1
	  else
	    icol=int(25.735*col+79.5)
	  endif
c         plot point (square)
	  v=box(1,i+39+iskew,linecount+64,i+39+iref+iskew,
     +             linecount+64+iref,icol)
777     continue
	linecount=linecount+iref
      endif
      GO TO 220
  900 if (ivga.ne.0) then i=video(0)
      write(nout,2900)
 2900 format(' Do you wish to calculate another valence map(Y/N)?  ')
      call readyn(*28,*990,*990,*900)
  990 nwrite=nwri
      RETURN
      END
c
c
***********************************************************************
c
c
      subroutine valint(nor,maxv,maxh,*,istart)
c     -----------------
c
c     written by D.Altermatt, March 1986, Uni Lausanne
c     modified by T.Stephen, April 1993
c
c     Interactively read conditions for map plotting
c
c     CAUTION: Opens and uses UNIT 16 with function IFILE
c
c     Calls: ifile, inerr, integer, readyn, upcase
c
      implicit character *8 (g)
      implicit character *80 (h)
      logical lopen
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      common /cell/   cell(3),acell(3)
      include 'comatom.for'      
      common /gsym/   glat,gcent,hsym,hall
      common /asymu/  iasym,aslim(3,2),isyel(3,2)
      common /val/    xpm(3,3),pmap(3),dmax,valim(2),jt,
     1                nz1,igrid(3)
      common /gval/   gor(3),gem(20),gdm(20),ivga,ref,skew
      common /range/  nj,iox(20),top(20,20),bot(20,20),ddmax,ddmin,
     1                rmax,rmin,vmax,vmin,auglx
      common /grange/ gel(20)
c
      dimension xint(3),nor(3)
c
c     default parameters
c     ------------------
      conv=180./3.141592654
      nz1=0
      nz2=20
      call bonrng
      do 1 i=1,100
1       valnce(i)=100.
      if(istart.ne.0) then
2       write(nout,2009)
2009    format(' Do you want to use the previous plotting',
     1   ' limits(Y/N/E)  ')
	call readyn(*250,*3,*410,*2)
	endif
3     istart=istart+1
      do 5 i=1,3
	pmap(i)=0.0
	xpm(i,1)=0.0
	xpm(i,2)=1.0
	xpm(i,3)=0.04
	nor(i)=i
5       continue
      gor(1)='X'
      gor(2)='Y'
      gor(3)='Z'
      dmax=3.4
      valim(1)=99.
      valim(2)=-9.
c
c     define output stream
c     --------------------
      if(machine.eq.1) then
	gstat='new'
	gfort='fortran'
	hfile='VALMAP.DAT'
      else if(machine.eq.2) then
	gstat='unknown'
	gfort='print'
	hfile='VALMAP:DAT'
      else if(machine.eq.4) then
	gstat='unknown'
	gfort='fortran'
	hfile='valmap.dat'
      else if(machine.eq.5) then
	gstat='unknown'
	gfort='fortran'
	hfile='valmap.dat'
      else
	gstat='unknown'
	gfort=' '
	hfile='VALMAP'
	endif
10    write(nout,2000) hfile(1:10)
2000  format(////,' Valence Map Calculations',//,
     1  ' The map may either be displayed to the screen or be',/,
     2  ' printed to a file called ',A10,//,
     3  ' Screen or File output (S/F): ')
      read(nin,1000,err=10) gfile
1000  format(A1)
      call upcase(gfile)
      if(gfile.eq.'S') then
	nwrite=nout
	write(nout,2001)
2001    format('Would you like: ',/,
     1  '1. Ascii output',/,
     2  '2. VGA 640X480, 16 colour representation (Watcom graphics)',/,
     3  '3. SVGA 640X480, 256 colour representation (Watcom graphics)',/,
     4  '4. SVGA 1024X768, 256 colour graphics (Watcom graphics)',/)
	read(nin,1001,err=10)inn
1001    format(i2)
	ivga=inn-1
      else if(gfile.eq.'F') then
	nunit=16
	inquire(nunit,opened=lopen)
	if(lopen) then
15        write(nout,2005) hfile,hfile,hfile,hfile
2005      format(//,1X,A10,' already used in previous run.',/,
     1      ' Type: A if you want to add current map to ',A10,/,
     2      '       D if you want to close and re-open ',A10,/,
     3      '       E to EXIT, so you may print ',A10,' first',
     4      //,' Enter your choice (A/D/E): ')
	  read(nin,1000,err=15) gade
	  call upcase(gade)
	  if(gade.eq.'A') then
	    goto 20
	  else if(gade.eq.'E') then
	    return 1
	  else if(gade.eq.'D') then
	    nnw=iclosu(nunit,hfile)
	    if(nnw.ne.0) then
	      iunit=0
	      nnw=iclosu(iunit,hfile)
	      endif
	  else
	    call inerr(*15)
	    endif
	  endif
	nnw=ifile(hfile,gfort,gstat,nunit)
	if(nnw.le.0) call inerr(*10)
	nwrite=nnw
      else
	call inerr(*10)
	endif
c
c     Setup plotting limits
c     ---------------------
20    write(nout,2007) cell,(acell(i)*conv,i=1,3),hsym,hall
2007  format(' The unit cell is ',6F7.2,//,
     +       ' The space group is ',A15,' (',A15,')')
      write(nout,2010)
2010  format(//,' Set up plotting limits:',//,
     1  '  1: Entire cell',/,
     2  '  2: Asymmetric unit as derived earlier in program',/,
     3  '  3: User specified limits',//,
     4  ' Enter your choice: ')
      read(nin,1010,err=20) lim
1010  format(I5)
      if(lim.lt.1.or.lim.gt.3) call inerr(*20)
      if(lim.eq.1) goto 100
30    if(lim.eq.2) then
	if(iasym.ne.1) then
	  write(nout,2020)
2020      format(//,' No asymmetric cell given yet',//)
	  lim=2
	  goto 30
	  endif
	do 40 i=1,3
	  do 40 j=1,3
40        xpm(i,j)=aslim(i,j)
45      write(nout,2060) (xpm(i,1),xpm(i,2),i=1,3)
	call readyn(*100,*20,*410,*45)
      else
50      write(nout,2030)
2030    format(//,' Enter mapping limits in cell fractions:',//,
     1    ' Enter X-min, X-max: ')
	if(machine.eq.3.or.machine.eq.5) then
	  read(nin,*,err=50) xpm(1,1),xpm(1,2)
	else
	  read(nin,1020,err=50) xpm(1,1),xpm(1,2)
	  endif
1020    format(2F10.0)
60      write(nout,2040)
2040    format(' Enter Y-min, Y-max: ')
	if(machine.eq.3.or.machine.eq.5) then
	  read(nin,*,err=60) xpm(2,1),xpm(2,2)
	else
	  read(nin,1020,err=60) xpm(2,1),xpm(2,2)
	  endif
70      write(nout,2050)
2050    format(' Enter Z-min, Z-max: ')
	if(machine.eq.3.or.machine.eq.5) then
	  read(nin,*,err=70) xpm(3,1),xpm(3,2)
	else
	  read(nin,1020,err=70) xpm(3,1),xpm(3,2)
	  endif
	do 80 i=1,3
	  if((abs(xpm(i,1))+abs(xpm(i,2))).gt.3.0) call inerr(*50)
80        if((xpm(i,2)-xpm(i,1)).lt.0.01) call inerr(*50)
90      write(nout,2060) (xpm(i,1),xpm(i,2),i=1,3)
2060    format(//,' Mapping limits as read:',//,
     1    ' X-min, X-max: ',2F10.4,/,' Y-min, Y-max: ',2F10.4,/,
     2    ' Z-min, Z-max: ',2F10.4,//,' Limits OK (Y/N/E): ')
	call readyn(*100,*50,*410,*90)
	endif
c
c     Get plotting directions and grid intervals
c     ------------------------------------------
100   gor(1)='X'
      gor(2)='Y'
      gor(3)='Z'
      if(nwrite.eq.nout) then
	maxh=35
	maxv=20
      else
	maxh=60
	maxv=50
	endif
      if (ivga.ge.1.and.ivga.le.3) then
	maxv=352
	maxh=600
	if (ivga.eq.3) then
	  maxv=640
	  maxh=984
	endif
	if (ivga.eq.1) then
	  ref=4
	else
	  ref=1
	endif
      endif
      do 105 i=1,3
105     nor(i)=i
110   if (ivga.eq.0) then
       igrid(1)=maxv
       igrid(2)=maxh
       igrid(3)=int(cell(nor(3))*(xpm(nor(3),2)-xpm(nor(3),1))/0.1)+1
       do 120 i=1,3
	xint(i)=(cell(nor(i))*(xpm(nor(i),2)-xpm(nor(i),1)))/igrid(i)
120     xpm(nor(i),3)=xint(i)/cell(nor(i))
      else
c     take intervals in Angstroms to be fixed
	xint(1)=(cell(nor(1))*(xpm(nor(1),2)-xpm(nor(1),1)))/maxv*ref
	xint(2)=xint(1)
	xint(3)=xint(1)
c     set fractions and pixel from that
	do 121 i=1,3
	  xpm(nor(i),3)=xint(i)/cell(nor(i))
121       igrid(i)=int((xpm(nor(i),2)-xpm(nor(i),1))/xpm(nor(i),3))
c       out of screen?
	if (igrid(2)*ref.gt.maxh) then
	  do 19 i=1,3
19          xpm(i,3)=xpm(i,3)*igrid(2)*ref/maxh
	  igrid(1)=igrid(1)*maxh/igrid(2)/ref
	  igrid(2)=maxh/ref
	endif
	angle=acell(6-nor(1)-nor(2))
	skew=0
21      if (angle.lt.1.55.or.angle.gt.1.6) then
c      adjust for length measured along axes rather than perpendicular
c          igrid(1)=igrid(1)*sin(angle)
c          xpm(nor(1),3)=xpm(nor(1),3)/sin(angle)
c          xint(1)=xint(1)/sin(angle)
	  skew=ref*igrid(1)/tan(angle)
	endif
c     out of screen?
	if (igrid(2)*ref+abs(skew).gt.maxh) then
	  do 22 i=1,3
22            xpm(i,3)=xpm(i,3)*(igrid(2)*ref+abs(skew))/maxh
	  igrid(1)=igrid(1)*maxh/(igrid(2)*ref+abs(skew))
	  igrid(3)=igrid(3)*maxh/(igrid(2)*ref+abs(skew))
	  igrid(2)=igrid(2)*maxh/(igrid(2)*ref+abs(skew))
	  skew=ref*igrid(1)/tan(angle)
	endif
      endif
130   write(nout,2100) (gor(i),igrid(i),xint(i),xpm(nor(i),3),i=1,3)
2100  format(//,' Current plotting directions are:',//,
     1  '  Down the page  : ',A1,', Points: ',I3,
     2  ', Grid int.: ',F6.4,'A or ',F6.4,' latt.fract.',/,
     3  '  Across the page: ',A1,', Points: ',I3,
     4  ', Grid int.: ',F6.4,'A or ',F6.4,' latt.fract.',/,
     5  '  Page to page   : ',A1,', Pages : ',I3,
     6  ', Grid int.: ',F6.4,'A or ',F6.4,' latt.fract.',//,
     7  ' Choose one of the following options:',//,
     8  '  1:  Above proposal OK',/,'  2:  Change directions',/,
     9  '  3:  Change grid intervals',/,
     1  '  4:  Get default values',//,
     2  ' Enter your choice: ')
      read(nin,1010,err=130) ich
      if(ich.lt.1.or.ich.gt.4) call inerr(*130)
      if(ich.eq.4) goto 100
      if(ich.eq.2) then
140     write(nout,2110)
2110    format(//,' Enter direction down page (X,Y,Z): ')
	read(nin,1000,err=140) gor(1)
	call upcase(gor(1))
	if(gor(1).eq.'X') then
	  nor(1)=1
	else if(gor(1).eq.'Y') then
	  nor(1)=2
	else if(gor(1).eq.'Z') then
	  nor(1)=3
	else
	  call inerr(*140)
	  endif
150     write(nout,2120)
2120    format(/,' Enter direction across page (X,Y,Z): ')
	read(nin,1000,err=150) gor(2)
	call upcase(gor(2))
	if(gor(2).eq.'X') then
	  if(nor(1).eq.1) call inerr(*150)
	  nor(2)=1
	  gor(3)='Z'
	  if(nor(1).eq.3) gor(3)='Y'
	  nor(3)=6-(nor(1)+nor(2))
	else if(gor(2).eq.'Y') then
	  if(nor(1).eq.2) call inerr(*150)
	  nor(2)=2
	  gor(3)='Z'
	  if(nor(1).eq.3) gor(3)='X'
	  nor(3)=6-(nor(1)+nor(2))
	else if(gor(2).eq.'Z') then
	  if(nor(1).eq.3) call inerr(*150)
	  nor(2)=3
	  gor(3)='X'
	  if(nor(1).eq.1) gor(3)='Y'
	  nor(3)=6-(nor(1)+nor(2))
	else
	  call inerr(*150)
	  endif
	goto 110
      else if(ich.eq.3) then
	if (ivga.eq.0) then
155     write(nout,2130)
2130    format(//,' Grid intervals in Angstroms of Fractions (A/F): ')
	read(nin,1000,err=155) gt
	call upcase(gt)
	if(gt.ne.'A'.and.gt.ne.'F') call inerr(*155)
	xlim=cell(nor(1))*(xpm(nor(1),2)-xpm(nor(1),1))/maxv
	if(gt.eq.'F') xlim=xlim/cell(nor(1))
160     write(nout,2200) xlim,gt
2200    format(/,' Enter grid interval for down page (>',F6.3,A1,'): '
     +    )
	read(nin,1030,err=160) xint(1)
1030    format(F10.0)
	if(xint(1).lt.xlim.or.xint(1).gt.100.0) call inerr(*160)
	xlim=cell(nor(2))*(xpm(nor(2),2)-xpm(nor(2),1))/maxh
	if(gt.eq.'F') xlim=xlim/cell(nor(2))
170     write(nout,2210) xlim,gt
2210    format(/,' Enter grid interval for across page (>',
     1    F6.3,A1,'): ')
	read(nin,1030,err=170) xint(2)
	if(xint(2).lt.xlim.or.xint(2).gt.100.0) call inerr(*170)
	xlim=0.1
	if(gt.eq.'F') xlim=xlim/cell(nor(3))
180     write(nout,2220) xlim,gt
2220    format(/,' Enter grid interval to next page (>',F6.3,A1,'): ')
	read(nin,1030,err=180) xint(3)
	if(xint(3).lt.xlim.or.xint(3).gt.100.0) call inerr(*180)
	do 190 i=1,3
	  if(gt.eq.'F') xint(i)=xint(i)*cell(nor(i))
	  xpm(nor(i),3)=xint(i)/cell(nor(i))
190       igrid(i)=int(cell(nor(i))*(xpm(nor(i),2)-xpm(nor(i),1))
     1      /xint(i))+1
	goto 130
	else
c       more restrictive graphics routines.
	  tref=ref
	  write(nout,2219)
2219      format('Grid intervals across and down screen are',
     +    /,'proportional to the resolution number.',/)
2221      write(nout,2222)ref
2222      format('Resolution is the size of the blocks that will',/,
     +     'be printed on the screen, in pixels.',/,'Decreasing the ',
     +     'resolution number improves picture at the expense of time.'
     +     ,/,'Current resolution is ',f6.3,'.  Enter new (>=1)')
	  read(nin,1030,err=2221)ref
	  if (ref.lt.1) ref=tref
	  igrid(1)=igrid(1)*tref/ref
	  igrid(2)=igrid(2)*tref/ref
	  do 2225 i=1,2
2225        xpm(nor(i),3)=xpm(nor(i),3)*ref/tref
	  xint(1)=xint(1)*ref/tref
	  xint(2)=xint(2)*ref/tref
2224      write(nout,2223)
2223      format('Input lattice fraction between pages: ')
	  read(nin,1030,err=2224)xpm(nor(3),3)
	  if (xpm(nor(3),3).le.0) goto 2224
	  igrid(3)=int((xpm(nor(3),2)-xpm(nor(3),1))/xpm(nor(3),3))+1
	  xint(3)=cell(3)*(xpm(nor(3),2)-xpm(nor(3),1))/igrid(3)
	  goto 130
	endif
	endif
c
c     Get atom/element to look/search for
c     -----------------------------------
250   write(nout,2230)
2230  format(//,' Enter element name of atom to look for (2 char.)',
     +     ' or EXIT: ')
      read(nin,1040,err=250) gelo
1040  format(A2)
      call upcase(gelo)
      if(gelo.eq.' ') call inerr(*250)
      if(gelo.eq.'EX') go to 410
255   write(nout,2235)
2235  format(/,' Enter oxidation state of above atom (0 for all): ')
      read(nin,1030,err=255) oxo
      if(abs(oxo).gt.15.0) call inerr(*255)
      ifound=0
      pmap(1)=oxo
      if(ndebug.ne.0) then
	do 257 i=1,nx
	  write(ndebug,2570) gelem(i),oxid(i)
 2570     format(' *DEBUG*VALINT*2570* ',A2,A6)
  257     continue
	endif
      do 260 i=1,nx
	ggelem=gelem(i)
	call upcase(ggelem)
	if(ggelem.ne.gelo) goto 260
	if(abs(oxo).gt.0.01.and.abs(oxid(i)-oxo).gt.0.1) goto 260
	if(ifound.eq.0) then
	  ifound=1
	  write(nout,2240)
2240      format(//,' Specified element found in list as:',/,
     1      ' Pos.  Elem.  Ident.  Ox.State  Coordinates',/)
	  endif
	gflag=' '
	if(npos(i).gt.0) then
	  nz1=nz1+1
	  if(nz1.gt.nz2) then
	    write(nout,2250) nz2,nz2
2250        format(//,' More than ',I3,' positions to exclude --',
     1        ' first ',I3,' taken only',/)
	    nz1=nz2
	    goto 270
	    endif
	  gem(nz1)=gelem(i)
	  gdm(nz1)=gdent(i)
	  gflag='excluded'
	  endif
	if(npos(i).eq.0) gflag='unknown'
	if(npos(i).lt.0) gflag='dummy'
	write(nout,2255) i,gelem(i),gdent(i),oxid(i),(xx(j,i),j=1,3),
     1    gflag
2255    format(1X,I3,5X,A2,3X,A6,4X,F5.2,3X,3(F6.3,2X),A8)
260     continue
270   if(ifound.eq.0) then
	write(nout,2265) gelo,oxo
2265    format(//,' ***ERROR*** Element ',A2,' in ox. state ',F5.2,
     1    ' not found -- TRY AGAIN',//)
	goto 250
      else if(ifound.eq.1) then
c     select dmax
c
	do 276 i=1,nj
	   call upcase(gel(i))
	   if(gelo.eq.gel(i)) then
	      dmax=0
	      do 274 j=1,nj
274              if(top(i,j).gt.dmax) dmax=top(i,j)
	      goto 280
	   endif
 276     continue
	dmax=4.0
c     dmax effects noticable at high resolutions.
280     dmax=1.4*dmax
	write(nout,2260)
2260    format(//,' Do you want to include one of above ',/,
     1    ' excluded atoms in the map calculations (Y/N/E): ')
	call readyn(*290,*200,*410,*280)
290     write(nout,2270)
2270    format(/,' Enter position number as of listing (0 to end): ')
	read(nin,1010) npp
	if(npp.eq.0) goto 200
	if(npp.lt.1.or.npp.gt.nx) call inerr(*290)
	ifound=0
310     nnz=nz1
	do 300 i=1,nnz
	  if(gem(i).ne.gelem(npp).or.gdm(i).ne.gdent(npp)) goto 300
	  ifound=1
	  nz1=nz1-1
	  if(nz1.eq.0) then
	    nz1=1
	    write(nout,2275)
2275        format(///,' Sorry, but one position of chosen atom',
     1                 ' must remain "excluded"',/,
     2        ' If necessary, add a dummy atom to the input list',///)
	    call wait
	    goto 200
	    endif
	  do 320 j=i,nz1
	    gem(j)=gem(j+1)
320         gdm(j)=gdm(j+1)
	  goto 310
300       continue
	if(ifound.ne.1) call inerr(*290)
	goto 290
	endif
c
c     Get the type of map the user wants
c     ----------------------------------
200   write(nout,2300)
2300  format(//,' The following functions may be mapped:',//,
     1  '  1:  Valence Map v = K*V',/,
     2  '  2:  Density Map D = K*(V/OX)**(-N)',/,
     3  '  3:  Energy  Map E = K*N*(V/OX)',/,
     4  '  4:  Function    S = K*(V-OX)**2',/,
     5  '  5:  Function    T = K*((V-OX)+1)**(2-N)',/,
     5  '  6:  Best Fit    B = K*rms(V-OX) (all bonded atoms)',//,
     6  '  where:  OX ... oxidation state of atom you look for',/,
     7  '          N  ... exponent/factor as specified',/,
     8  '          K  ... scale factor',/,
     8  '          NA ... number of atoms',//,
     9  ' Enter your choice for the mapping function: ')
      read(nin,1010,err=200) jt
      if(jt.lt.1.or.jt.gt.6) call inerr(*200)
      pdef=1
      if(jt.eq.2) pdef=100
      pmap(3)=1.0
      if (ivga.eq.0) then
	write(nout,2306) pdef
2306    format(' Enter your choice of scale factor, K.',/,
     +    '  Default is ',F4.0,':  ')
	read(nin,1030,err=200)pmap(3)
	if(pmap(3).lt.0.01) pmap(3)=pdef
      endif
      if(jt.ne.1.and.abs(pmap(1)).lt.0.0001) then
210     write(nout,2310)
2310    format(//,' Enter the oxidation state of the atom you are',
     1    ' looking for: ')
	read(nin,1030,err=210) pmap(1)
	if(abs(pmap(1)).gt.15.0.or.abs(pmap(1)).lt.0.0001)
     1    call inerr(*210)
	endif
      if(jt.eq.3) then
220     write(nout,2320)
2320    format(/,' Enter factor for energy map (0 for default): ')
	read(nin,1030,err=220) pmap(2)
	if(pmap(2).lt.0.0001.or.pmap(2).gt.1000.0) pmap(2)=16.0
      else if(jt.eq.2.or.jt.eq.5) then
230     write(nout,2330)
2330    format(/,' Enter exponent for mapping (< 37, default=16 ) : ')
	read(nin,1030,err=230) pmap(2)
	if(abs(pmap(2)).lt.0.0001.or.abs(pmap(2)).gt.36.0)
     1    pmap(2)=16.0
	endif
c
c     Exclude more atoms from calculations?
c     -------------------------------------
330   write(nout,2350)
2350  format(//,' Do you want to exclude other atoms ',
     1  'from calculations (Y/N/E): ')
      call readyn(*335,*350,*410,*330)
335   write(nout,2360)
2360  format(/,' Enter element name of atom to be excluded (2 char.): '
     +  )
      read(nin,1040,err=335) gelo
      call upcase(gelo)
      if(gelo.eq.' ') goto 330
340   write(nout,2235)
      read(nin,1030,err=340) oxo
      if(abs(oxo).gt.15.0) call inerr(*335)
      ifound=0
      do 345 i=1,nx
	ggelem=gelem(i)
	call upcase(ggelem)
	if(ggelem.ne.gelo) goto 345
	if(abs(oxo).gt.0.01.and.oxid(i).ne.oxo) goto 345
	if(ifound.eq.0) then
	  ifound=1
	  write(nout,2240)
	  endif
	gflag=' '
	if(npos(i).gt.0) then
	  nz1=nz1+1
	  if(nz1.gt.nz2) then
	    write(nout,2250) nz2,nz2
	    nz1=nz2
	    goto 355
	    endif
	  gflag='excluded'
	  gem(nz1)=gelem(i)
	  gdm(nz1)=gdent(i)
	  endif
	if(npos(i).eq.0) gflag='unknown'
	if(npos(i).lt.0) gflag='dummy'
	write(nout,2255) i,gelem(i),gdent(i),oxid(i),(xx(j,i),j=1,3),
     1    gflag
345     continue
355   if(ifound.ne.1) call inerr(*330)
      goto 330
c
c     Assign partial valences
c     -----------------------
350   write(nout,2400)
2400  format(//,' Do you want to assign partial valences (Y/N/E): ')
      call readyn(*360,*400,*410,*350)
360   write(nout,2410)
2410  format(/,' Enter element name (2 char., blank for end-of-list): '
     +  )
      read(nin,1040,err=360) gelo
      if(gelo.eq.' ') goto 400
      call upcase(gelo)
370   write(nout,2420)
2420  format(/,' Enter element identifier (blank for ALL, ',
     1  'max. 6 char.): ')
      read(nin,1050,err=370) gid
1050  format(A6)
      ifound=0
      do 380 i=1,nx
	if(gelem(i).ne.gelo) goto 380
	if(gid.ne.' '.and.gdent(i).ne.gid) goto 380
	ifound=1
	write(nout,2440) i
2440    format(/,' Specified atom found in list, position ',I3,/)
390     write(nout,2430)
2430    format(/,' Enter partial valence for this atom: ')
	read(nin,1030,err=390) vall
	if(abs(vall).gt.15.0) call inerr(*390)
	valnce(i)=vall
380     continue
      if(ifound.ne.1) call inerr(*360)
      goto 350
c
c     That's it guys, bye-bye
c     -----------------------
400   return
410   return 1
      end
C
C
***********************************************************************
C
C
      SUBROUTINE FINDAT(IFLAG,GELEM,GDENT,npos,NX,GEM,GDM,NZ1)
C
C   *****SUBROUTINE*****          FINDAT
C                                 ------
C
C  WRITTEN 81.2.28 BY I.D.BROWN
C  Updated 86.3.13 by D. Altermatt, Uni Lausanne
C
C  Sets IFLAG(i) = 1 for all atoms in /ident/ that are to be included
C  in the valens map calculations. All others have IFLAG(i) = 0.
C
C  If there is no list of elements to be excluded, the first element
C  name in list will be taken as default.
C
      IMPLICIT CHARACTER*8 (G)
C
      DIMENSION GELEM(NX),GDENT(NX),npos(nx),GEM(20),GDM(20),IFLAG(NX)
C
C     Get a default of NZ1=0
C
      IF(NZ1.NE.0) GO TO 20
      GEM(1) = GELEM(1)
      GDM(1) = 'ALL'
      NZ1 = 1
   20 CONTINUE
C
C     Flag according to list in GEM/GDM, drop dummy positions
C
      DO 100 J=1,NX
	if(npos(j).le.0) then
	  iflag(j)=0
	  goto 100
	  endif
	DO 50 I=1,NZ1
	  IF(GEM(I).NE.GELEM(J)) GO TO 50
	  IF(GDM(I).EQ.'ALL') GO TO 40
	  IF(GDM(I).NE.GDENT(J)) GO TO 50
   40     IFLAG(J) = 0
	  GO TO 100
   50     CONTINUE
	IFLAG(J) = 1
  100   CONTINUE
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
***********************************************************************
C
C
      SUBROUTINE VSUM(VALNC,DMAX,ACORR,IFLAG)
C
C   *****SUBROUTINE*****          VSUM
C                                 ----
C
C    WRITTEN 81.4.29 BY I.D.BROWN
C    Updated 86.3.13 by D. Altermatt, Uni Lausanne
C
C    CALLS: (distnc,) valen, vminv, dist
C
C    CALCULATES THE PARTIAL VALENCE SUM AROUND EACH remaining ATOM
C    using a box of enclosure +/- ACORR(i)
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,nread,nwrite,nspgp,machine,nterm
      common /cell/   a,b,c,al,be,aga,vol,am(3,3),icell
      include 'comatom.for'      
C
      DIMENSION VALNC(MX),ACORR(3),x(3),AMIN(3),AMAX(3)
      DIMENSION IFLAG(mx),xxx(3)
C
C   SELECT XATOM
C
      DO 200 II=1,NX
	IF(IFLAG(II).EQ.0) GO TO 200
	IF(VALNC(II).LT.90.) GO TO 200
	DO 100 I=1,3
	  X(I) = XX(I,II)
	  AMIN(I) = X(I) - ACORR(I)
  100     AMAX(I) = X(I) + ACORR(I)
	SS = 0.
C
C   SELECT YATOM
C
c  ******PROGRAMMERS NOTE:
C    This routine does not take any notice of dummy atoms so that
C    H atoms, for example, whose coordinates are not given are
C    ignored.  This should be corrected
C  END OF PROGRAMMERS NOTE ********
	DO 160 JJ=1,NY
	  IJ = IYNUM(JJ)
	  if(oxid(ii)*oxid(ij).gt.-0.01) go to 160
	  IF(IFLAG(IJ).EQ.0) GO TO 160
	  DO 110 I=1,3
  110       if(yx(i,jj).lt.amin(i).or.yx(i,jj).gt.amax(i)) goto 160
	  call vminv(xxx,x,yx(1,jj))
	  dd=dist(xxx,am)
	  if(dd.gt.dmax) goto 160
	  if(dd.lt.0.01) goto 160
	  ss=ss+valen(dd,ii,ij)
  160     CONTINUE
	VALNC(II) = SS
  200   CONTINUE
      IF(NDEBUG.EQ.0) RETURN
      WRITE(NDEBUG,2000)
2000  FORMAT(' **DEBUG**VSUM**2000**',5X,'ARRAY VALNC')
      WRITE(NDEBUG,2010) (VALNC(i),i=1,nx)
2010  FORMAT(' **DEBUG**VSUM**2010**',1X,10F8.5,9(/,23X,10F8.5))
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE ORIENT(GOR,KOR)
C
C   *****SUBROUTINE*****          ORIENT
C                                 ------
C
C   WRITTEN 81.4.29 BY I.D.BROWN
C
C   SETS ARRAYS TO PROVIDE THE CORRECT ORIENTATION OF THE VALENCE MAP
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON /FILES/  NIN,NOUT,ndebug,nread,nwrite,nspgp,machine,nterm
C
      DIMENSION GOR(3),GJOR(3),KOR(3)
C
      DATA GJOR/'X','Y','Z'/
      DO 50 J=1,3
   50   KOR(J) = 0
      DO 80 I=1,3
	DO 80 J=1,3
	IF(GOR(I).NE.GJOR(J)) GO TO 80
	KOR(J) = I
   80   CONTINUE
      IF(KOR(1)+KOR(2)+KOR(3).EQ.6) GO TO 90
      WRITE(NOUT,1080) GOR
 1080 FORMAT(' **ORIENT** ','PROBLEM IN ASSIGNING THE ORIENTATION OF
     +   THE MAP',3(1X,A1))
   90 RETURN
      END
C
C
***********************************************************************
C
C
      FUNCTION DISTNC(X,Y)
C
C   *****FUNCTION*****          DISTNC
C                               ------
C
C   WRITTEN 81.4.29 BY I.D.BROWN
C
C   CALCULATES THE DISTANCE BETWEEN THE POINT X AND THE POINT Y
C
      COMMON /CELL/ A,B,C,AL,BE,AGA,VOL,AM(3,3),ICELL
C
      DIMENSION X(3),Y(3),DX(3)
C
      Z = 0.
      DO 10 I=1,3
   10   DX(I) = Y(I) - X(I)
      DO 20 I=1,3
	DO 20 J=1,3
   20   Z = Z + DX(I)*DX(J)*AM(I,J)
      IF(Z.LT.0.)GO TO 30
      DISTNC = SQRT(Z)
      RETURN
   30 CONTINUE
      DISTNC = 0.
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
***********************************************************************
C
C
      FUNCTION VALEN(D,II,JJ)
C
C   *****FUNCTION*****          VALEN
C                               -----
C
C   WRITTEN 81.5.4 BY I.D.BROWN
C   Updated 86.3.11 by D. Altermatt
C
C   CALCULATES THE VALENCE FOR BOND LENGTH D BETWEEN ATOMS II AND JJ
C   tries to load missing valence curves
C   if LDVAL returns ier=1 (no curve calculated by CALVAL), a 'zero'
C   curve is loaded that can be used for anion-anion and cation-cation
c   valences in maplib
C
C   Calls: integer, ldval, radius
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,nread,nwrite,nspgp,machine,nterm
      include 'comatom.for'
c PROGRAMMER'S NOTE:  /bost/ has been changed to /bostt/ for
c                     Unix system      
      COMMON /BOSTT/   NBOST,MBOST,IOXBOST(2,30),IFBOST(30),PBOST(5,30)
      COMMON /GBOST/  GBOST(2,30)
C
C   SELECT BOND VALENCE PARAMETERS
C   ------------------------------
      isear=0
30    isear=isear+1
      DO 20 J=1,NBOST
	if(GELEM(II).ne.GBOST(1,J)) GOTO 40
	IF(GELEM(jj).ne.GBOST(2,J)) GOTO 20
	if(integer(oxid(ii),0).ne.ioxbost(1,j)) goto 20
	if(integer(oxid(jj),0).ne.ioxbost(2,j)) goto 20
	goto 50
   40   if(gelem(ii).ne.gbost(2,j)) goto 20
	IF(GELEM(jj).ne.GBOST(1,J)) GOTO 20
	if(integer(oxid(ii),0).ne.ioxbost(2,j)) goto 20
	if(integer(oxid(jj),0).ne.ioxbost(1,j)) goto 20
	goto 50
   20   continue
      if(isear.eq.1) then
	call ldval(gelem(ii),gelem(jj),oxid(ii),oxid(jj),ier)
	if(ier.eq.0) then
	  goto 30
	else if(ier.eq.1) then
	  icnn=0
	  rnot=1.0*(radius(gelem(ii),oxid(ii),icnn)+
     1      radius(gelem(jj),oxid(jj),icnn))
c             (the use of icnn=0 forces the use of default icn in radius)
	  if(nbost.eq.mbost) goto 100
	  nbost=nbost+1
	  gbost(1,nbost)=gelem(ii)
	  gbost(2,nbost)=gelem(jj)
	  ioxbost(1,nbost)=integer(oxid(ii),0)
	  ioxbost(2,nbost)=integer(oxid(jj),0)
	  ifbost(nbost)=2
	  if(oxid(ii).ge.0.and.oxid(jj).ge.0) then
	    pbost(1,nbost)=rnot+1.0
	    if (pbost(1,nbost).gt.2.0) pbost(1,nbost)=2.0
	  else if(oxid(ii).lt.0.and.oxid(jj).lt.0) then
	    pbost(1,nbost)=rnot-0.8
	  else
	    pbost(1,nbost)=rnot
	    endif
	  pbost(2,nbost)=0.1
	  pbost(3,nbost)=1.0
	  j=nbost
	  goto 50
	  endif
	endif
      GO TO 100
C
C  CALCULATE VALENCE
C
   50 IF(IFBOST(J).EQ.2) THEN
	VALEN = OCCU(JJ)*EXP((PBOST(1,J)-D)/PBOST(2,J))
      ELSE if(ifbost(j).eq.1) then
	VALEN = OCCU(JJ)*((D/PBOST(1,J))**(-PBOST(2,J))*PBOST(3,J))
      else if(ifbost(j).eq.3) then
	if(d.le.pbost(1,j)) then
	  valen=abs(oxid(ii))
	  go to 90
	  endif
	valen=occu(jj)*(pbost(2,j)/(d-pbost(1,j))-pbost(3,j))
	if(valen.lt.0.) valen=0.
      else
	valen=0.0
	ENDIF
   90 return
  100 CONTINUE
      WRITE(Nout,1100) GELEM(II),GELEM(JJ)
 1100 FORMAT(1X,'++ERROR++VALEN: BOND VALENCE PARAMETERS NOT LOADED',
     +  ' FOR ATOMS ',A2,1X,A2)
      VALEN = 0.
      RETURN
      END
C
C
***********************************************************************
C
C
      FUNCTION FUNCT(SS,JJ,PMAP)
C
C   *****FUNCTION*****          FUNCT
C                               -----
C
C   WRITTEN 81.5.4 BY I.D.BROWN
C   Updated 86.3.14 by D. Altermatt, Uni Lausanne
C
C   CALCULATES THE FUNCTION TO BE PLOTTED FROM THE VALENCE SUMS SS
C
      DIMENSION PMAP(3)
C
c   Prevent overflows
c
      if(ss.gt.99999.0) ss=99999.0
      if(ss.lt.0.01) ss=0.01
C
      IF(JJ.NE.2) GO TO 200
C
C   DENSITY FUNCTION
C
      FUNCT = pmap(3)*abs((SS/PMAP(1)))**(-PMAP(2))
      RETURN
  200 IF(JJ.NE.3) GO TO 300
C
C   ENERGY FUNCTION
C
      FUNCT = pmap(3)*PMAP(2)*(SS/PMAP(1))
      RETURN
  300 IF(JJ.NE.4) GO TO 400
C
C   SQUARED DIFFERENCE FUNCTION
C
      FUNCT = pmap(3)*(SS-PMAP(1))*(SS-PMAP(1))
      RETURN
  400 CONTINUE
      IF(JJ.NE.5) GO TO 500
C
C   T FUNCTION
C
      FUNCT = pmap(3)*abs(((SS - PMAP(1))+1))**(2-pmap(2))
      RETURN
  500 CONTINUE
C
C   VALENCE FUNCTION
C
      FUNCT = pmap(3)*SS
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END

c
***********************************************************************
c
       subroutine maphelp
c
c      written by Lucien Jamieson  -  March 22, 1989
c      updated by Tamon Stephen - March, 1993
c
c      Used as a help screen for Maplib.
c
c     
       common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
100       write (nout,1100)
1100     format(/,'                            Valence Sum Map'//,
     +5X,'    The valence sum map is a technique for finding those'/,
     +5X,'places in a crystal that provide a suitable chemical '/,
     +5X,'environment for a given atom.   The technique was first '/,
     +5X,'described by Waltersson in Acta Cryst A34, 901 (1978).')
       write (nout,1110)
1110     format(' ',
     +5X,'    An atom is removed from the structure and placed '/,
     +5X,'at an arbitrary point, r, in the unit cell.   The '/,
     +5X,'distances between it and its neighbours are calculated '/,
     +5X,'and converted to bond valences.   If the sum (V) of these'/,
     +5X,'bond valences around the atom equals its oxidation state '/,
     +5X,'(OX), the position, r, is a favourable one for the atom.')
       write (nout,1120)
1120     format(' ',
     +4X,'If V > OX the position is too close to one or more '/,
     +5X,'other atoms. If V < OX it is in a cavity where the '/,
     +5X,'bond distances are too long.   The atom is then '/,
     +5X,'systematically moved through all the points in the unit '/,
     +5X,'cell to produce a map of the values of V.'////)
c
       call pageflip(*200,*100,*9999)
c
200    write (nout,2100)
2100     format(//' ',
     +5X,'    It is also possible to calculate other related functions'/,
     +5X,'such as D = (V/OX)**(-N).   In many cases, if N=16 the D map'/,
     +5X,'looks like an atomic density map.   The E function produces '/,
     +5X,'a map that looks like an energy map.   Examples of D maps '/,
     +5X,'are given in J.Solid State Chem. 42, 183 (1982) and '/,
     +5X,'Phys. Chem. Miner. 15, 30 (1987).')
210    write (nout,2110)
2110   format(' ',
     +5X,'    There are two modes of display available, ASCII',/,
     +5x,'output, in which the valence sum map is displayed as an'/,
     +5X,'integer matrix, and graphics where the map is represented'/,
     +5x,'in coloured pixels.',/,
     +5x,'    If the user selects ASCII mode,',/,
     +5X,'the results may be shown on the screen or sent to a file '/,
     +5X,'for printing.   The user will be prompted for the '/,
     +5X,'orientation of the cell, the size of the region to be ')
       write(nout,2120)
2120   format(' ',
     +4X,'plotted and the grid interval.  The program then asks which'/,
     +5X,'atom or atoms are to be omitted and the type of function to'/,
     +5X,'be calculated.  A few other options are offered but these '/,
     +5X,'have speciallized applications.   Since the map '/,
     +5X,'displays only integers, it is usually wise to choose the '/,
     +5X,'scale factor to be 10 or 100.'/)
c
       call pageflip(*300,*100,*9999)
300    write(nout,3100)
3100   format(//,' ',
     +5x,'    If you have the Watcom Fortran 77 graphics library and',/, 
     +5x,'a VGA monitor, you may wish to view graphics maps.  There',/,
     +5x,'at present 3 types of graphics supported.  The first, is a',/,
     +5x,'block type 16 colour map which shows numbers in each of',/,
     +5x,'about a dozen computer selected intervals in a different',/,
     +5x,'shade of blue or grey with the numbers close to the target',/,
     +5x,'highlighted in cyan.  The others are 256 colour versions',/,
     +5x,'with the target value in white, and a fade to black as',/,
     +5x,'the numbers move away, through blue as they increase and',/,
     +5x,'red as they decrease.  In both cases a colour scale is',/,
     +5x,'visible on the bottom.')
       write(nout,3110)
3110   format(' ',
     +5x,'    In each case, the computer will initially determine',/,
     +5x,'the geometric scale, once the fraction of the unit',/,
     +5x,'cell to be plotted has been given.  The user can then',/,
     +5x,'change the directions on the axes, the resolution and',/,
     +5x,'the number of pages.  The resolution controls the quality',/,
     +5x,'of the picture you will get; decreasing it to 1 will',/,
     +5x,'allow you to view the map in great detail, by calculating',/
     +/)
c
      call pageflip(*400,*200,*9999)
400   write(nout,4100)
4100  format(/////,
     +5x,'a valence for every pixel.  Changing the resolution',/,
     +5x,'to n will cause the computer to calculate the valence',/,
     +5x,'only for each sqaure of length n pixels by n pixels.',/,
     +5x,'This will give less detail, but increasing the',/,
     +5x,'resolution by a factor of n will decrease the time it',/,
     +5x,'takes to draw by around n**2.')
      write(nout,4115)
4115  format(' ',
     +5x,'    The questions for graphics mode will be the',/,
     +5x,'same as for ASCII, and the same features will',/,
     +5x,'be available.')
      write(nout,4120)
4120  format(' ',
     +5x,'    The output will consist of a valence map with ',/,
     +5x,'the border of the unit cell and a scale indicating which',/,
     +5x,'colours represent which range of numbers.',///////)
      call pageflip(*9999,*300,*9999)
c
9999   return
       end
c
