C
C           FILE MATHLB                        Last update: 5 June 1992
C           -----------                                     C. Brodie
C
C******************************************************************
C  CONTAINS MACHINE DEPENDENT CODE IDENTIFIED BY
C    'machine dependent'
C******************************************************************
c  This file contains DOUBLE PRECISSION arrays
c******************************************************************
C
C   THIS LIBRARY CONTAINS THE FOLLOWING ROUTINES
C
C             Mathematical Functions
C             ----------------------
C  F INTEGER(X,N)              CONVERTS X*(10**N) TO INTEGER
C  F ARCOS(X)                  FAILSAFE VERSION OF ACOS
C  F ELOG(X)                   FAILSAFE VERSION OF ALOG
C  F SQART(X)                  FAILSAFE VERSION OF SQRT
C +F DSQART(X)                 DOUBLE PRECISSION VERSION OF SQART
C  S MEAN(X,XM,SIG,N)          ADDS NEW VALUE OF X TO MEAN XM
C
C             Vector operations
C             -----------------
C  F DIST(A,AM)                CALCULATES LENGTH OF A IN METRIC AM
C  F DOT(A,B,AM)               CALCULATES A.B USING METRIC AM
C  S CROSS(C,A,B)              C=AXB (SEE NOTE FOR NON-ORTHOGONAL COORDINATES)
C  F ANGLE(A,B,AM)             CALCULATES ANGLE (DEG) BETWEEN A AND B USING AM
C  F RANGLE(A,B,AM)             ENTRY FOR ANGLE (RAD)
C  S ROTX(XM,A)                XM = MATRIX TO ROTATE A RADS ABOUT X-AXIS
C  S ROTY(XM,A)                XM = MATRIX TO ROTATE A RADS ABOUT Y-AXIS
C  S ROTZ(XM,A)                XM = MATRIX TO ROTATE A RADS ABOUT Z-AXIS
C  S ROTVEC(XM,A,B)            XM = MATRIX TO ROTATE A(3) TO B(3)
C  S ROTANG(XM,A,B)             ENTRY ROTATES A RADIANS ABOUT AXIS B(3)
C  S ROTCOM(Z,T,X,N)           ROTATES N SETS OF VECTORS X(3,N) USING MATRIX
C                              T(3) AND RETURNS THE COORDS Z(3,N)
C  S METRIC(A,B,C,AL,BE,GA,AM) CALCULATES METRIC AM FROM CELL
C  S TRANSP(PM,XM,N)           PM(N,N)=TRANSPOSE(XM(N,N))
C  S VXV(RV,V1,V2)             RV=V1 X V2 (RV REFERED TO RECIPROCAL COORDINATES)
C  S MXV(PM,XM,V)              PM(3)=XM(3,3)*V(3)
C  S VPLUSV(SV,V1,V2)          SV(3)=V1(3)+V2(3)
C  S IVPLUV(ISV,IV1,IV2)       SUM INTEGER VECTORS
C  S VMINV(DV,V1,V2)           DV(3)=V1(3)-V2(3)
C  S IVMINV(IDV,IV1,IV2)       DIFFERENCE OF INTEGER VECTORS
C  S MPLUSM(SM,XM1,XM2,N)      SM(N,N)=XM1(N,N)+XM2(N,N)
C  S MMINM(DM,XM1,XM2,N)       DM(N,N)=XM1(N,N)-XM2(N,N)
C  S MXM(PM,XM1,XM2,N)         PM(N,N)=XM1(N,N)*XM2(N,N)
C  S AXA(C,B,A,L,M,N,MC,MB,MA) C(l,n) = B(l,m)*A(m,n)
C  S AXV(C,B,A,M,N,MB)         C(m) = B(m,n,)*A(n)
C  S APLUSA(C,B,A,M,N,MC,MB,MA) C(m,n) = B(m,n) + A(m,n)
C  S AMINA(C,B,A,M,N,MC,MB,MA) C(m,n) = B(m,n) - A(m,n)
c  S TRANSA(B,A,N,M,MB,MA)     B is the transpose of A(n,m)
C  S ZEROA(A,N,M,MA)           A(n,m) is set to zero
C  S UNITM(A,N,MA)             A(n) is generated as a unitary matrix
C  F ICOMMA(XM1,XM2,N,ID)      =0 IF(XM1(N,N).EQ.XM2(N,N)) TO ID DECIMAL PLACES
C  F ICOMVE(V1,V2,N,ID)        =0 IF(V1(N).EQ.V2(N)) TO ID DECIMAL PLACES
C  F JCOMVE(IV1,IV2,N)         =0 IF(IV1(N).EQ.IV2(N))
C  F KCOMVE(V1,V2,ID)          =0 IF V1(3) AND V2(3) ARE IN SAME OR OPPOSITE
C                                 DIRECTION
C  S POLAR(VP,VX,Z,X,AM,RM)    VP=POLAR COORDS OF VX. Z=POLAR AXIS, X LIES IN
C                                 PLANE OF REFERENCE DIRECTION
C  S ORTHIEN(XM,XZ,XX)         X(cryst)=XM*X(orth) IN ARBITRARY ORIENTATION
C  S ORTHOM(OM,DM,IFLAG)       X(orth)=OM*X(cryst),X(cryst)=CM*X(orth)
C +S DIAG(B,BO,QP,A,IER)       DIAGONALISES B IN COORDS A(3,3). EIGENVALUE=BO
C                                 EIGENVECTORS=QP
C
C               Input output routines
C               ---------------------
C  S SCAN(G,N,X,EX,I,H,IE)     READS AND INTERPRETES NEXT STRING IN =H=, STAR-
C                              TING AT POSITION =I= TILL NEXT BLANK OR COMMA
c  S HSCAN(H,N,X,EX,I,H,IE)    AS SCAN BUT CAN RETURN CHARACTER *80 STRINGS
C  F SECOND(I)                 RETURNS TIME (MACHINE DEPENDENT, SEVERAL VERS.)
C *S READYN(*1,*2,*3,*4)       READS (Y/N/E/error) AND RETURNS
C *S WAIT                      REQUESTS [CR] IN ORDER TO CONTINUE
C  F HWORD(X,EX)               CODES NUMBER AND ERROR INTO A10 AS INTEGER
C  F HXWORD(X,EX)              CODES A NUMBER AND ERROR INTO A14 WITH ROUND OFF
C  S UPCASE(HLINE)             LEFT JUSTIFIES A CHARACTER STRING IN ITS FIELD
C                              AND CONVERTS ALL LETTERS TO UPPER CASE
c  s lowcase(hline)            convert to lower case
C  F HLEFT(H,N)                RETURNS N CHARACTERS FROM H LEFT JUSTIFIED
C  F HRIGHT(H,N)               RETURNS CHARACTERS H(1:N) RIGHT JUSTIFIED TO N
C  F LENGTH(HLINE)             RETURNS THE ACTUAL LENGTH OF HLINE
C  S EFNUM(X,EX,H,NFX,NFE)     DECODES A(NFX) IN =H= AS REAL =X= AND A(NFE) IN
C                              =H= AS CORRESPONDING REAL ERROR =EX=
C  F NUM(H,NFL)                RETURNS A(NFL) IN =H= AS INTEGER. BOMB-PROOF
C  F MAXDIM(ND,MD,H)           IF (ND.G.MD ND=MD, MAXDIM=1,) ELSE MAXDIM=0
C  F GINT(I)                   RETURNS =I= AS A CHARACTER
C  S INERR(*)                  PRINT INTERACTIVE I/O ERROR MESSAGE
c  F intread()                 returns an integer from next input record
C                 General routines
C                 ----------------
c  F largest(x,n,xl)           = address of largest x(n), xl is value
c  F lsmallst(x,n,s)          = address of smallest x(n), s is value
C  S SORTX(N,IX,IS)            SORTS ADRESSES OF IX(N) IN DESC. ORDER TO IS(N)
C  S SORTY(N,Y,IS)             SORTS ADRESSES OF Y(N) IN DESC. ORDER TO IS(N)
C  F 'INCH'(G)                 CONVERTS CHARACTER 'G' TO 'INCH' AND 'RECH'
C  F LIMIT(X,DMAX,DMIN)        =0 IF X(3) OUTSIDE RANGE (DMAX(3),DMIN(3))
C  F JTEST(X,Y)                =0 IF ABS(X-Y).LT.0.0008
C  F RHALF(X,DMAX)             = X+.5 MOD(DMAX,1)
C  F THIRD(X,DMAX)             = X+.3333 MOD(DMAX,1)
C  F TWO(X,DMAX)               = X+.6667 MOD(DMAX,1)
C  S PAGEFLIP(*,*,*)           FLIPS BETWEEN TEXT PAGES (NEXT,PREV,EXIT)
C
C   * interactive i/o in this routine
C   + machine dependent code
C
C  THE FOLLOWING EXTERNALS ARE CALLED:
C
C  FTN-ACOS, FTN-ALOG, FTN-SQRT, FTN-COS
C  LIB$INIT_TIMER, LIB$SHOW_TIMER, LIB$STAT_TIMER
c  CERNLB: reqn
C
C
C****** THERE MAY BE PROBLEMS WITH THE MACHINE DEPENDENT FUNCTION =SECOND=
C       CHECK THE CODE WHEN CHANGING MACHINES
C
C
C
C
C
C
      FUNCTION INTEGER(X,N)
C
C   *****FUNCTION*****     INTEGER
C                          -------
C
C   WRITTEN 81.1.7 BY I D BROWN
C
C   RETURNS THE VALUE OF X*10**N AS THE NEAREST INTEGER
C
      Y = 10.**n
      IF(X.LT.0.) GO TO 10
      INTEGER = int(X*Y + 0.5)
      RETURN
   10 INTEGER = int(X*Y - 0.5)
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      FUNCTION ARCOS(X)
C
C    *****FUNCTION*****     ARCOS
C                           -----
C
C   WRITTEN 82.3.1 BY I D BROWN
C
C   CALLS ACOS BUT SETS ARGUEMENT TO 1 IF GT 1
C
      IF(ABS(X).LE.1.) GO TO 10
      XNEW = X/ABS(X)
      if(abs(x).gt.1.001) WRITE(6,100)X,XNEW
  100 FORMAT(10H **ARCOS**,5X,'WARNING: VALUE ',G10.3,' SET TO ',F4.1,
     +' IN ARCOS '   )
      X = XNEW
   10 ARCOS = ACOS(X)
      RETURN
      END
C
C
C
C
      FUNCTION ELOG(X)
C
C   *****FUNCTION*****     ELOG
C                          ----
C
C   WRITTEN 82.11.9 BY I D BROWN
C
C   CALLS ALOG BUT SETS ARGUMENT TO 1E-37 IF ARG.LE.1E-37
C
      XMIN = 1E-37
      Y = X
      IF(X.GE.XMIN) GO TO 50
      WRITE(6,1000) X
 1000 FORMAT(' ++ELOG++1000++ ARGUMENT = ',G10.3,' REPLACED BY '
     + ,'1E-37.')
      Y = XMIN
   50 ELOG = ALOG(Y)
      RETURN
      END
C
C
C
C
      FUNCTION SQART(X)
C
C   *****FUNCTION*****     SQART
C                          -----
C   WRITTEN 82.3.1 BY I D BROWN
C
C   CALLS SQRT BUT SETS NEGATIVE ARGUMENT TO ZERO
C
      IF(X.GE.0.0) GO TO 10
      if(x.lt.-0.000000001) WRITE(6,100) X
  100 FORMAT(10H **SQART**,5X,'WARNING: VALUE ',G10.3,' RESET TO ZERO'
     +  ,' IN SQART')
      X = 0.00000001
   10 SQART = SQRT(X)
      RETURN
      END
C
C
C
C*************************************************
c machine dependent code follows
c*************************************************
*--VAX
*     OPTIONS/G_FLOATING
* --PC
*
      FUNCTION DSQART(DX)
C
C   *****FUNCTION*****     DSQART
C                          ------
C   MODIFIED FROM SQART 85.09.29 BY DAVID KNIGHT
C
C   CALLS DSQRT BUT SETS NEGATIVE ARGUMENT TO ZERO
C
      DOUBLE PRECISION DSQART, DX
      IF(DX.GE.0.0) GO TO 10
      if(Dx.lt.-0.000000001) WRITE(6,100) DX
  100 FORMAT(' **DSQART**',5X,'WARNING: VALUE ',G14.7,' RESET TO ZERO '
     +  ,'IN DSQART')
      X = 0.00000001
   10 DSQART = DSQRT(DX)
      RETURN
      END
C
C
C
C
      SUBROUTINE MEAN(X,XM,SIG,N)
C     ---------------------------
C
C   WRITTEN 83.11.29 BY I D BROWN
C
C   CALLS SQUART
C
C   UPDATES THE MEAN 'XM' AND STANDARD DEVIATION 'SIG' OF A COLLECTION
C   OF 'N' NUMBERS BY THE ADDITION OF A NEW NUMBER 'X'.
C
      M=N+1
      XO=XM
      XM=(N*XO+X)/M
      SIG=SQART((N*(SIG*SIG+XO*XO)+X*X)/M-XM*XM)
      N=M
      RETURN
      END
C
C
C
C
      FUNCTION DIST(A,AM)
C
C   *****FUNCTION*****     DIST
C                          ----
C
C    WRITTEN 82.3.1 BY I D BROWN
C
C    CALLS DOT, SQUART
C
C    CALCULATES THE LENGTH OF THE VECTOR A PUSING THE METRIC AM
C
      DIMENSION A(3),AM(9)
      X = DOT(A(1),A(1),AM(1))
      DIST = SQART(X)
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
      FUNCTION DOT(A,B,AM)
C
C   *****FUNCTION*****     DOT
C                          ---
C
C    WRITTEN 82.3.1 BY I D BROWN
C
C    CALCULATES THE DOT PRODUCT BETWEEN A AND B USING THE METRIC AM
C
      DIMENSION A(3),B(3),AM(3,3)
      X = 0
      DO 50 I=1,3
	 DO 50 J=1,3
   50       X = X + A(I)*AM(I,J)*B(J)
C     WRITE(6,1000) X,A,B,AM
C   THIS COMMAND CAN BE REACTIVATED FOR DEBUGGING PURPOSES
 1000 FORMAT(3X,16F8.4)
      DOT = X
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
      subroutine cross(c,a,b)
C
C   written 86.7.8 by I.D.Brown
c
c   This returns in c(3) the cross product of a(3) and b(3)
c   The answer is correct for orthogonal coordinate systems.  For
c   non-orthogonal coordinate systems c(3) should be multiplied by
c   the vector (Va*, Vb* Vc*) where V is the unit cell volume and
c   a*,b*,c* are the reciprocal cell lengths.   The components of the
c   cross product are in the directions of the reciprocal cell axes.
c
      dimension a(3),b(3),c(3)
c
      c(1) = a(2)*b(3)-b(2)*a(3)
      c(2) = a(3)*b(1)-b(3)*a(1)
      c(3) = a(1)*b(2)-b(1)*a(2)
      return
      end
C
C
c
      function angle(a,b,am)
c     ----------------------
c
c   written 86.12.10 by I.D.Brown
c
c   calculates the angle in degrees between a and b in the system
c   with metric tensor am
c
c   calls rangle
c
      dimension a(3),b(3),am(3,3)
      angle=57.2578*rangle(a,b,am)
      return
      end
c
c
c
      FUNCTION RANGLE(A,B,AM)
C
C   *****FUNCTION*****     RANGLE
C                          -----
C
C   WRITTEN 82.3.1 BY I D BROWN
C
C   CALLS ARCOS, DIST, DOT
C
C   CALCULATES THE ANGLE (in radians) BETWEEN VECTORS A AND B
c     USING THE METRIC AM
C
      DIMENSION A(3),B(3),AM(3,3)
      DIS1=DIST(A(1),AM(1,1))
      DIS2=DIST(B(1),AM(1,1))
      DIS3=DIS1*DIS2
      IF(ABS(DIS3).LT.0.00000000001) THEN
	ANGLE = 0.0
	rangle= 0.0
	RETURN
	ENDIF
      rangle = ARCOS(DOT(A(1),B(1),AM(1,1))/DIS3)
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
c
      subroutine rotx(xm,a)
C
c    written 86.7.9 by I.D.Brown
c
c    xm is the matrix corresponding to a rotation around the x-axis through
c    an angle a radians
c
      dimension xm(3,3)
c
      c=cos(a)
      s=sin(a)
      xm(1,1)=1.
      xm(1,2)=0.
      xm(1,3)=0.
      xm(2,1)=0.
      xm(2,2)=c
      xm(2,3)=-s
      xm(3,1)=0.
      xm(3,2)=s
      xm(3,3)=c
      return
      end
c
c
c
      subroutine roty(xm,a)
C
c    written 86.7.9 by I.D.Brown
c
c    xm is the matrix corresponding to a rotation around the y-axis through
c    an angle a radians
c
      dimension xm(3,3)
      c=cos(a)
      s=sin(a)
   10 continue
      xm(1,1)=c
      xm(1,2)=0.
      xm(1,3)=-s
      xm(2,1)=0.
      xm(2,2)=1.
      xm(2,3)=0.
      xm(3,1)=s
      xm(3,2)=0.
      xm(3,3)=c
      return
      end
c
c
c
      subroutine rotz(xm,a)
C
c    written 86.7.9 by I.D.Brown
c
c    xm is the matrix corresponding to a rotation around the z-axis through
c    an angle a radians
c
      dimension xm(3,3)
c
      c=cos(a)
      s=sin(a)
      xm(1,1)=c
      xm(1,2)=-s
      xm(1,3)=0.
      xm(2,1)=s
      xm(2,2)=c
      xm(2,3)=0.
      xm(3,1)=0.
      xm(3,2)=0.
      xm(3,3)=1.
      return
      end
c
c
C
      SUBROUTINE ROTVEC(XM,A,B)
C
c    written 86.7.9 by I.D.Brown
c    updated 87.7.9 by D.Lawrence
c    updated 89.8.28 by s.urquhart
c    ROTVEC:     
c      xm is the matrix which rotates A into the direction of B
C    ROTANG:
C      xm is the matrix which rotates through angab radians about axis c(3)
c    NOTE THAT THE COORDINATES MUST BE IN A UNITARY ORTHOGONAL SYSTEM
c
c     calls rangle, roty, rotz, mxm, cross
c
      common/files/nin,nout,ndebug
      dimension xm(3,3),a(3),b(3),am(3,3),x1(3,3),x2(3,3),zm(3,3)
      dimension zi(3,3),ym(3,3),yi(3,3),a1(3,3),y(3),z(3),c(3),d(3)
      dimension cc(3),cekrot(3)
      data y,z/0,1,0,0,0,1/
      data am/1,0,0,0,1,0,0,0,1/
c
c   clear the array xm
c
      do 13 ia=1,3
	do 12 ib=1,3
	  xm(ia,ib)=0
 12       continue
 13     continue
c
c   test for null vectors
c
      if((abs(a(1)).lt.0.000001.and.abs(a(2)).lt.0.000001.and.
     +  abs(a(3)).lt.0.000001).or.(abs(b(1)).lt.0.000001.and.
     +  abs(b(2)).lt.0.000001.and.abs(b(3)).lt.0.000001)) then
	write(nout,1900)
 1900   format(' *ERROR*ROTVEC*1900 input vector is null vector
     +  *RETURN*')
	return
	endif
c
c   check for unit vectors
c
      totala=0.0
      totalb=0.0
      do 20 i=1,3
	totala=totala+a(i)**2
	totalb=totalb+B(i)**2
 20     continue
      totala=sqrt(totala)
      totalb=sqrt(totalb)
      if(abs(totala-1.0).gt.0.000001.OR.abs(totalb-1.).gt.0.000001)then
	 do 25 i=1,3
	   a(i)=a(i)/totala
	   b(i)=b(i)/totalb
 25        continue
	end if
c
c   c is perpendicular to a and b
c
      call cross(c,a,b)
      angab=rangle(a,b,am)
      go to 50
      ENTRY ROTANG(XM,AN,CC)
      angab = an
      do 30 i=1,3
   30   c(i) = cc(i)
   50 angzc = rangle(z,c,am)
c
c   d is perpendicular to c and z
c
      call cross(d,z,c)
      angyd = rangle(y,d,am)
      if(d(1).ge.0.) then
	call rotz(zm,-angyd)
	call rotz(zi,angyd)
      else
	call rotz(zm,angyd)
	call rotz(zi,-angyd)
	endif 
      if(c(2).ge.0.) then
	call roty(ym,-angzc)
	call roty(yi,angzc)
       else 
	call roty(ym,-angzc)
	call roty(yi,angzc)
	endif
      call mxm(x1,zm,ym,3)
      call rotz(a1,angab)
      if (abs(angab-acos(-1.0)).lt.0.001.and.(abs(a(1)-0).lt.0.001.and.
     + abs(a(2)-0).lt.0.001.and.abs(abs(a(3))-1.).lt.0.001)) then
	a1(3,3)=-1.
	A1(1,1)=1.
	A1(2,2)=1.
	end if
      call mxm(x2,x1,a1,3)
      call mxm(x1,x2,yi,3)
      call mxm(xm,x1,zi,3)
      call mxv(cekrot,xm,a)
      nd=2
      nsiz=3
      if (icomve(cekrot,b,nsiz,nd).ne.0) then
	call mxm(x1,ym,zm,3)
	call mxm(x2,a1,x1,3)
	call mxm(x1,yi,x2,3)
	call mxm(xm,zi,x1,3)
	call mxv(cekrot,xm,a)
	end if
      if(ndebug.ne.0) then
	write(ndebug,1000) angab,angzc,angyd
 1000   format(' *debug*rotvec*1000* angab=',F6.3,' angzc=',F6.3,
     +                      ' angyd=',F6.3)
	write(ndebug,1001) zm,ym,a1
 1001   format(' *debug*rotvec*1001* zm=',9F5.2,/,
     +         '                     ym=',9F5.2,/,
     +         '                     a1=',9F5.2)
	write (ndebug,1002) (c(i),i=1,3),(d(i),i=1,3)
1002    format (' *debug*rotvec*1002* c=',3F7.4,/,
     +          '                     d=',3F7.4)
	write (ndebug,1003) a,b
1003    format (' *debug*rotvec*1003* a=',3F7.4,/,
     +          '                     b=',3F7.4)
	endif
      if (icomve(cekrot,b,nsiz,nd).ne.0) then
	write(nout,1090)
 1090   format('*** ERROR *** Cannot calculate rotation matrix')
	do 110 ia=1,3
	  do 120 ib=1,3
	    xm(ia,ib)=am(ia,ib)
  120       continue
  110     continue
	endif
      if (abs(totala-1.0).gt.0.001.OR.abs(totalb-1.).gt.0.001) then
	 do 85 i=1,3
	   a(i)=a(i)*totala
	   b(i)=b(i)*totalb
85         continue
	end if
      return
      end
c
c
      subroutine rotcom(z,t,x,n)
c     ------------------------
c
c written 1988.3.10 by J. Merrall
c
c this subroutine rotates n atoms in array x(3,n) to new
c co-ordinates z(3,n) through the matrix t
c
c externals called:
c MATHLB: mxv
c
      dimension x(3,n),z(3,n),t(3,3)
c
      do 200 i=1,n
	call mxv(z(1,i),t,x(1,i))
 200    continue
 999  return
      end
C
C
C
      SUBROUTINE METRIC(A,B,C,AL,BE,AGA,AM)
C
C   *****SUBROUTINE*****     METRIC
C                            ------
C
C   WRITTEN 82.3.2 BY I D BROWN
C
C   CALCULATES METRIC TENSOR, AM, FROM THE CELL A,B,C,AL,BE,GA
C
      DIMENSION AM(3,3)
      CONV = 0.01745329
      AM(1,1) = A*A
      AM(2,2) = B*B
      AM(3,3) = C*C
      AM(1,2) = A*B*COS(CONV*AGA)
      AM(2,1) = AM(1,2)
      AM(2,3) = B*C*COS(CONV*AL)
      AM(3,2) = AM(2,3)
      AM(1,3) = A*C*COS(CONV*BE)
      AM(3,1) = AM(1,3)
      RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
      END
C
C
C
C
C
      SUBROUTINE TRANSP(A,B,N)
C
C   WRITTEN 85.7.26 BY I.D.BROWN
C
C   A(N,N) IS THE TRANSPOSE OF B(N,N)
C
      dimension a(n,n),b(n,n)
      do 10 i=1,n
	do 10 j=1,n
   10     a(i,j)=b(j,i)
      return
      end
C
C
C
      SUBROUTINE VXV(RV,V1,V2)
C
C   Written 86.4.1 by I.D.Brown
c
c    RV(3) contains the cross product  V1xV2.
c
c ***************************************************************************
c ****** NB RV is refered to the RECIPROCAL CELL if V1 and V2 are refered ***
c ****** to the REAL CELL and vice versa                                  ***
c ***************************************************************************
C
      dimension rv(3),v1(3),v2(3)
      rv(1)=v1(2)*v2(3)-v1(3)*v2(2)
      rv(2)=v1(3)*v2(1)-v1(1)*v2(3)
      rv(3)=v1(1)*v2(2)-v1(2)*v2(1)
      return
      end
C
C
      SUBROUTINE MXV(VPROD,SM,V)
C
C  *****SUBROUTINE*****     MXV
C                           ---
C
C   WRITTEN 83.9.7 BY I D BROWN
C
C   CALCULATES VPROD(3) = SM(3,3)*V(3)
C
      DIMENSION VPROD(3),SM(3,3),V(3)
      DO 50 I=1,3
	 VPROD(I) = 0
	 DO 50 J=1,3
   50       VPROD(I) = VPROD(I) + SM(I,J)*V(J)
      RETURN
      END
C
C
C
C
      SUBROUTINE VPLUSV(VPROD,V1,V2)
C
C   ******SUBROUTINE******    VPLUSV
C                             ------
C
C   WRITTEN 89.9.7 BY I D BROWN
C
C   CALCULATES VPROD(3) = V1(3) + V2(3)
C
      DIMENSION VPROD(3),V1(3),V2(3)
      DO 50 I=1,3
   50    VPROD(I) = V1(I) + V2(I)
      RETURN
      END
C
C
C
C
      SUBROUTINE IVPLUV(K,I,J)
C
C   ******SUBROUTINE******    IVPLUV
C                             ------
C
C   WRITTEN 89.9.7 BY I D BROWN
C
C   CALCULATES K(3) = I(3) + J(3)
C
      DIMENSION K(3),I(3),J(3)
      DO 50 L=1,3
   50    K(L) = I(L) + J(L)
      RETURN
      END
C
C
C
C
      SUBROUTINE VMINV(Z,X,Y)
C
C   *****SUBROUTINE***** VMINV
C                        -----
C
C   WRITTEN 83.9.23 BY I D BROWN
C
C   CLACULATES Z(3) = X(3)-Y(3)
C
      DIMENSION X(3),Y(3),Z(3)
      DO 10 I=1,3
   10   Z(I) = X(I) - Y(I)
      RETURN
      END
C
C
C
C
      SUBROUTINE IVMINV(K,I,J)
C
C   *****SUBROUTINE***** IVMINV
C                        ------
C
C   WRITTEN 83.9.23 BY I D BROWN
C
C   CALCULATES K(3) = I(3)-J(3)
C
      DIMENSION K(3),I(3),J(3)
      DO 10 L=1,3
   10   K(L) = I(L) - J(L)
      RETURN
      END
C
C
C
C
      SUBROUTINE MPLUSM(C,A,B,N)
C      -------------------------
C
C   WRITTEN 84.1.25 BY I.D.BROWN
C
C   CALCULATES C(N,N)=A(N,N)+B(N,N)
C
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 50 I=1,N
	DO 50 J=1,N
   50     C(I,J)=A(I,J)+B(I,J)
      RETURN
      END
C
C
C
C
      SUBROUTINE MMINM(C,A,B,N)
C     -------------------------
C
C   WRITTEN 84.1.25 BY I.D.BROWN
C
C   CALCULATES C(N,N)=A(N,N)-B(N,N)
C
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 50 I=1,N
	DO 50 J=1,N
   50     C(I,J)=A(I,J)-B(I,J)
      RETURN
      END
*
*
*
*
      SUBROUTINE MXM (C,A,B,N)
*---- CALCULATES THE MATRIX PRODUCT FOR N*N MATRICES AS
*     A * B = C
*
*     A, B   INPUT MATRICES
*     C      OUTPUT MATRIX
*     N      ORDER OF MATRICES
*
*     WRITTEN BY D. ALTERMATT
*---- MCMASTER UNIVERSITY, DAWN OF ORWELL'S YEAR
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 10 I=1,N
      DO 20 K=1,N
      C(I,K)=0
      DO 30 J=1,N
      C(I,K)=C(I,K)+A(I,J)*B(J,K)
   30 CONTINUE
   20 CONTINUE
   10 CONTINUE
      RETURN
      END
*
*
*
       subroutine axa(c,b,a,l,m,n,mc,mb,ma)
c
c   written 88.6.17 by I.D.Brown
c
c    multiplies asymmetric matrices
c    c(l,n) = b(l,m)*a(m,n)
c
      dimension a(ma,n),b(mb,m),c(mc,n)
c
      do 100 i=1,l
	do 100 j=1,n
	  c(i,j)=0.
	  do 100 k=1,m
  100       c(i,j) = c(i,j) + b(i,k)*a(k,j)
      return
      end
c
c
c
      subroutine axv(c,b,a,m,n,mb)
c
c   written 88.6.21 by I.D.Brown
c
c    c(m) = b(m,n)*a(n)
c
      dimension a(n),b(mb,n),c(m)
c
      do 100 i=1,m
	c(i) = 0
	do 90 j=1,n
	  c(i) = c(i) + b(i,j)*a(j)
   90     continue
  100   continue
      return
      end
c
c
c
      subroutine aplusa(c,b,a,n,m,mc,mb,ma)
c
c   written 88.6.6 by I.D.Brown
c
c   c(n,m) = a(n,m) + b(n,m)
c
      dimension a(ma,m),b(mb,m),c(mc,m)
c
      do 50 i=1,n
	do 40 j=1,m
	  c(i,j) = b(i,j) + a(i,j)
   40     continue
   50   continue
      return
      end
c
c
c
      subroutine amina(c,b,a,n,m,mc,mb,ma)
c
c   written 88.6.6 by I.D.Brown
c
c   c(n,m) = a(n,m) - b(n,m)
c
      dimension a(ma,m),b(mb,m),c(mc,m)
c
      do 50 i=1,n
	do 40 j=1,m
	  c(i,j) = b(i,j) - a(i,j)
   40     continue
   50   continue
      return
      end
c
c
c
      subroutine transa(b,a,n,m,mb,ma)
c
c   written 88.6.6 by I.D.Brown
c
c   transposes the asymmetric matrix a(n,m) to b(m,n)
c
      dimension a(ma,m),b(mb,n)
c
c
      do 50 i=1,n
	do 40 j=1,m
	  b(j,i) = a(i,j)
   40     continue
   50   continue
      return
      end
c
c
c
      subroutine zeroa(a,n,m,ma)
c
c    written 88.6.6 by I.D.Brown
c
c   Sets the asymmetric matrix a(n,m) to zero
c
      dimension a(ma,m)
c
      do 50 i=1,n
	do 40 j=1,m
	  a(i,j) = 0.
   40     continue
   50   continue
      return
      end
c
c
c
      subroutine unitm(a,n,ma)
c
c   written 88.6.6 by I.D.Brown
c
c   generates a(n,n) as a unitary matrix
c
      dimension a(ma,n)
c
      call zeroa(a,n,n,ma)
      do 50 i=1,n
	a(i,i) = 1.0
   50   continue
      return
      end
c
*
*
      FUNCTION ICOMMA (A,B,N,ID)
*---- COMPARES TWO N*N MATRICES, GIVES ICOMMAT=0 IF THEY
*     ARE EQUIVALENT WITHIN ID DECIMAL PLACES, ELSE ICOMMAT=1
*
*     CALLS INTEGER
*
*     A, B  MATRICES TO CHECK
*     N     ORDER OF MATRICES
*     ID    NUMBER OF DECIMAL PLACES TO CHECK
*
*     WRITTEN BY D. ALTERMATT
*     MODIFIED 84.5.16 BY I D BROWN
*---- MCMASTER UNIVERSITY, DAWN OF ORWELL'S YEAR
      DIMENSION A(N,N),B(N,N)
      ICOMMA=0
      DO 10 I=1,N
      DO 20 J=1,N
	IF(INTEGER(A(I,J),ID).NE.INTEGER(B(I,J),ID)) THEN
	  ICOMMA=1
	  RETURN
	  ENDIF
   20 CONTINUE
   10 CONTINUE
   99 RETURN
      END
C
C
C
C
      FUNCTION ICOMVE (X,Y,N,ID)
C     --------------------------
C
C   WRITTEN 84.2.7 BY I D BROWN
C
C   RETURNS THE VALUE OF 0 IF X(N) AND Y(N) ARE IDENTICAL TO ID DECIMAL
C   PLACES
C
      DIMENSION X(N),Y(N)
      ICOMVE=0
      DO 100 I=1,N
  100   IF(INTEGER(X(I),ID).NE.INTEGER(Y(I),ID)) ICOMVE=1
      RETURN
      END
c
c
C--------------------------------------------------------------------
      FUNCTION JCOMVE(IV1,IV2,N)
C--------------------------------------------------------------------
C
c  written 86.3.17 by I.D.Brown
c
c  returns jcomve=0 if iv1(n).eq.iv2(n)
c
      dimension iv1(n),iv2(n)
c
      jcomve=1
      do 100 i=1,n
  100   if(iv1(i).ne.iv2(i)) return
      jcomve=0
      return
      end
c
c
c
c
      function kcomve(v,w,n)
c     ----------------------                 written 1988.5.17
c                                              by J. Merrall
c
c     this function =0 if vectors v(3) and w(3) are multiples of each
c     other to n decimal places.
c
c     (in other words, the function is the same as icomve except
c     that the vectors need not be equal, but can be real multiples.)
c
c     WARNING: in case of V or W being null vectors, the function will
c     AUTOMATICALLY be set to 0.
c
c     internals called:
c     MATHLB: icomve,dist
c
      dimension v(3),w(3),x(3),um(3,3)
c
      data um/1.0,0,0,0,1.0,0,0,0,1.0/
c
      kcomve=0
c
      if(abs(dist(w(1),um)).lt.0.0001.or.
     +   abs(dist(v(1),um)).lt.0.0001) return
c
      do 100 i=1,3
 100    if(abs(w(i)).gt.0.0001) ratio=v(i)/w(i)
c
      do 200 i=1,3
 200    x(i)=ratio*w(i)
c
      kcomve=icomve(v,x,3,n)
c
      return
      end
c
c
c
c--------------------------------------------------------------------
      SUBROUTINE POLAR(VP,VX,Z,X,AM,RM)
C--------------------------------------------------------------------
C
C  WRITTEN 86.4.1 by I.D.Brown
c
c   This subroutine returns the polar coordinates (r,theta,phi) in VP
c   of the vector whose cartesian coordinates are given in VX.
c   The angles are given in degrees.
c   AM is the metric tensor of the cartesian system used for VX
c   and RM is the reciprocal metric tensor.  (For orthogonal cartesian
c   coordinates boh tensors are unit matrices). Z is a cartesian vector
c   that defines the polar (z) axis, and X is a cartesian vector which
c   lies in the plane of Z and the azimuthal reference direction.
c   NOTE vectors labelled 'y' are refered to the reciprocal cell
c
      dimension vp(3),vx(3),z(3),x(3),y(3),y1(3),am(3,3),rm(3,3),x1(3)
      vp(1)=dist(vx,am)
      vp(2)=angle(z,vx,am)
      call vxv(y1,z,x)
      call vxv(y,z,vx)
      vp(3)=angle(y1,y,rm)
c  to get the sense of the angle vp(3)
      call mxv(x1,rm,y)
      if (angle(x1,x,am).gt.90.) vp(3)=360.-vp(3)
      return
      end
c
c
c
      subroutine orthien(xm,xz,xx)
c     -------------------------------
c
c    written 86.12.5 by I.D.Brown
c
c    returns xm(3,3) which is the matrix that transforms an orthogonal
c    set of coordinates to the crystal system 
c    The orthogonal z axis is set along the 
c    crystal xz(3) direction and the orthogonal y axis is set perpendicular
c    to the crystal xx(3) direction.
c
c     calls orthom,mxv, rotvec, mxm
c
      common/files/nin,nout,ndebug
      common/cell/a,b,c,al,be,aga,vol,am(3,3),icell
      dimension xm(3,3),xz(3),xx(3),rm1(3,3),rm2(3,3),ox(3)
      dimension oz(3),ox1(3),om(3,3),cm(3,3),qm(3,3)
      dimension yx(3),yz(3)
      data oz/0,0,1/,ox/1,0,0/
c
      do 100 i=1,3
	do 100 j=1,3
  100     xm(i,j)=0.
c
c    first orthogonalise the crystal system relative to arbitrary axes
c    -----------------------------------------------------------------
      iflag=1
      call orthom(om,cm,iflag)
      if(ndebug.ne.0) write(ndebug,1001) om,cm
 1001 format(' *debug*orthien*1001* om=',9F5.2,' cm=',9F5.2)
      if(iflag.ne.0) then
	write(nout,1000) iflag
 1000   format(' *error*orthien*1000* problem ',I3,' in ',
     +     'orthogonalising matrix')
	return
	endif
      call mxv(yz,om,xz)
      call mxv(yx,om,xx)
      if(ndebug.ne.0) write(ndebug,1003) xz,xx,yz,yx
 1003 format(' *debug*orthien*1003 xz=',3F7.4,' xx=',3F7.4,' yz=',3F7.4,
     +                  ' yx=',3F7.4)
c
c    then reorient the axes xx and xz
c    --------------------------------
      call rotvec(rm1,oz,yz)
      call mxv(ox1,rm1,ox)
      call rotvec(rm2,ox1,yx)
      if(ndebug.ne.0) write(ndebug,1002) ox1,rm1,rm2
 1002   format(' *debug*orthien*1002*  ox1=',
     +            3F7.4,' rm1=',9F5.2,/,20X,' rm2=',9F5.2)

c 
c    then generate the required transformation matrix
c    ------------------------------------------------
      call mxm(qm,rm2,rm1,3)
      call mxm(xm,cm,qm,3)
      if(ndebug.ne.0) write(ndebug,1004) qm,xm
 1004 format(' *debug*orthien*1004* qm=',9F5.2,' xm=',9F5.2)
      return
      end
c
c
      subroutine orthom(om,cm,iflag)
c     ------------------------------
c
c   written 86.12.5 by I.D.Brown
c
c   returns om and cm where
c      x(orthogonal) = om*x(crystal)
c      x(crystal)    = cm*x(orthogonal)
c
c   iflag should determine the relative orientations of the axes.
c   iflag = 1: z is along c, y perpendicular to a
c   other orientations not yet programmed
c
c   on output iflag = 0 for successful completion
c                   = 1 for incorrect iflag input
c                   = -1 for incorrect input to rinv
c                   = -2 for om as singular matrix
c
c   call CERNLB: rinv
c
      common/cell/ a,b,c,al,be,aga,vol,am(3,3),icell
      common/rcell/ as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      dimension om(3,3),cm(3,3),vect(3)
c
      do 100 i=1,3
	do 100 j=1,3
	  om(i,j)=0.
	  cm(i,j)=0.
  100     continue
      if(iflag.eq.1) then
c                       z parallel c, y perpendicular a
c                       -------------------------------
	om(1,1)=a*sin(be)
	om(1,2)=-b*sin(al)*cos(rga)
	om(2,2)=1/bs
	om(3,1)=a*cos(be)
	om(3,2)=b*cos(al)
	om(3,3)=c
	do 200 i=1,3
	  do 200 j=1,3
  200       cm(i,j)=om(i,j)
	call rinv(3,cm,3,vect,ifail)
	iflag=ifail
      else
	iflag=1
	endif
      return
      end
C
C
C******************************************************
c machine dependent code follows
c******************************************************
*--VAX
*     OPTIONS/G_FLOATING
*--PC
      SUBROUTINE DIAG(b,bo,qp,a,ier)
C     ---------- -------
C
C  WRITTEN 1965 BY I.D.BROWN
C
c  This program diagonalises the matrix b(3,3) in the coordinate system whose
c   metric tensor is a(3,3).  The diagonal elements are given in bo(3)
c   and the eigenvectors are stored in qp(3,3).  If b is dimensionless, the
c   dimensions of bo are the inverse dimensions of a.  For atomic displacement
c   parameters b is the dimensionless tensor, a the reciprocal cell metric
c   and bo the principal components of the mean square displacement (A**2)
c
c   the equations solved are :
c
c    ||B(i,j)-y*A(i).A(j)|| = 0
c
c   ier = -1 if matrix is not positive definite
c          0 if eigenvalues are all different
c          1 if eigenvalues are all the same
c          2 if two eigenvalues are the same
c
C   Calls: MATHLB:SQART,DSQART
C
C
C     THIS SUBROUTINE HAS BEEN LIFTED FROM MOLG.
C     NEW TEMP SUBROUTINE AS TRIAL WRITTEN BY JKBrandon
C     THIS VERSION WRITTEN TO OVERCOME THE PROBLEMS OF ANALYSING THE
C       TEMP FACTOR MATRIX WHEN THE ATOM LIES ON A SYMMETRY ELEMENT.
C     JKB'S PRODUCTION RUNS HAVE TESTED MOST OF THE POSSIBLE CASES.
C     SOME REVISION MAY BE DESIRED TO ALLOW A MARGIN OF ERROR IN THE
C       FINAL DECIMAL PLACES SINCE THE PRESENT DECISIONS ARE ALL MADE
C       ON THE BASIS OF EXACT TESTS.
C     (LATEST CHANGE IS TO GIVE RMS ATOM DISPLACEMENTS IN A FROM THE
C       THERMAL TENSOR EIGENVALUES  4/5/67 JKB.(now in temper 86.4.23))
C     MODIFIED 1980 TO CRYST SYSTEM BY DOUG THIERS
c     modified 1986.4.22 by I.D.Brown to make it suitable for any matrix
c       diagonalisation
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
      DOUBLE PRECISION BB,DSQART,Q(3),TEMP1,TL
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR
      COMMON/FILEB/NVAL,NVALEN,NPAGE,NBOND,NINDEX,NTEMP,NMAP
      dimension bo(3),b(3,3),a(3,3),qp(3,3),p(3,3),z(3,3)
      dimension t(3,3),tt(3,3),xx(3),qq(3,1),d(3)
      pi=3.1415926
      ps=9.8696044
      ier=0
      DO 120 I=1,3
	do 110 j=1,3
  110     qp(i,j)=0.0
	qp(i,i)=1.0
	bo(i)=1.0
  120   continue
c
c   remove rounding off errors in a
c
      do 130 i=1,3
	do 130 j=1,3
	  if(abs(a(i,j)).lt.abs(0.00001*sqart(a(i,i))*sqart(a(j,j))))
     +      a(i,j)=0.
  130     continue
C
C     METHOD OF FINDING PRINCIPAL AXES LENGTHS AND DIRECTIONS FOLLOWS
C                  THAT OF J.WASER, A.C. 8, 731, (1955)
C     SOLVE TRICLINIC EIGENVALUE EQUATIONS
C     THIRD DEGREE EIGENVALUE EQN IS
C                  C1(Y**3) + C2(Y**2) + C3(Y**1) + C4 = 0
C
c      in this program Y is called BO
c
      C1=-A(1,1)*A(2,2)*A(3,3)
     1   +A(1,1)*A(2,3)*A(2,3)
     2   +A(2,2)*A(1,3)*A(1,3)
     3   +A(3,3)*A(1,2)*A(1,2)
     4   -A(1,2)*A(1,3)*A(2,3)*2.0
      C2=-B(1,1)*(A(2,3)*A(2,3)-A(2,2)*A(3,3))
     1   -B(2,2)*(A(1,3)*A(1,3)-A(1,1)*A(3,3))
     2   -B(3,3)*(A(1,2)*A(1,2)-A(1,1)*A(2,2))
     3   -B(1,2)*(A(1,2)*A(3,3)-A(1,3)*A(2,3))*2.0
     4   -B(1,3)*(A(1,3)*A(2,2)-A(1,2)*A(2,3))*2.0
     5   -B(2,3)*(A(2,3)*A(1,1)-A(1,2)*A(1,3))*2.0
      C3=-A(1,1)*(B(2,2)*B(3,3)-B(2,3)*B(2,3))
     1   -A(2,2)*(B(1,1)*B(3,3)-B(1,3)*B(1,3))
     2   -A(3,3)*(B(1,1)*B(2,2)-B(1,2)*B(1,2))
     3   -A(1,2)*(B(1,3)*B(2,3)-B(1,2)*B(3,3))*2.0
     4   -A(1,3)*(B(1,2)*B(2,3)-B(1,3)*B(2,2))*2.0
     5   -A(2,3)*(B(1,2)*B(1,3)-B(1,1)*B(2,3))*2.0
      C4=-B(1,1)*B(2,3)*B(2,3)
     1   -B(2,2)*B(1,3)*B(1,3)
     2   -B(3,3)*B(1,2)*B(1,2)
     3   +B(1,1)*B(2,2)*B(3,3)
     4   +B(1,2)*B(1,3)*B(2,3)*2.0
C
C     GET THIRD DEGREE EQN INTO FORM (Y**3) + P1(Y**2) + P2(Y) + P3 = 0
C
      P1=C2/C1
      P2=C3/C1
      P3=C4/C1
C
C     SUBSTITUTING Y=X-P1/3 GIVES EQN AS (X**3) + PA*(X) + PB = 0
C
      PA=(3.*P2-P1*P1)/3.
      PB=(2.*P1**3-9.*P1*P2+27.*P3)/27.
C
C     IF MATRIX IS POSITIVE DEFINITE, THEN WE GET THREE
C                  REAL ROOTS.  IMAGINARY ROOTS ARISE WHEN
C                  (PB**2)/4 + (PA**3)/27  IS GREATER THAN 0.
C                WHEN THIS OCCURS WE HAVE NON POSITIVE DEFINITE MATRIX.
C     QUIT ANALYSIS IF NON POS DEF MATRIX
C
      TEST1=(PB**2)/4.+(PA**3)/27.
      small=abs(0.00025*(pb**2))
c
c   test for symmetric matrix
c   -------------------------
      if(abs(test1).lt.small) then
	IF(PB.GE.0.) then
	  ssmall=sqart(small)
	  WRITE(NOUT,362) PB,ssmall
  362     FORMAT(' *WARNING*DIAG*362* VALUE ',G10.3,' SET TO ',G10.3)
	  PB = -ssmall
	  endif
	IF(ABS(PB).LT.0.000001) then
c
c  three components equal (isotropic)
c  ----------------------------------
	  ier=1
	  x1=0.0
	  x2=0.0
	  x3=0.0
	else
c
c  two components equal (cylindrical)
c  ----------------------------------
	  ier=2
	  X1=2.*(-PB/2.)**(1./3.)
	  X2=-(X1*0.5)
	  X3=X2
	  endif
c
c   test for non positive definite matrix
c   -------------------------------------
      else if(test1.gt.0.) then
	ier=-1
	go to 1000
c
c   matrix has three roots
c   ----------------------
      else
	ier=0
  350   TEMP=(-PA**3)/27.
	stemp=sqart(temp)
	vsmall=0.000000000001
	if(stemp.lt.vsmall) stemp=vsmall
	ZONK=(-PB*0.5)/STEMP
	IF(ZONK.LE.1.0) GO TO 1350
	ZONK=1.00000000
 1350   TONK=SQART(1.0 - ZONK**2)
	PHI=ATAN2(TONK,ZONK)
	if(pa.gt.0.) write(nout,1355) pa
 1355   format(' *error*diag*1355* Value of PA =',G10.3,'. It should ',
     +     'be negative')
	TEMP=-PA/3.
	X1=2.0*SQART(TEMP)*COS(PHI/3.)
	X2=2.0*SQART(TEMP)*COS(PHI/3.+4.*PI/3.)
	X3=2.0*SQART(TEMP)*COS(PHI/3.+2.*PI/3.)
	endif
C
C     THREE EIGENVALUES OF ORIG EQN ARE BO(I),I=1,2,3.
C
      BO(1)=X1-P1/3.
      BO(2)=X2-P1/3.
      BO(3)=X3-P1/3.
      if(ier.eq.1) go to 1000
c
C
C     CONSTRUCT UNIT EIGENVECTORS AND FIND DIRECTION COSINES BY DOTTING
C                  THEM WITH EACH OF THE B(I) AND A(I)
c     equation is  B.QP=bo*A.QP i.e. (B-bo*A).QP=0
C
      DO 510 L=1,3
	IF(ier.EQ.2.AND.L.Gt.2) then
	  do 410 i=1,3
	    t(i,3)=0.
	    if(abs(t(i,1)).lt.zr) t(i,3)=1.0
  410       continue
	  go to 500
	  endif
	DO 430 I=1,3
	  DO 430 J=1,3
  430       P(I,J)=B(I,J)-BO(L)*A(I,J)
	DO 699 I=1,3
	  DO 699 J=1,3
  699       Z(I,J)=ABS(P(I,J))
	DO 700 I=1,3
  700     Q(I)=0.0
c
c   check the matrix for zero elements
c
	ZR=0.00001*(z(1,1)+z(2,2)+z(3,3))
	IF((Z(1,2).LT.ZR).AND.(Z(1,3).LT.ZR).AND.(Z(2,3).LT.ZR))GOTO 701
	IF((Z(1,3).LT.ZR).AND.(Z(2,3).LT.ZR).AND.(Z(3,3).LT.ZR))GOTO 710
	IF((Z(1,2).LT.ZR).AND.(Z(2,3).LT.ZR).AND.(Z(2,2).LT.ZR))GOTO 720
	IF((Z(1,2).LT.ZR).AND.(Z(1,3).LT.ZR).AND.(Z(1,1).LT.ZR))GOTO 730
	IF((Z(1,3)+Z(2,3)).LT.ZR) GOTO 740
	IF((Z(1,2)+Z(3,2)).LT.ZR) GOTO 750
	IF((Z(1,2)+Z(1,3)).LT.ZR) GOTO 760
	GO TO 429
  701   itt=0
	DO 702 I=1,3
	  t(i,l)=0.
	  IF(Z(I,I).LT.ZR) then
	    Q(I)=1.0
	    t(i,l)=1.0
	    if(itt.ne.0) t(i,l)=-1.0
	    itt=itt+1
	    endif
  702     CONTINUE
	GO TO 431
  710   Q(3)=1.0
	t(1,l)=0.
	t(2,l)=0.
	t(3,l)=1.0
	GO TO 431
  720   Q(2)=1.0
	t(1,l)=0.
	t(2,l)=1.0
	t(3,l)=0.
	GO TO 431
  730   Q(1)=1.0
	t(1,l)=1.0
	t(2,l)=0.
	t(3,l)=0.
	GO TO 431
  740   t(1,l)=-p(2,1)
	t(2,l)=p(1,1)
	t(3,l)=0.
	go to 431
  750   t(1,l)=-p(3,1)
	t(2,l)=0.
	t(3,l)=p(1,1)
	go to 431
  760   t(1,l)=0.
	t(2,l)=-p(3,2)
	t(3,l)=p(2,2)
	go to 431
c  This program has been thoroughly reworked by I.D.B in 1986. The code
c  below was in the old program as was the code in upper case letters,
c  particularly the code involving Q which is not used in the current
c  version.  It seems to work in most cases but there are some problems
c  that I have not been able to trouble shoot.  The determinant of P
c   should be zero for each eigenvalue (and this seems to be the case
c  even when the program does not give sensible answers.  I assume that
c  the routine reqn is working correctly but the answers that come out
c  sometimes do not make sense. e.g. sum(j)[qp(j,i)*qp(j,i)] should be
c  unity in cases of orthogonal cells and close to unity otherwise, but
c  this is not always the case.
c
* 429   T1=(P(1,1)*P(2,3)-P(1,3)*P(2,1))
*       T2=(P(2,2)*P(1,3)-P(1,2)*P(2,3))
*       T3=(P(3,3)*P(1,2)-P(1,3)*P(3,2))
C
C     COMPONENTS OF THE LTH EIGENVECTOR HAVE LENGTHS IN THE RATIO
C                  Q(1)$Q(2)$Q(3)
C
*       Q(1)=T2*T3
*       Q(2)=T3*T1
*       Q(3)=T1*T2
* 431   BB=0.
*       DO 435 I=1,3
*         DO 435 J=1,3
* 435       BB=BB+B(I,J)*Q(I)*Q(J)
*       TEMP1=BB/BO(L)
*       TL=DSQART(TEMP1)
C
C     A UNIT VECTOR ALONG THE LTH EIGENVECTOR HAS QP(,L) AS COMPONENTS IN
C                  THE CELL corresponding to A.
C
*       IF(DABS(TL).LT.1.0E-38) THEN
*         WRITE(NTEMP,620) TL
* 620     FORMAT(' ***DIAG**620***   WARNING:  VALUE OF TL (=',G15.7,
*    +      ') RESET TO 1.0E-38')
*         TL=1.0E-38
*         ENDIF
*       DO 436 I=1,3
* 436     QP(I,L)=Q(I)/TL
  429   t(1,l)=p(2,1)*p(3,2)-p(2,2)*p(3,1)
	t(2,l)=p(1,2)*p(3,1)-p(1,1)*p(3,2)
	t(3,l)=p(1,1)*p(2,2)-p(1,2)*p(2,1)
  431   do 433 i=1,3
	  tt(i,l)=t(i,l)*t(i,l)
  433     continue
  510   continue
c
c  normalise the direction cosines by recognising that the (d.c's)**2 of the
c  unit cell axes in the coordinate system of the matrix principal axes
c  must sum to 1.
c
  500 continue
      do 520 i=1,3
  520   qq(i,1)=1.0
      call reqn(3,tt,3,xx,ifail,1,qq)
      if(ifail.eq.-1) then
	write(nout,1520)
 1520   format(' *error*diag*1520* Singular matrix in calculating ',
     +      'eigenvectors')
	go to 1000
      else if(ifail.eq.-2) then
	write(nout,1521) tt
 1521   format(' *error*diag*1521* improper values in TT=',9F8.3)
	go to 1000
	endif
      do 530 i=1,3
	if(qq(i,1).lt.0.) write(nout,1525) qq,t
 1525   format(' *error*diag*1525* qq =',3G10.3,/,'   t =',9G10.3)
	d(i)=sqart(qq(i,1))
	do 525 j=1,3
  525     qp(j,i)=d(i)*t(j,i)
  530   continue
 1000 RETURN
      END
C
C
c
c
C--------------------------------------------------------------------
       SUBROUTINE SCAN (GA,N,X,EX,IP,HLINE,IRFL)
C--------------------------------------------------------------------
C
C  Written 05 23 85, By D Clarkson
C  Rewritten 06 12 85, by D Altermatt
C
C  Purpose : SCAN is a general purpose scan routine that will read
C            the 80 character input string (HLINE) starting at
C            position IP until a seperator is found (ie. a blank or
C            a comma ). The characters read are then interpreted as
C            either INTEGERS, REALS, or CHARACTERS.
C
C  Notes: 1. Character strings;
C
C              If the character string read has a length > 8, the
C              string is read in it's entirety but the pointer (IP)
C              will be positioned back as if only 8 characters were
C              read.
C
C         2. Real numbers along with errors;
C
C              Valid formats are : 12.345+6
C                                  12.345(6)
C
C              The character before the error term must be a digit,
C              no space can be there.
C
C         3. Flags and special cases;
C
C              INUM = 1    if string read is numeric ( ie. INTEGER or REAL )
C              ICHR = 1    if string read is alphanumeric.
C              IDEC > 0    if a decimal point was read.
C              IPLS > 0    if an error term seperator was read.
C                          These are : '+'  and  '('
C              IL,IR       limits to the number string
C              IFL         local pointer
C              IP   = 99   if input string blank or end-of-file found
C
C Subroutines called : EFNUM ; to decode a real number and error.
C
C Functions called   : NUM   ; to decode an integer.
C
C Input  : IP    ; the start position.
C          HLINE ; the buffer holding the 80 character line.
C          IRFL  < 0: Reset pointer and set IRFL to -IRFL(found) if
C                     string read is larger than the 8 characters
C                     returned in character string GA
C               >= 0: Ignore character 9 onwards in character string
C
C Output : GA    ; 8 character buffer holds CHARACTER representation
C                  of expression found
C          N     ; holds the INTEGER value of the expression found
C          X,EX  ; REAL number and ERROR of the expression found
C          IP    ; the final line position pointer
C          IRFL  = 1   if CHARACTER string found
C                = 2   if INTEGER expression found
C                = 3   if REAL expression found
C                = 4   if CHARACTER plus INTEGER found
C                = 5   if CHARACTER plus REAL found
C                = 0   if nothing found
C                negativ  (see 'input')
C
C----------------------------------------------------------------------
       IMPLICIT CHARACTER *80 (H)
       IMPLICIT CHARACTER *8  (G)
       CHARACTER *1 BLNK,COMA,DEC,PLS,LPR,RPR,MNS
       DATA BLNK,COMA,DEC,PLS,LPR,RPR,MNS /' ',',','.','+','(',')','-'/
C
C  Initialization
C  note : the string is assumed to be numeric to start
C
       IRFL1=IRFL
       IL=0
       IR=0
       IFL=0
       IFL2=0
       INUM=1
       ICHR=0
       IDEC=0
       IPLS=0
       GA='        '
       HTEMP=' '
       N=0
       X=0.
       EX=0.
C
C  Check pointer
C
       IF (IP.GT.80) THEN
	 IP = 99
	 IRFL=0
	 RETURN
	 ENDIF
C
C  Skip over leading blanks or commas
C
100    IF (HLINE(IP:IP).NE.BLNK.AND.HLINE(IP:IP).NE.COMA) GOTO 200
       IP=IP+1
       IF (IP.EQ.81) THEN
	 IP=99
	 IRFL=0
	 RETURN
       ENDIF
       GOTO 100
C
C  Load HTEMP with characters until a seperator is found or end of line
C
200    IFL=IFL+1
       HTEMP(IFL:IFL)=HLINE(IP:IP)
C
C  The following statements check what was read in.
C  ie. was a decimal point read ?
C      was an error seperator read ?
C      was a character outside the allowed numeric range read ?
C      - valid numeric characters are [ 0..9 + - ( ) ]
C
       IF (HLINE(IP:IP).EQ.DEC) IDEC=IFL
       IF (HLINE(IP:IP).EQ.PLS.OR.HLINE(IP:IP).EQ.LPR) THEN
	 IF (IP.GT.1.AND.HLINE(IP-1:IP-1).GE.'0'.AND.HLINE
     1     (IP-1:IP-1).LE.'9'.AND.IPLS.EQ.0.AND.INUM.EQ.1) IPLS=IFL
	 ENDIF
       IF ((HLINE(IP:IP).LT.'0'.OR.HLINE(IP:IP).GT.'9').AND.
     1      HLINE(IP:IP).NE.PLS.AND.HLINE(IP:IP).NE.MNS.AND.
     2      HLINE(IP:IP).NE.DEC.AND.HLINE(IP:IP).NE.LPR.AND.
     3      HLINE(IP:IP).NE.RPR) THEN
	 ICHR=1
       ELSE
	 IF (HLINE(IP:IP).GE.'0'.AND.HLINE(IP:IP).LE.'9') INUM=1
	 IF (IR.EQ.0.OR.IFL.EQ.(IR+1)) IR=IFL
	 IF (IL.EQ.0) IL=IFL
	 IF (HLINE(IP:IP).EQ.MNS.AND.((IP.GT.1.AND.HLINE(IP-1:IP-1)
     1     .NE.BLNK.AND.HLINE(IP-1:IP-1).NE.COMA).OR.((HLINE(IP+1:IP+1)
     2     .LT.'0'.OR.HLINE(IP+1:IP+1).GT.'9').AND.HLINE(IP+1:IP+1)
     3     .NE.DEC))) THEN
	   IR=IR-1
	   IF (IR.LT.IL) THEN
	     IR=0
	     IL=0
	     INUM=0
	     ENDIF
	   ICHR=1
	   ENDIF
	 IF (HLINE(IP:IP).EQ.PLS.AND.IFL.NE.IPLS.AND.IP.NE.1.AND.
     1     HLINE(IP-1:IP-1).NE.BLNK.AND.HLINE(IP-1:IP-1).NE.COMA) THEN
	   IR=IR-1
	   IF (IR.LT.IL) THEN
	     IR=0
	     IL=0
	     INUM=0
	     IPLS=0
	     IFL2=0
	     ENDIF
	   ICHR=1
	   ENDIF
	 ENDIF
C
C  IFL1 and IFL2 are the field lenghts of REAL number and ERROR part
C  respectively.
C
       IF (IPLS.NE.0.AND.IFL.EQ.IR) IFL2=IFL2+1
       IP=IP+1
       IF (IP.EQ.81) GOTO 300
       IF (HLINE(IP:IP).NE.BLNK.AND.HLINE(IP:IP).NE.COMA) GOTO 200
C
C  At this point HTEMP is loaded with the string read in.
C  HTEMP will then be processed based on the flags set during the scan.
C
300    GA=HTEMP(1:IFL)
       IFLO=IFL
       IF (IDEC.LT.IL.OR.IDEC.GT.IR) IDEC=0
       IF ((IPLS.NE.0.AND.IPLS.LT.IL).OR.(IPLS.GE.IR.AND.IR.NE.0)) THEN
	 IF (IPLS.EQ.IR) IR=IR-1
	 IFL2=0
	 IPLS=0
	 ICHR=1
	 ENDIF
       IF (IL.GE.IR.AND.(HTEMP(IL:IL).LT.'0'.OR.HTEMP(IL:IL).GT.'9'))
     1   INUM=0
       IF (INUM.EQ.1) THEN
	 HTEMP(1:80)=HTEMP(IL:80)
	 IFL=IR-IL+1
	 IRFL=2
	 IF (ICHR.EQ.1) IRFL=4
	 IF (IDEC.NE.0) THEN
	   IRFL=3
	   IF (ICHR.EQ.1) IRFL=5
	   IFL1=IFL-IFL2
	   CALL EFNUM (X,EX,HTEMP,IFL1,IFL2)
	   N=INT(X)
	 ELSE IF (IPLS.NE.0) THEN
	   IFL1=IFL-IFL2
	   CALL EFNUM (X,EX,HTEMP,IFL1,IFL2)
	   N=INTEGER(X,0)
	   X=FLOAT(N)
	 ELSE
	   N=NUM (HTEMP,IFL)
	   X=FLOAT(N)
	   EX=0.0
	   ENDIF
       ELSE
	 IRFL=1
	 ENDIF
       IF (IRFL1.LT.0.AND.IFLO.GT.8.AND.ICHR.EQ.1) THEN
	 IP=IP-IFLO+8
	 IRFL=-IRFL
	 RETURN
	 ENDIF
400    IP=IP+1
       IF (IP.GT.80) THEN
	 IP=99
	 RETURN
	 ENDIF
       IF (HLINE(IP:IP).EQ.BLNK.OR.HLINE(IP:IP).EQ.COMA) GOTO 400
       RETURN
       END
C
C
C
      subroutine hscan(ha,n,x,ex,ip,hline,irlf)
c
c   written 86.4.8 by I.D.Brown
c
c   as scan, but will read into ha character strings up to 80 characters long
c
      implicit character *8 (g)
      implicit character *80 (h)
c
      irfl=-1
      i2=8
      call scan(ha(1:i2),n,x,ex,ip,hline,irfl)
      if(irfl.lt.0.and.ii.ne.99) then
   55   i1=i2+1
	if(i1.gt.80) return
	i2=i1+7
	if(i2.gt.80) i2=80
	irfl=-1
	call scan(ha(i1:i2),nn,xx,exx,ip,hline,irfl)
	if(irfl.lt.0.and.ii.ne.99) go to 55
	endif
      return
      end
C
c
c
c
c
      function second (itype)
c
***************************************************
*     machine dependent code in this function     *
***************************************************
*
c   ***COMMENTS MARKED * ARE LINES OF CODE THAT MAY HAVE TO BE REACTIVATED
C   ***ON CHANGE OF MACHINE
c
C---  system dependent runtime call function, VAX-version (machine.eq.1)
C
C     itype = 1      initnialize system timer
C           = 2      print elapsed CPU time to SYS$OUTPUT
C           = 3      return elapsed CPU time
C
C     written by Daniel Altermatt
C---  McMaster University, September 1984
C
c---VAX headers (machine.eq.1)
*     external LIB$INIT_TIMER
*     external LIB$STAT_TIMER
*     external LIB$SHOW_TIMER
      character*80 htime
c        the following statement prevents tused being identified as an
c         unsatisfied external call
      dimension tused(2)
c---NORD headers (machine.eq.2)
*      integer *4 tused
*      common /timer/ istart,iactual
c---CYBER headers (machine.eq.3)
*      EXTERNAL SECOND()
c---UNIX headers (machine.eq.4)
c      
c---COMMON headers (all machines)
      common /files/ nin,nout,ndebug,nwrite,nread,nspgr,machine
      common /vaxtime/ti
c
C
      if(machine.eq.1) then
	second = 0.0
c  to avoiid the problems in this routine it has been converted into
c  a dummy routine with the following statement.  Some work is needed
c  to get this functioning properly on the vax.
	return
c        if (itype.ne.1) goto 10
c        ti=secnds(0.0)
c        call LIB$INIT_TIMER(it)
c        return
C
c 10     if (itype.ne.2) goto 20
c        icode = 2
c        call LIB$SHOW_TIMER(it,icode,,htime)
c        return
C
c 20     icode = 2
c        second = secnds(ti)
c        call LIB$STAT_TIMER(icode,itime,it)
c        second = itime / 100.
c        return
c
C  END OF VAX CODE
c
c     version (machine.eq.2)
c
c     itype = 1       initialize system timer
c             2       print elapsed CPU-time
c             3       return elapsed CPU-time
c
c     written by D. Altermatt
c---  Uni Lausanne, Oct. 1985
C
      else if(machine.eq.2) then
*        if (itype.le.1) then
*          second = 0.0
*          istart = tused(dummy)
*          iactual = tused(dummy)
*        else if (itype.eq.2) then
*          iactual = tused(dummy)
*          second = (iactual - istart)/50.
*          print 1000, second
* 1000     format(10X,'Elapsed CPU-time since program start: ',F10.2,
*     *    ' seconds')
*        else if (itype.eq.3) then
*          iactual = tused(dummy)
*          second = (iactual - istart)/50.
*        else
*          second = 0.0
*          endif
	return
C
C---  CYBER-version (machine.eq.3)
C
C     itype = 1      initialize system timer
C           = 2      print elapsed CPU time to SYS$OUTPUT
C           = 3      return elapsed CPU time
C
C     written by Daniel Altermatt
C---  McMaster University, June 1985
C
C
      else if(machine.eq.3) then
c
c     dummy, so far...
c
	second = 0.0
*       IF (ITYPE.LE.1) THEN
*         SECOND = 0.0
*       ELSE IF (ITYPE.EQ.2) THEN
*          WRITE(OUTPUT,1001) SECOND()
*1001      FORMAT(10X,'ELAPSED CPU-TIME SINCE START: ',F10.2,
*    +               ' SECONDS')
*       ELSE
*         SECOND = SECOND()
*         ENDIF
	RETURN
c
c---UNIX Version  (machine.eq.4)
c   Useless dummy code to by-pass on Unix, added by S. Healy 09/27/1990 :)
c
      else if(machine.eq.4) then
c
c  again, dummy code ... add here later!
c
	 second = 0.0
      else if(machine.eq.5) then
	 second = 0.0
	RETURN
	endif
      end
C
C
C
C
      SUBROUTINE READYN(*,*,*,*)
C                ---------------
C
C     Written 85.3.25 by I.D.Brown
c
c     returns to exits 1,2,3 or 4 depending on whether it reads a
c     y, n, e or other character in the input file
c
      implicit character*8 (g)
      common/files/nin,nout
c
      read(nin,1000,err=100) g
 1000 format(A1)
      if(g.eq.'Y'.or.g.eq.'y') return 1
      if(g.eq.'N'.or.g.eq.'n') return 2
      if(g.eq.'E'.or.g.eq.'e') return 3
 100  write(nout,2000)
 2000 format(/,' **input error** TRY AGAIN ** ',/,
     +  ' Type Y for YES, N for NO or E for EXIT',/)
      return 4
      end
C
C
C
      subroutine wait
c
c    written 85.12.13 by I.D.Brown
c    updated 86.01.31 by D.Altermatt
c
c    requests a carriage return from interactive input
c    for paging on screen
c
      implicit character*8 (g)
      common/files/nin,nout,ndebug,nread,nwrite,nspgp,machine
c
      if(machine.eq.3) then
	write(6,1010)
      else
	write(6,1000)
	endif
 1000 format(/,' Hit <RETURN> to continue:')
 1010 format(/,'+Hit <RETURN> to continue:')
      read(nin,1020,err=10) gdum
 1020 format(A1)
 10   return
      end
C
C
C
      FUNCTION HWORD(X,EX,IS)
C              --------------
C
C   WRITTEN 81.6.9 BY I.D.BROWN
C
C   CODES THE NUMBER X AND ITS ERROR INTO THE FORM IX(IEX) WHERE
C   IX = X*(10**IS)
C
      IMPLICIT CHARACTER*80 (H)
C
      IX=INTEGER(X,IS)
      IEX=INTEGER(EX,IS)
      IF(IEX.EQ.0) GO TO 200
      IF(IEX.GE.100) THEN
	WRITE(HTEMP,1001) IX,IEX
      ELSE IF(IEX.GE.10) THEN
	WRITE(HTEMP,1000) IX,IEX
      ELSE
	WRITE(HTEMP,999) IX,IEX
      ENDIF
      READ(HTEMP,1010) HLINE
      HWORD = HLINE
      RETURN
 200  WRITE(HTEMP,1002) IX
      READ(HTEMP,1010) HLINE
      HWORD = HLINE
      RETURN
  999 FORMAT(I5,'(',I1,')')
 1000 FORMAT(I5,'(',I2,')')
 1001 FORMAT(I5,'(',I3,')')
 1002 FORMAT(I5)
 1010 FORMAT(A10)
      END
C
C
C
C
      FUNCTION HXWORD(X,EX)
C              ------
C
C    WRITTEN 81.7.20 BY I.D.BROWN
C    MODIFIED 85.6.18 BY DAVID KNIGHT
C
C   COMBINES A NUMBER AND ITS ERROR IN AN A14 WORD FOR PRINTING
C   ROUNDED TO THE APPRORIATE NUMBER OF SIGNIFICANT FIGURES
C
C   CALLS UPCASE
C
      IMPLICIT CHARACTER*80 (H)
      DIMENSION A(3)
C
      I=0
      B=X
      XX=EX
      if(xx.lt.0.000001) then
	write(htemp,1040) b
 1040   format(F14.5)
	read(htemp,1004) hline
      else
   10   IF(XX.GE.2.) GO TO 20
	I=I+1
	XX=XX*10.
	GO TO 10
   20   CONTINUE
	WRITE (HFORM,1002)  I
 1002   FORMAT(5H(F10.,I1,',1H(,I2,1H))')
	IX=INTEGER(XX,0)
	IF(IX.LT.10) WRITE(HFORM,1003) I
 1003   FORMAT(5H(F10.,I1,',1H(,I1,1H))')

	WRITE(htemp,HFORM) B,IX

	READ(htemp,1004) hline
 1004   FORMAT(A14)
	endif
      CALL UPCASE(hline)
      hxword=hline
      RETURN
      END
C
C
C
      SUBROUTINE UPCASE(HLINE)
C
C     Written by: Daniel Altermatt
C     June 1985
C     Modified 85.8.8 by David Knight/ 86.5.02 N.HEINIG
C
C     Left justifies HLINE and converts lower case letters to upper case.
C
C     Calls:  LENGTH
C
      CHARACTER *(*) HLINE
      CHARACTER*80 HTEMP
C
      N=LENGTH(HLINE)
      IF(N.LE.0) RETURN
C
      DO 10 I=1,N
	IF(HLINE(I:I).NE.' ') THEN
	  HTEMP = HLINE(I:N)
	  GOTO 20
	  ENDIF
   10   CONTINUE
   20 HLINE = HTEMP
      DO 30 J=1,N-I+1
	IF(ICHAR(HLINE(J:J)).LT.ICHAR('a').OR.ICHAR(HLINE(J:J)).GT.
     1    ICHAR('z')) GOTO 30
	NCHAR=ICHAR(HLINE(J:J))-ICHAR('a')+ICHAR('A')
	HLINE(J:J)=CHAR(NCHAR)
   30 CONTINUE
      RETURN
      END

C
C
      SUBROUTINE LOWCASE(HLINE)
C     
C     Written by: Stephen Urquhart, August 1989
C     by revising UPCASE by: Daniel Altermatt, June 1985
C     which was Modified 85.8.8 by David Knight/ 86.5.02 N.HEINIG...
C
C     Left justifies HLINE and converts upper case letters to lower case.
C
C     Calls:  LENGTH
C
      CHARACTER *(*) HLINE
      CHARACTER*80 HTEMP
C
      N=LENGTH(HLINE)
      IF(N.LE.0) RETURN
C
      DO 10 I=1,N
	IF(HLINE(I:I).NE.' ') THEN
	  HTEMP = HLINE(I:N)
	  GOTO 20
	  ENDIF
   10   CONTINUE
   20 HLINE = HTEMP
      DO 30 J=1,N-I+1
	IF(ICHAR(HLINE(J:J)).LT.ICHAR('A').OR.ICHAR(HLINE(J:J)).GT.
     1    ICHAR('Z')) GOTO 30
	NCHAR=ICHAR(HLINE(J:J))+ICHAR('a')-ICHAR('A')
	HLINE(J:J)=CHAR(NCHAR)
   30 CONTINUE
      RETURN
      END
C
c
c
c
      function hleft(h,n)
c
c   written 86.4.8 by I.D.Brown
c
c   This function will return the first n characters in h left justified
c
      implicit character *80 (h)
c
      hleft(1:n)=h(1:n)
      if(h(1:1).ne.' ') return
      do 50 i=1,n
   50   hleft(i:i)=' '
      do 100 i=1,n
	if(h(i:i).ne.' ') then
	  hleft(1:n-i+1)=h(i:n)
	  return
	  endif
  100   continue
      return
      end
C
c
c
      function hright(h,n)
c
c   written 86.4.8 by I.D.Brown
c
c   this function returns the first n characters in h right justified
c   to the nth position,  i.e. if h is CHARACTER *n, then hright is the
c   right justified version of h.  Thus, if g is CHARACTER *8 then
c   hright is CHARACTER *8 with the contents of g right justified.
c
      implicit character *80 (h)
c
      hright(1:n)=h(1:n)
      if(h(n:n).ne.' ') return
      do 50 i=1,n
   50   hright(i:i)=' '
      do 100 i=n,1,-1
	if(h(i:i).ne.' ') then
	  hright(n-i+1:n)=h(1:i)
	  return
	  endif
  100   continue
      return
      end
C
C
      FUNCTION LENGTH(HLINE)
C
C     Written by: Daniel Altermatt
C     June 1985
C
C     Returns the actual length af the given character string (position
C     of the last non-blank character on the right).
C
      CHARACTER *(*) HLINE
C
      N=LEN(HLINE)
      DO 10 I=N,1,-1
	IF(HLINE(I:I).EQ.' ') GOTO 10
	LENGTH=I
	RETURN
   10   CONTINUE
      LENGTH=0
      RETURN
      END
C
C
C
C
C ----------------------------------------------------------------------
       SUBROUTINE EFNUM (P,EP,LRBUF,NFL1,NFL2)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 14, By D Clarkson.
C
C ----------------------------------------------------------------------
C
C Purpose : EFNUM will convert a real number along with an error from
C           character representation into floating point format.
C
C Input   : LRBUF ; character buffer that holds the number and the error.
C
C Output  : P,EP  ; floating point values of the number and the error.
C
C ----------------------------------------------------------------------
       CHARACTER *18 LRBUF
       CHARACTER *12 CNUM
       DATA CNUM /'0123456789.-'/
       IF (NFL1+NFL2.EQ.0) RETURN
       P=0.
       EP=0.
       IEX=0
       INT=0
       INUMDP=0
       ISIGN=1
C
       DO 630 J=1,NFL1
	 DO 620 I=1,12
	   IF (CNUM(I:I).NE.LRBUF(J:J)) GO TO 620
	     IF (I.GT.10) THEN
	       IF (I.EQ.12) ISIGN= -1
	       IF (I.EQ.11) INUMDP=1
	     ELSE
	       IF (INUMDP.EQ.1) IEX=IEX+1
	       INT=10*INT+I-1
	       ENDIF
	     GO TO 630
620        CONTINUE
630      CONTINUE
       P=INT*ISIGN
       P=P/(10.**IEX)
       INT=0
       IF (NFL2.NE.0) THEN
	 DO 660 J=NFL1+1,NFL1+NFL2
	   DO 650 I=1,11
	     IF (CNUM(I:I).EQ.LRBUF(J:J)) THEN
	       IF (I.EQ.11) THEN
		 IEX=IEX+1
		 GO TO 660
		 ENDIF
	       INT=INT*10+I-1
	       GO TO 660
	       ENDIF
650          CONTINUE
660        CONTINUE
	 ENDIF
       EP=INT
       EP=EP/(10.**IEX)
       RETURN
       END
C
C
C
C
C ----------------------------------------------------------------------
       FUNCTION NUM(LRBUF,NFL)
C ----------------------------------------------------------------------
C
C WRITTEN 84 06 07, BY M JACKA
C
C Revised 85 05 13, BY D Clarkson
C
C ----------------------------------------------------------------------
C
C Purpose : This function will convert a number represented in character
C           format into the integer equivalent.
C
C Input   : LRBUF ; holds the character representation
C
C Output  : NUM   ; the integer equivalent.
C
C ----------------------------------------------------------------------
       CHARACTER *11  CNUM
       CHARACTER *10  LRBUF
       COMMON /FILES/ NIN,NOUT
       DATA CNUM /'0123456789-'/
       INT=0
       ISIGN=1
       IF(NFL.GT.10.OR.(NFL.GT.9.AND.LRBUF(1:1).GT.'1')) THEN
	 WRITE(NOUT,2000) LRBUF,NFL
2000     FORMAT(/,' **ERROR**NUM**2000** STRING "',A10,'" HAS',I3,
     1     ' DIGITS -- CANNOT BE CONVERTED TO INTEGER **',/)
	 NUM=999999999
	 RETURN
	 ENDIF
       DO 610 J=1,NFL
	 DO 600 I=1,11
	   IF (CNUM(I:I).EQ.LRBUF(J:J)) THEN
	     IF (I.EQ.11) THEN
	       ISIGN= -1
	     ELSE
	       INT=10*INT+I-1
	       ENDIF
	     GO TO 610
	     ENDIF
600        CONTINUE
610      CONTINUE
C
       NUM=INT*ISIGN
       RETURN
       END
C
C
C
C
      FUNCTION MAXDIM(N,M,H)
C     ----------------------
C
C   written 86.1.9 by I D Brown
c
c
      character*80 h
      common/files/nin,nout,ndebug
c
      maxdim=0
      if(n.le.m) return
      maxdim=1
      n=m
      write(nout,1000) h(1:20),m
 1000 format(1X,A20,' Array limit of ',I4,' exceeded')
      return
      end
c
c
c
      function gint(i)
c
c     returns the integer i as an 8 character word.  The characters are right
c      justified and, if necessaary, are truncated at the LEFT end
c
c     written 1986.9.5 by I.D.Brown
c
c     calls upcase
c
      character gchar(10),gint*(8)
      data gchar/'0','1','2','3','4','5','6','7','8','9'/
c
      k=9
      gint='        '
      j=abs(i)
   10 j1=j
      k=k-1
      if(k.le.0) return
      j2=j1/10
      jr=j1-j2*10
      if(jr.gt.9.or.jr.lt.0) go to 900
      gint(k:k)=gchar(jr+1)
      if(j2.gt.0) then
	j=j2
	go to 10
      else
	if(i.lt.0) then
	 k=k-1
	 if(k.le.0) return
	 gint(k:k)='-'
	 endif
	return
	endif
  900 write(6,9999)
 9999 format(' *ERROR*GINT*9999* Unrecognised integer')
      return
      end
c
c
      subroutine inerr(*)
c
c     writes an input error message for interactive programs,
c     returns to '*'
c
c     written by D. Altermatt
c     McMaster University, July 1984
c
      common /files/ nin,nout
      write(nout,2000)
2000  format(/,1X,'***** fatal input error ***** TRY AGAIN ',/)
      return 1
      end
c
c
c
      FUNCTION INTREAD()
c
c   written 91.12.10 by I.D.Brown
c
c   reads the next record and returns the integer it finds.  If it finds
c   anything other than an integer, it prints a warning and returns 9999999
c
      implicit character*8 (g)
c
      common/files/ nin,nout
      dimension gint(13)
      data gint/'0','1','2','3','4','5','6','7','8','9','+','-',' '/
c
      intread = 0
      isign = 1
      read(nin,1000) g
 1000 format(A8)
      do 100 i=1,8
	if(g(i:i).eq.gint(12)) then
	  isign = -1
	else if(g(i:i).ne.gint(13).and.g(i:i).ne.gint(11)) then
	  do 50 j=1,10
	    if(g(i:i).eq.gint(j)) then
	      intread = 10*intread + j - 1
	      go to 100
	      endif
   50       continue
	  go to 900
	  endif
  100   continue
      intread = isign*intread
      return
  900 write(nout,1900) g
 1900 format(' **ERROR** ',A8,' is not a legal resoponse',//
     1 '    Please give an integer only',/)
      intread = 99999999
      return
      end
c
c
c
      FUNCTION LARGEST(x,n,xlargest)
c     ------------------------------
c
c    written 91.11.25 by I.D.Brown
c
c   largest is the address of the largest element in x(n).  xlargest is
c    its value
c
      dimension x(n)
c
      xlargest = 0.
      do 100 i=1,n
	if(x(i).gt.xlargest) then
	  xlargest = x(i)
	  largest = i
	  endif
  100   continue
      return
      end
c
c
      FUNCTION LSMALLST(x,n,smallest)
c     --------------------------------
c
c     written 91.11.25 by I.D.Brown
c
c   lsmallest is the address of the smallest element of x(n). smallest
c     is its value.
c
      dimension x(n)
c
      smallest = 100000000000000.
      do 100 i=1,n
	if(x(i).lt.smallest) then
	  lsmallst = i
	  smallest = x(i)
	  endif
  100   continue
      return
      end
c
c
c
c
c
      SUBROUTINE SORTX(M,IX,N)
C     ----------------
C
C   WRITTEN 84.2.27 BY I D BROWN
C
C   AFTER EXECUTION N(M) CONTAINS THE ADDRESSES OF ELEMENTS OF IX(M)
C   ARRANGED IN ORDER OF DESCENDING VALUES OF IX
C
      DIMENSION IX(M),N(M)
      DO 200 I=1,M
	N(I)=I
	IF(I.EQ.1) GO TO 200
	J=I
  100   NJ=N(J)
	NJ1=N(J-1)
	IF(IX(NJ).LE.IX(NJ1)) GO TO 200
	NN=N(J-1)
	N(J-1)=N(J)
	N(J)=NN
	J=J-1
	IF(J.LE.1) GO TO 200
	GO TO 100
  200   CONTINUE
      RETURN
      END
C
C
C
C
      SUBROUTINE SORTY(N,Y,IS)
C                -------------
C
C    WRITTEN 85.1.11 BY I.D.BROWN
C
C    AFTER EXECUTION IS(N) CONTAINS THE ADDRESSES OF THE ARRAY Y(N)
C    IN DESCENDING ORDER OF THE VALUES OF Y
C
      DIMENSION IS(N),Y(N)
      DO 200 I=1,N
	IS(I)=I
	IF(I.EQ.1) GO TO 200
	J=I
  100   NJ=IS(J)
	NJ1=IS(J-1)
	IF(Y(NJ).LE.Y(NJ1)) GO TO 200
	NN=IS(J-1)
	IS(J-1)=IS(J)
	IS(J)=NN
	J=J-1
	IF(J.LE.1) GO TO 200
	GO TO 100
  200   CONTINUE
      RETURN
      END
C
C
C
C
      FUNCTION INCH(G)
C              -------
C
C    WRITTEN 85.1.4 BY I.D.BROWN
C    EDITED 90.11.19 BY S. HEALY
C
C    CONVERTS THE CHARACTER ARRAY G INTO AN INTEGER (INCH).
C    THE STRING IS TERMINATED BY A CHARACTER OTHER THAN A
C    NUMBER + - OR .
C
      IMPLICIT  CHARACTER*8(G)
      DIMENSION GCH(14)
      DATA GCH/'0','1','2','3','4',
     + '5','6','7','8','9',
     + ' ','+','-','.'/
      RECH=0.0
      INCH=0
      SGN=1.
      ISTART=0
      DEC=-1.0
      DO 100 I=1,8
       DO 50 J=1,14
	  IF(GCH(J).EQ.G(I:I)) THEN
	    K=J
	    GO TO 70
	    ENDIF
   50     CONTINUE
	IF(ISTART.NE.0) GO TO 150
	RETURN
   70   IF(K.EQ.11) THEN
	 IF(ISTART.NE.0) GO TO 150
	 GO TO 100
	 ENDIF
	istart=1
	IF(K.EQ.14.AND.DEC.LT.0) DEC=0.1
	IF(K.EQ.13) SGN=-1.0
	IF(K.LE.10) THEN
	  IK=K-1
	  IF(DEC.LT.0) THEN
	    INCH=INCH*10+IK
	  ELSE
	    RECH=RECH+IK*DEC
	    DEC=DEC*0.1
	    ENDIF
	  ENDIF
  100   CONTINUE
C
C   END OF READ
C
  150 RECH=  SGN*(RECH+INCH)
      INCH=NINT(RECH)
      RETURN
      END
C
C
      FUNCTION RECH(G)
C
C
C    WRITTEN 90.11.19 BY S. HEALY
C            (to replace ENTRY in INCH function)
C    ORIGINAL CODE BY I.D. BROWN  (85.1.4)
C
C    Converts the character array G into a real number (RECH).
C    The string is terminated by a character other then a
C    number + - or .
C
C    This function replaces ENTRY RECH(G) in FUNCTION INCH(G) to operate
C    properly under UNIX.
C
      IMPLICIT CHARACTER*8(G)
      DIMENSION GCH(14)
      DATA GCH/'0','1','2','3','4',
     +  '5','6','7','8','9',
     +  ' ','+','-','.'/
      RECH=0.0
      INCH=0
      SGN=1.
      ISTART=0
      DEC=-1.0
      do 100 i=1,8
	do 50 j=1,14
	  if(gch(j).eq.g(i:i)) then
	     k=j
	     goto 70
	     endif
 50          continue
	if(istart.ne.0) goto 150
	return
 70     if (k.eq.11) then
	 if (istart.ne.0) goto 150
	 goto 100
	 endif
	istart=1
	if(k.eq.14.and.dec.lt.0) dec=0.1
	if(k.eq.13) sgn=-1.0
	if(k.le.10) then
	  ik=k-1
	  if(dec.lt.0) then
	    inch=inch*10+ik
	  else
	    rech=rech+ik*dec
	    dec=dec*0.1
	    endif
	  endif
 100      continue
C
C end of read
C
 150      rech=  sgn*(rech+inch)
	  return
	  end
C
C
      FUNCTION LIMIT(X,DMAX,DMIN)
C
C    *****FUNCTION*****          LIMIT
C                                -----
C
C    WRITTEN 83.9.8 BY I D BROWN
C
C    IF X LIES BETWEEN DMAX AND DMIN LIMIT=1, OTHERWISE LIMIT=0
C
      DIMENSION X(3),DMAX(3),DMIN(3)
      LIMIT = 1
      DO 10 I=1,3
	IF(X(I).LE.DMIN(I).OR.X(I).GT.DMAX(I)) LIMIT = 0
   10 CONTINUE
      RETURN
      END
C
C
C
C
      FUNCTION JTEST(X,Y)
C
C   *****FUNCTION****      JTEST
C                          -----
C
C   WRITTEN 83.9.8 BY I D BROWN
C
C   JTEST=0 IF X AND Y ARE THE SAME, OTHERWISE JTEST=1
C
      DIMENSION X(3),Y(3)
      JTEST=1
      DT=0.0008
      T=0.
      DO 10 I=1,3
	T = T + ABS(X(I)-Y(I))
   10   CONTINUE
      IF(T.LT.DT) JTEST=0
      RETURN
      END
C
C
C
C
       FUNCTION RHALF(X,DMAX)
C
C  *****FUNCTION*****         RHALF
C                             -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  GENERATES HALF TRANSLATION
C
       RHALF = X + 0.5
       IF(RHALF.GT.DMAX) RHALF = RHALF - 1.0
       RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
       END
C
C
C
C
       FUNCTION THIRD(X,DMAX)
C
C  *****FUNCTION*****        THIRD
C                            -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS
C
C  GENERATES TRANSLATION OF ONE THIRD
C
       THIRD = X + 0.3333333
       IF(THIRD.GT.DMAX) THIRD = THIRD - 1.0
       RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
       END
C
C
C
C
       FUNCTION TWO(X,DMAX)
C
C  *****FUNCTION*****          TWO
C                              ---
C
C  WRITTEN 80.6.1 BY I D BROWN
C
C  GENERATES TRANSLATION OF TWO THIRDS
C
       TWO = X + 0.666667
       IF(TWO.GT.DMAX) TWO = TWO - 1.0
       RETURN
C**  THIS PROGRAM VALID ON FTN4 AND FTN5 **
       END
c
c
      subroutine pageflip (*,*,*)
c
c            PAGEFLIP - this is a small subroutine called by struhelp
c                        and its associated subroutines.
c
c            written by  C. Carr - 21 mar 89
c
c            last updated : L.Jamieson - 29 mar 89
c
      implicit character*8 (g)
      common/files/nin,nout
c
100   write(nout,10)
10    format(5X,'Press N for next page, P for previous page, ',
     1                  'or E to exit: ')  
c
      gflip=' '
      read(nin,20,err=100) gflip
20    format(A1)
      if(gflip.eq.' ')return1
      if(gflip.eq.'N'.or.gflip.eq.'n') return1
      if(gflip.eq.'P'.or.gflip.eq.'p') return2
      if(gflip.eq.'E'.or.gflip.eq.'e') return3
      goto 100
      end
