C
c                     FILE NETLIB                   Last update: 1993.6.3
c                     -----------                                I.D.Brown
C
c   This file contains standard FORTRAN-77 code
C*********************************************************
C   This file contains DOUBLE PRECISSION arrays indicated by +
c*********************************************************
C
C     THIS FILE CONTAINS
C
C  S+* KIRCH(IRE)         MAIN SUBROUTINE FOR BOND-NETWORK ANALYSIS
C  S*  FIXBND             Identify bonds to be held fixed
C  S   ATOMAR             COPIES /BONDS/ ARRAY TO /SPAN/
C  S*  ITENET(IRE)        SOLVES NETWORK ITERATIVELY
c  s   weight()           Sets weights used by itenet
C  S   JSR                SOLVES NETWORK USING J.S.RUTHERFORD METHOD
c  S   COOSPHTABLE        Writes table about first coordination sphere
C  S   POLPLOT            DRAWS PSEUDOPLOT OF COORDINATION SPHERE
c  S   EQBND              FINDS RELATED BONDS IN SYMMETRICALLY EQUIVALENT
C                         COORDINATION POLYHEDRA
c  f   lcomv         (should be transfered to mathlb.for)
C  S   VLINK(IS,IE,VECT,IER) FINDS VECTOR FROM IS TO IE
C  S   LINK(IS,IE,NL,LINK,IER) FINDS SHORTEST BOND PATH FROM IS TO IE
C  S+  NODE               SETS UP NODE SUM EQUATIONS
C  S+  LOOPER             FINDS LOOPS WITHIN THE NETWORK
C  S+  LOOP(I,IS,IE,K,IF,IER)   FINDS SHORTEST LINKS FOR =LOOPER=
C  S+  STORE2(K)          STORES BONDS IN THE NETWORK MATRIX
C  S   OPEN(I,IER)        WRITES A BOND INTO SCRATCH STORAGE =IOP=
C  S   CLOSE(I,IER)       COPIES FROM =IOP= TO =ICL= SCRATCH STORAGE
C  F   NODE1(I)           FUNCTION TO FIND NODE NUMBERS IN BONDS
C  F   NODE2(I)           ANALOG NODE1
C  S   PULL(IL,J,IER)     PULLS BONDS IN REVERSE ORDER FROM =ICL=
C  S   VALPOT             CALCULATES VALENCE POTENTIALS
C  S   SET(I)             SETS VALENCE POTENTIALS FOR =VALPOT=
C  S   WRITEV(NT)       WRITES VALENCES AND VALENCE POTENTIALS TO =NOUT/NDEBUG=
C  S   NETHELP            HELP SCREEN
C
C     * includes interactive code
c
C     ALLS TO EXTERNALS: (routines in parentheses are not called by active code
C
C     (VAX*IMS-LIBRARY: LINV3F  (DOUBLE PRECISION)) not currently used
C     CERNLB: DEQINV      (DOUBLE PRECISION)
c     CHEMLB: bost
C     IOLIB: HFILEN, capsuplo
c     MATHLB: WAIT,readyn,axa,axv,zeroa,unitm,transa,aplusa,amina,inerr,vminv,
c             scan
C     SPANNER:IYAT,expand,closer,jyat,allcosphr
C     SYMLIB: symop
C
C******* KIRCH IS INTERACTIVE ***********
C
C    The following common blocks are found only in netlib
c     /loop/
c     /dls/
c     /atoms/ val(600),rij(600)  = valence potential  
c                         bond weights (capacitances) for network equations
C
C
***********************************************************************
C
      SUBROUTINE KIRCH(IREFIN)
C
C    WRITTEN 85.1.15 BY I.D.BROWN
C    IMPLEMENTED BY  D. ALTERMATT
C
C    CALLS (CERNLB:DEQINV, LOOPER, NODE,) VALPOT, WRITEV, INERR
*
C
C   This program uses Kirchhoff's laws to solve for a chemical
C   network.   The input is given in ATOMS and BONDS sections
C   of SCFS. The output gives the valences of the bonds and the
C   valence potentials of the atoms.
c
c   In the original version two choices were availble for the method of
c   solving the network.  One uses Kirchhoff's laws, looks for loops
c   constructs a matrix and solves it (itype=1), the other uses an
c   iterative approach (I.D.Brown Acta Cryst. (1977) B33, 1305) which
c   achieves the same result (itype=2).  Since the second approach is
c   faster and more robust, the code for the first is no longer called
c   though still present in the program. The subroutines LOOPER and
c   NODE are no longer used. LOOP is not used put provides useful code
c   for finding links in networks and so should be kept.  The subroutines
c   called by LOOP are still in active use (VALPOT).  Note
C   that this code calls the matrix inversion routine DEQINV from
C   CERNLB in double precision.
c      Other methods of solving the matrix have been proposed by 
c   Rutherford (89 Acta Cryst A) and O'Keeffe (89 Acta Cryst A).
c   The analytical approach of J.S.Rutherford has now
c   been added (itype=3), and with unit weights gives results identical
c   to the iterative method.
c      irefin.ne.0 means that an intial set of valences has already been
c   provided overriding the initial assignment calculated in itenet.
C
c   Theory for code not currently accessible
c   ----------------------------------------
C   THIS PROGRAM READS NX ATOMS AND NB BONDS AND SOLVES
C   KIRCHHOFF'S LAWS FOR THE NETWORK TO DETERMINE THE
C   BOND VALENCES.  TO DO THIS IT expands the bond set and
C   CONSTRUCTS THE MATRIX
C   MX AND THE VECTOR V AND SOLVES THE EQUATION
C         S = (XM**-1)*V
C   XM:V CONTAINS NX-1 LINES FROM THE NODES AND NL LINES
C   FROM THE LOOPS WHERE NX-1+NL=NBON
C   OTHER DEFINITIONS AS READ BY RSCFS EXCEPT THAT
C      NFBON(1,I)=+/-1 IS A FLAG FOR THE DIRECTION OF THE BOND
C      NFBON(2,I)=1/0 IS A FLAG SET WHEN A BOND IS INCLUDED IN
C                     A LOOP
C
c   If the loop is not electrically neutral the following errors will
c   ---------------------------------------
c   occur: In the disabled version, the charge on the last atom
c   listed will be assumed to be the charge necessary to ensure electrical
c   neutrality.  In the active itype=2 version, convergence will not occur
c   and the calculation will be terminated with a warning.  If the imbalance
c   is small, the solution may still be acceptable.   I still have to find
c   out what happens with itype=3
c
      IMPLICIT CHARACTER*8 (G)
      implicit character*80 (h)
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500)
C
C     system common blocks
C
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,nwrite
      include 'comatom.for'
      include 'combond.for'
      common /span/  nya,mya,iyatom(5,600),iterm(600),
     1               nbo,mbo,ibond(2,600),xbond(6,600),
     2               iat,bondlim(2),netind(20),ifbond(4,600)
C
C Programmer's Note:  /loop/ changed to /loopp/ for
c                     UNIX system
      COMMON /LOOPP/  ML,IO1,IO2,IC,IOP(1000),ICL(1000)
      COMMON /DLS/   NFBON(2,600)
C
      DIMENSION jyatom(5),idatom(3),jdatom(3)
C
      IF(nbon.EQ.0) GO TO 910
      NM=0
      ML=1000
      DO 50 I=1,MM
	DO 50 J=1,MM
   50     XM(J,I)=0.
      write(nout,1049)
 1049 format(////,
     2 10X,'Welcome to the Network Analysis Routine',//,
     4 '   This routine will predict the valences of the bonds in a ',
     4 'bond network',/,
     5 ' using the network laws (Brown Acta ',
     5 'Cryst. B33, 1305 (1977), Rutherford',/,
     5 ' Acta Cryst. B46, 289 (1990)).',//,
     6 ' It is only necessary to know the bond topology of the ',
     6 'crystal and the atom',/,
     7 ' oxidation states.  You will be asked if you wish the stored ',
     7 'bond topology to be',/,
     8 ' expanded to fill the whole unit cell.  This is normally ',
     8 'necessary unless ',/,
     9 ' the network supplied is closed (i.e. finite).  The results ',
     9 'can be displayed ',/,
     + ' on the terminal or sent to the printer.  The latter is ',
     + 'preferred unless ',/,
     1 ' network is very small.')
c
   55 write(nout,1055)
 1055 format(' '/,
     1       '       Network Menu',/,
     2       '       ------------',//,
     4 ' 1.   Help',/,
     5 ' 2.   Use iterative (Brown) Method',/,
     6 ' 3.   Use the analytical (Rutherford) method ',//,
     8 ' 20.  Exit',//,
     + ' Enter your choice : ')
      read(nin,1056) itype
 1056 format(I5)
      if(itype.eq.20) return
      if(itype.le.0.or.itype.gt.3) go to 55
      if(itype.eq.1) call nethelp
      if(itype.eq.2) go to 62
      if(itype.eq.3) go to 56
      go to 55
c
c  the option itype=4 is disabled
c
c   ---------------------------------
c   see if any bonds are to be fixed
c   ---------------------------------
   62 do 63 i=1,nbon
	if(ifbon(4,i).ne.0) then
	  write(nout,1063)
 1063     format(' ***WARNING*** Some bonds will be held fixed in ',
     1       'this calculation')
	  go to 60
	  endif
   63   continue
   60 write(nout,1060)
 1060 format(/,' Do you want to fix or unfix any bond valences ',
     1     '(Y/N/E) ')
      call readyn(*65,*56,*220,*60)
   65 call fixbnd
      go to 56
C   ------------------------------------------------------------------
C   Test if there are bonds stored in /bonds/ and expand into /span/
C   if necessary
C   ------------------------------------------------------------------
   56 CALL ATOMAR
      write (nout,1050) nya, nbo
 1050 format(/,' ',I3,' atoms and ',I3,' bonds found in the unit cell')
c
c  the following code is needed only if itype=4
c
*      if(nbo.lt.1.or.nbo.gt.100) then
*  80   write(nout,1051)
*1051   format(//,' Do you wish to continue calculation (Y/N/E): ')
*       call readyn(*90,*220,*220,*80)
*       endif
C   --------------------------
C   SET FLAGS
C   ---------------------------
   90 DO 100 i=1,nbo
	NFBON(1,I)=1
	IF(OXID(iyatom(1,ibond(2,i))).GT.OXID(iyatom(1,ibond(1,i))))
     +     NFBON(1,I)=-1
  100   NFBON(2,I)=0
      IF(NDEBUG.NE.0) THEN
	write(ndebug,1104)(ibon(1,i),jbon(1,i),
     1    (ifbon(k,i),k=1,4),i=1,nbon)
	WRITE(NDEBUG,1100)(I,GELEM(I),GDENT(I),OXID(I),icn(i),I=1,NX)
	WRITE(NDEBUG,1101)(i,ibond(1,i),ibond(2,i),
     1    (ifbond(j,i),j=1,4),(nfbon(j,i),j=1,2),i=1,nbo)
	WRITE(NDEBUG,1102)(J,(IYATOM(K,J),K=1,5),J=1,NYA)
 1100   FORMAT(' *DEBUG*KIRCH*1100* ',I3,2X,A2,A6,F5.2,I3)
 1101   FORMAT(' *DEBUG*KIRCH*1101* ',3(1X,I3,2X,2(I3,1X),6I3))
 1102   FORMAT(' *DEBUG*KIRCH*1102* ',I3,2X,5I3)
 1104   format(' *DEBUG*KIRCH*1104* ',5(2I4,4I3))
	ENDIF
c   -------------------------------
c   select calculation
c   -------------------------------
      if(itype.eq.4) then
	write(nout,1103)
 1103   format(' Menu item 4 (analytical solution) is not accessible')
	go to 55
	endif
c   the completion of this IF branch occurs at the call to itenet (itype=2)
c   and has been deleted
******************************************************************************
c   since itype is no longer set to 4 the following code is never called
C
C   CONSTRUCT MATRIX
C
*        CALL NODE
*        IF(NDEBUG.NE.0) THEN
*          DO 120 I=1,NYA-1
*            WRITE(NDEBUG,1104) I
* 1104       FORMAT('    MATRIX LINE ',I4,' FOLLOWS')
*  120       WRITE(NDEBUG,1105)(XM(I,J),J=1,nbo),V(I)
* 1105       FORMAT(' *DEBUG*KIRCH*1105* ',30(10F5.1,/,20X))
*          ENDIF
*        CALL LOOPER
*        IF(NDEBUG.NE.0) THEN
*          DO 150 I=NYA,NM
*            WRITE(NDEBUG,1104) I
*  150       WRITE(NDEBUG,1106)(XM(I,J),J=1,nbo),V(I)
* 1106       FORMAT(' *DEBUG*KIRCH*1106* ',30(10F5.1,/,20X))
*          ENDIF
*        IF(NM.NE.nbo) THEN
*          WRITE(NOUT,1120) NM,nbo
* 1120     FORMAT(//,' *ERROR*KIRCH*1120 ',I4,' equations found for ',
*     +     I4,' bonds. Calculation ended',//)
*          GOTO 999
*          ENDIF
*        write(nout,1150) nm
* 1150   format(' ',I3,' equations found. Inverting matrix')
C
C   SOLVE MATRIX
C
*--VAX (may be reactivated if necessary. If so WK must be declared
*       double precission)
*     CALL LINV3F(XM,V,2,NM,MM,-1,0,WK,IER)
*--N-500
*        ISOL=1
*        CALL DEQINV(NM,XM,MM,WK,IER,ISOL,V)
*--N-500 end
*        IF(IER.EQ.130.OR.IER.LT.0) THEN
*          WRITE(NOUT,1200)
* 1200     FORMAT(' *ERROR*KIRCH*1200* Singular matrix found')
*          ENDIF
*        DO 200 I=1,nbo
*  200     xbond(3,I)=V(I)
c   end of the code that is never called (itype=1)
**********************************************************************
      ier=0
      if(itype.eq.2) call itenet(irefin)
      if(itype.eq.3) call jsr(ier)
      if(ier.ne.0) go to 55
C
C   DETERMINE THE VALENCE POTENTIAL OF EACH ATOM
C
      CALL VALPOT
C
C   WRITE OUTPUT
C
  210 if(itype.eq.2)  htype = 'Brown iterative method'
      if(itype.eq.3)  htype = 'Rutherford analytical method'
      write(nwrite,2011) htype
 2011 format('1      Network analysis using the ',A30,/,
     1 7X,'-------------------------------------------------------')
      call writev(nwrite)
      WRITE(NOUT,2010) htype,hfilen(nwrite)
 2010 FORMAT(//,' Network analysis done using the ',A30,'.',//,
     1 ' The results are written to the file ',A30,/, 
     2 ' Do you want results also displayed on screen (Y/N/E): ')
      call readyn(*215,*217,*220,*210)
  215 call writev(nout)
      call wait
  217 continue
c
c   store bonds if required
c
  230 write(nout,2020)
 2020 format(/,' Do you want to save the valences for use as target ',
     +  'valences',/,
     2  ' e.g. in distance least squares (Y/N/E)  ')
      CALL READYN(*240,*220,*220,*230)
  240 do 270 k=1,nbon
	i=0
	j=0
	do 250 l=1,nya
	  call jyat(l,jyatom)
	  if(jcomve(ibon(1,k),jyatom,2).eq.0) then
	    call ivminv(idatom,ibon(3,k),jyatom(3))
	    i=l
	    if(j.ne.0) go to 255
	  else if(jcomve(jbon(1,k),jyatom,2).eq.0) then
	    call ivminv(jdatom,jbon(3,k),jyatom(3))
	    j=l
	    if(i.ne.0) go to 255
	    endif
  250     continue
	if(jcomve(idatom,jdatom,3).eq.0) go to 255
	write(nout,2500) k
 2500   format(' **error**kirch**2500** No valence found for bond ',
     +                 I3)
	go to 270
  255   do 260 l=1,nbo
	  if((i.eq.ibond(1,l).and.j.eq.ibond(2,l)).or.
     +       (i.eq.ibond(2,l).and.j.eq.ibond(1,l))) then
	    xbon(3,k)=xbond(3,l)
	    ifbon(4,k)=1
	    go to 270
	    endif
  260     continue
  270   continue
  220 RETURN
  910 WRITE(NOUT,9100)
 9100 FORMAT(//,' No bonds found, it is impossible to do any '
     1   ,'calculations!',//)
  999 call wait
      END
C
***********************************************************************
c
      subroutine fixbnd
c     -----------------
c
c   written 90.7.4 by I.D.Brown
c   allows bonds to be fixed in the solution
c
c   calls bost, wait
c
      implicit character*8 (g)
      implicit character*80 (h)
      common /files/ nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      include 'comatom.for'
      include 'combond.for'
      dimension gfix(2)
c
c   ------------------------------------------------------------------
c   check if any valences are to be held fixed to their observed value
c   ------------------------------------------------------------------
      if(nout.ne.6) return
   45 write(nout,1045)
 1045 format('       Menu for fixing bonds',/,
     1       '       ---------------------',/,
     2 ' 1. List the stored bonds indicating which are ',
     2     'already fixed',/,
     3 ' 2. Fix additonal bonds',/,
     4 ' 3. Unfix bonds',/,
     5 ////,
     6 ' 20. Return to Network analysis',//,
     7 '    Enter a number from the menu')
      read(nin,1046) itype
 1046 format(I5)
      if(itype.eq.1) go to 50
      if(itype.eq.2) go to 70
      if(itype.eq.3) go to 83
      if(itype.eq.20) go to 900
      go to 45
c-----------------------
c list the bonds
c-----------------------
   50 gfix(1) = 'variable'
      gfix(2) = 'fixed'
      linc=0
      do 55 i=1,nbon
	iff = ifbon(4,i)+1
	write(nout,1055) i,gelem(ibon(1,i)),gdent(ibon(1,i)),
     +    gelem(jbon(1,i)),gdent(jbon(1,i)),xbon(1,i),gfix(iff)
 1055   format(2X,I4,' is the bond between ',A2,A4,' and ',A2,A4,
     +          ' of length ',F5.2,1X,A8)
	linc=linc+1
	if(linc.gt.20) then
	  call wait
	  linc=0
	  endif
   55   continue
      call wait
      go to 45
c-----------------------
c fix bonds
c-----------------------
   70 write(nout,1070)
 1070 format(' Enter the number of a bond to be fixed (0 to end)')
      read(nin,1071) ifix
 1071 format(I5)
      if(ifix.eq.0) go to 45
      write(nout,1072)
 1072 format(' What valence should this be set to ',
     +    '(blank for observed value) ')
      read(nin,1073) xbb
 1073 format(F10.5)
      if(abs(xbb).lt.0.0001) then
	call bost(ibon(1,ifix),jbon(1,ifix),xxb,est,xbon(2,ifix),
     +               xbon(1,ifix))
	xbon(3,ifix)=abs(xxb)
      else
	xbon(3,ifix)=xbb
	endif
      ifbon(4,ifix)=1
      go to 70
c--------------------------
c  unfix bonds
c-------------------------
   83 write(nout,1083)
 1083 format(' Enter the number of the bond to be unfixed.',
     1    '  End with zero')
      read(nin,1071) ifix
      if(ifix.eq.0) go to 45
      ifbon(4,ifix) = 0
      go to 83
  900 return
      end
c
c
***********************************************************************
c
C
      subroutine atomar
c     -----------------
c
c   written 85.11.1 by I.D.Brown
c   major update 86.2.12 by D. Altermatt
c
c   generates all the atoms and bonds in the unit cell (in /span/)
c   using expand, then converts these into a finite network by
c   equating translationally related atoms. Note that in doing this,
c   the values (iyatom(j,i),j=3,5) loose any significance.
c
c
      implicit character*8 (g)
      common /files/ nin,nout,ndebug,nread,nwrite,machine
      include 'comatom.for'
      include 'combond.for'
      common /span/  nya,mya,iyatom(5,600),iterm(600),
     1               nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2               netind(20),ifbond(4,600)
c
      nya = 0
      nbo = 0
   60 write(nout,1000)
 1000 format(//,' Do you wish to expand the network, to fill the',
     +   ' unit cell (Y/N/E): ')
      call readyn(*200,*300,*900,*60)
  200 write(nout,1010)
 1010 format('Do you want to choose the central atom manually?(Y/N/E)')
      call readyn(*210,*230,*900,*200)
  210 write(nout,1015)
 1015 format('The structure contains the following independent atoms:')
      j=0
      do 220 i=1,nx
       j=j+1
       write(nout,1020)i,gelem(i),gdent(i)
 1020  format(2X,I3,3X,A2,A6)
       if(j.gt.19) then
	call wait
	j=0
       endif
  220 continue
  225 write(nout,1030)
 1030 format(/,'Select an atom as central atom:')
      read(nin,1040,err=225) iat
 1040 format(I3)      
      go to 240
  230 iat=1
  240 nya=0
      nbo=0
c
c     consider dummy bonds !
c
      bondlim(1)=-.00001
      bondlim(2)=4.0
      itype=1
      call expand(itype)
      if(ndebug.ne.0) write(ndebug,2001) nya,nbo
 2001 format(' *DEBUG*ATOMAR*2001* nya=',I4,'; nbo=',I4,' after ',
     +          'expand')
      call closer
      if(ndebug.ne.0) write(ndebug,2002) nya,nbo
 2002 format(' *DEBUG*ATOMAR*2002* nya=',I4,'; nbo=',I4,' after ',
     +          'closer')
      goto 900
c
c     Copy /bonds/ to /span/, recalculate the coordination numbers
c
300   do 305 i=1,nx
305     icn(i)=0
      iat=ibon(1,1)
      do 310 i=1,nbon
	do 320 j=1,nya
320       if(iyatom(1,j).eq.ibon(1,i).and.iyatom(2,j).eq.ibon(2,i).
     +      and.iyatom(3,j).eq.ibon(3,i).and.iyatom(4,j).eq.ibon(4,i).
     +      and.iyatom(5,j).eq.ibon(5,i)) goto 330
	nya=nya+1
	do 325 l=1,5
325       iyatom(l,j)=ibon(l,i)
	iterm(j)=-1
330     do 340 k=1,nya
340       if(iyatom(1,k).eq.jbon(1,i).and.iyatom(2,k).eq.jbon(2,i).
     +      and.iyatom(3,k).eq.jbon(3,i).and.iyatom(4,k).eq.jbon(4,i).
     +      and.iyatom(5,k).eq.jbon(5,i)) goto 350
	nya=nya+1
	do 345 l=1,5
345       iyatom(l,k)=jbon(l,i)
	iterm(k)=-1
350     if(ifbon(1,i).lt.1) ifbon(1,i)=1
	do 355 m=1,ifbon(1,i)
	  nbo=nbo+1
	  ibond(1,nbo)=j
	  ibond(2,nbo)=k
	  icn(iyatom(1,j))=icn(iyatom(1,j))+1
	  icn(iyatom(1,k))=icn(iyatom(1,k))+1
	  do 360 l=1,6
360         xbond(l,nbo)=xbon(l,i)
	  do 365 l=1,4
	    ifbond(l,nbo)=ifbon(l,i)
365         continue
355       continue
310     continue
  900 return
      end
C
C
***********************************************************************

      subroutine ITENET(irefin)
c                -------
c
c    written 85.12.4
c
c    solves the network using an iterative procedure
c    see I.D.Brown Acta Cryst. B33 1305-1310 (1977)
c
c   calls CHEMLB: bost
c         MATHLB: wait
c
c
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      include 'comatom.for'
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     +            netind(20),ifbond(4,600)
      common/atoms/ val(600), rij(600)
c  rij has the dimensions of mbo
c
c
      ibv=0
c
c   irefin is set to zero in runpro.  It should be  1 if an initial set
c    of valences is already supplied
c
      if(irefin.ne.0) go to 190
c
   40 irij=0
      do 50 i=1,nbo
   50  rij(i)=1.00
  110 call weight(iexit,ibv,irij)
      if(iexit.eq.1) go to 900
c   ------------------------------------
c    generate the initial bond valences
c   ------------------------------------
   95 do 100 i=1,nbo
	iat1=iyatom(1,ibond(1,i))
	iat2=iyatom(1,ibond(2,i))
	if(icn(iat1).le.0.or.icn(iat2).le.0) then
	  write(nout,1001) iat1,iat2
 1001     format(' Coordination number of atom ',I3,' or ',I3,
     +      'zero or negative.  Calculation aborted')
	  go to 900
	  endif
	if(ifbond(4,i).eq.0) then
	  xbond(3,i)=(abs(oxid(iat1))/icn(iat1)+
     +      abs(oxid(iat2))/icn(iat2))/2
	  xbond(3,i)=rij(i)*xbond(3,i)
	  endif
  100   continue
      if(ndebug.ne.0) then
	write(ndebug,2800) ncycle
	write(ndebug,2801) (xbond(3,j),j=1,nbo)
	endif
c   ---------------------
c   refine bond valences
c   ---------------------
  190 ncycle=0
  200 idif=0
      ncycle=ncycle+1
      do 280 i=1,nya
	iat=iyatom(1,i)
	sum=0.0
	rsum=0.0
	ncn=0
	do 220 j=1,nbo
	  if(ibond(1,j).eq.i.or.ibond(2,j).eq.i) then
	    sum=sum+xbond(3,j)
	    if(ifbond(4,j).eq.0) ncn=ncn+1
	    rsum=rsum+rij(j)
	    endif
  220     continue
	dif=sum-abs(oxid(iat))
	if(abs(dif).gt.0.003) idif=1
	sdif = 0
	if(ncn.ne.0) sdif=dif/rsum
	do 250 j=1,nbo
  250     if((ibond(1,j).eq.i.or.ibond(2,j).eq.i).and.ifbond(4,j).eq.0)
     +       xbond(3,j)=xbond(3,j) - (sdif*rij(j))
  280   continue
      if(ndebug.ne.0)then
	write(ndebug,2800) ncycle
 2800   format(' *DEBUG*ITENET*2800*  Valences after cycle ',I5)
	write(ndebug,2801) (xbond(3,j),j=1,nbo)
 2801   format(' *DEBUG*ITENET*2801* ',20F5.2)
	endif
      if(idif.ne.0.and.ncycle.lt.(nya*10)) go to 200
      if(idif.ne.0) then
       write(nout,2802) ncycle
 2802  format('    Calculation terminated after ',I5,' cycles',/,
     1        '    Convergence not completely achieved')
       go to 110
      endif
      write(nout,2803)
 2803 format('Calculation terminated regularly')
      ibv=1
      vals1=0.  
      vals2=0.
      do 300 i=1,nbo
	ia=iyatom(1,ibond(1,i))
	ib=iyatom(1,ibond(2,i))
	call bost(ia,ib,valobs,valerr,xbond(2,i),
     1            xbond(1,i))
	bleng=dbost(gelem(ia),gelem(ib),
     1              oxid(ia),oxid(ib),xbond(3,i))
	vals1=vals1+((xbond(3,i)-abs(valobs))*
     1               (xbond(3,i)-abs(valobs)))
	vals2=vals2+
     1          ((xbond(1,i)-bleng)*
     2          (xbond(1,i)-bleng)) 
  300 continue
      vals1=SQRT(vals1/nbo)
      vals2=SQRT(vals2/nbo)
      write(nout,1120) vals1,vals2
 1120 format('rms bond valence deviation :',1X,F8.5,/,
     1       'rms distance deviation     :',1X,F8.5)
	call wait
      go to 110
  900 return
      end
C
c
***********************************************************************
c     
      subroutine weight(iexit,ibv,irij)
c
c    written  by Martin Kunz 4.5.93
c                                                     
c    subroutine to assign weights to individual bonds for the iterative                             
c    method. Weights can be regarded as capacitances as described in 
c    Brown & Kunz 1993, Acta Cryst. (submitted)
c
c   calls CHEMLB: bost
c         MATHLB: wait
c
c
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      include 'comatom.for'
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     +            netind(20),ifbond(4,600)
      common/atoms/ val(600), rij(600)
      dimension iexbnd(600) 
c
  110 iexit=0
      iweight=0
      write(nout,1010)
 1010 format(//,'(1)     List bonds & weights around atoms',/,
     1       '(2)     Draw Pseudoplot of coordination spheres',/,
     2       '(3)     set weights for individual bonds',/,
     3       '(4)     list modified weights',/,
     4       '(5)     reset all weights to 1.0',/,
     5       '(6)     perform network equations',/,
     6       '(20)    exit',//)
      read(nin,1020,err=110) itype
 1020 format(I3)
      if(itype.eq.1) call coosphtable(ibv)
      if(itype.eq.2) call polplot
      if(itype.eq.3) go to 120
      if(itype.eq.4) go to 160
      if(itype.eq.5) go to 170
      if(itype.eq.6) go to 900
      if(itype.eq.20) then 
       iexit=1 
       go to 900
      endif
      go to 110
  120 write(nout,1035)
 1035 format(//,'Do you want do apply your weights on bonds ',/,
     1       'attached to atoms which are symmetric equivalent ',/,
     2       'to one of the present bonded atoms?',/,
     3       'Type N if your weight breaks the crystal symmetry.',/,
     4       '(Y/N/E)')
      call readyn(*130,*135,*900,*120)
  130 iweight=1
  135 write(nout,1030)
 1030 format('Give bond number and new weight:',/,
     1       '(0,0 for no more weights)')
      read(nin,*) ib,r1
      if(ib.eq.0) go to 110
      if(rij(ib).eq.1.00) irij=irij+1
      rij(ib)=r1
      if(iweight.eq.1) then
       iatt=ibond(1,ib)
       call eqbnd(ib,iatt,iexbnd,nct)
c----------------------------------------------------------------------------
c      check for symmetry equivalent bonds
c      'Symmetry equivalent' here means corresponding bonds in coordination 
c       polyhedra related by symmetry operations other than pure translations. 
c------------------------------------------------------------------------
       do 150 i=1,nct
	if(rij(iexbnd(i)).eq.1.000) irij=irij+1
	rij(iexbnd(i))=r1
  150  continue
      endif
      go to 135
c---------------------------------------
c   list weights
c---------------------------------------
  160 if(irij.eq.0) then
       write(nout,1055)
 1055  format('All bonds have unit-weights',//)
       call wait
       go to 110
      endif 
      ilst=0
      write(nout,1060)
 1060 format('Bond Number',5X,'Weight')
      do 165 i=1,nbo
       if(rij(i).eq.1.000) go to 165
       write(nout,1065) i,rij(i) 
 1065  format(3X,I3,11X,F5.3)
       ilst=ilst+1
       if(ilst.gt.19) then
	ilst=0
	call wait
       endif
  165 continue
      call wait
      go to 110
  170 write(nout,1070)
 1070 format('Do you really want to reset all your weights to 1.0 ??',/,
     1       '(Y/N/E)')
      call readyn(*173,*177,*900,*170)
  173 irij=0
      do 175 i=1,nbo
  175  rij(i)=1.00
  177 go to 110
  900 return
      end
***********************************************************************
c
c
      subroutine jsr(ier)
c
c   written 88.6.6 by I.D.Brown
c   based on a method proposed by J.S.Rutherford, Acta Cryst. B46, 289 (1990)
c
c   Routine for solving the Kirchhoff equations for a chemical bond
c   network.   The network is brought in through array /span/ and the
c   output is written to xbond(3,)
c    B(m,n) is the adjacency matrix
c    WT(m,m) is a diagonal matrix with the bond weights
c    BB(n,m) is the transpose of B
c    BT(n,nt) and BN(n,nn) contain the spanning tree and chord
c       sections of BB
c    BTT(nt,n) is the transposes of BT
c    C(,) is the cycle matrix
c    BTI(nt,n) is the inverse of BB
c    BIN(m,n) transforms the atomic valences to bond valences
c
c   Subroutines called:
c   MATHLB:axa,axv,amina,aplusa,transa,unitm,zeroa,wait
c   CERNLB:rinv
c
      common/files/nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      include 'comatom.for'
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),
     +            iat,bondlim(2),netind(20)
      dimension ix(100),iw1(100),x(100),wt(100),wtp(100)
      dimension xxx(100,100),yy(100,100),zz(100,100)
      dimension aa(100,100),bb(100,100)
c   -----------------------------------------------------------------------
c     matdim is the fixed dimension of the matrices in the above statements
c     wt(nbo) is a weighting assigned to each bond
c   -----------------------------------------------------------------------
      ier=0
      matdim = 100
      n=nya
      m=nbo
      call zeroa(xxx,m,m,matdim)
      call zeroa(yy,m,m,matdim)
      call zeroa(zz,m,m,matdim)
      call zeroa(aa,m,m,matdim)
      call zeroa(bb,m,m,matdim)
      if(m.gt.matdim.or.n.gt.matdim) then
	write(nout,7200) n,m,matdim
 7200   format(' *error*jsr*7200* Either number of atoms,',I4,
     +  ', or number of bonds,',I4,/
     2  ' exceeds the dimension limit of ',I4)
	ier=1
	return
	endif
c
c   Until required, WT is set to unity
c
      do 10 i=1,m
   10   wt(i) = 1.
   20 write(nout,7100)
 7100 format('Do you want to change the bond-weights? (Y/N/E)')
      call readyn(*30,*50,*999,*20)
   30 write(nout,7110)
 7110 format('Which weight do you want to change?',/,
     1       'Enter bond number:')
      read(nin,7120,err=30) iw
 7120 format(I3)
   40 write(nout,7130) iw
 7130 format('Enter new weight for bond',I4,':',/,
     1       ' (enter as real number, e.g. 2. or 1.5)')
      read(nin,7140, err=40) wt(iw)
 7140 format(F7.4)
   45 write(nout,7150)
 7150 format('Another weight to change? (Y/N/E)')
      call readyn(*30,*50,*999,*45)
c   ---------------------------------------------------------
c   create the transpose, XXX(n,m), of the incidence matrix, B
c   ---------------------------------------------------------
   50 call zeroa(xxx,n,m,matdim)
      do 80 i=1,m
	do 70 j=1,n
	  if(j.eq.ibond(1,i).or.j.eq.ibond(2,i)) then
	    if(oxid(iyatom(1,j)).ge.0) then
	      xxx(j,i) = 1.
	    else
	      xxx(j,i) = -1.
	      endif
	    endif
   70     continue
   80   continue
      if(ndebug.ne.0) then
	do 85 i=1,m
	  write(ndebug,7085) i,(xxx(j,i),j=1,n)
 7085     format('  xxx(j,',I3,') = ',20(20F5.0,/))
   85     continue
	endif
c   ----------------------------------------------------------------------
c   locate the bonds in the spanning tree and
c   write XXX into BB with the spanning tree occupying BB(n,nt) = BT and the
c   chords occupying BB(n,n-m) = BN.  IX(m) stores the addresses of the bonds
c   in this new ordering.  NT = N-1, NN = M-NT when transformation finished
c   This method of locating the spanning tree may not work for all ordering
c   of bonds but should work with any ordering derived from a spanning tree
c   ----------------------------------------------------------------------
      nt=0
      nn=0
      nf=0
      ntn=0
      do 200 i=1,n
	iw1(i)=0
  200   continue
      call zeroa(bb,n,m,matdim)
      do 280 i=1,m
	do 270 j=1,n
	  if(abs(xxx(j,i)).gt.0.001) then
	    do 260 k=(j+1),n
	      if(abs(xxx(k,i)).gt.0.0001) then
		if(iw1(j).eq.iw1(k)) then
		  if(iw1(j).eq.0) then
c       start a new fragment (0,0)
		    nf=nf+1
		    iw1(j)=nf
		    iw1(k)=nf
		    go to 225
		  else
c       bond is not part of the spanning tree (n,n)
		    go to 245
		    endif
		else if(iw1(j).eq.0) then
c       new bond in existing fragment (0,n)
		  iw1(j)=iw1(k)
		  go to 225
		else if(iw1(k).eq.0) then
c       new bond in existing fragment (n,0)
		  iw1(k)=iw1(j)
		  go to 225
		else
c       link two existing frgments (n,m)
		  if(iw1(j).lt.iw1(k)) then
		    iold=iw1(k)
		    inew=iw1(j)
		  else
		    iold=iw1(j)
		    inew=iw1(k)
		    endif
		  do 220 l=1,n
  220               if(iw1(l).eq.iold) iw1(l)=inew 
		  go to 225
		  endif
c      store bond into spanning tree
  225           nt=nt+1
		if(maxdim(nt,m,'error*jsr*230').ne.0) then
		  ier=2
		  if(ndebug.ne.0) write(ndebug,7230) n,m,nt,ntn,
     1                      ((bb(il,jl),il=1,n),jl=1,m)
 7230             format(' *debug*jsr*7230 BB(',I3,',',I3,
     1                      '), NT =',I3,', NTN =',I3,/,
     2                      30(10X,20F5.2))
		  return
		  endif
		do 230 l=1,n
  230             bb(l,nt)=xxx(l,i)
		ix(nt)=i
		go to 280
c         store outside spanning tree
  245           nn=nn+1
		ntn=n-1+nn
		if(maxdim(ntn,m,'error*jsr*250').ne.0) then
		  ier=3
		  if(ndebug.ne.0) write(ndebug,7250) n,m,nt,ntn,nn,
     1                    ((bb(il,jl),il=1,n),jl=1,m)
 7250             format(' *debug*jsr*7250 BB(',I3,',',I3,
     1                    '), NT =',I3,', NTN =',I3,', NN=',I3,/,
     2                    30(10X,20F5.2,/))
		  return
		  endif
		do 250 l=1,n
  250             bb(l,ntn)=xxx(l,i)
		ix(ntn)=i
		go to 280
		endif
  260         continue
	    endif
  270     continue
  280   continue
      do 285 i=1,m  
  285  wtp(i)=wt(ix(i))
      do 286 i=1,m
  286  wt(i)=wtp(i)
c   --------------------------------------
c   Print debug information
c   --------------------------------------
      if(ndebug.ne.0) then
	write(ndebug,7389)
 7389   format(/,' *debug*jsr*7389*  matrices before inversion',/,
     +           '                      -------------------------')
	write(ndebug,7395) n,m,((xx(i,j),i=1,n),j=1,m)
 7395   format(/,' transposed incidence matrix(',I4,',',I4,')',/,
     +              30(10X,20F5.2,/))
	write(ndebug,7390) n,m,((bb(i,j),i=1,n),j=1,m)
 7390   format(/,' Factored TIM(',I4,',',I4,')',/,30(10X,20F5.2,/))
	write(ndebug,7391) m,(wt(i),i=1,m)
 7391   format(/,' WT(',I4,') array ',/,30(10X,20F5.2,/))
	write(ndebug,7392) m,(ix(i),i=1,m)
 7392   format(/,' IX(',I4,') array ',/,30(10X,20I5,/))
	write(ndebug,7393) n,m,nt,nn
 7393   format(/,' N =',I4,', M =',I4,', NT =',I4,', NN =',I4)
	endif
c   ------------------------------------
c   check that nt+nn=m and that nt=n-1
c   ------------------------------------
      if((nt+nn).ne.m.or.nt.ne.(n-1)) then
	write(nout,7400) n,m,nt,nn
 7400   format(' *error*jsr*7400* spanning tree not correctly ',
     +         'found',/,
     2  10X, 'Correct relations are (nt = n-1) and (nt+nn = m)',/,
     3  10X, ' n =',I4,', m =',I4,', nt =',I4,', nn =',I4,/,
     4  10X, ' Returning to menu')
	if(nterm.eq.6) call wait
	ier=4
	return
	endif
c   -----------------------------------------------------------------
c   the following sections are taken from the appendix of JSR's paper
c   AA(nt,n) = BTT is the transpose of BT,
c   XXX(n,n) = AT is the incidence matrix of the spanning tree,
c   ------------------------------------------------------------------
      call zeroa(aa,m,n,matdim)
      call transa(aa,bb,n,nt,matdim,matdim)
      call axa(xxx,bb,aa,n,nt,n,matdim,matdim,matdim)
      do 410 i=1,n
	xxx(i,i)=0.
  410   continue
      do 440 i=1,n
	do 440 j=1,m
	  xxx(i,j)=(-1)*xxx(i,j)
  440     continue
      if(ndebug.ne.0) write(ndebug,7440) n,n,((xxx(i,j),i=1,n),j=1,n)
 7440 format(' *debug*jsr*7440* Incidence matrix(',I4,',',I4,')',/,
     +   30(10X,20F5.2,/))
c   ------------------------------------------------------------
c   by modifying the incidence matrix, the distance matrix of the spanning tree
C   ZZ(n,n) = P is created
c   ------------------------------------------------------------
 1100 if=0
      call zeroa(zz,n,n,matdim)
      do 1190 i=1,n-1
	do 1150 j=i+1,n
	  if(xxx(i,j).gt.0.001) then
	    zz(i,j) = xxx(i,j)
	    zz(j,i) = zz(i,j)
	  else
	    do 1130 k=1,n
	      if(k.ne.i.and.k.ne.j) then
		if(xxx(k,i).gt.0.and.xxx(k,j).gt.0.) then
		  if=1
		  zz(i,j)=xxx(k,i)+xxx(k,j)
		  zz(j,i)=zz(i,j)
		  go to 1150
		  endif
		endif
 1130         continue
	    endif
 1150     continue
 1190   continue
      if(if.gt.0) then
	do 1200 i=1,n
	  do 1200 j=1,n
 1200       xxx(i,j) = zz(i,j)
	go to 1100
	endif
      if(ndebug.ne.0) write(ndebug,8200) n,n,((zz(i,j),i=1,n),j=1,n)
 8200 format(' *debug*jsr*7440* Distance matrix(',I4,',',I4,')',/,
     +   30(10X,20F5.2,/))
c   ------------------------------------------------------------------
c   the inverse of BT is generated from the transpose of BT (= AA)
c   by comparing elements in the distance matrix
c   The inverse BTI(nt,n) is stored in AA
c   ------------------------------------------------------------------
      do 1290 i=1,nt
	do 1230 j=1,n
	  if(aa(i,j).gt.0.) j1=j
	  if(aa(i,j).lt.0.) j2=j
 1230     continue
	do 1270 j=1,n
	  if(j.ne.j1.and.j.ne.j2) then
	    if((zz(j,j2)-zz(j,j1)).gt.0.0001) then
	      aa(i,j)=aa(i,j1)
	    else
	      aa(i,j)=aa(i,j2)
	      endif
	    endif
 1270     continue
 1290   continue  
      do 1390 i=1,nt
	do 1350 j=1,n
	  aa(i,j) = 0.5*aa(i,j)
 1350     continue
 1390   continue
      if(ndebug.ne.0) then
	write(ndebug,8390) m,n,((aa(i,j),i=1,m),j=1,n)
 8390   format(/,' *debug*jsr*8390* inverse spanning tree matrix (
     +   ',I4,',',I4,')',/,30(10X,20F5.2,/))
	endif
c   -------------------------------------------
c   Create the cycle matrix XXX(m,nn) = (BTI*BN | I)
c   -------------------------------------------
      call axa(yy,aa,bb(1,nt+1),nt,n,nn,matdim,matdim,matdim)
      call zeroa(xxx,n,n,matdim)
      do 1450 i=1,nn
	do 1440 j=1,nt
	  xxx(j,i)=-yy(j,i)
 1440     continue
	xxx(i+nt,i)=1
 1450   continue
      if(ndebug.ne.0) then
	write(ndebug,8450) m,nn,((xxx(i,j),i=1,m),j=1,nn)
 8450   format(/,' *debug*jsr*8450* Cycle matrix (',I4,',',I4,
     +          ')',/,30(10X,20F5.2,/))
	endif
c   ---------------------------------------------------
c   This section calculates BB(m,n) = M from Equation 9 of JSR
c   ---------------------------------------------------
      call transa(zz,xxx,m,nn,matdim,matdim)
      do 1460 i=1,nn
	do 1460 j=1,m
 1460     zz(i,j)=zz(i,j)/wt(j)
      if(ndebug.ne.0) write(ndebug,8470) 'ZZ',nn,m,
     +          ((zz(i,j),i=1,nn),j=1,m)
 8470 format(' *debug*jsr*8470* ',A2,'(',I4,',',I4,')',/
     +          30(10X,20F5.2,/))
      call axa(yy,zz,xxx,nn,m,nn,matdim,matdim,matdim)
      if(ndebug.ne.0) write(ndebug,8470) 'YY',nn,nn,
     +          ((yy(i,j),i=1,nn),j=1,nn)
      call rinv(nn,yy,matdim,x,ifail)
      if(ifail.ne.0) then
	write(nout,8460) ifail     
 8460   format(' *error*jsr*8460* ',I4,' error in matrix inversion',/,
     +         ' Returning to menu')
	if(nterm.eq.6) call wait
	ier=5
	return
	endif
      if(ndebug.ne.0) write(ndebug,8470) 'YY',nn,nn,
     +          ((yy(i,j),i=1,nn),j=1,nn)
      call axa(bb,yy,zz,nn,nn,m,matdim,matdim,matdim)
      if(ndebug.ne.0) write(ndebug,8470) 'BB',nn,m,
     +          ((bb(i,j),i=1,nn),j=1,m)
      call axa(zz,xxx,bb,m,nn,m,matdim,matdim,matdim)
      if(ndebug.ne.0) write(ndebug,8470) 'ZZ',m,m,
     +          ((zz(i,j),i=1,m),j=1,m)
      call unitm(yy,m,matdim)
      call amina(xxx,yy,zz,m,m,matdim,matdim,matdim)
      if(ndebug.ne.0) write(ndebug,8470) 'XXX',m,m,
     +          ((xxx(i,j),i=1,m),j=1,m)
      call axa(bb,xxx,aa,m,m,n,matdim,matdim,matdim)
      if(ndebug.ne.0) then
	write(ndebug,8490) m,n,((bb(i,j),i=1,m),j=1,n)
 8490   format(/,' *debug*jsr*8490* Transformation matrix(',I4,',',I4,
     +          ')',/,30(10X,20F5.2,/))
	endif
c   ---------------------------------------------------------
c   use BB(m,n) = M to calculate the new valences and store in XBOND
c   ---------------------------------------------------------
      do 1520 i=1,m
	k=ix(i)
	xbond(3,k)=0.
	do 1510 j=1,n
	  xbond(3,k) = xbond(3,k) + bb(i,j)*oxid(iyatom(1,j))
 1510     continue
 1520   continue
  999  return
      end
c
C
**************************************************************************
C
      subroutine coosphtable(ibv)
c
c     written by Martin Kunz 12/92
c     coosphtable is supposed to find the first coordination shell 
c     around any atom in the structure (with subroutine allcosphr)
c     and displays them in a tabular form without overwriting /span/
c
c     calls: MATHLIB: wait
c            SPANNER: jyat, allcosphr
c            CHEMLIB: bost
c     
c
      implicit character*8 (g)
      implicit character *80 (h)
      double precision xm(500,500),v(500)
      common /files/ nin,nout,ndebug,nread,nwrite,machine
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1              nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2              netind(20),ifbond(4,600)
      common /matrixx/ nm,mm,xm,v,im(500,500)
      common /sym/ nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/ mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1              multsp(30)
      dimension jyatom(5),xxbond(6,20),iibond(3,20),jiatom(5,20) 
      dimension jjyatom(5)
      common/atoms/val(600),rij(600)

      include 'comatom.for'
      include 'combond.for'
c
c
c
  100 iflag=0
      isyfl=0
  140  write(nout,1040)
 1040  format('By default the atom with symmetry operator 1 ',
     1        'is chosen as central atom.',/,
     2        'Do you accept this default? (Y/N/E) ')
       call readyn(*145,*143,*900,*140)
  143 isyfl=1
  145 write(nout,1000)
 1000 format('The structure contains the following independent atoms:')
      j=0
      do 110 i=1,nx
       j=j+1
       write(nout,1010) i,gelem(i), gdent(i)
 1010  format(2X,I3,3X,A2,A6)
       if(j.gt.19) then
	 j=0
	 call wait
       endif
  110 continue
  120 gtype=' '
      write(nout,1020)
 1020 format(/,'Select an atom to display its bonds',/,
     1         'or hit <RETURN> to exit')
      read(nin,1025) gtype
 1025 format(A2)      
      if(gtype.eq.' ') go to 900
      ict=1
      ict=num(gtype,3)
      if((ict.gt.nx).or.(ict.le.0)) go to 120
  130 isym=1
  135  if(isyfl.ne.1) go to 180
  150  ii=0
       write(nout,1050)
 1050  format(1X,'Sym.Op.',4X,'Rot.Matrix',9X,'Transl.Matrix')
       do 160 i=1,nsym
	ii=ii+1
	write(nout,1060)(s(1,j,i),j=1,3),i,(s(2,j,i),j=1,3),
     1                  (t(j,i),j=1,3),(s(3,j,i),j=1,3)
 1060   format(10X,3(F5.1),/,2X,I3,5X,3(F5.1),4X,3(F5.1),/,
     1         10X,3(F5.1),/)
	if(ii.gt.4) then
	 call wait
	 ii=0
	endif
  160  continue
  170  write(nout,1070) 
 1070  format('Enter the symmetry operation:')
       readin(nin,1030,err=170) isym
 1030  format(I5) 
       if(isym.eq.0) go to 170
  180 do 190 i=1,nx
       if(i.ne.ict) go to 190
c----------------------------------------------------------------
c      searches for coordination sphere in /span/ with subroutine
c      allcosphr.(ict = address in /ident/)
c-----------------------------------------------------------------
       call allcosphr(ict,isym,xxbond,iibond,jiatom)
c------------------------------------------------------------------------       
c      test for 0-result of allcosphr due to redundant symmetry operation
c------------------------------------------------------------------------
       if(iibond(3,1).eq.0) then
	write(nout,1080) isym,gelem(ict),gdent(ict)
 1080   format('Symmetryoperation',I3,1X,'for atom ',A3,A3,' is ',
     1         'redundant.',/,'Try a lower symmetryoperation!')
	call wait
	go to 130
       endif
       write(nout,1090) gelem(ict),gdent(ict),isym
 1090  format(//'Environment of',1X,A2,1X,A6,3X,'Sym.: ',I2,2X,/)
       write(nout,1100) 
 1100  format('Bnd.Nr.',2X,'Atom:',1X,'Sym:',8X,
     1        'distances',5X,'bond valences',6X,'  Weight  ',/,
     2        24X,'calc.',4X,'obs.',5X,'calc.',5X,'obs.')
       vals1=0.
       vals2=0.
       do 200 j=1,icn(ict)
       call jyat(iibond(1,j),jjyatom)
       call jyat(iibond(2,j),jyatom)
       call bost(ict,jyatom(1),valobs,valerr,xxbond(2,j),xxbond(1,j))
       bleng=dbost(gelem(jyatom(1)),gelem(jjyatom(1)),oxid(jyatom(1)), 
     1            oxid(jjyatom(1)),xbond(3,iibond(3,j))) 
	write(nout,1110)iibond(3,j),gelem(jyatom(1)),
     1      gdent(jyatom(1)),jyatom(2),(bleng*ibv),xxbond(1,j),
     2      (xbond(3,iibond(3,j))*ibv),valobs,rij(iibond(3,j))
 1110   format(1X,I3,4X,A2,A4,I3,6X,F6.3,3X,F6.3,3X,F6.3,4X,F6.3,
     2         5X,F6.3)
	vals1=vals1+((xbond(3,iibond(3,j))-abs(valobs))*
     1               (xbond(3,iibond(3,j))-abs(valobs)))
	vals2=vals2+
     1          ((xbond(1,iibond(3,j))-bleng)*
     2          (xbond(1,iibond(3,j))-bleng)) 
  200  continue
       vals1=SQRT(vals1/icn(ict))
       vals2=SQRT(vals2/icn(ict))
       write(nout,1120)gelem(ict),gdent(ict),vals1,
     1                 gelem(ict),gdent(ict),vals2
 1120  format('rms bond valence deviation for',1X,A3,A5,':',F8.5,/,
     1        'rms distance deviation for',1X,A3,A5,':',F8.5)
       gwait='0'
       write(nout,1125)
 1125  format(/,'Hit: <RETURN> to go back to atom list',/
     1          '   : "c" to continue listing of atoms')
       read(nin,1025) gwait
       if(gwait.eq.'c') then
	ict=i+1
	go to 190
       endif
       go to 145 
  190 continue
      go to 145
  900 return
      end
*************************************************************************
c
      subroutine polplot
c
c     written by Martin Kunz 12/92
c
c     Polplot is designed to produce a pseudoplot of the first coordination
c     sphere of any chosen atom in the structure. The pseudoplot-procedure
c     (capsuplo) is a modification of the structure pseudoplot-procedure 
c     (psuplo) written by D.Altermatt
c
c     calls MATHLIB: wait, readyn
c           SPANNER: allcosphr, jyat
c           BOST: chemlib
c
c
      implicit character*8 (g)
      implicit character *80 (h)
      double precision xm(500,500),v(500)
      common /files/ nin,nout,ndebug,nread,nwrite,machine
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1              nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2              netind(20),ifbond(4,600)
      common /matrixx/ nm,mm,xm,v,im(500,500)
      common /sym/ nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/ mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1              multsp(30)
      dimension xxbond(6,20),iibond(3,20),jiatom(5,20) 

      include 'comatom.for'
      include 'combond.for'
c
c---------------------------------------------------------------------
c     Selection of the central atom                        
c---------------------------------------------------------------------
  100 write(nout,1000)
 1000 format('The structure contains the following independent atoms:')
      j=0
      do 110 i=1,nx
       j=j+1
       write(nout,1010) i,gelem(i), gdent(i)
 1010  format(2X,I3,3X,A2,A6)
       if(j.gt.19) then
	 j=0
	 call wait
       endif
  110 continue
  120 write(nout,1020)
 1020 format(/,'Which atom do you want as central atom?')
      read(nin,1030,err=120) itype
 1030 format(I5) 
      if((itype.lt.1).or.(itype.gt.nx)) go to 100
      ict=itype
  130 isym=1
c---------------------------------------------------  
c     asks whether Sym.op.-default should be changed:
c---------------------------------------------------
  140 write(nout,1040)
 1040  format('By default the atom with symmetry operator 1 ',
     1        'is chosen as central atom.',/,
     2        'Do you want to change this default? (Y/N/E) ')
      call readyn(*150,*180,*900,*140)
c-------------------------------------- 
c     asks for Sym.Op.:
c--------------------------------------
  150  ii=0
       write(nout,1050)
 1050  format(1X,'Sym.Op.',4X,'Rot.Matrix',9X,'Transl.Matrix')
       do 160 i=1,nsym
	ii=ii+1
	write(nout,1060)(s(1,j,i),j=1,3),i,(s(2,j,i),j=1,3),
     1                  (t(j,i),j=1,3),(s(3,j,i),j=1,3)
 1060   format(10X,3(F5.1),/,2X,I3,5X,3(F5.1),4X,3(F5.1),/,
     1         10X,3(F5.1),/)
	if(ii.eq.5) then
	 call wait
	 ii=0
	endif
  160  continue
  170 write(nout,1070) 
 1070  format('Enter the symmetry operation:')
      readin(nin,1030,err=170) isym
      if(isym.eq.0) go to 170
  180 call allcosphr(ict,isym,xxbond,iibond,jiatom)
      if(iibond(3,1).eq.0) then
       write(nout,1080) isym,gelem(ict),gdent(ict)
 1080  format('Symmetryoperation',I3,1X,'for atom ',A3,A3,'is ',
     1         'redundant.',/,'Try a lower symmetryoperation!')
       go to 130
      endif
      call capsuplo(ict,xxbond,iibond)
  900 return
      end
**************************************************************************

**************************************************************************
c
      subroutine eqbnd(ibnd,iatt,iexbnd,nct)
c
c     This subroutine looks for atoms related to iatt by symmetry operations 
c     other than pure translations. Within the coordination polyhedron
c     of these atoms, bonds equivalent to ibnd are found and written into 
c     iexbnd. Nct is a book-keeping variable, which counts the numbers of
c     bonds found (i.e. the number of symmetry related atoms)
c     ibnd=input, bond# in /span/ for which a symmetric equivalent is searched for 
c     iatt=input, central atom # in /span/ the symmetric equivalent of which 
c                 has to be found
c     iexbnd=output; bond # in /span/ of bonds found
c     nct=output; amount of bonds found
c 
c     written Jan/Feb 1993 by Martin Kunz
c
c     calls SPANNER: jyat
c           SYMLIB: symop
c           MATHLIB: vminv
c
      implicit character*8 (g)
      implicit character *80 (h)
      common /files/ nin,nout,ndebug,nread,nwrite,machine
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1              nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2              netind(20),ifbond(4,600)
      common /sym/ nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/ mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1              multsp(30)
      common /cell/ a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      dimension jyatom(5),iexbnd(500),xvect(3),xvect2(3),xvect3(3)
      dimension xorg(3),xorg2(3),jjyatom(5)
      include 'comatom.for'
      include 'combond.for'
      
      nct=0
       do 115 j=1,3
  115   xorg(j)=0.0
c-----------------------------------------------------------------------------  
c     sets the bond vector of ibnd into xvect. 
c     Bond vector always points away from iatt
c-----------------------------------------------------------------------------
      do 100 j=1,3
  100  xvect(j)=xbond(j+3,ibnd)
	     write(ndebug,1004) (xvect(k),k=1,3)
 1004        format('**DEBUG**EQBND*1004*bonding vector 1',
     1              3(F8.4,1X))
      if(ibond(2,ibnd).eq.iatt) then
       do 105 j=1,3
  105   xvect(j)=-xvect(j)
      endif
c----------------------------------------------------------------------------         
c      calculates the direction of the bond vector for any symmetry
c      operation, which is non-redundant for iatt.
c----------------------------------------------------------------------------
      do 120 j=2,nsym
       if(ndebug.ne.0) then
	write(ndebug,1014) (xvect(k),k=1,3),j
 1014   format('**DEBUG**EQBND*1014*bonding vector before symm.op.',
     1          3(F8.4,1X),'Symop: ',I3)
       endif
       call symop(xvect2,s(1,1,j),t(1,j),xvect)
       call symop(xorg2,s(1,1,j),t(1,j),xorg)
       call vminv(xvect3,xvect2,xorg2)
       if(ndebug.ne.0) then
	write(ndebug,1015) (xvect3(k),k=1,3)
 1015   format('**DEBUG**EQBND*1015*bonding vector after symm.op.',
     1          3(F8.4,1X))
       endif
c----------------------------------------------------------------------------  
c       goes through all bonds and finds the bond number which corresponds
c       to the symmetry-transformed xvect.
c----------------------------------------------------------------------------
  130  do 135 jj=1,nbo
	if(jj.eq.ibnd) go to 135 
	do 140 k=1,3
  140    xvect2(k)=xbond(k+3,jj)
	call jyat(ibond(1,jj),jyatom)
	call jyat(iatt,jjyatom)
c----------------------------------------------------------------------------         
c       makes sure the bondvector to be compared with xvect points into
c       the right direction
c----------------------------------------------------------------------------
	if(jyatom(1).ne.jjyatom(1)) then
	 do 145 k=1,3
  145     xvect2(k)=-xvect2(k)   
	 call jyat(ibond(2,jj),jyatom)
	 if(jyatom(1).ne.jjyatom(1)) go to 135
	endif
	if(ndebug.ne.0) then
	 write(ndebug,1290)jj,(xvect3(k),k=1,3),(xvect2(k),k=1,3),
     1    lcomve(xvect3,xvect2,3,4)
 1290    format('*DEBUG*EQBND*1290*** vector-comparison:',/, 
     1          'bond-number: ',I3,/,
     2          'xvect= ',3(F8.4,1X),'xvect2= ',3(F8.4,1X),
     3          'lcomve= ',I3)
	endif
	if(lcomve(xvect3,xvect2,3,4).eq.0) then
c----------------------------------------------------------------------------          
c       if bond found: increases nct (=# of symmetry equivalent bonds) 
c                      writes bond number into iexbnd array
c-----------------------------------------------------------------------------
	 if(nct.ne.0) then
	  do 147 k=1,nct
  147      if(jj.eq.iexbnd(k)) go to 135
	 endif
	 nct=nct+1
	 iexbnd(nct)=jj
	 if(ndebug.ne.0) then
	  write(ndebug,1300) nct,iexbnd(nct)
 1300     format('*DEBUG*EQBND*1300* nct=',I3,1X,'iexbnd=',I3)
	 endif
	 go to 120
	endif
  135  continue
  120 continue
      return
      end
***********************************************************************
c
      function lcomve(x,y,n,id)
c
c     written 6. may '93 by M.Kunz
c
c     returns lcomve=0 if x and y are within +/- 1 within id decimals.
c
      dimension x(n),y(n)
      lcomve=0
      do 100 i=1,n
       x1=x(i)*10.**(id)
       x2=y(i)*10.**(id)
       if(abs(x1-x2).gt.1.) lcomve=1
  100 continue
      return
      end
***********************************************************************
c 
      subroutine vlink(istart,iend,vector,ier)
c
c   written 87.10.20 by I.D.Brown
c
c   Finds the vector between atoms istart and iend in the iyatom list
c   by finding the shortest bond path between these atoms and adding
c   the appropriate vectors.
c
c   calls link
c
      common/files/nin,nout,ndebug
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
      dimension lnk(50),vector(3)
c
      call link(istart,iend,nl,lnk,ier)
      if(ier.ne.0) go to 900
      do 100 i=1,3
	vector(i)=0.
	do 50 j=1,nl
	  ll=iabs(lnk(j))
	  vector(i)=vector(i)+xbond(3+i,ll)
   50     continue
  100   continue
      if(ndebug.ne.0) then
	nll=nl
	if(nll.gt.26) nll=26
	write(ndebug,1100) istart,iend,(vector(j),j=1,3),nl,
     +     (lnk(j),j=1,nll)
 1100   format(' *debug*vlink*1100* ',2I3,3F6.3,I3,1X,26I3)
	endif
      return
  900 continue
      return
      end
c
c
***********************************************************************
c
      subroutine link(istart,iend,nlink,lnk,ier)
c
c   written 87.10.16 by I.D.Brown
c
c   This routine finds the shortest link in /span/ between atoms
c   istart and iend.   The result is a list of nl bonds stored in
c   lnk(ml).
c
c  The algorithm is based on that of E.B.Hunt in 'Artificial Intelligence'
c  page 226.
c
c   ier=1 for error in OPEN
C       2              CLOSE
C       3              CLOSE
C       4
C       5              PULL
c       6 memory overflow in LINK
C
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/nin,nout,ndebug
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     +              nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c programmer's note:  /loop/ changed to /loopp/ under
C                     Unix system
      common/loopp/ ml,io1,io2,ic,iop(1000),icl(1000)
      dimension lnk(50)
c
      ml=1000
      mlink=50
      io1=0
      io2=0
      ic=0
c
c   since the program stores the link starting from the end (rather than
c   the beginning) the search is done backwards i.e. the search starts at
c   atom iend and finds a route to istart.  Then the bonds are pulled of
c   from istart to iend, so that they are stored in link correctly
c
c   open all the bonds around iend
c
      do 50 i=1,nbo
	if(ibond(1,i).eq.iend) then
	  k=i
	  if(ibond(2,i).eq.istart) then
	    k1=i
	    go to 200
	    endif
	else if(ibond(2,i).eq.iend) then
	  k=-i
	  if(ibond(1,i).eq.istart) then
	    k1=-i
	    go to 200
	    endif
	else
	  go to 50
	  endif
	call open(k,ier)
	if(ier.ne.0) go to 900
   50   continue
c
c   close the next bond
c
  100 call close(j,ier)
      if(ier.ne.0) go to 900
c
c   look for match of node2(j)
c
      do 150 k=1,nbo
	if(k.ne.iabs(j).and.k.ne.iabs(iop(1))) then
	  if(node2(j).eq.ibond(1,k)) then
	    k1=k
	    if(ibond(2,k).eq.istart) go to 200
	  else if(node2(j).eq.ibond(2,k)) then
	    k1=-k
	    if(ibond(1,k).eq.istart) go to 200
	  else
	    go to 150
	    endif
c
c   avoid atoms and bonds already in the link
c
	  do 120 kl=1,io2
	    if(k1.eq.iop(kl).or.node2(k1).eq.node2(iop(kl))) go to 150
  120       continue
c
c   open new bond
c
	  call open(k1,ier)
	  if(ier.ne.0) go to 900
	  endif
  150   continue
      go to 100
c
c   extract the link, starting at the end (hence lnk = -k1)
c
  200 continue
      lnk(1)=-k1
      nlink=1
      if(node1(k1).eq.iend) go to 300
c
c     begin implicit loop through the bonds
  220   continue
	nod1=node1(k1)
	call pull(nod1,jj,ier)
	if(ier.ne.0) go to 900
	nlink=nlink+1
	if(maxdim(nlink,mlink,'*link*220*').ne.0) then
	  ier=6
	  go to 900
	  endif
	lnk(nlink)=-jj
	k1=jj
	if(node1(jj).ne.iend) go to 220
c       end implicit loop through the bonds
c
  300 continue
      return
  900 write(nout,1900)ier
 1900 format(' *error*link*1900* error type ',I3)
      if(ndebug.ne.0) then
	nio2=io2
	if(nio2.gt.18) nio2=15
	nic=ic
	if(nic.gt.18) nic=15
	write(ndebug,1901)io1,io2,ic,(iop(j),j=1,nio2)
	write(ndebug,1902) (icl(j),j=1,nic)
 1901   format(' *debug*link*1901* ',3I4,1X,15I3)
 1902   format('+',77X,15I3)
	endif
      return
      end
c
c
***********************************************************************
c
      SUBROUTINE NODE
C                ----
C
C    WRITTEN 85.1.15 BY I.D.BROWN
C
C   CREATES THE LINES IN XM CORESPONDING TO THE NODE SUMS
C
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500)
C
      common /files/ nin,nout,ndebug
      include 'comatom.for'
      COMMON /DLS/   NFBON(2,600)
      common /span/nya,mya,iyatom(5,300),iterm(300),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      vsum=0.
      do 50 i=1,nya
	v(i)=oxid(iyatom(1,i))
	vsum=vsum+v(i)
   50   CONTINUE
      if(abs(vsum).gt.0.03) write(nout,1050) vsum
 1050 format(//,' ++error++node++1050++ The sum of all the oxidation ',
     1        'states is ',F6.3,/,
     1          '                        but should be 0.000')
      DO 100 I=1,nbo
	im(ibond(1,i),i)=nfbon(1,i)
	im(ibond(2,i),i)=-nfbon(1,i)
  100   CONTINUE
      NM=nya-1
      DO 120 I=1,NM
	DO 120 J=1,nbo
  120     XM(I,J)=IM(I,J)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE LOOPER
C                ----
C
C   WRITTEN 85.1.15 BY I.D.BROWN
C
C   CALLS CLOSE, LOOP, OPEN
C
c ***************************************************************************
c ********* Although this code is stil active, it is inefficient and*********
c *********unlikely to be used.  There may be problems with the dimensions***
c *********of some arrays ***************************************************
c
C   FINDS THE LOOPS IN A NETWORK USING A BREADTH SEARCH ALGORITHM (SEE E.B.HUNT
C    - ARTIFICIAL INTELLIGENCE P226)
C   IN THE FIRST RUN THE ROUTINE CALLS LOOP TO FIND THE SHORTEST LOOP THAT
C   INCLUDES ONE BOND THAT HAS NOT YET BEEN USED.
C   IF THIS DOES NOT FIND ALL THE LOOPS THE ROUTINE THEN LOOKS FOR LOOPS
C   THAT HAVE NO BONDS IN COMMON WITH OTHER LOOPS AND THEN LOOKS FOR A
C   LOOP STARTING WITH ONE OF THESE BONDS BUT FINDING THE SECOND SHORTEST
C   LOOP WITH THAT BOND.   THE FLAG ITIMES IS USED TO FORCE THIS
C
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500)
C
      include 'comatom.for'
      include 'combond.for'
      COMMON /FILES/ NIN,NOUT,NDEBUG
c programmer's note:  /loop/ changed to /loopp/ under
c                     Unix system
      COMMON /LOOPP/  ML,IO1,IO2,IC,IOP(1000),ICL(1000)
      COMMON /DLS/   NFBON(2,600)
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      DIMENSION ILOOP(500), JBOND(300),NOLOOP(500,500),ILN(100)
C
C   FIND AN UNUSED BOND AND USE THIS AS THE BEGINING OF A LOOP
C
      DO 250 I=1,nbo
	IF(NFBON(2,I).EQ.0) THEN
	  IO1=0
	  IO2=0
	  IC=0
	  NM=NM+1
	  I10 = 0
	  CALL LOOP(I,IBOND(1,I),IBOND(1,I),LAST,I10,IER)
	  ENDIF
  250   CONTINUE
      IF(nbo.EQ.NM) RETURN
C
C     AT THIS POINT EVERY BOND IS IN A LOOP
C     BUT WE DO NOT HAVE ENOUGH EQUATIONS.   THIS
C     MEANS THAT THE LOOPS EXIST IN TWO OR MORE GROUPS WHICH DO NOT SHARE
C     A COMMON LOOP
C     THE NEXT BIT OF PROGRAM LABELS (IN ILOOP AND IBOND) WHICH SET EACH
C     LOOP AND EACH BOND BELONGS TO AND THEN SEARCHES FOR LOOPS THAT LINK
C     THESE
C
      DO 405 I=1,NM
  405   ILOOP(I)=0
      DO 407 I=1,nbo
  407   JBOND(I)=0
      ISET=0
      I=NYA
C
C   FIND A CONNECTED SEQUENCE OF LOOPS
C
  410 CONTINUE
      IO1=0
      IO2=0
      IC=0
      ISET=ISET+1
      CALL OPEN(I,IER)
      IF(IER.NE.0) GO TO 900
  420 CALL CLOSE(J,IER)
      IF(IER.EQ.3) GO TO 500
      IF(IER.NE.0) GO TO 900
      ILOOP(J)=ISET
      DO 480 K=1,nbo
	IF(ABS(XM(J,K)).GT.0.01) THEN
	  IF(JBOND(K).EQ.ISET) GO TO 480
	  JBOND(K)=ISET
	  DO 440 L=NYA,NM
	    IF(ABS(XM(L,K)).GT.0.01.AND.L.NE.J) THEN
	      CALL OPEN(L,IER)
	      IF(IER.NE.0) GO TO 900
	      ENDIF
  440       CONTINUE
	  ENDIF
  480   CONTINUE
      GO TO 420
C
C   TEST IF ANY LOOPS HAVE NOT BEEN ASSIGNED TO A SET
C
  500 DO 520 K=NYA,NM
	IF(ILOOP(K).EQ.0) THEN
	  I=K
	  GO TO 410
	  ENDIF
  520   CONTINUE
      IF(NDEBUG.NE.0) WRITE(NDEBUG,5201) (K,ILOOP(K),JBOND(K),K=1,nbo)
 5201 FORMAT(' *DEBUG*LOOPER*5201* ',I3,' ILOOP=',I4,' JBOND= ',I4)
      IF(ISET.LE.1) GO TO 700
C
C   FIND A COMMON ATOM BETWEEN BONDS IN DIFFERENT SETS
C
      DO 590 I=1,nbo
	I1=I+1
	DO 580 J=I1,nbo
	  IO1=0
	  IO2=0
	  IC=0
	  IF(JBOND(I).NE.JBOND(J)) THEN
	    IF(IBOND(1,I).EQ.IBOND(1,J)) THEN
	      ISTART=IBOND(2,I)
	      I2=-I
	      J2=J
	      GO TO 600
	    ELSE IF(IBOND(1,I).EQ.IBOND(2,J)) THEN
	      ISTART=IBOND(2,I)
	      I2=-I
	      J2=-J
	      GO TO 600
	    ELSE IF(IBOND(2,I).EQ.IBOND(1,J)) THEN
	      ISTART=IBOND(1,I)
	      I2=I
	      J2=J
	      GO TO 600
	    ELSE IF(IBOND(2,I).EQ.IBOND(2,J)) THEN
	      ISTART=IBOND(1,I)
	      I2=I
	      J2=-J
	      GO TO 600
	      ENDIF
	    GO TO 580
  600       CONTINUE
	    IBONDJ=JBOND(J)
	    DO 610 KK=1,nbo
  610         IF(JBOND(KK).EQ.IBONDJ) JBOND(KK)=JBOND(I)
	    CALL OPEN(I2,IER)
	    IF(IER.NE.0) GO TO 900
	    CALL CLOSE(K,IER)
	    IF(IER.NE.0) GO TO 900
	    NM=NM+1
	    I10 = 0
	    CALL LOOP(J2,ISTART,ISTART,LAST,I10,IER)
	    IF(IER.EQ.0) THEN
	      DO 630 KK=1,nbo
		IF(ABS(XM(NM,KK)).GT.0.01) THEN
		  IBONDK=JBOND(KK)
		    IF(IBONDK.NE.JBOND(I)) THEN
		    DO 620 KL=1,nbo
  620                 IF(JBOND(KL).EQ.IBONDK) JBOND(KL)=JBOND(I)
		    ENDIF
		  ENDIF
  630           CONTINUE
	      IF(NM.EQ.nbo) RETURN
	      ENDIF
	    ENDIF
  580     CONTINUE
  590   CONTINUE
C
C    ALL THE LOOPS ARE NOW LINKED BUT THERE MAY BE SOME ATOMS THAT
C    DO NOT HAVE SUFFICIENT LOOPS PASSING THROUGH THEM.  THEY SHOULD
C    HAVE NOT HAVE FEWER THAN ONE LESS THAN THE COORDINATION NUMBER
C    (EXCLUDING TERMINAL BONDS THAT DO NOT BELONG TO ANY LOOP)
C
C    STORE IN NOLOOP THE LOOPS WHICH PASS THROUGH A GIVEN 
C
  700 DO 710 I=1,MM
	DO 710 J=1,MM
  710     NOLOOP(I,J)=0
      DO 730 I=NYA,NM
	I1=I-NYA+1
	DO 720 J=1,nbo
	  IF(ABS(XM(I,J)).GT.0.01) THEN
	    IBON1=IBOND(1,J)
	    IBON2=IBOND(2,J)
	    NOLOOP(IBON1,I1)=1
	    NOLOOP(IBON2,I1)=1
	    ENDIF
  720     CONTINUE
  730   CONTINUE
C
C   FIND THE EFFECTIVE COORDINATION NUMBER OF EACH NODE
C     (DO NOT COUNT TERMINAL BONDS THAT DO NOT BELONG IN LOOPS)
C
      DO 750 I=1,NYA
	ICN(I)=0
	DO 740 J=1,nbo
	  IF(IM(I,J).NE.0.AND.JBOND(J).NE.0)
     +      ICN(I)=ICN(I)+1
  740     CONTINUE
  750   CONTINUE
C
C   FIND THE NODES THAT HAVE INSUFFICIENT LOOPS
C    THESE WILL HAVE VALUES OF ILN.GT.0
C
      NM1=NM-NYA+1
      DO 770 I=1,NYA
	ILN(I)=ICN(I)-1
	DO 760 J=1,NM1
	  ILN(I)=ILN(I)-NOLOOP(I,J)
  760     CONTINUE
  770   CONTINUE
C
C   LOOK FOR A SUITABLE LOOP THROUGH NODES THAT HAVE TOO FEW LOOPS
C
C   FIRST TRY TO FIND A LOOP CONNECTING TWO NODES WITH INSUFFICIENT
C   LOOPS,  CONNECTING THEM ALONG A PATH IN WHICH NO ADJACENT BONDS
C   ARE SHARED WITH ANOTHER LOOP
      DO 800 I=1,NYA
	IF(ILN(I).GT.0) THEN
	  IF(NDEBUG.NE.0) WRITE(NDEBUG,7701) ILN(I),I
 7701     FORMAT(' *DEBUG*LOOPER*7701* ',I3,' TOO FEW LOOPS AT ATOM ',
     +     I3)
	  I1=I+1
	  DO 790 J=I1,NYA
	    IF(ILN(J).GT.0) THEN
	      DO 780 L=1,nbo
		IF(IM(I,L).NE.0) THEN
		  IO1=0
		  IO2=0
		  IC=0
		  NM=NM+1
		  I10 = 1
		  CALL LOOP(L,I,J,K,I10,IER)
		  IF(IER.EQ.0) THEN
		    DO 778 KK=1,nbo
		      IF(IM(J,KK).NE.0.AND.KK.NE.IABS(K)) THEN
			IO1=0
			IO2=0
			IC=0
			I10 = 0
			CALL LOOP(KK,J,I,L,I10,IER)
			IF(IER.NE.0) THEN
			  NM1=NM+1
			  DO 775 KKK=1,nbo
  775                       XM(NM1,KKK)=0.
			    GO TO 778
			  ENDIF
			ILN(I)=ILN(I)-1
			ILN(J)=ILN(J)-1
			IF(NM.EQ.nbo) RETURN
			IF(ILN(I).LE.0) GO TO 800
			IF(ILN(J).LE.0) GO TO 790
			ENDIF
  778                 CONTINUE
		    ENDIF
		  ENDIF
  780           CONTINUE
	      ENDIF
  790       CONTINUE
C   FIND ANY NEW LOOP THROUGH THIS BOND
	  DO 795 L=1,nbo
	    IF(IM(I,L).NE.0) THEN
	      IO1=0
	      IO2=0
	      IC=0
	      NM=NM+1
	      I10 = 2
	      CALL LOOP(L,I,I,K,I10,IER)
	      IF(IER.NE.0) GO TO 795
	      ILN(I)=ILN(I)-1
	      IF(ILN(I).LE.0) GO TO 800
	      IF(NM.EQ.nbo) RETURN
	      ENDIF
  795       CONTINUE
	  ENDIF
  800   CONTINUE
C
C   IF THERE ARE STILL INSUFFICIENT LOOPS IT IS PROBABLY BECAUSE
C   THERE ARE POTENTIAL LOOPS THAT EXIST IN WHICH NO TWO ADJACENT
C   BONDS ARE LINKED INTO THE SAME LOOP.   WE NOW LOOK FOR THESE
C
C   LOOK FOR LOOPS THAT HAVE NO ADJACENT BONDS IN COMMON WITH
C   ANY OTHER LOOP
C
      IF(NDEBUG.NE.0) WRITE(NDEBUG,8001)
 8001 FORMAT(' *DEBUG*LOOPER*8001*  LOOKING FOR ANY LOOP WITHOUT'
     +  ,' ADJACENT BONDS ALREADY CONNECTED IN OTHER LOOPS')
      DO 810 I=1,nbo
	IO1=0
	IO2=0
	IC=0
	NM=NM+1
	I10 = 1
	CALL LOOP(I,IBOND(1,I),IBOND(1,I),K,I10,IER)
	IF(NM.EQ.nbo) RETURN
  810   CONTINUE
      RETURN
  900 WRITE(NOUT,9001) IER
 9001 FORMAT(' *ERROR*LOOPER*9001* OVERFLOW ERROR =',I2)
      RETURN
  910 WRITE(NOUT,9101) ISET,nbo-NM
 9101 FORMAT(' *ERROR*LOOPER*9101* ',I3,' SETS FOUND, ',I3,
     +  ' EQUATIONS NEEDED')
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE LOOP(I,ISTART,IEND,K2,IFLAG,IER)
C
C    WRITTEN 85.1.23 BY I.D.BROWN
C
C    CALLS CLOSE, NODE1, NODE2, OPEN, PULL, STORE2
C
C   IF(IFLAG.EQ.0) THEN
C    THIS ROUTINE FINDS THE SHORTEST LINK THAT STARTS WITH BOND I
C    AND ENDS AT ATOM IEND.  ALL CONNECTIONS BETWEEN ATOMS ISTART
C    AND IEND WILL BE STORED IN THE MATRIX (PROVIDING THAT ISTART
C    IS THE ATOM AT THE BEGINNING OF THE FIRST BOND IN OPEN)
C   IF(IFLAG.EQ.1) THE LINK WILL CONTAIN NO TWO CONSECUTIVE BONDS
C    THAT ALREADY BELONG TO ANOTHER LOOP
C   IF(IFLAG.EQ.2) THE PROGRAM ENSURES THAT IT DOES NOT FIND AN
C    EXISTING LOOP
C   IF O LINK IS FOUND IER=3
C
C   This routine uses the breadth search algorithm given by E.B.Hunt
c    in 'Artificial Intelligence', p.226
c
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500)
C
      COMMON /FILES/ NIN,NOUT,NDEBUG,nread,nwrite
c programmer's note:  /loop/ changed to /loopp/ under
c                     Unix system
      COMMON /LOOPP/  ML,IO1,IO2,IC,IOP(1000),ICL(1000)
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
	CALL OPEN(I,IER)
	IF(IER.NE.0) GO TO 900
C
C   CLOSE NEXT BOND
C
   50   CALL CLOSE(J,IER)
	IF(IER.NE.0) GO TO 900
C
C   LOOK FOR MATCH OF NODE2(J)
C
	DO 100 K=1,nbo
	  IF(K.NE.IABS(J).AND.K.NE.IABS(IOP(1))) THEN
	    IF(NODE2(J).EQ.IBOND(1,K).OR.NODE2(J).EQ.IBOND(2,K)) THEN
c       avoid adjacent bonds in an existing loop if iflag=1
	      IF(IFLAG.EQ.1) THEN
		DO 65 L=NYA,NM-1
		  IF(ABS(XM(L,J)).GT.0.001.AND.ABS(XM(L,K))
     +               .GT.0.001) THEN
		    IF(NDEBUG.NE.0) WRITE(NDEBUG,6501) J,K,L
 6501               FORMAT(' *DEBUG*LOOP*6501* ',2I3,'ARE ADJACENT ',
     +                   'BONDS IN LOOP ',I3)
		    GO TO 100
		    ENDIF
   65             CONTINUE
		ENDIF
c       set k1=+/-k depending on direction of bond k
	      IF(NODE2(J).EQ.IBOND(1,K)) THEN
		K1=K
		IF(IBOND(2,K).EQ.IEND) GO TO 150
		GO TO 70
	      ELSE IF(NODE2(J).EQ.IBOND(2,K)) THEN
		K1=-K
		IF(IBOND(1,K).EQ.IEND) GO TO 150
		GO TO 70
		ENDIF
	      GO TO 100
c        avoid bonds and atoms that have already been visited
   70         DO 75 KL=1,IO2
		IF(K1.EQ.IOP(KL)) GO TO 100
		if(node2(k1).eq.node2(iop(kl))) go to 100
   75           continue
c        open the new bond
	      CALL OPEN(K1,IER)
	      IF(IER.NE.0) GO TO 900
	      ENDIF
	    ENDIF
  100     CONTINUE
	GO TO 50
  150   CONTINUE
	IF(NDEBUG.NE.0) THEN
	  WRITE(NDEBUG,1120) NM,I,ISTART,K1,IEND
	  WRITE(NDEBUG,1121) (IOP(L),L=1,IO2)
	  WRITE(NDEBUG,1122) (ICL(L),L=1,IC),K
 1120     FORMAT(' *DEBUG*LOOP*1120* NM=',I4,' START(BOND/ATOM)=',
     +        2I4,' END(BOND/ATOM)=',2I4)
 1121     FORMAT(' *DEBUG*LOOP*1121* OPEN=',100(10I4,/,24X))
 1122     FORMAT(' *DEBUG*LOOP*1122* CLOSE=',100(10I4,/,25X))
	  ENDIF
C
C    STORE THE LOOP THAT HAS BEEN FOUND
C
	ICSAVE=IC
	CALL STORE2(K1)
c       begin implicit loop through bonds of loop
  200     CONTINUE
*         IF(NDEBUG.NE.0) WRITE(NDEBUG,1123) K1,NODE1(K1),NODE2(K1)
 1123     FORMAT(' *DEBUG*LOOP*1123* K1,NODE1(K1),NODE2(K1)=',3I4)
	  NOD1=NODE1(K1)
	  CALL PULL(NOD1,JJ,IER)
	  IF(IER.NE.0) GO TO 900
	  CALL STORE2(JJ)
	  K1=JJ
	  IF(NODE1(JJ).NE.ISTART) GO TO 200
c         end implicit loop through bonds of loop
C
C   IF IFLAG.EQ.2 TEST IF THIS MATRIX LINE HAS ALREADY BEEN GENERATED.
C   IF SO, CONTNUE SEARCHING FOR A NEW LOOP TERMINATION
C
	IF(IFLAG.EQ.2) THEN
	  DO 250 L=NYA,NM-1
	    DO 240 K=1,nbo
	      IF(ABS(ABS(XM(L,K))-ABS(XM(NM,K))).GT.0.001) GO TO 250
  240         CONTINUE
	    IC=ICSAVE
	    DO 245 K=1,nbo
  245         XM(NM,K)=0.
	    NM=NM-1
	    GO TO 50
  250       CONTINUE
	  ENDIF
      V(NM)=0.
      RETURN
c
c              error type 1 from OPEN
c                         2 from CLOSE
c                         3 from CLOSE
c                         4
c                         5 from PULL
c
  900 CONTINUE
      IF(NDEBUG.EQ.0.and.nwrite.ne.0) THEN
	IF(IER.EQ.3) THEN
	  IF(IFLAG.EQ.0) WRITE (nwrite,9002) I
 9002     FORMAT(' *ERROR*LOOP*9002* UNABLE TO FIND LOOP STARTING ',
     +      'AT BOND ',I3)
	ELSE
	  WRITE(nwrite,9001) IER
 9001     FORMAT(' *ERROR*LOOP*9001* ERROR TYPE ',I3,' IN THE',
     +      ' ATTEMPT TO FIND A LINK')
	  ENDIF
      ELSE
	IF(IER.EQ.3) THEN
	  WRITE(NDEBUG,9002) I
	ELSE
	  WRITE(NDEBUG,9001) IER
	  ENDIF
	WRITE(NDEBUG,1120) NM,I,ISTART,K1,IEND
	WRITE(NDEBUG,1121) (IOP(L),L=1,IO2)
	WRITE(NDEBUG,1122) (ICL(L),L=1,IC),K
	ENDIF
      NM=NM-1
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE STORE2(K1)
C                ------
C
C    WRITTEN 85.1.15 BY I.D.BROWN
C
C    STORES ONE BOND OF A LOOP IN MATRIX XM
C
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500)
C
      COMMON /DLS/   NFBON(2,600)
C
      K=IABS(K1)
      XM(NM,K)=NFBON(1,K)*(K1/K)
      NFBON(2,K)=1
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE OPEN(I,IER)
C                -----
C
C   WRITTEN 85.1.15 BY I.D.BROWN
C
C   WRITES BOND INTO OPEN FILE IOP
C   IO2 IS THE ADDRESS OF THE LAST ITEM ADDED TO IOP
C   IO1 IS THE ADDRESS OF THE LAST ITEM REMOVED FROM IOP
C
c   programmer's note:  /loop/ changed to /loopp/ under
c                       Unix system
      COMMON /LOOPP/ ML,IO1,IO2,IC,IOP(1000)
C
      IER=0
      IO2=IO2+1
      IF(IO2.GT.ML) THEN
	IER=1
	RETURN
	ENDIF
      IOP(IO2)=I
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE CLOSE(I,IER)
C                -----
C
C   WRITTEN 85.1.15 BY I.D.BROWN
C
C   COPIES THE TOP BOND FROM THE OPEN FILE IOP TO THE CLOSE FILE ICL
C   AND RETURNS THE BOND NUMBER AS I
C   IC IS THE LAST ADDRESS WRITTEN IN ICL
C
C programmer's note:  /loop/ changed to /loopp/ under
c                     Unix system
      COMMON /LOOPP/ ML,IO1,IO2,IC,IOP(1000),ICL(1000)
C
      IER=0
      IC=IC+1
      IF( IC.GT.ML) THEN
	IER=2
	RETURN
	ENDIF
      IO1=IO1+1
      IF(IO1.GT.IO2) THEN
	IER=3
	RETURN
	ENDIF
      ICL(IC)=IOP(IO1)
      I=IOP(IO1)
      RETURN
      END
C
C
***********************************************************************C
C
      FUNCTION NODE1(I)
C              -----
C
C   WRITTEN 85.1.15 BY I.D.BROWN
C
C   THIS FUNCTION RETURNS THE ATOM NUMBER FROM EITHER THE FIRST
C   OR SECOND NODE OF BOND(I) WHERE THE LOOP MOVES FROM NODE1 TO
C   NODE2
C
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      NODE1=0
      IF(I.GT.0) THEN
	NODE1=IBOND(1,I)
      ELSE IF (I.LT.0) THEN
	J=IABS(I)
	NODE1=IBOND(2,J)
	ENDIF
      RETURN
      END
C
C
***********************************************************************C
C
      FUNCTION NODE2(I)
C              -----
C
C     WRITTEN 85.1.18 BY I.D.BROWN
C
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      NODE2=0
      IF(I.GT.0) THEN
	NODE2=IBOND(2,I)
      ELSE IF(I.LT.0) THEN
	J=IABS(I)
	NODE2=IBOND(1,J)
	ENDIF
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE PULL(IL,J,IER)
C                ----
C
C    WRITTEN 85.1.15 BY I.D.BROWN
C
C    CALLS NODE2
C
C    THIS ROUTINE PULLS FROM THE CLOSE FILE (ICL) IN REVERSE ORDER
C    THE BOND J THAT HAS IL AS ITS SECOND NODE
C
      COMMON /FILES/ NIN,NOUT,NDEBUG
c programmer's note:  /loop/ changed to /loopp/ under
c                     Unix system
      COMMON /LOOPP/ ML,IO1,IO2,IC,IOP(1000),ICL(1000)
C
      IER=0
  100   IF(IC.LE.0) THEN
	IER=5
	RETURN
	ENDIF
      J=ICL(IC)
*     IF(NDEBUG.NE.0) WRITE(NDEBUG,1000) IC,J,NODE2(J),IL
 1000 FORMAT(' *DEBUG*PULL*1000* IC,J,NODE2(J),IL=',4I4)
      IC=IC-1
      IF(NODE2(J).NE.IL) GO TO 100
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE VALPOT
C                ------
C
C   WRITTEN 85.1.22 BY I.D.BROWN
C
C   CALLS CLOSE, NODE2, OPEN, SET
C
C   CALCULATES THE VALENCE POTENTIALS OF ATOMS GIVEN THEIR BOND VALENCES
C
      COMMON /FILES/ NIN,NOUT,NDEBUG
C programmer's note:  /loop/ changed to /loopp/ under
c                     Unix system
      COMMON /LOOPP/  ML,IO1,IO2,IC,iop(1000)
c      the dimensions of val are 'mya'
      COMMON /ATOMS/ VAL(600), rij(600)
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      IO1=0
      IO2=0
      IC=0
      DO 50 I=1,NYA
   50   VAL(I)=200.
C
C   SET FIRST ATOM AND BOND
C
      VAL(1)=0.
      DO 220 K=1,nbo
	IF(IBOND(1,K).EQ.1) THEN
	  ibo=ibond(2,k)
	  if(val(ibo).gt.100) then
	    CALL OPEN(K,IER)
	    CALL SET(K)
C
C   SEARCH ALL PATHS THAT LEAD TO ATOMS WITH UNASSSIGNED valence
C
  190       CALL CLOSE(J,IER)
	    if(ndebug.ne.0)write(ndebug,1000)io1,io2,(iop(i),i=1,io2)
 1000       format(' *debug*valpot*1000* ',25I4)
	    IF(IER.EQ.3) GO TO 220
	    IF(IER.NE.0) GO TO 900
	    DO 200 I=1,nbo
	      IF(I.NE.IABS(J)) THEN
		IF(NODE2(J).EQ.IBOND(1,I)) THEN
		  IBO=IBOND(2,I)
		  IF(VAL(IBO).GT.100.) THEN
		    CALL SET(I)
		    CALL OPEN(I,IER)
		    IF(IER.NE.0) GO TO 900
		    ENDIF
		ELSE IF(NODE2(J).EQ.IBOND(2,I)) THEN
		  IBO=IBOND(1,I)
		  IF(VAL(IBO).GT.100.) THEN
		    I1=-I
		    CALL SET(I1)
		    CALL OPEN(I1,IER)
		    ENDIF
		  ENDIF
		ENDIF
  200         CONTINUE
	    go to 190
	    endif
	  ENDIF
  220   CONTINUE
C
C    SHIFT ORIGIN OF VALENCE POTENTIAL TO MEAN VALUE
C
  300 SUM=0.
      DO 350 I=1,NYA
  350   SUM=SUM+VAL(I)
      SHIFT=SUM/nya
      DO 400 I=1,NYA
  400   VAL(I)=VAL(I)-SHIFT
      RETURN
  900 WRITE(NOUT,9001) IER
 9001 FORMAT(' *ERROR*VALPOT*9001* ERROR IN SUBROUTINE =',I3)
      return
      END
C
C
***********************************************************************
C
C
      SUBROUTINE SET(I)
C                -----
C
C   WRITTEN 85.1.21 BY I D BROWN
C
C   CALLS NODE1, NODE2
C
C     SETS THE VALENCE POTENTIAL OF THE ATOM REACHED THROUGH BOND I
C
      COMMON /ATOMS/ VAL(600),rij(600)
      COMMON /DLS/   NFBON(2,600)
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
C
      K=IABS(I)
      NOD1=NODE1(I)
      NOD2=NODE2(I)
      VAL(NOD2)=VAL(NOD1)-xbond(3,K)*NFBON(1,K)*(I/K)/rij(k)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE WRITEV(NT)
C                ------
C
C    WRITTEN 85.1.15 BY I.D.BROWN
C    REWRITTEN BY D. ALTERMATT
C
C    WRITES CALCULATED BOND VALENCES
C
C    NT = FILE NUMBER TO WRITE ON
c
c    CALLS MATHLB:  WAIT, SQART
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON /FILES/ NIN,NOUT,NDEBUG
      include 'comatom.for'
      COMMON /ATOMS/ VAL(600),rij(600)
      common /span/nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2             netind(20),ifbond(4,600)
c       the dimensions of vl are 'mx'
      dimension vl(1000)
      dimension gfix(2)
C
      gfix(1) = ' '
      gfix(2) = 'F'
      NTOUT=NT
      IF(NT.EQ.0)NTOUT=NOUT
      WRITE(NTOUT,1001)
 1001 FORMAT(//,'     BOND VALENCES',//,
     +  9X,'Atom(1)      Atom(2)         V calc  V obs   D calc  ',
     +   'D obs   Diff',/)
      LINC=0
      dsum=0.
      vsum=0.
      DO 100 i=1,nbo
	LINC=LINC+1
	iff = ifbond(4,i) + 1
	IA=iyatom(1,IBONd(1,I))
	IB=iyatom(1,iBONd(2,I))
	dbs=dbost(gelem(ia),gelem(ib),oxid(ia),oxid(ib),xbond(3,i))
	call bost(ia,ib,valobs,valerr,xbond(2,i),xbond(1,i))
	diff=xbond(1,i)-dbs
	if (XBONd(3,I).lt.0) diff=0.000
	WRITE(NTOUT,1000) GELEM(IA),GDENT(IA),iyatom(2,ibond(1,i)),
     +    GELEM(IB),GDENT(IB),iyatom(2,ibond(2,i)),gfix(iff),
     +    XBONd(3,I),valobs,dbs,xbond(1,i),diff
 1000   FORMAT(8X,2(A2,A6,I4,1X),A1,5F8.3)
	dsum=dsum + (xbond(1,i)-dbs)**2
	vsum=vsum + (abs(xbond(3,i))-abs(valobs))**2
	IF(NT.EQ.6.AND.(LINC.GE.17)) THEN
	  LINC=0
	  call wait
   10     IF(I.NE.nbo) WRITE(NTOUT,1001)
	  ENDIF
  100   CONTINUE
      if(nbo.ne.0) write(ntout,1002) sqart(vsum/nbo),sqart(dsum/nbo)
 1002 format('  RMS valence difference =',F5.2,', RMS bond length ',
     + 'difference =',F5.2)
      if(nt.eq.6) call wait
      WRITE(NTOUT,2003)
 2001 FORMAT(//,'     ATOM     VALENCE POTENTIAL (ASYMMETRIC UNIT)',/)
 2003 FORMAT(//,'     ATOM   IDENT   VALENCE POTENTIAL',/)
      LINC=0
      DO 200 I=1,NYA
	II=IYATOM(1,I)
	LINC=LINC+1
	WRITE(NTOUT,2002) GELEM(II),GDENT(II),IYATOM(2,I),VAL(I)
 2002   FORMAT(5X,A2,A6,1X,I2,6X,F6.2)
	vl(ii)=val(i)
	IF(NT.EQ.6.AND.LINC.GE.20) THEN
	  LINC=0
	  call wait
	  ENDIF
  200   CONTINUE
      if(nt.eq.6) call wait
      write(ntout,2001)
      linc=0
      do 250 i=1,nx
	linc=linc+1
	write(ntout,2500) gelem(i),gdent(i),vl(i)
 2500   format(5X,A2,A6,1X,F6.2)
	if(nt.eq.6.and.linc.ge.20) then
	  linc=0
	  if(nt.eq.6) call wait
	  endif
  250   continue
      RETURN
      END
c
C
***********************************************************************
C
C
	SUBROUTINE NETHELP
C
c
c       written by Lucien Jamieson - April 6, 1989
c
c       Used as a help screen for Netlib.
c
       common /files/nin,nout
110    write(nout,1000)
1000   format(/,'                 Network Calculation '/,
     +5X,'            ------------------- '//,
     +5X,'   This module treats the structure as a network of '/,
     +5X,'chemical bond linking the atoms.  It predicts bond '/,
     +5X,'lengths using network '/,
     +5X,'equations viz:')
       write(nout,1003)
1003   format(' ',
     +/, 
     +5X,'          Sum[S(i,j)] = oxidation state (i)'/,
     +5X,'           j'/,
     +5X,' '/,
     +5X,'          Sum[S(i,j)*R(i,j)] = 0'/,
     +5X,'           loop'/,
     +5X,'   where R(i,j) is a weight (=1 by default'/)
       write(nout,1005)
1005   format(' ',
     +5X,'   These are solved for the valences (S) of the bonds '/,
     +5X,'between atoms i and j.  Bond distances are calculated '/,
     +5X,'from these bond valences. '////)
       call pageflip(*200,*110,*9999)
 200   write(nout,1010)
1010   format(' '//,
     +5X,'   Two techniques are available, iterative'/,
     +5X,'solution described in Acta. Cryst. A33, 1305 (1977) '/,
     +5X,'and an analytical technique proposed by Rutherford '/,
     +5X,'(Acta. Cryst. (1990) B46, 289). ')
       write(nout,1015)
1015   format(' ',
     +4X,'The input to the routine is a bond network.  This can be '/,
     +5X,'expanded from the asymmetric bond set to fill one unit '/,
     +5X,'cell if the network is infinitly connected.   Observed '/,
     +5X,'bond lengths are not necessary but if given they are '/,
     +5X,'compared with those predicted.  The output consists of '/,
     +5X,'predicted bond valences and bond lengths which can be ')
       write(nout,1020)
1020   format(' ',
     +4X,'stored for further use.   The program also calculates the '/,
     +5X,'"valence potential" of each atom.   This is a valence'/,
     +5X,'analogue of the electrical potential in electrical '/,
     +5X,'networks, and is probably related to the electronegativity.'/,
     +5X,' '//////)
       call pageflip(*9999,*110,*9999)
9999    return
	end
