C
C      FILE PLOTLB                              Last update: ? April 1993
C      ===========                                             T. Stephen
***********************************************************
C  Machine depended code indicated by 'machine dependent' *
***********************************************************
C
C    This file contains the following routines:
C
C    a) General routines
C
C  S PLOTMN  : MAIN DRIVER for all calculations and plotting
C  S ABSOL   : Creates virtual screen in atomic position array ICO
C  S ATOMCAL : Input multiples of unit cell dimensions to fill YATOM c.block
C  S BONDCAL : Identifies atom pairs to which bonds are attached
C  S CALCPT  : Calculates exit coordinates for bonds out of atoms shell
C  S CBASE2  : Automatic/Manual color selection
C  S DISPLAY : Shows next help section on screen
C  S EDGPLO  : Store cell edges/lattice vectors in IBO-array
C  S FILTER  : Driver to FILTER1
C  S FILTER1 : Drops atomic positions outside plotting ranges
C  S FILTER3 : Filters bond array according to atoms left after FILTER1
C  S GLU     : Graphics Language Utility for interactive flag settings
C  S LOCATE  : Finds a specified section in the helpfile
C  F NEWBOND : Checks if present bondpair already stored
C  S ORTHO   : Orthoganalizes YX array
C  S ORTHOC  : Orthoganalizes projection volume and determines the scale
C  S PERSP   : Applies perspective to the atom coordinates
C  S PLOT1   : Driver for plotting
C  S PLOTBD  : Draws all bonds from present atom to lower (z-)level atoms
C  S PLOTER  : Sets up filters, perspective and calls PLOT1
C  S RADII   : Gets radii of all atoms in /IDENT/
C  S ROTATE  : Calculates the rotation matrix
C  S STEROT  : Rotates the image for stereo viewing
C  S STERSFT : Shifts the image after rotation for stereo viewing
C  S SPATRE  : Loads the spanning tree to YX and IBO arrays
C  S STIPLO  : Calculate and draw a Stick Bond
c  S PLOHELP : Gives a clue on getting started
C
C    b) Plotting on LUNDY/SIGMEX and VGA terminals
C
C  S ciratom  : Draws an atom on the LUNDY/SIGMEX terminal
C  S bondline    : Draws line from an initial to a final coordinate (LUNDY/SIGMEX)
C  S TINT    : Colours in atoms to give a 3-D effect
c  s cscale  : Draws a scale showing what bond colours represent
c  s coldis  : Shows the available colours
C
C    c) Plotting on TEKTRONIX 4010/PERICOM 7000 series terminals
C
C  S TEKON   : Set terminal to Tektronix mode
C  S TEKOFF  : Set terminal to VT100 mode
C  S GMODE   : Switch to graphics for Tektronix/Pericom
C  S AMODE   : Switch to alpha mode for Tektronix/Pericom
C  S PMODE   : Switch to special point mode for Tektronix/Pericom
C  S TMODE   : Switch to text mode for Tektronix/Pericom
C  S CLRSCR  : Clear graphic screen of Tektronix/Pericom
C  S LOHOME  : Move cursor to dawn left home position for Tektronix/Pericom
C  S HIHOME  : Move cursor to up left home position for Tektronix/Pericom
C  S ATOPLO  : Plots an atom to the virtual screen
C  S VECPLO  : Plots a vector (bond) to the virtual screen
C  S SCRPLO  : Plots the virtual screen to the Tektronix/Pericom screen
C  S LABPLO  : Plots the element names to Tektronics/Pericom
C  S LABSCA  : Labels the scale on Tektronix/Pericom
C  S SCLPLO  : Plots the scale and cell edges on Tektronix/Pericom
C*************
C  S OUTCH   : Routine to be envoked when used with machine other
C              than Norsk Data 500
c  S FLUSH   : Empties buffer to terminal at end of plot
C  F ISHFT   : Function to envoked when used with Cyber, only!
C************:
C  S ENDFIL  : Dummy routine to allow existance of above comment deck
C
C-----------------------------------------------------------------------
C
C
C      Externals called by this file:
C**************************
C      ND-500 Monitor call:  OUTCH   (see subroutine list above)
C**************************
C      CHEMLB:  bost, radius
C      DISTLB:  chebon, checon, cobotr, indepb, xleng
C      IOLIB:   iclosu, ifile
C      MAINLB:  calcy, inerr
C      MATHLB:  integer, lowcase, mxm, mxv, pageflip, readyn, scan,
c               sortx, sqart, upcase, vminv, wait
c      W77PLOT or DUDPLOT: charplo, circle, glint, line, plottest,
c                          shade, video, box, background
C      SYMLIB:  getstl, mltstl, siteop, symopl, symtrn
C
C
C
c----------------------------------------------------------------------
c
c  COMPATABILITY PROBLEMS?
c
c
c  If you do not have the Watcom Fortran 77 graphics libraries, you
c  should include the object file "DUDPLOT.OBJ" compiled from
c  "DUDPLOT.FOR" in place of "W77PLOT.OBJ".  Include only ONE of these 
c  files.
c 
c
c  Once you have done this, it is advisable not to select the option
c  on the menu (3,17) which will try to run these routines.
c  If you do, there is a chance Strumo will crash, depending on
c  how the program has been doctored.
C
c
c-----------------------------------------------------------------------
c
c  While adapting this program to the VGA monitors, the routines for
c  Lundy/Sigmex may have been slightly damaged.  It has been left so
c  that it would be possible to reconstruct them, but as I do not have
c  that monitor I can't test it anyway.  If someone attempts this,
c  the hardcopy from an older "PLOTLB.FOR" might make it easier.
c
c  Future modifier of the program should note that this was done 
c  so that the VGA code could follow the same path ie. nterml=1 for
c  both the VGA and Lundy monitors.  ivgafl is used to distinguish
c  between these monitors, but some conditions have been messed with.
c
c  Tektronix/Pericom monitors may or may not still work.  If there are
c  problems, they should be relatively minor, and easy to correct.
c
c
C-----------------------------------------------------------------------
      SUBROUTINE PLOTMN
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza   Summer 1985
C  Revised by : Daniel Altermatt, University of Lausanne, Switzerland
C               Winter 1985/86
c         and : Tamon Stephen, Spring 1993
C
C  Purpose    : PLOTMN is the main driver for the Graphics Package for
C               Inorganic Crystalline Structures. PLOTMN the same common
C               block structure as the modelling package STRUMO.
C
C               The graphics package is designed for use with:
C               nterml = 1: the Lundy/Sigmex graphics terminal
C                      = 2: the Tektronix 4010/Pericom 7800
C                           graphics terminal series
c               nterml = 1:
c                 ivgafl = 1: VGA 640X480, 16 colour graphics 
c                 ivgafl = 2: SVGA 640X480, 256 colour graphics  
c                 ivgafl = 3: SVGA 1024X768, 256 colour graphics
C
C  Calls      : absol, atomcal, bondcal, cbase2, clrscr, filter,
C               filter3, glu, ortho, orthoc, persp, plot1, radii,
C               sortx, tmode, wait, plohelp, plottest
C
C  *** Note:  This code has not been modified for the Unix system.
C             Use "dplotf.f" to compile the plotlb functions as
C             dummy variables.    -SH. 09/27/90
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
C  System common blocks
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,machine
      COMMON /CELL/   A(3),AL(3),VOL,AM(3,3),ICELL
      COMMON /SYM/    NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON /GSYM/   GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /oatom/  nyo,myo,oyx(3,1000)
      common /span/   nya,mya,iyatom(5,600),iterm(600),nbo,mbo,
     1                ibond(2,600),xbond(6,600),iat,bondlim(2)
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /rottr/  rotm(3,3),nrot,nrro
      common /gplot/  gcol(16)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,
     1                n177,n45,n60,n62
c   These represent   Ff  Can Esc Fs  Gs  Us  Sp  !   7   @    \   
c                     Del  %   0   2
C
C  Explanations to common blocks of plot package:
C
C  common /flag/:
C    ijoyfl = 0/1   for joystick (off/on) (joystick mode not yet implemented)
C    iscafl = 0/1   for (do not/do) plot the scale on the screen
C    istofl = 0/1   for (do not/do) return to calling program after next plot
C    ifilfl = 0-3   for (default/full range/unit cell/user selected) volume
C                   to be filtered through for plotting
C    istifl = 0/1   for (normal/stickbond framework) plot
C    icolfl = 0/1   for (automatic/manual) colour selection
C    ibonfl = 0/1   for (do not/do) draw bonds
C    istrfl = 0/1   for strobe mode (off/on) (effects LUNDY/SIGMEX only)
C    ispafl = 0/1   for (do not/do) pick information from common /span/
C                   to plot spanning tree
C    iaxifl = 0-2   for (do not/as three basic vectors/as full unit cell
C                   edges) plot the unit cell axis
C    ilabfl = 0/1   for (do not/do) label the plotted atoms by element name
C    ishifl = 0/1   for (if not/if) lattice vectors are shifted onto screen
C    iautfl = 0/1   for (do not/do) automatically adjust scale to plotting
C                   ranges as selected by user
C    istefl = 0-2   for (do not/first plot/second plot) creating a stereo 
C                   molecule
C    iquifl = 0/1   for (do not/do) draw only a quick bond picture
c    itinfl = 0/1/2 for (do not/do/back) draw shaded bonds (monochrome only)
c    ivgafl = 0-3   VGA monitor & type
c    ibclfl = 1-15  bond colour
c    iprifl = 0/1   do/do not have printer options
c    ibacfl = 0-255 background colour
C
C  common /plot/:
C    nrc         number of atoms in ICO-array, = NY
C    nrb         number of bonds in IBO-array
C    mrb         maximal number of bonds, = 1600
C    ico(m,n)    atom array, m=1...6, n=1...1000
C       m = 1    x-screen coordinate
C         = 2    y-screen coordinate
C         = 3    z-screen coordinate
C         = 4    final atomic radius
C         = 5    element reference to /IDENT/
C         = 6    (0/1) for (do not/do) plot this atom
C    rad(n)      elementary radii for atoms stored in /IDENT/, n=1...NX
C    radmul(n)   multipliers to RAD(n) for plots
C    ibo(m,n)    bond array, m=1...7, n=1...1600
C       m = 1    reference to ICO-array for origin atom
C         = 2    reference to ICO-array for terminal atom
C         = 3    x-screen coordinate of starting point
C         = 4    y-screen coordinate of starting point
C         = 5    x-screen coordinate of ending point
C         = 6    y-screen coordinate of ending point
C         = 7    bond type flag (only implemented for Tektronix/Pericom):
C                0 ... do not draw
C                1 ... solid line
C                2 ... dotted line
C                3 ... dot-dashed line
C                4 ... short dashed line
C                5 ... long dashed line
C    xbo(n)      bond valence or (if negativ) bond length, n=1...1600
C    xblim(2)    xbo-window to be plotted
C
C  common /attrib/:
C    nter        terminal unit, output stream for terminal commands
C    pixwid      screen width in pixels
C    ascale      scale to be used for next plot (pixels/Angstroms)
C    dscale      default scale for resets
C    viewd       view distance for zooming (Angstroms)
C    window(3,4) startup limits to default plotting ranges
C          (1,1) minimum x
C          (1,2) maximum x
C          (1,3) orthogonalized (1,1)
C          (1,4) orthogonalized (1,2)
C          (2,1) minimum y
C          ...
C    orgsft(3)   orthogonalized position of the centre of the plotted volume
C                screen coordinates are calculated as YX=(X-ORGSFT)*ASCALE
C    icol(n)     reference to colour code in GCOL for atoms stored in /IDENT/
C    bbten(3,3)  orthogonalization tensor
C    prange(3,4) user selected plotting ranges
C          (1,1) minimum x  (as fractions of the unit cell)
C          (1,2) maximum x
C          (1,3) minimum x-coordinate on screen
C          (1,4) maximum x-coordinate on screen
C           ...
C
C  common /gplot/:
C    gcol(n)     colour codes for LUNDY/SIGMEX, n=1...16
C
C  common /rottr/:
C    rotm(3,3)   transformation matrix for specified rotation
C    nrot        present step number for step rotation
C    nrro        total step number for step rotation
C
C  common /screen/:
C    limits(4)   left-right, up-down maxima of ISCR
C    iscr(n,n)   (0/1) for (do not/do) plot a point on the
C                Tektronix/Pericom screen at this position
C                n=1...1000, virtual screen
C    cellax(3,n) n=1...8: orthogonalized corners of unit cell
C                (0,0,0; 1,0,0; 0,1,0; ...)
C                n=9...16: screen coordinates of unit cell corners (real)
C
c      dimension y(3),ixz(1000),ixs(1000) (not used)
      dimension angl(3)
      double precision pi / 3.1415926535898 /, convrt
C
C  Initialization
c  --------------
C
c  set values of characters
c  ------------------------
C
c*******************************************************
c These next few lines are machine dependent           *
c*******************************************************
c
      ivgafl=0
      i=plottest(1)
4001  write(nout,4000)
4000  format(/,'STRUMO Plotting Package',///,
     1 '1:Help',//,
     2 '  the following terminals are known to the system:',//,
     3 '2:LUNDY/SIGMEX',/,'3:TEKTRONIX/PERICOM',/,
     4 '4:VGA 640X480, 16 Colour',/,
     5 '5:SVGA 640X480, 256 Colour',/,
     6 '6:SVGA 1024X768, 256 Colour',///////,
     7 '20:Return to Menu Level 3',///,
     8 'Enter your choice: ')
      read(nin,4010,err=31) nterml
4010  format(i5)
      if (nterml.eq.20) goto 4099
      if (nterml.lt.1.or.nterml.gt.6) goto 4001
      if (nterml.eq.1) then
	call plohelp
	goto 4001
      endif
      if (nterml.ge.4) then
	i=plottest(0)
	if (i.eq.0) then
	  i=plottest(1)
	  write(nout,4777)
4777      format('In order to use VGA and SVGA modes, you must',/,
     +     'link this program with W77PLOT.FOR rather than',/,
     +     'DUDPLOT.FOR, and with the W77 graphics libraries in',/,
     +     'your include path!')
	  call wait
	  goto 4001
	endif
	ivgafl=nterml-3
	nterml=1
      else
	nterml=nterml-1
      endif
      if(machine.eq.1) then
	n14='14'o
	n30='30'o
	n33='33'o
	n34='34'o
	n35='35'o
	n37='37'o
	n40='40'o
	n41='41'o
	n67='67'o
	n100='100'o
	n140='140'o
	n177='177'o
	n45='45'o
	n60='60'o
	n62='62'o
      else if(machine.eq.4.or.macine.eq.5) then
*        n14=14b
*        n30=30b
*        n33=33b
*        n34=34b
*        n35=35b
*        n37=37b
*        n40=40b
*        n41=41b
*        n67=67b
*        n100=100b
*        n140=140b
*        n177=177b
*        n45=45b
*        n60=60b
*        n62=62b
      else if(machine.eq.3) then
*        n14=o"14"
*        n30=o"30"
*        n33=o"33"
*        n34=o"34"
*        n35=o"35"
*        n37=o"37"
*        n40=o"40"
*        n41=o"41"
*        n67=o"67"
*        n100=o"100"
*        n140=o"140"
*        n177=o"177"
*        n45=o"45"
*        n60=o"60"
*        n62=o"62"
	endif
C
C  Define unit number of terminal stream for command output
c  --------------------------------------------------------
C
      nter=nout
c     if (ivga.eq.0) nter=nin
c  nter=nin was in the original (pre-VGA) plotlb.  I find it a bit
c   surprising, as it is used for output statements, and leave it
c   as a comment, in the case that it is a way of channeling escape
c   sequences.  If someone is trying to fix this for the other
c   monitors, they might try using this(?).  If reinstated, it will 
c   cause crahes if compiled by Watcom Fortran 77, and the Lundy
c   routines (and perhaps Tektronix-don't know) are called.
c
C  Screen dimension in pixels
      if (ivgafl.ge.1.and.ivgafl.le.2) then
	pixwid = 480.
      else
	pixwid=960.
      endif
C  View distance in Angstroms
      viewd = 75.
C  Z-range for filtering
      izma = pixwid
      izmi = - pixwid
C  Screen limits to be de-/incremented
      limits(1)=1000
      limits(2)=0
      limits(3)=1000
      limits(4)=0
C  Default flag settings
      ijoyfl = 0
      iscafl = 0
      istofl = 0
      ibonfl = 1
      istifl = 0
      icolfl = 0
      ifilfl = 1
      istrfl = 0
      ispafl = 0
      iaxifl = 1
      ilabfl = 1
      ishifl = 0
      ibacfl = 0
      iautfl = 1
      istefl = 0
      iquifl = 0
      itinfl = 0
      ibclfl = 10
      iprifl = 0
C  Default rotation flags
      nrot = 0
      nrro = 0
C  Default multipliers
      DO 5 I=1,1000
5       RADMUL(I)=1.0
C  Don't trust previous programs -- reset bond/contact checks
      ind1=0
      ind2=0
C
C  Display initial information
c  ---------------------------
C
      write(nout,2000)
2000  format(//,' Graphics Package V-ND 3.0,  1985/86',//)
      if(nterml.eq.1) then
	write(nout,2010)
2010    format(' for PC with VGA',////)
      else if (nterml.eq.2) then
	write(nout,2020)
2020    format(' for Tektronix 4010/Pericom 7800 terminals',////)
	endif
      write(nout,2030)
2030  format(' General Information',/,' -------------------',
     1 //,'       CELL DIMENSIONS    ANGLES',
     2 /,'   I         A/B/C        A/B/G',
     3 /,'  ---       -------      -------- ',/)
      DO 10 I=1,3
	CONVRT = 180.00/ PI
	ANGL(I) = AL(I)*CONVRT
10      WRITE(nout,2040) I,A(I),ANGL(I)
2040    FORMAT(2X,I3,2(3X,F11.5))
      write(nout,2050) nx,nbon,nbo,ncon
2050  format(//,1X,I4,' independent atomic positions and',/,
     1          1X,I4,' independent bonds and',/,
     2          1X,I4,' bonds in spanning tree and',/,
     3          1X,I4,' independent contacts stored at present.',/)
C
C  Test on key information
c  -----------------------
C
      if(icell.ne.1) then
	hcom='(//,'' Lattice constants missing -- Sorry, no way'',/)'
      else if(nx.le.0) then
	hcom='(//,'' No atomic positions given -- Sorry, no way'',/)'
      else if(nbon.le.0.and.nbo.le.0.and.ncon.le.0) then
	hcom='(//,'' ***** No bonds/contacts given at all *****'',/)'
	goto 30
      else if(nbo.le.0) then
	goto 30
      else
12      write (nout,2100)
2100    format(//,' Shall we do a normal Structure Plot (1)',/,
     1          /,'$            or a Spanning Tree Plot (2): ')
	read(nin,1010,err=25) ispafl
1010    format(I5)
25      if(ispafl.lt.1.or.ispafl.gt.2) call inerr(*12)
	ispafl=ispafl-1
	if(ispafl.eq.1) then
	  call spatre(hcom,*17)
	  goto 35
	  endif
	goto 30
	endif
17    write(nout,hcom)
      call wait
      return
C
C  Empty space volume, try again
c  -----------------------------
C
20    write(nout,2060)
2060  format(//,'  +++ERROR+++',/,
     1 '  There is nothing to plot within those bounds!',
     2 /,'  Re-enter limits:',/)
C
C  Calculate the atomic coordinates, fill YX array (contains XL coordinates)
c  -------------------------------------------------------------------------
C
30    CALL ATOMCAL
35    WRITE (nout,2070) NY
2070  FORMAT(/,' There are ',I3,' atoms to be plotted',/)
      IF(NY.EQ.0) GOTO 20
C
c  Create colour table from colourbase (all terminals)
c  ---------------------------------------------------
C
      HLINE=' '
      IP=1
      CALL CBASE2 (HLINE,IP,nterml)
C
C  Find the ionic radii for the atoms stored in /IDENT/
c  ----------------------------------------------------
C
      CALL RADII (DRMAX)
      rmax=drmax
C
C  Expand bond set --- identify atoms to be joined.
c  ------------------------------------------------
C
      if(ispafl.eq.0) CALL BONDCAL
C
C  Orthogonalize coordinates, unit cell and plotting ranges
c  --------------------------------------------------------
C
      CALL ORTHO (BBTEN,rotm)
      do 45 i=1,3
	window(i,3)=window(i,1)
45      window(i,4)=window(i,2)
      CALL ORTHOC (window(1,4),window(1,3),rmax,iautfl)
      dscale=ascale
C
C  Default: Plot full window ranges.
C  -------  Shift origin to the centre of the orthogonalized plotting
C           volume.
C  ------------------------------------------------------------------
c
      DO 40 I=1,3
	prange(i,1)=window(i,1)
	prange(i,2)=window(i,2)
	orgsft(i)=(window(i,4)-window(i,3))/2 + window(i,3)
40      CONTINUE

C     Default plotting run
c
      call plotr(RMAX, nterml, izma, izmi)
C
C  Invoke Graphics Language Utility  (GLU)
C  ---------------------------------------
c
50    CALL GLU (nterml, rmax, izma, izmi)
      istofl=0
      RETURN
31    write(nout,4002)
4002  format('Invalid input.')
      call wait
4099  return
      END
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ABSOL (rmax)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : Create virtual screen in ICO-, CELLAX- and PRANGE-array
C               Adjust scaling if IAUTFL = 1
C
C  Calls      : orthoc,vminv,mxv
C
C-----------------------------------------------------------------------
C
      include 'comatom.for'
      common /oatom/  nyo,myo,oyx(3,1000)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      common /rottr/  rotm(3,3),nrot,nrro
C
      DIMENSION xyz1(3),xyz2(3)
C
C  Calculate screen coordinates for plotting ranges and update scale
C  -----------------------------------------------------------------
c
      do 60 i=1,3
	prange(i,3)=prange(i,1)
60      prange(i,4)=prange(i,2)
      nautfl=0
      call orthoc(prange(1,4),prange(1,3),rmax,nautfl)
      do 70 i=3,4
	call vminv (xyz1,prange(1,i),orgsft)
	call mxv (xyz2,rotm,xyz1)
	do 80 j=1,3
80        prange(j,i)=xyz2(j)
70      continue
      do 90 i=1,3
	if(prange(i,4).lt.prange(i,3)) then
	  xyz1(i)=prange(i,3)
	  prange(i,3)=prange(i,4)
	  prange(i,4)=xyz1(i)
	  endif
90      continue
      if(iautfl.eq.1) then
	ascale=(pixwid-50.)/(prange(1,4)-prange(1,3)+2.*rmax)
	asc1=(.75*pixwid-50.)/(prange(2,4)-prange(2,3)+2.*rmax)
	if(ascale.gt.asc1) ascale=asc1
	endif
      do 100 i=1,3
	prange(i,3)=prange(i,3)*ascale
100     prange(i,4)=prange(i,4)*ascale
C
C  Calculate initial screen coordinates for atomic positions
C  ---------------------------------------------------------
c
      nrc=nyo
      DO 20 i=1,nyo
	  call vminv(xyz1,oyx(1,i),orgsft)
	  call mxv(xyz2,rotm,xyz1)
	DO 30 j=1,3
	    ico(j,i)=int(xyz2(j)*ASCALE)
30        continue
	  ico(4,i)=int(rad(iynum(i))*radmul(iynum(i))*ascale)
	ico(5,i)=iynum(i)
	ico(6,i)=1
20      continue
C
C  Calculate initial screen coordinates for unit cell corners
C  ----------------------------------------------------------
c
      do 40 i=1,8
	n=i+8
	call vminv (xyz1,cellax(1,i),orgsft)
	call mxv (xyz2,rotm,xyz1)
	do 50 j=1,3
50        cellax(j,n)=xyz2(j)*ascale
40      continue
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ATOMCAL
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : ATOMCAL reads in the subcell limits and calls CALCY to
C               load the YX array with atomic coordinates.
C
C  Calls      : CALCY
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,machine
      include 'comatom.for'
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
C
C  Read in the subcell limits
c  --------------------------
C
500   WRITE (NOUT,2000)
2000  FORMAT (/,' Enter limits for maximal subcell you will ever',
     1          ' want to plot:',/,
     2  ' (subsequent restrictions in GLU)',/,
     3  ' Give two values on each line separated by a comma',///,
     4  '$Minimum and maximum for X: ')
      if(machine.eq.3.or.machine.eq.5) then
	READ (NIN,*,ERR=500) window(1,1),window(1,2)
      else
	READ (NIN,1000,ERR=500) window(1,1),window(1,2)
	endif
1000  FORMAT(2F10.0)
510   WRITE (NOUT,2010)
2010  FORMAT('$Minimum and maximum for Y: ')
      if(machine.eq.3.or.machine.eq.5) then
	READ (NIN,*,ERR=510) window(2,1),window(2,2)
      else
	READ (NIN,1000,ERR=510) window(2,1),window(2,2)
	endif
520   WRITE (NOUT,2020)
2020  FORMAT('$Minimum and maximum for Z: ')
      if(machine.eq.3.or.machine.eq.5) then
	READ (NIN,*,ERR=520) window(3,1),window(3,2)
      else
	READ (NIN,1000,ERR=520) window(3,1),window(3,2)
	endif
C
C  load the YX array with atomic coordinates
c  -----------------------------------------
C
      CALL CALCY(window(1,2),window(1,1))
      if(ny.gt.500) then
530     write(nout,2030) ny
2030    format(//,' ***** ',I4,' atoms found in specified volume',
     1   ' -- this is enormous!!',//,
     2   '$Do you want to choose smaller limits (Y/N): ')
	call readyn(*500,*540,*540,*530)
	endif
C
540   RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE BONDCAL
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : BONDCAL blows up the bond/contact array according to
C               symmetry and stores the envolved atom pairs in IBO
C
C  Calls      : bost, CHEBON, checon, cobotr, getstl, INDEPB, MLTSTL,
C               newbond, SITEOP, SYMTRN
C
C  Note       : This follows now the same logic as subroutine PSUPLO
C               in file MODOUT
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
c
c     System common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      include 'combond.for' 
C
C     plot package common block
C
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
C
      DIMENSION NT(3,1000),MMT(3),L0(3),NNT(3),XLT(3),lcon(10)
      DATA (L0(I),I=1,3)/0,0,0/
C
      nrb=0
      mrb=1600
      if(nbon.le.0.and.ncon.le.0) return
      xblim(1)=-15.0
      xblim(2)=5.0
C
C  First do bonds (solid lines)
C  ----------------------------
c
      IF (IND1.LT.NBON) CALL INDEPB(1,(IND1+1),NBON)
      IF (IND2.LT.NCON) CALL INDEPB(2,(IND2+1),NCON)
      do 100 i=1,ny
100     call getstl(nt(1,i),iynum(i),yx(1,i),iysym(i))
      DO 160 I=1,NY
	IF (OXID(IYNUM(I)).LT.0.0) GOTO 170
	NN=0
	IF (NBON.LE.0) goto 170
	DO 140 J=1,NBON
	  CALL CHEBON (IYNUM(I),NN,NBON,IBON)
	  IF (NN.EQ.0) GOTO 170
	  IF (JBON(2,NN).LE.0.OR.INDE1(NN).LT.NN) GOTO 140
	  N=1
	  CALL SITEOP (N,ISP,MMT,IYSYM(I),NT(1,I),IALTNO(IYNUM(I)),L0)
	  CALL MLTSTL (JSYM,NNT,ISP,MMT,JBON(2,NN),JBON(3,NN),
     1                IALTNO(JBON(1,NN)))
	  N=0
135       N=N+1
	  CALL SYMTRN (N,ISP,MMT,JBON(1,NN),JSYM,NNT,IYNUM(I),
     1                IYSYM(I),NT(1,I),XLT)
	  IF (N.le.0) GOTO 140
	  DO 145 K=1,NY
	    IF (IYNUM(K).NE.JBON(1,NN)) GOTO 145
	    IF(IYSYM(K).NE.ISP) GOTO 145
	    DO 250 M=1,3
250           IF (NT(M,K).NE.MMT(M)) GOTO 145
	    nrb=nrb+1
	    ibo(1,nrb)=I
	    ibo(2,nrb)=K
	    ibo(7,nrb)=1
	    if(ifbon(4,nn).eq.2) then
	      xbo(nrb)=-xbon(1,nn)
	    else
	      call bost(ibon(1,nn),jbon(1,nn),xbo(nrb),x1,x2,xbon(1,nn))
	      if(xbo(nrb).lt.0.01) xbo(nrb)=-xbon(1,nn)
	      endif
	    if(newbond(nrb,ibo).eq.0) nrb=nrb-1
C              bond exists already
	    IF (nrb.GE.mrb) then
	      write(nout,2000) mrb
2000          format(//,' ++BONDCAL++  STOPPED AFTER ',I4,' BOND',
     1          'PAIRS FOUND',//)
	      return
	      endif
	    GOTO 135
145         CONTINUE
	  goto 135
140       CONTINUE
C
C  Now add contacts (as dotted lines)
c  ----------------------------------
C
170     IF (ncon.LE.0) goto 160
	NN=0
	DO 300 J=1,ncon
	  CALL checon (IYNUM(I),NN,ncon,icon,jcon,ipn1,ipn2)
	  IF (NN.EQ.0) goto 160
	  IF (INDE2(NN).LT.NN) goto 300
	  do 310 l=1,5
	    lcon(l)=icon(l,nn)
310         lcon(l+5)=jcon(l,nn)
	  n=-1
	  call cobotr (n,lcon(ipn1),lcon(ipn1+1),lcon(ipn1+2),
     1                lcon(ipn2),lcon(ipn2+1),lcon(ipn2+2))
	  N=1
	  CALL SITEOP (N,ISP,MMT,IYSYM(I),NT(1,I),IALTNO(IYNUM(I)),L0)
	  CALL MLTSTL (JSYM,NNT,ISP,MMT,lcon(ipn2+1),lcon(ipn2+2),
     1                IALTNO(lcon(ipn2)))
	  N=0
320       N=N+1
	  CALL SYMTRN (N,ISP,MMT,lcon(ipn2),JSYM,NNT,IYNUM(I),
     1                IYSYM(I),NT(1,I),XLT)
	  IF (N.le.0) goto 300
	  DO 330 K=1,NY
	    IF (IYNUM(K).NE.lcon(ipn2)) GOTO 330
	    IF (IYSYM(K).NE.ISP) GOTO 330
	    DO 340 M=1,3
340           IF (NT(M,K).NE.MMT(M)) GOTO 330
	    nrb=nrb+1
	    ibo(1,nrb)=I
	    ibo(2,nrb)=K
	    ibo(7,nrb)=2
	    xbo(nrb)=-xcon(1,nn)
	    if(newbond(nrb,ibo).eq.0) nrb=nrb-1
C              contact exists already
	    IF (nrb.GE.mrb) then
	      write(nout,2000) mrb
	      return
	      endif
	    GOTO 320
330         CONTINUE
300       continue
160     CONTINUE
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE CALCPT (ip,istifl)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : CALCPT determines the points at which the bond leaves the
C               origin atom and where it enters the terminal atom
C               If both atoms are at the same xy-point, IBO(7,IP) will
C               be set negative
C
C  Input      : IP     ; Present pointer position in IBO-array
C               ISTIFL ; Stickbonds flag
C
C  Calls      : --
C
C-----------------------------------------------------------------------
C
      common /plot/  nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1               ibo(7,1600),xbo(1600),xblim(2)
C
      DIMENSION IA(3),IB(3)
C
C  Get atomic coordinates
C  ----------------------
c
      do 10 i=1,3
	ia(i)=ico(i,ibo(1,ip))
10      ib(i)=ico(i,ibo(2,ip))
      irada=ico(4,ibo(1,ip))
      iradb=ico(4,ibo(2,ip))
      if(istifl.eq.1) then
	irada=8
	iradb=8
	endif
C
C  determine sphere shapes
C  -----------------------
c
      DIS=SQRT(FLOAT(IB(1)-IA(1))**2+FLOAT(IB(2)-IA(2))**2)
      IF (DIS.EQ.0) then
	ibo(7,ip)=-ibo(7,ip)
	RETURN
	endif
      ALPHA=ATAN(ABS(IB(3)-IA(3))/DIS)
      IDELTA=COS(ALPHA)*irada
      ideltb=cos(alpha)*iradb
      IF(ABS(IB(1)-IA(1)).EQ.0) THEN
	BETA=3.141592/2.
      ELSE
	BETA=ATAN(FLOAT(ABS(IB(2)-IA(2)))/FLOAT(ABS(IB(1)-IA(1))))
      ENDIF
      IDXa=IDELTA*COS(BETA)
      idxb=ideltb*cos(beta)
      IDYa=IDELTA*SIN(BETA)
      idyb=ideltb*sin(beta)
      if(ia(1).ge.ib(1)) then
	idxa=-idxa
      else
	idxb=-idxb
	endif
      if(ia(2).ge.ib(2)) then
	idya=-idya
      else
	idyb=-idyb
	endif
C
C  Store coordinates of starting and ending points
C  -----------------------------------------------
c
      ibo(3,ip)=ia(1)+idxa
      ibo(4,ip)=ia(2)+idya
      ibo(5,ip)=ib(1)+idxb
      ibo(6,ip)=ib(2)+idyb
      RETURN
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE CBASE2 (HLINE,IP,nterml)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C               N. Heinig, Aug. 1986
c  VGA'd by : Tamon Stephen, March 1993
C
C  Purpose    : CBASE2 handles automatic and manual color assignment to
C               atoms.
C
C  Calls      : scan, upcase
C
C-----------------------------------------------------------------------
c
      IMPLICIT CHARACTER *80 (H)
      IMPLICIT CHARACTER *8  (G)
C
C  System common blocks
C
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
C
C  Plot package common blocks
C
      COMMON /ATTRIB/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /gplot/  gcol(16)
C
      dimension icolta(7,18)
C
C  Fill in colour names 
C  --------------------
c
c    These are for other monitors
      if (ivgafl.eq.0) then
	GCOL(1)='white'
	GCOL(2)='black'
	GCOL(3)='red'
	GCOL(4)='blue'
	GCOL(5)='cyan'
	GCOL(6)='orange'
	GCOL(7)='yellow'
	GCOL(8)='lt.green'
	GCOL(9)='green'
	GCOL(10)='dk.green'
	GCOL(11)='dk.red'
	GCOL(12)='magenta'
	GCOL(13)='grey'
	GCOL(14)='pink'
	GCOL(15)='lt.blue'
	GCOL(16)='dk.blue'
      else if ((ivgafl.ge.2.and.itinfl.eq.0).or.ivgafl.eq.1) then
c  VGA
c  tint off - use sixteen default colours
	GCOL(16)='black'
	GCOL(1)='blue'
	GCOL(2)='green'
	GCOL(3)='cyan'
	GCOL(4)='red'
	GCOL(5)='magenta'
	GCOL(6)='brown'
	GCOL(7)='white'
	GCOL(8)='grey'
	GCOL(9)='lt.blue'
	GCOL(10)='lt.green'
	GCOL(11)='lt.cyan'
	GCOL(12)='lt.red'
	GCOL(13)='lt.magenta'
	GCOL(14)='yellow'
	GCOL(15)='br.white'
      else
c  tint on, 256 colours, using extended shades from SHADE
	gcol(1)='red'
	gcol(2)='red-or'
	gcol(3)='or-red'
	gcol(4)='orange'
	gcol(5)='ye-or'
	gcol(6)='yellow'
	gcol(7)='ye-gr'
	gcol(8)='gr-ye'
	gcol(9)='green'
	gcol(10)='gr-bl'
	gcol(11)='bl-gr'
	gcol(12)='blue'
	gcol(13)='purple'
	gcol(14)='magenta'
	gcol(15)='white'
	gcol(16)='black'
      endif
C
C  For update runs skip default part
C  ---------------------------------
c
      GA=' '
      IF (HLINE.NE.' ') GOTO 100
C
C  Fill default colour table
C  -------------------------
c
5     do 10 i=1,7
	do 15 j=1,18
	  icolta(i,j)=13
15        if((j-i).le.10) icolta(i,j)=8
10      continue
      icolta(1,1) = 1
      icolta(2,14)= 2
      icolta(2,15)= 3
      icolta(2,16)= 12
      icolta(2,17)= 5
      icolta(3,15)= 6
      icolta(3,16)= 7
      icolta(3,17)= 8
      icolta(4,17)= 14
      icolta(5,17)=10
      icolta(3,1)=11
      icolta(4,1)=4
      icolta(4,2)=15
C
C  Assign default colours to all atoms in /IDENT/
C  ----------------------------------------------
c
20    do 30 i=1,nx
	if (nza(2,i).ge.1.and.nza(2,i).le.18) then
	  icol(i)=icolta(nza(1,i),nza(2,i))
	else
	  icol(i)=8
	endif
30    continue
C
C  Set flag and exit from default run
C  ----------------------------------
c
      icolfl = 0
      return
C
C  For Tektronix/Pericom update runs do not make sense
C  ---------------------------------------------------
c
100   if(nterml.eq.2) return
C
C  Check for element names, LIST/RESET command or position number
C  on the line.
C  --------------------------------------------------------------
c
      CALL SCAN (GA,N,X,EX,IP,HLINE,IRFL)
      IF (IRFL.lt.1.or.irfl.gt.2) GOTO 140
      if (irfl.eq.2) goto 145
      CALL UPCASE(GA)
      if (ga(1:3).eq.'DIS') then
	call coldis
	return
      endif
      IF (GA(1:3).EQ.'LIS') THEN
	write(nout,2000)
2000    format(/,'   Colour name listing by colour number',//,
     1  '  nr. colour     nr. colour     nr. colour     nr. colour',/,
     2  '  --- ------     --- ------     --- ------     --- ------',/)
	DO 110 I=1,4
	  WRITE (nout,2010) I,GCOL(I),I+4,GCOL(I+4),I+8,GCOL(I+8),I+12,
     1          GCOL(I+12)
2010      FORMAT (2X,I2,')',1X,A8,3(3X,I2,')',1X,A8))
110       CONTINUE
120     write(nout,2020)
2020    format(//,'  Colours as assigned to the different',
     1    ' crystallographic positions:',//,
     2    '   Nr.  Multipl.  Elem.   Ident.  Oxid.  Colour',/,
     2    '   ---  --------  -----   ------  -----  ------',/)
	DO 130 J=1,nx
	  WRITE (nout,2030)J,npos(j),GELEM(J),gdent(j),OXID(J),
     1      GCOL(icol(j))
2030      FORMAT (3X,I3,4X,I3,6X,A2,5X,A6,3X,F4.1,2X,A8)
130       CONTINUE
	RETURN
	ENDIF
      IF (GA(1:3).EQ.'RES') GOTO 5
      GELM=GA
145   ipn=999
      ox=99.0
      if(irfl.eq.2) then
C     change one cristallographic position only
	ipn=n
	if(ipn.lt.1.or.ipn.gt.nx) goto 140
	goto 150
	endif
C
C  Check for oxidation state present
C  ---------------------------------
c
      CALL SCAN (GA,N,X,EX,IP,HLINE,IRFL)
      IF (IRFL.eq.2) THEN
C       only colour number given
	OX=99.0
	GOTO 160
      ELSEIF (IRFL .EQ. 1) THEN
C       only colour name given
	OX = 99.0
	GOTO 160
      ELSEIF (IRFL .EQ. 3) THEN
C       save oxidation state 
	OX=X
      else
C       garbage
	goto 140
	ENDIF
      goto 150
140   write(nout,2040)
2040  format(/,' ***ERROR in COLOUR command, check and re-enter',/)
      return
C
C  Check for new colour number or name
c  -----------------------------------
C
150   CALL SCAN (GA,N,X,EX,IP,HLINE,IRFL)
160   IF (IRFL.EQ.2) THEN
	JCOL=N
	IF (JCOL.LT.1. OR. JCOL.GT.16) THEN
	  write(nout,2050)
2050      format(/,' Invalid colour choice, re-enter entire command'/)
	  RETURN
	  ENDIF
      ELSEIF(IRFL .EQ. 1) THEN
	DO 165 I=1,16
	  GCOLU = GCOL(I)
	  CALL UPCASE(GCOLU) 
	  call upcase(ga)
	  IF(GA.EQ.GCOLu) THEN
	    JCOL = I
	    GOTO 167
	    ENDIF
 165      CONTINUE
	WRITE(NOUT,2050)
	RETURN
      ELSE
	GOTO 140
 167    ENDIF
C
C  Modify position/element colour accordingly
c  ------------------------------------------
C
      ICHG=0
      ilim1=1
      ilim2=nx
      if(ipn.ne.999) then
	ilim1=ipn
	ilim2=ipn
	endif
      if (gelm.eq.'BCL') then
	ibclfl=jcol
	ichg=ichg+1
	write(nout,2061)gcol(ibclfl)
2061    format('Changed bond colour to ',a8)
      endif
      if (gelm.eq.'BAC') then
	if (itinfl.ne.0) then
	  write(nout,2064)
2064      format('Turn tint off first.')
	  return
	endif
	ibacfl=jcol
	if (jcol.eq.16) ibacfl=0
	ichg=ichg+1
	write(nout,2066)gcol(jcol)
2066    format('Changed background colour to ',a8)
      endif
      DO 170 KK=ilim1,ilim2
	gelll=gelem(kk)
	call upcase(gelll)
	IF (ipn.ne.999.or.GELll.EQ.GELM) THEN
	  IF ((OXID(KK).EQ.OX).OR.(OX.EQ.99.0)) THEN
	    icol(kk)=JCOL
	    ICHG=ICHG+1
	    write(nout,2060) kk,gelem(kk),gdent(kk),oxid(kk),
     1        gcol(icol(kk))
2060        format(' Changed Pos.',I3,' Elem. ',A2,1X,A6,' Ox. ',
     1        F4.1,' to ',A8)
	    ENDIF
	  ENDIF
170     CONTINUE
      if(ichg.eq.0) goto 140
      icolfl = 1
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE DISPLAY(nunit)
C-----------------------------------------------------------------------
C
C  Written by : David Clarkson  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : DISPLAY will print the current section of the HELPFILE
C               (NUNIT) to the output stream
C
C  Calls      : --
C
C------------------------------------------------------------------------
C
      IMPLICIT CHARACTER *80 (H)
C
      common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine
C
      nl=0
10    READ (nunit,'(A80)',END=999) HLINE
      IF (HLINE(1:2).NE.'%%') THEN
	nl=nl+1
	WRITE (nout,1000) hline
1000    FORMAT (' ',A78)
	if(nl.ge.20) then
	  nl=0
	  write(nout,1010)
1010      format(//'$Hit <RETURN> to continue: ')
	  read(nin,1020,err=10) hdum
1020      format(A1)
	  endif
	GOTO 10
	ENDIF
999   RETURN
      END
C
C
C
C
C----------------------------------------------------------------------
      subroutine edgplo(iaxifl,ishifl)
C----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/86
C
C     Stores the cell corners to the ICO-array and the cell
C     edges/lattice vectors to IBO-array for plotting
C
C     CAUTION: Make sure you don't get into endless loops when
C              calling this routine several times. NRB and NRC
C              have to be reset after each run!!
C
C     Calls : --
C----------------------------------------------------------------------
C
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
C
      dimension ivect(7)
C
      nrbo=nrb
      nrco=nrc
C
C  Store cell axes or cell edges
C  -----------------------------
c
100   if(iaxifl.eq.0) return
      if(iaxifl.eq.1) goto 200
C
C  Cell edges
C  Test if enough room left and if all corners are on screen
C  ---------------------------------------------------------
c
      if((nrb+12).gt.mrb) return
      if((nrc+8).gt.1000) return
      do 110  i=9,16
	if(int(cellax(1,i)).lt.5.or.int(cellax(1,i)).gt.995) return
110     if(int(cellax(2,i)).lt.5.or.int(cellax(2,i)).gt.745) return
      do 120 i=9,16
	if(limits(1).gt.int(cellax(1,i)-3.)) limits(1)=cellax(1,i)-3.
	if(limits(2).lt.int(cellax(1,i)+3.)) limits(2)=cellax(1,i)+3.
	if(limits(3).gt.int(cellax(2,i)-3.)) limits(3)=cellax(2,i)-3.
	if(limits(4).lt.int(cellax(2,i)+3.)) limits(4)=cellax(2,i)+3.
	nrc=nrc+1
	do 125 j=1,3
125       ico(j,nrc)=int(cellax(j,i))
	ico(4,nrc)=0
	ico(5,nrc)=0
	ico(6,nrc)=1
120     continue
C
C  Draw all 12 edges as long-dashed vectors
C  ----------------------------------------
c
      ivect(1)=nrco+1
      ivect(7)=5
      ivect(3)=int(cellax(1,9))
      ivect(4)=int(cellax(2,9))
      do 130 n=10,12
	nrb=nrb+1
	ivect(2)=nrco+n-8
	ivect(5)=int(cellax(1,n))
	ivect(6)=int(cellax(2,n))
	do 135 i=1,7
135       ibo(i,nrb)=ivect(i)
130     continue
      m=13
      n=10
140   ivect(3)=int(cellax(1,m))
      ivect(4)=int(cellax(2,m))
      ivect(1)=nrco+m-8
150   ivect(5)=int(cellax(1,n))
      ivect(6)=int(cellax(2,n))
      ivect(2)=nrco+n-8
      nrb=nrb+1
      do 155 i=1,7
155     ibo(i,nrb)=ivect(i)
      if(m.eq.13) then
	if(n.eq.10) then
	  n=11
	  goto 150
	else if(n.eq.11) then
	  n=16
	  goto 150
	else
	  m=14
	  n=10
	  goto 140
	  endif
      else if(m.eq.14) then
	if(n.eq.10) then
	  n=12
	  goto 150
	else if(n.eq.12) then
	  n=16
	  goto 150
	else
	  m=15
	  n=11
	  goto 140
	  endif
      else
	if(n.eq.11) then
	  n=12
	  goto 150
	else if(n.eq.12) then
	  n=16
	  goto 150
	  endif
	endif
      return
C
C  Three lattice vector indicators only of length approx. 90 pixels
C  ----------------------------------------------------------------
c
200   if((nrb+3).gt.1600) return
      if((nrc+4).gt.1000) return
      do 210 i=10,12
220     dx=cellax(1,i)-cellax(1,9)
	dy=cellax(2,i)-cellax(2,9)
	dlxy=dx**2 + dy**2
	if(dlxy.lt.100.) goto 210
	dlxy=sqrt(dlxy)
	if(dlxy.lt.80.) then
	  cellax(1,i)=cellax(1,9)+1.08*dx
	  cellax(2,i)=cellax(2,9)+1.08*dy
	  goto 220
	else if(dlxy.gt.100.) then
	  cellax(1,i)=cellax(1,9)+.92*dx
	  cellax(2,i)=cellax(2,9)+.92*dy
	  goto 220
	  endif
210     continue
C
C  Shift into screen and plotting ranges if necessary
C  --------------------------------------------------
c
      sftx=0.0
      sfty=0.0
      sftz=0.0
      xmin=1000.0
      xmax=0.0
      ymin=1000.0
      ymax=0.0
      zmin=1000.0
      zmax=0.0
      do 230 i=9,12
	if(cellax(1,i).lt.xmin) xmin=cellax(1,i)
	if(cellax(1,i).gt.xmax) xmax=cellax(1,i)
	if(cellax(2,i).lt.ymin) ymin=cellax(2,i)
	if(cellax(2,i).gt.ymax) ymax=cellax(2,i)
	if(cellax(3,i).lt.zmin) zmin=cellax(3,i)
230     if(cellax(3,i).gt.zmax) zmax=cellax(3,i)
      if(xmin.lt.10.) sftx=10.-xmin
      if(prange(1,3).gt.10..and.prange(1,3).gt.xmin)
     1  sftx=prange(1,3)-xmin
      if(xmax.gt.990.) sftx=990.-xmax
      if(prange(1,4).lt.990..and.prange(1,4).lt.xmax)
     1  sftx=prange(1,4)-xmax
      if(ymin.lt.55.) sfty=55.-ymin
      if(prange(2,3).gt.10..and.prange(2,3).gt.ymin)
     1  sfty=prange(2,3)-ymin
      if(ymax.gt.740.) sfty=740.-ymax
      if(prange(2,4).lt.740..and.prange(2,4).lt.ymax)
     1  sfty=prange(2,4)-ymax
      if(prange(3,3).gt.zmin) sftz=prange(3,3)-zmin
      if(prange(3,4).lt.zmax) sftz=prange(3,4)-zmax
      if(abs(sftx).gt.0.1.or.abs(sfty).gt.0.1.or.abs(sftz).gt.0.1)
     1  ishifl=1
      do 240 i=9,12
	cellax(1,i)=cellax(1,i)+sftx
	cellax(2,i)=cellax(2,i)+sfty
	cellax(3,i)=cellax(3,i)+sftz
	if(limits(1).gt.int(cellax(1,i)-3.))
     1    limits(1)=int(cellax(1,i)-3.)
	if(limits(2).lt.int(cellax(1,i)+3.))
     1    limits(2)=int(cellax(1,i)+3.)
	if(limits(3).gt.int(cellax(2,i)-3.))
     1    limits(3)=int(cellax(2,i)-3.)
	if(limits(4).lt.int(cellax(2,i)+3.))
     1    limits(4)=int(cellax(2,i)+3.)
	nrc=nrc+1
	do 245 j=1,3
245       ico(j,nrc)=int(cellax(j,i))
	ico(4,nrc)=0
	ico(5,nrc)=0
	ico(6,nrc)=1
240     continue
      ivect(1)=nrco+1
      ivect(7)=1
      ivect(3)=int(cellax(1,9))
      ivect(4)=int(cellax(2,9))
      do 250 i=10,12
	ivect(2)=nrco+i-8
	ivect(5)=int(cellax(1,i))
	ivect(6)=int(cellax(2,i))
	nrb=nrb+1
	do 260 j=1,7
260       ibo(j,nrb)=ivect(j)
250     continue
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
	SUBROUTINE FILTER (ifilfl,nrel,rmax)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/85
C
C  Purpose    : Screens the atom array to flag whatever dropped outside
C               the plotting ranges
C
C  Input      : IFILFL ; flag specifing filtering in the Z direction
C               RMAX   ; maximum radius in Angstroms
C               CELLAX ; Array containing corners of unit cell
C
C  Output     : NREL   ; Number of atoms left to plot
C
C  Calls      : filter1
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
C
      common /files/   nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /cell/    a,b,c
      include 'comatom.for'
      common /oatom/   nyo,myo,oyx(3,1000)
      common /plot/    nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                 ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/  nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                 orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
C
C  Filter for xy-plane
C  -------------------
c
      call filter1(nrel,rmax)
      if(nrel.le.0) return
C
C  Filterflag is assumed to be set!!
C  ---------------------------------
C
C  Full z-view
C  -----------
c
50    if(ifilfl.ne.1.or.istefl.eq.2) goto 60
c      write(nout,2020)
2020  format(/,' Full z view will be filtered through',/)
      return
C
C  Plot unit cell only
C  -------------------
60    if(ifilfl.ne.2) goto 90
      write(nout,2030)
2030  format(/,' Unit Cell view will be filtered through',/)
      DO 70 J=1,nrc
	do 80 i=1,3
	  if(ico(i,j).lt.prange(i,3).or.ico(i,j).gt.prange(i,4)) then
	    if(ico(6,j).le.0) goto 70
	    ico(6,j)=0
	    nrel=nrel-1
	    endif
80        continue
70      continue
      return
C
C  User specified limits
C  ---------------------
c
90    do 100 i=1,nrc
	do 110 j=1,3
	  if(ico(j,i).lt.prange(j,3).or.ico(j,i).gt.prange(j,4)) then
	    if(ico(6,i).le.0) goto 100
	    ico(6,i)=0
	    nrel=nrel-1
	    ENDIF
110       continue
100     CONTINUE
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE FILTER1 (nrel,rmax)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : Applies origin shifts and flags the atomic positions
C               that dropped outside the plotting ranges
C
C  Calls      : --
C
C-----------------------------------------------------------------------
C
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
      DIMENSION lim(3)
C
      if(nrc.le.0) return
      irdm=rmax*ascale
      lim(1)=pixwid
      lim(2)=.75*pixwid
      lim(3)=pixwid
C
C  Shift plot origin to centre of screen
C  -------------------------------------
c
      do 10 i=1,nrc
	do 15 j=1,3
15        ico(j,i)=ico(j,i)+lim(j)/2
10      continue
      do 40 i=9,16
	do 50 j=1,3
50        cellax(j,i)=cellax(j,i)+lim(j)/2.
40      continue
      do 60 i=1,3
	do 70 j=3,4
70        prange(i,j)=prange(i,j)+lim(i)/2.
60      continue
C
C  Drop all atoms outside screen ranges (y only from 0 to .75*PIXWID)
C  Depth in Z direction is unlimited
C  ------------------------------------------------------------------
c
      nrel=nrc
      DO 20 i=1,nrc
	ico(6,i)=1
	do 30 j=1,2
	  if(ico(j,i).lt.irdm.or.ico(j,i).gt.(lim(j)-irdm)) then
	    ico(6,i)=0
	    nrel=nrel-1
	    goto 20
	    endif
30        continue
20      CONTINUE
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE FILTER3 (nrel)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : Flags the bonds in IBO with respect to the atoms
C               left in ICO and the bond valence/length limits set
C               in XBLIM(2)
C
C  Output     : NREL   ; number of bonds left after treatment
C
C  Calls      : calcpt
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
C
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
C
      if(nrb.le.0.or.nrc.le.0) return
C
      nrel=nrb
      DO 10 i=1,nrb
	ibo(7,i)=iabs(ibo(7,i))
	if(ico(6,ibo(1,i)).eq.1.and.ico(6,ibo(2,i)).eq.1) then
	  if(xbo(i).ge.xblim(1).and.xbo(i).le.xblim(2)) then
	    CALL CALCPT(i,istifl)
	  else
	    ibo(7,i)=-ibo(7,i)
	    nrel=nrel-1
	    endif
	else
	  ibo(7,i)=-ibo(7,i)
	  nrel=nrel-1
	  endif
10      continue
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE GLU (nterml, RMAX, izma, izmi)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C             : Nina Heinig, August 1986
c             : Tamon Stephen, March 1993
C
C  Purpose    : The Graphics Language Utility (GLU) is the interactive means
C               by which the user can manipulate the plotting of the selected
C               structural portion
C
C  Input      : NTERML ; Terminal type flag
C
C  Calls      : cbase2, display, iclosu, ifile, locate, mxv, readyn, rotate,
C               scan, upcase
C
C  NOTE       : Expects the existance of a helpfile called:
C                   on the VAX:          GR:STRUPLO.HLP
C                   on ND-500 :          STRUPLO:HELP
C                   on Cyber  :          PLOTHLP
c                   on the PC :          struplo.hlp
C               This file will be opened for read on unit 29
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
      IMPLICIT CHARACTER *80 (H)
C
C  System common blocks
C
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /oatom/  nyo,myo,oyx(3,1000)
      include 'comatom.for'
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      COMMON /ATTRIB/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /rottr/  rotm(3,3),nrot,nrro
C
      dimension axs(3),xxx(3)
C
c
      WRITE(NOUT,2001)
2001  FORMAT(//,' You are now in the Graphics Language Utility (GLU)',
     1 /,' which is command driven',/,
     2 ' To find out about possible commands type HELP')
C
 9    IMESSAG=0
      IF (nrot.le.nrro .and. nrro.ne.0) goto 160
C
C  Angstroms per cm scale
c  ----------------------
C
      PIXPCM = PIXWID/25.
      APCM=PIXPCM/ASCALE
C
C  Prompt user
c  -----------
C
10    HLINE=' '
      istofl=0
      WRITE(nout,2005)
2005  FORMAT(//,'$GLU> ')
      READ(nin,'(A80)',ERR=20) HLINE
      CALL UPCASE(HLINE)
      IF (HLINE.EQ.' ') GOTO 10
      IP=1
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF(IRFL.NE.1) goto 20
      IF(GA(1:3).EQ.'ROT') GOTO 100
      IF(GA(1:3).EQ.'TRA') GOTO 200
      IF(GA(1:3).EQ.'LIS') GOTO 300
      IF(GA(1:3).EQ.'SCA') GOTO 400
      IF(GA(1:3).EQ.'ZOO') GOTO 500
      IF(GA(1:3).EQ.'STI') GOTO 600
      IF(GA(1:3).EQ.'BON') GOTO 700
      IF(GA(1:3).EQ.'RAD') GOTO 800
      IF(GA(1:3).EQ.'STR') GOTO 900
      IF(GA(1:3).EQ.'RAN') GOTO 1000
      IF(GA(1:3).EQ.'SHO') GOTO 1100
      IF(GA(1:2).EQ.'EX')  GOTO 1200
      IF(GA(1:3).EQ.'QUI') GOTO 1200
      IF(GA(1:3).EQ.'COL') GOTO 1300
      IF(GA(1:3).EQ.'JOY') GOTO 1400
      IF(GA(1:3).EQ.'HEL') GOTO 1500
      if(ga(1:3).eq.'AXI') goto 1600
      if(ga(1:3).eq.'LAB') goto 1700
      IF(GA(1:3).EQ.'TIN') GOTO 1750
      IF(GA(1:3).EQ.'STE') GOTO 1800
      IF(GA(1:3).EQ.'FAS') GOTO 1850
      IF(GA(1:3).EQ.'PLO') GOTO 1900
      if(ga(1:3).eq.'PRI') goto 1950
      if(ga(1:3).eq.'GRA') goto 4000
20    write(nout,2010)
2010  format(//,' ****ERROR*** Unrecognized command or missing ',
     1  'parameters ',/,' ------------ Try again or see HELP!')
      GOTO 10
C---------------------------------------------------------------
C  ROT[ations] ; rotations about X/Y/Z
C---------------------------------------------------------------
100   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      XINC=0
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 1'
2020    format(//,' ***ERROR*** mixed numbers & characters in',
     1    ' Argument ',A3,' -- Try again!')
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) THEN
110     WRITE (nout,2030)
2030    FORMAT (/,'$Enter rotation angle: ')
	READ (nin,*,ERR=110) ANG
120     WRITE (nout,2040)
2040    FORMAT('$Enter rotation axis (as vector triplet): ')
	READ (nin,*,ERR=120) (AXS(I),I=1,3)
	GOTO 130
	ENDIF
      ANG=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 2'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) XARG=0.
      AXS(1)=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 3'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) XARG=0.
      AXS(2)=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF(IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 4'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) XARG=0.
      AXS(3)=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 5'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) XARG=0.
      XINC=XARG
C
C  Process information read
c  ------------------------
C
130   IF (XARG.EQ.0) XINC=ABS(ANG)
      if(xinc.lt.0.1) then
	write(nout,2045)
2045    format(//,' ***ERROR*** Rotation angle too small')
	goto 10
	endif
      IF (ANG.LT.0.0.AND.XINC.GT.0.0) XINC=-1.0*XINC
      nrro=INT(ANG/XINC)
140   IF(AXS(1).EQ.0.0.AND.AXS(2).EQ.0.0.AND.AXS(3).EQ.0.0) THEN
	write(nout,2050)
2050    format(//,' ***ERROR*** Rotation about 0,0,0 is impossible')
	GOTO 10
	ENDIF
150   IF (nrro.EQ.0) nrro=1
C
C  Perform matrix operations for rotation
c  --------------------------------------
C
      AXS(1)=-1.0*AXS(1)
      AXS(2)=-1.0*AXS(2)
      AXS(3)=-1.0*AXS(3)
c
C  At the end of plot, reset flags
c  -------------------------------
c
160   IF(NROT .GE. NRRO) THEN
	IF(NROT .GT. NRRO) THEN
	  WRITE(NOUT,2067) NROT,NRRO,ANG
2067      FORMAT(' AABH',/,' **WARNING*Invalid rotation parameters,',
     *        ' reenter command. Step no.',I2,', total steps',I2,
     *        ', angle:',I3)
	  ENDIF
	NROT = 0
	NRRO = 0
	GOTO 10
	ENDIF  
      CALL ROTATE (ANG/nrro,AXS)
      nrot=nrot+1
c
C  Plot if in the middle of a strobe plot otherwise, wait for plot cue
c  -------------------------------------------------------------------
c
      IF(NROT .EQ. 1) THEN
	GOTO 10
      ELSE
	CALL PLOTER(RMAX,NTERML,IZMA,IZMI)
	GOTO 160
	ENDIF
C--------------------------------------------------------------
C  TRA[nslation]  ; Define new centre of screen
C--------------------------------------------------------------
200   itrafl=0
205   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 1'
	GOTO 10
	ENDIF
      if(irfl.eq.1) then
	if(ga(1:3).eq.'ATO') then
	  itrafl=1
	  goto 205
	else if(ga(1:3).eq.'POS') then
	  itrafl=2
	  goto 205
	else if(ga(1:3).eq.'DEF') then
	  do 215 i=1,3
215         orgsft(i)=(window(i,4)-window(i,3))/2. + window(i,3)
	  goto 10
	else
	  goto 20
	  endif
	endif
      IF (IRFL.EQ.0.and.itrafl.ne.1) then
	hcom='Angstroms'
	if(itrafl.ne.0) hcom='fractions'
	write(nout,2075) hcom
2075    format(//,' Enter new view point (centre of screen) in ',
     1    A9,' from cell origin')
210     WRITE (nout,2070)
2070    FORMAT(/,'$Enter new X position: ')
	READ (nin,*,ERR=210) X
220     WRITE (nout,2080)
2080    FORMAT('$Enter new Y position: ')
	READ (nin,*,ERR=220) Y
230     WRITE (nout,2090)
2090    FORMAT('$Enter new Z position: ')
	READ (nin,*,ERR=230) Z
	GOTO 250
      else if(irfl.eq.0.and.itrafl.eq.1) then
240     write(nout,2100)
2100    format(/,'$Enter atom number as of atom list: ')
	read(nin,*,err=240) itrat
	if(itrat.le.0.or.itrat.gt.ny) call inerr(*240)
	goto 250
	ENDIF
      xxx(1)=XARG
      if(itrafl.eq.1) then
	itrat=narg
	goto 250
	endif
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 2'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) xarg=0.0
      xxx(2)=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 3'
	GOTO 10
	ENDIF
      IF (IRFL.EQ.0) xarg=0.0
      xxx(3)=XARG
C
C  Perform the actual translation here
c  -----------------------------------
C
250   if(itrafl.eq.1) then
	do 260 i=1,3
260       orgsft(i)=oyx(i,itrat)
      else if(itrafl.eq.2) then
	call mxv(orgsft,bbten,xxx)
      else
	do 270 i=1,3
270       orgsft(i)= orgsft(i) - xxx(i)
	endif
      goto 10
C-----------------------------------------------------------
C  LIS[t] ;  LIST element and bond attributes
C-----------------------------------------------------------
300   call scan(ga,narg,xarg,xe,ip,hline,irfl)
      if(irfl.ne.1) goto 20
      if(ga(1:3).eq.'BON') goto 350
      if(ga(1:3).ne.'ATO') goto 20
C
C  List all atoms stored in ICO
C  ----------------------------
c
      j1=1
310   write(nout,2110)
2110  format(//,' List of currently stored atoms',//,
     1  '  Nr.  Element  Identifier  Coordinates (in Angstroms)',/)
      DO 320 J=j1,NRC
	I=ico(5,j)
	WRITE (nout,2120) J,GELEM(I),gdent(i),(oyx(ii,J),ii=1,3)
2120    FORMAT (1X,I4,5X,A2,6X,A6,2X,3(2X,F6.2))
	if(j.ge.(j1+19)) then
	  write(nout,2130)
2130      format(/,'$Hit <RETURN> to continue: ')
	  read(nin,3000,err=330) gdum
3000      format(A1)
330       j1=j1+20
	  if(j1.lt.nrc) goto 310
	  goto 10
	  endif
320     CONTINUE
      GOTO 10
C
C  List all bonds stored in IBO
C  ----------------------------
c
350   j1=1
360   write(nout,2140)
2140  format(//,' List of currently stored bonds',//,
     1  '  Nr.  from                to   (see atom list)',/)
      do 370 j=j1,nrb
	ji=ibo(1,j)
	jj=ibo(2,j)
	write(nout,2150) j,ji,gelem(ico(5,ji)),gdent(ico(5,ji)),
     1    jj,gelem(ico(5,jj)),gdent(ico(5,jj))
2150    format(1X,I4,2('  (',I4,')  ',A2,2X,A6))
	if(j.ge.(j1+19)) then
	  write(nout,2130)
	  read(nin,3000,err=380) gdum
380       j1=j1+20
	  if(j1.lt.nrb) goto 360
	  goto 10
	  endif
370     continue
      goto 10
C--------------------------------------------------------------
C  SCA[le]  ;  change scale
C--------------------------------------------------------------
400   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      CALL UPCASE(GA)
      pixpcm=pixwid/25.
      angpcm=pixpcm/ascale
      apcm=angpcm
      IF(IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 1'
	GOTO 10
      else IF (IRFL.EQ.0) THEN
	WRITE (nout,2200) angpcm
2200    FORMAT(//,' The current scale is ',F6.1,' A/cm',/)
410     WRITE (nout,2210)
2210    FORMAT(/,'$Enter new scale (Angs/cm): ')
	APCM=0.
	READ (nin,*,err=410) apcm
	if(apcm.le.0.0) apcm=angpcm
	ASCALE=PIXPCM/APCM
	iautfl=0
      else IF (IRFL.EQ.1.AND.GA(1:3).EQ.'DEF') THEN
	ASCALE=DSCALE
	iautfl=1
      else IF (IRFL.EQ.1.AND.GA(1:2).EQ.'ON') then
	ISCAfl=1
      else IF (IRFL.EQ.1.AND.GA(1:2).EQ.'OF') then
	ISCAfl=0
      elseif( irfl.eq.2 .or. irfl.eq.3) then
	APCM=XARG
	ASCALE=PIXPCM/APCM
	iautfl=0
	endif
      hcom='be plotted    '
      if(iscafl.eq.0) hcom='be plotted'
      WRITE (nout,2215) APCM,hcom
2215  FORMAT(//,' Current scaling ',F6.1,' A/cm, scale will ',A16,/)
      GOTO 10
C---------------------------------------------------------------------
C  ZOO[m]  ; Zoom observer closer to or further away from view point
C            at centre of screen
C---------------------------------------------------------------------
500   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.4.OR.IRFL.EQ.5) THEN
	write(nout,2020) '# 1'
	GOTO 10
      else IF (IRFL.EQ.0) THEN
	WRITE (nout,2220) viewd
2220    FORMAT(//,' Present view distance is',F9.2,' Angstroms',/)
510     WRITE (nout,2230)
2230    FORMAT(/,'$Enter new view distance (> 1 A): ')
	READ (nin,*,err=510) xarg
	ENDIF
      if(xarg.lt.viewd) write(nout,2240)
2240  format(//,' Some atoms may be shifted out of screen!',/)
      viewd=xarg
      IF (viewd.lt.1.0) viewd=1.0
      GOTO 10
C------------------------------------------------------------------
C  STI[ck]  ;  plot atoms or just stick network
C------------------------------------------------------------------
600   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      CALL UPCASE (GA)
      IF (IRFL.EQ.0.OR.GA(1:3).EQ.'OFF') THEN
	istifl=0
	hcom='(//,'' Normal structure plot selected'',/)'
      ELSE if(irfl.eq.1.and.ga(1:2).eq.'ON') then
	istifl=1
	hcom='(//,'' Stick network plot selected'',/)'
      ELSE
	GOTO 20
	ENDIF
      write(nout,hcom)
      goto 10
C------------------------------------------------------------------
C  BON[d] ; switch bond plot on or off
C------------------------------------------------------------------
700   CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.EQ.0.OR.GA(1:2).EQ.'ON') THEN
	ibonfl=1
	hcom='(//,'' Bonds will be plotted'',/)'
	if(nrb.le.0) then
	  ibonfl=0
	  hcom='(//,'' Sorry, no bonds there to be plotted'',/)'
	  endif
      ELSE if (irfl.eq.1.and.ga(1:3).eq.'MON') then
	ibonfl=2
	hcom='(//,'' Bonds will not be multi-coloured'',/)'
	if(nrb.le.0) then
	  ibonfl=0
	  hcom='(//,'' Sorry, no bonds there to be plotted'',/)'
	  endif
      else if (irfl.eq.1.and.ga(1:3).eq.'THI') then
	ibonfl=3
	hcom='(//,'' Bonds will be thickened & coloured to reflect '// 
     +         'valence'',/)'
	if (itinfl.eq.0) hcom='(//,''Must be tinted!'',/)'
	if (ivgafl.le.1) hcom='(//,''Sorry, only in SVGA mode'',/)'
	if (nrb.le.0) then
	  ibonfl=0
	  hcom='(//,'' Sorry, no bonds to plot'',/)'
	  endif
      else if(irfl.eq.1.and.ga(1:3).eq.'OFF') then
	ibonfl=0
	hcom='(//,'' Bonds will not be plotted'',/)'
	if(ispafl.eq.1) then
	  ibonfl=1
	  hcom='(//,'' Spanning tree: Bonds will still be '//
     1      'plotted!!'',/)'
	  endif
      else if(irfl.eq.1.and.ga(1:3).eq.'ION') then
	if (ibonfl.eq.0) ibonfl=1
	xblim(1)=0.001
	xblim(2)=5.00
	hcom='(//,''Ionic bonds with valences in the range 0.001-'
     1    //'5.0 v.u. will be plotted'',/)'
      else if(irfl.eq.1.and.ga(1:3).eq.'NON') then
	if (ibonfl.eq.0) ibonfl=1
	xblim(1)=-15.0
	xblim(2)=0.0
	hcom='(//,'' Non-ionic bonds less than 15 Angstroms only will'
     1    //' be plotted'',/)'
      ELSE if(irfl.eq.2.or.irfl.eq.3) then
	if (ibonfl.eq.0) ibonfl=1
	xblim(1)=xarg
	if(ip.ne.99) call scan(ga,narg,xblim(2),xe,ip,hline,irfl)
	if(xblim(2).le.xblim(1)) xblim(2)=xblim(1)+0.1
	if(xblim(2).gt.0.0) then
	  xlimi=0.0
	  if(xblim(1).gt.xlimi) xlimi=xblim(1)
	  write(nout,1710) xlimi,xblim(2)
 1710     format(//,' Bonds with valence between ',F5.2,' and ',F5.2,
     +      ' will be plotted')
	  endif
	if(xblim(1).lt.0.0) then
	  xlimi=0.0
	  if(xblim(2).lt.xlimi) xlimi=xblim(2)
	  write(nout,1715) -xlimi,-xblim(1)
1715      format(//,' Bonds with length between ',F5.2,' and ',F5.2,
     1      ' will be plotted')
	  endif
	go to 10
      else if(irfl.eq.1.and.ga(1:3).eq.'DEF') then
	xblim(1)=-15.0
	xblim(2)=5.0
	ibonfl=1
	xlimi=0.0
	write(nout,1710) xlimi,xblim(2)
	write(nout,1715) xlimi,-xblim(1)
	hcom='(/)'
      ELSE
	GOTO 20
	ENDIF
      write(nout,hcom,err=1024)
1024  goto 10
C-----------------------------------------------------------------
C  RAD[ius]  ;  change the radii of atoms
C-----------------------------------------------------------------
800   IF (istifl.EQ.1) THEN
	write(nout,2300)
2300    format(//,' You''re drawing a stick network with',
     *        ' fixed radii!!',/)
	GOTO 10
	ENDIF
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (irfl.eq.0.or.IRFL.EQ.1) GOTO 810
      IF (IRFL.EQ.2.OR.IRFL.EQ.3) GOTO 860
      GOTO 20
810   GNAME=GA
      CALL UPCASE (GNAME)
      IF (GNAME(1:3).EQ.'DEF'.or.irfl.eq.0) THEN
	DO 820 I=1,nx
820       RADMUL(I)=1.0
	write(nout,2310)
2310    format (//,' Radii defaulted to table values',/)
	GOTO 10
	ENDIF
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.NE.2.AND.IRFL.NE.3) GOTO 20
      XHOLD=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.NE.0) GOTO 840
      irads=0
      DO 830 I=1,nx
	glm=gelem(i)
	call upcase(glm)
	IF (glm(1:2).EQ.GNAME(1:2)) then
	  write(nout,2320) gelem(i),i,rad(i)*radmul(i),
     1      rad(i)*radmul(i)*xhold
2320      format(' Radius of element ',A2,' in position ',I3,
     1      ' set from ',F6.2,' to ',F6.2,' A.')
	  RADMUL(I)=RADMUL(I)*xhold
	  irads=1
	  endif
830     continue
      if(irads.eq.0) then
	write(nout,2325) gname
 2325   format(' Element ',A2,' not found')
	endif
      GOTO 10
840   IF (IRFL.NE.2.AND.IRFL.NE.3) GOTO 20
      XOX=XHOLD
      XRAD=XARG
      irads=0
      DO 850 I=1,nx
	glm=gelem(i)
	call upcase(glm)
	IF (glm(1:2).EQ.GNAME(1:2).AND.OXID(I).EQ.XOX) then
	  write(nout,2320) gelem(i),i,rad(i)*radmul(i),
     1      rad(i)*radmul(i)*xrad
	  RADMUL(I)=RADMUL(I)*XRAD
	  irads=1
	  endif
850     CONTINUE
      if(irads.eq.0) then
	write(nout,2325) gname
	endif
      GOTO 10
860   XHOLD=XARG
      CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.NE.0) GOTO 20
      XRAD=XHOLD
      DO 870 I=1,nx
870     RADMUL(I)=RADMUL(I)*XRAD
      write(nout,2330) xrad
2330  format(//,' All radii multiplied by a factor of ',F9.4,/)
      GOTO 10
C------------------------------------------------------------------
C  STR[obe]  ;  turn strobe effect on or off
C------------------------------------------------------------------
900   CALL SCAN(GA,NARG,XARG,XE,IP,HLINE,IRFL)
      CALL UPCASE(GA)
      IF(IRFL.NE.1) goto 20
      IF(GA(1:2).EQ.'ON') then
	istrfl=1
	hcom='(//,'' Strobe effect turned on'',/)'
      else IF(GA(1:2).EQ.'OF') then
	istrfl=0
	hcom='(//,'' Strobe effect switched off'',/)'
      else
	goto 20
	endif
      write(nout,hcom)
      GOTO 10
C-----------------------------------------------------------------
C  RAN[ge]  ;  select the plotting range in the Z direction
C-----------------------------------------------------------------
1000  CALL SCAN(GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.NE.1.AND.IRFL.NE.0) goto 20
      IF(GA(1:3).EQ.'UNI') then
	ifilfl=2
	do 1005 i=1,3
	  prange(i,1)=-.05
1005      prange(i,2)=1.05
      else IF(GA(1:3).EQ.'SEL') THEN
	ifilfl=3
	WRITE(nout,2340) ((window(i,j),j=1,2),
     1    (prange(i,j),j=1,2),i=1,3)
2340    format(//,' Axis  Maximal window possible  Current plotting ',
     1    'ranges',//,'  x :   ',F6.3,'   to   ',F6.3,';   ',F6.3,
     2    '  to  ',F6.3,/,'  y :   ',F6.3,'   to   ',F6.3,';   ',
     3    F6.3,'  to  ',F6.3,/,'  z :   ',F6.3,'   to   ',F6.3,
     4    ';   ',F6.3,'  to  ',F6.3,/)
1010    WRITE(nout,2350)
2350    FORMAT(/,'$Do you want to update the plotting ranges (Y/N): ')
	call readyn(*1020,*1030,*10,*1010)
1020    WRITE(nout,2360)
2360    FORMAT(' Enter new x-min, x-max, y-min, y-max,',
     1    ' z-min, z-max:',/)
	READ(nin,*,err=1020) ((prange(i,j),j=1,2),i=1,3)
	do 1015 i=1,3
1015      if((prange(i,2)-prange(i,1)).lt.0.1) call inerr(*1020)
	write(nout,2340) ((window(i,j),j=1,2),(prange(i,j),j=1,2),i=1,3)
1025    write(nout,2370)
2370    format(/,'$This update OK (Y/N): ')
	call readyn(*1030,*1020,*1030,*1025)
1030    iautfl=0
	write(nout,2380)
2380    format(/,'$Adjust scale automatically to plotting ',
     1    'ranges (Y/N): ')
	call readyn(*1040,*1050,*1050,*1030)
1040    iautfl=1
1050    continue
      else if(ga(1:3).eq.'FUL'.or.ga(1:3).eq.'DEF') then
	ifilfl=1
	do 1060 i=1,3
	  prange(i,1)=window(i,1)
1060      prange(i,2)=window(i,2)
	iautfl=1
      else
	goto 20
	ENDIF
      do 1070 i=1,3
1070    xxx(i)=(prange(i,2)-prange(i,1))/2.+prange(i,1)
      call mxv(orgsft,bbten,xx)
      GOTO 10
C----------------------------------------------------------------
C  SHO[w]  ;  show flags, scale, range...
C----------------------------------------------------------------
1100  if(ip.eq.99) then
	write(nout,2395)
2395    format(' Give qualifier.  Possible qualifiers are: ',/,
     1   ' FLAG, RANGE and SCALE')
	read(nin,'(A80)') hline
	ip=1
	endif
      CALL SCAN(GA,NARG,XARG,XE,IP,HLINE,IRFL)
      CALL UPCASE(GA)
      IF (IRFL.NE.1) goto 20
      IF (GA(1:3).EQ.'FLA') THEN
	hcom1='JOY[stick]  : Is disabled'
	hcom2='SCA[le]     : Will be plotted'
	if(iscafl.eq.0) hcom2(15:80)='Will not be plotted'
	hcom3='RAN[ge]     : Full z-range will be plotted'
	if(ifilfl.eq.2) hcom3(15:80)='One unit cell depth will'
     1    //' be plotted'
	if(ifilfl.eq.3) hcom3(15:80)='User specified z-range'
     1    //' will be plotted'
	hcom4='STI[ck]     : Normal structure plot'
	if(ispafl.eq.1) hcom4(15:80)='Spanning tree plot   '
	if(istifl.eq.1) hcom4(38:80)='with Stick Bonds'
	hcom5='COL[our]    : Automatic colour selection'
	if(icolfl.eq.1) hcom5(15:80)='Manual colour selection'
	hcom6='BON[ds]     : Will be drawn'
	if(ibonfl.le.0) hcom6(15:80)='Will not be drawn'
	hcom7='STR[obe]    : Effect switched off'
	if(istrfl.eq.1) hcom7(15:80)='Effect turned on'
	hcom8='AXI[s]      : Three basic lattice directions will be'
     1    //' shown'
	if(iaxifl.eq.0) hcom8(15:80)='No axes will be drawn'
	if(iaxifl.eq.2) hcom8(15:80)='All unit cell edges will be'
     1    //' drawn'
	hcom9='LAB[el]     : Plotted atoms will be labelled by '
     1    //'element name'
	if(ilabfl.eq.0) hcom9(34:80)='not be labelled'
	hcom10='RAN[ge] SEL : Automatic scale adjustment to plotting'
     1    //' ranges'
	if(iautfl.eq.0) hcom10(15:80)='Manual scale selection only'
	write(nout,2400) hcom1,hcom2,hcom3,hcom4,hcom5,hcom6,hcom7,
     1    hcom8,hcom9,hcom10
2400    format(//,10(3X,A76,/))
      else IF (GA(1:3).EQ.'SCA') THEN
	pixpcm=pixwid/25.
	angpcm=pixpcm/ascale
	WRITE(nout,2410) angpcm
2410    format(//,' Currently stored scale is ',F6.2,' Ang./cm',/)
	if(iautfl.eq.1) write(nout,2430)
2430    format(/,' Scale might get changed by automatic adjustment',/)
      else IF(GA(1:3).EQ.'RAN') THEN
	write(nout,2340) ((window(i,j),j=1,2),(prange(i,j),j=1,2),
     1    i=1,3)
      else
	goto 20
	ENDIF
      GOTO 10
C----------------------------------------------------------------------
C  QUI[t]  or EXI[t] ;  exit plotting package and return to calling
C                       program system
C----------------------------------------------------------------------
1200  if(nterml.eq.1) then
	WRITE(nter,2500)
2500    FORMAT(' +-*/AABH')
      elseif(nterml .eq. 2) then
	call clrscr(nter)
	call tmode(nter)
	call tekoff(nter)
	endif
      return
C----------------------------------------------------------------------
C  COL[our] ;  Change the atom colours
C----------------------------------------------------------------------
1300  if(nterml.eq.1) then
	CALL CBASE2 (HLINE,IP,nterml)
      else if(nterml.eq.2) then
	write(nout,2520)
2520    format(//,' Only black/white possible for this terminal',/)
	endif
      GOTO 10
C----------------------------------------------------------------------
C  JOY[stick]  ;  engage the joystick
C----------------------------------------------------------------------
1400  write(nout,2530)
2530  format(//,' Sorry, JOYSTICK mode has not been installed yet',/)
      goto 10
C----------------------------------------------------------------------
C  HEL[p]  ;  enter HELP facility
C----------------------------------------------------------------------
1500  CALL SCAN (GA,NARG,XARG,XE,IP,HLINE,IRFL)
      IF (IRFL.eq.1) THEN
	nunit=29
	if(machine.eq.1) then
	  helpfi='gr:struplo.hlp'
	else if(machine.eq.2) then
	  helpfi='struplo:help'
	else if(machine.eq.3) then
	  helpfi='plothlp'
	else if (machine.eq.5) then
	  helpfi='struplo.hlp'
	else
	  helpfi='no-help-file'
	  endif
	nnh=iclosu(nunit,helpfi)
	if(nnh.ne.0) then
	  iunit=0
	  nnh=iclosu(iunit,helpfi)
	  endif
	gfort=' '
	gstat='old'
	nnh=ifile(helpfi,gfort,gstat,nunit)
	if(nnh.le.0) goto 10
	nunit=nnh
	HLINE=GA
	GOTO 1520
	ENDIF
1510  write(nout,2600)
2600  format(//,' HELP information currently available on:',//,
     1  ' AXIS        BONDS       COLOUR      EXIT        FASTDRAW ',/,
     2  ' HELP        LABEL       LIST        PLOT        RADIUS   ',/,
     3  ' RANGE       ROTATE      SCALE       SHOW        STEREOPLOT',/       
     4 ,' STICK       STROBE      TRANSLATE   QUIT        ZOOM',/,
     *  ' TINT        GRAPHICS    PRINTER',//,
     5  ' Type:  HELP [command]      for more information',/,
     6  ' The first 3 characters are sufficient to identify a',
     7  ' command.',/)
      goto 10
1520  CALL UPCASE (HLINE)
      IF (HLINE.EQ.' ') THEN
	nnc=iclosu(nunit,helpfi)
	GOTO 20
	ENDIF
      CALL LOCATE (HLINE,IFND,nunit)
      IF (IFND.EQ.1) THEN
	CALL DISPLAY(nunit)
      ELSE
	write(nout,2620) hline
2620    format(//,' Sorry, no HELP on ',A50,/)
	goto 1510
	ENDIF
      nnc=iclosu(nunit,helpfi)
      goto 10
C---------------------------------------------------------------------
C     AXI[s] :  Draw cell axes or edges
C---------------------------------------------------------------------
1600  call scan(ga,narg,xarg,xe,ip,hline,irfl)
      call upcase(ga)
      if(irfl.eq.0) goto 1610
      if(irfl.ne.1) goto 20
1610  if(irfl.eq.0.or.ga(1:2).eq.'ON') then
	iaxifl=1
	hcom='(//,'' Three basic lattice vectors will be shown '',/)'
      else if(ga(1:3).eq.'FUL') then
	iaxifl=2
	hcom='(//,'' Unit cell edges will be drawn (if all on '
     1    //'screen!)'',/)'
      else if(ga(1:3).eq.'OFF') then
	iaxifl=0
	hcom='(//,'' No cell axes will be drawn'',/)'
      else
	goto 20
	endif
      write(nout,hcom)
      goto 10
C-------------------------------------------------------------------
C     LAB[el] :  Do or do not label plotted atoms by element name
C-------------------------------------------------------------------
1700  call scan (ga,narg,xarg,xe,ip,hline,irfl)
      call upcase (ga)
      if(irfl.gt.1.or.irfl.lt.0) goto 20
      if(irfl.eq.0.or.ga(1:2).eq.'ON') then
	ilabfl=1
	hcom='(//,'' Plotted atoms will be labelled by element '
     1    //'name'',/)'
      else if (ga(1:3).eq.'OFF') then
	ilabfl=0
	hcom='(//,'' Plotted atoms will not be labelled'',/)'
      else
	goto 20
	endif
      write(nout,hcom)
      goto 10
C
C--------------------------------------------------------------------
C     TIN[T] : Create a 3-D effect with shading on the atoms. All 
c              atoms come out blue.
C--------------------------------------------------------------------
1750  call scan (ga,narg,xarg,xe,ip,hline,irfl)
      call upcase (ga)
      if(irfl.gt.1.or.irfl.lt.0) goto 20
      if(irfl.eq.0.or.ga(1:2).eq.'ON') then
	itinfl=1
	hcom='(//,'' Plotted atoms will be blue and shaded.'',/)'
       if (ivgafl.ge.2) hcom='(//,'' Plotted atoms will be shaded.'',/)'
       if (ivgafl.eq.1) then
	 hcom='(//,'' No can do.'',/)'
	 itinfl=0
       endif
      else if (ga(1:3).eq.'OFF') then
	itinfl=0
	hcom='(//,'' Plotted atoms will be coloured'',/)'
      else if (ga(1:3).eq.'BAC') then
	itinfl=2
	if (ivgafl.ge.2) hcom='(//,''Backlighting'',/)'
      else
	goto 20
	endif
      write(nout,hcom)
      goto 10
C
C---------------------------------------------------------------------
C     STE[REOPLOT] :  Plot a unit cell in stereo
C---------------------------------------------------------------------
C
1800  call scan(ga,narg,xarg,xe,ip,hline,irfl)
c
c  pix is the one half the number of pixels the image centers are
c  seperated by.
c  --------------------------------------------------------------
c
      if(irfl.eq.3 .or. irfl.eq.2 ) then
	pix = xarg
      elseif(irfl .eq. 0) then
	pix = 100.
      else
	goto 20
	endif
c
c  set flags (scale adjustment and strobe must be off)
c  ---------------------------------------------------
c
      istefl=1
      istrfl=0
      iautfl=0
      iaxifl=0
c
c  reduce scale to one-half previous value
c  ---------------------------------------
c
      pixpcm = pixwid/25
      angpcm = pixpcm/ascale
      angpcm = angpcm*2
      ascale = pixpcm/angpcm
c
      call ploter(rmax, nterml, izma, izmi)      
c
c  reset flag, scale, position, and exit STEREOPLOT
c  ------------------------------------------------
c
      angpcm = angpcm/2
      ascale = pixpcm/angpcm
      iprifl = 0
      goto 10
C---------------------------------------------------------------------
C     FAS[TDRAW] : Do or do not draw only bond network
C---------------------------------------------------------------------
1850  call scan(ga,narg,xarg,xe,ip,hline,irfl)
      call upcase(ga)
      if( irfl .ne. 1) goto 20
      if(ga(1:2) .eq. 'ON')then
	istifl = 0
	iquifl = 1
	hcom = '(/,'' Only bonds will be drawn.'',//)'
      elseif(ga(1:2) .eq. 'OF') then
	iquifl = 0
	hcom = '(/,'' Normal plotting will take place.'',//)'
      else
	goto 20
	endif
      write(nout,hcom)
      goto 10
C----------------------------------------------------------------------
C  Plot image and ask for prompt 
C----------------------------------------------------------------------
1900  CALL PLOTER(RMAX, NTERML, IZMA, IZMI)
      GOTO 9
c----------------------------------------------------------------------
c  Printer Plot
c----------------------------------------------------------------------
1950  iprifl=1
      call scan(ga,narg,xarg,xe,ip,hline,irfl)
      write(nout,1951)
1951  format('Once the title line has appeared, you will have',/,
     +       'approximately 4 seconds to active your TSR screen',/,
     +       'dump utility.')
      call wait
      if (ga(1:3).eq.'STE') goto 1800
      call ploter(rmax,nterml,izma,izmi)
      iprifl=0
      goto 9
c----------------------------------------------------------------------
c  Switch graphics on and off
c----------------------------------------------------------------------
 4000 call scan(ga,narg,xarg,xe,ip,hline,irfl)
      call upcase(ga)
      if( irfl .ne. 1) goto 20
      if(nterml.eq.2) then
	if(ga(1:2) .eq. 'ON')then
	  call tekon(nter)
	elseif(ga(1:2) .eq. 'OF') then
	  call tekoff(nter)
	else
	  goto 20
	  endif
      else 
	write(nout,4001)
 4001   format(' Does not work for this terminal')
	endif
      go to 10
      END
C
C
C
C
C-----------------------------------------------------------------------
	SUBROUTINE LOCATE (HLINE,IFND,nunit)
C-----------------------------------------------------------------------
C
C  Written by : David Clarkson  Summer 1985
C
C  Purpose    : LOCATE will position the read pointer at the first line of
C               required help section on the HELPFILE (NUNIT)
C
C  Input      : HLINE ; String buffer holding section header to search for
C
C  Output     : HLINE ; unchanged
C               IFND  ; flag indicating whether the requested section has
C                       been found in the HELPFILE
C
C  Calls      : --
C
C-----------------------------------------------------------------------
C
	IMPLICIT CHARACTER *80 (H)
C
	IFND=-1
	I=81
10      I=I-1
	IF (HLINE(I:I).NE.' '.AND.I.GT.1) GOTO 10
	ILEN=I
	HTARG='%%'//HLINE(1:ILEN)
20      READ (nunit,'(A80)',END=999) HTEMP
	IF (HTEMP(1:5).EQ.HTARG(1:5)) THEN
	  IFND=1
	  RETURN
	ELSE
	  GOTO 20
	  ENDIF
999     RETURN
	END
C
C
C
C
C-----------------------------------------------------------------------
      INTEGER FUNCTION NEWBOND (i,ibo)
C-----------------------------------------------------------------------
C
C  Written by : David Clarkson  Aug 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose  : NEWBOND determines if a bond pair's coordinates have
C             already been calculated.
C
C  Input    : I      ; present pointer position in IBO-array
C             IBO    ; bond pair array
C
C  Output   : NEWBOND ; =1 if a new bond calculation is necessary
C                       (say: Bond doesn't exist yet)
C
C  Calls    : --
C
C-----------------------------------------------------------------------
C
      DIMENSION ibo(7,i)
C
      newbond=1
      i1=i-1
      DO 10 j=1,i1
	if(ibo(7,j).le.0) goto 10
	if(ibo(1,j).eq.ibo(1,i).and.ibo(2,j).eq.ibo(2,i)) then
	  newbond=0
	  return
	else if(ibo(1,j).eq.ibo(2,i).and.ibo(2,j).eq.ibo(1,i)) then
	  newbond=0
	  return
	  endif
10      continue
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ORTHO (BBTEN,rotm)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : ORTHO converts crystallographic coordinates into orthogonal
C               coordinates.
C
C  Input      : none
C
C  Output     : BBTEN  ; orthogonalization matrix
C               ROTM   ; defaulted rotation matrix
C               CELLAX ; orthogonalized corners of unit cell
C
C  Calls      : mxv
C
C  NOTE       : The atomic coordinates in YX(n,m) are changed to orthogonal
C               coordinates in Angstrom units and stored in OYX(n,m)
C
C-----------------------------------------------------------------------
C
      COMMON /CELL/  A,B,C,ALPHA,BETA,AGAMMA
      COMMON /RCELL/ AS,BS,CS,RAL,RBE,RGA
      include 'comatom.for'
      common /oatom/  nyo,myo,oyx(3,1000)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
      DIMENSION BBTEN(3,3),V(3),rotm(3,3)
C
      BBTEN(1,1)=A*SIN(BETA)
      BBTEN(1,2)=-B*SIN(ALPHA)*COS(RGA)
      BBTEN(1,3)=0
      BBTEN(2,1)=0
      BBTEN(2,2)=1/BS
      BBTEN(2,3)=0
      BBTEN(3,1)=A*COS(BETA)
      BBTEN(3,2)=B*COS(ALPHA)
      BBTEN(3,3)=C
C
C  Orthogonalize YX-array
C  ----------------------
c
      DO 150 J=1,NY
	CALL MXV(V,BBTEN,YX(1,J))
	DO 160 I=1,3
160       OYX(I,J)=V(I)
150     CONTINUE
      nyo=ny
      myo=my
C
C  Orthogonalize the 8 corners of the unit cell
C  --------------------------------------------
c
      do 170 i=1,8
	do 180 j=1,3
	  cellax(j,i)=0.0
180       if(i.gt.4) cellax(j,i)=1.0
170     continue
      cellax(1,2)=1.0
      cellax(2,3)=1.0
      cellax(3,4)=1.0
      cellax(3,5)=0.0
      cellax(2,6)=0.0
      cellax(1,7)=0.0
      do 200 i=1,8
	call mxv (v,bbten,cellax(1,i))
	do 210 j=1,3
210       cellax(j,i)=v(j)
200     continue
C
C  Default rotation matrix
C  -----------------------
c
      do 220 i=1,3
	do 220 j=1,3
	rotm(i,j)=0.0
220     if(i.eq.j) rotm(i,j)=1.0
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ORTHOC (xmax,xmin,rmax,iautfl)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : ORTHOC converts the projection volume to the screen system
C               and determines the scale
C
C  Input      : RMAX  ; maximal ionic radius occuring
C               XMAX  ; maxima of plotting ranges
C               XMIN  ; minima of plotting ranges
C
C  Output     : XMAX  ; orthogonalized values
C               XMIN  ; orthogonalized values
C
C  Calls      : mxv
C
C-----------------------------------------------------------------------
C
      COMMON /CELL/   A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
C
      DIMENSION V(3,8),VPROD(3,8),XMIN(3),XMAX(3),XM(3)
C
C  orthogonalize the 8 corners of the chosen projection volume
C  -----------------------------------------------------------
c
      V(1,1)=XMIN(1)
      V(2,1)=XMIN(2)
      V(3,1)=XMIN(3)
      V(1,2)=XMIN(1)
      V(2,2)=XMIN(2)
      V(3,2)=XMAX(3)
      V(1,3)=XMIN(1)
      V(2,3)=XMAX(2)
      V(3,3)=XMIN(3)
      V(1,4)=XMIN(1)
      V(2,4)=XMAX(2)
      V(3,4)=XMAX(3)
      V(1,5)=XMAX(1)
      V(2,5)=XMIN(2)
      V(3,5)=XMIN(3)
      V(1,6)=XMAX(1)
      V(2,6)=XMIN(2)
      V(3,6)=XMAX(3)
      V(1,7)=XMAX(1)
      V(2,7)=XMAX(2)
      V(3,7)=XMIN(3)
      V(1,8)=XMAX(1)
      V(2,8)=XMAX(2)
      V(3,8)=XMAX(3)
      DO 20 J=1,8
	CALL MXV (VPROD(1,J),BBTEN,V(1,J))
20      CONTINUE
C
C  pick out orthogonalized limits
C  ------------------------------
c
      do 30 i=1,3
	xmin(i)=100.0
30      xmax(i)=-100.0
      DO 40 I=1,3
	DO 50 J=1,8
	  IF(VPROD(I,J).LT.XMIN(I)) XMIN(I)=VPROD(I,J)
	  IF(VPROD(I,J).GT.XMAX(I)) XMAX(I)=VPROD(I,J)
50        CONTINUE
40      CONTINUE
      DO 60 I=1,3
	XM(I)=ABS(XMAX(I)-XMIN(I))
60      CONTINUE
      ixm=1
      do 70 i=1,3
70      if(xm(i).gt.xm(ixm)) ixm=i
C
C  calculate scale for iautfl = 1
C  ------------------------------
c
      if(iautfl.ne.1) return
      ascale=pixwid/(xm(ixm)+2.0*rmax)
      ypix=0.75*pixwid
      xtes=(xm(2)+2.0*rmax)*ascale
      if(xtes.gt.ypix) ascale=ypix/(xm(2)+2.0*rmax)
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE PERSP 
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : PERSP applies perspective to the ICO- and CELLAX-array
C
C  Calls      : --
C
C-----------------------------------------------------------------------
C
      COMMON /ATTRIB/ nter, pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
C
C  Perspective depends on the view distance (zoom effect)
C  ------------------------------------------------------
c
      xview=viewd*ascale
C
C  Apply perspective to radii and x-y-coordinates with respect
C  to z-level
C  -----------------------------------------------------------
c
      DO 10 J=1,nrc
	xpmu=xview/(xview-ico(3,j))
	do 20 i=1,4
	  if(i.eq.3) goto 20
	  xx=ico(i,j)*xpmu
	  ico(i,j)=xx
20        continue
10      continue
C
C  Apply perspective to cell corners
C  ---------------------------------
c
      do 30 i=9,16
	xpmu=xview/(xview-cellax(3,i))
	do 40 j=1,2
40        cellax(j,i)=cellax(j,i)*xpmu
30      continue
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE PLOT1 (ixs,nterml)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C             : Nina Heinig, August 1986
C
C  Purpose    : PLOT1 is the driver for all plotting
C
C  Input      : NTERML ; terminal type flag
C               IXS    ; vector containing the z-level sorted addresses
C                        for ICO
C
C  Calls      : atoplo, ciratom, edgplo, hihome, labplo, labsca,
C               lohome, plotbd, sclplo, scrplo, tmode, flush
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
C
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /oatom/  nyo,myo,oyx(3,1000)
      include 'comatom.for'
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /rottr/  rotm(3,3),nrot,nrro
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
      DIMENSION ixs(1000),galph(3)
      data galph/'a','b','c'/
C
      nrc=nyo
      limits(1)=1000
      limits(2)=0
      limits(3)=1000
      limits(4)=0
C
C  igrafl=0/1  local flag that (does/does not) reset to graphics mode.
c  ------------------------------------------------------------------- 
C
      igrafl = 0
      if(istefl.eq.2 .or. (istrfl.eq.1 .and. nrot.ge.2)) igrafl = 1
      if(nterml.eq.1) then
	if(igrafl .eq. 0 ) then
	  if(ivgafl.eq.0) then
	    WRITE(nter,2000)
2000        FORMAT(' +-*/DD000DADCGAABGI250600GJHI3BA',
     1    '  GRAPHICS PACKAGE for INORGANIC CRYSTALLINE STRUCTURES',
     2    '  1985/86  B.BEREZA & D.ALTERMATT +-*/ALHI7')
	  else
	    write(nout,2007)
2007        format('GRAPHICS PACKAGE for INORGANIC ',
     +       'CRYSTALLINE STRUCTURES by B.BEREZA & D.ALTERMATT')
	  endif
C        elseif(istefl .eq. 2) then
C          goto 10
	  ENDIF
      else if(nterml.eq.2) then
	if(istefl.eq.2) go to 10
	call tekon(nter)
	write(nout,2015)
2015    format(//,' Calculating virtual screen -- this may take a',
     1    ' few seconds!',//)
	ENDIF
c
c  reset colour table in case TINT was used
c  ??????? this may not be necessary because of reset mode??
c  ---------------------------------------------------------
c
      if(nterml.eq.1) then
	IF(ITINFL.eq.0.and.ivgafl.eq.0) WRITE(NTER,2035)
2035    FORMAT(1X,'JG40FF0 JG5F0F0 JG6F800 JG7FF00 JG8AFA0 ',
     +       'JG9AAF0')      
	endif
C
C
C  Save original number of bonds (NRB) and atoms (NRC),
C  add desired cell vectors
c  ----------------------------------------------------
C
10    nrbo=nrb
      nrco=nrc
      call edgplo(iaxifl,ishifl)
C
C  Draw atoms and bonds simultaneously from z-min to z-max
c  -------------------------------------------------------
C
      do 20 i=nrc,1,-1
	if(ico(6,ixs(i)).le.0) goto 20
	if(ibonfl.le.0) goto 30
	nstifl=istifl
	if(ixs(i).gt.nrco) nstifl=0
	call plotbd(ixs(i),nstifl,nter,nterml)
30      if(ixs(i).gt.nrco) goto 20
	if(nterml.eq.1 .and. iquifl.ne.1) then
	  call ciratom(ixs(i))
	else if(nterml.eq.2) then
	  call atoplo(ixs(i),istifl)
	  endif
20      continue
c
C  Reset bond and atom number
C  --------------------------
c
      if(istefl.ne.1) then
	nrb=nrbo
	nrc=nrco
	endif
C
C  Draw the scale if wanted, Tektronix: Plot virtual screen,
C  label elements, scale and lattice vectors
C  ---------------------------------------------------------
c
      if(nterml.eq.1) then
	apcm=pixwid/(25*ascale)
c        WRITE(nter,2020)
2020    format(' GI')
	IF(iscafl.EQ.1) THEN
	 IF (APCM*4.LT.100.0) THEN
	  if (ivgafl.eq.0) then
	    WRITE (nter,2030)
2030        FORMAT (' DCGAABGI675550GJHI3BA ',
     1        ' +----+----+----+----+ +-*/')
	    WRITE (nter,2040) APCM-APCM,APCM,APCM*2,APCM*3,APCM*4
2040        FORMAT (' DCGAABGI675540GJHI4BA',F4.1,1X,F4.1,1X,F4.1,1X,
     1        F4.1,1X,F4.1,' +-*/')
	    WRITE (nter,2050) apcm
2050        FORMAT(' DCGAABGI675530GJHI4BA',F4.1,' Angstroms/cm +-*/')
	    else
	      write(nter,2031)
2031          format(' +----+----+----+----+ +-*/')
	      write(nter,2032)apcm-apcm,apcm,apcm*2,apcm*3,apcm*4
2032          format(f4.1,4(1x,f4.1),' +-*/')
	      write(nter,2033)apcm
2033          format(f4.1,' Angstroms/cm +-*/')
	    endif
	   endif
	  ENDIF
      else if(nterml.eq.2) then
	if(iscafl.eq.1) call sclplo(ascale)
	call scrplo(nter)
	do 50 i=1,nrc
	  if(ico(6,i).ne.1) goto 50
	  if(iscr(ico(1,i),ico(2,i)).ne.2) goto 50
	  if(ilabfl.ne.1) goto 55
	  is=1
	  if(istifl.eq.1) is=-1
	  ip1=ico(1,i)-is*8
	  ip2=ico(2,i)+is*5
	  call labplo(ip1,ip2,gelem(ico(5,i)),nout,nter)
55        iscr(ico(1,i),ico(2,i))=0
50        continue
	if(iscafl.eq.1) call labsca(nout,nter)
	if(iaxifl.eq.1) then
	  do 60 i=10,12
	    ip1=cellax(1,i)-5.
	    ip2=cellax(2,i)+18.
	    if(cellax(1,9).lt.(cellax(1,i)-.5)) ip1=ip1+5
	    if(cellax(1,9).gt.(cellax(1,i)+.5)) ip1=ip1-5
	    if(cellax(2,9).lt.(cellax(2,i)-.5)) ip2=ip2+5
	    if(cellax(2,9).gt.(cellax(2,i)+.5)) ip2=ip2-5
	    n=i-9
60          call labplo(ip1,ip2,galph(n),nout,nter)
	  endif
	if(ishifl.eq.1) then
	  call hihome(nter)
	  call flush
	  if(istefl.ne.2) write(nout,2070)
2070      format(40X,'CAUTION: Lattice vectors shifted',/,
     1      49X,'Origin not on screen')
	  ishifl=0
	  endif
	call lohome(nter)
	call flush
	if(istefl.eq.1) return
	write(nout,2060)
2060    format('$Hit <RETURN> to continue: ')
	read(nin,1000,err=40) gdum
1000    format(A1)
40      call tmode(nter)
	call flush
	endif
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE PLOTBD (iato,istifl,nter,nterml)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Revised by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : PLOTBD will plot all bonds from current atom (iato) to
C               already drawn atoms in lower z-levels
C
C  Input      : IATO  ; number of present atom in ICO-array
C               NTER  ; terminal command stream
C               NTERML; terminal type flag
C
C  Calls      : bondline, vecplo
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
C
      common /plot/  nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1               ibo(7,1600),xbo(1600),xblim(2)
      common /ident/ nx,mx,oxid(300),npos(300),icn(300),
     1               nza(2,300),nh(300),nd(300),ialtno(300),
     2               igrp(300)
      common /bonds/ nbon,mbon,ibon(5,1000),jbon(5,1000),
     1               xbon(6,1000),ifbon(4,1000)
C
      do 10 i=1,nrb
	if(ibo(1,i).ne.iato.and.ibo(2,i).ne.iato) goto 10
	if(ibo(7,i).le.0) goto 10
	nato=ibo(2,i)
	if(ibo(2,i).eq.iato) nato=ibo(1,i)
	if(ico(6,nato).le.0) goto 10
	if(ico(3,iato).lt.ico(3,nato)) goto 10
	iclr=int(12.728*sqart(abs(xbo(i)))-2.5)
	if (iclr.gt.14) iclr=14
	if (iclr.lt.1) iclr=1
	if (xbo(i).eq.0.0) iclr=15
	if(istifl.eq.1) then
	  call stiplo(i,nter,nterml)
	else if(nterml.eq.1) then
c          print *,xbo(i),iclr
	  call bondline(ibo(1,i),nter,iclr)
	else if (nterml.eq.2) then
	  call vecplo(ibo(1,i))
	  endif
10      continue
      RETURN
      END
C
C
C
CC
C------------------------------------------------------------------------
      SUBROUTINE PLOTER(RMAX, nterml, izma, izmi)
C------------------------------------------------------------------------
C
C  Set up and revised by: Nina Heinig  July/August 1986
c  VGA chunk by: Tamon Stephen  March 1993
C
C  Filters, sorts and call plotting programs
C
C  Input: rmax = maximum atom radius
c         nterml = terminal type
c         izma = ???
C         izmi = ???
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
C  System common blocks
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,machine
      COMMON /CELL/   A(3),AL(3),VOL,AM(3,3),ICELL
      COMMON /SYM/    NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON /GSYM/   GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /oatom/  nyo,myo,oyx(3,1000)
      common /span/   nya,mya,iyatom(5,600),iterm(600),nbo,mbo,
     1                ibond(2,600),xbond(6,600),iat,bondlim(2)
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /rottr/  rotm(3,3),nrot,nrro
      common /gplot/  gcol(16)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
      common /gtitle/ gent,hname,hcodn(2),htitl(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      dimension ixz(1000),ixs(1000)
c                                    y(3),axs(3),dumrot(3,3)
C
C  Set up default flag, check for end-of-program, check for stick-ball plot.  
c  -------------------------------------------------------------------------
C
  50  idefl = 1
      if(istofl.eq.1) return
      if(istifl.eq.1) rmax=0.2
      if(iquifl.eq.1) rmax=0.1
      nrc=nyo
C
C  Entry point for default run
c  ---------------------------
C
c        
      entry plotr(rmax, nterml, izma, izmi)
c     set up screen
      if(ivgafl.ge.2.and.istefl.ne.2) then
	if(ivgafl.eq.2) then
	  vi=video(2)  
	  if (vi.eq.0) go to 3141
	else if(ivgafl.eq.3) then
	  vi=video(3)
	  if (vi.eq.0) go to 3141
	endif
	if (itinfl.ge.1) vi=shade(itinfl)
	vi=background(ibacfl)
      endif
      if (ivgafl.eq.1.and.istefl.ne.2) then
	vi=video(1)
	if (vi.eq.0) goto 3141
	vi=background(ibacfl)
      endif
C
C  Create virtual screen in ICO array with origin 
c  at centre, applying current rotation
c  ----------------------------------------------
c
      call absol(rmax)
c
      if(istefl.ne.0) then
	call sterot
      endif
c
C  Sort ICO array according to z-levels from back to front
c  -------------------------------------------------------
C
      do 70 i=1,nrc
70      ixz(i)=ico(3,i)
      nnc=nrc
      nnr=iaxifl * 4
      if((nrc+nnr).le.my) then
c
C  Add unit cell corners
c  ---------------------
c
	nnc=nrc+nnr
	do 80 i=1,nnr
	  ixz(nrc+i)=int(cellax(3,i+8))
80        continue
	endif
      do 85 i=1,8
85       ixz(nnc+i)=(0,0)
      call sortx(nnc,ixz,ixs)
C
C  Perspective applied to relative coordinates and radius
C  ------------------------------------------------------
c
      call persp
c
C  Transform to true screen origin and drop atomic
c  positions outside plotting limits (set flag in ICO)
C  ---------------------------------------------------
c
      call filter (ifilfl,nrel,rmax)
      if(nrel.le.0.and.nrc.gt.0) then
	write(nout,2080)
2080    format(//,' ++++ERROR++++  No atoms left after',
     1      ' filtering -- Try again!!',//)
	call wait
	if (ivgafl.ge.1) vi=video(0)
	RETURN
	endif
c
      if(istefl.ne.0) call stersft
C
C  Flag bonds that became useless after above filtering
C  ----------------------------------------------------
c
      CALL FILTER3 (nrel)
      if(nrel.le.0.or.nrb.le.0) ibonfl=0
c
c  Default run returns
c  -------------------
c
      IF (idefl .EQ. 0) return
      idefl = 0
C
C  Plot the molecule
C  -----------------
c
      if(nterml.eq.2 .and. istefl.ne.2) then
	call clrscr(nter)
	call tmode(nter)
	endif
      CALL PLOT1 (ixs,nterml)
      istefl=istefl+1
      if(istefl.eq.2) goto 50
      if (ivgafl.ge.2.and.itinfl.eq.1.and.(ibonfl.eq.1.or.ibonfl.eq.3)) 
     +    call cscale
C
C  Return to Alpha mode.  (Get out of graphics)
C  --------------------------------------------
c
      IF (nrot.lt.nrro .and. istrfl.eq.1)then
	istefl=0
	write(nout,4440)
4440    format('Hit return to continue...')
	read(nin,4441)gdum
4441    format(a1)
	if (ivgafl.ge.1) vi=video(0)
	RETURN
	endif
      if(nterml.eq.1.and.ivgafl.eq.0) then
	WRITE(nter,99)
99      FORMAT(' AABH')
	endif
      istefl=0
      if (iprifl.eq.1) then
	write(nout,9221)hname
9221    format('PLOT of : ',a70)
	do 9224 i=0,20000000
9224      vi=i+1+2*i
      endif
      write(nout,4440)
      read(nin,4441)gdum
      if (ivgafl.ge.1) vi=video(0)
      RETURN
3141  continue
      vi=video(0)
      write(nout,3142)
3142  format('Your graphics or monitor does not support the',
     +     /,'video mode you have selected.')
      call wait
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE RADII (RMAX)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/85
C
C  Purpose    : RADII gets the ionic radius of each individual element.
C
C  Input      : none
C
C  Output     : RMAX ; maximum radius found
C
C  Calls      : radius
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *8 (G)
C
      include 'comatom.for'
      COMMON /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
C
      RMAX=0
      DO 20 I=1,nx
	jcn=icn(i)
	IF(ICN(i).LE.1) jcn=6
	RAD(I)=RADIUS(GELEM(i),OXID(i),jcn)
	IF(RAD(I).LT.0.1) RAD(I)=0.1
	IF(RAD(I).GT.RMAX) RMAX=RAD(I)
20      CONTINUE
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ROTATE (ANG,X)
C-----------------------------------------------------------------------
C
C  Written by : David Clarkson  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/86
C
C  Purpose    : ROTATE does the necessary matrix computations involved in
C               rotating the coordinates about an axis X by ANG degrees.
C
C  Input      : ANG   ; the angle of rotation in degrees
C               X     ; the axis of rotation given as an vector triple
C
C  Calls      : mxm
C
C-----------------------------------------------------------------------
C
      common /rottr/  rotm(3,3),nrot,nrro
C
      DIMENSION X(3),R(3,3),RR(3,3),RX(3,3),RY(3,3),RZ(3,3)
      DIMENSION RXI(3,3),RYI(3,3),RZI(3,3)
C
C  Initialize Radian conversion and other values used
c  --------------------------------------------------
C
      CONV=3.141592/180.0
      V=SQRT(X(2)*X(2)+X(3)*X(3))
      VL=SQRT(X(1)*X(1)+X(2)*X(2)+X(3)*X(3))
C
C  Determine the X transformation matrix
c  -------------------------------------
C
      RX(1,1)=1.0
      RX(1,2)=0.0
      RX(1,3)=0.0
      RX(2,1)=0.0
      IF (V.EQ.0) THEN
	RX(2,2)=1.0
	RX(2,3)=0.0
	RX(3,2)=0.0
	RX(3,3)=1.0
      ELSE
	RX(2,2)=X(3)/V
	RX(2,3)=X(2)/V
	RX(3,2)=-X(2)/V
	RX(3,3)=X(3)/V
	ENDIF
      RX(3,1)=0.0
C
C  Determine the X inverse transformation matrix
c  ---------------------------------------------
C
      DO 100 I=1,3
	DO 150 J=1,3
	  RXI(I,J)=RX(I,J)
150       continue
100     continue
      RXI(2,3)=RXI(2,3)*(-1.0)
      RXI(3,2)=RXI(3,2)*(-1.0)
C
C  Determine the Y transformation matrix
c  -------------------------------------
C
      RY(1,1)=V/VL
      RY(1,2)=0.0
      RY(1,3)=X(1)/VL
      RY(2,1)=0.0
      RY(2,2)=1.0
      RY(2,3)=0.0
      RY(3,1)=-X(1)/VL
      RY(3,2)=0.0
      RY(3,3)=V/VL
C
C  Determine the Y inverse transformation matrix
c  ---------------------------------------------
C
      DO 500 I=1,3
	DO 250 J=1,3
	  RYI(I,J)=RY(I,J)
250       continue
500     continue
      RYI(1,3)=RYI(1,3)*(-1.0)
      RYI(3,1)=RYI(3,1)*(-1.0)
C
C  Determine the Z transformation matrix
c  -------------------------------------
C
      RZ(1,1)=COS(ANG*CONV)
      RZ(1,2)=SIN(ANG*CONV)
      RZ(1,3)=0.0
      RZ(2,1)=-SIN(ANG*CONV)
      RZ(2,2)=COS(ANG*CONV)
      RZ(2,3)=0.0
      RZ(3,1)=0.0
      RZ(3,2)=0.0
      RZ(3,3)=1.0
C
C  Determine the overall transformation matrix
c  -------------------------------------------
C
      CALL MXM (Rr,RX,RY,3)
      CALL MXM (R,Rr,RZ,3)
      CALL MXM (Rr,R,RYI,3)
      CALL MXM (R,Rr,RXI,3)
C
C  Transform the stored rotation matrix
c  ------------------------------------
C
      call mxm (rr,r,rotm,3)
C
C  Store the matrix derived
c  ------------------------
C
      DO 300 i=1,3
	do 300 j=1,3
300     rotm(i,j)=rr(i,j)
      RETURN
      END
C
C
C
C
C-----------------------------------------------------------------------
      subroutine spatre(hcom,*)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Load the spanning tree to YX- and IBO-array for plotting.
C     Return to * with comment HCOM if there could be problems.
C
C     Stick-bond plot selected
C
C     calls: bost, vminv, xleng
C----------------------------------------------------------------------
C
      implicit character *80 (h)
      implicit character *8  (g)
C
C  System common blocks
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,machine
      COMMON /SYM/    NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON /GSYM/   GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      include 'comatom.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),nbo,mbo,
     1                ibond(2,600),xbond(6,600),iat,bondlim(2)
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
C
      dimension l(3),x(3)
C
C  Check input data
C  ----------------
c
      if(nya.le.0) then
	hcom='(//,'' ERROR in common /span/: no atoms given!!'',/)'
	return 1
      else if(nbo.le.0) then
	hcom='(//,'' ERROR in common /span/: no bonds given!!'',/)'
	return 1
	endif
C
C  Calculate coordinates for common /yatom/
C  ----------------------------------------
c
      do 10 i=1,nya
	if(iyatom(2,i).gt.0) then
	  iynum(i)=iyatom(1,i)
	  iysym(i)=iyatom(2,i)
	  call symopl(yx(1,i),s(1,1,iysym(i)),t(1,iysym(i)),
     1      iyatom(3,i),xx(1,iyatom(1,i)))
	else
	  iynum(i)=iyatom(1,iyatom(1,i))
	  iysym(i)=iyatom(2,iyatom(1,i))
	  do 30 j=1,3
30          l(j)=iyatom(j+2,i) + iyatom(j+2,iyatom(1,i))
	  call symopl(yx(1,i),s(1,1,iysym(i)),t(1,iysym(i)),
     1      l,xx(1,iyatom(1,iyatom(1,i))))
	  endif
10      continue
C
C  Set window
C  ----------
c
      do 40 i=1,3
	window(i,1)=100.
40      window(i,2)=-100.
      ny=nya
      do 50 i=1,ny
	do 50 j=1,3
	  if(yx(j,i).gt.window(j,2)) window(j,2)=yx(j,i)
	  if(yx(j,i).lt.window(j,1)) window(j,1)=yx(j,i)
50        continue
C
C  Make sure it's all on screen
C  ----------------------------
c
      do 60 i=1,3
	window(i,1)=window(i,1)-0.2
60      window(i,2)=window(i,2)+0.2
C
C  Store bonds to IBO
C  ------------------
c
      mrb=1600
      nrb=nbo
      xblim(1)=-15.0
      xblim(2)=5.0
      do 70 i=1,nrb
	ibo(1,i)=ibond(1,i)
	ibo(2,i)=ibond(2,i)
	ibo(7,i)=1
	call vminv(x,yx(1,ibo(2,i)),yx(1,ibo(1,i)))
	xbo(i)=xleng(x(1),x(2),x(3),0)
	call bost(iynum(ibo(1,i)),iynum(ibo(2,i)),xval,xer1,xer2,xbo(i))
	if(xval.lt.0.005) then
	  xbo(i)=-xbo(i)
	else
	  xbo(i)=xval
	  endif
70      continue
      istifl=1
      return
      end
C
C
C-----------------------------------------------------------------------
      subroutine sterot
C-----------------------------------------------------------------------
C  Written by : Liz Brown
C
C  Purpose    : rotates the virtual screen in ICO-, CELLAX- and PRANGE-array
C               for stereo viewing
C
C  Calls      : orthoc,vminv,mxv
C
C-----------------------------------------------------------------------
c
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
C  System common blocks
C
      common /oatom/  nyo,myo,oyx(3,1000)
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
c
c  rotate to give perspective
c  assume distance between eyes = eyed = 6.5 cm, screen distance = 45 cm  
c  ANG = arctan((eyed/2)/screend) (in degrees)
c  ----------------------------------------------------------------------
c
      CONV=3.141592/180.0
      if(istefl.eq.1) then
	ANG = 4.13
      elseif(istefl.eq.2) then
	ANG = -8.26
	endif
c
c  Rang is the angle of rotation in radians
c
      rang=ang*conv
      cosang=cos(rang)
      sinang=sin(rang)
C
C  Calculate screen coordinates for plotting ranges and update scale
C  -----------------------------------------------------------------
c
      do 70 i=3,4
	px=prange(1,i)
	pz=prange(3,i)
	prange(1,i)=nint(px*cosang+pz*sinang)
	prange(3,i)=nint(px*sinang+pz*cosang)
70      continue
C
C  Calculate initial screen coordinates for atomic positions
C  ---------------------------------------------------------
c
      nrc=nyo
      DO 20 i=1,nyo
	cx=(ico(1,i))
	cz=(ico(3,i))
	ico(1,i)=nint(cx*cosang+cz*sinang)
	ico(3,i)=nint(cx*sinang+cz*cosang)
20      CONTINUE
C
C  Calculate initial screen coordinates for unit cell corners
C  ----------------------------------------------------------
c
      do 40 i=1,8
	n=i+8
	tcx=(cellax(1,n))
	tcz=(cellax(3,n))
	cellax(1,n)=nint(tcx*cosang+tcz*sinang)
	cellax(3,n)=nint(tcx*sinang+tcz*cosang)
40      continue
      RETURN
      END
c*********************************************************
c-----------------------------------------------------------------------
C
c
c
c
c
c
C-----------------------------------------------------------------------
      subroutine stersft
C-----------------------------------------------------------------------
C  Written by : Liz Brown
C
C  Purpose    : shifts the virtual screen in ICO-, CELLAX- and PRANGE-array
C               after rotation for stereo viewing
C
C  Calls      : vminv
C
C-----------------------------------------------------------------------
c
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
C  System common blocks
C
      common /oatom/  nyo,myo,oyx(3,1000)
C
C  Plot package common blocks
C
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /attrib/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      dimension xyz1(3),xyz2(3),rico(3,1000),plosft(3)
c
c  translate to left half of screen
c  --------------------------------
C
      if(istefl.eq.1) then
	plosft(1) = orgsft(1) + pixwid/5
	endif
C
c  translate to right half of screen
c  ---------------------------------
c
      if(istefl.eq.2) then
	plosft(1) = plosft(1) - pixwid*2/5
	endif
C
C  Calculate screen coordinates for plotting ranges and update scale
C  -----------------------------------------------------------------
c
      do 70 i=3,4
	call vminv (xyz1,prange(1,i),plosft)
	do 80 j=1,3
80        prange(j,i)=xyz1(j)
70      continue
      do 90 i=1,3
	if(prange(i,4).lt.prange(i,3)) then
	  xyz2(i)=prange(i,3)
	  prange(i,3)=prange(i,4)
	  prange(i,4)=xyz2(i)
	  endif
90      continue
C
C  Calculate initial screen coordinates for atomic positions
C  ---------------------------------------------------------
c
      nrc=nyo
      DO 20 i=1,nyo
	do 25 j=1,3
25        rico(j,i)=ico(j,i)*1.0
	call vminv(xyz1,rico(1,i),plosft)
	do 30 j=1,3
30        ico(j,i)=nint(xyz1(j))          
20      CONTINUE
C
C  Calculate initial screen coordinates for unit cell corners
C  ----------------------------------------------------------
c
      do 40 i=1,8
	n=i+8
	call vminv (xyz1,cellax(1,n),plosft)
      do 50 j=1,3
50      cellax(j,n)=nint(xyz1(j))*ascale   
40      continue
      RETURN
      END
c*********************************************************
c-----------------------------------------------------------------------
C
C-----------------------------------------------------------------------
      subroutine stiplo(ip,nter,nterml)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Calculates the parameters to draw Stick Bonds
C
C     Calls : bondline, vecplo
C-----------------------------------------------------------------------
C
      common /plot/  nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1               ibo(7,1600),xbo(1600),xblim(2)
      common /ident/ nx,mx,oxid(300),npos(300),icn(300),
     1               nza(2,300),nh(300),nd(300),ialtno(300),
     2               igrp(300)
      common /bonds/ nbon,mbon,ibon(5,1000),jbon(5,1000),
     1               xbon(6,1000),ifbon(4,1000)
C
      dimension ellip(8,2),ivect(7)
C
      data ellip / 4.,3.,0.,-3.,-4.,-3.,0.,3.,
     2             0.,-3.,-4.,-3.,0.,3.,4.,3. /
C
C  Calculate distances of atoms involved
C  -------------------------------------
c
      delx = ico(1,ibo(2,ip)) - ico(1,ibo(1,ip))
      dely = ico(2,ibo(2,ip)) - ico(2,ibo(1,ip))
      dist = delx**2 + dely**2
      if(dist.gt.0.0001) dist = sqrt(dist)
      ih = 1
      il = 2
      high = ico(3,ibo(ih,ip)) - ico(3,ibo(il,ip))
      if(high.lt.0.0) then
	ih = 2
	il = 1
	endif
      if(abs(high).gt.50.) then
	rat = abs(dist/high)
      else
	rat = 100.
	endif
C
C  Choose correct radius at both ends
C  ----------------------------------
c
      ih = 2 * ih + 1
      il = 2 * il + 1
      xfac = .8
      xxfac = 4.
      if(rat.gt.99.) then
	xfac = 2.
	xxfac = 2.
	endif
C
C  Calculate coordinates and draw composing vectors
C  ------------------------------------------------
c
      do 10 i=1,8
	ivect(3) = ibo(ih,ip) + integer((ellip(i,1)/xfac),0)
	ivect(4) = ibo(ih+1,ip) + integer((ellip(i,2)/xfac),0)
	ivect(5) = ibo(il,ip) + integer((ellip(i,1)/xxfac),0)
	ivect(6) = ibo(il+1,ip) + integer((ellip(i,2)/xxfac),0)
	ivect(7) = ibo(7,ip)
	if(nterml.eq.1) then
	  iclr=int(12.728*sqart(abs(xbo(ip)))-2.5)
	  if (iclr.gt.14) iclr=14
	  if (iclr.lt.1) iclr=1
	  if (xbo(ip).eq.0.0) iclr=15
c          print *,iclr,xbo(ip)
	  call bondline(ivect,nter,iclr)
	else if(nterml.eq.2) then
	  call vecplo(ivect)
	  endif
10      continue
      return
      end
C
C
c--------------------------------------------------------------------
      SUBROUTINE PLOHELP
c--------------------------------------------------------------------
c
c  Written by:  Tamon Stephen, April 6, 1993
c  Purpose:  Provides information on starting the plot routines.
c
c
      common /files/ nin,nout,ndebug,nread,nwrite,nspgp,machine
100   write (nout,1000)
1000  format(/////,'                     Structure Plotting',//,
     +5x,'      In this version of Strumo, there is a flexible set',/,
     +5x,'of graphics routines.')  
      write(nout,1010)
1010  format(' ',
     +5x,'     Before you enter the plotting routines, you should',/,
     +5x,'calculate and store the bonds (Menu level 3, option 13).')
      write(nout,1020)
1020  format(' ',
     +5x,'     If you intend to use the VGA and SuperVGA monitor',/,
     +5x,'modes (4-6) you should have installed Strumo with the',/,
     +5x,'Watcom F77 Graphics library in your "include" path, and',/,
     +5x,'compiled and included W77PLOT.OBJ instead of DUDPLOT.OBJ',/,
     +5x,'A message at the top of the plot menu will tell you if',/,
     +5x,'your version of Strumo has VGA compatibility.')
      write(nout,1030)
1030  format(' ',
     +5x,'     It is best to use the highest resolution you can',/,
     +5x,'support, and the 256 colour modes provide a number of',/,
     +5x,'extra options.  The computer can sense which VGA graphics',/,
     +5x,'modes are supported, and if you select a graphics mode',/,
     +5x,'which is not available, it will give you a message to that',/,
     +5x,'effect.',///)
c
      call pageflip(*200,*100,*9999)
c
200   write(nout,2000)
2000  format(///,
     +5x,'      The Lundy/Sigmex and Tektronix 4010/Pericom',/, 
     +5x,'7000 monitor options are somewhat dated, and may',/,
     +5x,'have been slightly damaged by the conversion to VGA.',/,
     +5x,'If this is the case, the problems are in plotlb.for,',/,
     +5x,'and the comments in the source code may assist you in',/,
     +5x,'trying to repair it.  We, unfortunatly, can''t repair',/,
     +5x,'these routines as the terminals are not available.')
      write(nout,2005)
2005  format(' ',
     +5x,'     Once you have selected the monitor, you will be asked',/,
     +5x,'for the limits, in unit cell fractions, of the plotting',/,
     +5x,'region.  The larger the volume you plot, the smaller and',/,
     +5x,'the atoms.  However, even one unit cell may not')
      write(nout,2008)
2008  format(
     +5x,'show all bonds.  Depending on the structure you are',/,
     +5x,'examining, the unit cell may contain anywhere from eight',/,
     +5x,'to several hundred atoms.  If the spanning tree has been',/,
     +5x,'expanded, you may also be asked if you would like to do a',/,
     +5x,'normal or spanning tree plot.  The normal option is safer.')
      write(nout,2010)
2010  format(' ',
     +5x,'     It should be noted that the plotting utility can',/,
     +5x,'expand the picture, but will not include any atoms',/,
     +5x,'outside of the original limits.',//)
c
      call pageflip(*300,*100,*9999)
c
300   write(nout,3000)
3000  format(///////////,' ',
     +5x,'      After you have selected the limits, the program will',/,
     +5x,'plot the structure with default settings.  Hitting <ENTER>',/,
     +5x,'will bring you to the Graphical Language Utility, which',/,
     +5x,'gives the GLU prompt, GLU>.  Type HELP to list the',/,
     +5x,'recognized commands, and HELP (command) for information',/,
     +5x,'about each one, for example HELP BONDS.')
      write(nout,3010)
3010  format(' ',
     +5x,'      The commands PLOT, STEREO and PRINTER will generate',/,
     +5x,'a new picture, hitting <ENTER> will return you to the GLU',/, 
     +5x,'prompt.'////////)
      call pageflip(*9999,*200,*9999)
9999  return
      end
C
C
C
C     LUNDY/SIGMEX PLOTTING ROUTINES
C     ==============================
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE ciratom (iato)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
C  Updated by : Daniel Altermatt, Winter 1985/86
c  mauled by : Tamon Stephen, Spring 1993
C
C  Purpose    : Draws the coloured ciratom representing present atom
C               on the Lundy/Sigmex* terminal
c               * VGA now. L/S doubtful.
C
C  Input      : IATO  ; number of present atom in ICO-array
C
C  Calls      : primer??, circle, charplo, lowcase, tint, upcase
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER *80 (H)
      IMPLICIT CHARACTER *8  (G)
C
C
      include 'comatom.for'
C
C  Plot package common blocks
C
      COMMON /ATTRIB/ nter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /rottr/  rotm(3,3),nrot,nrro
      
C
      KK=0
c set radius. Note that all pixel adresses must have 100 added and that
c the radius must be followed by a zero digit for circles
c problems will arise in colour filling if the circle has a radius smaller
c than 4 pixels
      icox=ico(1,iato)
      icoy=ico(2,iato)+30
      if (istefl.eq.2) icox=icox+int(pix)
      if (ivgafl.ge.1) goto 600
      irad=10*(ico(4,iato)+100)
      IF (istifl.EQ.1.or.irad.lt.1050) irad=1050
c pen up
      write(nter,'('' GI'')')
c clear hidden atoms
c select grey
      write(nter,'('' HIE'')')
c draw grey circle
      write(nter,4) icox,icoy,irad+10
c fill grey circle (with grey)
      write(nter,6) icox+1,icoy+1
c select black
      write(nter,'('' HI0'')')
c draw black outline around atom
c draw black circle
      write(nter,4) icox,icoy,irad+10
c draw white atom filled with appropriate colour
c select white
      write(nter,'('' HIF'')')
c draw white circle of given radius
      WRITE (nter,4) icox,icoy,irad
C check on strobe flag
      IF (ISTRfl.EQ.1) THEN
	IF (nrot.LT.nrro.AND.nrro.NE.0) THEN
	  WRITE(nter,12)
	  GOTO 98
	  ENDIF
	ENDIF
C
c
C  set selected colour
c  -------------------
c
       goto (201,202,203,204,205,206,207,208,209,210,211,212,213,
     1     214,215,216) icol(ico(5,iato))
c
c white
c
201    write(nter,11)
11     format(' HIF')
       goto 98
c black
c
202    write(nter,12)
12     format(' HI0')
       goto 98
c blue
c
203    write(nter,13)
13     format(' HI3')
       goto 98
c 
c cyan
204    write(nter,14)
14     format(' HI2')
       goto 98
c orange
c
205    write(nter,15)
15     format(' HI4')
       goto 98
c yellow
c
206    write(nter,16)
16     format(' HI6')
       goto 98
c light green
c
207    write(nter,17)
17     format(' HI7')
       goto 98
c green
c
208    write(nter,18)
18     format(' HI8')
       goto 98
c dark green
c
209    write(nter,19)
19     format(' HI1')
       goto 98
c dark red
c
210    write(nter,20)
20     format(' HIB')
       goto 98
c magenta
c
211    write(nter,21)
21     format(' HID')
       goto 98
c grey
c
212    write(nter,22)
22     format(' HI5')
       goto 98
c pink
c
213    write(nter,23)
23     format(' HIE')
       goto 98
c light blue
c
214    write(nter,24)
24     format(' HIA')
       goto 98
c dark blue
c
215    write(nter,25)
25     format(' HI9')
       goto 98
c black
c
216    write(nter,26)
26     format(' HIC')
       goto 98
c
c  fill white circle with colour
c  -----------------------------
c
98    IF(ITINFL .ge. 1) THEN
	CALL TINT(ICOX,ICOY,IRAD,NTER,iato,ivgafl)
	GOTO 100
	ENDIF
      WRITE (nter,5)icox+1,icoy+1
c
c  pen up
c  ------
c
      write(nter,'('' GI'')')
c
c  label if flagged
c  ----------------
c
100     if(ilabfl.eq. 1) then
	write(nter,250) icox,icoy, gelem(ico(5,iato))
250     format(' DCGAABGI',2I3,'GJHI3BA',A2,' +-*/')
	endif
C
c  draw white circle
c  -----------------
c
    4 FORMAT(1X,2I3,'JM',I4)
c
c  fill white circle with selected colour
c  --------------------------------------
c
    5 FORMAT(1X,2I3,' JKF')
c
c  fill grey circle with selected colour
c  -------------------------------------
c
    6 FORMAT(1X,2I3,' JKE')
C
      RETURN
c
c Comments for Lundy/Sigmex routine above apply
600   irad=ico(4,iato)
      if (istifl.eq.1) then irad=1
      if (itinfl.eq.0) vi=circle(1,icox,icoy,irad+1,0)
      if (itinfl.eq.0) vi=circle(1,icox,icoy,irad,15)
      vi=circle(1,icox,icoy,irad-1,icol(ico(5,iato)))
      ig=1
      if (itinfl.eq.2.or.ibacfl.ge.10.or.ibacfl.eq.7) ig=0
      IF(ITINFL .ge. 1) CALL TINT(ICOX,ICOY,IRAD,NTER,iato,ivgafl,ig)
      if (ilabfl.eq.1) then
	if (icol(ico(5,iato)).eq.8.or.icol(ico(5,iato)).eq.2) then
	  icolour=15
	else
	  icolour=8
	endif
	if (itinfl.ne.0) icolour=0
	gg1=gelem(ico(5,iato))(1:1)
	gg2=gelem(ico(5,iato))(2:2)
	call upcase(gg1)
	call lowcase(gg2)
	gg=gg1(1:1)//gg2(1:1)
	vi=charplo(irad/3,irad/3,icox-irad/6,icoy-irad/6,2,
     +     gg(1:2),icolour)
      endif
      END
C
C
C-----------------------------------------------------------------------
      SUBROUTINE TINT(ICOX,ICOY,IRAD,NTER,iato,ivgafl,ig)
C-----------------------------------------------------------------------
C
C  Written by:  Nina Heinig, August 14,1986
c  vgaized by:  Tamon Stephen, March 15,1993
C
C  Purpose:  To create a 3-D effect on the atom balls by varying
c            the shading as one moves to the middle.
c
c  Input:  icox; x position on the screen
c          icoy; y position on the screen
c          irad; radius
c          nter; terminal number  
c
c   Calls: circle, glint
C
C  Set up different tints (of blue) in the colour transformation table
c  -------------------------------------------------------------------
C
c
      COMMON /ATTRIB/ ntry,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)

      if (ivgafl.eq.0) WRITE(NTER,100)
 100  FORMAT(1X,' JG9AAF0 JG877F0 JG755F0 JG633F0 JG500F0 JG400A0')
C
C  Set up concentric rings of varying shades of blue
c  -------------------------------------------------
C
      icp=icol(ico(5,iato))
      IRRAD = IRAD/2
      if (ivgafl.eq.0) then
	DO 10 I=4,9
	if (irad.le.2) goto 20
c
C  Draw circle in appropriate colour
c  ---------------------------------
c

	  WRITE(NTER,200)  I,ICOX,ICOY,IRAD
 200      FORMAT(1X,' HI',I1,2I3,'JM',I4)
c
C  Fill in circle
c  --------------
c
	  WRITE(NTER,300) ICOX+1,ICOY+1,I,I
 300      FORMAT(1X,2I3,' HI',I1,' JK',I1)
c
C  Reduce radius size
c  ------------------
c
	IRAD = IRAD - (I-2)
  10    CONTINUE
      else if (ivgafl.ge.2) then
	vi=circle(1,icox,icoy,2*irrad+1,16*icp+15)
	do 11 i=14,0,-1
	  irad=2*irrad*(2*i+1)/(15+i)
	  if (irad.ge.2) then
c           print *,i,irad,16*icp+i
	    vi=circle(1,icox,icoy,irad,16*icp+i)
	  endif
  11    continue
c      glint (at along % of radius away from center, angle ang around)
	if (irrad.ge.3) then
	  ang=0.5
	  along=0.4
	  ixpt=icox+int(2*irrad*along*cos(ang))
	  iypt=icoy-int(2*irrad*along*sin(ang))
	  ilng=irrad/3
	  if (ig.eq.1) vi=glint(ixpt,iypt,ilng,15)
	endif
      endif
c
C  Draw in white accent
c  --------------------
c
c  **THIS PART DOES NOT WORK. IT INVERTS THE SCREEN AND MAKES A MESS.
c
C  20  IAX = ICOX+IRRAD
C      IAY = ICOY+IRRAD
C      WRITE(NTER,400) IAX,IAY,IAX+2,IAY+2
C 400  FORMAT(1X,2I3,' HIF FH BG',2I3)
c
C  Reset radius value
c  ------------------
c
  20  IRAD = IRRAD*2
      RETURN
      END
C      
C
C
C
C-----------------------------------------------------------------------
      SUBROUTINE bondline (ivect,nter,iclr)
C-----------------------------------------------------------------------
C
C  Written by : Bill Bereza  Summer 1985
c  VGA bit : Tamon Stephen  Spring 1993
C
C  Purpose    : bondline will draw a line from ivect(3),ivect(4) to
C               ivect(5),ivect(6) on a LUNDY/SIGMEX* terminal
c                  *and VGA
C
C  Input      : IVECT   ; current line of IBO-array
C               NTER    ; terminal command stream
C
C  Output     : a line is drawn on the screen
C
C  Calls      : line
C
C-----------------------------------------------------------------------
C
      COMMON /ATTRIB/ nnter,pixwid,ascale,dscale,viewd,window(3,4),
     1                orgsft(3),icol(100),bbten(3,3),prange(3,4),pix
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /rottr/  rotm(3,3),nrot,nrro
      dimension ivect(7)
C
      iclr=iclr-1
      ix1=ivect(3)
      iy1=ivect(4)+30
      ix2=ivect(5)
      iy2=ivect(6)+30
      if (istefl.eq.2) then
	ix1=ix1+int(pix)
	ix2=ix2+int(pix)
      endif
      ia=1
      if (ivgafl.ge.2.and.ibonfl.eq.3.and.itinfl.ne.0.and.iclr.le.13) 
     +    ia=(iclr+9)/6
      ia1=0
      if (ia.ge.3) ia1=-1
      ia2=ia-1
      if (ia.ge.3) ia2=1
      if ((ibacfl.ge.10.or.ibacfl.eq.7).and.iclr.lt.14.
     +    and.itinfl.ne.0) iclr=13-iclr
      if (ivgafl.eq.0) then
c select black
	write(nter,'('' HI0'')')
c draw broad black background line
	write(nter,100) ix1-1,iy1-1,ix2-1,iy2-1
	write(nter,100) ix1-1,iy1+1,ix2-1,iy2+1
	write(nter,100) ix1+1,iy1-1,ix2+1,iy2-1
	write(nter,100) ix1+1,iy1+1,ix2+1,iy2+1
      else
	if (istifl.eq.0) then
	  ii=0
	else
	  if (itinfl.eq.0) ii=ibclfl
	endif
	i=line(ix1+ia2+1,iy1-ia2-1,ix2+ia2+1,iy2-ia2-1,ii)
	i=line(ix1+ia2+1,iy1+ia2+1,ix2+ia2+1,iy2+ia2+1,ii)
	i=line(ix1+ia1-1,iy1-ia1+1,ix2+ia1-1,iy2-ia1+1,ii)
	i=line(ix1+ia1-1,iy1+ia1-1,ix2+ia1-1,iy2+ia1-1,ii)
      endif
      if(istrfl.eq.1 .and. nrot.lt.nrro) then
C select pink for silhouette picture
	if (ivgafl.eq.0) then
	  write(nter,'('' HIA'')')
	else
	  ii=ibclfl
	  if (itinfl.ne.0) ii=16*(11-iclr/2)+14-2*((iclr+1)/2)
	  if (itinfl.ne.0.and.iclr.eq.14) ii=15
	endif
      else
c select red
      if (ivgafl.eq.0) then
	WRITE(nter,'('' HI3'')')
      else   
	ii=ibclfl
	if (itinfl.ne.0) ii=16*(11-iclr/2)+14-2*((iclr+1)/2)
	if (itinfl.ne.0.and.iclr.eq.14) ii=15
      endif
      endif
      if (ibonfl.eq.2) ii=ibclfl
c draw bond on the black background
      if (ivgafl.eq.0) then
	write(nter,100) ix1,iy1,ix2,iy2
  100   format('ix1,iy1',2I3,'ix2,iy2',2I3,'GI')
      else
	do 116 jr=ia1,ia2
	  i=line(ix1+jr,iy1-jr,ix2+jr,iy2-jr,ii)
116       i=line(ix1+jr,iy1+jr,ix2+jr,iy2+jr,ii)
      endif
      return
      END
C
c-----------------------
      subroutine CSCALE
c-----------------------
c
c Written by: T. Stephen, April 1993    
c Purpose: Draws up a little scale telling what valence each bond
c          colour represents
c
c Calls: charplo
c
      implicit character*8 (g)
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
c
      maxv=480
      if (ivgafl.eq.3) maxv=768
      maxh=640
      if (ivgafl.eq.3) maxh=1024
      ii=charplo(maxv/50,maxh/80,0,48*maxv/50,4,'BOND',15)
      ii=charplo(maxv/50,maxh/80,0,49*maxv/50,5,'SCALE',15)
      ii=charplo(maxv/50,maxh/80,19*maxh/20,49*maxv/50,4,'V.U.',15)
      do 10 i=0,13
	write(gbuf,1000)((i+3.5)/9.0)**2/2
1000    format('<',f6.3)
	i1=i
	if (ibacfl.ge.10.or.ibacfl.eq.7) i1=13-i1
	ic=16*(11-i1/2)+14-2*((i1+1)/2)
10      ii=charplo(maxv/40,maxh/128,(i+1)*maxh/16,39*maxv/40,8,gbuf,ic)
      return
      end
c
c----------------------
      SUBROUTINE COLDIS
c----------------------
c
c  Written by:  T. Stephen, April 15th, 1993
c  Purpose:  Shows available colours
c
c  calls charplo, video, shade, wait, box
      implicit character*8 (g)
      common /flag/   ijoyfl,iscafl,istofl,ifilfl,istifl,icolfl,
     1                ibonfl,istrfl,ispafl,iaxifl,ilabfl,ishifl,ibacfl,
     1                iautfl,istefl,iquifl,itinfl,ivgafl,ibclfl,iprifl
      common /gplot/ gcol(16)
      ix=video(ivgafl)
      maxv=480
      if (ivgafl.eq.3) maxv=768
      maxh=640
      if (ivgafl.eq.3) maxh=1024
      if (itinfl.gt.0) ix=shade(itinfl)
      do 10 i=0,3
	do 10 j=0,3
	  ix=4*j+i
	  if (itinfl.gt.0) ix=16*ix
	  vi=box(1,i*maxh/4+maxh/16,j*maxv/4+maxv/16,
     +           (i+1)*maxh/4-maxh/16,(j+1)*maxv/4-maxv/16,ix)
	  vi=charplo(maxv/8,maxh/64,i*maxh/4+maxh/16,
     +               j*maxv/4+maxv/16,8,gcol(4*j+i),0)
10    continue
      call wait
      ix=video(0)
      return
      end
C
C
C
C     TEKTRONIX/PERICOM PLOTTING ROUTINES
C     ===================================
C
C
C
C-----------------------------------------------------------------------
      subroutine tekon(nter)
C-----------------------------------------------------------------------
c     written 88.5.19 by I.D.Brown  
c
c     sets Tektronix terminal to receive TEK commands
c     NTER : Terminal command stream
c
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,
     1                n177,n45,n60,n62
      call outch(nter,n33)
      call outch(nter,n45)
      call outch(nter,n41)
      call outch(nter,n60)
      call flush
      return
      end
c
C-----------------------------------------------------------------------
      subroutine tekoff(nter)
C-----------------------------------------------------------------------
c     written 88.5.19 by I.D.Brown  
c
c     sets Tektronix terminal to receive TEK commands
c     NTER : Terminal command stream
c
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,
     1                n177,n45,n60,n62
      call outch(nter,n33)
      call outch(nter,n45)
      call outch(nter,n41)
      call outch(nter,n62)
      call flush
      return
      end
c
C-----------------------------------------------------------------------
      subroutine gmode(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Switch Tektronix/Pericom to graphics (vector) mode
C     NTER : Terminal command stream
C
C     Calls: outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
c   calls to outch used on Nord computer
      call outch(nter,0)
      call outch(nter,n35)
      call outch(nter,0)
      return
      end
C
C
C
C
C----------------------------------------------------------------------
      subroutine amode(nter)
C----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Switch Tektronix/Pericom to alpha mode
C     NTER : Terminal command stream
C
C     Calls: outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call outch(nter,0)
      call outch(nter,n37)
      call outch(nter,0)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine pmode(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Switch Tektronix/Pericom to special point mode
C     NTER : Terminal command stream
C
C     Calls: gmode, outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call gmode(nter)
      call outch(nter,n34)
      call outch(nter,0)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine tmode(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Switch Tektronix/Pericom to text mode
C     NTER : Terminal command stream
C
C     Calls: amode, outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call amode(nter)
      call outch(nter,n30)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine clrscr(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Clear Tektronix/Pericom graphics page
C     NTER : Terminal command stream
C
C     Calls: amode, gmode, outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
      call gmode(nter)
      call outch(nter,n33)
      call outch(nter,n14)
      call amode(nter)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine lohome(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Bring cursor home to dawn left corner of screen on Tektronix/
C     Pericom
C     NTER : Terminal command stream
C
C     Calls: amode, pmode, outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call pmode(nter)
      call outch(nter,n41)
      call outch(nter,n140)
      call outch(nter,n40)
      call outch(nter,n100)
      call amode(nter)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine hihome(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, University of Lausanne
C
C     Bring cursor to up left corner of screen on Tektronix/Pericom
C     NTER : Terminal command stream
C
C     Calls: amode, pmode, outch
C-----------------------------------------------------------------------
C
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call pmode(nter)
      call outch(nter,n67)
      call outch(nter,n177)
      call outch(nter,n40)
      call outch(nter,n100)
      call amode(nter)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine atoplo(iato,istifl)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/85
C
C     Plots atom(iato) as defined in ICO-array to virtual screen
C     NTER : Terminal command stream
C
C     Calls: MATHLB sqart
C-----------------------------------------------------------------------
C
      common /plot/   nrc,nrb,mrb,ico(6,1000),rad(100),radmul(100),
     1                ibo(7,1600),xbo(1600),xblim(2)
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
C  In case of stick bond network use fix radius of 5 pixels
C  --------------------------------------------------------
c
      n=iato
      irad=ico(4,n)
      if(istifl.eq.1) irad=5
C
C  i**2 + j**2 = irad**2, centred at ico(i)/ico(j)
C  -----------------------------------------------
c
      l1=ico(1,n)-irad-7
      if(l1.lt.1) l1=1
      if(l1.lt.limits(1)) limits(1)=l1
      l2=ico(1,n)+irad+7
      if(l2.gt.1000) l2=1000
      if(l2.gt.limits(2)) limits(2)=l2
      l3=ico(2,n)-irad-7
      if(l3.lt.1) l3=1
      if(l3.lt.limits(3)) limits(3)=l3
      l4=ico(2,n)+irad+7
      if(l4.gt.1000) l4=1000
      if(l4.gt.limits(4)) limits(4)=l4
      do 20 i=l1,l2
	do 30 j=l3,l4
	  ip=(i-ico(1,n))**2 + (j-ico(2,n))**2
	  p=ip
	  if(ip.gt.0) ip=sqart(p)
C
C  First paint black to clear overlapped area
C  ------------------------------------------
c
	  if(ip.gt.(irad+4)) goto 30
	  iscr(i,j)=0
C
C  Draw circles of thickness 3 pixels
C  ----------------------------------
c
	  if(ip.gt.(irad-1).and.ip.lt.(irad+1)) iscr(i,j)=1
30        continue
20      continue
      iscr(ico(1,n),ico(2,n))=2
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine vecplo(ivect)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/85
C
C     Draw a bond (vector) to virtual screen for Tektronix/Pericom
C     and use dashing type IVECT(7) (mod 5)
C     IVECT: current line of IBO-array
C
C     Calls: integer
C-----------------------------------------------------------------------
C
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
     
      dimension ivect(7)
C
      if(ivect(7).le.0) return
C
C  Set up line equation  Y = xm*X + xa
C  -----------------------------------
c
      xmx=(ivect(5)-ivect(3))
      xmy=(ivect(6)-ivect(4))
      if(abs(xmx).gt.(abs(xmy)+0.0001)) then
	xm=xmy/xmx
	xa=ivect(4)-xm*ivect(3)
	i1=ivect(3)
	i2=ivect(5)
	idirfl=1
      else if(abs(xmy).gt.(abs(xmx)+0.0001)) then
	xm=xmx/xmy
	xa=ivect(3)-xm*ivect(4)
	idirfl=2
	i1=ivect(4)
	i2=ivect(6)
	endif
      i3=1
      if(i1.gt.i2) i3=-1
      if(i1.lt.1.or.i2.gt.1000) return
      if(i1.gt.1000.or.i2.lt.1) return
C
C  Dashing
C  -------
c
      k=ivect(7)
      if(k.lt.1) return
      if(k.gt.5) k=1
C
C  Calculate trace
C  ---------------
c
      do 10 i=i1,i2,i3
	xj=xm*i+xa
	j=integer(xj,0)
	if(idirfl.eq.1) iscr(i,j)=0
	if(idirfl.eq.2) iscr(j,i)=0
	goto (20,30,40,50,60) k
C
C       k = 1 ... solid line (default)
C           2 ... dashed line
C           3 ... dot-dashed line
C           4 ... short-dashed line
C           5 ... long-dashed line
C
30      jj=mod(i,4)
	if(jj.gt.1) goto 10
	goto 20
40      jj=mod(i,9)
	if(jj.eq.0.or.jj.eq.2.or.jj.eq.3.or.jj.eq.8) goto 10
	goto 20
50      jj=mod(i,8)
	if(jj.gt.3) goto 10
	goto 20
60      jj=mod(i,12)
	if(jj.gt.5) goto 10
20      if(idirfl.eq.1) iscr(i,j)=1
	if(idirfl.eq.2) iscr(j,i)=1
10      continue
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine scrplo(nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/86
C
C     Plot the virtual screen to the physical screen on Tektronix/Pericom
C     and erase whatever was in the virtual screen
C
C     Calls  : amode, outch, pmode
C-----------------------------------------------------------------------
C
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call pmode(nter)
      do 40 i=limits(1),limits(2)
	do 50 j=limits(3),limits(4)
	  if(iscr(i,j).ne.1) goto 50
	  ihx=(ishft(i,-5).and.n37).or.n40
	  ilx=(i.and.n37).or.n100
	  ihy=(ishft(j,-5).and.n37).or.n40
	  ily=(j.and.n37).or.n140
	  call outch(nter,ihy)
	  call outch(nter,ily)
	  call outch(nter,ihx)
	  call outch(nter,ilx)
	  iscr(i,j)=0
50        continue
40      continue
      call amode(nter)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine labplo(i,j,gelm,nout,nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/86
C
C     Label atom at position (I,J) in plot with element name GELM
C
C     Calls : amode, outch, pmode
C-----------------------------------------------------------------------
C
      implicit character *8 (g)
      common /charac/ n14,n30,n33,n34,n35,n37,n40,n41,n67,n100,n140,n177
C
      call pmode(nter)
      ihx=(ishft(i,-5).and.n37).or.n40
      ilx=(i.and.n37).or.n100
      ihy=(ishft(j,-5).and.n37).or.n40
      ily=(j.and.n37).or.n140
      call outch(nter,ihy)
      call outch(nter,ily)
      call outch(nter,ihx)
      call outch(nter,ilx)
      call amode(nter)
      write(nout,2000) gelm
2000  format(1X,A2)
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine labsca(nout,nter)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/86
C
C     Labels the already plotted scale
C
C     Calls :  hihome
C-----------------------------------------------------------------------
C
      call hihome(nter)
      write(nout,2000)
2000  format(' Scale: 1 Angstrom is')
      return
      end
C
C
C
C
C-----------------------------------------------------------------------
      subroutine sclplo(ascale)
C-----------------------------------------------------------------------
C     Written by : Daniel Altermatt, Winter 1985/86
C
C     Plots scale as dash of length one angstrom
C
C     Calls: --
C-----------------------------------------------------------------------
C
      common /screen/ limits(4),iscr(1000,1000),cellax(3,16)
C
C  Draw scale -- dash of length one Angstrom
C  -----------------------------------------
c
      ione=int(ascale)
      ix1=286
      ix2=ix1+ione
      iy=748
      do 10 i=ix1,ix2
10      iscr(i,iy)=1
      if(limits(1).gt.ix1) limits(1)=ix1
      if(limits(2).lt.ix2) limits(2)=ix2
      if(limits(4).lt.(iy+3))  limits(4)=iy+3
      do 20 i=1,3
	iscr(ix1,(iy+i))=1
	iscr(ix1,(iy-i))=1
	iscr(ix2,(iy+i))=1
20      iscr(ix2,(iy-i))=1
      return
      end
C
C
C
C

C*****************************************************************
C*  machine dependent code  in   OUTCH and FLUSH                 *
C*  This subroutine is needed on all machines except NORD        *
C*  -- Switch off for PC --                                      *
C*****************************************************************
C-----------------------------------------------------------------------
*     subroutine outch(nout,numb)
C-----------------------------------------------------------------------
c     written 88.5.19 by Beame and Whiteside
c
C     Send a number as command to the output stream
C
C     NUMB is the octal command to be output
C-----------------------------------------------------------------------
C
*       common /qiostuf/ buffer, count, channel
*       byte buffer(80)
*       integer count, channel, status, sys$assign
*       character*132 filename
*
*        if (channel .eq. 0) then
*               inquire(nout,name=filename)
*               status = sys$assign(filename,channel,,)
*               if (.not. status) status = sys$assign('TT',channel,,)
*               if (.not. status) stop 'Channel open failed... OutCh'
*       endif
c
*      if (count .ge. 80) call flush
*       count = count + 1
*       buffer(count) = numb
*       end
c
c************************************************************************
c
c  Dummy routine OUTCH to be turned on for PC to allow linking.
c
      subroutine outch(nout,numb)
      n=nout-numb
      return
      end
c************************************************************************
c
c------------------------------------------------------------------
*       subroutine flush
c------------------------------------------------------------------
c       written 88.5.19 by Beam and Whiteside
c
c       writes partially filled buffer to the terminal at the
c       conclusion of a plot
c
c-------------------------------------------------------------------
*       common /qiostuf/ buffer, count, channel
*       byte buffer(80)
*       integer count, channel, status, sys$qiow
*       Parameter (IO$_WRITEVBLK = 48)
*       Parameter (IO$M_NOFORMAT = 256)
*
*       status = sys$qiow(,%val(channel),
*    1   %val(IO$_WRITEVBLK + IO$M_NOFORMAT),,,,
*    1   buffer,%val(count),,,,)
*       if (.not. status) stop 'Write to channel failed ... Flush'
*       count = 0
*       end
c
c*************************************************************************
c  
c  Dummy subroutine FLUSH to be turned on for PC to allow linking.
c
      subroutine flush
      return
      end
c**************************************************************************
c
*      function ishft(i,j)
c
c**************************************************************************
c     INTRINSIC SHIFT CALL FOR CYBER
c
c     machine dependent code -- 'switch off' for all machines except Cyber
c**************************************************************************
c
*      ishft=shift(i,j)
*      return
*      end
C
      subroutine endfil
      return
      end
