C
C                   FILE SCFSLB           Last update: 31 July 1992
C                   -----------                        C. Brodie
C
c   This file contains standard FORTRAN-77 code (well, almost standard)
c   This file contains only single precision arrays
c
c
c   This file contains machine dependent code, marked
c   'machine dependent code'.
c
c
c
c
C    THIS FILE CONTAINS THE FOLLOWING ROUTINES
C
C1   S RSCFS(GENT,NENT)      MAIN ROUTINE TO READ SCFS-ENTRY GENT (UNIT 'NREAD')
C2   S SKIPIN(NENT,IER)      SKIPS =NENT= ENTRIES (no longer in use)
C2.5 S SKPSCF(NENT,ICOUNT,IERR) SKIPS =NENT= ENTRIES (in use)
C3   S FIND(GENT,IER)        SEARCHES FOR ENTRY =GENT=
C4   S RDIM(IER)             READS LATTICE PARAMETERS AND ERRORS
C5   S SPACGR(ISEL,IER)      READS SPACE GROUP SECTION (NON-STANDARD)
C6   S SYMMET(IER)           READS SYMMETRY SECTIONS  SCFS-87 
C6B  S MAKETU1               CREATES TU FROM THE SYMMETRY TABLE
C6C  S MAKETU                CREATES TU FROM THE LATTICE SYMBOL,GLAT
C7   S COORD(IER)            READS ATOM COORDINATES
C8   S REMARK(IER)           READS IN REMARKS
C9   S CLEAR(IER)            SKIPS REST OF A SECTION
C10  S REFER(IER)            READS REFERENCE SECTION
C11  S CONDT(GR,D1,D1,IER)   READS CONDITIONS SECTION
C12  S FORML(IER)            READS FORMULA SECTION
C13  S ATOMR(ISEL,IER)       READS ATOM SECTIONS
C14  S OPERA(ISEL IER)       READS SYMMETRY SECTIONS (NON-STANDARD) SCFS-84
C15  S BONY(IER)             READS BOND SECTION
C16  S CONNE(IER)            READS IN CONNECTIONS 
C16B S SORTEL                FINDS SEQUENCE NUMBER OF AN ELEMENT
C17  S WORDS(IER)            READS IN COMMENTS AND TEXT
C    S CRYS(IER)             READS IN CRYSTAL DATA
C18  S WSCFS(MMM)            MAIN ROUTINE TO WRITE AN NEW ENTRY IN SCFS
C18B S INCREA(JSYM,MMM)      TO EXPAND AND WRITE OUT SYMMETRY TABLE   
C
C     EXTERNALS CALLED FROM THIS FILE:
C
C   FOR-COS, FOR-SIN, FOR-SQRT
C
C   MAINLB: PARAM,ueq
C   MATHLB: INTEGER,upcase,NUM, HLEFT, maxdim
C
C****** STANDARD FORTRAN 77 CODE ONLY USED IN THIS FILE
C
C
C
C   ENTRY NUMBER GENT1 (STORED IN A8 IN CHARACTER FORM) WILL BE READ BY A
C   CALL TO SCFS(GENT1,NENT). IF GENT1 IS BLANK OR ZERO, THE NEXT ENTRY ON
C   THE FILE WILL BE READ.
C
C
      SUBROUTINE RSCFS(GENT1,NENT)
C     ----------------------------
C1
C  WRITTEN 80.6.1 BY DOUG THIERS
C   SUPPLIED BY DR I D BROWN, MCMASTER UNIVERSITY, HAMILTON, ONT. CANADA
C
C   MASTER CHANGES AND SYSTEM COMPARTIBILITY BY D. ALTERMATT, JUNE 1984
C   ADDITIONAL CHANGES BY N. HEINIG, JUNE 1986
C
C   RETURNS NENT=-1 IF END-OF-FILE ENCOUNTERED BEFORE "END" CARD
C
C   CALLS ATOMR, BONY, CLEAR, CONDT, COORD, FIND, FORML, OPERA, PARAM,
C         RDIM, REFER, SYMMET, SPACGR, upcase
C
C   VARIABLES USED
C
C   HD-PRESENT HEADER
C   HLAST-PREVIOUS HEADER
C
C   FOR OTHER VARIABLES SEE DESCRIPTION OF 'IDB-STANDARD' COMMON BLOCKS
c
c   Before reading in new data it is important to zero and blank the arrays
c    (Use reseta and resetc)
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON /CELL/  A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON /DENS/  ZFORM,DENS,DDENS,CDENS,FWGHT,WAVELN,ABSORB,
     +               FZERO,TMPRTR
      include 'comatom.for'
      include 'combond.for'
      COMMON /ECELL/ DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /RCELL/ AS,BS,CS,RALPHA,RBETA,RGAMMA,RVOL,RM(3,3)
      COMMON /SYM/   NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON /GSYM/  GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      COMMON /SYMA/  MULT(48,48),IT(3,48),NSPE,MSPE,ISPTAB(48,30),
     +               MUSP(30),SP(3,3,30),TSP(3,30)
      COMMON /GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     +               HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON /TITLE/ JAUTH,KCOMM
      COMMON /HREF/ HINFO(2,2), GFILEN(5),hist(40)
      COMMON /GCOND/ GCRAD, GIRAD, GDIFF, GABS, GREFN, GSTAT
      COMMON /COND/ CONDIT(6,8), NSTD,MSTD, CSTD(6,4)
      COMMON /HTEXT/ GCRYST(2),GTWIN(5),HTXT(5,10),HDATA(19,4)
      COMMON /FORM/ NFORM(3), MFORM, FORM(24,3)
      COMMON /GFORM/ GFORM(24,3)
      DIMENSION JSYM(5,192)
C
C
      HLAST = 'END     '
      IER=0
      if(ndebug.ne.0) write(ndebug,1001) gent1,mx,mbon,mcon
 1001 format(' *debug*rscfs*1001* gent1=,'A8,', mx=',I5,', mbon=',I5,
     1           ', ncon=',I5)
      write(nout,1000)
 1000 format('1')
C
C   GO THROUGH FOLLOWING LOOP ONCE FOR EACH ENTRY
C
      READ(NREAD,100,END=300) HD
  100 FORMAT(BZ,A80)
      call upcase(hd)
C
  200 FORMAT(16X,A80)
C
C  LOOP THROUGH FOR EACH SECTION, UNTIL "END" HEADER
C
C  CHECK CARD CONTENT ON KNOWN HEADERS, THEN
C  CALL APROPRIATE READING SUBROUTINE FOR SECTION FOUND
C
    2 IF(HD(1:8).NE.'TITLE   ') GOTO 19
        GENT = GENT1
        HNAME = HD(9:80)
        CALL UPCASE(HNAME)
        IF(HLAST .NE. 'END     ') WRITE(NOUT,140)
  140   FORMAT(1X,'"TITLE" CARD FOLLOWS CARD OTHER THAN "END"')
        IF(NDEBUG .NE. 0) WRITE(NDEBUG,150) HLAST
  150   FORMAT(' ***DEBUG**RSCFS*150 HLAST=', A8)
        CALL FIND(GENT1,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
   19 IF(HD(1:8).NE.'CELL DIM') GOTO 29
        CALL RDIM(IER)
        IF (IER.EQ.-1) GOTO 300
        CALL PARAM
        ICELL=1
        GOTO 999
   29 IF(HD(1:8).NE.'SPACE GR') GOTO 39
        ISEL=0
        CALL SPACGR(ISEL,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
   39 IF(HD(1:8).NE.'SYMMETRY') GOTO 49
c  UNIX problem with JSYM here.
        CALL SYMMET(JSYM,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
   49 IF(HD(1:8).NE.'ATOM COO') GOTO 59
        CALL COORD(IER)
 
       IF (IER.EQ.-1) GOTO 300
        GOTO 999
   59 IF(HD(1:8).NE.'REMARK  ') GOTO 69
        CALL REMARK(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
   69 IF(HD(1:8).NE.'FORM FAC'.AND.HD(1:3).NE.'HKL') GOTO 79
        CALL CLEAR(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
   79 IF(HD(1:8).NE.'END     ') GOTO 89
        HLAST=HD(1:8)
        GOTO 109
   89 IF(HD(1:8).NE.'REFERENC') GOTO 99
        CALL REFER(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO  999
   99 IF(HD(1:8).NE.'CONDITIO') GOTO 119
        CALL CONDT(IER)
        IF (IER.EQ.-1) GOTO 300
        IF (DENS.LT.0.000001) DENS=DENS1
        GOTO 999
  119 IF(HD(1:8).NE.'FORMULA ') GOTO 129
        CALL FORML(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  129 IF(HD(1:8).NE.'ATOM    ') GOTO 139
        ISEL=0
        CALL ATOMR(ISEL,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  139 IF(HD(1:8).NE.'SG OPERA') GOTO 149
        CALL OPERA(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  149 IF(HD(1:8).NE.'BONDS   ') GOTO 159
        CALL BONY(IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  159 IF(HD(1:8).NE.'SG NAME ') GOTO 169
        ISEL=1
        CALL SPACGR(ISEL,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  169 IF(HD(1:8).NE.'ATOMS   ') GOTO 179
        ISEL=1
        CALL ATOMR(ISEL,IER)
        IF (IER.EQ.-1) GOTO 300
        GOTO 999
  179 IF(HD(1:8) .NE. 'CONNECTI') GOTO 182
        CALL CONNE(JSYM,IER)
        IF(IER .EQ. -1) GOTO 300
        GOTO 999
  182 IF(HD(1:8).NE.'COMMENTS') GOTO 183
        CALL WORDS(IER)
        IF(IER .EQ. -1) GOTO 300
        do 184 i=1,2
  184     if(hdata(I,5).ne.'    ') hform(i)=hdata(5,i)
        GOTO 999
  183 IF(HD(1:8) .NE. 'CRYSTAL ') GOTO 185
        CALL CRYS(IER)
        IF(IER .EQ. -1) GOTO 300
        GOTO 999
  185 IF(HD(1:8).NE. '       ')THEN
        WRITE(NOUT,199) HD(1:8)
  199   FORMAT(' ***INVALID HEADER CARD,',A8,' DATA IS NOT STORED.') 
  189   READ(NREAD, 100,END=300) HD
        CALL UPCASE(HD)
        WRITE(NOUT,200) HD
        IF(HD(1:4) .NE. '*EOS') GOTO 2
      ELSE
        GOTO 989
        ENDIF   
  999 HLAST=HD
  989 READ(NREAD,100,END=300) HD
      call upcase(hd)
      if(hd(1:8).ne.'TITLE   ') WRITE(NOUT,200) HD
      GOTO 2
  109 CONTINUE
      if(ndebug.ne.0) write(ndebug,1109) icell,nsym,nx,nbon,ncon
 1109 format('*debug*rdscfs*1109* icell=',I2,', nsym=',I3,', nx=',
     +    I4,', nbon=',I4,', ncon=',I4)
      RETURN
  300 WRITE(NOUT,310)
  310 FORMAT(1X,'***** E-O-F  REACHED ON "SCFS" BEFORE "END" CARD',/)
      NENT=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE SKIPIN(NENT,HLINE,IER)
C2
*  THIS ROUTINE SKIPS NENT ENTRIES IN SCFS FORMAT
*  IF THE SKIPSCFS OPTION WAS CHOSEN BACK IN THE
*  CONTROL PROGRAM...
*
*  83.6.1 PHIL ARSENAULT / 84.6.5 D. ALTERMATT
*  86.2.14 This routine no longer in use because find can do everything
*          that is necessary in positioning an entry.  If it is to be
*          used, it should be called from the calling program (not rscfs)
*          in response to a special user request of a kind not currently
*          available.
*
      implicit character*80 (h)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      NUMBER=0
      IF(NENT.LE.0) RETURN
   50 READ(NREAD,100,END=900) HLINE
  100 FORMAT(A80)
      IF(HLINE(1:5).EQ.'TITLE') NUMBER = NUMBER+1
      IF(NUMBER.EQ.NENT) THEN
        WRITE(NOUT,1000) HLINE
 1000   FORMAT(1X,A80)
        RETURN
        ENDIF
      GOTO 50
  900 WRITE(NOUT,200) NUMBER,NENT
  200 FORMAT('0','***ERROR**SKIPIN** ONLY ',I2,' ENTRIES WERE',
     +  ' SKIPPED OF THE DESIRED ',I2,'.  EOF ENCOUNTERED.')
      IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
      SUBROUTINE SKPSCF(NENT,IERR)
C2.5
*  THIS ROUTINE SKIPS NENT ENTRIES IN SCFS FORMAT
*  IF SKPSCF WAS CALLED BACK IN THE CONTROL PROGRAM...
*
*  83.6.1 PHIL ARSENAULT / 84.6.5 D. ALTERMATT
*  86.2.14 This routine no longer in use because find can do everything
*          that is necessary in positioning an entry.  If it is to be
*          used, it should be called from the calling program (not rscfs)
*          in response to a special user request of a kind not currently
*          available.
*  89.7.20 S.Urquhart  This routine revived for new improved sugar free
*          Strumo and Strumor!
*
* IERR =+1 for EOF
* IERR =-1 for error in reading

c  Variable Trans-dictionary
c  -------------------------
c  nent:       From strumo(r) number of entries to read before returning
c              ...eg if one then read next entry....
c              On return ...nent equals the number of entries read in
c              sucessfully...eg. before an eof
c  ierr        Returning to strumo(r)  +1 if eof
c                                       0 if ok
      implicit character*8 (g)
      implicit character*80 (h)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      common/gtitle/ gent,hname

      gent=' '
      ierr=0
      itempnent=0
      IF(NENT.LT.0) RETURN
   50 READ(NREAD,100,END=900) HLINE
  100 FORMAT(A80)
      IF(HLINE(1:5).EQ.'TITLE') THEN
        itempnent=itempnent+1
        ENDIF
      IF(itempnent.GE.NENT) THEN
        backspace(nread)
        CALL rscfs(gent,nentry)
        if (nentry.eq.-1) nent=+1
        RETURN
        ENDIF
      GOTO 50
  900 IERR=+1
      nent=itempnent
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE FIND(GENT1,IER)
C
C3 ****SUBROUTINE*****          FIND
C                               ----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT/ 89.8.1 S. URQUHART
C
C  SUBROUTINE TO SEARCH FOR A GIVEN ENTRY IN IENT1 IN SCFS FORMAT,
C  WHICH WILL BE FOUND ON THE TITLE CARD.
C
C  CALLS CHAR, HLEFT
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     +             HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/COUNTER/ ICOUNT
C
C  IER =0 OK
C  IER =+1 FILE READ ERROR
C  IER =-1 EOF

      NENT=0
      IFLAG=0
    
 
    1 READ(NREAD,200,END=10) GCHAR,HNAME,GENT
  200 FORMAT (BZ,A1,A66,A8)
      gen1=hleft(gent1,8)
      gen=hleft(gent,8)
      if(ndebug.ne.0) write(ndebug,1200) gent1,gen1,gent,gen
 1200 format(' *debug*find*1200* *',4(A8,'*'))
      IF(GEN1.EQ.'0       '.or.gent1.eq.'        '
     +                      .or.gen1(1:2).eq.'* ') GOTO 20
      IF(GEN1.NE.GEN) GOTO 19
   20 IFLAG=1
      WRITE (NOUT,210) GCHAR,HNAME,GENT
  210 FORMAT (16X,A1,A66,A8)
      IF(GCHAR.EQ.'*') GOTO 19
      IER=+1
      WRITE (NOUT,300)
  300 FORMAT (1X,'***** TITLE HEADER NOT FOLLOWED BY *EOS CARD,',
     +          ' REMAINDER IGNORED')
   19 IF(IFLAG.EQ.1) THEN
        if (nent.gt.0) ICOUNT=NENT-1
        RETURN
        ENDIF
    2 READ(NREAD,100,END=10) gd
  100 FORMAT(BZ,A8)
      IF(gd.eq.'TITLE   ') then
        NENT=NENT+1
        GOTO 1
        endif
      GOTO 2
   10 WRITE (NOUT,400)  GENT1
  400 FORMAT(1X,'***** END-OF-FILE WHILE SEARCHING FOR ENTRY ',A8,//)
      IER=-1
      if (nent.gt.0) ICOUNT=NENT-1
      return
      END
*
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE RDIM(IER)
C
C4 *****SUBROUTINE*****          RDIM
C                                ----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT / 86.5.21 N.HEINIG
C
C  ROUTINE TO READ AND RETURN THE LATTICE PARAMETERS,AND ERRORS
C
C  CID = CELL    CELL PARAMETERS READ AND STORED
C      = ERRS    CORRESPONDING ERRORS READ AND STORED
C      = VOL     READS AND STORES VOLUME AND DENSITY
C      = PHYS    READS AND STORES MEASURED DENSITY
C    
      IMPLICIT CHARACTER*8 (G)
      DIMENSION REALL(7)
      COMMON/CELL/ A,B,C,AL,BE,AGA,VOL,AM(3,3),ICELL
      COMMON/DENS/ ZFORM,DENS,DDENS,CDENS,FWGT
      COMMON/ECELL/ DA,DB,DC,DAL,DBE,DGA
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
C
      PI=3.141592654
    1 READ(NREAD,100,END=500) GCHAR,GCID,GDSK,REALL
      IF(GCHAR .EQ. '*') RETURN              
      IF(GCID .EQ. 'CELL' .OR. GCID .EQ. '    ')THEN
        WRITE(NOUT,110) GCHAR,GCID,GDSK,REALL
        A=REALL(1)
        B=REALL(2)
        C=REALL(3)
        AL=REALL(4)
        BE=REALL(5)
        AGA=REALL(6)
        ZFORM=REALL(7)
        NFORM=INT(REALL(7)+0.01)
        IF(NFORM.EQ.0) ZFORM=1.
        AL=AL*PI/180.0
        BE=BE*PI/180.0
        AGA=AGA*PI/180.0   
      ELSEIF(GCID .EQ. 'ERRS') THEN
        WRITE(NOUT,110) GCHAR,GCID,GDSK,REALL
        DA=REALL(1)
        DB=REALL(2)
        DC=REALL(3)
        DAL=REALL(4)
        DBE=REALL(5)
        DGA=REALL(6)
        DAL=DAL*PI/180.0
        DBE=DBE*PI/180.0
        DGA=DGA*PI/180.0
      ELSEIF(GCID .EQ. 'VOL ')THEN
        WRITE(NOUT,110) GCHAR,GCID,GDSK,REALL
        VOL = REALL(1)
        FWGT = REALL(3)
        CDENS = REALL(4)
      ELSEIF(GCID .EQ. 'PHYS')THEN
        WRITE(NOUT,110) GCHAR,GCID,GDSK,REALL
        DENS = REALL(1)
        DDENS = REALL(2)
      ELSE
        WRITE(NOUT,200) GCHAR,GCID,GDSK,REALL
  200   FORMAT(1X,'**** SKIPPED  ',A1,A4,A4,6F10.4,F5.0)
        ENDIF  
      GOTO 1
  100 FORMAT(BZ,A1,A4,A4,1X,6F10.4,F5.0)
  110 FORMAT(1X,A1,A4,A4,1X,6F10.4,F5.0)
  500 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE SPACGR(ISEL,IER)
C
C5 *****SUBROUTINE*****          SPACGR
C                                ------
C
C  WRITTEN 80.6.1 BY DOUG THIERS
*  REWRITTEN JULY,6,82 BY DAVID R. MOSSCROP
C  MASTER CHANGES 84.6.5 BY D. ALTERMATT / 86.5.21 BY N.HEINIG
C
C  ROUTINE TO READ THE INFOMATION OF THE SPACE GROUP SECTION
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL
*
*  CID = HALL      HALL SPACE GROUP SYMBOL READ AND STORED
*      = HERM      HERMANN-MAUGIN SG-SYMBOL READ AND STORED
*      = SCHN,SYST    IGNORED
*      = LATT      SYMMETRY PARAMETERS READ AND STORED
*
      IER = 0
    1 IF (ISEL.EQ.0) THEN
        READ(NREAD,100,END=500)GCHAR,GLAT,GCENT,GCID,GDSK,HSYM
  100   FORMAT(BZ,3A1,A2,A4,2X,A23)
        WRITE(NOUT,90)GCHAR,GLAT,GCENT,GCID,GDSK,HSYM
   90   FORMAT(1X,3A1,A2,A4,2X,A23)
        IF(GCID .EQ. 'HA') THEN
          HALL = HSYM
        ELSE
          HMSYM = HSYM
          ENDIF
        IF(GCHAR.EQ.'*' .AND. GLT.EQ.'E') RETURN
      ELSEIF(ISEL .EQ. 1) THEN
        READ (NREAD,200,END=500) GCHAR,GCID,GDSK,HNAME
  200   FORMAT(BZ,A1,2A4,1X,A24)
        WRITE(NOUT,290) GCHAR,GCID,GDSK,HNAME
  290   FORMAT(1X, A1,2A4, 1X,A24)
        IF(GCID.EQ.'SCHN'.OR.GCID.EQ.'SYST') THEN
          WRITE(NOUT,210)GCHAR,GCID,GDSK,HNAME
  210     FORMAT(1X,'**** IGNORED:',A1,2A4,1X,A24)
        ENDIF  
        IF(GCID .EQ. 'HERM') THEN
          HMSYM = HNAME(1:12)
        ELSEIF(GCID .EQ. 'HALL') THEN
          HALL = HNAME
        ELSEIF(GCID .EQ. 'LATT') THEN
          GCE = HNAME(1:1)
          IF(GCE .EQ. 'C' .OR. GCE .EQ. '-')THEN
            GCENT = 'C'
          ELSE
            GCENT = 'A'
            ENDIF
          GLAT = HNAME(2:2)
          ENDIF
        IF(GCHAR .NE. '*') GOTO 1
        RETURN
      ENDIF
  500 IER=-1
      RETURN
      END
*
*
*
*
***********************************************************************
*
*
*
*
*
      SUBROUTINE SYMMET(JSYM,IER)
C
C6 *****SUBROUTINE*******     SYMMET
C                             ------
C
C  WRITTEN SUMMER '86 BY NINA HEINIG
C
C  READS AND STORES STANDARD SYMMETRY AND SPECIAL POSITION OPERATORS
C
C  OUTPUT:  JSYM(1,192) ; number of matrix in S  i.e. S(*,*,!)
C           JSYM(2,192) ; number of row in TU  i.e. TU(*,!)
C           JSYM(3-5,192) ; compensation matrix for reduction
C           IER ;  ERROR MESSAGE
C
C  CID = SYOP     SYMMETRY OPERATORS OF THE SPACE GROUP
C      = SPOS     SPECIAL POSITION COORDINATES
C      = TWIN     NOT STORED
C
C  CALLS  INTEGER, INCH
C  TU is in fractional form
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C    
      DIMENSION ITOT(3) 
      DIMENSION ISYM(3,3),TK(3), TX(3),TTU(3)
      COMMON/FILES/NIN,NOUT,NDEBUF,NREAD
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/SYMA/MULT(48,48),IT(3,48),NSPE,MSPE,ISPTAB(48,30),
     +      MUSP(30),SP(3,3,30),TSP(3,30)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      DIMENSION JSYM(5,192)
C
      
      do 50 i=1,192
       do 50 j=1,5
   50   jsym(j,i)=0
      jjsym=0
      IFLAG = 0
      IASYM = 0
      ISTART = 0
      NLT = 1
      NSYM = 1
      DO 7 I=1,3
   7    TU(I,1) = 0.
  10  READ(NREAD,1100,END=500) GCHAR,GCID,GDSK,GA,((ISYM(J,K),
     *     K=1,3),TK(J), J=1,3), GASYM, IA,IB
 1100 FORMAT(BZ,A1,2A4,A1, 2(3I2,F10.7,4X), 3I2,F10.7,A3,2I2)
      IASYM=INCH(GASYM)
C
C  READ SYMMETRY OPERATORS
C
      IF(GCID .EQ. '    ' .OR. GCID .EQ. 'SYOP') THEN
C
C  CHECK IF GASYM CAN BE USED AS A COUNTER
C
        IF(IASYM .ne. 0 .OR. IFLAG .EQ. 1)THEN
          IFLAG = 1
          JJSYM = IASYM
        ELSE
          JJSYM = JJSYM + 1
          ENDIF
C
C  READ IN THE FIRST SYMMETRY OPERATION
C
        IF(ISTART .EQ. 0) THEN
          JSYM(1,JJSYM) = 1
          JSYM(2,JJSYM) = 1
          DO 17 I=1,3

            DO 27 J=1,3
              S(I,J,NSYM) = ISYM(I,J)
   27         CONTINUE
            T(I,NSYM) = TK(I)
   17       CONTINUE
          ISTART = 1
          GOTO 100
          ENDIF
C
C  READ IN NEW SYMMETRY OPERATION
C  CHECK IF S VALUES ARE THE SAME AS PREVIOUSLY
C
        DO 75  I=1,NSYM
          DO 49 J=1,3
            DO 49 K=1,3
              IF(ISYM(J,K) .NE. INTEGER(S(J,K,I),0)) GOTO 75
   49         CONTINUE  
          JSYM(1,JJSYM) = I
          DO 53 J=1,3
            TX(J) = TK(J) - T(J,I)
   53       TTU(J) = ABS( INTEGER(TX(J)*12.,0) )
C
C  CHECK IF TU VALUE IS REPEATED
C
          DO 70 J=1,NLT
            ITEMP = 0
            DO 71 K=1,3
   71         IF(ABS(TTU(K)/12.- TU(K,J)).GT. 0.1) ITEMP = 1
            IF( ITEMP .EQ. 1) GOTO 70
            JSYM(2,JJSYM) = J
            GOTO 100      
   70       CONTINUE
C
C  IF NOT, CREATE NEW TU VALUES
C
          NLT = NLT + 1 
          JSYM(2,JJSYM) = NLT
          DO 73 J=1,3
   73       TU(J, NLT) = TTU(J)/12.
          GOTO 100
   75     CONTINUE
C
C  IF NO MATCH FOR S, CREATE NEW VALUES OF S AND T
C
        NSYM = NSYM + 1
        DO 87 K=1,3
          DO 89 J=1,3
            S(K,J,NSYM) = ISYM(K,J)
   89       CONTINUE
          T(K,NSYM) = TK(K)
   87     CONTINUE
        JSYM(1,JJSYM) = NSYM
        JSYM(2,JJSYM) = 1
  100   CONTINUE
C
C  READ SPECIAL POSITION OPERATOR
C
      ELSEIF(GCID .EQ. 'SPOS') THEN
        NSPE = NSPE + 1
        IF(NSPE.LE.MSPE .and. NSPE.GT.0) THEN
          GSYSP(1,NSPE) = GA
          DO 150 I=1,3
            DO 140 J=1,3
              SP(I,J,NSPE) = ISYM(I,J)
  140         CONTINUE
            TSP(I,NSPE) = TK(I)
  150       CONTINUE 
          MUSP(NSPE) = IA
        ELSE
          WRITE(NOUT,1300)NSPE,MSPE
 1300     FORMAT(1X,'***SCFSLB*SYMMET**special position counter,',
     *          i3,' is outside array limits. Max. size:',i3)
          ENDIF                                                    
C
      ELSEIF(GCID .EQ. 'TWIN') THEN
        WRITE(NOUT,1400)GCHAR,GCID,GDSK 
 1400   FORMAT(1X,'***LINE WITH HEADER TWIN IGNORED:',A1,2A4 )
C
C  AT THE END, COMPARE TU ROWS FOR DUPLICATION.
C
      ELSEIF(GCID .EQ. 'EOS ') THEN
        GOTO 170
        ENDIF                                                          
      WRITE(NOUT,2100) GCHAR,GCID,GDSK,GA,((ISYM(J,K),K=1,3),
     *      TK(J), J=1,3), IASYM,IA,IB
 2100 FORMAT(1X,A1,2A4,A1,2(3I2,F10.7,4X),3I2,F10.7,I3,2I2)
      IF(GCHAR .NE. '*') GOTO 10
  170 NLTEMP = NLT
  171 DO 180 I=1,NLTEMP-1
        NNLT = NLTEMP
        DO 175 II=1,3
          ITOT(II) = INTEGER(12.*TU(II,NNLT),0)  
  175     ITOT(II) = ITOT(II) + INTEGER(12.*TU(II,I),0)
        DO 190 J=1,3
  190     IF(MOD(ITOT(J),12) .NE. 0) GOTO 180
C
C  IF THEY ARE THE SAME, CHANGE JSYM(2,*), AND GET RID OF ONE ROW.
C  IF NECESSARY, CREATE COMPENSATION VECTOR, JSYM(3-5,*).
C
        DO 185 J=1,JJSYM
          IF(JSYM(2,J).EQ.NNLT) THEN
            JSYM(2,J)= I
            DO 195 JJ=1,3
            IF(ITOT(JJ)/12 .NE. 0) THEN
                JSYM(JJ+2,J) = 12*TU(JJ,I)-12*TU(JJ,NNLT)
                ENDIF
  195         CONTINUE
            ENDIF              
  185     CONTINUE
  180   CONTINUE
      NLTEMP = NLTEMP-1
      IF(NLTEMP .GT. 1) GOTO 171
      RETURN
 500  IER = -1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE MAKETU1(GCHAR,GCID,TK ,IER)
C
C6B ****SUBROUTINE****        MAKETU1
C                             -------
C WRITTEN  1986/06/05  BY NINA HEINIG
C
C ROUTINE TO CALCULATE TU FROM T VALUES WHEN USING GLAT
C IS INAPPROPRIATE .
C
      IMPLICIT CHARACTER*8 (G)
      DIMENSION TK(3),ITU(3,4)
      COMMON/FILES/NIN,NOUT,NDEBUF,NREAD
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON/GSYM/ GLAT
C
C      ICNT = 0
C      JCNT = 0
C      NLT = 1
C      DO 17 I = 1,3
C        TU(I,NLT) = 0.
C   17   CONTINUE 
C   27 IF(GCID.NE.'SYOP' .AND. GCID.NE.'    ') THEN
      IF(NLT .EQ. 4) THEN
        GLAT = 'F'
      ELSEIF(NLT .EQ. 3) THEN
        GLAT = 'G'
      ELSEIF(NLT .EQ. 1) THEN
        GLAT = 'P'
      ELSE
        IF(TU(1,2) .EQ. 0.) THEN
          GLAT = 'A'
        ELSEIF(TU(2,2) .EQ. 0.) THEN
          GLAT = 'B'
        ELSEIF(TU(3,2) .EQ. 0.) THEN
          GLAT = 'C'
        ELSE
          GLAT = 'I'
          ENDIF
        ENDIF
      RETURN
C      ENDIF
C      NLT = JCNT/NSYM + 2
C      ICNT = MOD(JCNT,NSYM)
C      ICNT = ICNT + 1
C      DO 37 I=1,3
C        ITU(I,NLT) = (TK(I) - T(I,ICNT)) * 12
C        ITU(I,NLT) = ABS(ITU(I,NLT))
C        TU(I,NLT) = ITU(I,NLT)
C   37   CONTINUE
C      JCNT = JCNT + 1
C      READ(NREAD,1000,END=500) GCHAR,GCID,(TK(J), J=1,3)
C 1000 FORMAT(BZ,A1,A4,5X,3(6X,F10.7,4X))
C      GOTO 27
C  500 IER = -1
C      RETURN
      END
*
*
*
***********************************************************************
*
*
*
       SUBROUTINE MAKETU
C
C6C ****SUBROUTINE****       MAKETU
C                            ------
C  WRITTEN 86/06/05 BY NINA HEINIG
C
C  SUBROUTINE TO CONSTRUCT TU GIVEN GLAT
C
C
       IMPLICIT CHARACTER*8 (G)
C
       COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGR
       COMMON /GSYM/ GLAT
       COMMON /SYM/ NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
C
       DO 17 I=1,3
         TU(I,1) = 0.
   17    CONTINUE  
       IF(GLAT .EQ. 'P') THEN
         NLT = 1
         RETURN
         ENDIF
       IF(GLAT.EQ.'A'.OR. GLAT .EQ. 'F') THEN
         NLT = 2
         TU(1,2) = 0.
         TU(2,2) = 0.5
         TU(3,2) = 0.5
         IF(GLAT .EQ. 'A') RETURN
         ENDIF
       IF(GLAT .EQ. 'B' .OR. GLAT .EQ. 'F') THEN
         NLT = 2
         IF(GLAT .EQ. 'F') NLT = 3
         TU(1,NLT)= 0.5
         TU(2,NLT)= 0.
         TU(3,NLT)= 0.5
         IF(GLAT .EQ. 'B')RETURN              
         ENDIF
       IF(GLAT .EQ.'C' .OR. GLAT .EQ. 'F') THEN
         NLT = 2
         IF(GLAT .EQ. 'F') NLT = 4
         TU(1,NLT)= 0.5
         TU(2,NLT)= 0.5
         TU(3,NLT)= 0.
         RETURN
         ENDIF
       IF(GLAT .EQ.'I') THEN
         NLT = 2
         TU(1,NLT)= 0.5
         TU(2,NLT)= 0.5
         TU(3,NLT)= 0.5
         RETURN
         ENDIF
       IF(GLAT .EQ. 'G' .OR. GLAT .EQ.'H') THEN
         NLT = 3
         TU(1,NLT-1)= 0.66666
         TU(2,NLT-1)= 0.33333
         TU(3,NLT-1)= 0.33333
         TU(1,NLT)= 0.33333
         TU(2,NLT)= 0.66666
         TU(3,NLT)= 0.66666
         RETURN
         ENDIF
       WRITE(NOUT,1000) GLAT
 1000  FORMAT(' ***WARNING*MAKETU*** TU not constructed,',
     *       ' unidentified GLAT;',A1)
       RETURN
       END         
*
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE COORD(IER)
C
C7 *****SUBROUTINE*****          COORD
C                                -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT
C
C  READS THE ATOM COORDINATES SECTION
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON/FILES/ NIN,NOUT,NDEBUG,NREAD
      include 'comatom.for'
    1 READ(NREAD,100,END=500) GCHAR,GNAME,GID,X,Y,Z,XND,OC,DX,DY,DZ,
     +                      SND,SOC
      IF(GCHAR.EQ.'*'.AND.GNAME.EQ.'EOS') GOTO 250
      WRITE(NOUT,110) GCHAR,GNAME,GID,X,Y,Z,XND,OC,DX,DY,DZ,SND,SOC
      J=0
      IF(N.EQ.0) GOTO 5
      DO 45 I=1,N
   45 IF(GNAME.EQ.GELEM(I).AND.GID.EQ.GDENT(I)) J=I
      IF(J.NE.0) GOTO 10
      IF(N.LT.M) GOTO 5
      WRITE(NOUT,200) GNAME,GID
  200 FORMAT(' NEW ATOM ',A2,A3,' IN ATOM COO SECTION WHEN',
     +  ' MAXIMUM ALREADY STORED')
      GOTO 20
    5 N=N+1
      J=N
      GELEM(J)=GNAME
      GDENT(J)=GID
   10 XX(1,J)=X
      XX(2,J)=Y
      XX(3,J)=Z
      U(1,1,J)=XND
      U(2,2,J)=0.0
      OCCU(J)=OC
      DXX(1,J)=DX
      DXX(2,J)=DY
      DXX(3,J)=DZ
      DU(1,1,J)=SND
      DU(2,2,J)=0.0
      SOCCU(J)=SOC
      OXID(J)=0.0
      NPOS(J)=1
      ICN(J)=0
      NZA(1,J)=0
      NZA(2,J)=0
   20 IF(GCHAR.EQ.' ') GOTO 1
      RETURN
  250 WRITE(NOUT,110) GCHAR,GNAME
      RETURN
  100 FORMAT(BZ,A1,A2,A3,4X,3F8.5,2F6.4,3F6.5,2F5.4)
  110 FORMAT(1X,A1,A2,A3,4X,3F8.5,2F6.4,3F6.5,2F5.4)
  500 IER=-1
      RETURN
      END
*
*
*
*
***********************************************************************
*
*
*
*
*
      SUBROUTINE REMARK(IER) 
C
C8 ******SUBROUTINE***    REMARK
C                         ------
C  SET UP 86/5/23 BY N.HEINIG
C
C  READS AND STORES IN REMARKS
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     +               HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON /TITLE/ JAUTH,KCOMM
C
   60 READ(NREAD,61,END=500) HLINE
      WRITE (NOUT,62) HLINE
   61 FORMAT(A80)
   62 FORMAT (16X,A80)
      IF (HLINE(1:4).EQ.'*EOS') RETURN
      KCOMM = KCOMM + 1
      IF(KCOMM .GT. 7) THEN
        KCOMM = KCOMM - 1
        WRITE(NOUT,*) ' ***** TOO MANY COMMENTS, PREVIOUS LINE LOST.'
        IF (HLINE(1:1) .EQ. '*') RETURN
        GOTO 60
        ENDIF
      HCOMM(KCOMM) = HLINE
      IF(HLINE(1:1) .EQ.'*') RETURN
      GOTO 60
  500 IER = -1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE CLEAR(IER)
C
C9 *****SUBROUTINE*****          CLEAR
C                                -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT
C
C  SKIPS REST OF SECTION
C
      CHARACTER*80 HLINE
      COMMON/FILES/ NIN,NOUT,NDEBUG,NREAD
      DATA IBL/'  '/
    1 READ(NREAD,100,END=500) ICHAR,HLINE
      WRITE(NOUT,110) ICHAR,HLINE
      IF(ICHAR.EQ.IBL) GOTO 1
      RETURN
  100 FORMAT(BZ,A80)
  110 FORMAT(1X,'***** IGNORED  ',A80)
  500 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE REFER(IER)
*     ---------- ----------
C10
C  READS REFERENCE SECTION
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT / 86.5.23 N.HEINIG
C
C  CID:
C  --- 
C  JRNL: JOURNAL REFERENCE;  AUTH: AUTHORS' NAMES;  TITL: PAPER'S TITLE
C  CAS: CHEM. ABSTRACT REGISTRY NO.;  PDB:  PROTEIN DATA BANK NO.
C  RFCD: CAMBRIDGE FILE REFCODE;  INOR: INORGANIC DATABASE NO.
C  METL: METAL FILE ENTRY NO.;  PDF: POWDER DIFFRACTION FILE NO.
C  HIST: HISTORY OF DATA MODIFICATIONS;  RMRK: COMMENTS
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON/GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     +             HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON/TITLE/JAUTH,KCOMM
      COMMON/HREF/ HINFO(2,2),GFILEN(5), hist(40)
      DIMENSION IG(5)
      DIMENSION GSAVE(5)
      DATA GSAVE /'PDB ','RFCD','INOR','METL','PDF '/
C
      I1=1
      I2=1
      I3=1
      ic=1
      ih=1
C
    1 READ(NREAD,1000,END=500) GCHAR,GCID,HLINE
 1000 FORMAT(BZ,A1,A4,5X,A65)
      HCOM='               '
*
*  THERE ARE MANY (12) SUB-CATEGORIES WITHIN THE 'REFERENCE'
*  SECTION, EACH OF THESE MUST BE STORED SEPARATELY...
*
      IF(GCID.EQ.'JRNL')THEN
        IF(I1.GT.2) GOTO 36
        HCODN(I1) = HLINE
        I1 = I1+1
      ELSEIF(GCID.EQ.'AUTH') THEN
        IF(I2.GT.10) GOTO 36
        HAUTH(I2)=HLINE
        JAUTH=I2
        I2=I2+1
      ELSEIF(GCID.EQ.'TITL') THEN
        IF(I3.GT.2) GOTO 36
        HTITLE(I3) = HLINE
        I3=I3+1
      ELSEIF(GCID.EQ.'RMRK') THEN
        KCOMM = KCOMM + 1
        IF (KCOMM.GT.7) THEN
          KCOMM = KCOMM - 1
          GO TO 36
        END IF
        HCOMM(KCOMM)=HLINE
      ELSE
        DO 46  I=1,5
          IF(GCID .EQ. GSAVE(I)) THEN
            GFILEN(I) = HLINE
            GOTO 39
            ENDIF
   46     CONTINUE
        if(gcid(1:3).eq.'CAS'.and.ic.le.2) then
          hinfo(ic,1)=hline
          ic=ic+1
          go to 39
          endif
        if(gcid(1:4).eq.'HIST'.and.ih.le.40) then
          hist(ih)=hline
          ih=ih+1
          go to 39
          endif
        IF(GCID .NE. 'EOS ') HCOM = '***** SKIPPED  '
        ENDIF
   39 WRITE(NOUT,1110) HCOM,GCHAR,GCID,HLINE
 1110 FORMAT(1X,A15,A1,A4,5X,A65)
      IF(GCHAR .NE. '*') GOTO 1
      RETURN
   36 GCOM='***** IGNORED  '
      GOTO 39
  500 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE CONDT(IER)
C
C11 ****SUBROUTINE*****          CONDT
C                                -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT/ 86.5.22 N.HEINIG
C
C  READS CONDITIONS SECTION
C
C  CID = '    '    CONDITIONS IN NON-STANDARD FORMAT
C      = CELL      CONDITIONS IN THE UNIT CELL
C      = INT       CONDITIONS IN MEASURING INTENSITIES
C      = HKL       REFLECTION RANGES
C      = STD       CONDITIONS IN STANDARD REFLECTION
C      = ABS       ABSORPTION CORRECTIONS
C      = EQIV      MERGING EQUIVALENT REFLECTIONS
C      = REFN      FINAL REFINEMENT
C      = STAT      STATISTICS OF REFINEMENT
C      = VAR       NUMBER OF VARIABLES USED IN REFINEMENT
C      = FACE,FARE,ORNT,OUB1,OUB2      NOT STORED
C
C
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      DIMENSION GNAME(9),GDATA(13)
      DIMENSION X(6)
      COMMON/FILES/ NIN,NOUT,NDEBUG,NREAD
      COMMON/GCOND/ GCRAD,GIRAD,GDIFF,GABS,GREFN,GSTAT
      COMMON/COND/ CONDIT(6,8),NSTD,MSTD,CSTD(6,4)
      COMMON /DENS/ Z,DENS,DDENS,CDENS
      DATA GNAME /'CELL','INT ', 'HKL ', 'ABS ', 'EQIV',
     *            'REFN','STAT','VAR','STD '/
C   
      NSTD = 0
      IFLAG = 0
  100 READ(NREAD,1000,END=500) GCHAR,GCID,GR,(X(I),I=1,6)
 1000 FORMAT(BZ,A1,A4,4X,A1,6F10.0)
      IFORM = 11
      IF(GCID.EQ.'    ') THEN
        GIRAD=GR
        IF(GR.EQ.' ') GIRAD='X'
        CONDIT(1,2) = X(1)
        CONDIT(2,2) = X(2)
        CONDIT(5,1) = X(4)
        DENS = X(5)
        CDENS = X(6)
        IFORM = 10
        GOTO 15 
        ENDIF
      DO 25 I=1,9
        IF(GCID .EQ. GNAME(I)) THEN
          IF(I .EQ. 9) THEN
            NSTD = NSTD + 1
            IF(NSTD .LE. MSTD) THEN
              DO 40  J=1,6
                CSTD(J,NSTD) = X(J)
   40           CONTINUE
              IFORM = 9
            ELSE
              WRITE(NOUT,1500)
 1500         FORMAT('*** ARRAY OVERFLOW IN STD, LINE LOST')
              ENDIF
            GOTO 15  
            ENDIF
          GDATA(I) = GR
          DO 50 J=1,6
            CONDIT(J,I) = X(J)
   50       CONTINUE
          IFORM = I
          GOTO 15
          ENDIF
   25   CONTINUE
      IF(GCID .NE. 'EOS ') THEN
        WRITE(NOUT,1200) GCID
 1200   FORMAT('***IGNORED**',A4,' VALUES ARE NOT STORED')
        GOTO 15
        ENDIF  
   15 IF(IFORM.LE.2 .OR. IFORM.EQ.7)THEN
        ASSIGN 1 TO IFORM
      ELSEIF(IFORM .EQ. 3 .OR. IFORM .EQ. 8)THEN
        ASSIGN 2 TO IFORM
      ELSEIF(IFORM .EQ. 9)THEN
        ASSIGN 4 TO IFORM
      ELSEIF(IFORM .EQ. 5)THEN
        ASSIGN 6 TO IFORM
      ELSEIF(IFORM .EQ. 6)THEN
        HDEFLT = '999999999.'
        IF(CONDIT(5,6).GT.9000000.AND.CONDIT(6,6).LT.9000000)THEN   
          IFLAG =1
          ASSIGN 7 TO IFORM
        ELSEif(condit(6,6).gt.9000000.AND.CONDIT(5,6).LT.9000000)
     *    then
          IFLAG =2
          ASSIGN 8 TO IFORM
        ELSE
          IFLAG = 3
          ASSIGN 9 TO IFORM
          ENDIF
      ELSE
        ASSIGN 5 TO IFORM
      ENDIF
      IF (GCHAR .EQ. '*') THEN
        GCRAD = GDATA(1)
        GIRAD = GDATA(2)
        GDIFF = GDATA(3)
        GABS  = GDATA(4)
        GREFN = GDATA(6)
        GSTAT = GDATA(7)
        IF(GCID.EQ.'EOS ') THEN
          WRITE(NOUT,1100) GCHAR,GCID
 1100     FORMAT(1X,A1,A4)
          RETURN
          ENDIF
        IF(IFLAG .EQ. 1)THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,4),HDEFLT
     *          ,X(6)
        ELSEIF(IFLAG.EQ.2)THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,5),HDEFLT
        ELSEIF(IFLAG.EQ.3)THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,4),HDEFLT,
     *         HDEFLT
        ELSE
          WRITE(NOUT,IFORM) GCHAR,GCID,GR,(X(I),I=1,6)
    1     FORMAT(1X,A1,A4,A1,4F10.5,2F10.0)
    2     FORMAT(1X,A1,A4,A1,6F10.0)
    4     FORMAT(1X,A1,A4,A1,3F10.0,3F10.5)
    5     FORMAT(1X,A1,A4,A1,6F10.5)
    6     FORMAT(1X,A1,A4,A1,2F10.0,4F10.5)
    7     FORMAT(1X,A1,A4,A1,4F10.0,A10,F10.5)
    8     FORMAT(1X,A1,A4,A1,5F10.5, A10)
    9     FORMAT(1X,A1,A4,A1,4F10.5, 2A10)
          ENDIF
        RETURN
        ENDIF
        IF(IFLAG .EQ. 1)THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,4),HDEFLT,
     *          X(6)
          ELSEIF(IFLAG.EQ.2)THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,5),HDEFLT
          ELSEIF(IFLAG.EQ.3) THEN
          WRITE(NOUT,IFORM)GCHAR,GCID,GR,(X(I),I=1,4),HDEFLT,HDEFLT
          ELSE
          WRITE(NOUT,IFORM) GCHAR,GCID,GR,(X(I),I=1,6)
          ENDIF
      IFLAG = 0
      GOTO 100      
  500 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
C
C 
C
      SUBROUTINE FORML(IER)
C
C12 ****SUBROUTINE*****          FORML
C                                -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT /86.6.12 N.HEINIG
C
C  READS FORMULA SECTION
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      DIMENSION GDUMMY(6)
      DIMENSION DUMMY(6)
C
      COMMON/FILES/ NIN,NOUT,NDEBUG,NREAD
      common/gform/ gform(24,3)
      COMMON/FORM/ NFORM(3), MFORM, FORM(24,3)
C
      DATA  NFORM, DUMMY / 3*0, 6*0 /
      DATA  GDUMMY / 6*'        '/
C
    1 READ(NREAD,100,END=500) GCH,GCID,(GDUMMY(I),DUMMY(I),I=1,6)
  100 FORMAT(A1,A4,4X,6(A2,F8.4))
      WRITE(NOUT,200) GCH,GCID,(GDUMMY(I),DUMMY(I),I=1,6)
  200 FORMAT(A1,A4,4X,6(A2,F8.4))
      IF(GCID.EQ.'    ' .OR. GCID.EQ.'FORL')THEN 
        L = 1
      ELSEIF(GCID .EQ. 'CRYS') THEN
        L = 2
      ELSEIF(GCID .EQ. 'ANAL') THEN
        L = 3
      ELSEIF(GCID .EQ. 'EOS ') THEN
        RETURN
      ELSE
        WRITE(NOUT,710)
  710   FORMAT(' **ERROR*FORML* UNIDENTIFIED HEADER, LINE LOST.')
        ENDIF
      IF( NFORM(L) .GT. MFORM) THEN
        WRITE(NOUT,210) 
  210   FORMAT(' **ERROR*FORML* STORAGE SPACE TOO SMALL, PREVIOUS ',
     *         'LINE LOST.')
        GOTO 1
        ENDIF
      ICOUNT = NFORM(L)
      DO 15 I = 1,6
        IF(GDUMMY(I) .EQ. '        ' .AND. DUMMY(I) .EQ. 0.) GOTO 25
        ICOUNT = NFORM(L) + I
        GFORM(ICOUNT,L) = GDUMMY(I)
        FORM(ICOUNT,L) = DUMMY(I)
   15   CONTINUE
   25 NFORM(L) = ICOUNT
      IF(GCH .EQ. '*') RETURN
      GOTO 1
  500 IER = -1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE ATOMR(ISEL,IER)
C
C13 ****SUBROUTINE*****          ATOMR
C                                -----
C
C  WRITTEN 80.6.1 BY DOUG THIERS / 84.6.5 D. ALTERMATT / 86.5.21 N.HEINIG
C
C  READS ATOM SECTION
C
C  CID = ATCO             ATOMIC COORDINATES
C      = ATCE             ERRORS IN ATCO
C      = UIJ,BETA,BIJ     ANISOTROPIC TEMPERATURE FACTORS
C      = UIJE,BETE,BIJE   ERRORS IN ANISOTROPIC TEMP. FACTORS
C      = CHEM             CHEMICAL PROPERTIES
C
C       
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      DIMENSION PARAM(6)
      COMMON/RCELL/RA(6),RVOL,RM(3,3)
      COMMON/FILES/ NIN,NOUT,NDEBUG,NREAD
      include 'comatom.for'
      COMMON/DUMMY/ NXH,XNUMH(10)
      COMMON/GDUMMY/ GDH(10)
      DATA KSTAR,IATCO,IATCE,IUIJ,IBETA,IBIJ,IUIJE,IBETE,IBIJE
     +  /'*','ATCO','ATCE','UIJ ','BETA','BIJ ','UIJE','BETE','BIJE'/
      DATA TWOPIS/19.7392/
      DATA IUALL, IUALE, ICHEM /'UALL','UALE','CHEM'/
      DATA IEOS/'EOS '/
*
      IF(ISEL .EQ. 0) THEN
        ASSIGN 100 TO IFM1
        ASSIGN 110 TO IFM2
        ASSIGN 150 TO IFM3
      ELSEIF(ISEL.EQ.1) THEN
        ASSIGN 200 TO IFM1
        ASSIGN 210 TO IFM2
        ASSIGN 250 TO IFM3
        ENDIF
*
    1 READ(NREAD,IFM1,END=29) ICHAR,KID,GDSK,GNAME,GDE,PARAM,IPAR,
     +                GW,GFO,IUF
      IF(ICHAR.EQ.KSTAR.AND.KID.EQ.IEOS) GOTO 999
  100 FORMAT(A1,A4,A4,A2,A3,6F8.5,I4,A1,A4,I1)
  110 FORMAT(16X,A1,A4,A4,A2,A3,6F8.5,I4,A1,A4,I1)
  200 FORMAT(A1,A4,A4,A2,A6,6F8.5,I3,A1,1X,A4,I1)
  210 FORMAT(16X,A1,A4,A4,A2,A6,6F8.5,I3,A1,1X,A4,I1)
*
c  Is atom already stored?
c  -----------------------
      IF(GNAME.EQ.'  ') GNAME=GFO
      JX=0
      IF(NX.NE.0) THEN
        DO 5 I=1,NX
          IF(GELEM(I).EQ.GNAME.AND.Gdent(I).EQ.GDE) JX=I
    5     CONTINUE
        ENDIF
      IF(JX.EQ.0) THEN
        NX=NX+1
        IF(NX.GT.MX) THEN
          NX=NX-1
          WRITE(NOUT,120) MX
  120     FORMAT(' ***ERROR*** NUMBER OF ATOMS IS',
     +  ' GREATER THAN  ',I5,',THE MAXIMUM ALLOWED')
          GOTO 59
          ENDIF
        JX=NX
        GELEM(JX)=GNAME
        Gdent(JX)=GDE
        ENDIF
c
c  ATCO CARDS
C  ----------
      IF(KID.EQ.IATCO)THEN
         DO 60 I1=1,3
           DO 65 I2=1,3
             U(I2,I1,JX)=0.0
   65        CONTINUE
           xx(I1,JX)=PARAM(I1)
   60      CONTINUE
         IF(IUF.LE.1)  THEN
            U(1,1,JX)=PARAM(4)
            DU(2,2,JX)=0.0
            ENDIF
         occu(JX)=PARAM(5)
         OXID(JX)=PARAM(6)
         NPOS(JX)=IPAR
         GWYck(JX)=GW
C
C  MAKE ADJUSTMENTS FOR DUMMY ATOMS
C  (This approach is obsolete - we continue to use x=9.0 as a flag)
C  Trying to use igrp=-1 as a flag
         IF(xx(1,JX) .GT. 8.0) THEN
           igrp(jx)=-1
*           IF(NXH.NE.0) THEN
*             DO 300 J=1,NXH
*               IF(GELEM(JX).EQ.GDH(J)) THEN
*                 XNUMH(J) = XNUMH(J) + NPOS(JX)*OC(JX)
*                 GOTO 310
*                 ENDIF
*  300          CONTINUE
*             ENDIF
*           IF(NXH.EQ.10) THEN
*             WRITE(NOUT,1000)
* 1000        FORMAT(' *****WARNING***** ATTEMPT TO STORE MORE THAN ',
*     +         '10 DUMMY ATOMS.  REMAINDER IGNORED.')
*             GOTO 310
*             ENDIF
*           NXH = NXH + 1
*           GDH(NXH) = GELEM(JX)
*           XNUMH(NXH) = NPOS(JX)*OC(JX)
*  310      CONTINUE
*           DO 312 I=1,3
*             xx(I,NX)=0.0
*             dxx(I,NX)=0.0
*  312        CONTINUE
*           occu(NX)=0.0
*           soccu(NX)=0.0
*           U(1,1,NX)=0.0
*           DU(1,1,NX)=0.0
*           IF(JX.EQ.NX) NX = NX - 1
           ENDIF
c
c   ATCE CARD
C   ---------
       ELSEIF(KID.EQ.IATCE) THEN
         DO 70 I1=1,3
           DO 75 I2=1,3
   75        DU(I2,I1,JX)=0.0
   70      dxx(I1,JX)=PARAM(I1)
         IF(U(2,2,JX).LT.0.0000001) DU(1,1,JX)=PARAM(4)
         soccu(JX)=PARAM(5)
         if(gfo.eq.'   ') then
           IALTNO(JX)=IPAR
         else
           ialtno(jx)=num(gfo,3)
           endif
C
C  ATOMIC DISPLACEMENT PARAMETERS
C  ------------------------------
      ELSEIF(KID.EQ.IUIJ.OR.KID.EQ.IBETA.OR.KID.EQ.IBIJ) THEN
        U(1,2,JX)=PARAM(4)
        U(1,3,JX)=PARAM(5)
        U(2,3,JX)=PARAM(6)
        DO 80 I1=1,3
          U(I1,I1,JX)=PARAM(I1)
          DO 85 I2=I1,3
   85       U(I2,I1,JX)=U(I1,I2,JX)
   80     CONTINUE
        IF(KID.EQ.IUIJ) GOTO 10
          IF(KID.NE.IBETA) GOTO 119
          DO 25 I1=1,3
            DO 28 I2=1,3
   28         U(I1,I2,JX)=U(I1,I2,JX)/(TWOPIS*RA(I1)*RA(I2))
   25       CONTINUE
          GOTO 10
  119     DO 35 I1=1,3
            DO 40 I2=1,3
   40         U(I1,I2,JX)=U(I1,I2,JX)/(8.0*3.1416**2)
   35       CONTINUE
C
C   errors in ATOMIC DISPLACEMENT PARAMETERS
C   ----------------------------------------
        ELSEIF(KID.EQ.IUIJE .OR. KID.EQ.IBETE .OR. KID.EQ.IBIJE)THEN    
          DU(1,2,JX)=PARAM(4)
          DU(1,3,JX)=PARAM(5)
          DU(2,3,JX)=PARAM(6)
          DO 90 I1=1,3
            DU(I1,I1,JX)=PARAM(I1)
            DO 95 I2=I1,3
   95         U(I2,I1,JX)=U(I1,I2,JX)
   90       CONTINUE
          IF(KID.EQ.IUIJE) GOTO 10
          IF(KID.NE.IBETE) GOTO 129
          DO 45 I1=1,3
            DO 50 I2=1,3
   50         DU(I1,I2,JX)=DU(I1,I2,JX)/(2.0*3.1416**2)
   45       CONTINUE
          GOTO 10
  129     DO 55 I1=1,3
            DO 58 I2=1,3
   58         DU(I1,I2,JX)=DU(I1,I2,JX)/(8.0*3.1416**2)
   55       CONTINUE
C
C   overall atomic displacement parameters
c   --------------------------------------
        ELSEIF(KID .EQ. IUALL .OR. KID .EQ. IUALE) THEN
          WRITE(NOUT, 1139)
 1139     FORMAT(' NO STORAGE SPACE READY, THUS CARD IS IGNORED')
c
C   CHEM CARD
C   ---------
        ELSEIF(KID .eq. ICHEM) THEN
          ICN(JX) = PARAM(1)
          NH(JX) = PARAM(2)
          if(abs(oxid(jx)).lt.0.01) OXID(JX) = PARAM(3)  
          igrp(jx)=num(gfo,3)
          ENDIF      
   10 WRITE(NOUT,IFM2)ICHAR,KID,GDSK,GNAME,GDE,PARAM,IPAR,GW,GFO,IUF
      IF(ICHAR.EQ.KSTAR) RETURN
      GOTO 1
   59 WRITE(NOUT,IFM3)ICHAR,KID,GDSK,GNAME,GDE,PARAM,IPAR,GW,GFO,IUF    
  150 FORMAT(1X,'***** SKIPPED  ',A1,A4,A4,A2,A3,6F8.5,I4,A1,A4,I1)
  250 FORMAT(1X,'***** SKIPPED  ',A1,A4,A4,A2,A6,6F8.5,I3,A1,1X,A4,I1)
      IF(ICHAR.EQ.KSTAR) RETURN
      GOTO 1
  999 WRITE(NOUT,140) ICHAR,KID
  140 FORMAT(1X,A1,A4)
      RETURN
   29 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE OPERA(IER)
*
C14 ****SUBROUTINE*****          OPERA
*                                -----
*
*  WRITTEN BY DANIEL ALTERMATT
*
*  LOADS SYMMETRY AND SPECIAL POSITION OPERATORS (NON-STANDARD) SCFS-84
*
C  ISEC = 0 FOR SYMMETRY
C  ISEC = 1 FOR SG OPERATORS (NON-STANDARD)
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
*
      DIMENSION ISYM(3,3),TK(3)
      COMMON/FILES/NIN,NOUT,NDEBUF,NREAD
      COMMON/SYM/NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON/SYMA/MULT(48,48),IT(3,48),NSPE,MSPE,ISPTAB(48,30),
     +      MUSP(30),SP(3,3,30),TSP(3,30)
      COMMON/GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      DATA I4BL,ISTAR,IEOS,ISPE/'    ','*','EOS ','SPEC'/
c
      ISEL = 1
      IF(ISEL.EQ.1) NSPE=0
      ISFL=0
   15 READ(NREAD,1000,END=500) NT,ICID,IDSK,((ISYM(J,K),K=1,3),TK(J),
     +     J=1,3),IA,GB,GC,GD
 1000 FORMAT(BZ,A1,2A4,1X,3(3I2,F10.7),I3,A1,1X,A6,A1)
      IF(NT.EQ.ISTAR.AND.ICID.EQ.IEOS) GOTO 999
      ASSIGN 15 TO JUMP2
      IF(NT.EQ.ISTAR) ASSIGN 400 TO JUMP2
      IF(ICID.EQ.I4BL) THEN
        IF(ISFL.EQ.0) NSYM=0
        ISFL=1
        ENDIF
   25 IF(ICID.NE.I4BL) GOTO 100
*
*---- SYMMETRY OPERATOR OF GENERAL POSITION
*
      NSYM=NSYM+1
      IF(NSYM.GT.MSYM) GOTO 300
      DO 20 J=1,3
      DO 30 K=1,3
   30 S(J,K,NSYM)=FLOAT(ISYM(J,K))
   20 T(J,NSYM)=TK(J)
      WRITE(NOUT,2000) NT,ICID,IDSK,((ISYM(J,K),K=1,3),TK(J),J=1,3)
 2000 FORMAT(1X,A1,2A4,1X,3(3I2,F10.7))
      GOTO JUMP2
*
*---- SYMMETRY OPERATOR TO SPECIAL POSITION
*
  100 IF(ICID.NE.ISPE) GOTO 200
      NSPE=NSPE+1
      IF(NSPE.GT.MSPE) GOTO 300
      DO 120 J=1,3
      DO 130 K=1,3
  130 SP(J,K,NSPE)=FLOAT(ISYM(J,K))
  120 TSP(J,NSPE)=TK(J)
      MUSP(NSPE)=IA
      GSYSP(1,NSPE)=GB
      GSYSP(2,NSPE)=GC
      WRITE(NOUT,2010) NT,ICID,IDSK,((ISYM(J,K),K=1,3),TK(J),J=1,3),
     +     IA,GB,GC,GD
 2010 FORMAT(1X,A1,2A4,1X,3(3I2,F10.7),I3,A1,1X,A6,A1)
      GOTO JUMP2
*
*---- UNKNOWN CARD OR ARRAY OVERFLOW
*
  200 WRITE(NOUT,2020) NT,ICID,IDSK,((ISYM(J,K),K=1,3),TK(J),J=1,3),
     +     IA,GB,GC,GD
 2020 FORMAT(1X,'***** SKIPPED  ',A1,2A4,1X,3(3I2,F10.7),I3,A1,1X,A6,A1)
      GOTO JUMP2
  300 WRITE(NOUT,2030) NT,ICID,IDSK,((ISYM(J,K),K=1,3),TK(J),J=1,3),
     +     IA,GB,GC,GD
 2030 FORMAT(1X,'***** IGNORED  ',A1,2A4,1X,3(3I2,F10.7),I3,A1,1X,A6,A1)
      GOTO JUMP2
*---- END OF SECTION FOUND
  999 WRITE(NOUT,2040) NT,IEOS
 2040 FORMAT(1X,A1,A4)
  400 RETURN
  500 IER=-1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE BONY(IER)
*
C15 ****SUBROUTINE*****          BONY
*                                ----
*
*  WRITTEN BY DANIEL ALTERMATT / 86.6.6 N.HEINIG
*
*  CALLS CLEAR, INTEGER, SORTEL, NUM
*
*  STORE INFORMATION OF BONDS SECTION IN COMMON /BONDS/
*  AND COMMON /CONTAC/ USING LOCAL DEFINITIONS
*
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
*
      DIMENSION GA(4),IBUF(4),XBUF(6)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      include 'comatom.for'
      include 'combond.for'
*
      DATA IBL,ISTAR,IEOS/'    ','*','EOS '/
      DATA KBON,ISYM,KCON,ICSY/'BOND','BSYM','CONT','CSYM'/
*
      IF(NBON .GT. 0) NBON = NBON - 1
      IF(NX.EQ.0) GOTO 90
      IF(NBON .GT. MBON) GOTO 85 
    6 READ(NREAD,1000,END=600) IT,ICID,(GA(I),I=1,4),IBUF(3),
     1   GBUF,(XBUF(J),J=1,2),(XBUF(J),J=4,6),XBUF(3),IBFL
 1000 FORMAT(A1,A4,5X,2(A2,A6),BZ,I2,A1,6F7.4,I4)
      IBUF(4) = NUM(GBUF,1)
      IF(ICID.EQ.IEOS.AND.IT.EQ.ISTAR) GOTO 99
      IF(ICID.EQ.ISYM) THEN
        IF(IBFL.LE.0) IBFL=NBON
        IF(IBFL.EQ.0.OR.IBFL.GT.NBON) GOTO 95
        IBON(2,IBFL)=1
        IBON(3,IBFL)=0
        IBON(4,IBFL)=0
        IBON(5,IBFL)=0
        JBON(2,IBFL)=INTEGER(XBUF(1),0)
        JBON(3,IBFL)=INTEGER(XBUF(2),0)
        JBON(4,IBFL)=INTEGER(XBUF(4),0)
        JBON(5,IBFL)=INTEGER(XBUF(5),0)
        WRITE(NOUT,1005) IT,ICID,(JBON(J,IBFL),J=2,5),IBFL
 1005   FORMAT(1X,A1,A4,24X,4I7,14X,I4)
        IF(IT.EQ.ISTAR) GOTO 500
        GOTO 6
        ENDIF
      IF(ICID.EQ.IBL.OR.ICID.EQ.KBON) THEN
        NBON=NBON+1
        IF(NBON.GT.MBON) GOTO 85
        JBON(2,NBON)=0
        DO 5 J=1,6
          XBON(J,NBON)=XBUF(J)
    5     CONTINUE
        IFBON(1,NBON)=IBUF(3)
C
C  GBUF will determine the value of ifbon(2,nbon) when defined (acyclic/cyclic)
C
        IFBON(4,NBON)=0
        WRITE(NOUT,1010) IT,ICID,(GA(I),I=1,4),IFBON(1,NBON),
     1    (XBON(J,NBON),J=1,2),(XBON(J,NBON),J=4,6),XBON(3,NBON),
     2    IBFL 
 1010   FORMAT(1X,A1,A4,5X,2(A2,A6),I2,1X,6F7.4,I4)
        CALL SORTEL(GA(1),GA(2),GELEM,gdent,J,NX)
        IBON(1,NBON)=J
        IF(J .EQ. -1) GOTO 90
        CALL SORTEL(GA(3),GA(4),GELEM,gdent,J,NX)
        JBON(1,NBON)=J
        IF(J .EQ. -1) GOTO 90
   40   IF(IT.EQ.ISTAR) GOTO 500
        GOTO 6
        ENDIF
      IF(ICID.EQ.KCON) THEN
        NCON=NCON+1
        IF(NCON.GT.MCON) GOTO 85
        WRITE(NOUT,1010) IT,ICID,(GA(I),I=1,4),IFCON(1,NCON),
     1    (XBUF(J),J=1,2),(XBUF(J),J=4,6),XBUF(3),IBFL
        CALL SORTEL(GA(1),GA(2),GELEM,gdent,I,NX)
        ICON(1,NCON)=I
        IF(I .EQ. -1) GOTO 90
        CALL SORTEL(GA(3),GA(4),GELEM,gdent,I,NX)
        JCON(1,NCON)=I
        IF(I .EQ. -1) GOTO 90
  110   DO 120 J=1,6
  120     XCON(J,NCON)=XBUF(J)
        IFCON(1,NCON)=IBUF(3)
        IFCON(2,NCON)=IBUF(4)
C
C  GBUF will determine the value of ifcon(2,ncon) when defined
C  
        IF(IT.EQ.ISTAR) GOTO 500
        GOTO 6
        ENDIF
      IF(ICID.EQ.ICSY) THEN
        IF(IBFL.LE.0) IBFL=NCON
        IF(IBFL.EQ.0.OR.IBFL.GT.NCON) GOTO 95
        ICON(2,IBFL)=INTEGER(XBUF(1),0)
        ICON(3,IBFL)=INTEGER((XBUF(2)/1000.),0)
        ICON(4,IBFL)=INTEGER(XBUF(2),0)-1000*ICON(3,IBFL)
        ICON(5,IBFL)=INTEGER(XBUF(4),0)
        JCON(2,IBFL)=INTEGER(XBUF(5),0)
        JCON(3,IBFL)=INTEGER((XBUF(6)/1000.),0)
        JCON(4,IBFL)=INTEGER(XBUF(6),0)-1000*JCON(3,IBFL)
        JCON(5,IBFL)=INTEGER(XBUF(3),0)
        WRITE(NOUT,1040)IT,ICID,(GA(I),I=1,4),(ICON(I,IBFL),I=2,5),
     1    (JCON(I,IBFL),I=2,5),IBFL
 1040   FORMAT(1X,A1,A4,5X,A2,A6,A2,A6,5X,8I5,I4)
        IF(IT.EQ.ISTAR) GOTO 500
        GOTO 6
        ENDIF
      WRITE(NOUT,1020) IT,ICID,(GA(I),I=1,4),IBUF(3),GBUF,
     1  (XBUF(J),J=1,2),(XBUF(J),J=4,6),XBUF(3),IBFL
 1020 FORMAT(1X,'***** SKIPPED  ',A1,A4,5X,2(A2,A6),I2,A1,6F7.4,I4)
      IF(IT.EQ.ISTAR) GOTO 500
      GOTO 6
   85 IF(NBON.GT.MBON) WRITE(NOUT,1050) MBON
 1050 FORMAT(1X,'MORE THAN',I4,' BONDS LISTED')
      IF(NCON.GT.MCON) WRITE(NOUT,1055) MCON
 1055 FORMAT(1X,'MORE THAN',I4,' CONTACTS LISTED')
   95 WRITE(NOUT,1080) IT,ICID,(GA(I),I=1,4),IBUF(3),GBUF,
     1   (XBUF(J),J=1,2),(XBUF(J),J=4,6),XBUF(3),IBFL
 1080 FORMAT(1X,'***** IGNORED  ',A1,A4,5X,2(A2,A6),I2,A1,6F7.4,I4)
      GOTO 6
   90 WRITE(NOUT,1060)
 1060 FORMAT(1X,'***ERROR***',4X,'ATOM(S) ON BOND/CONTACT CARDS NOT',
     1   'YET READ IN.',//,16X,'ALL BOND/CONTACT INFORMATION DELETED!')
      IF(IT.NE.ISTAR) CALL CLEAR(IER)
      NBON=0
      NCON=0
   99 WRITE(NOUT,1070)
 1070 FORMAT(1X,'*EOS')
  500 RETURN
  600 IER=-1
      RETURN
      END
*
*
*
*
***********************************************************************
*
*
*
*
      SUBROUTINE CONNE(JSYM,IER)
C
C16 ****SUBROUTINE****           CONNE
C                                -----
C  WRITTEN 86/6/03 BY NINA HEINIG
C
C  READS AND STORES CONNECTION DATA INTO COMMON/BONDS/
C
C  CALLS SORTEL(GAN,GAT,GELEM,GDENT,I,NX), INCH
C
C  INPUT:   JSYM(5,192)
C  OUTPUT:  IER
C
C  CID = DIST
C      = VECT
C
C **WARNING** THE CONNECTION TABLE MUST COME BEFORE THE BONDS TABLE AND
C             AFTER THE SYMMETRY TABLE WHEN READING IN OR THE DATA IS LOST
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      DIMENSION REALL(4),INT(2),IXT1(3),IXT2(3), J(2),I(2)
      DIMENSION IP(3),JP(3),IC(3),JC(3)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
      include 'comatom.for'
      include 'combond.for'
      COMMON/SYM/ NSYM, MSYM, S(3,3,48), T(3,48), NLT, TU(3,4)
      DIMENSION JSYM(5,192)
C
C  Programmer's note:  this program assumes DIST and VECT cards of
C  the same bond are together, and, for correct bond type storage,
c  the DIST card must come first.
C
    1 READ(NREAD,1007,END=500) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
 1007 FORMAT(A1,2A4,2(A2,A6,A3,3I2),4F7.4,I2,I1,A1)
      WRITE(NOUT,2007) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,      
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
 2007 FORMAT(1X,A1,2A4,2(A2,A6,A3,3I2),4F7.4,I2,I1,A1)
    2 IF(GCH .EQ. '*' .AND. GCID .EQ. 'EOS ') RETURN
      write(nout,2008) gasym1,gasym2
 2008 format('gasym1= ',A4,'gasym2= ',A4)
      J(1) = INCH(GASYM1)
      J(2) = INCH(GASYM2)
      write(nout,2009) j(1),j(2)
 2009 format('j(1)= ',I4,'j(2)= ',I4)
      IF(J(1).GT.192 .OR. J(1).LE.0) WRITE(NOUT,2017) j(1) 
      IF(J(2).GT.192 .OR. J(2).LE.0) WRITE(NOUT,2017) j(2)
 2017 FORMAT(' *WARNING*CONNE*2017* Symmetry index ',I4,
     +        ' out of range')
      CALL SORTEL(GAN1,GAT1,GELEM,gdent,II,NX)
      CALL SORTEL(GAN2,GAT2,GELEM,gdent,IJ,NX)
      IF(INT(1) .NE. 10) THEN   
c           int=10 flags a connection rather than a bond
        nbon=nbon+1
 2020   if(maxdim(nbon,mbon,'*error*conne*2020*').ne.0)
     +           return
        IBON(1,NBON) = II
        IBON(2,NBON) = JSYM(1,J(1))
        JBON(1,NBON) = IJ
        JBON(2,NBON) = JSYM(1,J(2))
        DO 67  NC = 1,2
          DO 27  ndd = 1,NLT
            I(NC) = JSYM(1,J(NC)) - (ndd-1)*NSYM
            IF( I(NC) .LE. NSYM .AND. I(NC) .GE. 1)GOTO 47
   27       CONTINUE
          do 28 ii=1,192
           write(nout,2028) ii,ii,(jsym(jj,ii),jj=1,5)
 2028      format('jsym(1,',I3,')-jsym(5,',I3,'): ',5(I4,2X))
   28     continue      
          WRITE(NOUT, 2027)  i(nc),ndd,jsym(1,j(nc))
 2027     FORMAT(' *ERROR**CONNE*2027* Symmetry index ',I3, 
     1           'out of range ','ndd: ',I3,' jsym: ',I3)
          goto 67
   47     DO 57 NF=1,3
            IF(NC .EQ. 1) THEN
              JS2 = JSYM(2,J(1))
              IC(NF) = 12.*TU(NF,JS2)
              IP(NF) = 12*IXT1(NF)
              IBON(NF+2,NBON) = IC(NF) + IP(NF) + JSYM(NF+2,J(1))    
            ELSE 
              JS2 = JSYM(2,J(2))
              JC(NF) = 12.*TU(NF,JS2)
              JP(NF) = 12*IXT2(NF)
              JBON(NF+2,NBON) = JC(NF) + JP(NF) + JSYM(NF+2,J(2))
              ENDIF
   57       CONTINUE  
   67     CONTINUE
C
   77   IF(GCID .EQ. 'DIST') THEN
          IFBON(2,NBON) = INT(1)
          XBON(1,NBON) = REALL(1)
          XBON(2,NBON) = REALL(2)
          XBON(3,NBON) = REALL(3)
        ELSEIF( GCID .EQ. 'VECT') THEN
          IFBON(1,NBON) = INT(1)
          XBON(4,NBON) = REALL(1)
          XBON(5,NBON) = REALL(2)
          XBON(6,NBON) = REALL(3)
        ELSEIF(GCID .EQ. 'EOS ') THEN
          RETURN
          ENDIF
        if(gch.eq.'*') go to 400
        READ(NREAD,1007,END=500) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
        WRITE(NOUT,2007) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,      
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
        IF(GAN1.EQ.GELEM(II) .AND. GAN2.EQ.GELEM(IJ) .AND.
     *   GAT1.EQ.gdent(II) .AND. GAT2.EQ.gdent(IJ) .AND.
     *   J(2).EQ. NUM(GASYM2,3)) then
          do 80 k=1,3
            if(12*ixt2(k).ne.jp(k)) go to 2
   80       continue
          go to 77
          endif
        go to 2
C  Store in connections
      ELSE
        ncon=ncon+1
 2022   if(maxdim(ncon,mcon,'*error*conne*2022*')
     +          .ne.0) return
        ICON(1,NCON) = II
        ICON(2,NCON) = JSYM(1,J(1))
        JCON(1,NCON) = IJ
        JCON(2,NCON) = JSYM(1,J(2))
        DO 65  NC = 1,2
          DO 25  ndd = 1,NLT
            I(NC) = JSYM(1,J(NC)) - (ndd-1)*NSYM
            IF( I(NC) .LE. NSYM .AND. I(NC) .GE. 1)GOTO 45
   25       CONTINUE
          WRITE(NOUT, 2025)
 2025     FORMAT(' *ERROR**CONNE*2025* Symmetry index out of range')
          goto 65
   45     DO 55 NF=1,3
            IF(NC .EQ. 1) THEN
              JS2 = JSYM(2,J(1))
              IC(NF) = 12.*TU(NF,JS2)
              IP(NF) = 12*IXT1(NF)
              ICON(NF+2,NCON) = IC(NF) + IP(NF) + JSYM(NF+2,J(1))    
            ELSE 
              JS2 = JSYM(2,J(2))
              JC(NF) = 12.*TU(NF,JS2)
              JP(NF) = 12*IXT2(NF)
              JCON(NF+2,NCON) = JC(NF) + JP(NF) + JSYM(NF+2,J(2))
              ENDIF
   55       CONTINUE  
   65     CONTINUE
C
   75   IF(GCID .EQ. 'DIST') THEN
          IFCON(2,NCON) = INT(1)
          XCON(1,NCON) = REALL(1)
          XCON(2,NCON) = REALL(2)
          XCON(3,NCON) = REALL(3)
        ELSEIF( GCID .EQ. 'VECT') THEN
          XCON(4,NCON) = REALL(1)
          XCON(5,NCON) = REALL(2)
          XCON(6,NCON) = REALL(3)
          IFCON(1,NCON) = INT(1)
        ELSEIF(GCID .EQ. 'EOS ') THEN
          RETURN
          ENDIF
        if(gch.eq.'*') go to 400
        READ(NREAD,1007,END=500) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
        WRITE(NOUT,2007) GCH,GCID,GDSK,GAN1,GAT1,GASYM1,      
     *     IXT1,GAN2,GAT2,GASYM2,IXT2,REALL,INT,GA
        IF(GAN1.EQ.GELEM(II) .AND. GAN2.EQ.GELEM(IJ) .AND.
     *   GAT1.EQ.gdent(II) .AND. GAT2.EQ.gdent(IJ) .AND.
     *   J(2).EQ. NUM(GASYM2,3)) go to 75
        go to 2
        endif
      IF(GCH.NE.'*') GOTO 1
  400 RETURN   
  500 IER = -1
      RETURN
      END 
*
*
***********************************************************************
*
*
*
      SUBROUTINE SORTEL(GAN,GAT,GELEM,GDENT,I,NX)
C
C16B ****SUBROUTINE***       SORTEL
C                            ------
C  WRITTEN JUNE 4,1986 BY N.HEINIG
C  SORTS THROUGH AND FINDS THE SEQUENCE NUMBER OF THE MATCHING ELEMENT
C
      IMPLICIT CHARACTER*8 (G)
      DIMENSION GELEM(nx),GDENT(nx)
      COMMON/FILES/NIN,NOUT,NDEBUG,NREAD
C
      DO 12 I=1,NX
        IF(GAN .EQ. GELEM(I))THEN
          IF (GAT.EQ.GDENT(I)) RETURN
          ENDIF
   12   CONTINUE
      WRITE(NOUT, 1012) GAn,GAt
 1012 FORMAT(' ***ERROR*SORTEL*UNIDENTIFIED ELEMENT LABEL,',2A4)
      I = -1
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE CRYS(IER)
C
C  ****SUBROUTINE****    CRYS
C                        ----
C  WRITTEN 86/06/11 BY NINA HEINIG
C
C  READS AND STORES CRYSTAL PARAMETERS
C
C  CID = SCAL
C        EXTN
C        EXTE
C        ABSR
C        TWIN 
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /HTEXT/ GCRYST(2),GTWIN(5), HTXT(5,10),HDATA(19,4)
      COMMON /CRYST/ SCALE,ESCALE,EXTE(6,2),ABST,ABSTE,TVOL(5),ETVOL(5)
      DIMENSION DUMMY(6)
C
      ITWIN = 0
      MAXTW = 5
    1 READ(NREAD,1000, END=500) GCHAR,GCID,GA,GB,DUMMY
 1000 FORMAT(A1,A4,4X,2A3,6E10.7)
      WRITE(NOUT,2000) GCHAR,GCID,GA,GB,DUMMY
 2000 FORMAT(1X,A1,A4,4X,2A3,6F10.5)
      IF(GCID .EQ. 'SCAL') THEN
        SCALE = DUMMY(1)
        ESCALE = DUMMY(2)
      ELSEIF(GCID .EQ. 'EXTN') THEN
        GCRYST(1) = GA
        GCRYST(2) = GB
        DO 23 I = 1,6
          EXTE(I,1) = DUMMY(I)
   23     CONTINUE
      ELSEIF(GCID .EQ. 'EXTE') THEN
        DO 33  I=1,6
          EXTE(I,2) = DUMMY(I)
   33     CONTINUE
      ELSEIF(GCID .EQ. 'ABSR') THEN
        ABST = DUMMY(1)
        ABSTE = DUMMY(2)
      ELSEIF(GCID .EQ. 'TWIN') THEN
        ITWIN = ITWIN + 1
        IF(ITWIN .GT. MAXTW) THEN
          WRITE(NOUT,2023)
 2023     FORMAT(' **ERROR* DATA OVERFLOWS STORAGE SPACE, LINE LOST')
          GOTO 333
          ENDIF
        GTWIN(ITWIN) = GA
        TVOL(ITWIN) = DUMMY(1)
        ETVOL(ITWIN) = DUMMY(2)
      ELSEIF(GCID .EQ. 'EOS ') THEN
        RETURN
      ELSE
        WRITE(NOUT, 2043)
 2043   FORMAT(' ***ERROR**CRYST* UNIDENTIFIED HEADER')
        ENDIF
  333 IF(GCH .EQ. '*') RETURN
      GOTO 1
  500 IER = -1
      RETURN
      END
*
*
*
*
***********************************************************************
*
*
*
*          
*
      SUBROUTINE WORDS(IER)
C
C   ****SUBROUTINE****        WORDS
C                             -----
C  WRITTEN 86/05/28 BY NINA HEINIG
C
C  READS AND STORES TEXT COMMENTS, FROM CRYSTAL AND COMMENTS
C
C  CID = ABST,INTR,EXPT,DESC,DISC,UPAC,NAME,SRCE,PREP,FORM,PHAS,ANGL,
C        ABS,OUB,EXTN,SSOL,SCAT,WGHT,HYDR,PROG,ORIG,SYOP,SPOS,DISO,
C        SCAL,EXTN,EXTE,ABSR
C
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
C
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /HTEXT/ GCRYST(2),GTWIN(5),HTXT(10,5),HDATA(4,19)
C
      DIMENSION GNAME(24)
      DIMENSION IH(24), ICOUNT(24)
      DATA IH,ICOUNT / 5*10, 19*4, 24*1/
      DATA GNAME / 'ABST','INTR','EXPT','DESC','DISC','UPAC','NAME',
     *     'SRCE','PREP','FORM','PHAS','ANGL','ABS ','OUB ','EXTN',
     *     'SSOL','SCAT', 'WGHT','HYDR','PROG','ORIG','SYOP','SPOS',
     *     'DISO' /
C
    1 READ(NREAD,1004,END=500) GCHAR, HLINE
 1004 FORMAT(A1,A79)
      WRITE(NOUT,2004) GCHAR,HLINE
 2004 FORMAT(1X,A1,A79)
      DO 44 I=1,24
        IF(HLINE(1:4) .EQ. GNAME(I)) THEN
          IF(ICOUNT(I) .LE. IH(I)) THEN
            IF(I .LE. 5) THEN
              HTXT(ICOUNT(I),i) = HLINE(10:75)
            ELSE
              J = I-5
              HDATA(ICOUNT(I),j) = HLINE(10:75)
              ENDIF
            ICOUNT(I) = ICOUNT(I)+1
          ELSE
            WRITE(NOUT, 2044) 
 2044       FORMAT('***WORDS**2044* STORAGE ALLOTTED TOO SMALL,',
     *            ' LINE LOST.')
            ENDIF
          ENDIF
   44   CONTINUE
      IF(GCHAR .EQ. '*')RETURN
      GOTO 1
  500 IER = -1             
      RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE WSCFS(MMM)
*
C
C18 ****SUBROUTINE***     WSCFS
C                         ----- 
* JULY 20-24 82 BY DAVID R. MOSSCROP / 84.6.5 D. ALTERMATT / 86.8.10 N.HEINIG
*
* CALLS MATHLB:INTEGER, JCOMVE
*       MAINLB:ueq
*
* THIS SUBROUTINE PRINTS THE COMMON BLOCK CONTENTS IN STANDARD
* FORMAT, AS DESCRIBED IN " THE STANDARD CRYSTALLOGRAPHIC FILE
* STRUCTURE" BY I. D. BROWN.(ACTA CRYST (1983) A39, 216-224).
*
*
      IMPLICIT CHARACTER*8 (G)
      IMPLICIT CHARACTER*80 (H)
*
      DIMENSION GAI(4)
      DIMENSION IXT(5), JXT(5)
      DIMENSION GCIDF(5)
      DIMENSION JSYM(2,192), XIBO(3),XJBO(3), TTU(3)
*
      COMMON /CELL/  A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,nterm
      COMMON /ECELL/ DA,DB,DC,DALPHA,DBETA,DGAMMA,DM(9,6)
      COMMON /SYM/   NSYM,MSYM,S(3,3,48),T(3,48),NLT, TU(3,4)
      COMMON /GSYM/  GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      COMMON /SYMA/  MULT(48,48),IT(3,48),NSPE,MSPE,ISPTAB(48,30),
     1               MUSP(30),SP(3,3,30),TSP(3,30)
      include 'comatom.for'
      include 'combond.for'
      COMMON /DENS/  ZFORM,DENS,DDENS,CDENS,FWGT,WAVELN,ABSORB,
     1               FZERO,TMPRTR
      COMMON /GTITLE/GENT,HNAME,HCODN(2),HTITLE(2),
     1               HAUTH(10),HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON /TITLE/ JAUTH,KCOMM
      COMMON /HREF/ HINFO(2,2), GFILEN(5),hist(40)
      COMMON /GCOND/ GCRAD,GIRAD,GDIFF,GABS,GREFN,GSTAT
      common /cond/  condit(6,8), nstd,mstd,cstd(6,4)
      COMMON /HTEXT/ GCRYST(2),GTWIN(5),HTXT(10,5),HDATA(4,19)
      COMMON /GFORM/ GFORM(24,3)
      COMMON /FORM/ NFORM(3), MFORM, FORM(24,3)
      COMMON /CRYST/ SCALE,ESCALE,EXTE(6,2),ABST,ABSTE,TVOL(5),ETVOL(5)
      dimension gtext(5),gdata(19),idate(3)
      data gtext/'ABST','INTRO','EXPT','DESC','DISC'/
      data gdata/'UPAC','NAME','SRCE','PREP','FORM','PHAS','ANGL',
     + 'ABS','OUB','EXTN','SSOL','SCAT','WGHT','HYDR','PROG','ORIG',
     + 'SYOP','SPOS','DISO'/
*
      DATA I1BL/' '/
      DATA GCIDF /'PDB ', 'RFCD', 'INOR', 'METL', 'PDF'/
      DIFF=1.0E-7
      IFSPGR=0
      HBLANK = '                                                     
     *            '
      GBLANK = '        '
*
* IF TITLE AVAILABLE PRINT IT OUT.
*
      WRITE(MMM,1000) HNAME,HNAME,GENT
 1000 FORMAT(//'TITLE   ',A66,/,'*',A66,A8,/)
*
* CHECK FOR AND OUTPUT REFERENCE INFORMATION.
* -------------------------------------------
      WRITE(MMM,1005)
 1005 FORMAT('REFERENCE')
      IF(HTITLE(1)(1:8).NE.'        ') WRITE(MMM,1006) HTITLE(1)
      IF(HTITLE(2)(1:8).NE.'        ') WRITE(MMM,1006) HTITLE(2)
 1006 FORMAT(' TITL',5X,A65)
      IF(JAUTH.ne.0) then
        DO 60 I1=1,JAUTH
          WRITE(MMM,1007) HAUTH(I1)
 1007     FORMAT(' AUTH',5X,A65)
   60     CONTINUE
        endif
   70 IF (HCODN(1)(1:8).ne.'        ') then
        WRITE (MMM,1008) HCODN(1)
        IF(HCODN(2)(1:8).NE.'        ') WRITE(MMM,1008) HCODN(2)
 1008   FORMAT (' JRNL',5X,A65)
        endif
   75 IF (HINFO(1,1) .NE. HBLANK)THEN
        WRITE(MMM,1010) 'CAS ',HINFO(1,1)
 1010   FORMAT(1X,A4,5X, A65)
        IF(HINFO(2,1) .NE.HBLANK)WRITE(MMM,1011)'CAS ',HINFO(2,1)
 1011   format(1X,A4,5X,A10)
        ENDIF
      DO 80 I = 1,5
        IF(GFILEN(I) .NE.GBLANK)WRITE(MMM,1012) GCIDF(I),GFILEN(I)
 1012   format(1X,A4,5X,A8)
   80   CONTINUE 
      do 85 i=1,40
   85   if(hist(i).ne.'  ') write(MMM,1010) 'HIST', hist(i)
c 'machine dependent code'
      if (machine.eq.5) then
        call getdat(idate(1),idate(2),idate(3))
        hdate='                                '
        do 87 i=1,3
          gdate=gint(idate(i))
          hdate(3*i:3*i+2)=gdate(7:8)//' '
 87       continue       
      else 
        call date(hdate)
      endif
      hmessage='                                        '
      if(nterm.ne.0) then
        write(nterm,1014)
 1014   format(' Give comment to be included in new version of the',
     +     ' file (max 31 characters)')
        read(nin,1013) hmessage
 1013   format(A31)
        endif
      write(MMM,1015) hdate,hmessage
 1015 format(' HIST             File rewritten ',A15,1X,A31)
   90 WRITE(MMM,1050)
*
* WRITE CELL DIMENSIONS 
* ---------------------
  100 IF(ICELL.EQ.0) GO TO 120
      WRITE(MMM,1020)
 1020 FORMAT('CELL DIMENSION ',4X,'A',9X,'B',9X,'C',5X,'ALPHA',
     1      6X,'BETA',5X,'GAMMA',3X,'Z')
      PI=3.141592654/180.
      AL=ALPHA/PI
      BE=BETA/PI
      AGA=AGAMMA/PI
*
* WRITE A,B,C,ALPHA,BETA,GAMMA,ZFORM (Z).
* ---------------------------------------
      WRITE(MMM,1030)A,B,C,AL,BE,AGA,ZFORM
 1030 FORMAT(1X,'CELL',5X,6F10.4,F5.0)
*
* CHECK IF STANDARD ERRORS HAVE BEEN GIVEN FOR THE CELL
* DIMENSIONS.
*
      IF(DA .GT. DIFF) THEN
        WRITE(MMM,1040)DA,DB,DC,DALPHA,DBETA,DGAMMA
 1040   FORMAT(' ERRS',5X,6F10.4)
        ENDIF
C
C WRITE VOLUME, FORMULA WEIGHT AND DENSITY.
C -----------------------------------------
      IF(VOL .GT. 0) THEN
        WRITE(MMM,1045)VOL,FWGT,CDENS
 1045   FORMAT(' VOL ',5X,F10.4,10X,2F10.4)
        ENDIF
      IF(DENS .GT. 0) WRITE(MMM,1047) DENS
 1047 FORMAT(' PHYS',5X,F10.4)
C
  110 WRITE(MMM,1050)
 1050 FORMAT('*EOS',/)
*
* WRITE SPACE GROUP SYMBOL(S), EITHER HALL AND/OR HERMANN-MAUGUIN.
* FIRST CHECK IF BOTH ARE PRESENT.
* ----------------------------------------------------------------
  120 IF(HALL(1:7).EQ.'       '.AND.HMSYM(1:7).EQ.'       ') GOTO 160
      IFSPGR=1
      WRITE(MMM, 1052)
 1052 FORMAT('SG NAME')
      if(hall(1:8).ne.gblank) WRITE(MMM,1060) HALL
 1060 FORMAT(1X,'HALL',5X,A24)
      IF(HMSYM(1:8).NE. GBLANK) WRITE(MMM,1065) HMSYM
 1065 FORMAT(1X,'HERM',5X,A24)
      ggcent=gcent
      if(gcent.eq.'A'.or.gcent.eq.'a') ggcent='N'
      write(MMM,1066) ggcent,glat
 1066 format(' LATT',5X,2A1)
      WRITE(MMM,1050)
*
* WRITE SYMMETRY OPERATORS.
* -------------------------
  160 IF(NSYM.EQ.0) GO TO 170
      WRITE(MMM,1080)
 1080 FORMAT('SYMMETRY')
      CALL INCREA(JSYM,MMM)
  170 IF(NSPE.EQ.0) GOTO 174
      WRITE(MMM,1095) (((INTEGER(SP(I,J,K),0),J=1,3),TSP(I,K),I=1,3),
     1     K, MUSP(K),K=1,NSPE)
 1095 FORMAT((1X,'SPOS',5X,2(3I2,F10.7,4X),3I2,F10.7,I3,I2))
  174 WRITE(MMM,1050)
*
* WRITE ATOM COORDINATE INFORMATION.
* ----------------------------------
  175 IF(NX.EQ.0) GO TO 270
*
* ATOM INFO STORED.
*
      WRITE(MMM,1100)
 1100 FORMAT('ATOMS',13X,'X',7X,'Y',7X,'Z',7X,'U(EQ)   OCC',5X,'VAL',
     1  '    WYCK AT  TF')
*
* THIS LOOP PRINTS OUT ATCO, ATCE, UIJ AND UIJE CARDS (IF
* INFORMATION AVAILABLE) ORDERED WITH RESPECT TO ELEMENTS
*
      DO 200 I=1,NX
        IF(GELEM(I).EQ.'  ') GO TO 270
        CALL UEQ(UU,DUU,I)
        ITFL=1
        IF(U(2,2,I).GT.DIFF) ITFL=2
        occup=occu(i)
        if(abs(occup).lt.0.0001) occup=1.0
        WRITE(MMM,1110)GELEM(I),GDENT(I),XX(1,I),XX(2,I),XX(3,I),
     1       UU,occup,OXID(I),NPOS(I),GWYCK(I),ITFL
 1110   FORMAT(' ATCO',4X,A2,A6,6F8.5,I3,A1,5X,I1)
*
* CHECK FOR STANDARD ERRORS ASSOCIATED WITH THIS ATCO.
*
        IFMAT = 8
        IF (SOCCU(I).LT.DIFF) IFMAT = IFMAT - 4
        IF (DUU.LT.DIFF) IFMAT = IFMAT - 2
        IF (DXX(1,I)+DXX(2,I)+DXX(3,I).LT.DIFF) IFMAT = IFMAT - 1
        IF (IFMAT.EQ.1.AND.IALTNO(I).NE.0) GOTO 199
        GO TO (210,185,187,189,191,193,195,197),IFMAT
  185   WRITE(MMM,1140)GELEM(I),GDENT(I),(DXX(J,I),J=1,3),IALTNO(I) 
 1140   FORMAT (' ATCE',4X,A2,A6,3F8.5,29X,I3)
        GO TO 210
  187   WRITE (MMM,1145) GELEM(I),GDENT(I),DUU,IALTNO(I)
 1145   FORMAT (' ATCE',4X,A2,A6,24X,1F8.5,21X,I3)
        GO TO 210
  189   WRITE (MMM,1142) GELEM(I),GDENT(I),(DXX(J,I),J=1,3),DUU,
     1  IALTNO(I)
 1142   FORMAT (' ATCE',4X,A2,A6,4F8.5,21X,I3)
        GO TO 210
  191   WRITE (MMM,1150) GELEM(I),GDENT(I),SOCCU(I),IALTNO(I)
 1150   FORMAT (' ATCE',4X,A2,A6,32X,F8.5,13X,I3)
        GO TO 210
  193   WRITE (MMM,1155) GELEM(I),GDENT(I),(DXX(J,I),J=1,3),
     1         SOCCU(I), IALTNO(I)
 1155   FORMAT (' ATCE',4X,A2,A6,3F8.5,8X,F8.5,13X,I3)
        GO TO 210
  195   WRITE (MMM,1147) GELEM(I),GDENT(I),DUU,SOCCU(I),IALTNO(I)
 1147   FORMAT (' ATCE',4X,A2,A6,24X,2F8.5,13X,I3)
        GO TO 210
  197   WRITE (MMM,1144) GELEM(I),GDENT(I),(DXX(J,I),J=1,3),DUU,
     1      SOCCU(I),IALTNO(I)
 1144   FORMAT (' ATCE',4X,A2,A6,5F8.5,13X,I3)
        GOTO 210
  199   WRITE (MMM,1160) GELEM(I),GDENT(I),IALTNO(I)
 1160   FORMAT (' ATCE',4X,A2,A6,53X,I3)
*
* SEARCH FOR ANISOTROPIC TEMPERATURE FACTORS AND
* THEIR ERRORS. IF EITHER OR BOTH ARE FOUND THEY ARE PRINTED.
*
  210   IF(ITFL.EQ.1) GOTO 205
        WRITE(MMM,1170) GELEM(I),GDENT(I),(U(J,J,I),J=1,3),U(1,2,I)
     1         ,U(1,3,I),U(2,3,I)
 1170   FORMAT(' UIJ ',4X,A2,A6,6F8.5)
        IF(DU(2,2,I).LT.DIFF) GO TO 205
        WRITE(MMM,1180) GELEM(I),GDENT(I),(DU(J,J,I),J=1,3),
     1    DU(1,2,I),DU(1,3,I),DU(2,3,I)
 1180   FORMAT(' UIJE',4X,A2,A6,6F8.5)
C
C  WRITE OUT CHEMICAL QUANTITIES
C
  205   CN = ICN(I)
        RH = NH(I)
        WRITE(MMM, 1187)GELEM(I),GDENT(I), CN,RH, OXID(I),igrp(I)
 1187   FORMAT(' CHEM',4X,A2,A6, 3F8.5, 24X, I3)
  200   CONTINUE
      WRITE(MMM,1050)       
*
*  WRITE MACHINE-READABLE FORMULA
*  ------------------------------
  270 NN = NFORM(1) + NFORM(2) + NFORM(3)
      IF(NN .EQ. 0) GOTO 310
      WRITE (MMM,1190)
 1190 FORMAT ('FORMULA')
      IF(NFORM(1) .GT. 0) THEN
        WRITE(MMM, 1193) (GFORM(NF,1),FORM(NF,1),NF=1,NFORM(1))
 1193   FORMAT((1X,'FORL',4X,6(A2,F6.2,2X)))
        ENDIF
      IF(NFORM(2) .GT. 0) THEN
        WRITE(MMM,1195) (GFORM(NF,2),FORM(NF,2),NF=1,NFORM(2))
 1195   FORMAT((1X,'CRYS',4X,6(A2,F6.2,2X)))
        ENDIF
      IF(NFORM(3) .GT. 0) THEN
        WRITE(MMM,1197) (GFORM(NF,3),FORM(NF,3),NF=1,NFORM(3))
 1197   FORMAT((1X,'ANAL',4X,6(A2,F8.0)))
        ENDIF
      WRITE (MMM,1050)
*
* WRITE BONDS AND CONTACTS
* ----------------------------
  310 IF(NBON.NE.0.OR.NCON.NE.0) WRITE(MMM,1310)
 1310 FORMAT('CONNECTIONS')
      IF(NBON.EQ.0) GOTO 275
      DO 320 I=1,NBON
C
C  Assign element identifiers to bonds
C
        GAI(1)=GELEM(IBON(1,I))
        GAI(2)=GDENT(IBON(1,I))
        GAI(3)=GELEM(JBON(1,I))
        GAI(4)=GDENT(JBON(1,I))
C
C  Assign default symmetry operator nos., calculate symmetry matrix (i.e. IXT)
C
        IB = 0
        JB = 0
        DO 307 NI= 1,3
          XIBO(NI) = ABS(MOD(IBON(NI+2,I),12))
          XJBO(NI) = ABS(MOD(JBON(NI+2,I),12))
          IXT(NI) = (IBON(NI+2,I) - XIBO(NI))/12
          JXT(NI) = (JBON(NI+2,I) - XJBO(NI))/12
  307     CONTINUE
C
C  Compare XIBO,XJBO with rows of TU to determine symmetry op. number
C  If ICOMVE = 0 there is a vector match
        DO 317 J=1,NLT
          DO 312 K=1,3
            TTU(K) = TU(K,J)*12.
  312       CONTINUE
          IF(ICOMVE(XIBO,TTU,3,2) .EQ. 0) THEN
            IB = IBON(2,I) + NSYM*(J-1)
            ENDIF
          IF(ICOMVE(XJBO,TTU,3,2) .EQ. 0) THEN
            JB = JBON(2,I) + NSYM*(J-1)
            ENDIF
          IF(IB.NE.0 .AND. JB.NE.0) GOTO 327
  317     CONTINUE
        IB = IBON(2,I)
        JB = JBON(2,I)
  327   IF(IB.LE.0 .AND. JB.LE.0) THEN
          WRITE(NOUT,1327) I,IB,JB
 1327     FORMAT(' *WARNING*WSCFS*1327* invalid symmetry no. in bonds'
     *        , ' at bond no.',I3,'.  Symmetry nos. are:',2I4,
     *        '.  Symmetry nos. are now set to 1.' )
          IB = 1
          JB = 1
          ENDIF
c  ibt is the bond type- 1 for bonds, 10 for connections
        ibt=1
        WRIte(MMM, 1340) GAI(1),GAI(2),IB,(IXT(J),J=1,3),
     *       GAI(3), GAI(4), JB, (JXT(K),K=1,3),
     *       (XBON(KI,I), KI=1,3),ibt
 1340   FORMAT(1X,'DIST',4X,2(A2,A6,I3,3I2),3F7.4,7X,I2)
        WRITE(MMM, 1350)GAI(1),GAI(2),IB,(IXT(J),J=1,3),
     *       GAI(3),GAI(4),JB,(JXT(K),K=1,3),
     *       (XBON(KI,I), KI=4,6),ifbon(1,i)
 1350   FORMAT(1X,'VECT',4X,2(A2,A6,I3,3I2),3F7.4,7X,I2 )
  320   CONTINUE
C
C  Check if there are any connections.
C
      IF(NCON.EQ.0) WRITE(MMM,1050)
  275 IF(NCON.EQ.0) GOTO 340
      DO 330 I=1,NCON
C
C  Assign element identifiers to connections
C
        GAI(1)=GELEM(ICON(1,I))
        GAI(2)=GDENT(ICON(1,I))
        GAI(3)=GELEM(JCON(1,I))
        GAI(4)=GDENT(JCON(1,I))
C
C  Assign default symmetry operator nos., calculate symmetry matrix (i.e. IXT)
C
        IB = 0
        JB = 0
        DO 335 NI= 1,3
          XIBO(NI) = ABS(MOD(ICON(NI+2,I),12))
          XJBO(NI) = ABS(MOD(JCON(NI+2,I),12))
          IXT(NI) = (ICON(NI+2,I) - XIBO(NI))/12
          JXT(NI) = (JCON(NI+2,I) - XJBO(NI))/12
  335     CONTINUE
C
C  Compare XIBO,XJBO with rows of TU to determine symmetry op. number
C
        DO 341 J=1,NLT
          do 342 k=1,3
  342       ttu(k)=12.*tu(k,j)
          IF(ICOMVE(XIBO,TTU,3,2) .EQ. 0) THEN
            IB = ICON(2,I) + NSYM*(J-1)
            ENDIF
          IF(ICOMVE(XJBO,TTU,3,2) .EQ. 0) THEN
            JB = JCON(2,I) + NSYM*(J-1)
            ENDIF
          IF(IB.NE.0 .AND. JB.NE.0) GOTO 347
  341     CONTINUE
        IB = ICON(2,I)
        JB = JCON(2,I)
  347   IF(IB.LE.0 .AND. JB.LE.0) THEN
          WRITE(NOUT,1360) I,IB,JB
 1360     FORMAT(' *WARNING*WSCFS*1360* invalid symmetry no. in '
     *        ,'connections at no.',I3,'.  Symmetry nos. are:',2I4,
     *        '.  Symmetry nos. are now set to 1.' )
          IB = 1
          JB = 1
          ENDIF
        ibt=10
        WRITE(MMM, 1365) GAI(1),GAI(2),IB,(IXT(J),J=1,3),
     *       GAI(3), GAI(4), JB, (JXT(K),K=1,3),
     *       (XCON(KI,I), KI=1,3),ibt
 1365   FORMAT(1X,'DIST',4X,2(A2,A6,I3,3I2),3F7.4,7X,I2)
        WRITE(MMM, 1369)GAI(1),GAI(2),IB,(IXT(J),J=1,3),
     *       GAI(3),GAI(4),JB,(JXT(K),K=1,3),
     *       (XCON(KI,I), KI=4,6),ifCon(1,i)
 1369   FORMAT(1X,'VECT',4X,2(A2,A6,I3,3I2),3F7.4,7X,I2 )
  330   CONTINUE
      WRITE(MMM,1050)
*
* WRITE REMARKS 
* -------------
  340 IF(KCOMM.EQ.0) GO TO 300
      WRITE(MMM,1200)
 1200 FORMAT('REMARK')
      write(mmm,1205) grfact
 1205 format(' The R-Factor = ',A5)
      DO 280 I=1,KCOMM
        WRITE(MMM,1210) HCOMM(I)
 1210   FORMAT(' ',A74)
  280   CONTINUE
      WRITE(MMM,1050)
*
* WRITE CONDITIONS
* ----------------
  300 WRITE(MMM,1400)
 1400 FORMAT('CONDITIONS')
      GID='CELL'
      WRITE(MMM, 1429) GID,GCRAD,(CONDIT(I,1), I=1,6)
      GID='INT '
      WRITE(MMM, 1421) GID,GIRAD,(CONDIT(I,2), I=1,6)
      GID='HKL '
      WRITE(MMM, 1422) GID,GDIFF,(CONDIT(I,3), I=1,6)
      GID='STD '
      DO 47  IJ=1,NSTD 
        WRITE(MMM, 1424) GID,' ',(CSTD(I,IJ), I=1,6)
   47   CONTINUE
      GID='ABS '
      WRITE(MMM, 1425) GID,GABS,(CONDIT(I,4), I=1,6)
      GID='EQIV'
      WRITE(MMM, 1426) GID,' ',(CONDIT(I,5), I=1,6)
      GID='REFN'
      HDEFLT = '999999999.'
      IF(CONDIT(5,6).GT.90000000 .AND. CONDIT(6,6).LT.90000000) THEN
        WRITE(MMM,1427) GID,GREFN,(CONDIT(I,6), I=1,4),HDEFLT,
     *       CONDIT(6,6)
      ELSEIF(CONDIT(5,6).LT.900000000.AND.CONDIT(6,6).GT.900000000)
     *       THEN 
        WRITE(MMM,1428) GID,GREFN,(CONDIT(I,6), I=1,5), HDEFLT
      ELSE
        WRITE(MMM,1431)GID,GREFN,(CONDIT(I,6),I=1,4),HDEFLT,HDEFLT
        ENDIF
      GID='STAT'
      WRITE(MMM, 1430) GID,GSTAT,(CONDIT(I,7), I=1,6)
      GID='VAR '
      WRITE(MMM, 1422) GID,' ',(CONDIT(I,8), I=1,6)
 1421 FORMAT(1X, A4,4X, A1, 4F10.5, 2F10.0 )
 1422 FORMAT(1X, A4,4X, A1, 6F10.0)
 1424 FORMAT(1X, A4,4X, A1, 3F10.0 ,3F10.5,1X)
 1425 FORMAT(1X, A4,4X, A1, 6F10.5 )
 1426 FORMAT(1X, A4,4X, A1, 2F10.0,4F10.5 )
 1427 FORMAT(1X, A4,4X, A1, 4F10.5 ,A10, F10.5)
 1428 FORMAT(1X, A4,4X, A1, 5F10.5, A10 )
 1429 FORMAT(1X, A4,4X, A1, 2F10.5, 4F10.0 )
 1430 FORMAT(1X, A4,4X, A1, 3F10.5, 3F10.0)
 1431 FORMAT(1X, A4,4X, A1, 4F10.5 ,2A10)
      WRITE(MMM,1050)
C
C WRITE CRYSTAL DATA 
C ------------------
      WRITE(MMM,1500)
 1500 FORMAT('CRYSTAL')
      IF(SCALE .NE. 0) WRITE(MMME,1507)'SCAL',SCALE,ESCALE
 1507 FORMAT(1X,A4,10X,2F10.5)
      IF(GCRYST(1) .NE. GBLANK) THEN
        WRITE(MMM, 1513) GCRYST(1),GCRYST(2),(EXTE(I,1),I=1,6)
 1513   FORMAT(1X,'EXTN',4X,2A3,6F10.5)
        WRITE(MMM, 1516) GCRYST(1),GCRYST(2),(EXTE(I,2),I=1,6)
 1516   FORMAT(1X,'EXTE',4X,2A3,6F10.5)
        ENDIF
      IF(ABS(ABST).GT.0.0001) WRITE(MMM,1507) 'ABSR',ABST,ABSTE
      DO 507 I=1,5
        IF(GTWIN(I) .NE. GBLANK) WRITE(MMM,1519)GTWIN(I),
     *                               TVOL(I),ETVOL(I)
 1519   FORMAT(1X,'TWIN', 4X, A3, 3X, 2F10.0)
  507   CONTINUE
      WRITE(MMM,1050)
C
C WRITE COMMENTS
C --------------
      WRITE(MMM,1570)
 1570 FORMAT('COMMENTS')
      if(hdata(1,5)(1:8).eq.gblank) then
        hdata(1,5)=hform(1)
        hdata(2,5)=hform(2)
        endif
      DO 532 I=1,5
        DO 531 J=1,10
          IF(HTXT(j,i)(1:8) .EQ. GBLANK ) GOTO 532
          WRITE(MMM,1521) gtext(i),HTXT(j,i)
 1521     FORMAT(1X, A4,5X,A65)
  531     CONTINUE
  532   CONTINUE
      DO 542 I=1,19
        DO 541 J=1,4
          IF(HDATA(j,i)(1:8).EQ. GBLANK) GOTO 542
          WRITE(MMM,1521) GDATA(I), HDATA(j,i)
  541     CONTINUE
  542   CONTINUE
      WRITE(MMM,1050)
*
* PRINT LIST OF KEY DATA THAT WAS MISSING.
* ----------------------------------------
      WRITE(MMM,1220)
 1220 FORMAT('END    ',/)
      IF((ICELL.EQ.1).AND.(IFSPGR.NE.0).AND.
     1  (NX.NE.0)) GO TO 400
      WRITE(NOUT,1250)
 1250 FORMAT(//,' THE FOLLOWING KEY INFORMATION IS NOT PROVIDED.',/,
     1  ' THIS MAY CAUSE DIFFICULTIES LATER ON.',/)
      IF(ICELL.EQ.0) WRITE(NOUT,1270)
 1270 FORMAT(' CELL DIMENSIONS MISSING.',/)
      IF(IFSPGR.EQ.0) WRITE(NOUT,1280)
 1280 FORMAT(' SPACE GROUP SYMBOL MISSING.',/)
      IF(NX.EQ.0)WRITE(NOUT,1290)
 1290 FORMAT(' ATOM COORDINATE INFORMATION MISSING.',/)
  400 RETURN
      END
*
*
*
***********************************************************************
*
*
*
      SUBROUTINE INCREA(JSYM,MMM)
C
C18B ****SUBROUTINE****     INCREA
C                           ------
C WRITTEN 86/06/05 BY NINA HEINIG
C 
C GIVEN TU AND THE INITIAL SYMMETRY OPERATORS, CREATE AND
C WRITE OUT ALL THE COMPONENTS
C
C OUTPUT:  JSYM(1,192) = number of symmetry operator (in S)
C          JSYM(2,192) = number of row in TU
C
      IMPLICIT CHARACTER*8 (G)
C
      COMMON /SYM/ NSYM,MSYM,S(3,3,48),T(3,48),NLT,TU(3,4)
      COMMON /FILES/ NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
      COMMON /GSYM/ GLAT,GCENT
      DIMENSION DS(3,3,48),DT(3,48), IT(3,48),IDT(3,48)
      DIMENSION JSYM(2,192)
C
      ICOUNT = 0
      IF(NDEBUG .NE. 0) WRITE(NDEBUG, 2000)NSYM,NLT,TU
 2000 FORMAT(' *DEBUG**INCREA*2000 NSYM,NLT,TU:',2I4,4(/,1X,3F5.2))
      DO 17 I=1,4
        DO 17 J=1,3
          IF(TU(J,I).NE.0.) THEN
            CALL MAKETU
            GOTO 18
            ENDIF
   17     CONTINUE
   18 DO 10 K=1,NSYM
        WRITE(MMM,1000)((INTEGER(S(I,J,K),0),J=1,3),T(I,K),I=1,3),K
 1000   FORMAT(1X,'SYOP',5X,2(3I2,F10.7,4X),3I2,F10.7,I3)
        ICOUNT = ICOUNT + 1
        JSYM(1,ICOUNT) = K
        JSYM(2,ICOUNT) = 1
   10   CONTINUE
      DO 50 IJ=2,NLT
        DO 40 K=1,NSYM
          DO 30 I=1,3
            DO 20 J=1,3
              DS(I,J,K) = S(I,J,K)
   20         CONTINUE
            IT(I,K) = T(I,K)*12.
            IDT(I,K) = IT(I,K) + integer(12.*TU(I,IJ),0)
            IDT(I,K) = MOD(IDT(I,K),12)
            DT(I,K) = IDT(I,K)/12.
   30       CONTINUE
          ICOUNT = ICOUNT + 1
          JSYM(1,ICOUNT) = K
          JSYM(2,ICOUNT) = IJ
          WRITE(MMM,1000)( (INTEGER(DS(I,J,K),0),J=1,3),
     *         DT(I,K),I=1,3) ,ICOUNT
   40     CONTINUE                                                      
   50   CONTINUE
      GLAT = 'P'
      RETURN
      END


c
c  Temporary subroutine installed until method of finding date from 
c  the PC can be found.
c
      subroutine date(hdate)
      implicit character*80 (h)
      hdate='dummy routine'
      return
      end
      
