C
C               FILE SHELIB 1985                 Last update: 17 Oct 90
C               ----------------                              S. Healy
C
C    This file contains the following routines:
C
C    SHELIB  ; Main routine for reading and processing a SHELX file
C    INSKEY  ; Function to determine the type of instruction
C    PROATM  ; Processes an ATOM instruction
C    PROINS  ; Processes a NORMAL SHELX instruction
C    RDREAL  ; Reads a specified number of real numbers from a line
C    SKIPINS ; Skips blank lines and meaningless instructions
C    XCONV   ; Converts real numbers read into correct values
C              ( see description in XCONV )
C
C    EXTERNALS CALLED IN HIS ROUTINE INCLUDE
C             MAINLB(PARAM,sghall)
C             MATHLB(SCAN,UPCASE) 
C             SPELIB(SPOST,SPESOR,xaltno)
C
C****** STANDARD FORTRAN 77 CODE ONLY USED IN THIS FILE
C
C
C
C-----------------------------------------------------------------------
        SUBROUTINE SHELIB (NSHELL)
C-----------------------------------------------------------------------
C
C Written by David Clarkson  JULY 1985
C
C Purpose : To read and process a SHELX file
C
C Input   : NSHELL ; the unit number of the SHELX data file
C
C Output  : various common block values are loaded
C
C-----------------------------------------------------------------------
C
      IMPLICIT CHARACTER*80 (H)
      IMPLICIT CHARACTER*8  (G)
C
C  Common blocks
C
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP,MACHINE,NTERM
      COMMON /CELL/   A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON /ECELL/  DA,DB,DC,DALPHA,DBETA,DGAMMA
      include 'comatom.for'
      COMMON /DENS/   ZFORM,DENS,DDENS,CDENS,FWGT,WAVELN,ABSORB,
     1                  FZERO,TMPRTR
      COMMON /GSYM/GLAT,GCENT,HMSYM,HALL,GSYSP(2,30)
      COMMON /SYMA/ MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30),
     1              MULTSP(30),SP(3,3,30),TP(3,30)
      COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     1                  HFORM(2),HCOMM(7),GRFACT,GRAD
C
C  VAR is a common block to hold the Free-variables from a SHELX
C  output file
C
        COMMON /VAR/    NVAR,MVAR,FVAR (100)
        DIMENSION NT(3)
        dimension ginst(31)
C
C  Known SHELX instructions
C
        DATA GINST /'TITL','CELL','FVAR','END ','LATT','SYMM','SFAC',
     1              'ABSC','UNIT','AXIS','FACE','HKLF','MERG','OMIT',
     2              'EERS','TANG','PHIX','L.S.','AFLS','BOND','LIST',
     3              'FMAP','GRID','PLAN','WGHT','ILSF','ANIS','AFIX',
     4              'BLOC','UNDO','DFIX'/
C
        MVAR=100
        NVAR=0
C
C
C  Read and interpret instruction from input file
C
  10    READ (NSHELL,1000,END=999) HLINE
        WRITE (NOUT,'(1X,A80)') HLINE
1000    FORMAT (A80)
  20    IP=1
        IRFL=0
        CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
C
C  Convert instruction read to upper-case for identification
C
        IF (IRFL.EQ.0) GOTO 10
        CALL UPCASE (GCHAR)
C
C  Determine if the instruction is one which contains information we can
C  use or is an Atom instruction.
C
C      Normal instructions are indicated by KEY=1
C      Atom instructions are indicated by   KEY=3
C      (KEY=2 signals irrelevant instructions)
C
C  Also, normal instructions are in the following form :
C  ( see list of known SHELX instructions )
C           4 characters     ( SCAN returns IRFL=1 )
C
C        Atom instructions can be of the following form :
C
C           Any combination of characters and numbers up to 4 characters
C           ( SCAN returns IRFL=4 )
C
        key=inskey(gchar,ginst,irfl)
        if(ndebug.ne.0) then
          write(ndebug,1002) gchar,irfl,key
 1002     format(' *debug*shelib*1002* gchar=',A4,', irfl=',I2,
     1           ', key=',I2)
          endif
        IF (IRFL.EQ.1) THEN
          IF (KEY.EQ.1) THEN
            CALL PROINS (HLINE,NSHELL)
          ELSE
            IF (KEY.EQ.3) CALL PROATM (HLINE,NSHELL)
          ENDIF
        ELSE
          IF (IRFL.EQ.4) THEN
            CALL PROATM (HLINE,NSHELL)
          ELSE
            WRITE (NOUT,1005) GCHAR
1005        FORMAT ('0+++ WARNING +++ SHELIB 1005',/,
     1          16X,'Invalid instruction ',A8,/,
     2          16x,'Continuation marker probably missing',/,
     3          16X,'Line ignored')
            GOTO 10
          ENDIF
        ENDIF
C
C  After an instruction has been read and processed, skip blank lines
C  or instructions not of the correct format and repeat until all
C  instructions have been processed.
C
        CALL SKIPINS (HLINE,NSHELL,IEND)
        IF (IEND.EQ.1) GOTO 25
        GOTO 20
C
C  AT THIS POINT WE PROMPT FOR INPUT FROM TERMINAL IF IN INTERATIVE MODE
C
 25     IF (NTERM.NE.6) GOTO 999
   26   WRITE(NTERM,2000)
 2000   FORMAT (' ','Do you wish to enter standard errors for cell',
     1    'parameters, Y/N/E?')
        call readyn(*27,*30,*999,*26)
 27     WRITE (NTERM,2010)
 2010   FORMAT (' ','Enter standard errors in format: error in "a",',
     1  ' "b", "c", "alpha", "beta", "gamma"')
        write (NTERM,2020)
 2020   FORMAT (' ','Enter zeros if no error given')
c note: cannot bracket information below under UNIX F77
        READ (NIN,*) DA,DB,DC,DALPHA,DBETA,DGAMMA
C
C   PROMPT FOR HALL SPACE GROUP SYMBOL
C
 30     WRITE (NTERM,2030)
 2030   FORMAT (' ','Enter Hall Space Group symbol, Blank if not ',
     1  'desired')
        read (nin,'(A80)') hall
        IF (HALL(1:5).NE.' ') THEN
          CALL SGHALL
          CALL SPOST (IER,0)
          CALL SPESOR
        ENDIF
C
C      GENERATE NEW MULTIPLICITY FOR SCFS
C
        DO 40 I=1,NX
        NPOSS=0
        CALL XALTNO(XX(1,I),NPOSS,IALTNO(I),NALTS,ICENT,NT,ZZ)
        IF (IALTNO(I).NE.0) THEN
          IF (MULTSP(IALTNO(I)).NE.0) THEN
            OCCU(I)=((OCCU(I)*MULTSP(1))/MULTSP(IALTNO(I)))
          ENDIF
        ENDIF
 40     CONTINUE       
 999    RETURN
        END
C
C
C-----------------------------------------------------------------------
        FUNCTION INSKEY (GCHAR,ginst,IRFL)
C-----------------------------------------------------------------------
C
C Written by David Clarkson JULY 1985
C
C Purpose : To determine what type of instruction we are working on
C           ie. Is it a NORMAL SHELX instruction ?
C               Is it an ATOM instruction ?
C
C Input   : GCHAR  ; instruction
C           IRFL   ; SCAN code indicating what type of string it is
C                    ( ie. alphabetic,alphanumeric etc...)
C
C-----------------------------------------------------------------------
C
        IMPLICIT CHARACTER *80 (H)
        IMPLICIT CHARACTER *8  (G)
        common/files/nin,nout,ndebug
      dimension ginst(31)
c
      if(ndebug.ne.0) then
        write(ndebug,1000) irfl,gchar(1:4),(ginst(i)(1:4),i=1,4)
 1000   format(' *debug*inskey*1000* ',I3,2X,A4,2X,4A4)
        endif
C
C  If SCAN returned IRFL=4, this indicates a string containing both
C  characters and digits. Thus the instruction cannot be NORMAL, it
C  must be an ATOM instruction (KEY=3)
C
        IF (IRFL.EQ.4) THEN
          INSKEY=3
          RETURN
        ENDIF
C
C  Check if the instruction is one we can process.
C  These instructions are : TITL,CELL,FVAR,END
C
        DO 100 I=1,4
          IF (GCHAR(1:4).EQ.GINST(I)(1:4)) THEN
            INSKEY=1
            RETURN
          ENDIF
 100    CONTINUE
C
C  If it is not one we can process, we must determine if the instruction
C  is NORMAL and can't be used (KEY=2) or an ATOM instruction (KEY=3).
C
        DO 200 I=5,31
          IF (GCHAR(1:4).EQ.GINST(I)(1:4)) THEN
            INSKEY=2
            RETURN
          ENDIF
 200    CONTINUE
        INSKEY=3
        RETURN
        END
C
C
C-----------------------------------------------------------------------
        SUBROUTINE PROATM (HLINE,NSHELL)
C-----------------------------------------------------------------------
C
C  Written by David Clarkson JULY 1985
C
C  Purpose : To process an ATOM instruction
C
C  Input   : HLINE  ; buffer holding current line of data file
C            NSHELL ; unit number of data file
C
C  Output  : various common block values are loaded
C
C-----------------------------------------------------------------------
C
        IMPLICIT CHARACTER *80 (H)
        IMPLICIT CHARACTER *8  (G)
        DIMENSION XHOLD (20)
C
        include 'comatom.for'
        COMMON /VAR/    NVAR,MVAR,FVAR (100)
C
C  Re-read instruction
C
        IP=1
        IRFL=0
        CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
C
C  Determine GELEM and GIDENT
C
        NX=NX+1
        I=0
  10    I=I+1
        IF (GCHAR(I:I).GE.'0'.AND.GCHAR(I:I).LE.'9') GOTO 20
        IF (I.LT.5) GOTO 10
  20    GELEM(NX)=GCHAR(1:I-1)
        GDENT(NX)=GCHAR(I:8)
C
C  Read the atom identifier
C
        CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
C
C  Read the remainder of the information on the line
C
C  ie. For the Anisotropic case...
C        10 values : X,Y,Z,S.O.F,U11,U22,U33,U23,U13,U12
C
C      For the Isotropic case...
C         5 values : X,Y,Z,S.O.F,U11
C
        ISIZE=5
        CALL RDREAL (XHOLD,ISIZE,HLINE,IP,NSHELL)
        CALL XCONV (XHOLD,ISIZE)
        XX(1,NX)=XHOLD(1)
        XX(2,NX)=XHOLD(2)
        XX(3,NX)=XHOLD(3)
        OCCU(NX)=XHOLD(4)
        U(1,1,NX)=XHOLD(5)
C
C  Check to see if another number is on the line
C  this would indicate an Anisotropic format.
C
        IF (IP.NE.99) THEN
          ISIZE=5
          CALL RDREAL(XHOLD,ISIZE,HLINE,IP,NSHELL)
          CALL XCONV (XHOLD,ISIZE)
          U(2,2,NX)=XHOLD(1)
          U(3,3,NX)=XHOLD(2)
          U(2,3,NX)=XHOLD(3)
          U(1,3,NX)=XHOLD(4)
          U(1,2,NX)=XHOLD(5)
        ENDIF
C
C  Check to see if another atom line is present
C  This would indicate standard errors have been given
C
          READ (NSHELL,1000,END=999) HLINE
 1000     FORMAT (A80)
          IF (HLINE(1:4).NE.'    ') THEN
C  Backspace the input file to reposition for the next instruction
            BACKSPACE (NSHELL)
            RETURN 
          ENDIF
          IP=1
          ISIZE=4
          CALL RDREAL(XHOLD,ISIZE,HLINE,IP,NSHELL)
          DXX(1,NX)=XHOLD(1)
          DXX(2,NX)=XHOLD(2)
          DXX(3,NX)=XHOLD(3)
          DU(1,1,NX)=XHOLD(4)
C
C  Check to see if another number is on the line
C  This would indicate the anisotropic temperature standard errors
C   have been given
C
        IF (IP.NE.99) THEN
          ISIZE=5
          CALL RDREAL (XHOLD,ISIZE,HLINE,IP,NSHELL)
          DU(2,2,NX)=XHOLD(1)
          DU(3,3,NX)=XHOLD(2)
          DU(2,3,NX)=XHOLD(3)
          DU(1,3,NX)=XHOLD(4)
          DU(1,2,NX)=XHOLD(5)
        ENDIF
 999    RETURN
        END
C
C
C-----------------------------------------------------------------------
        SUBROUTINE PROINS (HLINE,NSHELL)
C-----------------------------------------------------------------------
C
C Written by David Clarkson JULY 1985
C
C Purpose : To process a NORMAL SHELX instruction
C
C Input   : HLINE  ; buffer holding current line of data file
C           NSHELL ; unit number of the data file
C
C Output  : various common block values are loaded
C
C-----------------------------------------------------------------------
C
        IMPLICIT CHARACTER *80 (H)
        IMPLICIT CHARACTER *8  (G)
        DIMENSION XHOLD(20)
C
      COMMON /GTITLE/ GENT,HNAME,HCODN(2),HTITLE(2),HAUTH(10),
     1                  HFORM(2),HCOMM(7),GRFACT,GRAD
      COMMON /CELL/   A,B,C,ALPHA,BETA,AGAMMA,VOL,AM(3,3),ICELL
      COMMON /DENS/   ZFORM,DENS,DDENS,CDENS,FWGT,WAVELN,ABSORB,
     1                  FZERO,TMPRTR
        COMMON /VAR/    NVAR,MVAR,FVAR (100)
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
C
C  Re-read instruction
C
        IP=1
        IRFL=0
        CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
C
C  Process the instruction accordingly
C
        IF (GCHAR(1:4).EQ.'TITL') HNAME=HLINE(IP:80)
C
C  A CELL instruction should contain 7 real numbers representing :
C  Lambda,A,B,C,Alpha,Beta,Agamma
C
        IF (GCHAR(1:4).EQ.'CELL') THEN
          ISIZE=7
          CALL RDREAL (XHOLD,ISIZE,HLINE,IP,NSHELL)
          WAVELN=XHOLD(1)
          A=XHOLD(2)
          B=XHOLD(3)
          C=XHOLD(4)
          ALPHA=XHOLD(5)*3.1415926/180.0
          BETA=XHOLD(6)*3.1415926/180.0
          AGAMMA=XHOLD(7)*3.1415926/180.0
          ICELL=1
        ENDIF
C
C  An FVAR card may contain any number of real numbers.
C  ISIZE=99 signals RDREAL to read all numbers present on the line
C  and then return the total number read in ISIZE
C
        IF (GCHAR(1:4).EQ.'FVAR') THEN
          ISIZE=99
          CALL RDREAL (XHOLD,ISIZE,HLINE,IP,NSHELL)
          IF((NVAR+ISIZE).GT.MVAR) THEN
            WRITE (NOUT,1000) nvar+isize
 1000     FORMAT ('0*ERROR*PROINS*More than 100 variables read; rest ',
     1      'ignored ',f5.2,' read')
            goto 110
          endif
          DO 100 I=1,ISIZE
            J=I+NVAR
 100        FVAR(J)=XHOLD(I)-INT(XHOLD(I)/10.0)*10
          NVAR=NVAR+ISIZE
        ENDIF
C
C  An END instruction is processed by simply returning to calling
C  routine.
C
 110    RETURN
        END
C
C
C-----------------------------------------------------------------------
        SUBROUTINE RDREAL (XHOLD,ISIZE,HLINE,IP,NSHELL)
C-----------------------------------------------------------------------
C
C Written by David Clarkson JULY 1985
C
C Purpose : To read a specified number of real numbers from the input
C           line.
C
C Input   : HLINE  ; buffer holding current line of data file
C           IP     ; pointer to current position on the line
C           NSHELL ; unit number of the data file
C           ISIZE  ; number of real numbers to be read
C
C Output  : XHOLD  ; array to hold real numbers read
C           ISIZE  ; number of real numbers read ( special case ISIZE=99)
C
C-----------------------------------------------------------------------
C
        IMPLICIT CHARACTER *80 (H)
        IMPLICIT CHARACTER *8  (G)
        DIMENSION XHOLD (20)
      COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
C
C  Zero the array to hold the numbers read
C
        ICOUNT=0
        DO 100 I=1,20
 100      XHOLD(I)=0.0
C
C  Read ISIZE values from the line
C
  10    CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
C
C  Check for end-of-line based on what ISIZE equals.
C  ie. Is this the special case ISIZE=99 ?
C
        IF (IP.EQ.99.AND.X.EQ.0.0.AND.GCHAR.NE.'='.AND.
     1      ISIZE.NE.99) GOTO 20
        IF (ISIZE.EQ.99.AND.IP.EQ.99.AND.X.EQ.0.0) THEN
          ISIZE=ICOUNT
          RETURN
        ENDIF
C
C  Check for a non-real number on the line
C
        IF (IRFL.NE.3.AND.IRFL.NE.0.AND.IRFL.NE.1)
     1    WRITE (NOUT,1000) GCHAR,X
C
C  Check for continuation marker
C
        IF (GCHAR.EQ.'=') THEN
          IF (ICOUNT.GE.ISIZE) THEN
            WRITE (NOUT,1005)
            RETURN
          ENDIF
          READ (NSHELL,'(A80)') HLINE
          WRITE (NOUT,'(1X,A80)') HLINE
          IP=1
          GOTO 10
        ENDIF
C
C  At this point we have read a real number successfully
C
        ICOUNT=ICOUNT+1
        XHOLD(ICOUNT)=X
        IF (ISIZE.EQ.99.AND.(IP.NE.99.OR.X.NE.0.0)) GOTO 10
        IF (ICOUNT.LT.ISIZE) GOTO 10
C
C  Check to see if all ISIZE values were read
C
  20    IF (ICOUNT.LT.ISIZE) WRITE (NOUT,1010) ISIZE-ICOUNT
        RETURN
C
C  Error messages
C
1000    FORMAT ('0*WARNING*rdreal*1000* Expecting REAL ',
     1          ', found ',A8,/,27X,'A value of ',F7.4,' will ',
     2          'be used.')
1005    FORMAT ('0*warning*rdreal*1005* Continuation found ',
     1          'when all values read correctly.',/,27X,
     2          'Ignored.')
1010    FORMAT ('0*warning*rdreal*1010* There are ',I1,' values ',
     1          'missing on this line.',/,27X,'These values ',
     2          'will be assigned a value of 0.0')
        END
C
C
C-----------------------------------------------------------------------
        SUBROUTINE SKIPINS (HLINE,NSHELL,IEND)
C-----------------------------------------------------------------------
C
C Written by David Clarkson JULY 1985
C
C Purpose : To skip blank lines and irrelevent instructions
C
C Input   : HLINE  ; buffer holding current line of data file
C           NSHELL ; unit number of the data file
C
C Output  : HLINE  ; buffer holding new current line of data file
C
C-----------------------------------------------------------------------
C
        IMPLICIT CHARACTER *80 (H)
        IMPLICIT CHARACTER *8  (G)
        COMMON /FILES/  NIN,NOUT,NDEBUG,NREAD,NWRITE,NSPGP
C
        IEND=0
  30    READ (NSHELL,'(A80)',END=999) HLINE
        WRITE (NOUT,'(1X,A80)') HLINE
        IRFL=0
        IP=1
        CALL SCAN (GCHAR,N,X,EX,IP,HLINE,IRFL)
        IF (IRFL.NE.1.AND.IRFL.NE.4) GOTO 30
        RETURN
 999    IEND=1
        RETURN
        END
C
C
C-----------------------------------------------------------------------
        SUBROUTINE XCONV (XHOLD,ISIZE)
C-----------------------------------------------------------------------
C
C Written by David Clarkson JULY 1985
C
C Purpose : To convert real numbers stored in XHOLD into correct values
C           ( see description below )
C
C Input   : XHOLD  ; array holding real numbers
C           ISIZE  ; number of real numbers stored
C
C Output  : XHOLD  ; array holding corrected real numbers
C
C-----------------------------------------------------------------------
C
        DIMENSION XHOLD(ISIZE)
        COMMON /VAR/ nvar,mvar,FVAR (100)
C
C  For each number stored in XHOLD, determine if it has been "SET" ( this
C  is indicated by a value of 10.0 added to it) or has to be multiplied
C  by a free-variable ( Specifically determined by the digit in the 10's
C  column of the number eg. 20.5 is actually 0.5*FVAR(2)    )
C
C  Also, if the real number stored is < 0 we use the following format:
C
C      eg. -20.5 is actually 0.5*(1-FVAR(2)
C
        DO 100 I=1,ISIZE
          N=INT(XHOLD(I)/10.0)
          IF (ABS(N).GT.0) THEN
            IF (N.EQ.1) THEN
              XHOLD(I)=XHOLD(I)-10.0
            ELSE
              IF (XHOLD(I).LT.0.0) THEN
                XHOLD(I)=(XHOLD(I)-10.0*N)*(FVAR(N)-1.0)
              ELSE
                XHOLD(I)=(XHOLD(I)-10.0*N)*FVAR(N)
              ENDIF
            ENDIF
          ENDIF
 100    CONTINUE
        RETURN
        END
