C
C        FILE SPANNER                   Last update: 1993.02.30
C        ------------                                M. Kunz
C
C     This file contains routines for expanding a bond network
c     and analysing it.  The program assumes that at least the
c     asymmetric set of bonds is stored in the /bonds/ arrays.
c     It then expands this and stores the results in the /span/
c     arrays.  The user has the option of selecting the starting
c     atom in the spanning tree, otherwise the program will select
c     the most strongly bonded cation.  The user is also asked
c     to specify the smallest bond valence that is to be included
c     in the network. Bonds without valences and contacts may be included
c     at this point by setting the lower valence limit to 0.0.
c     See IYAT for a description of the way in which the atoms in
c     the spanning tree are stored.
c
c
c******* ROUTINE AND VARIALBE NAMES WITH MORE THAN 6 CHARACTERS IN THIS FILE
c
c  ******************************************************************
c   THIS FILE CONTAINS DOUBLE PRECISSION ARRAYS
c  ******************************************************************
c
c
c     routines stored in this file
c     ----------------------------
c
c     S SPANNER                   generates the spanning tree
c    *S SPARAM(itype)             selects calculation type
c    *S ATOMSL(iat)               selects starting atom
c    *S BONDSL(bondlim,iconfl)    selects bond valence limits
c     S EXPAND(itype)             expands the bonding network
c     S ALLCOSPHR(ict,xxbond,iibond) finds coordination sphere around ict
c                                 in /span/
c     S COSPHR(iat(5),kb,jat(5,20),jbon(20),bondlim(2),xbn(6,20))
c                                 generates the bonds around atom iat
c     F IYAT(ibn(5))              returns IYAT in IYATOM(IYAT)=IBN
c     S JYAT(i,jyatom(5))         returns JYATOM=IYATOM(I)
c   +*S SPANAL(itype)             analyses the spanning tree
c    *S SPANOU                    output
c     S CLOSER                    reduces infinite spanning tree to
c                                   closed network (cyclic closing)
c     S TEMPAN                    analyses ADP's
c     s spanhelp                  help routine called by struhelp
c
c
c    *) contains interactive i/o
c    +) contains double precission arrays
c
c
c     Routines called from this file:
c
c     CHEMLB: FORMUL
c     IOLIB:  IFILE
c     MATHLB: MXV, READYN, WAIT, sorty,inerr
c     SymLIB: INVSTL, SITTRN,mltstl
c     CERNLB: RG
c     SPANNER: jyat
c
c
***********************************************************************
c
      subroutine spanner
c     ------------------
c
c    written 85.08.02   by I.D.Brown
c    updated 85.10.29   by D.Altermatt
c
c    This routine calls subroutines which generate the spanning tree
c    for one unit cell based on information stored in /bonds/
c
c    IAT is the number of the atom in the bondfile to act as origin
c        of the spanning tree.  If it is zero, the program selects an
c        atom that forms the strongest bond
c
c    BONDLIM is the minimum and maximum valence that will be
c        considered for a bond
c
c    ITERM keeps track of the status of each iyatom as follows
c          0 = active (all bonds not yet generated)
c         -1 = inactive (all bonds from this atom are in IBOND)
c          1 = spanning tree links to next unit cell through this atom
c          2 = spanning tree terminates on this atom
c
c    calls sparam, expand, spanal, spanou
c
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2                netind(20)

c
c     local common blocks
c
      common /frag/   nfrag,mfrag,ianal(20),idir(3,20),nform(20),
     1                iform(20,20),charge(20),bases(20),nyat(20),
     3                nfrat,mfrat,iatfr(6,600),ifrcn(600),vsum(600)
      common /gfrag/  gelm(20,20)
c       the dimension of ibofl in 'mbon'
      common /icbfl/  iconfl,ibofl(1000)
c
      dimension ibn(5),jbn(5),ipt(3),jpt(3)
c
      write(nout,1000)
 1000 format(///)
      write(nout,1001)
 1001 format(10X,'Welcome to SPANNER',/,
     1       10X,'------------------',//,
     3'   Spanner will expand the bonds stored in the asymmetric ',/,
     4' bond set (as calculated by SINDBAD or STRUMO) into an',/,
     5' infinite network.  The atoms must be identified by their',/,
     6' Altermatt number (again done in SINDBAD or STRUMO) but ',/,
     7' coordinates are not necessary, thus the BONDFILE can be ',/,
     8' used as input. The user can decide which atom to start ',/,
     9' the network on, the range of bond strengths to be used,',/,
     1' and whether a full expansion of the network is required ',/,
     2' or only nearest neighbours.  The network found is analysed ',/,
     3' to discover if it is linked infinitely in any directions, ',/,
     4' and to determine its chemical properties.',//,
     5' As an option you may also include contacts (as defined',/,
     6' in STRUMO) and bonds without valences into the network',/,
     7' by setting the lower valence limit negative.',//)
      call wait
      mfrag=20
      nfrag=0
      mfrat=600
      nfrat=0
c
c   test if special position number given
c   -------------------------------------
      do 130 i=1,nx
	if(ialtno(i).le.0.and.npos(i).ge.0) then
	  write(nout,1301) gelem(i), gdent(i)
 1301     format(//' ++ERROR++SPANNER++1301++ Special position not',
     1           ' assigned for ',A2,A6,'. Calculation aborted',//)
	  return
	  endif
  130   continue
c
c   input instructions
c   ------------------
  200 call sparam(itype)
      if(itype.eq.20) return
      nya=0
      nbo=0
      ib=0
      do 205 i=1,nbon
  205   ibofl(i)=0
c     start implicit loop ib=1,nbon
  210   if(itype.eq.4) then
	  ib=ib+1
	  if(ib.gt.nbon) go to 900
	  if(ibofl(ib).ne.0) go to 210
	  if(xbon(3,ib).lt.bondlim(1).or.xbon(3,ib).gt.bondlim(2))
     +                         go to 210
	  iat=ibon(1,ib)
	  endif
c
c   expand the network around iat
c   -----------------------------
	call expand(itype)
	if(ndebug.ne.0) then
	  do 255 i=1,nya
	    write(ndebug,2550)i,(iyatom(k,i),k=1,5),iterm(i)
 2550       format(' *DEBUG*SPANNER*2550*   iyatom(',I3,')=',6I4)
  255       continue
	  do 260 i=1,nbo
	    write(ndebug,2600) (ibond(k,i),k=1,2),(xbond(k,i),k=1,6)
 2600       format(' *DEBUG*SPANNER*2600* ',2I3,6F6.3)
  260       continue
	  endif
c
c    analyse the network
c    -------------------
	nfrag=nfrag+1
	call spanal(itype)
  300   write(nout,1300)
 1300   format(/,' Do you want a thermal analysis of the fragment ',
     +    '(Y/N/E) ')
	call readyn(*310,*700,*900,*300)
  310   call tempan
	if(itype.eq.4) goto 210
c
c       end of implicit loop ib
c
c    write output
c    ------------
  700 if(itype.eq.2.or.itype.eq.5) call spanou
c
c    calculates and writes cartesian coordinates of a given fragment
c
  900 go to 200
      end
c
c
***********************************************************************
c
c
      subroutine sparam(itype)
c     -----------------
c
c   written 85.08.06   by I.D.Brown
c   updated 85.10.29   by D.Altermatt
c
c   reads interactively the initial atom for generating the spanning
c   tree and the lower limit of bond valence to be included in the
c   tree
c
c   calls atomsl, bondsl, inerr, readyn
     
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      include 'comatom.for'
      include 'combond.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c
      common /icbfl/  iconfl,ibofl(1000)
c
      dimension ibn(5),jbn(5),ipt(3),jpt(3)
c
      ntout=nwrite
      if(ndebug.ne.0) ntout=ndebug
c
c  select calculation required
c  ---------------------------
    5 write(nout,5001)
 5001 format(' MENU LEVEL 2.10   Select the calculation required ',
     1  'from the menu',//,
     1  '  1. Help',/,
     2  '  2. Calculate the network for one unit cell',/,
     3  '  3. Calculate the environment of a single atom',/,
     4  '  4. Analyse the compound for fragments',/,
     5  '  5. Calculate an environment to the nth neighbour',/,
     9  ///////////////,
     1  ' 20. Return to Menu 2 (keeping last tree stored)',//,
     2  '    Select calculation by number: ')
      read(nin,5002,err=10) itype
 5002 format(I5)
      if(itype.lt.0.or.itype.gt.20) go to 10
      if(itype.eq.20.or.itype.eq.0) go to 900
      if(itype.gt.5) go to 10
      if(itype.eq.1) then
	 call spanhelp
	 goto 5
      endif
      if(itype.eq.5) then
100     nn=-100
	write(nout,2100)
2100    format(/,' How many "shells" of neighbours shall be included',
     1    ' (1 - 5): ')
	read(nin,5002,err=110) nn
110     if(nn.lt.1.or.nn.gt.5) call inerr(*100)
	itype=-nn
	endif
      go to 14
   10 call inerr(*5)
c
c  read in atom as origin
c  ----------------------
   14 if(itype.ne.4) call atomsl(iat)
c
c   read in bondlim
c   ---------------
      call bondsl(bondlim,iconfl)
c
c    list input parameters
c    ---------------------
      xbl1=0.0
      if(bondlim(1).gt.xbl1) xbl1=bondlim(1)
      if(itype.eq.4) then
	write(nout,1204) xbl1,bondlim(2)
	if(ntout.ne.0) write(ntout,1204) xbl1,bondlim(2)
 1204   format(//,' List of fragments containing bonds with strength ',
     +     'between ',F4.2,' and ',F4.2,' v.u.')
	if(bondlim(1).lt.0.0) then
	  hcom='bonds'
	  if(iconfl.eq.1) hcom='bonds and contacts'
	  write(nout,2000) hcom, -bondlim(1)
2000      format(' Including ',A18,' out to ',F6.2,' Angstroms.')
	  endif
      else
	write(nout,1203) gelem(iat),gdent(iat),xbl1,bondlim(2)
 1203   format(//,' The spanning tree will begin at atom ',A2,A6,/
     1       ' Bonds of strength greater than ',F4.2,' and',/,
     2       ' less than ',F4.2,' v.u. will be included.')
	if(bondlim(1).lt.0.0) then
	  hcom='bonds'
	  if(iconfl.eq.1) hcom='bonds and contacts'
	  write(nout,2010) hcom, -bondlim(1)
2010      format(' As well as ',A18,' to ',F6.2,' Angstroms.')
	  endif
	endif
      write(nout,2020)
2020  format(' ')
      call wait
  900 return
      end
c
***********************************************************************
c
      subroutine atomsl(iat)
c
c    written 86.3.11 by I.D.Brown
c
c    selects the atom to start the spanning tree
c
c    calls readyn, inerr, wait
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/  nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
c
   14 iat=0
   15 write(nout,1015)
 1015 format(//,' Do you want to choose an atom as origin',
     1    ' of the expansion (Y/N/E): ')
      call readyn(*20,*100,*900,*15)
   20 write(nout,1020)
 1020 format(/,' Do you want a list of atoms (Y/N/E): ')
      call readyn(*25,*50,*900,*20)
   25 write(nout,1025)
 1025 format(//'    Atom     Spec.Pos. #'/)
      linc=0
      do 30 i=1,nx
	if(npos(i).lt.0) goto 30
	linc=linc+1
	write(nout,1030) i,gelem(i),gdent(i),ialtno(i)
 1030   format(2X,I3,2X,A2,A6,1X,I2)
	if(linc.eq.20) then
	  linc=0
	  call wait
   40     write(nout,1025)
	  endif
   30   continue
   50 write(nout,1050)
 1050 format(//,' Enter number of origin atom (or 0): ')
      read(nin,1051,err=55) iat
 1051 format(I5)
      if(iat.lt.1.or.iat.gt.nx) goto 55
      if(npos(iat).lt.0) goto 55
      goto 100
   55 call inerr(*50)
c
c   find the atom that forms the strongest bond
c   -------------------------------------------
  100 if(iat.eq.0) then
	iat=1
	xb=-15.0
	do 150 i=1,nbon
c    skip dummy atoms
	  if(npos(ibon(1,i)).lt.0.or.npos(jbon(1,i)).lt.0) goto 150
	  if(xbon(3,i).gt.xb) then
	    xb=xbon(3,i)
	    iat=ibon(1,i)
	    endif
  150     continue
	endif
  900 return
      end
c
c
***********************************************************************
c
      subroutine bondsl(bondlim,iconfl)
c
c   written 86.3.11 by I.D.Brown
c
c   reads in values of maximum and minimum bond valences
c   sets flag ICONFL if contacts are to be included
c
c   calls inerr, readyn
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/  nin,nout,ndebug
c
      dimension bondlim(2)
c
      iconfl=0
  200 write(nout,1200)
 1200 format(/,' Give lower bond valence limit (0.0 <= lim1 ',
     1  '< 5.0): ')
      read(nin,1201,err=200) bondlim(1)
      if(bondlim(1).lt.-0.01.or.bondlim(1).gt.5.0) call inerr(*200)
      if(abs(bondlim(1)).lt.0.0) then
210     write(nout,2000)
2000    format(/,' Do you want to include non-ionic bonds (Y/N): ')
	call readyn(*220,*201,*201,*210)
220     bondlim(1)=-15.0
230     write(nout,2010)
2010    format(' Do you want to include contacts, too (Y/N): ')
	call readyn(*240,*201,*201,*230)
240     iconfl=1
	endif
  201 write(nout,1202)
 1202 format(/,' Give upper bond valence limit (lim1 < lim2 ',
     1  '<= 5.0): ')
      read(nin,1201,err=201) bondlim(2)
 1201 format(F10.0)
      if(bondlim(2).le.bondlim(1)) call inerr(*201)
      if(bondlim(2).gt.5.0) bondlim(2) = 5.0
      return
      end
c
***********************************************************************
c
      subroutine expand(itype)
c     -----------------
c
c   written 85.08.06   by I.D.Brown
c   updated 85.10.29   by D.Altermatt
c
c   expands the bonds stored in /bonds/ to /span/
c
c   the algorithm selects an origin atom (iat) and generates the bonds
c   to its terminal atoms which are store in iyatom. It then repeats
c   this process around each iyatom in turn.  After an iyatom has been
c   processed, it is made inactive.  It is also made inactive if it
c   is related to an existing iyatom by a pure translation.  The calc-
c   ulation stops when all iyatoms are inactive.
c   Any iyatom related to a previous one only be translation has
c   iyatom(2).eq.0, iyatom(1).eq.address of related iyatom and
c   iyatom(3-5).eq.translation
c
c   if(itype.eq.-n) the expansion goes to nth nearest neighbours of iat
c   if(itype.eq.1, 2 or 4) the expansion fills the unit cell
c   if(itype.eq.3) only the first coordination sphere of iat is found
c   iterm = 1 for an atom that terminates the expansion, else = -1
c
c   Dummy atoms will be skipped.
c
c
c   calls cosphr, iyat
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for' 
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,ta(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30)
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2                netind(20),ifbond(4,600)
c
      dimension jatom(5,20),jbond(20),ibn(5),jbn(5),jyatom(5)
      dimension xbn(6,20),ifbn(4,20)
c
c   nz = order of nearest neighbours to iat already completed
c   nyy = iyatom address of last atom in the nzth shell
c
      do 90 i=1,20
   90   netind(i)=0
      nz = 0
      nyy = 1
c
c   set up first atom
c
      iyatom(1,1)=iat
      iyatom(2,1)=1
      do 160 k=3,5
  160   iyatom(k,1)=0
      iterm(1)=0
      nya=1
c
c   start generating bonds
c
      i=0
      nbo=0
c   start implict do loop with index i, locate the next active atom
c   i is the index of the current central atom
c   j is the index of the the current terminal atom
  300   i=i+1
	if(i.ne.1) then
	  if(i.gt.nya) go to 400
	  if(itype.eq.3) go to 400
	  if(iterm(i).ne.0) go to 300
	  endif
	if(ndebug.ne.0) write(ndebug,3005) itype,iat,bondlim(1),
     +                   bondlim(2)
 3005   format(' *debug*expand*3005*, iytpe=',I2,' iat=',I3,
     +             ' bondlim=',2F6.3)
c
c   look for all bonds around this atom
c
	call cosphr(iyatom(1,i),kbond,jatom,jbond,bondlim,xbn,ifbn)
	do 309 j=1,kbond
  309   continue
	if(ndebug.ne.0) then
	  write(ndebug,3000) i,(iyatom(k,i),k=1,5),kbond
 3000     format(' *DEBUG*EXPAND*3000* sphere around ',I3,'=',6I3)
	  do 310 j=1,kbond
	    write(ndebug,3100) (jatom(k,j),k=1,5),jbond(j)
 3100       format(' *DEBUG*EXPAND*3100* ',6I3)
  310       continue
	  endif
	do 390 n=1,kbond
	  kb=iyat(jatom(1,n))
c                                  label atom that terminates search
	  if(iyatom(2,kb).eq.0) iterm(kb)=1
c                                  test if bond is already stored
	  if(nbo.eq.0) go to 335
	  do 330 k=1,nbo
	    if((ibond(1,k).eq.i.and.ibond(2,k).eq.kb).or.
     1         (ibond(1,k).eq.kb.and.ibond(2,k).eq.i)) go to 390
  330       continue
c                                  the bond is not yet stored
  335     nbo=nbo+1
	  if(nbo.gt.mbo) then
	    nbo=mbo
	    write(nout,3351) mbo
 3351       format(//,' ++ERROR++EXPAND++3351++ more than ',I4,
     1             ' bonds found. Calculation terminated',//)
	    return
	    endif
	  ibond(1,nbo)=i
	  ibond(2,nbo)=kb
	  do 380 k=1,6
  380       xbond(k,nbo)=xbn(k,n)
	  do 385 k=1,4
  385       ifbond(k,nbo)=ifbn(k,n)
  390     continue
	if(icn(iyatom(1,i)).eq.0.and.iyatom(2,i).ne.0)
     +                        icn(iyatom(1,i))=kbond
c                                   make atom i inactive
	if(kbond.eq.0) iterm(i)=2
	if(iterm(i).eq.0) iterm(i)=-1
	if(i.eq.nyy) then
	  netind(nz+1)=nya-nyy
	  nyy=nya
	  nz=nz+1
	  if(-itype.eq.nz) go to 400
	  endif
	go to 300
c   end implicit do loop of index i
  400 continue
      return
      end
c
c
***********************************************************************
c
c
      subroutine allcosphr(jat,jsym,xxbond,iibond,jiatom)
c     -------------------------
c     subroutine to find all bonds attached to a given central atom
c     jat with symmetry operation jsym. The results are stored in
c     the variables xxbond(<-xbond),iibond(<-ibond,iibond(3)=address
c     of bond in /span/), and jiatom(<-iyatom of terminal atoms)
c     Note that translations are not stored in jiatom, since they are
c     lost during expansion into unit-cell in subroutine closer. 
c     ('closer' is called in expand)
c
c     written 12/92 by Martin Kunz
c
c     calls jyat (spanner)
c
      implicit character *8 (g)
      implicit character *80 (h)
c
c     system common blocks:
c
      common /files/ nin,nout
      include 'comatom.for'
      include 'combond.for'
      common /span/ nya,mya,iyatom(5,600),iterm(600), nbo,mbo,
     1              ibond(2,600),xbond(6,600),iat,bondlim(2),
     2              netind(20),ifbond(4,600)
      dimension xxbond(6,20),iibond(3,20),jiatom(5,20)
      dimension jyatom1(5),jyatom2(5)
c    
c   
      do 50 i=1,20
       do 60 j=1,6
	xxbond(j,i)=0.0
   60  continue
       do 70 j=1,3
	iibond(j,i)=0
   70  continue
   50 continue
      ibdct=0
c------------------------------------------------------------------------
c     start of loop to check each bond in /span/ whether it matches
c     with the chosen central atom jat.
c------------------------------------------------------------------------
      do 100 i=1,nbo
       call jyat(ibond(1,i),jyatom1)
       call jyat(ibond(2,i),jyatom2)
       if(((jyatom1(1).eq.jat).and.(jyatom1(2).eq.jsym)).or.
     1    ((jyatom2(1).eq.jat).and.(jyatom2(2).eq.jsym))) then
c-------------------------------------------------------------------------        
c     match of one atom and its sym.op. found for bond i
c-------------------------------------------------------------------------
	ibdct=ibdct+1
	do 130 j=1,6
  130    xxbond(j,ibdct)=xbond(j,i)
	do 132 j=1,2
  132    jiatom(j,ibdct)=jyatom2(j)
	iibond(1,ibdct)=ibond(1,i)
	iibond(2,ibdct)=ibond(2,i)
	iibond(3,ibdct)=i 
	if(jyatom2(1).eq.jat) then
c-------------------------------------------------------------------------
c       find which end of the bond i is formed by the central atom jat
c       and make sure terminal atom vectors point from central atom jat
c       towards terminal atom
c-------------------------------------------------------------------------
	 iibond(1,ibdct)=ibond(2,i)
	 iibond(2,ibdct)=ibond(1,i)
	 do 140 j=4,6
  140     xxbond(j,ibdct)=-1.*(xxbond(j,ibdct))
	endif
       endif
  100 continue
      return                                        
      end
***********************************************************************
c
c
      subroutine cosphr(iatom,kbond,jatom,jbond,bondlim,xbn,ifbn)
c     -----------------------------------------------------------
c
c    written 85.08.13    by I.D.Brown
c    updated 85.10.29    by D.Altermatt
c
c   this routine determines all the bonds around iatom that have a bond
c   valence less that bondlim.   It find the bonds in the /bonds/ array
c   and returns in jbond(20) the number of the bond found in the /bonds/
c   array, and in jatom(5,20) the terminal atoms of those bonds.  kbond
c   is the number of bonds found.
c
c   If iconfl.eq.1 then the same search is done for contacts, too.
c
c   Dummy atoms will be skipped.
c
c   calls invstl, mltstl, sittrn, mxv
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,ta(3,4)
      common /syma/   mult(48,48)
c
      common /icbfl/  iconfl,ibofl(1000)
c
      dimension iatom(5),jatom(5,20),jbond(20),xbn(6,20),ifbn(4,20)
      dimension ibn(5),jbn(5),ipt(3),jpt(3),kpt(3),xxbn(6)
      dimension bondlim(2)
c
c
	kbond=0
c
c   search through the /bonds/ array for candidate bonds
c
      do 390 j=1,nbon
	if(npos(ibon(1,j)).lt.0.or.npos(jbon(1,j)).lt.0) goto 390
	if(xbon(3,j).lt.bondlim(1).or.xbon(3,j).gt.bondlim(2))
     +                                                  go to 390
	if(iatom(1).ne.ibon(1,j)) then
	  if(iatom(1).ne.jbon(1,j)) go to 390
	  do 303 k=1,5
	    ibn(k)=jbon(k,j)
	    jbn(k)=ibon(k,j)
	    if(k.le.3) xxbn(k)=-xbon(k+3,j)
  303       continue
	else
	  do 306 k=1,5
	    ibn(k)=ibon(k,j)
	    jbn(k)=jbon(k,j)
	    if(k.le.3) xxbn(k)=xbon(k+3,j)
  306       continue
	  endif
	ibofl(j)=1
c
c   find the transformation that takes ibn(2) to iatom(2)
c
	nn=1
	call invstl(nn,ips,ipt,ibn(2),ibn(3),iatom(2),iatom(3))
c
c   find the transformation kps that is equivalent to one value of
c    jatom(2)
c
	call mltstl(kps,kpt,ips,ipt,jbn(2),jbn(3),ialtno(jbn(1)))
c
c    begin implict n loop to remove identical bonds generated by symmetry
c
	n=0
  320     n=n+1
	  call sittrn(n,jps,jpt,ialtno(jbn(1)),kps,kpt,
     1         ialtno(iatom(1)),iatom(2),iatom(3))
	  if(n.le.0) go to 390
c
c    test if new bond has already been stored
c
	  do 328 i=1,kbond
	    if(jbn(1).ne.jatom(1,i)) go to 328
	    if(jps.ne.jatom(2,i)) go to 328
	    do 324 k=1,3
  324         if(jpt(k).ne.jatom(k+2,i)) go to 328
	    go to 320
  328       continue
c
c    store new terminal atom
c
	  kbond=kbond+1
	  if(kbond.lt.20) then
	    jatom(1,kbond)=jbn(1)
	    jatom(2,kbond)=jps
	    do 330 k=1,3
  330         jatom(k+2,kbond)=jpt(k)
	    jbond(kbond)=j
	    do 340 k=1,3
  340         xbn(k,kbond)=xbon(k,j)
	    do 345 k=1,4
  345         ifbn(k,kbond)=ifbon(k,j)
	    lps=mult(n,inmul(1,ibn(2)))
	    call mxv(xbn(4,kbond),s(1,1,lps),xxbn)
	    go to 320
	  else if(kbond.eq.21) then
	    write(nout,3900) iatom
 3900       format(//,' ++ERROR++COSPHR++3900++ More than 20 bonds ',
     1        'found around atom ',5I3,', remainder ignored',//)
	    go to 400
	    endif
	  go to 320
c         end of implicit n loop
  390   continue
      if(iconfl.ne.1) goto 400
c
c     Do also contacts!
c
      do 100 j=1,ncon
	if(npos(icon(1,j)).lt.0.or.npos(jcon(1,j)).lt.0) goto 100
	if(xcon(3,j).lt.bondlim(1).or.xcon(3,j).gt.bondlim(2))
     +                                                  go to 100
	if(iatom(1).ne.icon(1,j)) then
	  if(iatom(1).ne.jcon(1,j)) go to 100
	  do 110 k=1,6
	    xxbn(k)=-xcon(k+3,j)
	    if(k.gt.5) go to 110
	    ibn(k)=jcon(k,j)
	    jbn(k)=icon(k,j)
  110       continue
	else
	  do 120 k=1,6
	    xxbn(k)=xcon(k+3,j)
	    if(k.gt.5) go to 120
	    ibn(k)=icon(k,j)
	    jbn(k)=jcon(k,j)
  120       continue
	  endif
c
c   find the transformation that takes ibn(2) to iatom(2)
c
	nn=1
	call invstl(nn,ips,ipt,ibn(2),ibn(3),iatom(2),iatom(3))
c
c   find the transformation kps that is equivalent to one value of
c   jatom(2)
c
	call mltstl(kps,kpt,ips,ipt,jbn(2),jbn(3),ialtno(jbn(1)))
c
c    begin implict n loop through symmetry related contacts
c
	n=0
  130     n=n+1
	  call sittrn(n,jps,jpt,ialtno(jbn(1)),kps,kpt,
     1         ialtno(iatom(1)),iatom(2),iatom(3))
	  if(n.le.0) go to 100
c
c    test if new bond has already been stored
c
	  do 140 i=1,kbond
	    if(jbn(1).ne.jatom(1,i)) go to 140
	    if(jps.ne.jatom(2,i)) go to 140
	    do 150 k=1,3
  150         if(jpt(k).ne.jatom(k+2,i)) go to 140
	    go to 130
  140       continue
c
c    store new terminal atom
c
	  kbond=kbond+1
	  if(kbond.lt.20) then
	    jatom(1,kbond)=jbn(1)
	    jatom(2,kbond)=jps
	    do 160 k=1,3
  160         jatom(k+2,kbond)=jpt(k)
	    jbond(kbond)=j
	    do 170 k=1,3
  170         xbn(k,kbond)=xcon(k,j)
	    lps=mult(n,inmul(1,ibn(2)))
	    call mxv(xbn(4,kbond),s(1,1,lps),xxbn)
	    go to 130
	  else if(kbond.eq.21) then
	    write(nout,3900) iatom
	    go to 400
	    endif
	  go to 130
c         end of implicit n loop
  100   continue
c
c     Ok, return
c
  400 if(kbond.gt.20) kbond=20
  900 return
      end
c
c
***********************************************************************
c
c
      function iyat(ibm)
c     ------------------
c
c    written 85.08.02   by I.D.Brown
c
c    This function returns the address in IYATOM of the atom that
c    is equal to ibm(5).  If no match is found it stores the atom.
c    If the atom is translationally equivalent to a stored atom
c    iyatom(1,iyat)=iyatom address of the match, iyatom(2,iyat)=0
c    and iyatom(3-5,iyat) is
c    the translation to go from the atom already stored to the
c    current atom.
c
c    calls jyat
c
c    system c
      include 'comatom.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c
      dimension ibn(5),ibm(5),ibl(5)
c
c   regenerate atoms that are translationally equivalent
c
      if(ibm(2).eq.0) then
	ib0=ibm(1)
	ibn(1)=iyatom(1,ib0)
	ibn(2)=iyatom(2,ib0)
	do 50 k=3,5
   50     ibn(k)=iyatom(k,ib0)+iyatom(k,ibn(1))
      else
	do 70 k=1,5
   70     ibn(k)=ibm(k)
	endif
      jbn=0
      if(nya.eq.0) go to 210
      do 200 i=1,nya
	call jyat(i,ibl)
	do 190 k=1,2
  190     if(ibl(k).ne.ibn(k)) go to 200
	do 195 k=3,5
	  if(ibl(k).ne.ibn(k)) then
	    jbn=i
	    go to 205
	    endif
  195     continue
c
c   a perfect match has been found
c
	iyat=i
	return
  200   continue
c
c   match found for atom and symmetry, check if translations already found
c
  205 do 208 i=1,nya
	if(iyatom(1,i).eq.jbn.and.iyatom(2,i).eq.0) then
	  do 207 k=3,5
  207       if(ibn(k).ne.(iyatom(k,i)+iyatom(k,jbn))) go to 208
	  iyat=i
	  return
	  endif
  208   continue
c
c    no match, store new atom
c
  210 if(nya.ge.mya) go to 900
      nya=nya+1
      iyat=nya
      if(jbn.eq.0) then
	iterm(nya)=0
	do 230 k=1,5
  230     iyatom(k,nya)=ibn(k)
	return
      else
c
c   atom found is related by translation to atom jbn
c
	iterm(nya)=1
	iyatom(1,nya)=jbn
	iyatom(2,nya)=0
	do 270 k=3,5
  270     iyatom(k,nya)=ibn(k)-iyatom(k,jbn)
	return
	endif
  900 write(nout,9001) mya
 9001 format(//,' ++ERROR++IYAT++9001++ More than ',I4,
     1       ' atoms generated in spanning tree',//)
      return
      end
c
c
***********************************************************************
c
c
      subroutine jyat(i,jyatom)
c     -------------------------
c
c   written 85.9.13 by I.D.Brown
c
c     Given the address i of an atom in the iyatom array, it returns the
c     full description of the atom in jyatom(5)
c
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c
      dimension jyatom(5)
c
c
      do 100 k=1,5
  100   jyatom(k)=iyatom(k,i)
      if(jyatom(2).eq.0) then
	ia=iyatom(1,i)
	jyatom(1)=iyatom(1,ia)
	jyatom(2)=iyatom(2,ia)
	do 200 k=3,5
  200     jyatom(k)=jyatom(k)+iyatom(k,ia)
	endif
      return
      end
c
c
***********************************************************************
c
c
      subroutine spanal(itype)
c     --------------------
c
c   written 85.8.6 by I.D.Brown
c
c   analyses the spanning tree developed above
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite
      include 'comatom.for'
      include 'combond.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
      common /frag/   nfrag,mfrag,ianal(20),idir(3,20),nform(20),
     1                iform(20,20),charge(20),bases(20),nyat(20),
     2                nfrat,mfrat,iatfr(6,600),ifrcn(600),vsum(600)
      common /gfrag/  gelm(20,20)
c
c   iatfr contains atom#, symmetry#, translations(3), fragment#
c
c   jyatom contains all the unit cell translations found in the spanning tree
c   kyatom contains the nka iyatom addresses of the atoms in the unit cell
c     i.e. omitting those that are related by cell transaltions
c
      dimension jyatom(3,100),isum(3),jsum(3),iya(5),kyatom(600)
      dimension xox(20)
c
c   the following arrays have dimensions mmtrx
c
      dimension mtrx(100,100),iv1(100),iwr(100),wwr(100)
      double precision amtrx(100,100),wr(100),wi(100),xmtrx(100,100),
     +    fv1(100)
      mmtrx=100
c
c  ntout is the dump.lis file
      ntout=nwrite
      if(ndebug.ne.0) ntout=ndebug
c
c   ADJACENCY MATRIX
C   ----------------
c   calculate adjacency matrix
c   -----------------------
      do 460 i=1,mmtrx
	do 460 j=1,mmtrx
	mtrx(i,j)=0
	xmtrx(i,j)=0.
  460   continue
      mtrxfg=0
      do 470 i=1,nbo
	if(ibond(1,i).gt.mmtrx.or.ibond(2,i).gt.mmtrx) then
	  mtrxfg=1
	  go to 470
	  endif
	mtrx(ibond(1,i),ibond(2,i))=1
	mtrx(ibond(2,i),ibond(1,i))=1
	xmtrx(ibond(1,i),ibond(2,i))=xbond(3,i)
	xmtrx(ibond(2,i),ibond(1,i))=xbond(3,i)
  470   continue
      nmtrx=nya
      if(nya.gt.mmtrx) nmtrx=mmtrx
c
c   print adjacency matrix
c   ----------------------
      if(ntout.ne.0) then
	write(ntout,1470)
 1470   format(/,'    Connection table',/)
	if(mtrxfg.ne.0) write(ntout,1474)
 1474   format('  WARNING: The connection table has been truncated')
	linc=0
	do 480 i=1,nmtrx
	  linc=linc+1
	  call jyat(i,iya)
	  write(ntout,1471) gelem(iya(1)),gdent(iya(1)),
     +    (iya(j),j=2,5),(mtrx(i,j),j=1,nmtrx)
 1471     format(1X,A2,A6,4I3,10(1X,5I2),/,3(21X,10(1X,5I2),/))
	  if(linc.eq.5)then
	    linc=0
	    write(ntout,1472)
 1472       format(1X)
	    endif
  480     continue
	write(nout,1473) hfilen(ntout)
 1473   format(/,' A connection table has been written on file ',
     +                   A30)
	endif
c
c   determine the rows in the finite adjacency matrix
C   -------------------------------------------------
      nka=0
      do 490 i=1,nya
	if(iyatom(2,i).ne.0) then
	  nka=nka+1
	  kyatom(nka)=i
	  endif
  490   continue
c
c  ANALYSE FRAGMENT
C  ----------------
      if(itype.eq.3) go to 400
      nyat(nfrag)=nya
      write(nout,1000) nfrag
      if(ntout.ne.0) write(ntout,1000) nfrag
 1000 format(//,' FRAGMENT NUMBER ',I3)
c
c   store atoms in this fragment
c   ----------------------------
      nfratl=nfrat
      do 90 i=1,nya
	nfrat=nfrat+1
	if(nfrat.gt.mfrat) then
	  nfrat=mfrat
	  write(nout,1090) mfrat
 1090     format(' *ERROR*SPANAL*1090* More than ',I3,' atoms ',
     +             ' generated in fragment search')
	  go to 900
	  endif
	call jyat(i,iya)
	do 80 k=1,5
   80     iatfr(k,nfrat)=iya(k)
	iatfr(6,nfrat)=nfrag
   90   continue
c
c   analyse the infinite connectivity of the spanning tree
c   ------------------------------------------------------
      ianal(nfrag)=-1
      n=0
      do 150 i=1,nya
	if(iyatom(2,i).eq.0) then
	  n=n+1
	  if(n.gt.100) then
	    write(nout,1010)
 1010       format(//,' ++ERROR++SPANALYSE++1010++ more than',
     1        ' 100 atoms in JYATOM')
	    go to 900
	  else
	    do 100 j=1,3
  100         jyatom(j,n)=iyatom(j+2,i)
	    endif
	  endif
  150   continue
      if(n.gt.1) then
c
c                  see if any two atoms have same translation (jyatom)
c                  ---------------------------------------------------
	i=0
c       begin implicit loop i=1,n
  210     i=i+1
	  if(i.gt.n) go to 290
	  j=i
c         begin implicit loop j=i+1,n
  220       j=j+1
	    if(j.gt.n) go to 210
	    do 230 k=1,3
  230         if((jyatom(k,i)+jyatom(k,j)).ne.0) go to 240
	    go to 260
  240       do 250 k=1,3
  250         if((jyatom(l,i)-jyatom(k,j)).ne.0) go to 220
c               reduce the jyatom array if atoms have same translation
c               ------------------------------------------------------
  260       do 280 l=j,n-1
	      do 270 k=1,3
  270           jyatom(k,l)=jyatom(k,l+1)
  280         continue
	    n=n-1
	    go to 220
  290       continue
c           end implicit j loop
c         end implicit i loop
	endif
c
c                   analyse the reduced jyatom array for fragment type
c                   --------------------------------------------------
      do 291 k=1,3
  291   isum(k)=0
      do 295 l=1,n
	do 293 k=1,3
  293     isum(k)=isum(k)+iabs(jyatom(k,l))
  295   continue
      do 297 k=1,3
  297   jsum(k)=0
      if(isum(1).eq.0.and.isum(2).eq.0) jsum(3)=1
      if(isum(1).eq.0.and.isum(3).eq.0) jsum(2)=1
      if(isum(2).eq.0.and.isum(3).eq.0) jsum(1)=1
  300 if(n.eq.0) then
c                     the atoms form an isolated cluster
c                     ----------------------------------
	ianal(nfrag)=0
	write(nout,1300)
	if(ntout.ne.0) write(ntout,1300)
 1300   format(/,' Spanning tree is an isolated cluster',/)
      else if(n.eq.1.or.jsum(1).eq.1.or.
     +                  jsum(2).eq.1.or.jsum(3).eq.1) then
c                     the atoms from a chain
c                     ----------------------
	ianal(nfrag)=1
	do 305 k=1,3
  305     idir(k,nfrag)=jsum(k)
	if((jsum(1)+jsum(2)+jsum(3)).eq.0) then
	  do 307 k=1,3
  307       idir(k,nfrag)=jyatom(k,1)
c  ***PROGRAMMERS NOTE in the above loop idir should be divided by
c                   the lowest common factor to give a nice result
	  endif
	write(nout,1301) (idir(k,nfrag),k=1,3)
	if(ntout.ne.0) write(ntout,1301) (idir(k,nfrag),k=1,3)
 1301   format(/,' Spanning tree forms chains along ',3I3,/)
	do 310 k=1,3
  310     idir(k,nfrag)=jyatom(k,1)
      else
	if(n.eq.2.or.isum(1).eq.0.or.isum(2).eq.0.or.isum(3).eq.0) then
c                     atoms form a sheet
c                     ------------------
	  ianal(nfrag)=2
c    determine the direction of the sheets
c      ***PROGRAMMERS NOTE this analysis may not work correctly for all
c          possible cases, particularly when jyatom has few 0 elements
	  x=1
	  y=1
	  z=1
	  do 350 l=1,n
	    if(jyatom(1,l).eq.0.and.jyatom(2,l).eq.0) z=0.
	    if(jyatom(2,l).eq.0.and.jyatom(3,l).eq.0) x=0.
	    if(jyatom(1,l).eq.0.and.jyatom(3,l).eq.0) y=0.
  350       continue
	  do 360 l=1,n
	    if(jyatom(1,l).ne.0.and.jyatom(2,l).ne.0)
     +                           y=-x*jyatom(1,l)/jyatom(2,l)
  360       continue
	  do 370 l=1,n
	    if(jyatom(1,l).ne.0.and.jyatom(3,l).ne.0)
     +                           z=-x*jyatom(1,l)/jyatom(3,l)
  370       continue
	  do 380 l=1,n
	    if(jyatom(2,l).ne.0.and.jyatom(3,l).ne.0)
     +                           z=-y*jyatom(2,l)/jyatom(3,l)
  380       continue
	  idir(1,nfrag)=integer(x,0)
	  idir(2,nfrag)=integer(y,0)
	  idir(3,nfrag)=integer(z,0)
	  write(nout,1302) (idir(k,nfrag),k=1,3)
	  if(ntout.ne.0) write(ntout,1302)(idir(k,nfrag),k=1,3)
 1302     format(/,' Spanning tree forms sheets perpendicular to',
     +                               3I3,/)
	else
c
c            atoms form a framework
c            ----------------------
	  ianal(nfrag)=3
	  write(nout,1303)
	  if(ntout.ne.0) write(ntout,1303)
 1303     format(/,' Spanning tree forms a framework',/)
	  endif
	endif
c
c   find formula of fragment
c   ------------------------
  400 n=0
      do 410 k=1,20
	gelm(k,nfrag)='        '
  410   iform(k,nfrag)=0
      do 430 i=1,nka
	ii=iyatom(1,kyatom(i))
	do 420 j=1,n
	  if(gelm(j,nfrag).eq.gelem(ii)) then
	    iform(j,nfrag)=iform(j,nfrag)+1
	    go to 430
	    endif
  420     continue
	  n=n+1
	  if(n.gt.20) then
	    n=n-1
	    go to 440
	    endif
	  gelm(n,nfrag)=gelem(ii)
	  iform(n,nfrag)=iform(n,nfrag)+1
	  xox(j) = oxid(ii)
  430   continue
      nform(nfrag)=n
      call sorfor(gelm(1,nfrag),iform(1,nfrag),xox,nform(nfrag))
      do 435 i=1,80
  435   hfor(i:i)=' '
      call formul(hfor,gelm(1,nfrag),iform(1,nfrag),nform(nfrag))
  440 write(nout,1440) hfor
      if(ntout.ne.0) write(ntout,1440) hfor
 1440 format(/,' Formula of fragment is: ',A80)
c
c   find the charge of the fragment
c   -------------------------------------------------
  500 charge(nfrag)=0.
      do 510 i=1,nka
	charge(nfrag)=charge(nfrag)+oxid(iyatom(1,kyatom(i)))
  510   continue
      write(nout,1510) charge(nfrag)
      if(ntout.ne.0) write(ntout,1510) charge(nfrag)
 1510 format(/,' The charge on the fragment is ',F5.1)
c
c   find the coordination number and valence sum of each atom
c                     in the fragment
c   ---------------------------------------------------------
      do 620 i=1,nya
	ni=nfratl+i
	ifrcn(ni)=0
	vsum(ni)=0.
	if(iyatom(2,i).ne.0) then
	  do 610 j=1,nbo
	    if(ibond(1,j).eq.i.or.ibond(2,j).eq.i) then
	      ifrcn(ni)=ifrcn(ni)+1
	      vsum(ni)=vsum(ni)+xbond(3,j)
	      endif
  610       continue
	    if(oxid(iyatom(1,i)).lt.0.) vsum(ni)=-vsum(ni)
	  endif
  620   continue
c
c    calculate basicity and acidity of fragment
c    --------------------------------------
      mcn=0
      acidex=0.
      basex=0.
      do 650 i=1,nka
	ni=nfratl+kyatom(i)
	ii=iyatom(1,kyatom(i))
	if(oxid(ii).lt.0) then
	  ncn = 4-ifrcn(ni)
	  if(ncn.lt.0) ncn=0
	  mcn=mcn+ncn
	  basex=basex+oxid(ii)-vsum(ni)
	else
	  acidex=acidex+oxid(ii)-vsum(ni)
	  endif
  650   continue
      bases(nfrag)=0.
      basex=-basex
      if(mcn.ne.0) bases(nfrag)=basex/mcn
      write(nout,1650) bases(nfrag)
      if(ntout.ne.0) write(ntout,1650) bases(nfrag)
 1650 format(/,' The base strength of the fragment is about ',F5.2)
      write(nout,1651) basex
      if(ntout.ne.0) write(ntout,1651) basex
 1651 format(' Excess base valence on the fragment is ',F5.2,' v.u.')
      write(nout,1652) acidex
      if(ntout.ne.0) write(ntout,1652) acidex
 1652 format(' Excess acid valence on the fragment is ',F5.2,' v.u.')
      if(nout.eq.6) call wait
c
c   search for bonding points on the fragment
c   -----------------------------------------
      linc=0
      do 750 i=1,nka
	ni=nfratl+kyatom(i)
	iy=iyatom(1,kyatom(i))
	linc=linc+1
	ox=oxid(iy)
	resval=abs(ox)-abs(vsum(ni))
	if(resval.lt.0) resval=0.
	lcn=0
	strn=0.
	if(ox.gt.0) then
	  temp=resval/acid(nza(1,iy),nza(2,iy),ox)
	  lcn=integer(temp,0)
	else if(ox.lt.0) then
	  if(abs(bases(nfrag)).gt.0.005)
     +             lcn=iabs(integer(resval/bases(nfrag),0))
	  endif
	if(lcn.ne.0) strn=resval/lcn
	write(nout,1750) gelem(iy),gdent(iy),ifrcn(ni),lcn,strn
	if(ntout.ne.0) write(ntout,1750) gelem(iy),gdent(iy),
     +                     ifrcn(ni),lcn,strn
 1750   format(' Atom ',A2,A6,' has',I3,' bonds and probably ',
     +         'forms ',I2,' more bonds of strength ',F5.2)
	if(linc.ge.20.and.nout.eq.6) then
	  call wait
	  linc=0
	  endif
  750   continue
      if(ndebug.ne.0) then
	write(ndebug,1780) ianal(nfrag),(idir(k,nfrag),k=1,3)
 1780   format(' *DEBUG*SPANAL*1780* ',4I3)
	do 790 i=nfratl+1,nfrat
	  write(ndebug,1790)i,(iatfr(j,i),j=1,6),ifrcn(i),vsum(i)
 1790     format(' *DEBUG*SPANAL*1790* ',I4,I5,4I4,2I5,F7.2)
  790     continue
	endif
      if(nout.eq.6.and.linc.ne.0) call wait
c
c   ANALYSE THE NETWORK AS A FINITE NETWORK
c   ---------------------------------------
c    calculate the eigenvalues of the (finite) adjacency matrix
c    ----------------------------------------------------------
  800 write(nout,1800)
 1800 format(' Do you want to calculate the eigenvalues of the ',
     + 'adjacency matrix (Y/N/E) ')
      call readyn(*810,*890,*900,*800)
  810 if(mtrxfg.ne.0) then
	write(nout,1810) mmtrx
 1810   format(' ERROR: The order of the matrix exceeds the storage ',
     +  'size ',I5,'.',/,'  Calculation aborted')
	go to 900
	endif
      iwght=0
  816 write(nout,1816)
 1816 format(' Do you want to weight the adjacency matrix with the ',
     + 'bond valences (Y/N/E) ')
      call readyn(*817,*819,*900,*816)
  817 iwght=1
  819 do 828 i=1,nka
	do 820 j=1,nmtrx
	  if(iwght.eq.0) then
	    amtrx(j,i)=mtrx(j,kyatom(i))
	  else
	    amtrx(j,i)=xmtrx(j,kyatom(i))
	    endif
  820     continue
	do 822 j=1,nmtrx
	  if(iyatom(2,j).eq.0) then
	    amtrx(iyatom(1,j),i)=amtrx(iyatom(1,j),i)+amtrx(j,i)
	    endif
  822     continue
	do 824 j=1,nka
	    amtrx(j,i)=amtrx(kyatom(j),i)
  824     continue
  828   continue
      write(ntout,1820)
 1820 format('  Matrix whose eigenvalues are to be found',/,
     +       '  ----------------------------------------')
      do 821 i=1,nka
	write(ntout,1821) (amtrx(j,i),j=1,nka)
 1821   format(10(1X,10F7.3,/))
  821   continue
c
c   matz=1 is flag to calculate eigenvectors
      matz=1
      call rg(mmtrx,nka,amtrx,wr,wi,matz,xmtrx,fv1,iv1,ierr)
      if(ierr.ne.0) write(nout,1830) ierr
 1830 format(' ERROR: Iteration count exceeded for ',I4,'th eigen',
     +  'value.',/,
     +  '  Values after this are probably correct')
c
c   list eigenvalues
c
      do 838 i=1,nka
  838   wwr(i)=wr(i)
      call sorty(nka,wwr,iwr)
      write(nout,1840)
 1840 format('     List of Eigenvalues',/,
     +       '     -------------------',/)
      linc=0
      do 840 i=1,nka
	write(nout,1841) i,wr(iwr(i)),wi(iwr(i))
 1841   format(10X,I4,2F10.4)
	linc=linc+1
	if(nout.eq.6.and.linc.eq.20) then
	  call wait
	  linc=0
	  endif
  840   continue
      if(nout.eq.6.and.linc.ne.0) call wait
      write(nout,1850)
 1850 format('     List of Eigenvectors',/,
     +       '     --------------------',/)
      linc=0
      iinc = 1 + nka/10
      do 850 i=1,nka
	ii=iwr(i)
	sum=0.
	do 845 j=1,nka
  845     sum=sum+xmtrx(j,ii)*xmtrx(j,ii)
	sum=sqart(sum)
	write(nout,1851) (xmtrx(j,ii)/sum,j=1,nka)
 1851   format(10(1X,10F7.3,/))
	linc=linc+iinc
	if(nout.eq.6.and.linc.ge.20) then
	  call wait
	  linc=0
	  endif
  850   continue
      if(nout.eq.6.and.linc.ne.0) call wait
      write(nout,1852)
 1852 format(' End of eigenvalue calculation')
  890 continue
  900 return
      end
c
c
***********************************************************************
c
c
      subroutine spanou
c     -----------------
c
c   written 85.08.06   by I.D.Brown
c   updated 85.10.29   by D.Altermatt
c
c   outputs interactively the spanning tree
c
c   calls readyn, jyat
c
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      include 'combond.for'
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c
      dimension jyatom(5),kyatom(5)
c
c
      write(nout,1000)
 1000 format(///)
  100 write(nout,1100)
 1100 format(' Do you want a listing of the network (Y/N/E): ')
      call readyn(*120,*200,*900,*100)
  120 nout1=nout
      write(nout,1120)
 1120 format(/,' Do you want output on terminal (Y/N/E): ')
      call readyn(*130,*125,*900,*120)
  125 if(machine.eq.1) then
	write(nout,1125)
      else if(machine.eq.2) then
	write(nout,1126)
      else if(machine.eq.3) then
	write(nout,1127)
      else if(machine.eq.4) then
	write(nout,1128)
      else if(machine.eq.5) then
	write(nout,1129)
	endif
 1125 format(/,' Output written to DUMP.LIS ',/
     +  ,' (Will be opened if not yet existing)',/)
 1126 format(/,' Output written to DUMP:LIS ',/
     +  ,' (Will be opened if not yet existing)',/)
 1127 format(/,' Output written to DUMPLIS ',/
     +  ,' (Will be opened if not yet existing)',/)
 1128 format(/,' Output written to dump.list ',/
     +  ,' (Will be opened if not yet existing)',/)
 1129 format(/,' Output written to dump.lst ',/
     +  ,' (Will be opened if not yet existing)',/)
      nout=ndebug
      if(ndebug.le.0) then
	hfile='debug'
	nunit=0
	gstat='new'
	gfort='fortran'
	nout=ifile(hfile,gfort,gstat,nunit)
	endif
  130 linc=0
      do 150 i=1,nbo
	linc=linc+1
	call jyat(ibond(1,i),jyatom)
	call jyat(ibond(2,i),kyatom)
	write(nout,1130) gelem(jyatom(1)),gdent(jyatom(1)),
     1    (jyatom(j),j=2,5),gelem(kyatom(1)),gdent(kyatom(1)),
     2    (kyatom(j),j=2,5),xbond(3,i),xbond(1,i)
 1130     format(5X,A2,A6,4I3,1X,A2,A6,4I3,2F8.3)
	if(linc.ge.20.and.nout.eq.6) then
	  linc=0
	  call wait
	  endif
  150   continue
      if(nout.eq.6) call wait
      nout=nout1
      write(nout,1000)
  200 write(nout,1200)
 1200 format(//,' If you want to save the network to the ',/
     1 ,' SCFS-output file, you will have to overwrite the bond',/
     2 ,' array at this point.',/
     3 ,' NOTE: The lastnetwork is kept in memory for ',
     4  'plotting !!',/,'       (no overwriting necessary for plots)',
     5 //, ' Do you want to overwrite the bonds with the network',
     3  ' (Y/N/E): ')
      call readyn(*210,*260,*900,*200)
  210 if(nbo.gt.mbon) then
	nbo=mbon
	write(nout,1201) nbo
 1201   format(//,' Sorry: Only the first',I5,
     1    'bonds can be stored',/)
	call wait
	endif
      nbon=nbo
c   the next line is to permit all the bonds to be written to SCFS
      ind1=nbo
      do 250 i=1,nbo
	call jyat(ibond(1,i),jyatom)
	call jyat(ibond(2,i),kyatom)
	do 230 j=1,5
	  ibon(j,i)=jyatom(j)
	  jbon(j,i)=kyatom(j)
  230     continue
	do 240 j=1,6
  240     xbon(j,i)=xbond(j,i)
  250   continue
  260 continue
  900 return
      end
c
c
***********************************************************************
c
      subroutine closer
c     -----------------
c
c   written 85.11.4 by I.D.Brown
c
c   Takes atoms and bonds in /span/ and coalesces translationally
c   equivalent bonds and atoms to produce a finite bond network.
c   Note that iyatom(3-5,) no longer has any significance.
c
      common/span/   nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),
     +            iat,bondlim(2),netind(20),ifbond(4,600)
c
      dimension nbat(600)
c ***NB  dimensions of nbat are mya in /span/*****
c
c
      nyb=0
      do 30 i=1,mya
   30   nbat(i)=0
      do 100 i=1,nya
	if(iyatom(2,i).eq.0) then
	  do 50 j=1,nbo
	    if(ibond(1,j).eq.i) ibond(1,j)=nbat(iyatom(1,i))
	    if(ibond(2,j).eq.i) ibond(2,j)=nbat(iyatom(1,i))
   50       continue
	else
	  nyb=nyb+1
	  do 70 k=1,5
	    iyatom(k,nyb)=iyatom(k,i)
   70       continue
	  iterm(nyb)=iterm(i)
	  nbat(i)=nyb
	  do 80 j=1,nbo
	    if(ibond(1,j).eq.i) ibond(1,j)=nbat(i)
	    if(ibond(2,j).eq.i) ibond(2,j)=nbat(i)
   80       continue
	  endif
  100   continue
      nya=nyb
c
c   remove duplicate bonds
c
      nbb=0
      do 160 i=1,nbo
	do 140 j=1,nbb
	  if((ibond(1,i).eq.ibond(2,j).and.ibond(2,i).eq.ibond(1,j)))
     +     go to 160
  140     continue
	nbb=nbb+1
	ibond(1,nbb)=ibond(1,i)
	ibond(2,nbb)=ibond(2,i)
	do 150 k=1,6
  150     xbond(k,nbb)=xbond(k,i)
	do 155 k=1,4
  155     ifbond(k,nbb)=ifbond(k,i)
  160   continue
      nbo=nbb
      return
      end
c
c
***********************************************************************
c
      subroutine tempan
c
c   written 87.10.23 by I.D.Brown
c
c  Analyses ADP's in terms of thermal motion.
c
c  calls: NETLIB: vlink
c         MAINLB: zedsq
c
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/nin,nout,ndebug
      include 'comatom.for'      
      common/sym/ nsym,msym,s(3,3,48),t(3,48),ntl,tu(3,4)
      common/span/nya,mya,iyatom(5,600),iterm(600),
     +            nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2)
c  DIF has dimensions of nya
      dimension x(3),dif(600),jyatom(5),ui(3,3),uj(3,3)
      ntout=nout
c
c   The following test to see if atom 1 has anisotropic atomic dispalcement
c   parameters stored.  If not the calculation is aborted.  This may not
c   be the best test.
      test=0.
      do 30 i=1,3
	do 30 j=1,3
	  test=test+abs(u(i,j,1))
   30     continue
      if(test.lt.0.00001) then
	write(ntout,1000)
 1000   format(' AADPs not stored, calculation not possible')
	return
	endif
c
c   TEST FOR RIGID BODY
c   -------------------
      write(ntout,1050)
 1050 format(/,' Rigid Body Test   sqrt(z(i)**2-z(j)**2)',/,
     +         ' ---------------------------------')
      do 100 i=1,nya
	call jyat(i,jyatom)
	ii=jyatom(1)
	is=jyatom(2)
	call symtem(ui,s(1,1,is),u(1,1,ii))
	if(ndebug.ne.0) then
	  write(ndebug,1060) i,ii,is
 1060     format(' *debug*tempan*1060* i=',I4,'; ii=',I4,'; is=',I4)
	  endif
	linc=0
	iinc = 1 + nya/10
	do 80 j=1,nya
	  if(j.eq.i) then
	    dif(j)=0.0
	  else
	    call vlink(i,j,x,ier)
	    call jyat(j,jyatom)
	    jj=jyatom(1)
	    js=jyatom(2)
	    call symtem(uj,s(1,1,js),u(1,1,jj))
	    temp=zedsq(x,ui)-zedsq(x,uj)
	    dif(j)=sqart(abs(temp))
	    endif
   80     continue
	linc=linc+iinc
	write(ntout,1100) gelem(ii),gdent(ii),is,(dif(j),j=1,nya)
 1100   format(1X,A2,A6,I3,5F6.3,1X,5F6.3,60(/,12X,5F6.3,1X,5F6.3))
	if(ntout.eq.6.and.linc.ge.20) then
	  call wait
	  linc=0
	  endif
  100   continue
      if(ntout.eq.6.and.linc.ne.0) call wait
      return
      end
c
c
***********************************************************************
c
      subroutine spanhelp
c
c       written by Lucien Jamieson - April 10, 1989
c
c       Used as a help screen for Spanner.
c
       common /files/nin,nout
100    write(nout,1000)
1000   format(' '////,
     +5X,'                           Network Module '//,
     +5X,'  This module explores the bond topology of a crystal. '/,
     +5X,'The unit cell, space group and asymmetric bond set '/,
     +5X,'(e.g. as calculated by STRUMO) are needed but not the '/,
     +5X,'atomic coordinates.'/,
     +5X,'  The asymmetric set of bonds is expanded in a number of ')
       write(nout,1100)
1100   format(' ',
     +5X,'possible ways:'/,
     +5X,'   1. TO FILL ONE UNIT CELL'/,
     +5X,'   2. TO NTH NEAREST NEIGHBOURS OF A GIVEN ATOM'/,
     +5X,'   3. TO SEARCH FOR FRAGMENTS'/,
     +5X,'The user can select the maximum and minimum values of the '/,
     +5X,'valences of the bonds to be included in the expanion. ')
       write(nout,1200)
1200   format(' ',
     +4X,'The expansion is finished when the designated region is '/,
     +5X,'filled or no further bonds of the correct valence are found,'/,
     +5X,'whichever occurs first.   When the expansion is completed '/,
     +5X,'the module determines if the network found is '/,
     +5X,'isolated or infinitely connected and, if the latter, in what'/,
     +5X,'directions. '//)
       call pageflip(*200,*100,*9999)
 200   write(nout,1300)
1300   format(///' ',
     +5X,'  The charge on the fragment and its base strength are '/,
     +5X,'calculated as well as an estimate of how many further bonds'/,
     +5X,'each atom may form.  The eigen values and eigen vectors of '/,
     +5X,'the connectivity matrix may, optionally, be calculated '/,
     +5X,'though it is not clear what significance these have except '/,
     +5X,'as a way of characterising the network.')
       write(nout,1400)
1400   format(' ',
     +5X,'  The anisotropic atomic diplacement parameters may be '/,
     +5X,'analyed for potential rigid fragments of the network using '/,
     +5X,'the method of Rosenfeld, Trueblood and Dunitz Acta Cryst. '/,
     +5X,'A34 (1978) 825.'////////////)
       call pageflip(*9999,*100,*9999)
9999   return
       end
