c
c               FILE STRUMO (export version)  Last Update: 21 April, 1993
c                   -----------                            T. Stephen
c   
c                              Version 93.1
c
c             *************************************************
c             *   COPYRIGHT 1991  I.D.BROWN and D.ALTERMATT   *
c             *************************************************
c                      Institute for Materials Research
c                            McMaster University
c                         Hamilton, Ontario, Canada
c
*****************************************************************************
c   Machine dependent code is indicated by the phrase 'machine dependent'
*****************************************************************************
c   This file contains DOUBLE PRECISION arrays marked +
c****************************************************************************
c
c 
c
c    this file contains the following, MOSTLY INTERACTIVE routines:
c
c  p+strumo              main program of structure modelling package
c  s struskip            file entry positioning routine
c  s strusrch            file entry search routine
c  s skpbnf              slave of struskip for bondfiles
c  s skpicsd             slave of struskip for icsd archive files
c  s skpcrn              slave of struskip for crystn files
c  s rident(mp)          reads/updates common /ident/
c  s rxatom(mp)          reads/updates common /xatom/
c  s rupdat(mp,*,*)      selects data updates from a menu
c  s rtemp               reads/updates common /temp/
c  s wtemp(i,isofl)      slave of rtemp
c  s rbonds              reads/updates common /bonds/
c  s raucom(ja,ha,m,gt)  updates authors and remarks
c  s roxide(mp)          updates oxidation states in common /ident/
c  s runpro(*,*)         structure modelling processor, main routine
c  s traltn(*,*)         transforms atomic positions to standard settings
c  s struhelp            accesses pages of help info (menu 3.1)
c
c    externals called from this file:
c  BONDER : bondl, bondhelp
c  CHEMLB : chemno, oxide, oxum, bost, qsdere
c  DISTLB : indepb, (corget)
c  ICSDLB : skpfiz, crystn, rdicsd
c  IOLIB  : iclosu,ifile,hfilen,rval,fdbonf,rdbonf,loadvl,atomgr,qmenu,chemgr
c  MAINLB : inerr, param, reseta, sghall, spacrd, resetc, resetb, losym
c  MATHLB : length,readyn,second,upcase,wait,mxv,vplusv,transp,pageflip
c  MAPLIB : valmap, maphelp
c  MODCOM : (fitcom)
c  MODOUT : (polyhe), (psuplo)
c  NETLIB : kirch, nethelp
c  PLOTLB : plotmn
c  SCFSLB : wscfs,skpscf,rscfs,find
c  SHELIB : shelib
c  SPANNER: spanner, spanhelp
c  SPELIB : spesor, spmult, spelib, naltno, symhelp, spsym
c  SYMLIB : grmult, conxyz
c  VISLS  : visls
c
c
c****** Machine specific code used in this file
c****** Set 'machine'=1 for VAX running VMS 4.x, VMS 5.x (so far)
c                     2 for NORD running SINTRAN III
c                     3 for CYBER running NOS/VE
c                     4 for UNIX under Sun  (not completed)
c                     5 for PC (in progress)
c
c****** This program is interactive (nterm=6)
c
c
c
c
c
      program strumo
c
c     this program helps you model structures  of inorganic materials
c     
c     calls chemno, fdbonf, iclosu, ifile, inerr, param, rdbonf, readyn,
c           reseta, rident, roxide, rscfs, rupdat, second, sghall, shelib,
c           spacrd, upcase, wscfs, resetc
c
c     the following file assignements are used:
c
c     unit = 5    system read input      (terminal screen)  NIN
c            6    system write output    (terminal screen)  NOUT
c            7                           (unused)
c            8                           (unused)
c
c            9    debug and dump output  (file DUMP.LIS)    NDEBUG, NWRITE
c
c           10    SCFS input             (readonly)
c           11    SHELX input            (readonly)
c           12    ICSD-bondfile as input (readonly)
c           13    spacegroup file        (readonly)
c           14                           (unused)
c
c           15    SCFS output            (standard system file output)
c           16                           (unused)
c           17    CHEMGRAF data file     ('transfer' file)
c           18    CHEMGRAF command file  ('transfer' file)
c
c     written by D. Altermatt
c     McMaster University, June 1984
c
c     facelifted by C. Carr
c     McMaster University, Feb-July 1989
c
c     further alterations by S.Urquhart
c     McMaster University, August 1989
c
c     even further alterations by S. Healy
c     McMaster University, September 1990
c
c
c
c
c
c
c
c
c
c
c     first initialize some arrays etc.
c
      implicit character*8 (g)
      implicit character*80 (h)
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
c /matrix/ is /matrixx/ for UNIX system
      common/matrixx/ nm,mm,xm,v,im(500,500),bwgt(500),cwgt(500)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine,nterm
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /rcell/  as,bs,cs,ral,rbe,rga,rvol,rm(3,3)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      common /switch/ jfix,ierra,nobe,noan,ifang,irang,ibnd
c
c PROGRAMMER'S NOTE :  the following data statements may need to be
c                       placed into the COMATOM.FOR, COMBOND.FOR
c                       include files when porting to VAX.
c

      include 'comatom.for'
      data mx,my /300,1500/
      include 'combond.for'
      data mbon,mcon /1000,1000/
      common /oatom/  nyo,myo,oyx(3,1000)
      common /dens/   zform,dens,ddens,cdens,fwgt,waveln,absorb,
     1                fzero,tmprtr
c programmer's note:  /bost/ is changed to /bostt/ for
c                     UNIX system
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),
     1                pbost(5,30)
      common /gbost/  gbost(2,30)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /range/  nj,iox(20),top(20,20),bot(20,20),ddmax,
     1                ddmin,rmax,rmin,vmax,vmin,anglx
      common /grange/ gel(20)
      common /val/    xpm(3,3),pmap(3),dmax,valim(2),jt,
     1                nz1,igrid(3)
      common /gval/   gor(3),gem(20),gdm(20)
      common /gmenu/  hmenu(20,3)
      common /asymu/  iasym,aslim(3,2),isyel(3,2)
      common /span/   nya,mya,iyatom(5,600),iterm(600),
     1                nbo,mbo,ibond(2,600),xbond(6,600),
     2                iat,bondlim(2),netind(20),ifbond(4,600)
C
      common /href/  hinfo(2,2), gfilen(5),hist(40)
      common /gcond/ gcrad,girad,gdiff, gabs, grefn, gstat
      common /cond/  condit(6,8), nstd,mstd,cstd(6,4)
      common /htext/ gcryst(2), gtwin(5), htxt(10,5), hdata(4,19)
      common /form/ nform(3), mform, form(24,3)
      common /gform/ gform(24,3)
      common /counter/ icount
      dimension hau(10),hfm(2)
c
c     initialize CPU clock
c
      timcal=second(1)
c
c     initialize some constants
c
*******************************************
* The next lines are machine dependent code *
*******************************************
      machine=5
c  HVALENCE is the name of the file containing the bond valence parameters
c  HSPACEGR is the name of the file containing the Hall space group symbols
      if(machine.eq.1) then
        hvalence='D2:[ZX008.ZX008]VALENCE.DAT'
        hspacegr='D2:[ZX008.ZX008]SPGR.DAT'
      else if(machine.eq.2) then
        hvalence='VALENCE.DAT'
        hspacegr='SPGR.DAT'
      else if(machine.eq.3) then
        hvalence='VALENCE'
        hspacegr='SPGRF'
      else if(machine.eq.4) then
        hvalence='valence.dat'
        hspacegr='spgr.dat'
      else if(machine.eq.5) then
        hvalence='valence.dat'
        hspacegr='spgr.dat'
        endif
*******************************************
      pi=3.141593
      radeg=57.2958
      nin=5
      nout=6
      nterm=6
      ndebug=0
      nread=5
      nwrite=9
      nshel=11
      ishel=0
      nspgp=13
      hft2='not open'     
      hsession='                                                '
c  reset arrays (including setting maximum array size variables)
      mm=500
      call reseta
      mya=600
      mbo=600
c
C     SET /SWITCH/ FLAGS
      JFIX=0
      IERRA=0
      NOBE=0
      NOAN=0
      IFANG=0
      IRANG=0
      IBND=1
c     Initialize INPUT and OUTPUT and open a DEBUG/DUMP file on NWRITE
c
      if(machine.eq.1) hfile='DUMP.LIS'
      if(machine.eq.2) hfile='DUMP:LIS'
      if(machine.eq.3) hfile='DUMPLIS'
      if(machine.eq.4) hfile='dump.list'
      if(machine.eq.5) hfile='dump.lst'
      gfort='fortran'
      gstat='unknown'
      if(machine.eq.1) gstat='new'
      nunit=nwrite
      nwrite=ifile(hfile,gfort,gstat,nunit)
      if(nwrite.le.0) goto 99
c   -----------------------
c   Initialize the program
c   ----------------------
c
      write(nwrite,1001)
 1001 format('1',/////////,5X,
     +       ' File written by STRUMO for printed output',/,
     +   5X,' -----------------------------------------',//)
c
c     entry point for re-runs
c
      irerun = 0
    1 call reseta
      CALL RESETB
      call resetc
      iasym=0
      icount=0
      isptab(1,1)=0
      zform=1
c   ---------------------------------------------
c     entry point to add data to common blocks
c   ---------------------------------------------
    5 irerun = irerun + 1
      if(irerun.gt.1) then
    6   write(nout,1980)
 1980   format(' Do you wish to read from the same file (Y/N/E) ')
        call readyn(*7,*9,*99999,*6)
    7   write(nout,1983)
 1983   format(' Do you want this file rewound (Y/N/E) ')
        call readyn(*8,*600,*99999,*7)
    8   rewind nread
        go to 45
    9   continue
        endif
      if(irerun.gt.1) then
        write(nwrite,1990)
        write(nwrite,1007) hsession
 1990   format('1')
        endif
      write(nout,1998)
1998  format(////,'STRUMO',/,
     +'A Modelling Program for Inorganic Structures',///,
     +'I.D.Brown, D.Altermatt and other',/,
     +'Institute for Materials Research',/,
     +'McMaster University, Hamilton, Ontario L8S 4M1',/,
     +'Canada',/,
     +'idbrowm@mcmaster.ca',//,
     +'Version 1993.1',/,
     +'1993.04.26',/,
     +'Copyright 1991 I.D.Brown and D.Altermatt',//////)
      call wait
c    -------------------------
c     read data file
c    ------------------------
*  next line inserted while debugging only ********************
*      nout=nwrite
      write (nout,1995)
   10 write (nout,2000)
1995  format(/,'             WELCOME TO STRUMO'/,
     +2X,'           -----------------'//,
     +5X,'STRUMO is an interactive STRucture MOdelling program.'/,
     +5X,'For more information choose Help (option 1) from the'/,
     +5X,'menu below.'//)
2000  format(/,5X,'INPUT MENU (level 1)'//,
     +2X,'Choose one of the following functions:'/,
     +5X,'  1: Help',/,
     +5X,'  2: Read SCFS file',/,
     +5X,'  3: Read SHELX file',/,
     +5X,'  4: Read Bondfile',/,
     +5X,'  5: Read ICSD- CISTI file',/,
     +5X,'  6: Read ICSD- FIZ file',/,
     +5X,'  7: Enter data from terminal'//,
     +5X,' 16: Switch on debug output',/,
     +5X,' 17: Switch off debug output',/,
     +5X,' 18: Assign a name to this session'/,
     +5X,' 19: EXIT from STRUMO',/,
     +5X,' 20: Call menu level 2',//,' Enter your selection: ')
      read(nin,1060,err=15) ifltyp
1060  format(I5)
      if(ifltyp.eq.20) goto 50
      if(ifltyp.eq.19.and.irerun.eq.1) goto99999
      if(ifltyp.eq.19) goto120
      if(ifltyp.eq.18) goto1005
      if(ifltyp.eq.17) then
        ndebug=0
        go to 10
        endif
      if(ifltyp.eq.16) then
12      ndebug=nwrite
        write(nout,1061) hfilen(ndebug)
 1061   format(/,' Debug output written to file ',A30)
        go to 10
        endif
      if(ifltyp.lt.1.or.ifltyp.gt.7) goto 15
c
c
c
      if(ifltyp.eq.1) then 
        call struhelp
        goto 10
        endif
      if(ifltyp.eq.7) then
        goto 50
        endif
      goto 20
c   --------------------
c   ifltyp.eq.18
c   --------------------
1005  write(nout,1008)
1008  format(//,5X,'  Give the name of this session below.  This'/,
     +5X,' name will appear at the top of the file DUMP.LIS'/,
     +5X,' which contains output that does not appear on the'/,
     +5X,' screen.'/)
      read(nin,1006) hsession
1006  format(A80)
      write(nwrite,1007) hsession
1007  format(//,5X,'******************************************',
     1              '*****************',/,
     2          5X,'*',62X,'*',/,
     3          5X,'* ',A60,' *',/,
     4          5X,'*',62X,'*',/,
     5          5X,'******************************************',
     6              '*****************',//)
      goto 10
15    call wait
      call inerr(*10)
c   --------------------------------
c   start reading data from the file
c   --------------------------------
20    write (nout,2020)
2020  format (/,' Name of file (including extension) : ')
      read (nin,1010,err=20) hfname
1010  format (A40)
      if(hsession(1:5).eq.'     ') then
        hsession=hfname
        write(nwrite,1007) hsession
        endif
      do 35 i=1,41
35      if(hfname(i:i).eq.' ') goto 36
36    l1=i-1
      if(ifltyp.eq.2) then
        nunit = 10
        hft2 = 'open'
      else if(ifltyp.eq.3) then
        nunit = 11
      else
        nunit = 12
        endif
      gstat='old'
      gfort=' '
      nread=ifile(hfname,gfort,gstat,nunit)
      if(nread.le.0) then
        call wait
        goto 10
        endif
45    write(nout,2040)
2040  format(//,1X,' Reading the first entry from the file'//)
      hentnm=' '
   42 icount=0
      rewind nread
      call reseta
      call resetb
      call resetc
      nent=1
      call struskip(ifltyp,nent,hfname)
c
c
c
c  *** Here I start the menu business, methinks ****
c
c  Give details of structure found and ask for instructions.
c
c
  600 nout=6
      n=0
      jau=1
      linc=0
      do 601 ia=1,10
  601 hau(ia)=' '
      hfm(1)=' '
      hfm(2)=' '
      write(nout,1605) gent
      linc=linc+1
 1605 format(/,2X,'Entry code :',13X,A8)
  610 write(nout,1615) hname
      linc=linc+1
 1615 format(2X,'Entry name :',13X,A52)
620   write(nout,625) hcodn(1),hcodn(2)
      linc=linc+1
625   format(2X,'Literature references :',2X,A30,2X,A20)
630   write(nout,635) htitle(1)(1:55),htitle(2)
      linc=linc+2
635   format(2X,'Title of Paper :',7X,A55,/,2X,A70)
      do 636 i=1,jauth       
      l=length(hauth(i))
      if(n+l.gt.80) then
        jau=jau+1
        n=0
        endif
      hau(jau)(n+1:n+l)=hauth(i)(1:l)
      hau(jau)(n+l+1:n+l+2)=','
      n=n+l+2
636   continue
      write(nout,638) hau(1)
      linc=linc+jau
638   format(2X,'Authors :',A68)
      if(jau.gt.1) then 
        write(nout,638) (hau(i),i=2,jau)
        endif
      if(hform(1).eq.' ') then
        hfm(1)=hdata(5,1)
        hfm(2)=hdata(5,2)
        goto 650
        endif
      hfm(1)=hform(1)
      hfm(2)=hform(2)
      linc=linc+1
650   write(nout,655) hfm(1),hfm(2)
655   format(2X,'Formula :',2X,A30,5X,A30)
      write(nout,675) hcomm(1)
675   format(2X,'Comments :',2X,A80)
      linc=linc+1
      if(kcomm.lt.2) goto 690
      do 680 i=2,kcomm
        do 682 ia=1,80
 682      if (hcomm(i)(ia:ia).ne.' ') goto 681
        goto 680
 681    write(nout,685) hcomm(i)
        linc=linc+1
 680    continue
685   format(2X,A77)
      linc=linc+4
690   write(nout,695) grfact,grad
695   format(2X,'R-Factor & Radiation used :',A4,5X,A1)
710   write(nout,715) a,b,c,alpha*radeg,beta*radeg,agamma*radeg
715   format(2X,'Unit cell dimensions (A):',3(F9.3,2X),/,
     +       2X,'Unit cell angles (deg):  ',3(F9.3,2X))
730   write(nout,735) hmsym,hall
735   format(2X,'Hermann-Mauguin space group symbol ',A10,' Hall ',
     +'symbol ',A10)
c
      if (linc.ge.16) call wait
750   write(nout,755)
755   format(' ----------------------- Menu level 1.2 --------------',
     + '--------------------',/,
     +5X,'1: Preview next entry       5: Copy this entry to a file'/,
     +5X,'2: Move to another entry    6: View this entry'/,
     +5X,'3: Preview previous entry  19: Return to menu 1'/,   
     +5X,'4: Move to specific entry  20: Store this entry (=>menu2)')
764   write(nout,765)
765   format(2X,'Enter option number : ')
770   read(nin,775,err=764) iview
775   format(I2)
      if(iview.eq.6) then
c     --------------------
920     write(nout,925)
925     format(/,2X,' Ah yes, haven''t quite finished this yet ...')
        call wait
c
c   I don't know what constitutes an entry so in here there must go
c   all the write statements applicable (with pageflips) to displaying 
c   an entry in its entirety, rather than the preview.
c
        goto600
        endif
      if(iview.eq.5) then
c     -------------------
900     write(nout,905)
905     format(/,2X,'Enter name of destination file : ')
        read(nin,907,err=900) hdest
907     format(A30)
909     write(nout,915)
915     format(/,2X,'Not yet written !!')
c
c *** code in here to do copying ***
c
        call wait
        goto 600
        endif
      if(iview.eq.4) then
c     -------------------
c 
c *** moving to another entry in same file ***
c
800     write(nout,803)
803     format(//////,2X,'Move to specific entry .....')
        write(nout,804)
804     format(/,2X,'Please input code of desired entry : ')
        read(nin,805,err=800) hentnm
805     format(A30)
        call upcase(hentnm)
        rewind nread
        icount=0
        call reseta
        call resetb
        call resetc
        call strusrch(nent,hentnm,ifltyp)
        goto 600
        endif
      if(iview.eq.1) then
c     -------------------
        nent=1
        call reseta
        call resetb
        call resetc
        call struskip(ifltyp,nent,hfname)
        goto 600
        endif
      if(iview.eq.2) then
c     -------------------
820     write(nout,823)
823     format(/,2X,'Please type number of entries to move 
     +(+=forwards,-=backwards) : ')
        read(nin,824,err=820) nent
824     format(I5)
        if ((icount.le.1).and.(nent.lt.0)) goto 600
        if ((icount+nent).eq.1) then
          nent=icount-1
          rewind nread
          icount=0
          goto 880
          endif
 875    if(nent.lt.1) then 
          nent=icount+nent
          if (nent.lt.1) nent=1
          rewind nread
          icount=0
          endif
        if (nent.eq.1) goto 880
        nent=nent-1
        call struskip(ifltyp,nent,hfname)
  880   call reseta
        call resetb
        call resetc
        nent=1
        call struskip(ifltyp,nent,hfname)
        goto 600
        endif
      if(iview.eq.3) then
c     -------------------
        if (icount.le.1) goto 600
        if (icount.eq.2) then
          nent=icount-1
          rewind nread
          icount=0
          goto 830
          endif
        nent=icount-2
        rewind nread
        icount=0
        call struskip(ifltyp,nent,hfname)
        nent=1
  830   call reseta
        call resetb
        call resetc
        call struskip(ifltyp,nent,hfname)
        goto 600
        endif
      if(iview.eq.19) goto 10
c     -----------------------
      if(iview.eq.20) goto 50
c     -----------------------
      write(nout,799) iview
799   format(/,2X,' *** option ',I2,' does not exist ***'/,
     + ' Enter a number from the list:')
      call wait
      goto750
c
c
c   ------------------------------
c   ask for missing information
c   ------------------------------
52    write(nout,2050)
2050  format(//,' Read information from an additional input file',
     1  ' (Y/N): ')
      call readyn(*5,*50,*120,*52)
50    write (nout,2070)
2070  format (///,1X,'Checking for missing key information',//)
c   ---------------------
c   check entry number
c   ---------------------
55    if (gent.eq.'        ') then
         write (nout,2080)
2080     format (//,' Enter an entry identification (max. 8 chara',
     1            'cters): ')
           read (nin,1030,err=59) gent
1030       format(A8)
59       continue
      endif
c   ----------------------
c   check chemical formula
c   ----------------------
      if (hform(1).eq.'          '.and.hdata(1,5).eq.'     ') then
         write (nout,2090) 
2090     format (/,1X,'Enter the chemical formula (max. 80 character',
     1            's): ',/)
         read (nin,1040,err=63) hdata(1,5)
1040     format (A80)
63       continue
      endif
c   -------------------------
c   check lattice parameters
c   -------------------------
      if (icell.eq.0) then
60       write (nout,2100)
2100     format (/,1X,'Enter cell constants a,b,c (in Angstroms),alp',
     1            'ha,beta,gamma (in degrees): ',/)
         if(machine.eq.3.or.machine.eq.5) then
           read (nin,*,err=65) a,b,c,alpha,beta,agamma
         else
           read (nin,1050,err=65) a,b,c,alpha,beta,agamma
           endif
1050     format(6F10.0)
         write (nout,2110)
2110     format (//,' Enter errors for cell constants : ')
         if(machine.eq.3.or.machine.eq.5) then
           read (nin,*,err=65) da,db,dc,dalpha,dbeta,dgamma
         else
           read(nin,1050,err=65) da,db,dc,dalpha,dbeta,dgamma
           endif
         summ=a+b+c+alpha+beta+agamma
         if (summ.lt.0.1) call inerr(*60)
         goto 67
65       call inerr(*60)
67       write (nout,2130) a,b,c,alpha,beta,agamma,da,db,dc,
     1           dalpha,dbeta,dgamma
2130     format (/,1X,'Lattice constants and errors',//,
     1       '   a         b         c         alpha     beta   ',
     2       '   gamma',//,2(6F10.5,/),//,' Section ok (Y/N/E) : ')
         call readyn(*95,*60,*120,*43)
43       call inerr(*67)
95       alpha=alpha*pi/180.
         beta=beta*pi/180.
         agamma=agamma*pi/180.
         dalpha=dalpha*pi/180.
         dbeta=dbeta*pi/180.
         dgamma=dgamma*pi/180.
         icell=1
         call param
      endif
c   ----------------
c   check space group
c   ----------------
      if(hall.eq.' ') then
         if(hmsym.eq.' ') then
112         write(nout,2125)
2125        format(//,' Enter Hermann-Maugin space group symbol: ')
            read(nin,1040,err=112) hmsym
            if(hmsym.eq.' ') goto 111
            call upcase(hmsym)
            endif
         hfile=hspacegr
         gstat='OLD'
         gfort=' '
         nunit=nspgp
         nn=ifile(hfile,gfort,gstat,nunit)
         if(nn.le.0) goto 111
         nspgp=nn
         call spacrd
         endif
  111 if (hall(1:5).eq.'     ') then
   73    write (nout,2120)
 2120    format (//,' Enter Hall-symbol for space group : ')
         read (nin,1040,err=73) hall
         if (hall.eq.' ') call inerr(*73)
         ipp=ichar('a')
         ipt=ichar('z')
         ipc=ichar('A')
         do 77 i=1,80
            if(ichar(hall(i:i)).lt.ipp.or.ichar(hall(i:i)).gt.ipt)
     1        goto 77
            ipq=ichar(hall(i:i))-ipp+ipc
            hall(i:i)=char(ipq)
77          continue
         endif
      if (nsym.eq.0.or.glat.eq.' ') call sghall
c   -----------
c   check atoms
c   -----------
      mp = 0
      if (nx.ne.0) then
         do 75 i=1,nx
75       call chemno(gelem(i),nza(1,i),nza(2,i))
      else
         call rident(mp)
         endif
      sum=0.0
      do 100 i=1,nx
100     sum=sum+abs(oxid(i))
      if(sum.lt..1.and.nx.ne.0) call roxide(mp)
      mp=0
c   ---------------------
c   find valence parameters
c   ---------------------
      call loadvl(hvalence,nn)
c   --------------------------------------------
c   --------------------------------------------
c   perform calculations (menu 1 is in rupdate)
c   -------------------------------------------
c   -------------------------------------------
110   call rupdat(mp,*5,*1,*10)
c   ----------------------------
c   write scfs output if desired
c   ----------------------------
120   write(nout,2200)
2200  format(//,' Do you want to write an SCFS file (Y/N/E): ')
      call readyn(*125,*910,*99999,*120)
125   if (hft2.eq.'not open') goto 140
      if (l1.lt.41) hfname(l1+1:41)=' '
130   write(nout,2210) hfname
2210  format(//,' The current name for the SCFS file is: ',A41,//,
     1    ' Do you want a new name for the SCFS file (Y/N/E): ')
      call readyn(*140,*160,*99999,*130)
160   nunit=0
      nn=iclosu(nunit,hfname)
      if(nn.ne.0) then
        write(nout,2810)
2810    format(//,' ***** Cannot close former SCFS file *****',/)
        goto 120
        endif
      goto 150
140   write (nout,2800)
2800  format (/,1X,'Enter a new SCFS-output file name')
890   write (nout,2020)
      read (nin,1010,err=890) hfname
c resetting l1 for "clean output" for the new filename
      do 145 i=1,41
145     if(hfname(i:i).eq.' ') goto 146
146   l1=i-1

150   if(machine.eq.1) then
         gstat='new'
         gfort='list'
      else if(machine.eq.2) then
         gstat='unknown'
         gfort='RW'
      else if(machine.eq.3) then
         gstat='unknown'
         gfort=' '
      else if(machine.eq.4) then
         gstat='new'
         gfort=' '
      else if(machine.eq.5) then
         gstat='new'
         gfort=' '
         endif
      nunit=0
      nn=iclosu(nunit,hfname)
      nunit=15
      nn=ifile(hfname,gfort,gstat,nunit)
      if(nn.le.0) goto 120
      call wscfs(nn)
      if (l1.lt.41) hfname(l1+1:41)=' '
      write (nout,2900) hfname
2900  format (/,1X,'SCFS-output written to ',A41,/)
910   write(nout,2910)
2910  format(//' Re-run STRUMO for another entry (Y/N): ')
      call readyn(*1,*99999,*99999,*910)
99    write (nout,*) ' ***** ERROR ***** EMPTY FILE NAME'
99999 stop 'END OF PROGRAM STRUMO'
      end
c
c
***********************************************************************
c
c
      subroutine STRUSKIP(ifltyp,nent,hfname)
c     =======================================
c
c                                 written July 1989
c                                 S.Urquhart
c  Purpose:  to serve as a slave to strumo(r) in handling input from
c  files, with proper array cleaning up before input, and entry counter 
c  handling. It calls a number of small routines to handle skips for 
c  different file protocols
c  An input of nent=1 is meant to read the next entry, nent=2 the second
c  next and so on. Icount is the counter of entries sucessfully read in
c
c
c  calls from:
c  Mainlb:     param,reset*(a,b,c)
c  This file:  skpcrn,skpfree,skpbnf
c  Shelib:     shelib
c  Scfslb:     skpscf
c  Icsdlb:     skpfiz 
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug,nread,nwrite
      common /title/  jauth,kcomm
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /counter/ icount
c
      a=0
      b=0
      c=0
      alpha=0
      beta=0
      agamma=0
      iflag=0
      jauth=0
      kcomm=0
      ierr=0
  96  continue
  10  if (ifltyp.eq.2) then
c
c  Read from SCFS
c
        ntem=nout
        nout=nwrite
        call skpscf(nent,ierr)
        nout=ntem
      else if(ifltyp.eq.4) then
c
c  Read from bond file
c
        call skpbnf(nent,ierr)
      else if(ifltyp.eq.5) then
c
c  Read from CRYSTIN (CISTI version)
c
        call skpcrn(nent,ierr)
c  
c  Read from ICSD (CD-ROM)
c
      else if(ifltyp.eq.6) then
        call skpfiz(nent,ierr)
      else if(ifltyp.eq.3) then
c ...not tested.....
        write(nout,1035)
 1035   format(' Shelx file input is set up to read one file entry only
     +.')
        rewind nread
        call shelib(nread)
        goto 999
        endif
      icount=icount+nent
      if (iflag.eq.1) then
        call reseta
        call resetb
        call resetc
        jauth=0
        kcomm=0
        ierr=0
        a=0
        b=0
        c=0
        alpha=0
        beta=0
        agamma=0
        iflag=0
        nent=1
        goto 10
        endif
      if (ierr.eq.-1) then
        write(nout,1040)
 1040   format(' A error occured while reading in this file')
        endif
      if (ierr.eq.1) then
        write(nout,1050)hfname
 1050   format(' ** END OF FILE ** on file ',A16)
        iflag=1
        rewind nread
        nent=icount-1
        icount=0
        goto 10
        endif
 999  if (icell.ne.0) call param
      return
      end
c
***********************************************************************
c
      subroutine STRUSRCH(nent,hent,ifltyp)
c     =====================================
c
c                                             written July 23, 1989
c                                             S.G.Urquhart
c  Slave of strumo(r).....this routine handles searches for the improved
c  menu-ing of strumo(r)... Note: all file handling programs called from 
c  this routine contain or have provisions for keeping track of file 
c  position, via icount. If changes are made to how the files are handled,
c  icount should be added so as a count of the files sucessfully read in 
c  is maintained. This allows strumo(r) to count forward/backwards after 
c  a search, with user/abuser friendly results
c
c  Calls from:
c  Mainlb:    Reset*(a,b,c)
c  Iolib:     Fdbonf,rdbonf
c  Icsdlb:    Crystn
c  Scfslb:    Rscfs, find
c  This file: Struskip
c  Mathlb:    Upcase, num
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug,nread,nwrite
      common /counter/ icount
      common /href/ hinfo(2,2),gfilen(5)
c
c NOTE!!!!  icount is the counter of the structures read in. On return from
c a subroutine , it should be the number of recorded sucessfully read in,
c eg... if there is an eof on the 14th structure, icount should be 13.
c 
      a=0
      b=0
      c=0
      alpha=0
      beta=0
      agamma=0
      iflag=0
      jauth=0
      kcomm=0
      gblank=' '
      call upcase(hent)
      gent=hent(1:8)
      if (ifltyp.eq.2) then
c   ------------------------
c
c  explaination of icount update in scf search
c  in find, icount is only updated for files sucessfully read in
c  find brings the file up to the start of the desired entry
c  The rdscf is called (below). If there is no eof, then icount is
c  incremented... eg. icount specifies the number of entries
c  sucessfully read in

        ntem=nout
        nout=0
        call find(gent,ierr)
        nout=ntem
        if (ierr.eq.-1) then
          write(nout,1045) hent
 1045     format(' Entry ',A10,' not found. The first entry of file',
     +    ' will be displayed')
          rewind nread
          nent=1
          icount=0
          call reseta
          call resetb
          call resetc
          call struskip(ifltyp,nent,hfname)
          return
          endif
        if (ierr.eq.1) write(nout,1040)
 1040   format(' Error detected while reading file')
        nentry=0
        ntem=nout
        nout=nwrite
        call rscfs(gblank,nentry)
        nout=ntem
        if (nentry.eq.-1) then
          write(nout,1055)hent
 1055     format(' EOF reached while reading in entry ',A18)
          rewind nread
          nent=icount
          icount=0
          call reseta
          call resetb
          call resetc
          call struskip(ifltyp,nent,hfname)
        else
          icount=icount+1
          endif
        endif
c
c not to be implemented...
c
      if (ifltyp.eq.3) then
c   ------------------------
        write(nout,1020)
 1020   format(' This search routine will not work for shelx files')
        endif
c ....
      if (ifltyp.eq.4) then
c   --------------------------
        ntem=nout
        nout=0
        call fdbonf(gent,ierr)
        nout=ntem
        if (ierr.ne.-1) then
          ntem=nout
          nout=nwrite
          call rdbonf(ierr)
          nout=ntem
          endif
        if (ierr.eq.-1) then
          write(nout,1045)hent
          rewind nread
          nent=1
          icount=0
          call reseta
          call resetb
          call resetc          
          call struskip(ifltyp,nent,hfname)
          endif
        endif
c  crystn files   ....tested......
c
      if (ifltyp.eq.5) then
c   --------------------------
c  search for entry indentified by nent 
        nent=num(gent,8)
        rewind nread
  500   read(nread,1500,end=590)lent,itype,irec,nind,hline
 1500   format(1X,I6,I2,I3,I3,A80)
        if (lent.ne.lentemp) then
          increment=increment+1
          lentemp=lent
          endif
        if (lent.eq.nent) goto 520
        goto 500
c
c  file found
c  ----------     
 520    ndummy=0
        backspace(nread)
        ntem=nout
        nout=nwrite
        call crystn(ndummy,jent)
        nout=ntem
        if(jent.eq.-1) then
          nentry=increment-1
          write(nout,1550)nent
 1550     format(' EOF reached while reading in entry ',I8,
     +    /,' File being set to last entry.')
          rewind nread
          icount=0
          call struskip(ifltyp,nentry,hfname)
          endif
        icount=icount+increment
        return
c  end of file
c  -----------
  590   write(nout,1555)nent
 1555   format(' Entry ',I8,' not found. File being reset to ',
     +  'first entry.')
        rewind nread
        icount=0
        nentry=1
        call struskip(ifltyp,nentry,hfname)
  599   return
        endif
c
c  ICSD (CD-ROM) files
c
      if(ifltyp.eq.6) then
c   ____________________________
c  search for entry defined by gent (converted to a number nent first)
        nent=num(gent,8)
 596    read (nread,1556,end=598) irec,jrec,hrec
1556    format (I4,I1,3X,A72)
        if (irec.ne.0) go to 596
        grec=hrec(1:6)
        jent=num(grec,6)
        if (nent.ne.0.or.nent.ne.jent) go to 596
        gfilen(3)=grec
 598    write (nout,1557) gfilen(nread)        
1557    format ('End of file on',A20)
        endif
      return
      end

c
c
***********************************************************************
c
      subroutine SKPBNF(nent,ierr)
c     ============================
c
c                                        written 25-Jul-1989
c                                        S.Urquhart
c   Purpose: to make the strumor and strumo file handling proceedures 
c   consistent and usable. This serves as an inbetween to manipulate 
c   subroutine rdbonf --- translating from rdbonf's ambiguities into a style
c   consitent for strumo and strumor.
c
      implicit character*8 (g)
      implicit character*80 (h)
      common /files/ nin,nout,ndebug,nread
c     
c
c  Variable trans-dictionary
c  -------------------------
c  nent:       From strumo(r): number of entries to read before 
c              returning...eg. if one then read next entry....
c              On return: nent equals the number of entries read in
c              sucessfully...eg. before and end of file, so that icount may
c              accurately be incremented
c  ierr        Returning to strumo(r) +1 if eof
c                                      0 all ok
c                                     -1 if file read error or ambiguity
c              Returning from rdbonf   1 if file read error
c                                     -1 if eof
c                                      0 all ok
      itempnent=0
   10 ierr =0
      ntem=nout
      nout=0
      call rdbonf(ierr)
      nout=ntem
      if (ierr.eq.-1) then
        ierr=1
        nent=itempnent
        return
        endif
      itempnent=itempnent+1
      if (itempnent.eq.nent) then
        if (ierr.eq.1) ierr=-1
        return
        endif
      goto 10
      end
c
***********************************************************************
c
      subroutine SKPICSD(nent,ierr)
c     =============================
c
c                                        written 25-Jul-1989
c                                        S.Urquhart
c   Purpose: to make the strumor and strumo file handling proceedures 
c   consistent and usable. This serves as an inbetween to manipulate 
c   subroutine rdicsd --- translating from rdicsd's ambiguities into a style
c   consitent for strumo and strumor.
c
      implicit character*8 (g)
      implicit character*80 (h)
      common /files/ nin,nout,ndebug,nread
c     
c
c  Variable trans-dictionary
c  -------------------------
c  nent:       From strumo(r): number of entries to read before 
c              returning...eg. if one then read next entry....
c              On return: nent equals the number of entries read in
c              sucessfully...eg. before and end of file, so that icount may
c              accurately be incremented
c  ierr        Returning to strumo(r) +1 if eof
c                                      0 all ok
c                                     -1 if file read error or ambiguity
c              Returning from rdicsd  lt -1 if file read error
c                                     -1    if eof
c                                     gt 0  all ok
      itempnent=0
   10 ierr =0
      nentry=0
      ntem=nout
      nout=0
      call rdicsd(nentry,ierr)
      nout=ntem
      if (ierr.eq.-1) then
        ierr=1
        nent=itempnent
        return
        endif
      itempnent=itempnent+1
      if (itempnent.eq.nent) then
        if (ierr.lt.-1) ierr=-1
        return
        endif
      goto 10
      end
c
c
***********************************************************************
c
c
      subroutine SKPCRN(nent,ierr)
c     ============================
c
c                                        written 20-Jul-1989
c                                        S.Urquhart
c   Purpose: to make the strumor and strumo file handling proceedures 
c   consistent and usable. This serves as an inbetween to manipulate 
c   subroutine crystn --- translating from crysyn's ambiguities into a style
c   consitent for strumo and strumor.
c
      implicit character*8 (g)
      implicit character*80 (h)
      common /files/ nin,nout,ndebug,nread
c
c  Variable trans-dictionary
c  -------------------------
c  nent:       From strumo(r): number of entries to read before 
c              returning...eg. if one then read next entry....
c              On return: nent equals the number of entries read in
c              sucessfully...eg. before and end of file, so that icount may
c              accurately be incremented
c  ierr        Returing to strumo(r) +1 if eof
c                                     0 all ok
c  jent        Number of file read in crysyn... =-1 if eof
c  nentry      Number of file being looked for..or 0 if next entry
c
c
      itempnent=0
c
c  call the filereading program
c  ----------------------------
  10  ierr=0
      nentry=0
      ntem=nout
      nout=0
      call crystn(nentry,jent)
      nout=ntem
      if (jent.eq.-1) then
        ierr=1
        nent=itempnent
        return
      else
        itempnent=itempnent+1
        endif
      if (nent.eq.itempnent) return
      goto 10
c
c  WARNING...IF CRYSTN FILE IS CUT OFF..EG IF END OF FILE COMES 1/2 WAY THRO
C  A STRUCTURE...THEN RESULTS FOR THE STRUCTURE DISPLAYED AFTER EOF WARNING
C  WILL BE NONSENSE
c
      end
c
c
***********************************************************************
c
      subroutine RIDENT(mp)
c     =====================
c
c     read in elements of common /ident/
c
c     calls inerr, chemno, readyn, rxatom
c
c     written by D. Altermatt
c     McMaster University, June 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      include 'combond.for'
c
      i1=1
      i2=mx
      if (mp.ne.0) goto 67
      write (nout,2130)
2130  format (//,1X,'Enter all atoms of compound ',//,
     1  ' (If you know the structure: Enter atoms by ',
     +  'crystallographic positions',/,
     2  ' Else: Enter each element of compound once, give its',/,
     3  ' stoichiometric multiplicity as Site Occupation ',/,
     4  ' and set the multiplicity to -1)',/)
50    nnx=nx
      if (i1.gt.nx) nnx=0
      do 60 i=i1,i2
55      write (nout,2140) i
2140    format (/,1X,'Element nr.',I4,//,' Element name (2 charac',
     1         'ters, blanks for end-of-list) : ')
        read (nin,1060,err=55) gelem(i)
1060    format (A2)
        if (gelem(i).eq.'  ') goto 65
        if(ichar(gelem(i)(1:1)).ge.ichar('a').and.ichar(gelem(i)(1:1))
     1      .le.ichar('z')) gelem(i)(1:1)=char(ichar(gelem(i)(1:1))-
     2      ichar('a')+ichar('A'))
        if(ichar(gelem(i)(2:2)).ge.ichar('a').and.ichar(gelem(i)(2:2))
     1      .le.ichar('z')) gelem(i)(2:2)=char(ichar(gelem(i)(2:2))-
     2      ichar('a')+ichar('A'))
        nx=i
        write (nout,2150)
2150    format (' Element identification (6 characters) : ')
        read (nin,1070,err=40) gdent(i)
1070    format (A6)
        write (nout,2160)
2160    format (' Oxidation state (real) : ')
        read (nin,1080,err=40) oxid(i)
1080    format (F10.0)
        write (nout,2170)
2170    format (' Coordination number (integer, 0 if unknown) : ')
        read (nin,1090,err=40) icn(i)
1090    format (I5)
        npos(i)=-1
        ialtno(i)=0
        do 110 j=1,3
          do 115 k=1,3
            u(k,j,i)=0.0
115         du(k,j,i)=0.0
110       xx(j,i)=2.0
        if(mp.eq.0) occu(i)=1.0
        call chemno(gelem(i),nza(1,i),nza(2,i))
        goto 35
40      call inerr(*55)
35      write (nout,2210)
2210    format (/,' Section ok (Y/N/E) : ')
        call readyn(*60,*55,*999,*35)
60      continue
      if (i1.eq.i2) then
        nx=nnx
30      write (nout,2270)
2270    format (//,' Listing of stored elements required (Y/N/E): ')
        call readyn(*67,*100,*999,*30)
        endif
      write (nout,2250) mx
65    if(i1.eq.i2) call inerr(*50)
      if (nnx.ne.0) nx=nnx
67    do 90 i=1,nx,20
        nnx=nx-i+1
        if (nnx.gt.20) nnx=20
        write (nout,2230)
2230    format (/,1X,'Element nr.,  El. name, Period nr, Group nr.,',
     1         '  Identif. , Ox.state , Coord. nr.',/)
        do 85 j=1,nnx
          jj=j+i-1
          write (nout,2220) jj,gelem(jj),nza(1,jj),nza(2,jj),gdent(jj),
     1        oxid(jj),icn(jj)
2220      format (4X,I5,9X,A2,2(6X,I5),6X,'''',A6,'''',3X,F6.3,7X,I5)
85        continue
        if(j.eq.21)call wait
90      continue
100   write (nout,2180)
2180  format (//,' Element list ok (Y/N/E): ')
      call readyn(*80,*105,*999,*100)
105   write (nout,2260)
2260  format (/,' Add more elements to list (Y/N/E): ')
      call readyn(*107,*72,*999,*105)
107   i1=nx+1
      if (i1.gt.mx) then
        write (nout,2250) mx
2250    format (/,1X,'***** Only',I4,' atoms allowed in list',/)
        goto 100
        endif
      i2=mx
      write (nout,2130)
      goto 50
72    write (nout,2190)
2190  format (/,' Number in list of element to change : ')
      read (nin,1090,err=78) i1
      if (i1.le.0.or.((i1.gt.nx).and.(nx.gt.0))) call inerr(*72)
95    write (nout,2200)
2200  format (' Delete or replace that entry (D/R) : ')
      read (nin,1000,err=95) gdel
1000  format(A1)
      if (gdel.eq.'D'.or.gdel.eq.'d') goto 70
      if (gdel.ne.'r'.and.gdel.ne.'R') call inerr(*95)
      i2=i1
      goto 50
70    nx=nx-1
      do 75 i=i1,nx
        do 120 j=1,3
          do 125 k=1,3
            u(j,k,i)=u(j,k,i+1)
125         du(j,k,i)=du(j,k,i+1)
          xx(j,i)=xx(j,i+1)
120       dxx(j,i)=dxx(j,i+1)
        occu(i)=occu(i+1)
        soccu(i)=soccu(i+1)
        gelem(i)=gelem(i+1)
        gdent(i)=gdent(i+1)
        oxid(i)=oxid(i+1)
        icn(i)=icn(i+1)
75      npos(i)=npos(i+1)
      if (nbon.eq.0) goto 67
      do 130 i=1,nbon
135     if (i.gt.nbon) goto 130
        if (ibon(1,i).eq.i1) goto 150
        if (jbon(1,i).eq.i1) goto 150
      goto 130
150   nbon=nbon-1
      if (nbon.eq.0) goto 67
      do 160 k=i,nbon
        do 170 l=1,6
          xbon(l,k)=xbon(l,k+1)
          if (l.gt.5) goto 170
          ibon(l,k)=ibon(l,k+1)
          jbon(l,k)=jbon(l,k+1)
          if (l.gt.4) goto 170
          ifbon(l,k)=ifbon(l,k+1)
170       continue
160     continue
      goto 135
130   continue
      do 180 i=1,nbon
        if (ibon(1,i).gt.i1) ibon(1,i)=ibon(1,i)-1
180     if (jbon(1,i).gt.i1) jbon(1,i)=jbon(1,i)-1
      goto 67
78    call inerr(*72)
80    continue
      if(mp.ne.0) then
81      write (nout,2001)
2001    format(/,2X,'Do you want to change the co-ordinates ? (Y/N/E) ')
        call readyn(*82,*999,*999,*81)
        endif
82    call rxatom(mp)
999   return
      end
c
c
***********************************************************************
c
c
      subroutine RXATOM(mp)
c     =====================
c
c     read in coordinates to elements
c
c     calls inerr, rbonds, readyn
c
c     written by D. Altermatt
c     McMaster University, June 1984
c
c     modifications by S. Healy (as the request of Dr. Brown)
c     McMaster U, November 1990
c
      implicit character*8 (g)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      include 'combond.for'
c
      write (nout,2000)
2000  format (//,1X,'Atomic position update')
      i1=1
      i2=nx
      jmp=0
5     do 10 i=i1,i2
        if (jmp.ne.0) goto 15
        sxx=xx(1,i)+xx(2,i)+xx(3,i)
        if (sxx.lt.5.99) goto 10
15      write (nout,2010) i,gelem(i),gdent(i),oxid(i)
2010    format (/,1X,'Atom nr.:',I4,4X,A2,4X,A8,4X,
     1    ', Ox.state: ',F6.2,//,1X,'Enter coordinates x,y,z, errors',
     2    ' dx,dy,dz :',/)
         if(machine.eq.3.or.machine.eq.5) then
          read (nin,*,err=20) (xx(j,i),j=1,3),(dxx(j,i),j=1,3)
        else
          read (nin,1000,err=20) (xx(j,i),j=1,3),(dxx(j,i),j=1,3)
          endif
1000    format(6F10.0)
17      write (nout,2020)
2020    format (' Enter Wyckoff symbol (1 character or blank): ')
        read (nin,1010,err=17) gwyck(i)
1010    format (A1)
        call upcase(gwyck(i))
        write (nout,2025)
2025    format (' Enter multiplicity of site (0 if unknown,',
     1    ' -1 for dummy position): ')
        read (nin,1030,err=20) npos(i)
1030    format(I5)
        write (nout,2030)
2030    format (' Enter occupation of site, error in oc. ',
     1    ' (default=1): ')
        if(machine.eq.3.or.machine.eq.5) then
          read (nin,*,err=20) occ,socc
        else
          read (nin,1000,err=20) occ,socc
          endif
        if (abs(occ).gt.0.0001) then
           occu(i)=occ
           soccu(i)=socc
        else
           occu(i)=1
           soccu(i)=0
        endif
25      write (nout,2040)
2040    format (/,' Section ok (Y/N/E): ')
        call readyn(*10,*15,*999,*25)
20      call inerr(*15)
10      continue
      if (jmp.ne.0) goto 50
35    linc=0
      write (nout,2050)
2050  format (//,1X,'Atom nr., Type, Ident. , Ox.state,       x,',
     1     '      y,      z, Occup., Wyck., Mul.',/,36X,' err. x,',
     2     ' err. y, err. z, err.occ.',/)
      do 30 i=1,nx
        jj=i
        linc=linc+1
        write (nout,2060) jj,gelem(jj),gdent(jj),oxid(jj),(xx(k,jj),
     1     k=1,3),occu(jj),gwyck(jj),npos(jj)
2060    format (4X,I3,5X,A2,2X,A8,1X,F7.4,3X,4(1X,F7.4),4X,A1,3X,I3)
        write (nout,2070) (dxx(k,jj),k=1,3),soccu(jj)
2070    format (35X,4(1X,F7.4))
        if(linc.eq.10.and.i.ne.nx) then
          call wait
          linc=0
          write(nout,2050)
          endif
30      continue
      if (jmp.ne.0) goto 70
45    write (nout,2090)
2090  format (//,' Coordinate list ok (Y/N/E): ')
      call readyn(*70,*50,*999,*45)
50    write (nout,2100)
2100  format (/,' Atom nr. in list to change (0 for no more',
     1     ' changes): ')
      read (nin,1020,err=60) i1
1020  format (I5)
      if (i1.gt.nx) call inerr(*50)
      if (i1.eq.0) goto 35
      if (nbon.eq.0) goto 55
      do 80 j=1,nbon
        if (ibon(1,j).eq.i1) ibon(1,j)=1000*i1
80      if (jbon(1,j).eq.i1) jbon(1,j)=1000*i1
55    i2=i1
      jmp=1
      goto 5
60    call inerr(*50)
70    if(mp.ne.0) then
72      write(nout,2001)
2001    format(/,2X,'Do you want to correct bonds ? (Y/N/E) ')
        call readyn(*75,*999,*999,*72)
        endif
75    mp=1
      call rbonds(mp)
999   return
      end
c
c
***********************************************************************
c
c
      subroutine RUPDAT(mp,*,*,*)
c     ===========================
c
c     update common blocks as selected from a menu
c
c     calls inerr, length, param, raucom, rbonds, readyn, rident, roxide,
c           rtemp, runpro, rxatom, sghall, spacrd, upcase, wait
c
c     written by D. Altermatt
c     McMaster University, June 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /ecell/  da,db,dc,dalpha,dbeta,dgamma,dm(9,6)
      common /dens/   zform,dens,ddens
      common /gtitle/ gent,hname,hcodn(2),htitle(2),hauth(10),
     1                hform(2),hcomm(7),grfact,grad
      common /title/  jauth,kcomm
      common /gmenu/  hmenu(20,3)
c programmer's note:  /bost/ is changed to /bostt/ for
c                     UNIX system
      common /bostt/   nbost,mbost,ioxbost(2,30),ifbost(30),pbost(5,30)
      common /gbost/  gbost(2,30)
      include 'comatom.for'
      include 'combond.for'
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
      common /asymu/  iasym,aslim(3,2),isyel(3,2)
c
c     dimension nt(3),zz(3),uu(3,3),ts(3,3)
c
      data hmenu /' 1 :  Update entry identifier and title',
     1     ' 2 :  Update authors and comments (remarks)',
     2     ' 3 :  Update cell constants',
     3     ' 4 :  Update space group symbol(s)',
     4     ' 5 :  Update formula(s)',
     5     ' 6 :  Update density and "Z"',
     6     ' 7 :  Update R-factor and radiation',
     7     ' 8 :  Update atomlist',
     8     ' 9 :  Update coordinates',
     9     '10 :  Update temperature factors',
     1     '11 :  Update bonds and contacts',
     2     '12 :  Update oxidation states',
     3     '13 :  Update size and space group of cell',
     4     '14 :  Update bond valence parameters',
     4            2*'   ',
     5     '17 :  Add data from an additional input file',
     6     '18 :  Drop all information and read a new entry',
     7     '19 :  Return to MENU LEVEL 1',
     2     '20 :  Check stored data and call MENU LEVEL 3',
     3     40*'     '/
c
10    if (mp.gt.0.and.nbon.ne.0) call rbonds(mp)
15    write (nout,2000) (hmenu(i,1),i=1,20)
2000  format (/,1X,'     MENU LEVEL 2',8X,
     1     'Choose one of the following functions:',//,
     2     20(5X,A70,/),/,' Enter function number: ')
      read (nin,1000,err=20) ifkt
1000  format (I5)
      goto 30
20    call inerr(*15)
30    if (ifkt.eq.20) goto 200
      if (ifkt.eq.17) return 1
      if (ifkt.eq.18) return 2
      if (ifkt.eq.19) return 3
      if (ifkt.gt.14) call inerr(*15)
      if (ifkt.ne.1) goto 40
c
c     ifkt = 1
c     --------
33    write (nout,2010) gent,htitle
2010  format (/,1X,'Entry identifier stored: ',A8,//,
     1     1X,'Title line 1 as stored:',/,1X,A79,/,
     2     1X,'Title line 2 as stored:',/,1X,A79,//,
     3     ' Enter new identifier (8 characters, ''old'' for no'
     4     ,' change): ')
      read (nin,1010,err=33) gen
1010  format (A8)
35    write (nout,2020) '1'
2020  format (/,1X,'Enter new title line ',A1,' (80 characters,'
     1  ,' ''old'' for no change):',/)
      read (nin,1020,err=35) htit1
1020  format (A80)
37    write (nout,2020) '2'
      read (nin,1020,err=37) htit2
      if (gen.ne.'old'.and.gen.ne.'OLD') gent=gen
      if (htit1.ne.'old'.and.htit1.ne.'OLD') htitle(1)=htit1
      if (htit2.ne.'old'.and.htit2.ne.'OLD') htitle(2)=htit2
      goto 15
40    if (ifkt.ne.2) goto 50
c
c     ifkt = 2
c     --------
      jmax=10
      gauth='  author'
      call raucom(jauth,hauth,jmax,gauth)
      jmax=7
      gcom=' comment'
      call raucom(kcomm,hcomm,jmax,gcom)
      goto 15
50    if(ifkt.ne.3) goto 70
c
c     ifkt = 3
c     --------
      pi=3.141593/180.
      alp=alpha/pi
      bet=beta/pi
      aga=agamma/pi
      dal=dalpha/pi
      dbe=dbeta/pi
      dga=dgamma/pi
      write (nout,2030) a,b,c,alp,bet,aga,da,db,dc,dal,dbe,dga
2030  format (//,1X,'Cell constants and their errors as stored:',//,
     1     1X,6F10.4,/,1X,6F10.4)
54    write(nout,2035)
2035  format(/,2X,'Do you want to update cell constants ? (Y/N/E) ')
      call readyn(*55,*15,*15,*54)
55    write (nout,2040)
2040  format (/,1X,'Enter new cell constants a,b,c (in Angstroms),',
     1     ' alpha,beta,gamma (in degrees):',/)
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=55) a,b,c,alpha,beta,agamma
      else
        read (nin,1030,err=55) a,b,c,alpha,beta,agamma
        endif
1030  format (6F10.0)
      alpha=alpha*pi
      beta=beta*pi
      agamma=agamma*pi
60    write (nout,2050)
2050  format (/,1X,'Enter errors da,db,dc,dalpha,dbeta,dgamma:',/)
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=60) da,db,dc,dalpha,dbeta,dgamma
      else
        read (nin,1030,err=60) da,db,dc,dalpha,dbeta,dgamma
        endif
      dalpha=dalpha*pi
      dbeta=dbeta*pi
      dgamma=dgamma*pi
      icell=-1
65    write (nout,2060)
2060  format (/,' Section correct (Y/N/E): ')
      call readyn(*67,*55,*69,*65)
67    summ=alpha+beta+agamma+a+b+c
      if (summ.lt.0.1) call inerr(*55)
      call param
      goto 15
69    a=10.
      b=10.
      c=10.
      alpha=90.*pi
      beta=90.*pi
      agamma=90.*pi
      call param
      write(nout,2065) a,b,c,alpha,beta,agamma
2065    format(//,' Lattice constants set to:',//,' a =',F5.1,', b =',
     1    F5.1,', c =',F5.1,', alpha =',F6.3,', beta =',F6.3,', gamma ='
     2    ,F6.3,///)
      goto 15
70    if (ifkt.ne.4) goto 80
c
c     ifkt = 4
c     --------
75    write (nout,2070) hmsym,hall
2070  format (//,1X,'Space group symbols stored are:',//,1X,
     1     'Hermann-Maugin: ',A24,/,1X,'Hall',10X,': ',A24,//,1X,
     2     'Enter new symbols (24 CAPITAL characters or "OLD":',
     3     //,' Hermann-Maugin Symbol: ')
      read (nin,1020,err=75) hsym1
      call upcase(hsym1)
77    write (nout,2080)
2080  format (' Hall Symbol          : ')
      read (nin,1020,err=77) hsym2
      call upcase(hsym2)
      if (hsym1.ne.'OLD') hmsym=hsym1
      if (hsym2.ne.'OLD') hall=hsym2
      if (hsym1.ne.'OLD'.or.hsym2.ne.'OLD') then
         nsp=0
         icell=-1
         asym=0
         iasym=0
         glat=' '
         gcent=' '
         if(hall.eq.' ') then
            if(hmsym.eq.' ') then
               call inerr(*75)
            else
               if(machine.eq.1) then
                  hfile='SPGRF.DAT'
                  gstat='OLD'
                  gfort=' '
               else if(machine.eq.2) then
                  hfile='SPGR:DAT'
                  gstat='OLD'
                  gfort=' '
               else if(machine.eq.3) then
                  hfile='SPGRF'
                  gstat='OLD'
                  gfort=' '
               else if(machine.eq.4) then
                  hfile='spgrf.dat'
                  gstat='OLD'
                  gfort=' '
               else if(machine.eq.5) then
                  hfile='spgrf.dat'
                  gstat='OLD'
                  gfort=' '
                  endif
               nunit=nspgp
               nn=ifile(hfile,gfort,gstat,nunit)
               if(nn.le.0) then
                  write(nout,2085)
2085              format(/,' ***** We need an explicit Hall Symbol',
     1              ' *****',/)
                  goto 77
                  endif
               nspgp=nn
               call spacrd
               call sghall
               endif
         else
            call sghall
            endif
         endif
      goto 15
80    if (ifkt.ne.5) goto 90
c
c     ifkt = 5
c     --------
85    write (nout,2090) hform
2090  format (//,1X,'Formula(s) as stored:',//,2(1X,A79,/))
86    write (nout,2092)
2092  format (//,5X,' 1: Update first formula'/,
     +5X,' 2: Update second formula'/,
     +5X,'20: Exit'//,
     +5X,' Enter your selection : ')
      read (nin,2094,err=86) ifform
2094  format(I5)
      if(ifform.eq.20) goto 15
      if(ifform.ne.1.and.ifform.ne.2) goto 86
87    write (nout,2096)
2096  format(/,1X,'Enter new formula (80 characters):',/)
      if(ifform.eq.1) goto 88
      if(ifform.eq.2) goto 89
88    read (nin,1020,err=85) hform1
      goto 85
89    read (nin,1020,err=85) hform2
      goto 85
90    if (ifkt.ne.6) goto 100
c
c     ifkt = 6
c     --------
      write (nout,2100) zform,dens,ddens
2100  format (//,1X,'Formula units per unit cell: ',F5.2,/,
     1             1X,'Density               : ',F5.2,/,
     2             1X,'Error in density      : ',F5.2,//)
92    write (nout,2102)
2102  format(//,5X,' 1: Update formula units per unit cell'/,
     +5X,' 2: Update density'/,
     +5X,'20: Exit'//,
     +5X,' Enter your selection : ')
      read (nin,2094,err=92) ifdens
      if(ifdens.eq.20) goto 15
      if(ifdens.eq.1) goto 93
      if(ifdens.eq.2) goto 95
      goto 92
93    write (nout,2104)
2104  format(//,2X,'Enter number of formula units per unit cell : ')
      read (nin,1050,err=93) zform
1050  format (F10.0)
      goto 90
95    write (nout,2110)
2110  format (/,1X,'Enter density, error in density (0''s for un',
     1     'known): ',/)
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=95) dens,ddens
      else
        read (nin,1060,err=95) dens,ddens
        endif
1060  format (2F10.0)
      goto 90
100   if (ifkt.ne.7) goto 110
c
c     ifkt = 7
c     --------
105   write (nout,2120) grad,grfact
2120  format (//,1X,'Radiation : ',A8,/,
     1             1X,'R-factor  : ',A8,//)
106   write (nout,2122)
2122  format (/,5X,' 1: Update radiation'/,
     +5X,' 2: Update R-factor'/,
     +5X,'20: Exit'//,
     +5X,' Enter your selection : ')
107   read (nin,2094,err=106) ifrad
      if(ifrad.eq.20) goto 15
      if(ifrad.eq.1) goto 108
      if(ifrad.eq.2) goto 109
      goto 106
108   write(nout,2124)
2124  format(//,2X,'Enter radiation (no default) : ')
      read (nin,1010,err=108) grad
      goto 105
109   write (nout,2130)
2130  format (/,' Enter R-factor (no default): ')
      read (nin,1010,err=109) grfact
      goto 105
110   if (ifkt.ne.8) goto 120
c
c     ifkt = 8 to 14
c     --------------
      mp=1
      call rident(mp)
      goto 15
120   if (ifkt.ne.9) goto 130
      mp=1
      call rxatom(mp)
      goto 15
130   if (ifkt.ne.10) goto 140
      call rtemp
      goto 15
140   if (ifkt.ne.11) goto 150
      call rbonds(mp)
      goto 15
150   if (ifkt.ne.12) goto 160
      mmp=1
      call roxide(mmp)
      goto 15
  160 if(ifkt.ne.13) go to 170
  162 write(nout,2162)
 2162 format(' Do you wish to lower the symmetry (Y/N/E)  ')
      call readyn(*163,*165,*15,*162)
  163 call losym
      go to 15
  165 write(nout,2165)
 2165 format(' Sorry, but it is not yet possible to raise the ',
     +  'symmetry')
      call wait
      go to 15
c
c     ifkt = 14
c     ---------
  170 if(ifkt.ne.14) go to 180
      write(nout,2170)
 2170 format(' Do you want a list of stored valence parameters ',
     +     '(Y/N/E) ')
  171 call readyn(*172,*176,*15,*170)
  172 write(nout,2172) (i,(gbost(j,i),ioxbost(j,i),j=1,2),
     +             ifbost(i),(pbost(j,i),j=1,5),i=1,nbost)
 2172 format(' Valence parameters loaded',/,
     +     30(1X,I2,1X,2(A2,'(',I2,')'),2X,I1,2X,5F6.3,/))
  176 write(nout,2174)
 2174 format(' Do you wish to change these values (Y/N/E)')
      call readyn(*175,*15,*15,*176)
  175 write(nout,2175)
 2175 format(' Enter the number of the parameters to be changed ',
     +    '(zero to exit)')
      read(nin,2176,err=170) iv
      if(iv.eq.0) then
        write(nwrite,2172) (i,(gbost(j,i),ioxbost(j,i),j=1,2),
     +             ifbost(i),(pbost(j,i),j=1,5),i=1,nbost)
        write(nout,2172) (i,(gbost(j,i),ioxbost(j,i),j=1,2),
     +             ifbost(i),(pbost(j,i),j=1,5),i=1,nbost)
        call wait
        go to 15
        endif
 2176 format(I10)
      write(nout,2177) 
 2177 format(' Enter the flag and up to 5 parameters ',
     + '(e.g. 2,2.285,0.37)')
      if(machine.eq.3.or.machine.eq.5) then
        read(nin,*,err=170) ifbost(iv),(pbost(i,iv),i=1,5)
      else
        read(nin,2178,err=170) ifbost(iv),(pbost(i,iv),i=1,5)
 2178   format(I2,5F10.5)
        endif
      go to 175
  180 go to 15
c
c     ifkt = 20: Check data and call next menu
c     ----------------------------------------
200   call runpro(*15,*210)
210   return
      end
c
c
***********************************************************************
c
c
      subroutine RTEMP
c     ================
c
c     reads/updates common temp
c
c     calls wtemp, inerr, readyn
c
c     written by D. Altermatt
c     McMaster University, June 1984
c
      implicit character*8 (g)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
c
      diff = 1.E-7
c
      if (nx.eq.0) then
         write (nout,2010)
2010     format (//,1X,'***** no atoms stored yet *****',/)
         return
         endif
5     write (nout,2000)
2000  format (/,1X,'Temperature factor update',//,
     1     ' Isotropic temperature factors only (Y/N/E): ')
      isofl=0
      call readyn(*7,*9,*999,*5)
7     isofl=1
9     do 10 i=1,nx
        if (isofl.eq.1) then
          if (u(1,1,i).gt.diff) goto 10
        else
          sum=0.0
          do 20 j=1,3
20          sum=sum+abs(u(j,j,i))
          if (sum.gt.diff) goto 10
          endif
        call wtemp(i,isofl)
10      continue
25    do 30 i=1,nx,10
        nnx=nx-i+1
        if (nnx.gt.10) nnx=10
        write (nout,2020)
2020    format (//,1X,'Nr. Element Identifier  u(1,1)   u(2,2)   ',
     1      'u(3,3)   u(1,2)   u(1,3)   u(2,3)',/,25X,'du(1,1)  ',
     2      'du(2,2)  du(3,3)  du(1,2)  du(1,3)  du(2,3)',/)
        do 40 j=1,nnx
          k=j+i-1
          write (nout,2030) k,gelem(k),gdent(k),(u(l,l,k),l=1,3),
     1        u(1,2,k),u(1,3,k),u(2,3,k)
2030      format (1X,I3,3X,A2,5X,A8,1X,6(1X,F8.5))
          write (nout,2040) (du(l,l,k),l=1,3),du(1,2,k),du(1,3,k),
     1        du(2,3,k)
2040      format (23X,6(1X,F8.5))
40        continue
        if(j.eq.11) write (nout,2050)
2050    format (/,' Hit <RETURN> to continue listing: ')
        if(j.eq.11) read (nin,1000,err=30) gdum
1000    format(A1)
30      continue
35    write (nout,2060)
2060  format (//,' Temperature factor list ok (Y/N/E): ')
      call readyn(*999,*45,*999,*35)
45    write (nout,2070)
2070  format (/,' Enter number in list of element to change,'/,
     +17X,' (0 to exit) : ')
      read (nin,1010,err=45) i1
1010  format (I4)
      if(i1.eq.0) goto 50
      if (i1.lt.1.or.i1.gt.nx) call inerr(*45)
      call wtemp(i1,isofl)
50    write (nout,2080)
2080  format (/,' Listing of elements required (Y/N/E): ')
      call readyn(*25,*35,*999,*50)
999   return
      end
c
c
***********************************************************************
c
c
      subroutine WTEMP(i,isofl)
c     =========================
c
c     calls inerr
c
c     update elements of common /temp/
c
c     written by D. Altermatt
c     McMaster University, July 1984
c
      implicit character*8 (g)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
c
      write (nout,2000) i,gelem(i),gdent(i)
2000  format (/,1X,'Update element nr.',I4,', ',A2,2X,A8,//,1X,
     1     'Temperature factors and errors as stored:',/,3X,'u(1,1)'
     2     ,'    u(2,2)    u(3,3)    u(1,2)    u(1,3)    u(2,3)',/)
      write (nout,2010) (u(j,j,i),j=1,3),u(1,2,i),u(1,3,i),u(2,3,i),
     1     (du(j,j,i),j=1,3),du(1,2,i),du(1,3,i),du(2,3,i)
2010  format (2(1X,6(F8.5,2X),/))
      if (isofl.ne.0) goto 30
10    write (nout,2020)
2020  format (/,1X,'Enter new u(1,1),u(2,2),u(3,3),u(1,2),u(1,3),'
     1     ,'u(2,3):',/)
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=10) (u(j,j,i),j=1,3),u(1,2,i),u(1,3,i),
     1     u(2,3,i)
      else
        read (nin,1000,err=10) (u(j,j,i),j=1,3),u(1,2,i),u(1,3,i),
     1     u(2,3,i)
        endif
1000  format (6F10.0)
15    write (nout,2030)
2030  format (/,1X,'Enter errors in same order:',/)
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=15) (du(j,j,i),j=1,3),du(1,2,i),du(1,3,i),
     1     du(2,3,i)
      else
        read (nin,1000,err=15) (du(j,j,i),j=1,3),du(1,2,i),du(1,3,i),
     1     du(2,3,i)
        endif
25    write (nout,2040)
2040  format (/,' Section ok (Y/N): ')
      read (nin,1010,err=25) gte
1010  format (A1)
      if (gte.eq.'N'.or.gte.eq.'n') goto 10
      if (gte.ne.'Y'.and.gte.ne.'y') call inerr(*25)
      do 20 j=1,3
      do 20 k=1,3
      du(k,j,i)=du(j,k,i)
20    u(k,j,i)=u(j,k,i)
      return
30    do 40 j=1,3
      do 40 k=1,3
      u(j,k,i)=0.0
40    du(j,k,i)=0.0
50    write (nout,2050)
2050  format (/,1X,'Enter isotropic temperature factor and error',
     1     '  u(1,1),du(1,1):',/)
      read (nin,1020,err=50) u(1,1,i),du(1,1,i)
1020  format (2F10.0)
      return
      end
c
c
***********************************************************************
c
c
      subroutine RBONDS(mp)
c     =====================
c
c     update common /bonds/ and common/contac/
c
c     MP is the run-type flag
c        mp = 0:  user selected update run
c             1:  update run forced by user changes in atomic coordinates
c             2:  update run forced by coordinate transformations
c
c     calls inerr, readyn, wait
c
c     written by D. Altermatt
c     McMaster University, July 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      include 'combond.for'
c
c     Keep only independent bonds/contacts
c
      if(ind1.lt.0.or.ind1.gt.nbon) ind1=0
      if(ind2.lt.0.or.ind2.gt.ncon) ind2=0
      i1=1
      if(nbon.ne.0.and.ind1.lt.nbon) call indepb(i1,(ind1+1),nbon)
      i1=1
      i2=mbon
c
c     Check for run type
c
      if (mp.eq.0) goto 100
      mp=-10*mp
      if (nbon.eq.0) then
        write (nout,2001)
2001    format(/,2X,'No bonds stored, use option 5 in menu 3')
        goto 300
        endif
c
c     Find bonds that have to be updated
c
10    do 20 i=1,nbon
15    if (i.gt.nbon) goto 20
      ifl=0
      if ((ibon(1,i)/1000).gt.0) then
        ifl=1
        ibon(1,i)=ibon(1,i)/1000
        endif
      if ((jbon(1,i)/1000).gt.0) then
        ifl=1
        jbon(1,i)=jbon(1,i)/1000
        endif
      if (ifl.eq.0) goto 20
      write (nout,2000) i
2000  format (//,1X,'Previous changes in coordinates affect',
     1     ' Bond nr.',I4,':',//,1X,'from',7X,'to',9X,'Mu BT Dso ',
     2     'Distance & error  Sym.  Translations Dep.on',/)
      ii=ibon(1,i)
      jj=jbon(1,i)
      if(ind1.lt.i) inde1(i)=0
      write (nout,2010) gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1     (ifbon(k,i),k=1,3),(xbon(k,i),k=1,2),jbon(2,i),(jbon(k,i),
     2     k=3,5),inde1(1)
2010  format (1X,2(A2,1X,A6,2X),3(I2,1X),1X,2(F7.4,2X),I4,2X,3(1X,I3),
     1     2X,I4,/)
c
c     Prompt for update type
c
40    write (nout,2020)
2020  format (/,' Delete or replace (D/R): ')
      read (nin,1000,err=40) gde
1000  format (A1)
      if (ind1.ge.ii) ind1=ii-1
      if (gde.eq.'D'.or.gde.eq.'d') then
         nbon=nbon-1
         if (nbon.eq.0.or.i.gt.nbon) goto 300
         do 45 k=i,nbon
            do 50 l=1,6
               xbon(l,k)=xbon(l,k+1)
               if (l.gt.5) goto 50
               ibon(l,k)=ibon(l,k+1)
               jbon(l,k)=jbon(l,k+1)
               if (l.gt.4) goto 50
               ifbon(l,k)=ifbon(l,k+1)
50             continue
45          continue
         goto 15
      else if(gde.ne.'R'.and.gde.ne.'r') then
         call inerr(*40)
         endif
c
c     Read bond parameters
c
52    write (nout,2030) i,gelem(ii),gdent(ii),gelem(jj),gdent(jj)
2030  format (/,1X,'Bond nr.',I4,' from ',A2,2X,A8,' to ',A2,2X,
     1     A8,//,' Enter bond multiplicity (integer): ')
      read (nin,1010,err=52) ifbon(1,i)
1010  format (I2)
      if(ifbon(1,i).lt.1) ifbon(1,i)=1
53    write(nout,2035)
2035  format(' Enter bond type key (integer; 0 if unknown): ')
      read(nin,1010,err=53) ifbon(2,i)
      if(ifbon(2,i).lt.0) ifbon(2,i)=0
55    ifbon(3,i)=0
      ifbon(4,i)=0
      write(nout,2045)
2045  format(' Disordered bond (Y/N): ')
      call readyn(*57,*54,*54,*55)
57    ifbon(3,i)=1
54    write (nout,2040)
2040  format(' Enter bond length (in Angstroms): ')
      read (nin,1020,err=54) xbon(1,i)
1020  format (F10.0)
      ifbon(4,i)=0
      xbon(3,i)=0.0
      if(xbon(1,i).lt.0.1) goto 58
56    write (nout,2050)
2050  format (' Enter error in bond length: ')
      read (nin,1020,err=56) xbon(2,i)
      call bost(ibon(1,i),jbon(1,i),xbon(3,i),xerr,xbon(2,i),xbon(1,i))
      if(abs(xbon(3,i)).lt.0.01) then
70      write(nout,2055)
2055    format(/,' No valence curve found for this atom pair',//,
     1    ' Enter preset bond length (or zero): ')
        read(nin,1020,err=70) xbon(3,i)
        xbon(3,i)=abs(xbon(3,i))
        if(xbon(3,i).lt.0.01) xbon(3,i)=xbon(1,i)
        if(xbon(3,i).gt.20.0) call inerr(*70)
        xbon(3,i)=-xbon(3,i)
        ifbon(4,i)=2
        endif
58    write (nout,2060)
2060  format (' Enter symmetry operator for terminal atom',
     1     ' (0 if unknown): ')
      read (nin,1030,err=58) jbon(2,i)
1030  format (I4)
      ibon(2,i)=1
      ibon(3,i)=0
      ibon(4,i)=0
      ibon(5,i)=0
      if (jbon(2,i).eq.0) then
         jbon(3,i)=0
         jbon(4,i)=0
         jbon(5,i)=0
         goto 20
         endif
60    write (nout,2070)
2070  format (' Enter total translations t1,t2,t3 (integers): ')
      if(machine.eq.3.or.machine.eq.5) then
        read (nin,*,err=60) (jbon(k,i),k=3,5)
      else
        read (nin,1040,err=60) (jbon(k,i),k=3,5)
        endif
1040  format (3I4)
20    continue
      goto 300
c
c     Start for normal update run
c
c     First display bonds as stored so far
c
100   if (nbon.eq.0) goto 160
      do 110 i=1,nbon,20
      write (nout,2080)
2080  format (//,1X,'Nr. from',7X,'to',9X,'Mu BT Dso Distance &',
     1     ' error  Sym.  Translations Dep.on',/)
      nnb=nbon-i+1
      if (nnb.gt.20) nnb=20
      do 120 j=1,nnb
      k=j+i-1
      ii=ibon(1,k)
      jj=jbon(1,k)
      if(ind1.lt.k) inde1(k)=0
120   write (nout,2090) k,gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1     (ifbon(l,k),l=1,3),(xbon(l,k),l=1,2),jbon(2,k),(jbon(l,k),
     2     l=3,5),inde1(k)
2090  format (1X,I3,1X,2(A2,1X,A6,2X),3(I2,1X),1X,2(F7.4,2X),I3,2X,
     1     3(1X,I3),2X,I4)
      if(j.eq.21) write (nout,2100)
2100  format (/,' Hit <RETURN> to continue listing: ')
      if(j.eq.21) read (nin,1000,err=110) dgum
110   continue
c
c     Prompt for update type
c
122   write (nout,2110)
2110  format (//,' Bond list ok (Y/N/E): ')
      call readyn(*300,*145,*300,*122)
145   write (nout,2115)
2115  format (/,' Update existing bonds (Y/N): ')
      read (nin,1000,err=145) gte
      if (gte.eq.'Y'.or.gte.eq.'y') then
125      write (nout,2120)
2120     format (/,' Enter number of bond to change: ')
         read (nin,1030,err=125) i1
         if (i1.gt.nbon.or.i1.lt.1) call inerr(*125)
130      write (nout,2020)
         read (nin,1000,err=130) gte
         if (ind1.ge.i1) ind1=i1-1
         if (gte.eq.'D'.or.gte.eq.'d') then
            nbon=nbon-1
            if (nbon.eq.0) goto 150
            if (nbon.lt.i1) goto 122
            do 135 i=i1,nbon
               do 140 j=1,6
                  xbon(j,i)=xbon(j,i+1)
                  if (j.gt.5) goto 140
                  ibon(j,i)=ibon(j,i+1)
                  jbon(j,i)=jbon(j,i+1)
                  if (j.gt.4) goto 140
                  ifbon(j,i)=ifbon(j,i+1)
140               continue
135            continue
            goto 100
            endif
         i2=i1
         goto 200
      else if(gte.ne.'N'.and.gte.ne.'n') then
         call inerr(*145)
         endif
      i1=nbon+1
      goto 160
c
c     Add new bonds
c
150   write (nout,2130)
2130  format (/,' Add new bonds (Y/N/E): ')
      call readyn(*160,*300,*300,*150)
160   write (nout,2140)
2140  format (/,' Listing of stored elements required (Y/N): ')
      read (nin,1000,err=160) glis
      if (glis.eq.'Y'.or.glis.eq.'y') then
         do 165 j=1,nx,40
         write (nout,2150)
2150     format (/,1X,2('Nr.  Element  Identifier',10X))
         nnx=nx-j+1
         if (nnx.gt.40) nnx=40
         do 170 k=1,nnx,2
         l=k+j-1
         l1=l+1
         write (nout,2160) (i,gelem(i),gdent(i),i=l,l1)
2160     format (1X,2(I3,4X,A2,6X,A8,11X))
170      continue
         if(k.eq.41) write (nout,2100)
         if(k.eq.41) read (nin,1000,err=165) gdum
165      continue
      else if (glis.ne.'N'.and.glis.ne.'n') then
         call inerr(*160)
         endif
c
c     Read bond parameters (see above for formats)
c
200   do 180 i=i1,i2
182     write (nout,2170) i
2170    format (/,' Enter nr. in list for origin atom of bond',
     1     I4,' (0 for end of list): ')
        read (nin,1030,err=182) ibon(1,i)
        if (ibon(1,i).eq.0) then
          if (i1.eq.i2) call inerr(*182)
          nbon=i-1
          if (nbon.eq.0) goto 300
          goto 100
          endif
        if (ibon(1,i).gt.nx) call inerr(*182)
184     write (nout,2180) i
2180    format (' Enter nr. in list for terminal atom of bond',
     1     I4,': ')
        read (nin,1030,err=184) jbon(1,i)
        if (jbon(1,i).gt.nx) call inerr(*184)
        ii=ibon(1,i)
        jj=jbon(1,i)
186     write (nout,2030) i,gelem(ii),gdent(ii),gelem(jj),gdent(jj)
        read (nin,1010,err=186) ifbon(1,i)
        if(ifbon(1,i).le.0) ifbon(1,i)=1
187     write(nout,2035)
        read(nin,1010,err=187) ifbon(2,i)
        if(ifbon(2,i).lt.0) ifbon(2,i)=0
189     ifbon(3,i)=0
        ifbon(4,i)=0
        write(nout,2045)
        call readyn(*191,*188,*188,*189)
191     ifbon(3,i)=1
188     write (nout,2040)
        read (nin,1020,err=188) xbon(1,i)
        ifbon(4,i)=0
        xbon(3,i)=0.0
        if(xbon(1,i).lt.0.001) goto 192
190     write (nout,2050)
        read (nin,1020,err=190) xbon(2,i)
        call bost(ibon(1,i),jbon(1,i),xbon(3,i),xerr,xbon(2,i),
     1    xbon(1,i))
        if(abs(xbon(3,i)).lt.0.01) then
193       write(nout,2055)
          read(nin,1020,err=193) xbon(3,i)
          xbon(3,i)=abs(xbon(3,i))
          if(xbon(3,i).lt.0.01) xbon(3,i)=xbon(1,i)
          if(xbon(3,i).gt.20.0) call inerr(*193)
          xbon(3,i)=-xbon(3,i)
          ifbon(4,i)=2
          endif
192     write (nout,2060)
        read (nin,1030,err=192) jbon(2,i)
        ibon(2,i)=1
        ibon(3,i)=0
        ibon(4,i)=0
        ibon(5,i)=0
        if (jbon(2,i).eq.0) then
          jbon(3,i)=0
          jbon(4,i)=0
          jbon(5,i)=0
          goto 180
          endif
194     write (nout,2070)
        if(machine.eq.3.or.machine.eq.5) then
          read (nin,*,err=194) (jbon(j,i),j=3,5)
        else
          read (nin,1040,err=194) (jbon(j,i),j=3,5)
          endif
180     continue
      if (i1.ne.i2) nbon=i2
205   write (nout,2190)
2190  format (/,' Listing of bonds required (Y/N/E): ')
      call readyn(*100,*122,*300,*205)
c
c     Display contacts as stored so far
c
300   i10 = 1
      if(ind1.lt.nbon) call indepb(i10,(ind1+1),nbon)
      if(mp.eq.-20) then
        mp=0
        return
        endif
      if(mp.eq.-10) then
        if(ncon.le.0) return
        write(nout,2200) ncon
2200    format(//,' ***** Previous changes in the coordinates',
     1      ' force me to remove',/,'       all',I4,' so far known ',
     2      'contacts *****')
        mp=0
        ncon=0
        ind2=0
        call wait
        return
        endif
      if(ncon.eq.0) then
        write(nout,2210)
2210    format(//,' No contacts stored so far, use function 6 in',
     1      ' MENU 3 for updates',//,' Hit <RETURN> to',
     2      ' continue: ')
        read(nin,1000,err=310) gdum
310     return
        endif
      i10 = 2
      if(ind2.lt.ncon) call indepb(i10,(ind2+1),ncon)
320   write(nout,2220) ncon
2220  format(//,' There are',I4,' contacts stored so far.',/,
     1    ' This program allowes you only to remove existing conta',
     2    'cts !!',//,' Do you want a listing of the known',
     3    ' contacts (Y/N/E): ')
      call readyn(*325,*350,*999,*320)
325   write(nout,2230)
2230  format(//,' Listing of known contacts in the structure:',/,
     1    ' Nr.  from       Symop.  Translations  to         Symop.  ',
     2    'Translations Dep.on',/)
2240  format(I4,2(2X,A2,1X,A6,3X,I3,4X,3I4),1X,I4)
      linc=0
      do 330 i=1,ncon
      linc=linc+1
      if(ind2.lt.i) inde2(i)=0
      write(nout,2240) i,gelem(icon(1,i)),gdent(icon(1,i)),(icon(j,i),
     1    j=2,5),gelem(jcon(1,i)),gdent(jcon(1,i)),(jcon(j,i),j=2,5),
     2    inde2(i)
      if(linc.eq.19.and.i.ne.ncon) then
        linc=0
        write(nout,2100)
        read(nin,1000,err=340)
340     write(nout,2230)
        endif
330   continue
c
c     Prompt for updates
c
350   write(nout,2250)
2250  format(//,' Do you want to remove a contact (Y/N/E): ')
      call readyn(*360,*999,*999,*350)
360   write(nout,2260)
2260  format(/,' Enter number in list of contact to be removed: ')
      read(nin,1030,err=360) nn
      if(nn.lt.1.or.nn.gt.ncon) call inerr(*360)
      ncon=ncon-1
      if(ind2.ge.nn) ind2=nn-1
      if(ncon.eq.0) return
      if(ncon.lt.nn) goto 320
      do 370 i=nn,ncon
        do 370 j=1,6
        xcon(j,i)=xcon(j,i+1)
        if(j.gt.5) goto 370
        icon(j,i)=icon(j,i+1)
        jcon(j,i)=jcon(j,i+1)
        if(j.gt.4) goto 370
        ifcon(j,i)=ifcon(j,i+1)
370     continue
      goto 320
999   i10 = 2
      if(ind2.lt.ncon) call indepb(i10,(ind2+1),ncon)
      return
      end
c
c
***********************************************************************
c
c
      subroutine RAUCOM(jauth,hauth,max,gtype)
c     ========================================
c
c     updates authors and comments (remarks)
c
c     calls inerr, readyn
c
c     written by D. Altermatt
c     McMaster University, July 1984
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      dimension hauth(max)
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
c
      if (jauth.eq.0) goto 40
42    write (nout,2030) jauth,gtype
2030  format (//,1X,I3,A8,'s stored so far: ',/)
      do 45 i=1,jauth
45    write (nout,2040) i,hauth(i)
2040  format (1X,'Nr.',I3,' : ',A70)
40    write (nout,2050) gtype
2050  format (//,' Update',A8,'s (Y/N/E): ')
      call readyn(*47,*999,*999,*40)
47    i1=1
      i2=max
      if (jauth.eq.0) goto 55
60    write (nout,2060) gtype
2060  format (/,' Additional',A8,'s (Y/N/E): ')
      call readyn(*62,*50,*999,*60)
62    i1=jauth
      goto 30
50    write (nout,2070) gtype
2070  format (/,' Enter number of',A8,' to be changed: ')
      read (nin,1000,err=50) i1
1000  format (I5)
      if(i1.gt.jauth) call inerr(*50)
65    write (nout,2080)
2080  format (' Delete or replace (D/R): ')
      read (nin,1030,err=65) gde
1030  format(A1)
      if (gde.eq.'D'.or.gde.eq.'d') then
         jauth=jauth-1
         if (jauth.eq.0) goto 42
         do 53 i=i1,jauth
53       hauth(i)=hauth(i+1)
         goto 42
      else if (gde.ne.'R'.and.gde.ne.'r') then
         call inerr(*65)
         endif
55    if (jauth.eq.0) then
           i1=1
      else
         i2=i1
         endif
30    do 58 i=i1,i2
35    write (nout,2090) gtype,i
2090  format (/,1X,'Enter',A8,' number',I3,' (80 characters, end ',
     1     'with blanks):',/)
      read (nin,1020,err=35) hauth(i)
1020  format(A80)
      if(hauth(i).eq.' ') goto 42
      if(i.gt.jauth) jauth=i
58    continue
      goto 42
999   return
      end
c
c
***********************************************************************
c
c
      subroutine ROXIDE(mp)
c     =====================
c
c     updates the oxidation states in common /ident/ either selectively
c     or by calling =oxide=
c
c     calls inerr, integer, oxide, oxum, readyn
c
c     if MP = 0  ...  =oxide= is called directly
c             1  ...  selective udate assumed
c
c     written by D. Altermatt
c     McMaster University, April 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp,machine
      include 'comatom.for'
      common /dens/   zform,dens,ddens
c
      dimension xoxid(1000),nox(1000),gatom(1000)
c
      if(mp.ne.0) goto 200
c
c     no oxidation states known yet
c
      write(nout,2000)
2000  format(///,' No oxidation states given. Let us try to calculate',
     1    ' them!',//)
10    call oxide(gatom,xoxid,na)
      if(abs(oxum(xoxid,na)).gt.0.49) then
        write(nout,2010)
2010    format(//,' ***** OXIDE cannot find a satisfactory solution',
     1      ' *****',//)
        goto 200
        endif
c
c     try to match the calculated oxidation states in =xoxid= onto =oxid=
c     in common /ident/
c
      do 20 i=1,na
20      nox(i)=1
30    do 40 i=2,na
        j=i
        if(gatom(i).eq.gatom(i-1).and.integer(xoxid(i),2).eq.integer
     1      (xoxid(i-1),2)) goto 50
40      continue
      goto 70
50    nox(j-1)=nox(j-1)+nox(j)
      na=na-1
      do 60 i=j,na
        gatom(i)=gatom(i+1)
        xoxid(i)=xoxid(i+1)
60      nox(i)=nox(i+1)
      goto 30
70    do 80 i=1,na
        if(ndebug.ne.0) write(ndebug,2120) i,gatom(i),xoxid(i),nox(i)
2120    format(' ++debug++roxide++2120++ i =',I3,', gatom = ',A2,', ',
     1      'xoxid =',F5.1,', nox =',I3)
        if(nox(i).le.0) goto 80
        itfl=1
        do 90 j=1,nx
          if(gelem(j).ne.gatom(i)) goto 90
          if(abs(oxid(j)).gt.0.1) then
            itfl=0
            goto 90
            endif
          npo=integer((npos(j)*occu(j)/zform),0)
          if(npo.le.0) npo=integer((occu(j)),0)
          if(ndebug.ne.o) write(ndebug,2130) j,gelem(j),gdent(j),
     1      npos(j),npo,nox(i)
2130      format(' ++debug++roxide++2130++ j =',I3,', gelem = ',A2,
     1      ', gdent = ',A6,', npos =',I3,', npo =',I3,', nox(i) =',I3)
          if(npo.le.nox(i)) then
            oxid(j)=xoxid(i)
            nox(i)=nox(i)-npo
            if(nox(i).eq.0) goto 80
            itfl=1
            goto 90
          else if(npos(j).le.0) then
            if(nx.eq.mx) then
              itfl=1
              goto 90
              endif
            itfl=0
            nx=nx+1
            gelem(nx)=gelem(j)
            gdent(nx)=gdent(j)
            do 100 k=6,1,-1
              if(gdent(nx)(k:k).ne.' ') then
                if(ichar(gdent(nx)(k:k)).lt.ichar('0').or.ichar(gdent
     1              (nx)(k:k)).gt.ichar('9')) then
                  gdent(nx)(k+1:k+1)='1'
                else if(ichar(gdent(nx)(k:k)).ne.ichar('9')) then
                  gdent(nx)(k:k)=char(ichar(gdent(nx)(k:k))+1)
                else
                  gdent(nx)(k:k)='0'
                  gdent(nx)(k-1:k-1)=char(ichar(gdent(nx)(k-1:k-1))+1)
                  endif
                goto 110
                endif
100           continue
110         oxid(nx)=0
            oxid(j)=xoxid(i)
            npos(nx)=-1
            icn(nx)=icn(j)
            nza(1,nx)=nza(1,j)
            nza(2,nx)=nza(2,j)
            ialtno(nx)=0
            nh(nx)=nh(j)
            nd(nx)=nd(j)
            gwyck(nx)=' '
            do 120 k=1,3
120           xx(k,nx)=0.0
            occu(nx)=occu(j)-nox(i)/zform
            occu(j)=nox(i)/zform
            nox(i)=0
            goto 80
            endif
90        continue
        gel=gatom(i)
        write(nout,2020) gel
2020    format(/,' Not able to assign the oxidation states for ele',
     1      'ment ',A2,//,' List of oxidation states as calculated:',//,
     2       ' Nr.  Element    Ox.state  Multipl.',/)
        do 130 k=1,na
          if(gatom(k).ne.gel) goto 130
          gbl='      '
          write(nout,2030) k,gatom(k),gbl,xoxid(k),nox(k)
2030      format(1X,I3,2X,A2,1X,A6,2X,F8.2,5X,I3)
          nox(k)=0
130       continue
        write(nout,2040)
2040    format(/,' Hit <RETURN> to continue: ')
        read(nin,1000,err=140) gdum
1000    format(A1)
140     write(nout,2050)
2050    format(//,' List of oxidation states as stored:',//,
     1      ' Nr.  Element    Ox.state  Multipl.',/)
        do 150 k=1,nx
          if(gelem(k).ne.gel) goto 150
          npo=integer((npos(k)*occu(k)/zform),0)
          if(npo.le.0) npo=integer((occu(k)),0)
          write(nout,2030) k,gelem(k),gdent(k),oxid(k),npo
150       continue
        write(nout,2040)
        read(nin,1000,err=160) gdum
160     do 170 k=1,nx
          if(gelem(k).ne.gel) goto 170
180       write(nout,2060) k,gelem(k),gdent(k)
2060      format(/,' Enter oxidation state for ',I3,2X,A2,1X,A6,
     1        ': ')
          read(nin,1010,err=190) oxid(k)
1010      format(F10.0)
          goto 170
190       call inerr(*180)
170       continue
        nox(i)=0
        goto 70
80      continue
c
c     display stored oxidation states, prompt for selective update
c
200   write(nout,2070)
2070  format(///,' Listing of oxidation states as stored:',//,
     1    ' Nr.  Element    Ox.state  Multipl.',/)
      linc=0
      do 210 i=1,nx
        linc=linc+1
        npo=integer((npos(i)*occu(i)/zform),0)
        if(npo.le.0) npo=integer(occu(i),0)
        write(nout,2030) i,gelem(i),gdent(i),oxid(i),npo
        if(linc.ge.20.and.i.ne.nx) then
          linc=0
          write(nout,2040)
          read(nin,1000,err=220) gdum
220       write(nout,2070)
          endif
210     continue
225   write(nout,2075)
2075  format(//,' Do you want to update oxidation states (Y/N/E): ')
      call readyn(*230,*999,*999,*225)
230   write(nout,2080)
2080  format(/,' Do you want to recalculate all oxidation states',
     1    ' (Y/N/E): ')
      call readyn(*10,*240,*999,*230)
240   write(nout,2090)
2090  format(/,' Enter number in list of element to update ',
     +          '(zero to exit): ')
      read(nin,1020,err=250) nn
1020  format(I5)
      if(nn.eq.0) go to 290
      if(nn.lt.1.or.nn.gt.nx) goto 250
      goto 260
250   call inerr(*240)
260   write(nout,2060) nn,gelem(nn),gdent(nn)
      read(nin,1010,err=270) oxid(nn)
      goto 240
270   call inerr(*260)
290   write(nout,2110)
2110  format(/,' Do you want a listing of the stored oxidation',
     1    ' states (Y/N/E): ')
      call readyn(*200,*999,*999,*290)
999   return
      end
c
c
***********************************************************************
c
      subroutine RUNPRO(*,*)
c     ======================
c
c     structure modeling processor, main routine
c
c     calls:  bodl, chemgr, conxyz, visls, grmult, ifnumb, 
c             inerr, integer, kirch, picwgt, plotmn, rbonds, readyn,
c             spanner, spesor, spmult, spost, spsym, traltn, valmap, 
c             volget,qmenu
c
c     Written by D. Altermatt
c     McMaster University, Winter 1984/85
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
c note:  hmenu = hmenu1, due to overlapping initializations
c
      common /files/  nin,nout,ndebug,nread,nwrite
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      include 'comatom.for'
      include 'combond.for'
      common /dens/   zform,dens,ddens
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
c edited due to overlapping initializations
      common /gmenu1/  hmenu1(20,3)
      common /asymu/  iasym,aslim(3,2),isyel(3,2)
c
c     dimension ishft(3),x(3),y(3),ts(3,3),uu(3,3)
c
      data (hmenu1(i,2),i=1,20)/
     1    ' 1:  Help',
     1    ' 2:  Display special positions and related info ',
     2    ' 3:  Derive density and Z from atom list',
     3    ' 4:  Assign special position flags to known coordinates',
     3    ' 5:  Calculate bonds for the whole structure',
     7    ' 6:  Calculate and analyse a valence map',
     8    ' 7:  Solve the bonding-network with Network analysis',
     9    ' 8:  Refine known coordinates using Least Squares',
     9    ' 9:  Calculate bonding topology (Spanning Tree)',
     2    5*' ',
     1    '15:  Send bonds to a Quattro compatible file',
     1    ' ',
     1    '17:  Do a structure plot (Graphics Terminal only!!)',
     2    '18:  Prepare an input file for CHEMGRAPH or ATOMS ',
     3    '19:  Call MENU LEVEL 2',
     4    '20:  EXIT from STRUMO (write an SCFS-output file)'/
c
c     check if the cell dimensions have changed in the meantime
c     ---------------------------------------------------------
      gyn=' '
      if (icell.gt.0) goto 10
      write (nout,2000) icell,iabs(icell)
2000  format (//,1X,'***** icell =',I3,', set to',I3,' *****',//)
      icell=iabs(icell)
      nbon=0
      ncon=0
      write(nout,2005)
2005  format(/,' ***** All bonds and contacts deleted *****',/)
c
c     do we have to calculate special positions ?
c     -------------------------------------------
10    if (nsp.ne.0) goto 20
      time=nsym/2.
      if(nsym.lt.24) time=4.
      if(nsym.lt.12) time=2.
      if(glat.eq.'P') goto 15
      time=time*2.
      if(glat.eq.'F') time=time*2.
15    write (nout,2020) time
2020  format(//,' Calculating the special positions. This will',
     1     /,' take roughly ',F4.0,' seconds in CPU-time.',//)
      call grmult(ier)
      if (ier.ne.0) goto 31
      ndeb=ndebug
      ndebug=0
        nbug=0
      call spost(ier,nbug)
      call spesor
      call spmult
      call spsym
      ndebug=ndeb
      if(ndebug.ne.0) then
        write(ndebug,2040)
2040    format(/,' ++DEBUG++RUNPRO++2040++   =ISPTAB= LISTING',/,
     1      ' Nr.  Mult.  Wyck.  Symmetry  Isptab(1-nsym)',/)
        do 5 i=1,nsp
5         write(ndebug,2050) i,multsp(i),gsysp(1,i),gsysp(2,i),(isptab
     1        (j,i),j=1,nsym)
2050    format(1X,I3,2X,I4,5X,A1,4X,A8,2X,30I3)
        endif
      goto 31
c
c     special positions read in, but some information is still missing
c     ----------------------------------------------------------------
20    if(isptab(1,1).eq.0) then
25      write(nout,2030)
2030    format(//,' Sorting the special positions.',//)
        call grmult(ier)
        if(ier.ne.0) goto 31
        ndeb=ndebug
        ndebug=0
        call spmult
        call spesor
        call spmult
        call spsym
        ndebug=ndeb
        if(ndebug.ne.0) then
          write(ndebug,2040)
          do 7 i=1,nsp
7           write(ndebug,2050) i,multsp(i),gsysp(1,i),gsysp(2,i),
     1          (isptab(j,i),j=1,nsym)
          endif
        endif
31    if(iasym.eq.0) then 
        iasym=1
        do 32 i=1,3
          isyel(i,1)=0
          isyel(i,2)=0
          aslim(i,1)=0.0
32        aslim(i,2)=1.0
      endif
      do 35 i=1,nx
        if(npos(i).lt.0) goto 35
        if(ialtno(i).le.0) then
          ifkt=4
          goto 300
          endif
35      continue
c
c   write menu level 3
c   ------------------
30    write(nout,2100) (hmenu1(i,2),i=1,20)
2100  format(/,1X,'     MENU LEVEL 3',8X,
     1    'Choose one of the following functions:',//,
     2    20(5X,A70,/),/,' Enter function number: ')
      read(nin,1010,err=40) ifkt
1010  format(I5)
      if(ifkt.lt.1.or.ifkt.gt.20) call inerr(*30)
      goto 50
40    call inerr(*30)
50    if(ifkt.eq.19) return 1
      if(ifkt.ne.1) goto 181
c
c       1: Help
c       -------
      call struhelp
      goto 30
181   if(ifkt.ne.2) goto 200
c
c       2: Display Special Positions
c       ----------------------------
        do 110 i=1,nsp,20
          jj=nsp-i+1
          if(jj.gt.20) jj=20
          write(nout,2110)
2110      format(//,' Listing of special positions',/,
     1        ' No.  coordinates                   multiplicity  ',
     2        ' Wyckoff-not  symmetry',/)
          do 120 j=1,jj
            hbuf=' '
            k=j+i-1
            call conxyz(sp(1,1,k),tp(1,k),hbuf)
            write(nout,2120) k,hbuf,multsp(k),(gsysp(l,k),l=1,2)
2120        format(1X,I3,2X,A30,4X,I4,9X,A8,4X,A8)
120         continue
          if(jj.lt.20) goto 110
          write(nout,2130)
2130      format(/,' Hit <RETURN> to continue: ')
          read(nin,1000,err=110) gdum
1000      format(A1)
110       continue
115     write(nout,2140)
2140    format(//,' Update Wyckoff-Letters (Y/N/E): ')
        call readyn(*117,*30,*30,*115)
117     gyn=' '
        i1=1
        i2=nsp
125     write(nout,2150)
2150    format(/,' Replace a particular letter (Y/N): ')
        read(nin,1000,err=130) gyn
        if(gyn.eq.'y'.or.gyn.eq.'Y') then
          write(nout,2160)
2160      format(/,' Enter special position number: ')
          read(nin,1010,err=130) i1
          i2=i1
        else if(gyn.ne.'n'.and.gyn.ne.'N') then
          goto 130
          endif
        goto 135
130     call inerr(*125)
135     do 140 i=i1,i2
          if(i1.ne.i2.and.gsysp(1,i).ne.'        ') goto 140
          call conxyz(sp(1,1,i),tp(1,i),hbuf)
          write(nout,2170)
2170      format(/,' Spec. Pos.:')
          write(nout,2120) i,hbuf,multsp(i),(gsysp(j,i),j=1,2)
145       write(nout,2180)
2180      format(' Enter Wyckoff-Letter: ')
          read(nin,1000,err=145) gsysp(1,i)(1:1)
140       continue
        goto 115
200   if(ifkt.ne.3) goto 300
c
c    3: Density and Z
c    ----------------
        npmax=0
        npmin=100
        do 210 i=1,nx
          if(npos(i).le.1) goto 210
          if(npos(i).gt.npmax) npmax=npos(i)
          if(npos(i).lt.npmin) npmin=npos(i)
210       continue
        k=2
        if(nsp.le.2) k=1
        if(npmin.eq.100) then
          npmin=multsp(nsp)
          npmax=multsp(k)
          xpm=1.0
        else
          if(ndebug.ne.0) write(ndebug,3000) npmin,npmax,multsp(k),
     1        multsp(nsp)
3000      format(/,' ++DEBUG++RUNPRO++3000++ NPMIN=',I3,', NPMAX=',I3
     1        ,', MULTSP(K)=',I3,', MULTSP(NSP)=',I3,/)
          if(npmax.lt.multsp(k)) npmax=multsp(k)
          if(npmin.lt.multsp(nsp)) npmin=multsp(nsp)
          xpm=npmin
          endif
220     wght=0.0
        tvol=0.0
        do 230 i=1,nx
          k=0
          if(nza(2,i).eq.3) k=ifnumb(gelem(i))
          xpos=npos(i)
          if(npos(i).le.0) xpos=1.0
          xpos=xpos*occu(i)/xpm
          if(xpos.lt..99.and.occu(i).gt..99) xpos=1.0
          if(oxid(i).lt.-.01) tvol=tvol+xpos*volget(gelem(i))
          xhd=1.0
          if(gelem(i).eq.'D ') xhd=2.0
          wght=wght+xpos*xhd*picwgt(nza(1,i),nza(2,i),k)
          if(ndebug.ne.0) write(ndebug,2250) wght,gelem(i),npos(i),
     1        xpos,xhd,picwgt(nza(1,i),nza(2,i),k),tvol,vol
2250      format(/,' ++DEBUG++RUNPRO++2250++ WGHT=',F7.2,', GELEM(I)'
     1        ,'= ',A2,', NPOS(I)=',I3,', XPOS=',F5.2,/,' XHD=',F5.2,
     2        ', WEIGHT(GELEM(I))=',F8.3,', TVOL=',F8.3,', VOL=',F8.3)
230       continue
        tzfo=1.0
        if(tvol.gt.0.01) tzfo=vol/tvol
        if(integer(tzfo,0).ge.multsp(1).and.npmax.lt.multsp(1))
     1      npmax=multsp(1)
        xpi=npmin
        xpa=npmax
        dmin=xpi*wght*1.66/vol
        dmax=xpa*wght*1.66/vol
        write(nout,2200) zform, dens, tzfo
2200    format(//,' The stored values are: ',//,' Z = ',F5.2,
     1      ' Formula Units, Density = ',F6.3,' g/cm3',//,
     2      ' Based on the anion volume, Z is expected to be close',
     3      ' to',F6.2,/)
        if(npmin.eq.npmax) then
235       write(nout,2210) xpi,dmin
2210      format(//,' Based special position multiplicities',
     1        ' my proposal is: ',//,' Z = ',F5.2,
     2        ' Formula Units, Density = ',F6.3,' g/cm3',//,
     3        ' Do you accept this proposal (Y/N/E): ')
          call readyn(*236,*237,*30,*235)
236       zform=xpi
          dens=dmin
          goto 30
237       write(nout,2215)
2215      format(/,' Enter Z of your choice (Integer): ')
          read(nin,1010,err=238) npc
          goto 239
238       call inerr(*237)
239       goto 250
        else
240       write(nout,2220) xpi,xpa,dmin,dmax
2220      format(//,' Based on special position multiplicities',
     1      ' my proposal is: ',//,19X,' Z between ',F5.2,
     2      '   and',F6.2,'  Formula Units',/,'  which leads to a den',
     3      'sity of ',F6.3,'  to  ',F6.3,' g/cm3',//,' Choose the',
     4      ' appropriate Z (Integer): ')
          read(nin,1010,err=245) npc
          goto 250
245       call inerr(*240)
          endif
250     if(npc.eq.npmin) then
          zform=xpi
          dens=dmin
          goto 30
        else if(npc.eq.npmax) then
          zform=xpa
          dens=dmax
          goto 30
        else
          itest=0
          do 260 j=1,nsp
260       if(multsp(j).eq.npc) itest=1
          if(itest.eq.1) then
            zform=npc
            dens=wght*zform*1.66/vol
            goto 30
          else
265         write(nout,2230) npc
2230        format(/,' Z =',I3,' does not match with any',
     1        ' special position multiplicity',//,' Do you want',
     2        ' this Z anyway (Y/N): ')
            read(nin,1000,err=265) gyn
            if(gyn.eq.'y'.or.gyn.eq.'Y') then
              zform=npc
              dens=wght*zform*1.66/vol
              goto 30
            else if(gyn.eq.'n'.or.gyn.eq.'N') then
270           write(nout,2240)
2240          format(/,' Choose an other Z (Integer): ')
              read(nin,1010,err=275) npc
              goto 280
275           call inerr(*270)
280           goto 250
            else
              call inerr(*265)
              endif
            endif
          endif
300   if(ifkt.ne.4) goto 400
c
c    4: Assign Special Position Numbers
c    ----------------------------------
         call traltn(*30,*350)
350      write(nout,2300)
2300     format(//,' Do you want to update the bonds now (Y/N/E): ')
         call readyn(*360,*30,*30,*350)
360      mp=2
         call rbonds(mp)
         goto 30
400   if(ifkt.ne.5) goto 500
c
c      5: calculate bonds for the whole structure
c      ------------------------------------------
        call bondl(icmpd)
        go to 30
c *** The following code is still under development ***
c    Fit new atomic positions into the structure
c    ----------------------------------------------
c        call newfit
c        goto 30
c *** The following code is still under development ***
c    Explore site coordinations, update /bonds/ and /contact/
c    -----------------------------------------------------------
c        call corget
c        goto 30
c *** The following code is still under development ***
c    Set up coordination spheres
c    ------------------------------
c        call polyhe
c        goto 30
500   if(ifkt.ne.6) goto 600
c
c    6: Valence map
c    --------------
        ntyp=1
        call valmap(ntyp)
        goto 30
600   if(ifkt.ne.7) goto 700
c
c    7: Try to solve the bonding-network with KIRCH
c    ----------------------------------------------
        irefin=0
        call kirch(irefin)
        goto 30
700   if(ifkt.ne.8) goto 800
c
c       8: Interactive Distance-Least-Squares
c       -------------------------------------
      call visls
      goto 30
800   if(ifkt.ne.9) goto 900
c
c      9: Find spanning tree
c      ---------------------
        call spanner
        go to 30
900   if(ifkt.ne.10) goto 1001
c
c      10: (spare)
c      -------------------------------------
        goto 30
1001  if(ifkt.gt.10.and.ifkt.lt.15) go to 8834
      if(ifkt.gt.16) go to 1700
c *** The following code is still under development ***
c      fit a complex into the structure
c      ------------------------------------
c        call fitcom
c        go to 30
c
c *** The following code is still under development ***
c      calculate quantum strucutre diagram parameters
c      --------------------------------------------------
c        call qsdere(sumnv,delrad,delel)
c        write(nout,1340) sumnv,delrad,delel
c 1340   format(/,' Quantum Structure Diagram parameters',/,
c     2           ' ------------------------------------',/,
c     3 ' Valence electron count           =',F6.2,/,
c     4 ' Psudopotential radius difference =',F6.2,/,
c     5 ' Electronegativity difference     =',F6.2)
c        call wait
c        go to 30
c
c
      if(ifkt.ne.15) goto 1700 
c
c      15: Quattro File
c      -----------------
        call qmenu
        goto 30
c
c *** The following code is still under development ***
c    16: Pseudo-plot
c    ---------------
c        call psuplo
c        goto 30
c
8834    write(nout,3100)
3100    format(/,' ***** Function not yet available *****',/)
        goto 30
c
1700  if(ifkt.ne.17) goto 1800
c
c    17: Envoke plot program for graphics terminal
c    ---------------------------------------------
c ***************************************************
c    PLOTMN had been commented out for PC version
c****************************************************
c 1710  write(nout,3110)
c 3110  format(///,' STRUMO plotting package',///,
c    1  ' The following terminals are known to the system: ',//,
c    2  ' 1:  LUNDY/SIGMEX',/,' 2:  TEKTRONIX/PERICOM',/,
c    3  ' 3:  other',///,' Enter code number for your terminal: ')
c     read(nin,1010,err=1720) nterml
c     if(nterml.lt.1.or.nterml.gt.3) goto 1720
c     if(nterml.eq.3) then
c       write(nout,3120)
c 3120    format(//' Sorry, PLOTLB cannot handle your terminal yet!!',
c    1    ///,' Hit <RETURN> to continue: ')
c       read(nin,1000,err=30) gdum
c       goto 30
c       endif
      call plotmn 
      goto 30
c 1720  call inerr(*1710)
1800  if(ifkt.ne.18) goto 999
c
c    18: Prepare input for CHEMGRAPH or ATOMS
c    ----------------------------------------
  65  write (nout,1850)
1850  format (//' 1. Prepare input file for CHEMGRAPH',/
     1          ' 2. Prepare input file for ATOMS',/
     2          '19. Return to menu level 3',/
     3          '20. Exit from Strumo',//
     4          'Enter your selection:')
      read (nin,1900,err=60) ilike
1900  format (I5)
      if (ilike.eq.1) then
        call chemgr
      else if (ilike.eq.2) then
        call atomgr
      else if (ilike.eq.19) then
        go to 30
      else if (ilike.eq.20) then
        go to 999
      else 
        go to 60
        endif
      call wait
      go to 30
999   return 2
 60   call wait
      call inerr(*65)
      end
c
c
***********************************************************************
c
c
      subroutine TRALTN(*,*)
c     ======================
c
c     Finds the special position number IALTNO for all atoms stored in
c     /ident/ and then transforms the atomic coordinates in /xatom/ and
c     the temperatur factors in /temp/ to become the special position
c     representative.
c
c     calls:  conxyz, inmul, mxm, mxv, naltno, transp, vplusv, wait
c
c     written by D. Altermatt, McMaster Universtity
c     cut from RUNPRO in March 1986 by D. Altermatt, Uni Lausanne
c     Updated July 14,1989 by s.g.urquhart
c     Modifications Nov. 1st, 1990 by S. Healy (as requested by Dr. Brown)
c
      implicit character *8 (g)
      implicit character *80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgr,machine,nterm
      include 'comatom.for'
      include 'combond.for'
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
c
      dimension x(3),y(3),ishft(3),dx(3),uu(3),duu(3),ts(3,3)
c
300    ibonfl=0
      linc=0
      ifail=0
      if(ndebug.ne.0) write(ndebug,2320)
      write(nout,2320)
2320  format(//,' Listing of Atomic Positions with Special Posi',
     1  'tions assigned',//,' Nr. Elem. Ident. Transformed Coor',
     2  'dinates  Spe.Op. Sym.Op. Centr. Cell Shift',/,17X,' Ge',
     3  'neral coordinates',11X,'Multipl. Wyck.Not. Symmetr.',/)
c loop through atoms
c
      ifail=0
      do 310 i=1,nx
        npfl=0
c search for special position
        if(npos(i).ge.0) then
          if(ndebug.ne.0) write(ndebug,2300) i,gelem(i)
2300      format(/,' ++DEBUG++TRALTN++2300++ CALL NALTNO FOR',
     1      ' ELEMENT NR.',I3,1X,A2)
305       npfl=npfl+1
          ndeb=ndebug
c          ndebug=0
          call naltno(xx(1,i),npos(i),ialtno(i),naltst,icent,ishft)

          ndebug=ndeb
          if(ndebug.ne.0) write(ndebug,2305) xx(1,i),npos(i),
     1      ialtno(i),naltst,tu(1,icent),ishft(1),(xx(jj,i),
     2      tu(jj,icent),ishft(jj),jj=2,3)
2305      format(/,' ++DEBUG++TRALTN++2305++ NALTNO RETURNED:',/,
     1      '     XX    NPOS    IALTNO  NALTST  TU(IC)  ISHFT',/,
     2      2X,F7.3,1X,3(I4,4X),F7.3,4X,I4,2(/,2X,F7.3,25X,F7.3,
     3      4X,I4))
          if(ialtno(i).eq.0) then
c special position not found
            if(npfl.eq.1) goto 305
            hbuf='no match for this position'
            ifail=1
            nn0=0
            write(nout,2310) i,gelem(i),gdent(i),(xx(k,i),k=1,3),
     1        (nn0,k=1,6),hbuf,npos(i)
            if(ndebug.ne.0) write(ndebug,2310) i,gelem(i),gdent(i),
     1        (xx(k,i),k=1,3),(nn0,k=1,6),hbuf,npos(i)
2310        format(1X,I3,1X,A2,2X,A8,1X,3F8.4,3X,I3,5X,I3,4X,I3,
     1        3X,3I3,/,20X,A26,2X,I4,8X,A1,5X,A8)
            linc=linc+1
            goto 320
            endif
          npos(i)=multsp(ialtno(i))
          if(gsysp(1,ialtno(i)).eq.' ') gsysp(1,ialtno(i))=gwyck(i)
          gwyck(i)=gsysp(1,ialtno(i))
          invs=inmul(1,naltst)
          if(ndebug.ne.0) write(ndebug,2260) i,(xx(k,i),k=1,3),invs
2260      format(' ++DEBUG++TRALTN++2260/2270++ UPDATING XX(',I3,') ',
     1      'FROM ',3F7.4,' USING S(',I3,')')
          call mxv(x,s(1,1,invs),xx(1,i))
          call mxv(dx,s(1,1,invs),dxx(1,i))
          call vplusv(x,x,t(1,invs))
          call vplusv(x,x,tu(1,icent))
          do 315 k=1,3
            dxx(k,i)=dx(k)
            y(k)=xx(k,i)
315         xx(k,i)=x(k)+(ishft(k)-2.)
330       do 340 k=1,3
            if(xx(k,i).gt.0.999999) then
              xx(k,i)=xx(k,i)-1.0
              ishft(k)=ishft(k)-1
              goto 330
            else if(xx(k,i).lt.-.000001) then
              xx(k,i)=xx(k,i)+1.0
              ishft(k)=ishft(k)+1
              goto 330
              endif
340         continue
          if(ndebug.ne.0) write(ndebug,2270) (xx(k,i),k=1,3)
2270      format('+',85X,' TO ',3F7.4)
c
c     Check if coordinates actually changed and if this affects bonds
c     and/or contacts
c     ---------------------------------------------------------------
          if(nbon.le.0.and.ncon.le.0) goto 390
          do 350 k=1,3
350         if(abs(y(k)-xx(k,i)).gt.0.01) goto 360
          goto 390
360       do 370 k=1,nbon
            if(ibon(1,k).eq.i) then
              ibon(1,k)=1000*i
              ibonfl=1
            else if(jbon(1,k).eq.i) then
              jbon(1,k)=1000*i
              ibonfl=1
              endif
370         continue
          do 380 k=1,ncon
            if(icon(1,k).eq.i) then
              icon(1,k)=1000*i
              ibonfl=1
            else if(jcon(1,k).eq.i) then
              jcon(1,k)=1000*i
              ibonfl=1
              endif
380         continue
c
c     Transform also temperature factors, if stored
c     ---------------------------------------------
390       if(naltst.ne.1) then
            if(abs(u(1,1,i)).gt.0.00001) then
              ju=3
              call transp(ts,s(1,1,naltst),ju)
              ju=3
              call mxm(uu,u(1,1,i),ts,ju)
              call mxm(duu,du(1,1,i),ts,ju)
              ju=3
              call mxm(u(1,1,i),s(1,1,naltst),uu,ju)
              call mxm(du(1,1,i),s(1,1,naltst),duu,ju)
              endif
            endif
**  There are occasionally problems in conxyz (access violation at line 28
**  in CONXYZ when called from this spot.  Until this problem is sorted out
**  the call to CONZYX is bypassed.  Entries 1271,200595 and 61142 cause
**  this problem.  Most entries do not.  It does not always occur on the
**  first call in the run. (? memory getting overwritten?)
c
**  This appears to have been caused by a typo in the above if block
**  ie. replacing "naltst" with "nalts".  The call to conxyz is back
          hbuf='              '
          if(ndebug.ne.0) then
            hbuf='                 '
            write(ndebug,2390) hbuf
 2390       format(' *debug*traltno*2390* ',A26)
            endif
          call conxyz(sp(1,1,ialtno(i)),tp(1,ialtno(i)),hbuf)
          linc=linc+1
          write(nout,2310) i,gelem(i),gdent(i),(xx(k,i),k=1,3),
     1      ialtno(i),naltst,icent,(ishft(k),k=1,3),hbuf,npos(i),
     2      (gsysp(k,ialtno(i)),k=1,2)
          if(ndebug.ne.0) write(ndebug,2310) i,gelem(i),gdent(i),
     1      (xx(k,i),k=1,3),ialtno(i),naltst,icent,(ishft(k),k=1,3),
     2      hbuf,npos(i),(gsysp(k,ialtno(i)),k=1,2)
        else
          hbuf='dummy, so far '
          nn0=0
          linc=linc+1
          write(nout,2310) i,gelem(i),gdent(i),(xx(k,i),k=1,3),
     1      (nn0,k=1,6),hbuf
          if(ndebug.ne.0) write(ndebug,2310) i,gelem(i),gdent(i),
     1      (xx(k,i),k=1,3),(nn0,k=1,6),hbuf
          endif
320     continue
        if(linc.eq.10.and.i.ne.nx) then
          linc=0
          call wait
          write(nout,2320)
          endif
310     continue
c assignment of special positions has failed
c
      if (ifail.ne.0) then
         write(nout,2380)

2380  format(//,' The atoms cannot be assigned to their special',
     1  ' positions.',/,' The reason may be a wrong choice of space ',
     2  'group  or an incorrect ',/,
     3  ' assignment of multiplicities.  In the later case, an ',/,
     4  ' assignment may be possible if the multiplicities ',
     5  'are set to',/,' zero.',//)

      if(nterm.eq.6) then
312   write(nout,2381)

2381  format(' Do you wish to set the multiplicities to zero and',/,
     1  ' try again (Y/N/E) : ')

      call readyn(*314,*317,*313,*312)

313   return 1
314   do 316 i=1,nx
316      if(npos(i).ge.0) npos(i)=0
      goto 300
      endif
      endif
317   continue
c
c     Display affected bonds/contacts
c     -------------------------------
      if(ibonfl.ne.0) then
        call wait
        write(nout,2000)
2000    format(//,' Above transformations affect some of the bonds',
     1    ' and/or contacts stored!',//)
        do 400 i=1,nbon
          if(ibon(1,i).gt.999.or.jbon(1,i).gt.999) then
            write(nout,2010)
2010        format(' Some bonds will have to be updated ',
     1        '(see Menu I)',//)
            goto 410
            endif
400       continue
        ibonfl=0
410     iconfl=0
415     do 420 i=1,ncon
          k=i
          if(icon(1,i).gt.999) then
            iconfl=iconfl+1
            icon(1,i)=icon(1,i)/1000
            goto 430
          else if(jcon(1,i).gt.999) then
            iconfl=iconfl+1
            jcon(1,i)=jcon(1,i)/1000
            goto 430
            endif
420       continue
425     if(ibonfl.eq.1) return 2
        call wait
        return 1
430     if(iconfl.eq.1) write(nout,2020)
2020    format(' The following contacts got removed: ',/,
     1    ' From        Position     To          Position',/)
        if(iconfl.eq.1) linc=0
        linc=linc+1
        ncon=ncon-1
        write(nout,2030) gelem(icon(1,k)),gdent(icon(1,k)),(icon(j,k),
     1    j=2,5),gelem(jcon(1,k)),gdent(jcon(1,k)),(jcon(j,k),j=2,5)
2030    format(1X,A2,1X,A6,1X,4I3,3X,A2,1X,A6,1X,4I3)
        if(ncon.le.0) goto 425
        do 440 l=k,ncon
          do 450 ll=1,6
            xcon(ll,l)=xcon(ll,l+1)
            if(ll.gt.5) goto 450
            icon(ll,l)=icon(ll,l+1)
            jcon(ll,l)=jcon(ll,l+1)
            if(ll.gt.4) goto 450
            ifcon(ll,l)=ifcon(ll,l+1)
450         continue
440       continue
        if(linc.ge.19) then
          linc=0
          call wait
          write(nout,2000)
          endif
        goto 415
        endif
      call wait
      return 1
      end
c
***********************************************************************
c
      subroutine STRUHELP
c     ===================
c
c    Written apr 89 by Chris Carr
c
c    Last Update: 26 apr 89              C.Carr
c
      common /files/nin,nout
      nin=5
      nout=6
c
345   write(nout,346)
c
346   format (/,5X,'                  STRUMO'/,
     +5X,'                  ------'/,
     +5X,'      Strumo is an interactive analysis and modelling'/,
     +5X,'program for inorganic solids.  It makes extensive use'/,
     +5X,'of the bond valence model (see Brown Phys. Chem. Miner.'/,
     +5X,'(1987) 15,30 and references therein).  It requires'/,      
     +5X,'oxidation numbers for the atoms and assigns bond valences'/,
     +5X,'to the bonds.'/,  
     +5X,'       The modelling techniques available permit'/,
     +5X,'the calculation of the asymmetric set of bonds, the'/,
     +5X,'expansion of these to form a network, the'/,
     +5X,'prediction of bond distances using network'/,
     +5X,'equations, and the fitting of atomic co-ordinates to'/,
     +5X,'match the predicted bond lengths.  The Valence Sum Map'/,
     +5X,'allows the user to explore the crystal to find sites'/,
     +5X,'that have the right chemical environment for a given'/,
     +5X,'atom.'/)
c
      call pageflip(*347,*345,*444)
c
347   write(nout,348) 
c
348   format(/,5X,'      Strumo is menu-driven.  The level 1 menu'/,
     +5X,'is used to load the data defining a particular crystal'/,
     +5X,'structure either from a file or from the terminal.  The'/,
     +5X,'level 2 menu allows changes to be made in these data.'/,
     +5X,'The level 3 menu provides access to a variety of different'/,
     +5X,'modelling techniques.  Each of these techniques may have'/,
     +5X,'its own fourth level menu and help options.'/,
     +5X,'      Besides menu selections you will be asked questions'/,
     +5X,'whose answers are Y(es), N(o), or E(xit).'//)
c
      call pageflip(*151,*345,*444)
c
151   write(nout,152)
c
152   format(/,1X,'          Help Menu (level 1.1/3.1)'//,
     +5X,'For further help select one of the following :'/,  
     +5X,'(20 to return to menu 1)'//,
     +5X,'1: Data entry and checking'/,
     +5X,'2: Calculation of bonds'/,
     +5X,'3: Handling of symmetry and special positions'/,
     +5X,'4: Use of Valence Sum Maps'/,
     +5X,'5: Prediction of bond lengths'/,
     +5X,'6: Least-Squares refinement of co-ordinates'/,
     +5X,'7: Bonding topology analysis'//,
     +4X,'20: Return to previous menu'//,
     +5X,'        Enter info number :')
c
      read(nin,153,err=199) ihelpno
c
153   format(I5)
      if(ihelpno.eq.20) goto 444
      if(ihelpno.lt.1.or.ihelpno.gt.7) call inerr(*151)
      if(ihelpno.eq.1) goto 1000
      if(ihelpno.eq.2) call bondhelp
      if(ihelpno.eq.3) call symhelp
      if(ihelpno.eq.4) call maphelp
      if(ihelpno.eq.5) call nethelp
      if(ihelpno.eq.6) call vishlp 
      if(ihelpno.eq.7) call spanhelp
c
199   go to 151
c
1000  write(nout,1001)
c
1001  format(/,1X,' At menu level 1 :'//,
     +5X,'STRUMO reads in crystal structure information from one or'/,
     +5X,'more files in either the Standard Crystallographic File'/,
     +5X,'Structure (SCFS), SHELX format, BONDFILE format (as produced'/,
     +5X,'by SINDBAD) or the Inorganic Crystal Structure Database'/,
     +5X,'format as output by CAN/SND (Ottawa) (CRYSTIN).'/,
     +5X,'Alternatively the data may be entered at the keyboard on'/,
     +5X,'prompt.  STRUMO then requests any missing information.  It'/,
     +5X,'calculates the symmetry operators from the Hall space group'/,
     +5X,'symbol.  If this is not given, it looks in the file SPGR.DAT'/,
     +5X,'for the Hall symbol that corresponds to the Hermann-Mauguin'/,
     +5X,'symbol given [NB: the Hermann-Mauguin symbol may not define'/,
     +5X,'the setting uniquely].  It then reads the valence parameters'/,
     +5X,'that it needs to calculate the bond valences from the file'/,
     +5X,'VALENCE.DAT and passes to menu level 2.'///)
c
      call pageflip(*1002,*151,*444)    
c
1002  write(nout,1003)
c
1003  format(/,1X,' At menu level 2 :'//,
     +5X,'Any parameter can be changed or checked.  On exiting menu'/,
     +5X,'2, if the unit cell dimensions have been changed, STRUMO '/,
     +5X,'deletes the existing bonds and contacts.  The special '/,
     +5X,'positions of the space group are then calculated and'/,
     +5X,'special position flags (Altermatt numbers - Altermatt and'/,
     +5X,'Brown Acta Cryst. A43 (1987) 125) are assigned to each'/,
     +5X,'atom.  These are required in the network expansion '/,
     +5X,'routines in menu level 3.  Descriptions are contained in'/,
     +5X,'menu level 3.1.'///)
c
      call pageflip(*151,*1000,*444)
c
      go to 151
c
444   return
      end
