C
C                         FILE SYMLIB             Last update: 91.10.7
C                         -----------                         I.D.Brown
C
C
C                SYMMETRY INDEX MANIPULATION:
C
C  S INVSOP(n,j3,l3,j2,l2,k,lk)  Gets STL(3) in STL(3)*[STL(2)*SPTP(K)]=SPTPL(K)
C  S INVSTL(n,j3,l3,j2,l2,j1,l1) N=1: J3,L3 IN (S/T/L)1 = (S/T/L)3 * (S/T/L)2
C                                N=2: J3,L3 IN (S/T/L)1 = (S/T/L)2 * (S/T/L)3
C  S MLTSTL(i3,l3,i2,l2,i1,l1,K) Gets STL(3) in STL(3)*SPTP(K) =
C                                                      STL(2)*[STL(1)*SPTP(K)]
C                                with respect to ISPTAB(K)
C  S SITEOP(n,j3,l3,j2,l2,k,lk)  Gets the STL(3) which is equivalent to STL(2)
C                                acting on SPTPL(K)
C  S SITTRN(n,i3,l3,k2,j2,l2,k1,j1,l1)  Gets STL(3) where
C                                STL(3)*SPTP(K2)=STL(4)*[STL(2)*SPTP(K2)]
C                                and STL(4)*[STL(1)*SPTP(K1)]=STL(1)*SPTP(K1)
C  S SYMTRN(n,i3,l3,i2,j2,l2,i3,j3,l3,x) analog to SITTRN, but also returns
C                                coordinates
C
C                FUNCTIONS
C
c  F DSTNCE(i,isy,ity,j,jsy,jty) returns distance between atoms i and j
C  F INMUL(I,J)                  RETURNS K IN  S(I) = S(K) * S(J)
C  F INMULT(I,J)                 RETURNS K IN  S(I) = S(J) * S(K)
C  F ISYMCH(...)                 DOES SYMMETRY CHECKS FOR BONDS/CONTACTS
C
C                COORDINATE TRANSFORMATION
C
C  S GETSTL(LT,I,YX,ISP)         GETS L IN YX = (S/T/L)ISP * XX(I)
C  S SYMOP(Y,S,T,X)              GETS Y(3) = (S/T) * X(3)
C  S SYMOPL(Y,S,T,L,X)           DOES Y = (S/T/L) * X
C  S SYMTEM(UT,S,U)              DOES UT = S(TRANS) * U * S
C  S SYMEQ(Y,X,IS,IT)            GETS IS,IT IN Y=(IS/IT)*X
C  S SYMRDC(NT,NAT)              SCANS NT ATOMS IN XATOM/BONDS AND REDUCES
C                                  TO NAT ATOMS IN ASYMMETRIC UNIT
C
C                MATRIX TRANSFORMATION
C
C  S MULSTL(.(3).(2).(1).)       DOES (S/T/L)3 = (S/T/L)2 * (S/T/L)1 WITH
C                                RESPECT TO LATTICE TRANSLATIONS, UPON REQUEST
C  S SYMULT(S3,T3,S2,T2,S1,T1)   GETS (S/T)3 = (S/T)2 * (S/T)1
C
C                OTHERS
C
C  S GRMULT(IER)                 CALCULATES THE POINT GROUP MULTIPLICATION TABLE
C  S INVMUL(IB,IC,IA,IER)        GETS IB IN IC(4)=IB(4)*IA(4)
C  S ISMULT(ISC,ISB,ISA)         GETS ISC(4) = ISB(4)*ISA(4)
C  S CONXYZ(S,T,HBUF)            HBUF CONTAINS THE ALPHANUMERIC-REPRESENTATION
C                                OF (S/T)
c  F IBONCH(IA(5),JA(5),IB(5),JB(5),ID,IS,IT,IL(3))  =0 IF THE BOND IA-JA
C                                IS SYMMETRY EQUIVALENT TO IB-JB
C
C    EXTERNALS CALLED FROM THIS FILE:
C
C  DISTLB: CHEBON
C  MATHLB: ICOMMA, INTEGER, MXM, MXV, VMINV, VPLUSV,IVPLUV,JCOMVE
C
C
c****** STANDARD FORTRAN 77 CODE ONLY USED IN THIS FILE
c
c
***********************************************************************
c
c
      subroutine invsop(n,j3,l3,j2,l2,k,lk)
c
c     calls inmult, integer, mxv, symult, vminv, vplusv
c
c     in special position group K find j3 and l3 for
c     (S/T/L)3 * (SP/TP/LP)2 = (SP/TP/LP)k
c
c     output:  n = 0 in case of error
c
c     function calculated is
c     (s/t/l)3*[(s/t/l)2*(sp/tp)]=(sp/tp/lk)
c       where j3 = isym >= n
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      dimension l3(3),lk(3),l2(3),t5(3),t1(3),t2(3),t3(3),t4(3)
      dimension spp(3,3),tpp(3)
c
      do 10 l=1,nsym
        n=l
 10     if(isptab(l,k).eq.j2) goto 20
 15   n=-20
      return
c
 20   j3=inmul(1,n)
      if(j3.le.0)then
        n=j3
        return
        endif
      do 30 l=1,3
        t1(l)=lk(l)/12.
 30     t2(l)=l2(l)/12.
      call symult(spp,tpp,s(1,1,j2),t(1,j2),sp(1,1,k),tp(1,k))
      call mxv(t3,s(1,1,j3),tpp)
      call mxv(t4,s(1,1,j3),t2)
      call vplusv(t3,t3,t4)
      call vplusv(t3,t3,t(1,j3))
      call vplusv(t4,t1,tp(1,k))
      call vminv(t5,t4,t3)
c
      do 40 l=1,3
 40     l3(l)=integer((12.*t5(l)),0)
      return
      end
c
c
***********************************************************************
c
c
      subroutine invstl(n,j3,l3,j2,l2,j1,l1)
c
c     calls inmul, inmult, integer, mxv, vminv, vplusv
c
c     does invers multiplications of symmetry operators with respect to
c     their lattice translations ln:
c
c     n = 1:  gets J3 and L3 in (S/T/L)3 * (S/T/L)2 = (S/T/L)1
c
c     n = 2:  gets J3 and L3 in (S/T/L)2 * (S/T/L)3 = (S/T/L)1
c
c     output: n = 0 in case of error
c
c     written by D. Altermatt
c     McMaster Univesity, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /gsym/   glat,gcent,hmsym,hall,gsysp(2,30)
c
      dimension l1(3),l2(3),l3(3),t1(3),t2(3),t3(3),t4(3),t5(3)
c
      do 10 m=1,3
        t1(m)=l1(m)/12.
 10     t2(m)=l2(m)/12.
      if(n.eq.2) goto 20
c
      if(n.ne.1) then
        n=0
        write(nout,2000)
 2000   format(//,' **ERROR**INVSTL**2000** N .ne. 1 or 2, no ',
     1    'calculation',//)
        return
        endif
      j3=inmul(j1,j2)
      call mxv(t4,s(1,1,j3),t(1,j2))
      call mxv(t5,s(1,1,j3),t2)
      call vplusv(t4,t4,t5)
      call vplusv(t4,t4,t(1,j3))
      call vplusv(t5,t1,t(1,j1))
      call vminv(t3,t5,t4)
      goto 30
c
 20   j3=inmult(j1,j2)
      call mxv(t4,s(1,1,j2),t(1,j3))
      call vplusv(t4,t4,t(1,j2))
      call vplusv(t4,t4,t2)
      call vminv(t5,t5,t4)
      m=inmul(1,j2)
      call mxv(t3,s(1,1,m),t5)
c
 30   do 40 m=1,3
 40     l3(m)=integer((12.*t3(m)),0)
      return
      end
c
c
***********************************************************************
c
c
      subroutine mltstl(i3,l3,i2,l2,i1,l1,k)
c
c     calls integer, mulstl, symult
c
c     calculates in the special position K the product
c     (SP/TP/LP)3 = (S/T/L)2 * (SP/TP/LP)1
c
c     for the general position set: k=1
c
c     the function calculated is
c     (s/t/l)3*(sp/tp)k=(s/t/l)2*[(s/t/l)1*(sp/tp)k]
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      dimension l1(3),l2(3),l3(3),ssp(3,3),ttp(3),spp(3,3),tpp(3)
c
      call symult(ssp,ttp,s(1,1,i1),t(1,i1),sp(1,1,k),tp(1,k))
      i10 = 1
      call mulstl(spp,tpp,l3,s(1,1,i2),t(1,i2),l2,ssp,ttp,l1,i10,tu)
c
      i3=mult(i1,i2)
      i3=isptab(i3,k)
      call symult(ssp,ttp,s(1,1,i3),t(1,i3),sp(1,1,k),tp(1,k))
c
      do 10 i=1,3
 10     l3(i)=l3(i)+integer((12.*(tpp(i)-ttp(i))),0)
      return
      end
c
c
***********************************************************************
c
c
      subroutine siteop(n,j3,l3,j2,l2,k,lk)
c
c     calls integer, mxv, symult, vminv, vplusv
c
c
c     output: n = -40 or -41 as error output
c
c     function calculated is
c     (s/t/l)3*(sp/tp)k=(s/t/l)2*(sp/tp/l)k
c       where j3 = isym >= n
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      dimension l3(3),lk(3),l2(3),t5(3),t1(3),t2(3),t3(3),t4(3)
      dimension spp(3,3),tpp(3)
c
      if(n.le.0) n=1
      if(n.gt.nsym) then
        n=-40
        return
        endif
      do 10 l=n,nsym
        if(isptab(l,k).eq.j2) then
          nn=l
          j3=l
          goto 20
          endif
 10     continue
 5    n=-41
      return
c
 20   n=nn
      do 30 l=1,3
        t1(l)=lk(l)/12.
 30     t2(l)=l2(l)/12.
      call mxv(t3,s(1,1,j3),tp(1,k))
      call mxv(t4,s(1,1,j3),t1)
      call symult(spp,tpp,s(1,1,j2),t(1,j2),sp(1,1,k),tp(1,k))
      call vplusv(t3,t3,t4)
      call vplusv(t3,t3,t(1,j3))
      call vplusv(t4,t2,tpp)
      call vminv(t5,t4,t3)
c
      do 40 l=1,3
 40     l3(l)=integer((12.*t5(l)),0)
      return
      end
c
c
***********************************************************************
c
c
      subroutine sittrn(n,j3,l3,k2,j2,l2,k1,j1,l1)
c
c     calls invsop, mltstl, msptpl, siteop
c
c     searches through =isptab(m,K1)= (m=N,nsym) to transform atom 2
c     in site K2, position J2, translation L2, with respect to atom 1
c     in site K1, position J1, translation L1, if possible.
c
c     output:     new position J3 and translation L3 (of atom 2)
c                 n = -8 or -9 if no transformation found
c
c     function calculated is
c
c      (s/t/l)3*(sp/tp)k2=(s/t/l)4*[(s/t/l)2*(sp/tp)k2]
c         where (s/t/l)4*[(s/t/l)1*(sp/tp)k1]=[(s/t/l)1*(sp/tp)k1]
c         and 4 = isym >= n
c
c     NB k1 and k2 are ialtno's
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      dimension l1(3),l2(3),l3(3),l4(3),l5(3),l6(3)
c
      if(n.le.0) n=1
      if(n.gt.nsym) then
        n=-9
        return
        endif
      do 10 i=n,nsym
        nn=i
        if(isptab(i,k1).eq.j1) goto 20
 10     continue
 5    n=-8
      return
c
 20   n=nn
      do 30 i=1,3
 30     l4(i)=0
c  j6/l6 is equivalent to j1/l1 with respect to k1
      call siteop(n,j6,l6,j1,l1,k1,l4)
      K=1
      call invsop(i,j5,l5,j1,l1,k,l4)
      if(i.le.0)then
        n=i
        return
        endif
      k=1
c  j4/l4 transforms the bond to its equivalent position
      call mltstl(j4,l4,j6,l6,j5,l5,k)
      call mltstl(j3,l3,j4,l4,j2,l2,k2)
      return
      end
c
c
***********************************************************************
c
c
      subroutine symtrn(n,j3,l3,i2,j2,l2,i1,j1,l1,yyx)
c
c     calls sittrn, symop
c
c     this subroutine gets the transformations for atom =i2= in origin
c     position =j2= (translations =l2=) to termial position =j3=
c     (translations =l3=) with respect to the point group of the
c     site of atom =i1= being in position =j1= (translations =l1=).
c
c     function calculated is
c
c      (s/t/l)3*(sp/tp)k2=(s/t/l)4*[(s/t/l)2*(sp/tp)k2]
c         where (s/t/l)4*[(s/t/l)1*(sp/tp)k1]=[(s/t/l)1*(sp/tp)k1]
c         and 4 = isym >= n
c
c     the routine searches through =isptab(l,ialtno(i1))= to find the symme-
c     try operators that transform =i1= onto itself and operate them on =i2=.
c
c     =n= input is the starting value for =l=
c
c     =n= output is the =l= found or -10 if end reached.
c
c     written by Daniel Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      include 'comatom.for'
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      dimension l1(3),l2(3),l3(3),yyx(3)
c
      if(ialtno(i1).le.0.or.ialtno(i2).le.0) then
        n=-10
        return
        endif
c
      call sittrn(n,j3,l3,ialtno(i2),j2,l2,ialtno(i1),j1,l1)
      if(n.le.0) return
c
      call symop(yyx,s(1,1,j3),t(1,j3),xx(1,i2))
      do 10 l=1,3
 10     yyx(l)=yyx(l)+l3(l)/12.
      return
      end
C
C
***********************************************************************
c
c
      FUNCTION DSTNCE(k,isk,itk,l,isl,itl)
c     ------------------------------------
c
c   written 91.7.31 by I.D.Brown
c
c   Calculates the distance between atoms k and l after symmetry
c    transformation is and translation it
c
c   calls MATHLB: dist,vmimv
c         SYMLB: symopl
c
      implicit character*8 (g)
c
      common /cell/ a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /sym/ nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      dimension xl(3),xk(3),dx(3),itk(3),itl(3)
      include 'comatom.for'
c
      call symopl(xk,s(1,1,isk),t(1,isk),itk,xx(1,k))
      call symopl(xl,s(1,1,isl),t(1,isl),itl,xx(1,l))
      call vminv(dx,xk,xl)
      dstnce = dist(dx,am)
      return
      end
c
c
***********************************************************************
C
C
      FUNCTION INMUL(I,J)
C
C     I AND J ARE THE NUMBERS OF SYMMETRY OPERATORS IN THE SPACE GROUP
C     MULTIPLICATION TABLE. THIS FUNCTION SEARCHES THE TABLE TO FIND
C
C     (IN SYMMETRY OPERATIONS:)                    I = J * INMUL
C     (I.E. IN MATRIX REPRESENTATION:)         S(I) = S(INMUL)*S(J)
C
C     INMUL = -30 FOR ERROR
C
C     WRITTEN BY I.D. BROWN, DEC. 1984
C
      COMMON /SYM/  NSYM,MSYM
      COMMON /SYMA/ MULT(48,48)
C
      DO 100 K=1,NSYM
        IF(MULT(J,K).EQ.I) THEN
          INMUL=K
          RETURN
          ENDIF
  100   CONTINUE
      INMUL=-30
      RETURN
      END
C
C
***********************************************************************
C
C
      FUNCTION INMULT(I,J)
C
C     I AND J ARE THE NUMBERS OF SYMMETRY OPERATORS IN THE SPACE GROUP
C     MULTIPLICATION TABLE. THIS FUNCTION SEARCHES THE TABLE TO FIND
C
C     (IN SYMMETRY OPERATIONS:)                    I = INMULT * J
C     (I.E. IN MATRIX REPRESENTATION:)         S(I) = S(J)*S(INMULT)
C
C     INMULT = 0 FOR ERROR
C
C     WRITTEN BY I.D. BROWN, DEC. 1984
C
      COMMON /SYM/  NSYM,MSYM
      COMMON /SYMA/ MULT(48,48)
C
      DO 100 K=1,NSYM
        IF(MULT(K,J).EQ.I) THEN
          INMULT=K
          RETURN
          ENDIF
  100   CONTINUE
      INMULT=0
      RETURN
      END
c
c
***********************************************************************
c
c
      function isymch(i,j,isp)
c
c     calls chebon
c
c     checks if the symmtery operator number =isp= is an element of the
c     point group of the site of atom =i= or if atom =i= and =j= are
c     bonded to the same atom =k= and =isp= is an element of the point
c     group of =k='s site
c
c     isymch = 0   if =isp= isn't element in any of the point groups
c              1   for a match
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      include 'comatom.for'
      include 'combond.for'
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
c
      isymch=0
      if(isptab(isp,ialtno(i)).eq.1) then
        isymch=1
        return
        endif
      nn=1
      mm=1
10    call chebon(i,nn,nbon,ibon)
      if(nn.eq.0) return
20    call chebon(j,mm,nbon,ibon)
      if(mm.eq.0) then
        mm=1
        goto 10
        endif
      if(ibon(1,nn).ne.ibon(1,mm)) goto 30
      if(isptab(isp,ialtno(ibon(1,nn))).eq.1) isymch=1
      return
30    if(jbon(1,nn).ne.jbon(1,mm)) goto 20
      if(isptab(isp,ialtno(jbon(1,nn))).eq.1) isymch=1
      return
      end
c
c
***********************************************************************
c
c
      subroutine getstl(lt,i,yy,isp)
c
c     calls symop, vminv, integer
c
c     gets LT = 12 * integer(YY - (S/T)isp * XXi)
c     for the (S/T/L)-representation of the coordinates YY
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      include 'comatom.for'
      common /sym/   ns,ms,s(3,3,48),t(3,48)
c
      dimension x(3),lt(3),yy(3)
c
      call symop(x,s(1,1,isp),t(1,isp),xx(1,i))
      call vminv(x,yy,x)
      do 10 j=1,3
 10     lt(j)=integer((12.*x(j)),0)
      return
      end
C
C
***********************************************************************
C
C
      SUBROUTINE SYMOP(Y,S,T,X)
C     ----------------
C
C   WRITTEN 84.2.8 BY I D BROWN
C
C   APPLIES THE SEITZ MATRIX S:T TO X TO GIVE Y
C
      DIMENSION X(3),Y(3),S(3,3),T(3)
      DO 50 I=1,3
        Y(I)=0.
        DO 20 J=1,3
  20      Y(I)=Y(I)+S(I,J)*X(J)
        Y(I)=Y(I)+T(I)
   50   CONTINUE
      RETURN
      END
c
c
***********************************************************************
c
c
      subroutine symopl(y,s,t,l,x)
c
c     calls symop
c
c     does Y = (S/T/L) * X
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      dimension y(3),s(3,3),t(3),l(3),x(3)
c
      call symop(y,s,t,x)
      do 10 i=1,3
 10     y(i)=y(i)+l(i)/12.
      return
      end
c
c
***********************************************************************
c
c
      subroutine symtem(uu,s,u)
c
c   written 87.10.28 by I.D.Brown
c
c   this transforms the atomic displacement parameters U by the symmetry
c   matrix S to give UU
c          _                _
c     UU = S * U * S  where S = transpose of S
c
      dimension uu(3,3),u(3,3),s(3,3),tp(3,3),ut(3,3)
c
      call transp(tp,s,3)
      call mxm(ut,u,tp,3)
      call mxm(uu,s,ut,3)
      return
      end
c
c
***********************************************************************
c
c
      subroutine symeq(xx,xy,isym,trans)
c
c   written 87.9.1 by I.D.Brown
c
c   this routine returns the symmetry operator (isym) and the translations
c   trans(3) in fractions of the unit cell that must be applies in
c   order to transform the coordinates xy into xx.  If xy cannot be
c   transformed into xx by any of the symmetry operations stored in
c   /sym/, then isym=0
c
c   calls MATHLB:jtest
c         SYMLIB:symop
c
      common/sym/nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      dimension xx(3),xy(3),xz(3),xa(3),xl(3),trans(3)
c
      isym=0
      do 110 k=1,3
  110   trans(k)=0.0
      do 180 i=1,nsym
        call symop(xz,s(1,1,i),t(1,i),xy)
        do 170 j=1,nlt
          do 160 l1=-2,2
            xl(1)=tu(1,j)+l1
            do 150 l2=-2,2
              xl(2)=tu(2,j)+l2
              do 140 l3=-2,2
                xl(3)=tu(3,j)+l3
                do 120 k=1,3
  120             xa(k)=xz(k)+xl(k)
                if(jtest(xx,xa).eq.0) then
                  isym=i
                  do 130 k=1,3
  130               trans(k)=xl(k)
                  return
                  endif
  140           continue
  150         continue
  160       continue
  170     continue
  180   continue
      return
      end
c
c
***********************************************************************
c
c
      subroutine symrdc(nt,nat)
c
c   written 87.9.1 by I.D.Brown
c
c   removes symmtery related atoms from the /xatom/ and /bonds/ array
c   speifically for file from the Cambridge Database (CRYSTOR)
c
c   nt is the total number of atoms read
c   nat is the number of these in the asymmetric unit
c     these must all appear in the list BEFORE the symmtery related ones
c
c   this routine calls:
c      SYMLIB:symeq
c      MATHLB:integer
c
      implicit character *8 (g)
      include 'comatom.for'
      include 'combond.for'
c       the dimensions of the next statement is MX=1000
      dimension ismscf(1000),trscf(3,1000),iscf(1000),trans(3)
c
c   the next statement allows one to access only the atoms in the
c   asymmetric unit
c
      nx=nat
c
c   identify the symmetry operations needed to generate the nt-nat
c   remaining atoms
c
      do 115 i=1,nt
        ismscf(i)=0
        do 110 k=1,3
  110     trscf(k,i)=0.
  115   continue
      do 190 i=1,nt
        if(i.le.nx) go to 175
        do 170 j=1,nx
          if(gelem(i).eq.gelem(j)) then
            call symeq(xx(1,i),xx(1,j),isym,trans)
            if(isym.ne.0) then
c             the ith atom is related by symmetry to the jth atom
              ismscf(i)=isym
              do 160 k=1,3
  160           trscf(k,i)=trans(k)
              iscf(i)=j
              go to 190
              endif
            endif
  170     continue
c       the ith atom is not related to any other
  175   ismscf(i)=1
        do 180 k=1,3
  180     trscf(k,i)=0.
        iscf(i)=i
  190   continue
c
c   label the symmetry related atoms that form bonds
c
      do 250 i=1,nbon
        if(ibon(1,i).gt.nx) then
          j=ibon(1,i)
          ibon(1,i)=iscf(j)
          ibon(2,i)=ismscf(j)
          do 210 k=1,3
  210       ibon(k,i)=integer(12.*trscf(k,j),0)
          endif
        if(jbon(1,i).gt.nx) then
          j=jbon(1,i)
          jbon(1,i)=iscf(j)
          jbon(2,i)=ismscf(j)
          do 220 k=1,3
  220       jbon(k,i)=integer(12.*trscf(k,j),0)
          endif
  250   continue
      return
      end
c
c
***********************************************************************
c
c
      subroutine mulstl(s3,t3,l3,s2,t2,l2,s1,t1,l1,nlt,tu)
c
c     calls integer, mxv, symult, vplusv
c
c     calculates symmetry operator products in (S/T/L)-format
c
c     (S/T/L)3 = (S/T/L)2 * (S/T/L)1
c
c     L3 is returned with respect to the NLT lattice translations in TU
c
c     written by D. Altermatt
c     McMaster University, March 1985
c
      dimension s1(3,3),s2(3,3),s3(3,3),t1(3),t2(3),t3(3),l1(3),l2(3)
      dimension l3(3),tu(3,nlt),xl1(3),xl2(3),xl3(3),t4(3)
c
      call symult(s3,t3,s2,t2,s1,t1)
      do 10 i=1,3
        xl1(i)=l1(i)/12.
 10     xl2(i)=l2(i)/12.
      call mxv(t4,s2,xl1)
      do 20 j=1,2
        do 30 i=1,3
          if(t3(i).gt.0.9999) then
            t3(i)=t3(i)-1.0
            t4(i)=t4(i)+1.0
          else if(t3(i).lt.-.0001) then
            t3(i)=t3(i)+1.0
            t4(i)=t4(i)-1.0
            endif
 30       continue
 20     continue
c
      if(nlt.le.1) goto 60
      do 40 k=2,nlt
        if(integer(t3(1),3).ge.integer(tu(1,k),3).and.integer(t3(2),3).
     1    ge.integer(tu(2,k),3).and.integer(t3(3),3).ge.integer
     2    (tu(3,k),3)) then
          do 50 l=1,3
            t3(l)=t3(l)-tu(l,k)
 50         t4(l)=t4(l)+tu(l,k)
          endif
 40     continue
c
 60   call vplusv(xl3,xl2,t4)
      do 70 i=1,3
 70     l3(i)=integer((12.*xl3(i)),0)
      return
      end
C
C
***********************************************************************
C
C
      SUBROUTINE SYMULT(S3,T3,S2,T2,S1,T1)
C     -----------------
C
C   WRITTEN 84.2.8 BY I D BROWN
C
C   CALLS MXM, MXV, VPLUSV
C
C   CALCULATES (S/T)3=(S/T)2*(S/T)1
C
      DIMENSION S1(3,3),S2(3,3),S3(3,3),T1(3),T2(3),T3(3),T4(3)
      I30 = 3
      CALL MXM(S3,S2,S1,I30)
      CALL MXV(T4,S2,T1)
      CALL VPLUSV(T3,T4,T2)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE GRMULT(IER)
C     -----------------
C
C   WRITTEN 84.1.13 BY I.D.BROWN
C
C   CALLS MXM AND ICOMMA
C
C***** CALLS TIME-FUNCTION =SECOND(I)=
C
C   THIS ROUTINE CALCULATES THE POINT GROUP MULTIPLICATION TABLE "MULT"
C   FROM THE ARRAY "S" IN /SYM/
C
      COMMON /FILES/ NIN,NOUT,NDEBUG
      COMMON /SYM/   NSYM,MSYM,S(3,3,48)
      COMMON /SYMA/  MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30)
C
      DIMENSION SS(3,3)
C
      IER=0
      DO 200 I=1,NSYM
        DO 150 J=1,NSYM
          I30 = 3
          CALL MXM(SS,S(1,1,J),S(1,1,I),I30)
          DO 100 K=1,NSYM
            IF(ICOMMA(S(1,1,K),SS,3,2).EQ.0) THEN
              MULT(I,J)=K
              GO TO 150
              ENDIF
  100       CONTINUE
          WRITE(NOUT,1000) I,J
 1000     FORMAT(' ++ERROR++GRMULT++1000++ NO MATCH FOR PRODUCT OF'
     +     ,' SYMMETRY OPERATORS ',2I3)
          IF(NDEBUG.NE.0) WRITE(NDEBUG,1001) SS
 1001     FORMAT(' ++DEBUG++GRMULT++1001++ ',9F6.2)
          IER=1
          RETURN
  150    CONTINUE
  200 CONTINUE
      IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,1200) SECOND(3)
 1200   FORMAT(' ++DEBUG++GRMULT++1200++(',F6.2,') GROUP ',
     + 'MULTIPLICATION TABLE')
        DO 220 I=1,NSYM
  220     WRITE(NDEBUG,1201) (MULT(I,J),J=1,NSYM)
 1201     FORMAT(5X,40I3,/,10X,8I3)
        ENDIF
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE INVMUL(IB,IC,IA,IER)
C     -----------------
C
C   WRITTEN 84.1.16 BY I.D.BROWN
C
C   CALLS MXV, VPLUSV
C
C   THIS SUBROUTINE WILL FIND IB(4) GIVEN IA(4) AND IC(4) IN THE SYMMETRY
C   EQUATION  IC = IB*IA
C   IA(1) = ROTATION MATRIX NUMBER
C   IA(2-4) = 12*TRANSLATION
C
      COMMON /FILES/ NIN,NOUT,NDEBUG
      COMMON /SYM/   NSYM,MSYM,S(3,3,48),T(3,48)
      COMMON /SYMA/  MULT(48,48)
C
      DIMENSION IA(4),IB(4),IC(4),X(3),Y(3),XA(3),XC(3)
      IER=0
      IA1=IA(1)
      IC1=IC(1)
      DO 50 I=1,3
        XA(I)=IA(I+1)
   50   XC(I)=IC(I+1)
C
C   FIND ROTATION MATRIX BY LOOKING IN MULT
C
      DO 100 I=1,NSYM
        IF(MULT(IA1,I).EQ.IC1) THEN
          IB(1)=I
          IB1=I
           GO TO 200
          ENDIF
  100    CONTINUE
        WRITE(NOUT,1000) IC(1),IA1
 1000   FORMAT(' ++ERROR++INVMUL++1000++ UNABLE TO FIND INVERSE ',
     +   'SYMMETRY OPERATOR FOR ',2I3)
        IER=1
        RETURN
C
C   FIND TRANSLATION OPERATOR
C
  200 CALL MXV(X,S(1,1,IB1),XA)
      CALL VPLUSV(Y,X,XC)
      DO 250 I=1,3
  250   IB(I+1)=Y(I)
      RETURN
      END
C
C
***********************************************************************
C
C
      SUBROUTINE ISMULT(ISC,ISB,ISA)
C     -----------------
C
C   WRITTEN 84.2.13 BY I D BROWN
C
C   MULTIPLIES TWO SYMMETRY OPERATORS STORED IN THE SHORT FORM
C       ISC(4)=ISB(4)*ISA(4)
C
      COMMON /SYM/  NSYM,MSYM,S(3,3,48)
      COMMON /SYMA/ MULT(48,48)
C
      DIMENSION ISC(4),ISB(4),ISA(4)
C
      IA=ISA(1)
      IB=ISB(1)
      ISC(1)=MULT(IA,IB)
      DO 100 I=1,3
        ISC(I+1)=0
        DO 50 J=1,3
   50     ISC(I+1)=ISC(I+1)+S(I,J,IB)*ISA(J+1)
        ISC(I+1)=ISC(I+1)+ISB(I+1)
  100   CONTINUE
      RETURN
      END
c
c
***********************************************************************
c
c
      subroutine conxyz(sp,tp,hbuf)
c
c     written by D. Altermatt
c     McMaster University, Nov. 1984
c
c     calls integer
c
c     for a given operator (sp/tp) this subroutine constructs the
c     general representation with x,y,z as in the International
c     Tables for Crystallograpy and as required by DLS
c
      implicit character*80 (h)
      character*8 g1(12),g2(12),g3(12),g4(12)
c
      common/files/nin,nout,ndebug
      dimension sp(3,3),tp(3)
      data g1/'1/8','1/4','3/8','1/2','5/8','3/4','7/8','1',4*'   '/
      data g2/'1/12','1/6','1/4','1/3','5/12','1/2','7/12','2/3',
     1        '3/4','5/6','11/12','1'/
      data g3/'9/8','5/4','11/8','3/2','13/8','7/4','15/8','2',4*' '/
      data g4/'13/12','7/6','5/4','4/3','17/12','3/2','19/12','5/3',
     1        '7/4','11/6','23/12','2'/
c
      if(ndebug.ne.0) write(ndebug,1000) hbuf
 1000   format(' *debug*conxyz*1000* ',A26)
      hbuf=' '
      hline=' '
      do 10 i=1,3
        j=(i-1)*17+1
        if((abs(sp(i,1))+abs(sp(i,2))+abs(sp(i,3))+abs(tp(i))).lt.
     1    0.01) then
c   --    i-th coordinate 0
          hline(j:j)='0'
          if(i.ne.3) hline(j+1:j+1)=','
          goto 10
          endif
        if(integer(sp(i,1),0).eq.0) goto 20
c   --    X-component
          hline(j+1:j+1)='X'
          if(sp(i,1).gt.1.01) hline(j:j)='2'
          if(sp(i,1).lt.-0.1) hline(j:j)='-'
20      if(integer(sp(i,2),0).eq.0) goto 30
c   --    Y-component
          hline(j+4:j+4)='Y'
          if(sp(i,2).gt.1.01) hline(j+3:j+3)='2'
          if(sp(i,2).lt.-0.1) hline(j+2:j+2)='-'
          if(hline(j+1:j+2).eq.'X ') hline(j+2:j+2)='+'
30      if(integer(sp(i,3),0).eq.0) goto 40
c   --    Z-component
          hline(j+7:j+7)='Z'
          if(sp(i,3).gt.1.01) hline(j+6:j+6)='2'
          if(sp(i,3).lt.-0.1) hline(j+5:j+5)='-'
          if(hline(j+5:j+5).eq.' '.and.hline(j:j+4).ne.'     ')
     1      hline(j+5:j+5)='+'
40      if(integer(tp(i),3).eq.0) goto 50
c   --    translation
          if(hline(j:j+7).ne.'       ') hline(j+8:j+8)='+'
          if(tp(i).lt.-0.1) hline(j+8:j+8)='-'
          do 45 k=1,12
            xk=k
            k1=k*125
            k2=integer((xk*83.33),0)
            if(iabs(integer(tp(i),3)).eq.k1)
     1        hline(j+9:j+14)=g1(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.(k1+1000))
     1        hline(j+9:j+14)=g3(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.k2)
     1        hline(j+9:j+14)=g2(k)(1:6)
            if(iabs(integer(tp(i),3)).eq.(k2+1000))
     1        hline(j+9:j+14)=g4(k)(1:6)
45          continue
50        if(i.ne.3) hline(j+15:j+15)=','
10      continue
c
c     remove blanks, store in hbuf
c
      j=0
      do 60 i=1,80
        if(hline(i:i).ne.' ') then
          j=j+1
          hbuf(j:j)=hline(i:i)
          endif
   60   continue
      return
      end
C
C
***********************************************************************
C
C
      function ibonch(iata,jata,iatb,jatb,idir,isym,itt,iv)
c
c   written 87.1.7 by I.D.Brown
c
c     ibonch=0 if the bond iata-jata is equivalent to the bond iatb-jatb
c     else ibonch=1
c     idir=1 if iata corresponds to iatb and jata corresponds to jatb
c     idir=-1 if iata corresponds to jatb and jata corresponds to iatb
c     if bonch=1, idir is used to indicate the return point 
c     isym is the symmetry operator to convert bond b to a
c     itt is the corresponding lattice centering operation (in tu)
c     iv is the corresponding unit lattice translation (in twelfths of cell)
c
c   calls grmult,mltstl
c   calls mathlb:ivpluv,ivminv,jcomve,integer
c
      include 'comatom.for'
      common/sym/nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      common/syma/ mult(48,48),it(3,48)
      dimension iata(5),jata(5),iatb(5),jatb(5),iatc(5),jatc(5)
      dimension iatd(5),jatd(5),iate(3),jate(3)
      dimension itu(3,4),iv(3),jv(3),nul(3)
      data nul/0,0,0/
c
      ibonch=1
      if(mult(1,1).eq.0) call grmult(ier)
      do 50 i=1,nlt
        do 40 k=1,3
          itu(k,i)=integer(12*tu(k,i),0)
   40     continue
   50   continue
c
c   find if terminal atoms match
c   ----------------------------
      if(iata(1).eq.iatb(1).and.jata(1).eq.jatb(1)) then
        do 120 k=1,5
          iatc(k)=iatb(k)
          jatc(k)=jatb(k)
  120     continue
        idir=1
        go to 200
      else if(iata(1).eq.jatb(1).and.jata(1).eq.iatb(1)) then
        do 130 k=1,5
          iatc(k)=jatb(k)
          jatc(k)=iatb(k)
  130     continue
        idir=-1
      else
        idir=2
        return
        endif
c
c   find if symmetries match
c   ------------------------
  200 do 220 k=1,5
        iatd(k)=iatc(k)
        jatd(k)=jatc(k)
  220   continue
      do 390 i=1,nsym
        call mltstl(iatd(2),iate,i,nul,iatc(2),iatd(3),
     +                           ialtno(iatc(1)))
        call mltstl(jatd(2),jate,i,nul,jatc(2),jatd(3),
     +                           ialtno(jatc(1)))
        if(iata(2).eq.iatd(2).and.jata(2).eq.jatd(2)) then
c
c find if translations match
c --------------------------
          do 370 j=1,nlt
            call ivpluv(iatd(3),itu(1,j),iate)
            call ivpluv(jatd(3),itu(1,j),jate)
            call ivminv(iv,iata(3),iatd(3))
            call ivminv(jv,jata(3),jatd(3))
            if(jcomve(iv,jv,3).eq.0) then
              do 360 k=1,3
                do 350 l=1,5
                  if((iv(k)-(l-3)*12).eq.0) go to 360
  350             continue
                  idir=4
                  return
  360           continue
c
c  we have a match!
c  ----------------
              ibonch=0
              itt=j
              isym=1
              return
              endif
  370       continue
          endif
  390   continue
c
c In the case where both atoms are the same we must check the other way
c ---------------------------------------------------------------------
      if(idir.eq.1.and.iata(1).eq.jatb(1).and.
     +                  jata(1).eq.iatb(1)) then
        do 230 k=1,5
          iatc(k)=jatb(k)
          jatc(k)=iatb(k)
  230     continue
        idir=-1
        go to 200
        endif
      idir=5
      return
      end
