c     FILE VISLS                                   Last update 1993.5.14
C     ==========                                             I.D.Brown
c
c  ------------------------------------------------------------------
c                 Copyright 1991     I.D.Brown
c  ------------------------------------------------------------------
c
c  ****************************************************
c   THIS FILE CONTAINS DOUBLE PRECISION ARRAYS
c  ****************************************************
c
c    contains routines for calculating valence least sqaures
c
c    This file call the following routines:
c              CHEMLB: bost,ibost,dbost,valsum
c              CERNLB: deqinv
c              DISTLB: calcon,indepb,bocole,corget
c              MATHLB: readyn,wait,maxdim,sqart,icomma,integer,vminv,dist,intread
c              MAINLB: param
c              NETLIB: kirch
c              SPANNER: expand,jyat
c              SYMLIB: symopl,dstnce
c
c   This file contains the following routines:
c    visls                  calling routine for VISLS
c    vishlp                 help file
c    vagree(npar)           calculates agreement index
c    selpar(*,npar,mpar,ipar) select parameters to be refined
c    seleq                  select equations to be used
c    selsum                 set equations for valence sums
c    selval                 set equations for bond valences
c    selbon                 set equations for bond lengths
c    selcon                 set equations for nonbond distances
c    igcomm(g1,g2,n)        = 0 if g1(1:n).eq.g2(1:n) [to go to mathlb]
c    decieq(i,ity,isy,ij)   decompose the flag ieq
c    bonout                 output new bonds
c    dissel                 display selected conditions
c    altsel                 alter selected conditions
c    caldif                 calculate the differences
c    makmat                 make the matrix
c    dervs(iat,ico,k,isk,itk,l,isl,itl) calculates derivative of valence
c    derc(ico,k,isk,itk,l,isl,itl) derivative wrt cell
c    derx(ico,k,isk,itk,l,isl,itl) derivative wrt xyz
c    newxyz(damp,xsh)       update the coordinates
c
c
c
      SUBROUTINE VISLS
c     ----------------
c
c   written 91-8-1 by I.D.Brown
c
c  does a least square fit of unit cell and coordinates to a variety
c  of chemical constraints (e.g. valence sum rule, target valences,
c  target distances)
c
c  calls: CERNLB: deqinv
c         DISTLB: calcon,indepb,bocole,bcout
c         MATHLB: readyn
c         VISLS:  vishlp,seleq,makmat,newxyz,selpar,vagree
c
c   irfl records the state of the refinemt.  Initially it is 0
c        after a successful refinement it is 2
c        after an unsuccessful refinement it is 1
c
      implicit character*8 (g)
      implicit character*80 (h)
c
c   DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
c
      common /matrixx/ nm,mm,xm,v,im(500,500),bwgt(500),cwgt(500)
      common /files/ nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      common /gmat/ gx(3,1000),gcell(6)
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      dimension wk(500),ipar(500)
      meq = 200
      mpar = 500
c
c        mx, the dimensions of the atom array, are the dimensions of GX
      if(mx.gt.1000) then
        write(nout,1005) mx
 1005   format(//,' *ERROR*VISLS*1005* Increase the dimensions of',
     1    ' array GX to ',I6,/,
     2    ' The calculation will be aborted')
        return
        endif 
      irfl = 0
      xsh = 0.25
      neq = 0
      leq = 0
c
      write(nout,1002)
  100 write(nout,1003)
 1002 format(///,'        Welcome to VISLS',/,
     1           '        ----------------',
     2 /,'   This module allows the fitting of the unit cell and',
     2 ' atomic coordinates ',/,
     3 ' (the parameters) to various chemical constraints ',
     3 '(the conditions)',/,
     4 ' Type 2 to select the parameters you wish to refine,',
     4 ' then type 3 to select',/,
     5 ' the conditions and 4 to start the refinement',//)
 1003 format(' Menu 3.15: Main VISLS Menu',/,
     1       ' ---------',/,
     1 '     1. Help',/,
     2 '     2. Select coordinates to be refined',/,
     3 '     3. Select constraints to be applied',/,
     4 '     4. Perform the refinement',///,
     3 '    20. Return to Menu level 3',//
     9 ' ENTER YOUR CHOICE ',$)
      ihcr = intread()
      if(ihcr.lt.1.or.ihcr.gt.20) go to 100
c
c  Help
c  ----
      if(ihcr.eq.1) call vishlp
c
c   select parameters
c   -----------------
      if(ihcr.eq.2) then
        irfl = 0
        npar = 0
        call selpar(*100,npar,mpar,ipar)
        endif
c
c   Select conditions
c   -----------------
      if(ihcr.eq.3) then
        if(nbon.le.0) then
          write(nout,1125)
 1125     format(' No bonds are stored.  Calculate the bonds ',
     1      'and try again')
          return
          endif
        irfl = 0
        i1 = 1
        if(ind1.lt.nbon) call indepb(i1,i1,nbon)
        call seleq(npar)
        endif
c
c   Perform refinement
c   ------------------
      if(ihcr.eq.4) go to 290
c
c   Return
c   ------
      if(ihcr.eq.20) return
      go to 100
c
c   Start refinement, check parameters and conditions are set
c   ---------------------------------------------------------
  290 if(npar.eq.0) then
        write(nout,1290)
 1290   format(' No parameters selected.  Use menu item 2.')
        go to 100
        endif
      if(neq.eq.0) then
        write(nout,1295)
 1295   format(' No conditions selected.  Use menu item 3.')
        go to 100
        endif
C
c   copy ipar to im (NB nm and xm were needed in network prediction of 
c   bond lengths)
c   -------------
      if(npar.gt.mm) then
        write(nout,1298) mm
 1298   format(' *warning*visls*1298* More than ',I5,' parameters'
     1   ' selected, remainder ignored')
        npar = mm
        endif
      nm = npar
      do 298 i=1,npar
        im(i,1) = ipar(i)
  298   continue
c
c   calculate the differences and agreement index
c   ---------------------------------------------
      if(irfl.ne.2) then
        call caldif
        CALL vagree(npar)
        endif
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9000) NM,(IM(I,1),I=1,NM)
 9000 FORMAT(' *DEBUG*visls*9000* NM=',I4,/,10(2X,20I4,/))
      IRFL=1
c
c
c   construct the matrix
c   --------------------
      call makmat
C
C   Invert the matrix
C   -----------------
      WRITE(NOUT,1300)
 1300 FORMAT(//,' Inverting the matrix...',//)
      IF(NDEBUG.NE.0) THEN
        WRITE(NDEBUG,9310) (IM(I,1),I=1,NM)
 9310   FORMAT(' *DEBUG*visls*9310* ',6X,17(10I9,/,26X))
        DO 310 I=1,NM
  310     WRITE(NDEBUG,9301) IM(I,1),(XM(I,J),J=1,NM)
 9301     FORMAT(' *DEBUG*visls*9301*  ',I5,17(10E9.2,/,26X))
        WRITE(NDEBUG,9320) (V(I),I=1,NM)
 9320   FORMAT(' *DEBUG*visls*9320* V(ORG)',17(10E9.2,/,26X))
        ENDIF
      IF(NM.GT.neq) then
        WRITE(NOUT,1301)NM,neq+1
 1301   FORMAT(/,' Warning: You are refining ',I4,' paramters using',
     +   /,' less than ',I4,' conditions.  ',
     +  'Your matrix will be singular!')
      else if(nm.gt.leq) then
        write(nout,1302) nm-neq
 1302   format(' If the matrix is singular, you may have to add',
     +  ' up to ',I3,/,
     2  ' more conditions')
        endif
      ISOL=1
      CALL DEQINV(NM,XM,MM,WK,IER,ISOL,V)
      IF(IER.NE.0) THEN
        if(ier.eq.-1) WRITE(NOUT,1305)
 1305   FORMAT(//' **ERROR**visls**1305** MATRIX SINGULAR',//)
        if(ier.eq.-2) write(nout,1303)
 1303   format(//' **ERROR**visls**1303** BADLY FORMED MATRIX',//)
        if(ier.lt.-2.or.ier.gt.0) write(nout,1304)
 1304   format(//' **ERROR**visls**1304** ERROR IN MATRIX INVERSION',
     +                                   //)
        IRFL=1
        call wait
        go to 100 
        endif
      irfl = 2
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9300) (V(I),I=1,NM)
 9300 FORMAT(' *DEBUG*visls*9300* V(INV)',17(12F7.3,/,26X))
C
C   Request damping parameters and cutoffs, calculate new parameters
C   ----------------------------------------------------------------
      DAMP=1.
  330 WRITE(NOUT,1330)
 1330 FORMAT(/,' This program automatically introduces an overall ',
     +  'damping factor ',/,' in case of large shifts.',//,
     +  ' Do you wish to introduce further damping (Y/N/E): ',$)
      CALL READYN(*335,*350,*900,*330)
  335 WRITE(NOUT,1332)
 1332 FORMAT(/,' Enter overall damping factor (1.0>DF>10**-5): ',$)
      READ(NIN,*,ERR=330) DAMP
      IF(DAMP.GT.1.0) DAMP = 1.0
      IF(DAMP.LT.0.00001) DAMP = 0.00001
  350 CONTINUE
  360 WRITE(NOUT,2360) XSH
 2360 FORMAT(/,' The maximum shift per coordinate is currently set',
     +  ' to',F5.2,' Angstroms.',//,' Do you want a different',
     +  ' cutoff (Y/N/E): ',$)
      CALL READYN(*370,*380,*900,*360)
  370 WRITE(NOUT,2370)
 2370 FORMAT(' Enter cutoff for coordinate shifts (.001<cut<3.00 A): '
     +  ,$)
      READ(NIN,*,ERR=370) XSH
      IF(XSH.LT.0.001) XSH=0.001
      IF(XSH.GT.3.00)  XSH=3.00
  380 CALL newxyz(DAMP,XSH)
c
c   calculate and store new bond lengths in /bonds/ arrays
c
      I10 = 1
      CALL BOCOLE(I10,I10,NBON)
      I10 = 1
      I20 = 2
      CALL BOCOLE(I20,I10,NCON)
      call caldif
      CALL vagree(npar)
      irfl = 2
  400 WRITE(NOUT,1400)
 1400 FORMAT(//,' Do you want a listing of the',
     +  ' new distances (Y/N/E): ',$)
      CALL READYN(*420,*425,*900,*400)
  420 CALL bonout
  425 write(nout,1425)
 1425 format(//,' Select 2, 3 or 4 to perform another iteration',
     1 ' or 20 to exit',/)
      go to 100
c
  900 return
      end
c
c
c
      SUBROUTINE VISHLP
c
      common/files/ nin,nout,ndebug
c
      write(nout,1000)
 1000 format('          VISLS (Valence and dIStance Least Squares)',/,
     1       '         ------------------------------------------',//,
     3'    This routine refines atomic and cell parameters by least',/,
     4' squares against selected geometric and valence conditions. ',/,
     5' At the first menu type 2 to select the parameters to be',/,
     6' refined, 3 to select the conditions to be satisfied by the',/,
     7' refinement and 4 to begin one cycle of refinement.  Further',/,
     8' cycles of refinement can be performed by typing 4 each time',/,
     9' this menu appears.',/,
     +'    Typing 2 will prompt you for the parameters that are to',/,
     1' be refined - unit cell and atomic coordinates.  Parameters',/,
     2' determined by symmetry are automatically fixed and you may',/,
     3' optionally fix any of the others. ')
      call wait
      write(nout,1001)
 1001 format(
     1'  Typing 3 will present a menu of the possible conditions',/,
     2' that the refinement will attempt to satisfy.  These are: 1.',/,
     3' fit the bond valence sums around each atom to the atomic',/,
     4' valence; 2. fit the individual bond valences to selected',/,
     5' values (atomic valence / coordination number); 3. fit the',/,
     6' bond lengths to the selected values (bond lengths predicted',/,
     7' by the valence network equations, see Menu 3.7); and',/,
     8' 4. fit the non-bonded contacts to selected values ',/,
     9' (target values within each coordination sphere are',/,
     +' generated by the program).  Weights generated by the',/,
     1' program are based on standard deviations of 0.05 v.u.',/,
     2' for bond valence sums, 0.10 v.u. for bond valences,',/,
     2' 0.032/(bond valence) A for bond distances and 0.5 A',/,
     3' for contacts in which the target is less than the distance,',/,
     4' otherwise the weight is zero.  In each case, the target ',/,
     5' values and weights can be modified and, by selecting zero',/,
     6' weight, can be removed from the calculation.  ')
      call wait
      write(nout,1002)
 1002 format(
     1'   Typing 4 will begin the least squares refinement.  If the',/,
     2' number of parameters being refined exceeds the number of',/,
     3' independent conditions, the matrix will be singular.  In',/,
     4' this case, either add more conditions or fix some of the ',/,
     5' parameters. The progress of the refinement can be',/,
     6' followed by changes in the agreement index (goodness',/,
     7' of fit) but the absolute value of the agreement index',/,
     8' may not be significant because of the difficulty',/,
     9' of estimating the number of conditions that are',/,
     +' independent.  Large shifts are automatically truncated,',/,
     1' but you will be asked if you wish to change the',/,
     2' truncation limit or introduce a damping factor, useful',/,
     3' if the refinement oscillates.  The new values of',/,
     4' the parameters and their changes are shown on the screen.')
      call wait
      write(nout,1003)
 1003 format(
     1'   You may call for further rounds of refinement by typing 4',/,
     2' at the main menu with or without a change in the selection',/,
     3' of parameters (type 2) or conditions (type 3).  After each',/,
     4' refinement, the new unit cell and atomic coordinates',/,
     5' replace the set previously held in memory.')
      call wait
      return
      end
c
c
c     
      SUBROUTINE VAGREE(npar)
c     -----------------
c
c  written 91-8-8 by I.D.Brown
c
c  calculates an agreement index for the refinement
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common/files/nin,nout,ndebug
      common/vils/neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
c
      sum = 0
      do 100 i=1,neq
        sum = sum + abs(eqwgt(i))*(diff(i))**2
  100   continue
      if(leq.gt.nm) then
        sum = sqart(sum/(leq-npar))
      else
        sum = sqart(sum)
        endif
      write(nout,1100) sum
 1100 format('   The agreement, {sum[(wgt*diff)**2]/(nc-np)}**0.5',
     1        '  = ',F6.3)
      return
      end
c
c
c
c
c
      SUBROUTINE SELPAR(*,npar,mpar,ipar)
c    ---------------------------------------
C
C    Written 85.2.25 by I.D.Brown as PAREF in DISLS
C    Updated in April 85 D.Altermatt
c    Moved to VISLS as SELPAR 91.8.9 by I.D.Brown
C
C    Determine which atomic coordinates will be refined
C
C    IM(*,1) contains a description of the variables to be
C    refined in integer form
C
C       IM=1,6    cell constants
C       IM=11,12,13  x,y,z for atom 1
C       IM=21,22,23  x,y,z for atom 2 etc.
C
C    CALLS ICOMMA,INTEGER, READYN
C
      IMPLICIT CHARACTER*8 (G)
      implicit character*80 (H)
c
c   mpar is the dimension of GX
c
      COMMON /FILES/  NIN,NOUT,NDEBUG
      include 'comatom.for'
      common /gsym/ glat,gcent,hmsym,hall,gsysp(2,30)
      COMMON /SYMA/   MULT(48,48),IT(3,48),NSP,MSP,ISPTAB(48,30),
     1                MULTSP(30),SP(3,3,30),TP(3,30)
      COMMON /GMAT/   GX(3,500),gcell(6)
      common /cellsym/ iax,iaxx,iayx
C
      dimension ipar(mpar)
      DIMENSION GY(6),TMAT(3,3)
      DATA GY/'a','b','c','Alpha','Beta','Gamma'/
      data TMAT/0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0/
C
      DO 10 I=1,3
        gcell(I)=' '
        gcell(I+3)=' '
        DO 10 J=1,nx
          GX(I,J)='F'
   10     continue
      do 11 i=1,mpar
        ipar(i)=0
   11   continue
c----------------------------------------------------
c   Find the fixed parameters for the unit cell
c ---------------------------------------------------
  200 write(nout,1200)
 1200 format(/,' Do you want to refine the cell parameters (Y/N/E) ')
      call readyn(*210,*20,*999,*200)
c
c   decode space group symbol to find symmetry-fixed lattice parameters
c   -------------------------------------------------------------------
  210 iaxx = 3
      iayx = 1
      mode = 0
c                  iax gives nature of first rotation axis
c                  iaxx gives direction of first rotation axis
c                  iayx gives direction of second rotation axis
      do 240 i=1,24
        if(hall(i:i).eq.'6') then
          mode = mode+1
          if(mode.eq.1) iax = 6 
        else if (hall(i:i).eq.'4') then
          mode = mode+1
          if(mode.eq.1) iax = 4
        else if (hall(i:i).eq.'3') then
          mode = mode+1
          if(mode.eq.1) iax = 3
          if(mode.gt.1) iax = 6
        else if (hall(i:i).eq.'2') then
          mode = mode+1
          if(mode.eq.1) iax = 2
          if(mode.eq.2.and.(iax.eq.3.or.iax.eq.6)) iayx = 5
        else if(hall(i:i).eq.'X') then
          if(mode.eq.1) iaxx = 1
          if(mode.eq.2) iayx = 1
        else if(hall(i:i).eq.'Y') then
          if(mode.eq.1) iaxx = 2
          if(mode.eq.2) iayx = 2
        else if(hall(i:i).eq.'Z') then
          if(mode.eq.1) iaxx = 3
          if(mode.eq.2) iayx = 3
        else if(hall(i:i).eq.'"') then
          if(mode.eq.1) iaxx = 4
          if(mode.eq.2) iayx = 4
        else if(hall(i:i).eq.'''') then
          if(mode.eq.1) iaxx = 5
          if(mode.eq.2) iayx = 5
        else if(hall(i:i).eq.'*') then
          if(mode.eq.1) iaxx = 6
          if(mode.eq.2) iayx = 6
          endif
  240   continue
      if(ndebug.ne.0) then
        write(ndebug,1240) iax,iaxx,iayx
 1240   format(' *debug*selpar*1240* iax =',I3,', iaxx =',I3,
     1               ', iayx =',I3)
        endif
c
c   set the values in GCELL and Ipar according to symmetry
c   ----------------------------------------------------
      if(npar.ge.mpar) go to 900
      npar = npar+1
      ipar(npar) = 1
c
c       1st case: cubic and rhombohedral
c       --------------------------------
      if(iayx.eq.6.or.iaxx.eq.6) then
        do 250 i=2,6
          gcell(i) = 'F'
  250     continue
        if(iax.eq.3) then
          if(npar.ge.mpar) go to 900
          npar = npar+1
          ipar(npar) = 4
          gcell(4) = ' '
          endif
c
c       2nd case: trigonal and tetragonal
c       ---------------------------------
      else if(iayx.ne.6.and.(iax.eq.6.or.iax.eq.4.or.iax.eq.3)) then
        if(npar.ge.mpar) go to 900
        npar = npar+1
        do 260 i=2,6
          gcell(i) = 'F'
  260     continue
        if(iaxx.eq.3) then
          gcell(3) = ' '
          ipar(npar) = 3
        else
          gcell(2) = ' '
          ipar(npar) = 2
          endif
c
c       3rd case:  orthorhombic or monoclinic
c       -------------------------------------
      else if(iax.eq.2) then
        do 270 i=4,6
          gcell(i) = 'F'
  270     continue
        if(npar.ge.mpar-1) go to 900
        do 275 i=2,3
          npar = npar+1
          ipar(npar) = i
  275     continue
        if(mode.eq.1.and.iaxx.le.3) then
          gcell(iaxx+3) = ' '
          if(npar.ge.mpar) go to 900
          npar = npar+1
          ipar(npar) = iaxx + 3
          endif
c
c       4th case: triclinic
c       -------------------
      else
        if(npar.ge.mpar-5) go to 900
        do 280 i=2,6
          npar = npar+1
          ipar(npar) = i
  280     continue
        endif
      if(ndebug.ne.0) then
        write(ndebug,1280) (gcell(i),i=1,6),(ipar(i),i=1,npar)
 1280   format(' *debug*selpar*1280* gcell =',6(A6,1X),', ipar =',6I3)
        endif
c
c   request for cell paramters to be fixed by user
c   ----------------------------------------------
  285 write(nout,1285)
 1285 format(/,' Do you want to fix any lattice parameters that ',
     1 'are not already',/,
     2 ' fixed by symmetry (Y/N/E) ')
      call readyn(*290,*20,*999,*285)
  290 do 295 i=1,npar
        write(nout,1291) gy(ipar(i))
 1291   format(' Do you wish to fix ',A5,' (Y/N/E) ')
        call readyn(*293,*295,*999,*285)
  293   gcell(ipar(i)) = 'R'
        ipar(i) = 0
  295   continue
  296 nnm = npar
      do 297 i=1,nnm
        if(ipar(i).eq.0) then
          do 298 j=i,nnm-1
            ipar(j) = ipar(j+1)
  298       continue
          npar = npar-1
          go to 296
          endif
  297   continue
c ---------------------------------------------------
c   request coordinate parameters to be refined
c ---------------------------------------------------
c                 irest=1/0 = fix/do not fix some paramters
c                 jrest=1/0 = fix/do not fix coordinates of current atom
   20 IREST=0
   25 write(nout,1004)
 1004 format(/,' Do you want to refine atomic coordinates (Y/N/E) ')
      call readyn(*30,*899,*999,*20)
   30 WRITE(NOUT,1000)
 1000 FORMAT(/,' Do you want to prevent the refinement ',
     +  'of any coordinates (Y/N/E): ')
      CALL READYN(*40,*50,*999,*30)
   40 write(nout,1001)
 1001 format(' Answering any of the following questions with E ',
     1'will cause all subsequent',/,
     2' parameters to be refined')
      IREST=1
   50 CONTINUE
      DO 130 I=1,NX
        JREST=0
        ISP=IALTNO(I)
        IF(ISP.LE.0.OR.NPOS(I).EQ.0.OR.ICOMMA(TMAT,SP(1,1,ISP),3,3)
     +    .EQ.0) GO TO 130
        IF(IREST.GT.0) THEN
  110     WRITE(NOUT,1002) GELEM(I),GDENT(I)
 1002     FORMAT(/,' Do you wish to restrict the refinement ',
     +           'of atom ',A2,1X,A6,' (Y/N/E): ',$)
          CALL READYN(*112,*113,*111,*110)
  111     IREST=0
          GOTO 113
  112     JREST=1
  113     CONTINUE
          ENDIF
        DO 120 J=1,3
          IF(INTEGER(SP(J,J,ISP),0).EQ.1) THEN
            GX(J,I)='R'
            IF(JREST.GT.0) THEN
  115         WRITE(NOUT,1003) GY(J)
 1003         FORMAT(' Prevent refinement of ',A1,' (Y/N/E) :',$)
              CALL READYN(*120,*117,*116,*115)
  116         jrest=0
              irest=0
  117         CONTINUE
              ENDIF
            IF(Npar.EQ.Mpar) GO TO 900
            GX(J,I)=' '
            Npar=Npar+1
            Ipar(Npar)=I*10+J
            ENDIF
  120     CONTINUE
  130   CONTINUE
  899 if(ndebug.ne.0) then
        do 896 i=1,nx
          write(ndebug,1898) (gx(j,i),j=1,3),i
 1898     format(' *debug*selpar*1898* gx = ',3A4,' for atom ',I3)
  896     continue
        write(ndebug,1899) (ipar(i),i=1,npar)
 1899   format(' *debug*selpar*1899* ipar(i) = ',40(15I3,/,27X))
        endif
      RETURN
  900 WRITE(NOUT,9001) Mpar
 9001 FORMAT(' **ERROR**selpar**9001** ',I4,' VARIABLES FOUND, REST',
     +  ' OMITTED')
      RETURN
  999 RETURN 1
      END
C
c
c
c
      SUBROUTINE SELEQ(npar)
c     -----------------
c
c
c   determines which equations are to be used in the least squares
c
c   neq counts the actual number of conditions selected.
c   leq counts the number of independent conditions stored
c       since this is a difficult number to get exactly it errs on
c       the side of being too small
c   ity determines the nature of the condition
c       = 1  refine on Valence Sums
c       = 2  refine on Bond Valences
c       = 3  refine on Bond Distances
c       = 4  refine on Contacts
c   WEIGHTS are assigned automatically as 1/(sigma)**2 where sigma
c       is a notional uncertainty assigned as follows:
c       ity = 1;  sigma = 0.05 v.u.   weight = 400
c           = 2;        = 0.10 v.u.   weight = 100
c           = 3;        = 0.032/(bond valence) A (range 0.01 ~ 0.20 A)
c           = 4;  (If the weight is negative, it is reset in caldif  
c                 to zero if contact distance > target)
c                       = 0.5 A       weight = -4
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
c
      if(neq.ne.0) then
   10   write(nout,1010)
 1010   format(' Do you wish to cancel previous selections of ',
     1  'conditions (Y/N/E) ',$)
        call readyn(*50,*100,*900,*10)
        endif
   50 neq = 0
      leq = 0
  100 write(nout,1000) neq,leq,npar
 1000 format(' You have currently selected ',I3,' conditions of ',
     1'which about ',I3,/,
     2' are independent.  You need at least ',I3,' to match the ',
     2'number of variable ',/,
     3' parameters',//,
     6'   Menu 3.15.3: Selection of Conditions',/,
     7' --------------------------------------',/,
     8'   1. Select bond valence sums',/,
     9'   2. Select individual bond valences',/,
     +'   3. Select individual bond lengths',/,
     1'   4. Select individual contacts',//,
     3'  17. Display selected conditions',/,
     4'  18. Change targets and weights',/,
     5'  19. Clear all selections',/,
     6'  20. Return to Menu 3.15',//,
     8'     ENTER YOUR SELECTION ',$)
      it = intread()
      if(it.eq.1) call selsum
      if(it.eq.2) call selval
      if(it.eq.3) call selbon
      if(it.eq.4) call selcon
      if(it.eq.17) call dissel
      if(it.eq.18) call altsel
      if(it.eq.19) go to 50
      if(it.eq.20) return
      go to 100
  900 return
      end
c
c
      SUBROUTINE SELSUM
c     -----------------
c
c   written 91.8.1 by I.D.Brown
c
c   selects the atomic valences to be matched by valence sums
c
c   calls MATHLB: masdim,readyn,wait
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common/files/ nin,nout,ndebug
      common/vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      include 'comatom.for'
c
      neqold = neq
  100 write(nout,1050)
 1050 format(//,' Menu 3.8.5.1:  Select target valence sums',/,
     1       ' -----------------------------------------',/,
     2' 1. Select atoms whose valence sums are to be used',/,
     3' 2. Modify the selection',////,
     9' 20. Return to Menu 3.8.2',//,
     9' ENTER YOUR SELECTION  ',$)
      iflag = intread()
      if(iflag.eq.1) then
   52   write(nout,1052)
 1052   format(' Do you want to include all the atoms (Y/N/E) ',$)
   55   call readyn(*60,*70,*900,*52)
   60   iflagg = 1
   70   do 80 i=1,nx
   75     if(iflagg.eq.0) then
            write(nout,1070) gelem(i),gdent(i),oxid(i)
 1070       format('  Do you want to include atom ',A2,A6,' with ',
     1           'valence',F5.2,' (Y/N/E)')
            call readyn(*78,*80,*900,*75)
            endif
   78     neq = neq + 1
          if(maxdim(neq,meq,'Conditions').ne.0) go to 900
          ieq(neq) = 1000 + i
          eqwgt(neq) = 100.
          targ(neq) = oxid(i)
          if(i.ne.nx) leq = leq + 1
   80     continue
      elseif(iflag.eq.2) then
  110   do 130 i=neqold+1,neq
          j = ieq(i) - 1000
          if(j.lt.1000.and.j.gt.0) then
  115       write(nout,1115) gelem(j),gdent(j),targ(i),eqwgt(i)
 1115       format(' Atom ',A2,A6,' has valence ',F6.3,' and weight ',
     1        F7.2,/,
     2        ' Do you wish to change this (E skips remaining atoms)',
     3        ' (Y/N/E) ',$)
            call readyn(*120,*130,*100,*115)
  120       write(nout,1120)
 1120       format(' Enter new value of the target valence and weight')
            read(nin,*) targ(i),eqwgt(i)
            endif
  130     continue
      elseif(iflag.eq.20)then
        go to 900
        endif
      go to 100
  900 return
      end
c
c
      SUBROUTINE SELVAL
c     -----------------
c
c   written 91-8-2 by I.D.Brown
c
c   sets flags for the inclusion of bond valence targets 
c
c   calls  MATHLB: maxdim,readyn,wait
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      include 'comatom.for'
      include 'combond.for'
      common /files/ nin,nout,ndebug
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
c
      neqold = neq
c
c   Menu
c   ----
  100 write(nout,1100)
 1100 format(//,'  Menu 3.8.3.2: Select target bond valences',/,
     1       '  -------------',/,
     1 ' Bonds will normally be included twice, once with ',
     1 'the target valence',/,
     2 ' calculated around the cation, and once with the target ',
     2 'valence calculated ',/,
     3 ' around the anion. Do you wish to include: ',//,
     4 '   1. All target valences',/,
     5 '   2. Targets around cations only',/,
     6 '   3. Targets around anions only',/,
     7 '   4. Targets for selected bonds only',/,
     8 '   5. Targets from network equations',/,
     9 '   6. Modify targets',///,
     9 '  19. Delete target valences',/,
     1 '  20. Return to Menu 3.8.3',//,
     4 ' ENTER YOUR SELECTION  ',$)
      iflag = intread()
      if(iflag.lt.1.or.iflag.gt.20) go to 100
      if(iflag.ge.1.and.iflag.le.5) go to 200
      if(iflag.eq.6) go to 500
      if(iflag.eq.19) go to 800
      if(iflag.eq.20) go to 900
      go to 100
c
  200 if(iflag.eq.5) then
        irefn = 0
        call kirch(irefn)
        endif
      do 400 i=1,nbon
        ii = ibon(1,i)
        jj = jbon(1,i)
        if(iflag.eq.5) go to 300
        if(icn(ii).le.0.or.icn(jj).le.0) then
          write(nout,1200)
 1200     format('  Zero coordination number found, Target ',
     1        'valences cannot be calculated')
          return
          endif
        if(oxid(ii).gt.0.and.iflag.eq.3) go to 300
        if(oxid(ii).le.0.and.iflag.eq.2) go to 300
c
c   Set up target valences
c   ----------------------
        if(iflag.eq.4) then
  250     write(nout,1250) gelem(ii),gdent(ii),gelem(jj),gdent(jj)
 1250     format(' Do you want to include the bond from ',
     1     A2,A6,' to ',A2,A6,' (Y/N/E): ')
          call readyn(*270,*300,*900,*250)
          endif
  270   neq = neq + 1
        if(maxdim(neq,meq,'Bond valence').ne.0) return
        targ(neq) = abs(oxid(ii)/icn(ii))
        eqwgt(neq) = 100
        ieq(neq) = 2000 + 2*i 
        leq = leq + 1
c
  300   continue
        if(oxid(jj).gt.0.and.iflag.eq.3) go to 400
        if(oxid(jj).le.0.and.iflag.eq.2) go to 400
        if(iflag.eq.4) then
  350     write(nout,1250) gelem(jj),gdent(jj),gelem(ii),gdent(ii)
          call readyn(*370,*400,*900,*350)
          endif
  370   neq = neq + 1
        if(maxdim(neq,meq,'Bond valence').ne.0) return
        if(iflag.eq.5) then
          targ(neq) = xbon(3,i)
        else
          targ(neq) = abs(oxid(jj)/icn(jj))
          endif
        eqwgt(neq) = 100
        ieq(neq) = 2000 + 2*i + 1
        if(iflag.ne.1) leq = leq + 1
  400   continue
c   There will be one redundancy in the conditions for every valence
c   sum rule selected.
      leq = leq - nx
      write(nout,1400) neq-neqold,leq
 1400 format(' You have now added ',I3,' conditions and have a',
     1 ' total of',/,
     2 ' about',I3,' independent conditions set')
  410 write(nout,1410)
 1410 format(' Do you wish to modify the target valences (Y/N/E) ')
      call readyn(*500,*900,*900,*410)
c
c   Modify selected bond valences
c   -----------------------------
  500 continue
      write(nout,1500)
 1500 format(' Each bond will be listed and you will be given an',
     1 ' asked if',/,
     2 ' you wish to change the target or weight.  An answer of',
     2 ' ''E'' will allow',/,
     3 ' you to escape from this routine')
      do 599 i=neqold+1,neq
        call decieq(i,ity,isy,ij)
        if(ij.eq.0) then
          ii = ibon(1,isy)
          jj = jbon(1,isy)
        else
          jj = ibon(1,isy)
          ii = jbon(1,isy)
          endif
        write(nout,1520) gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1         (jbon(j,isy),j=2,5),xbon(1,isy),targ(i),eqwgt(i)
 1520   format(1X,A2,A6,1X,A2,A6,4I3,2X,'Dist =',F8.3,', Target =',
     1    F8.3,', Weight =',F8.3)
  530   write(nout,1530)
 1530   format(' Do you wish to change this target or weight (Y/N/E) ')
        call readyn(*540,*599,*100,*530)
  540   write(nout,1540)
 1540   format(' Enter the new target valence and weight')
        read(nin,*) targ(i), eqwgt(i)
  599   continue
      go to 100
c
c   deselect valences
c   -----------------
  800 write(nout,1800)
 1800 format(' You are about to cancel all target valences. ',
     1  'Do you wish to continue (Y/N/E) ',$)
      call readyn(*810,*100,*900,*800)
  810   do 850 i=1,neq
          if(ieq(i).gt.2000.and.ieq(i).lt.3000) then
            ieq(i) = 0
            targ(i) = 0.
            eqwgt(i) = 0.
            diff(i) = 0.
            endif
  850     continue
        j=0
        do 860 i=1,neq
          if(ieq(i).ne.0) then
            j = j+1
            ieq(j) = ieq(i)
            endif
  860     continue
        neq = j
        write(nout,1860)
 1860   format(' All target valences deleted',/)
      go to 100
  900 return
      end
c
c
      SUBROUTINE SELBON
c     -----------------
c
c  selects derivatives for bond conditions
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      common/vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
 1000 format(' This feature is not yet available')
c
c   Menu
c   ----
      neqold = neq
   90 I1 = 1
      call bocole(i1,i1,nbon)
  100 write(nout,1100)
 1100 format(//,' Menu 3.8.3.3: Select target bond distances',/,
     3          ' -------------------------------------------',/,
     4'  1. Use the network equations to calculate target distances',/,
     5'  2. Update target distances',/////,
     8' 18. List bond distances and targets selected',/,
     9' 19. Delete all bond targets',/,
     9' 20. Return to Menu 3.8.3',//,
     9' ENTER YOUR SELECTION  ',$)
      iflag = intread()
c
c   Calculate bond valences (in xbon(3,)) using network equations
c   -------------------------------------------------------------
      if(iflag.eq.1) then
        irefn = 0
        call kirch(irefn)
  250   write(nout,1250)
 1250   format(' Do you wish to include all the target distances',
     1    ' (Y/N/E) ',$)
        call readyn(*300,*405,*900,*250)
  300   do 350 i=1,nbon
          neq = neq + 1
          if(maxdim(neq,meq,'Bond Length').ne.0) return
          ii = ibon(1,i)
          jj = jbon(1,i)
          targ(neq) = dbost(gelem(ii),gelem(jj),oxid(ii),oxid(jj),
     1                     xbon(3,i))
          eqwgt(neq) = 1000*(xbon(3,i))**2
          ieq(neq) = 3000 + i
          leq = leq + 1
  350     continue
        write(nout,1350) neq-neqold
 1350   format(1X,/,I3,' bonds have been added to the constraints')
  405   write(nout,1405)
 1405   format(' For each bond you will be asked if you wish to ',
     1   'Include the bond,',/,
     2   ' Modify the bond, Skip the bond or Exit (no further bonds',
     2   ' included).')
c
c   Update bond distance targets and weights
c   ----------------------------------------
      else if(iflag.eq.2) then
  400   write(nout,1400)
 1400   format(' For each bond you will be asked if you wish to ',
     1   'Include the bond,',/,
     2   ' Modify the bond, Skip the bond or Exit (no further bonds',
     2   ' included).')
        do 450 i=1,nbon
          ii = ibon(1,i)
          jj = jbon(1,i)
  430     tar = dbost(gelem(ii),gelem(jj),oxid(ii),oxid(jj),xbon(3,i))
          wgt= 10000*(xbon(3,i))**2
  435     write(nout,1430) gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1                      xbon(1,i),tar,wgt
 1430     format(//,' Bond between',/,
     2           1X,2(A2,A6,2X),'Obs.Dist. =',F6.3,', Target =',F6.3,
     2             ', Weight =',F9.3,//
     3     ' Do you want to Include, Modify, Skip or Exit (I/M/S/E) ')
          read(nin,1435) gflag
 1435     format(A1)
          call upcase(gflag)
          if(gflag.eq.'I') then
c         ----------------
            go to 440
          elseif(gflag.eq.'M') then
c         --------------------
            write(nout,1440)
 1440       format(' Give new target and new weight.  ',
     1             'Include decimal point')
            read(nin,*) tar, wgt
  448       write(nout,1448) gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1             xbon(1,i),tar,wgt
 1448       format(/,' The modified bond is',/,
     1        1X,2(A2,A6,2X),'Obs.Dist. =',F6.3,', Target =',F6.3,
     1        ', Weight =',F9.3,/,
     2        ' Do you want to accept this value (Y/N/E) ')
            call readyn(*440,*430,*100,*448)
          elseif(gflag.eq.'E') then
c         --------------------
  449       write(nout,1449)
 1449       format(' All remaining bonds will NOT be included. ',
     1        'Do you want to exit (Y/N/E) ')
            call readyn(*100,*435,*100,*449)
          elseif(gflag.eq.'S') then
c         --------------------
            go to 450
          else
            write(nout,1441) gflag
 1441       format(' **********************************************',/
     2         ' *** ',A1,' is not a legal response. Try again  ***',/
     3             ' **********************************************')
            go to 435
            endif
  440     neq = neq + 1
          if(maxdim(neq,meq,'Bond Length').ne.0) return
          targ(neq) = tar
          eqwgt(neq) = wgt
          ieq(neq) = 3000 + 1
          leq = leq + 1
  450     continue
c
c   List bond distances
c   -------------------
      else if(iflag.eq.18) then
        write(nout,1500)
 1500   format(' Atom 1                Atom 2               ',
     1     'Observed  Target  Weight')
        linc = 0
        do 550 i=1,neq
          call decieq(i,ity,isy,ij)
          if(ity.eq.3) then
            ii = ibon(1,isy)
            jj = jbon(1,isy)
            linc = linc + 1
            write(nout,1530) gelem(ii),gdent(ii),(ibon(j,isy),j=2,5),
     1           gelem(jj),gdent(jj),(jbon(j,isy),j=2,5),xbon(1,isy),
     2           targ(i),eqwgt(i)
 1530       format(1X,2(A2,A6,4I3,2X),2F8.3,F9.3)
            endif
          if(linc.ge.20) then
            call wait
            linc = 0
            endif
  550     continue
        if(linc.ne.0) call wait
c
c   Deselect bond distances
c   -----------------------
      else if(iflag.eq.19) then
  620   write(nout,1620)
 1620   format(' You are about to delete all target distances.',
     1    '  Do you wish to continue (Y/N/E) ',$)
        call readyn(*630,*100,*900,*620)
  630   do 650 i=1,neq
          if(ieq(i).gt.3000.and.ieq(i).lt.4000) then
            ieq(i) = 0
            targ(i) = 0.
            eqwgt(i) = 0.
            diff(i) = 0.
            endif
  650     continue
        j=0
        do 660 i=1,neq
          if(ieq(i).ne.0) then
            j = j+1
            ieq(j) = ieq(i)
            endif
  660     continue
        neq = j
        write(nout,1660)
 1660   format(' All target distances deleted',/)
c
c   Exit subroutine
c   ---------------
      else if(iflag.eq.20) then
        go to 900
        endif
      go to 100
  900 return
      end
c
c
      SUBROUTINE SELCON
c     -----------------
c
c   selects derivatives for contact conditions
c
c  calls DISLS: calcon
c        MATHLB: readyn,wait
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug
      common/cell/a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common/rcell/rs(6),rvol,rm(3,3)
      common/sym/nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      include 'comatom.for'
      include 'combond.for'
      common/vils/neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      dimension xmax(3),xmin(3),dx(3),itran(3)
 1000 format(' This feature is not yet available')
c
c   recalculate contacts from present coordinates
c   ---------------------------------------------
      neqold = neq
      if(ncon.gt.0) then
        i1 = 1
        i2 = 2
        call bocole(i2,i1,ncon)
        endif
  100 write(nout,1100)
 1100 format(' Menu 3.8.3.4: Select contact distances',/,
     2       ' ---------------------------------------',/,
     3 '  1. Generate target contact distances (Brown algorithm)',/,
     4 '      N.B. This deletes all existing contacts',/,
     4 '  2. Update target contacts',/,
     5 '  3. Reset all existing contact targets to a fixed value',/,
     6 '  4. Enter new contacts individually',/,
     7 '  5. Include all anion-anion distances', //,
     8 ' 18. List contacts selected',/,
     9 ' 19. Delete all contacts',/,
     9 ' 20. Return to Menu 3.8.3',//,
     9 '  ENTER YOUR SELECTION  ',$)
      iflag = intread()
c
c   Calculate contacts using Brown algorithm (Phys. Chem. Miner.(1987)15,30)
c   ---------------------------------------------------------------
      if(iflag.eq.1) then
  214   write(nout,1014)
 1014   format(' Contacts will only be included if they are ',
     1   'more than',/,/,
     2   '    X*SQRT(cation coordination number)  degrees',/,/,
     3   ' from the anion-cation bond.',/,
     4   ' Enter the value of X (default = 14.) ')
        read(nin,*) xlim
        if(xlim.lt.0.0001) xlim=14.
        call calcon(xlim)
  280   write(nout,1280) ncon
 1280   format(1X,I4,' contacts have been calculated',/,
     2' Do you want to include all these contacts in',
     2     ' the refinement (Y/N/E) ')
        call readyn(*290,*305,*900,*280)
  290   do 295 i=1,ncon
          neq = neq+1
          if(maxdim(neq,meq,'Contacts').ne.0) return
          ieq(neq) = 4000 + i
          targ(neq) = xcon(3,i)
          eqwgt(neq) = -4.
          leq = leq + 1
  295     continue
        write(nout,1295) neq-neqold
 1295   format(1X,I3,' contacts have been added to the constraints')
  305   write(nout,1305)
 1305   format(//,' For each contact you will be asked if you ',
     1   'wish to Include the contact,',/,
     2   ' Modify the contact, Skip the contact or Exit ',
     2   '(no further contacts included).')

c
c   Update contacts and weights
c   ---------------------------
      else if(iflag.eq.2) then
        if(ncon.le.0) then
          write(nout,1301)
 1301     format(' No contacts are stored.  Generate contacts using ',
     1      'menu item 1.')
          call wait
          go to 100
          endif
  300   write(nout,1300)
 1300   format(//,' For each contact you will be asked if you ',
     1   'wish to Include the contact,',/,
     2   ' Modify the contact, Skip the contact or Exit ',
     2   '(no further contacts included).')
        do 350 i=1,neq
          if(ieq(i).ge.4000.and.ieq(i).lt.5000) then
            ij = ieq(i) - 4000
            ii = icon(1,ij)
            jj = jcon(1,ij)
            wgt= 0
  335       write(nout,1330) gelem(ii),gdent(ii),(icon(j,ij),j=2,5),
     1        gelem(jj),gdent(jj),(jcon(j,ij),j=2,5),xcon(1,ij),
     2        targ(i),eqwgt(i)
 1330       format(//,' Contact between',/,
     2            1X,2(A2,A6,4I3,2X),'Obs.Contact =',F6.3,', Target =',
     1            F6.3,', Weight =',F6.3,//
     3      ' Do you want to Include, Modify, Skip or Exit (I/M/S/E) '
     4      ,$)
            read(nin,1335) gflag
 1335       format(A1)
            call upcase(gflag)
            if(gflag.eq.'I') then
c           ----------------
              continue
            elseif(gflag.eq.'M') then
c           --------------------
              write(nout,1340)
 1340         format(' Give new target and new weight. Negative '
     1         'weights will be ',/,
     2         ' set to zero if d(obs) > d(target)')
              read(nin,*) tar, wwgt
  346         write(nout,1346) gelem(ii),gdent(ii),(icon(j,ij),j=2,5),
     1          gelem(jj),gdent(jj),(jcon(j,ij),j=2,5),xcon(1,ij),
     2          tar,wwgt
 1346         format(' Modified contact',/,
     1             1X,2(A2,A6,4I3,2X),/,
     2          ' Obs.Contact =',F6.3,' Target =',F6.3,', Weight =',
     2             F7.2,/,
     3          ' Do you wish to accept this value (Y/N/E) ',/)
              call readyn(*347,*335,*335,*346)
  347         targ(i) = tar
              eqwgt(i) = wwgt
            elseif(gflag.eq.'E') then
c           --------------------
  348         write(nout,1348)
 1348         format(' All the remaining contacts will NOT be ',
     1         'included.  Do you wish to Exit (Y/N/E) ',$)
            call readyn(*100,*335,*100,*348)
            elseif(gflag.eq.'S') then
c           --------------------
              eqwgt(i) = 0.
            else
              write(nout,1350) gflag
 1350         format(' ********************************************',/
     1        ' *** ',A1,' is not a legal response. Try again ***',/
     2             ' ********************************************')
              go to 335
              endif
            endif
  350     continue
c
c   Set all targets to a constant value
c   -----------------------------------
      else if(iflag.eq.3) then
  360   write(nout,1360)
 1360   format(' Enter the TARGET LENGTH and WEIGHT to be ',
     1             'applied to all contacts',//,
     2    ' A negative weight will be set to zero if the observed',/,
     2    ' distance is greater than the target',//,
     3    ' Enter a blank to make no change',/)
  370   read(nin,*) tar,wwgt
        if(abs(tar).lt.0.001) go to 100
  375   write(nout,1375) tar,wwgt
 1375   format(' You have entered the target = ',F5.2,
     1            ', weight = ',F8.2,/,
     2    ' Is this correct (Y/N/E) ',$)
        call readyn(*380,*360,*100,*375)
  380   do 390 i=1,neq
          if(ieq(i).ge.4000.and.ieq(i).lt.5000) then
            targ(i) = tar
            eqwgt(i) = wwgt
            endif
  390     continue
c
c   Set additonal contacts individually
c   -----------------------------------
      else if(iflag.eq.4) then
  400   write(nout,1400) nsym
 1400   format(' Enter the element symbols, the identifiers, the ',
     1          'symmetry operators and',/,
     2     ' translation operators of the two atoms defining the ',
     2          'contact when requested',//,
     3     ' Note that only ',I3,' symmetry operators are stored.'
     3          'Use explicit translations',/,
     4     ' to generate the centering operators I, A, B or C',//,
     4     ' Negative weights are set to zero if the observed ',
     4          'distance is greater',/,
     5     ' than the target',/,
     6     ' Enter blank when finished',/)
  401   ncon = ncon + 1
        if(maxdim(ncon,mcon,'contact 410').ne.0) go to 100
        write(nout,1401)
 1401   format(/,' Enter the element symbol of the first atom',
     1      '. (Blank to exit)')
        read(nin,1402) gel1
 1402   format(A6)
        if(gel1.eq.'  ') then
          ncon = ncon - 1
          go to 100
          endif
        write(nout,1403)
 1403   format(' Enter the identifier of the first atom')
        read(nin,1402) gid1
        write(nout,1404)
 1404   format(' Enter the element symbol of the second atom')
        read(nin,1402) gel2
        write(nout,1405)
 1405   format(' Enter the identifier of the second atom')
        read(nin,1402) gid2
        write(nout,1406)
 1406   format(' Enter the symmetry operator and translation vector ',
     1    '(in units of 1/12) ',/,
     2    ' of the first atom in the form '' 11  6  0-12'' allowing,'
     2        ' three digits ',/,
     3    'per number (4I3)')
        read(nin,*) (icon(j,ncon),j=2,5)
        write(nout,1408)
 1408   format(' Enter the symmetry operator and translation vector ',
     1    'of the second atom')
        read(nin,*) (jcon(j,ncon),j=2,5)
        do 420 i=1,nx
          if(igcomm(gel1,gelem(i),2).eq.0
     1        .and.igcomm(gid1,gdent(i),6).eq.0) icon(1,ncon) = i
          if(igcomm(gel2,gelem(i),2).eq.0
     1        .and.igcomm(gid2,gdent(i),6).eq.0) jcon(1,ncon) = i
  420     continue
        ii = icon(1,ncon)
        jj = jcon(1,ncon)
        xcon(1,ncon) = dstnce(ii,icon(2,ncon),icon(3,ncon),
     1                        jj,jcon(2,ncon),jcon(3,ncon))
        write(nout,1409) gelem(ii),gdent(ii),(icon(j,ncon),j=2,5),
     1                   gelem(jj),gdent(jj),(jcon(j,ncon),j=2,5),
     2                   xcon(1,ncon)
 1409   format(/,' The distance ',2(A2,A6,4I3,1X), 'is ',
     1          F7.3,' A',//,
     1    ' Enter the TARGET LENGTH and WEIGHT (blank to cancel)',//,
     2    ' A negative weight is set to zero if the observed ',
     2        'distance is larger ',/,
     3    ' than the target')
        read(nin,*) tar,wwgt
        if(abs(tar).lt.0.01) then
          ncon = ncon - 1
          go to 100
          endif
        neq = neq + 1
        if(maxdim(neq,meq,'contacts 420').ne.0) go to 100
        ieq(neq) = 4000 + ncon
        targ(neq) = tar
        eqwgt(neq) = wwgt
        go to 401
c
c   Calculate all anion-anion distances below a given limit
c   -------------------------------------------------------
      else if(iflag.eq.5) then
 2500   write(nout,3500)
 3500   format(' This routine will include all anion-anion contacts ',
     1    'below a given',/,
     2    ' limit.  Enter the maximum contact length to be ',
     2    'calculated',/,
     3    ' (Blank to exit)')
       read(nin,*,err=2500) dmax
       if(abs(dmax).lt.0.0001) go to 100
 2505  write(nout,3505)
 3505  format(' Enter the global target contact distance and weight',/,
     1   ' A negative weight will be set to zero if the distance '
     1   'exceeds the target ')
       read(nin,*) tar,wwgt
c   calculate the y atom array
       do 2520 k=1,3
         xmax(k) = -10.
         xmin(k) = 10.
 2520    continue
       do 2530 i=1,nx
         do 2525 k=1,3
           if(xx(k,i).gt.xmax(k)) xmax(k) = xx(k,i)
           if(xx(k,i).lt.xmin(k)) xmin(k) = xx(k,i)
 2525      continue
 2530    continue
      do 2535 k=1,3
        xmax(k) = xmax(k) + rs(k)*dmax + 0.1
        xmin(k) = xmin(k) - rs(k)*dmax - 0.1
 2535   continue
      ny = 0
      call calcy(xmax,xmin)
c   calculate the distances between anions in xatom array and y atom array
      do 2580 i=1,nx
        if(oxid(i).lt.0.) then
          do 2570 j=1,ny
            if(oxid(iynum(j)).lt.0.) then
              call vminv(dx,xx(1,i),yx(1,j))
              d = dist(dx,am)
              if(d.lt.dmax.and.d.gt.0.001) then
                ncon = ncon + 1
                if(maxdim(ncon,mcon,'contact 2570').ne.0) go to 100
                icon(1,ncon) = i
                icon(2,ncon) = 1
                jcon(1,ncon) = iynum(j)
                jcon(2,ncon) = iysym(j)
                call getstl(itran,iynum(j),yx(1,j),iysym(j))
                do 2560 k=1,3
                  icon(k+2,ncon) = 0
                  jcon(k+2,ncon) = itran(k)
                  xcon(k+3,ncon) = dx(k)
 2560             continue
                xcon(1,ncon) = d
                xcon(3,ncon) = tar
                neq = neq + 1
                if(maxdim(neq,meq,'contact 2560').ne.0) go to 100
                ieq(neq) = 4000 + ncon
                targ(neq) = tar
                eqwgt(neq) = wwgt
                endif
              endif
 2570       continue
          endif
 2580   continue
c
c   List contacts
c   -------------
      else if(iflag.eq.18) then
        write(nout,1500)
 1500   format(' Atom 1                Atom 2               ',
     1     'Observed  Target  Weight')
        linc = 0
        do 550 i=1,neq
          call decieq(i,ity,isy,ij)
          if(ity.eq.4) then
            ii = icon(1,isy)
            jj = jcon(1,isy)
            linc = linc + 1
            write(nout,1530) gelem(ii),gdent(ii),(icon(j,isy),j=2,5),
     1           gelem(jj),gdent(jj),(jcon(j,isy),j=2,5),xcon(1,isy),
     2           targ(i),eqwgt(i)
 1530       format(1X,2(A2,A6,4I3,2X),3F8.3)
            endif
          if(linc.ge.20) then
            call wait
            linc = 0
            endif
  550     continue
        if(linc.ne.0) call wait
c
c   Deselect all contacts
c   ---------------------
      else if(iflag.eq.19) then
  620   write(nout,1620)
 1620   format(' You are about to delete all contact targets. ',
     1   'Do you wish to continue (Y/N/E) ',$)
        call readyn(*630,*100,*900,*620)
  630   do 650 i=1,neq
          if(ieq(i).gt.4000.and.ieq(i).lt.5000) then
            ieq(i) = 0
            targ(i) = 0.
            eqwgt(i) = 0.
            diff(i) = 0.
            endif
  650     continue
        j=0
        do 660 i=1,neq
          if(ieq(i).ne.0) then
            j = j+1
            ieq(j) = ieq(i)
            endif
  660     continue
        neq = j
        write(nout,1680)
 1680   format(' All target contacts have been removed.',//)
c
c   Exit subroutine
c   ---------------
      else if(iflag.eq.20) then
        if(ncon.gt.0) then
c   label symmetry independent contacts and exit
c   --------------------------------------------
          i1 = 1
          i2 = 2
          if(ind2.lt.ncon) call indepb(i2,i1,ncon)
          endif
        go to 900
        endif
      go to 100
  900 return
      end
c
c
c    ************  The following routine is to go to mathlb  **********
c
      FUNCTION IGCOMM(g1,g2,n)
c     ------------------------
c
c   written 91.12.4 by I.D.Brown
c
c   igcomm = 0 if g1(1:n) and g2(1:n) are identical (case insensitive)
c
      implicit character*8 (g)
c
      igcomm = 1
      g3 = g1
      g4 = g2
      call upcase(g3)
      call upcase(g4)
      if(g3(1:n).eq.g4(1:n)) igcomm = 0
      return
      end
c
c
c
      SUBROUTINE DECIEQ(i,ity,isy,ij)
c     -----------------
c
c   written 91-8-9 by I.D.Brown
c
c   decomposes the flag ieq(i) into ity, isy and ij
c   where
c     if ity = 1, condition = valence sum, isy = atom in /ident/
c     if ity = 2, condition = bond valence, isy = bond in /bonds/
c     if ity = 3, condition = bond length, isy = bond in /bonds/
c     if ity = 4, condition = contact, isy = contact in /contacc/
c
c   where, if ij = 0, the first listed bond is the central atom,
c          if ij = 1, the second listed bond is the central atom
c
      common/vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
c
      ij = 0
      ity = ieq(i)/1000
      isy = ieq(i) - 1000*ity
      if(ity.eq.2) then
        is = isy/2
        ij = isy - 2*is
        isy = is
        endif
      return
      end
c
c
      SUBROUTINE BONOUT
c     -----------------
c
c   written 91.9.18 by I.D.Brown  
c
c   Lists the contents of the bond array
c
      implicit character*8 (g)
      implicit character*80 (h)
      common/files/ nin,nout,ndebug
      include 'comatom.for'
      include 'combond.for'
      dimension sum(100)
c
      do 50 i=1,100
   50   sum(i) = 0.0
      write(nout,1000)
 1000 format(' Bond distances and Valences',/,
     1       ' ---------------------------')
c
      linc = 0
      do 200 i=1,nbon
        ii = ibon(1,i)
        jj = jbon(1,i)
        if(ii.le.100.and.jj.le.100) then
          call bost(ii,jj,str,estr,xbon(2,i),xbon(1,i))
          endif
        write(nout,1100) gelem(ii),gdent(ii),gelem(jj),gdent(jj),
     1    (jbon(j,i),j=2,5),xbon(1,i),str
 1100   format(1X,A2,A6,1X,A2,A6,'(',4I3,')',F7.3,'A, ',F7.3,' v.u.')
        linc = linc + 1
        if(linc.ge.18) then
          linc = 0
          call wait
          endif
  200   continue
      if(linc.ne.0) call wait
      write(nout,1300)
 1300 format(' Atomic Valences and Valence Sums'/,
     1       ' --------------------------------')
      linc = 0
      do 500 i=1,nx
        write(nout,1400) gelem(i),gdent(i),oxid(i),valsum(i)
 1400   format(1X,A2,A6,2X,2F10.3)
        linc = linc+1
        if(linc.eq.18) then
          linc = 0
          call wait
          endif
  500   continue
      if(linc.ne.0) call wait
      end
c
c
c
      SUBROUTINE DISSEL
c     -----------------
c
c  displays selected conditions
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      include 'comatom.for'
      include 'combond.for'
c
      linc = 0
      write(nout,1000)
 1000 format(' Listing of Stored Conditions',/,
     2       ' ----------------------------',/,
     3'   Condition     Atom 1  Atom 2      Target Weight',/,
     4' -------------------------------------------------')
      do 100 i=1,neq
        call decieq(i,ity,isy,ij)
        if(ndebug.ne.0) write(ndebug,9000) i,ity,isy,ij
 9000     format(' *debug*dissel*9000* i = ',I3,', ity = ',I3,
     1        ', isy = ',I3,', ij = ',I3)
        hty = '            '
        gel1 =  '  '
        gel2 = '  '
        gid1 = '      '
        gid2 = '      '
        if (ity.eq.1) then
          hty = 'Valence Sum'
          gel1 = gelem(i)
          gid1 = gdent(i)
        else if(ity.eq.2) then
          hty = 'Bond Valence'
          if(ij.eq.0) then
            ii = ibon(1,isy)
            jj = jbon(1,isy)
          else if(ij.eq.1) then
            ii = jbon(1,isy)
            jj = ibon(1,isy)
            endif
          gel1 = gelem(ii)
          gid1 = gdent(ii)
          gel2 = gelem(jj)
          gid2 = gdent(jj)
        else if(ity.eq.3) then
          ii = ibon(1,isy)
          jj = jbon(1,isy)
          gel1 = gelem(ii)
          gid1 = gdent(ii)
          gel2 = gelem(jj)
          gid2 = gdent(jj)
          hty = 'Bond Length'
        else if(ity.eq.4) then
          ii = icon(1,isy)
          jj = jcon(1,isy)
          gel1 = gelem(ii)
          gid1 = gdent(ii)
          gel2 = gelem(jj)
          gid2 = gdent(jj)
          hty = 'Contact'
          endif
        write(nout,1100) hty,gel1,gid1,gel2,gid2,targ(i),eqwgt(i)
 1100   format(4X,A12,2X,2(A2,A6,1X),F6.3,F8.1)
        linc = linc + 1
        if (linc.ge.16) then
          linc = 0
          call wait
          if(i.eq.neq) return
          write(nout,1000)
          endif
  100   continue
      call wait
      return
      end
c
c 
      SUBROUTINE ALTSEL
c     -----------------
c
c   changes the selection of conditions
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      common /files/ nin,nout,ndebug
c
      write(nout,1000)
 1000 format(' This feature is not yet available')
      call wait
      return
      end
c
c
      SUBROUTINE CALDIF
c     -----------------
c
c  calculate the differences
c
c  calls: SPANNER/expand,jyat
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      double precision xm(500,500),v(500)
      common/matrixx/nm,mm,xm,v,im(500,500),bwgt(500),cwgt(500)
      include 'combond.for'
      common/files/ nin,nout,ndebug
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      dimension ky(5),ly(5)
c
c
c   loop through the conditions
c   --------------------------
      do 500 i=1,neq
c   decompose ieq
c   -------------
        call decieq(i,ity,isy,ij)
        if(ity.lt.1.or.ity.gt.4) then
          write(nout,1190) i,ieq(i)
 1190     format(' *error*caldif*1190* Unrecognised ',
     1     'condition for ieq(',I2,') = ',I5)
          go to 500
          endif
c
c   calculate the difference (diff) for this condition
c   --------------------------------------------------
c   ity = 1 (valence sum rule); 
c   ity = 2 (fit bond valences)
c   ity = 3 (fit bond lengths)
c   ity = 4 (fit contacts)
c
c   Valence Sum
c   -----------
        if(ity.eq.1) then
          diff(i) = abs(targ(i)) - abs(valsum(isy))
c
c   Bond Valences
c   -------------
        else if (ity.eq.2) then
          dd = dstnce(ibon(1,isy),ibon(2,isy),ibon(3,isy),
     1                jbon(1,isy),jbon(2,isy),jbon(3,isy))
          if(ij.eq.0) then
            call bost(ibon(1,isy),jbon(1,isy),str,estr,err,dd)
          else
            call bost(jbon(1,isy),ibon(1,isy),str,estr,err,dd)
            endif
          diff(i) = abs(targ(i)) - abs(str)
c
c  Bond distances
c  --------------
        elseif(ity.eq.3) then
          xbon(1,isy) = dstnce(ibon(1,isy),ibon(2,isy),ibon(3,isy),
     1                jbon(1,isy),jbon(2,isy),jbon(3,isy))
          diff(i) = targ(i) - xbon(1,isy)
c
c   Contacts (reset weight)
c   -----------------------
        elseif(ity.eq.4) then
          xcon(1,isy) = dstnce(icon(1,isy),icon(2,isy),icon(3,isy),
     1                jcon(1,isy),jcon(2,isy),jcon(3,isy))
          diff(i) = targ(i) - xcon(1,isy)
          endif
c
c    Debug check
c    -----------
        if(ndebug.ne.0) then
          write(ndebug,9150) ity,isy,ij,targ(i),eqwgt(i),diff(i)
 9150     format(' *debug*caldif*9150* ity =',I3,', isy =',I3,
     1       ', ij =',I3,', targ =',F6.3,', eqwgt =',F6.1,
     2       ', diff =',F6.3)
          endif
c
  500   continue
      return
      end
c
c
      SUBROUTINE MAKMAT
c     -----------------
c
c  Create the matrix and vector
c
c  calls: SPANNER/expand,jyat
c
      implicit character*8 (g)
      implicit character*80 (h)
c
      double precision xm(500,500),v(500)
      common/matrixx/nm,mm,xm,v,im(500,500),bwgt(500),cwgt(500)
      include 'combond.for'
      common/files/ nin,nout,ndebug
      common /span/ nya,mya,iyatom(5,600),iterm(600),
     1             nbo,mbo,ibond(2,600),xbond(6,600),iat,bondlim(2),
     2             netind(20),ifbond(4,600)
      common /vils/ neq,meq,leq,ieq(200),targ(200),eqwgt(200),diff(200)
      dimension ky(5),ly(5)
c
c   clear matrix and vector and set bond limits
c   -------------------------------------------
      do 100 i=1,mm
        v(i) = 0.
        do 90 j=1,mm
          xm(i,j) = 0.
   90     continue
  100   continue
c
c
c   calculate bond limits and valences for EXPAND
c   ----------------------------------------------
      bondlim(1) = 0.01
      bondlim(2) = 5.0
      if(abs(xbon(3,1)).lt.0.001) then
        do 120 i=1,nbon
          call bost(ibon(1,i),jbon(1,i),xbon(3,i),estr,
     1         xbon(2,i),xbon(1,i))
  120     continue
        endif
c
c
c   loop through the conditions
c   --------------------------
      do 500 i=1,neq
c   decompose ieq
c   -------------
        call decieq(i,ity,isy,ij)
        if(ity.lt.1.or.ity.gt.4) then
          write(nout,1190) i,ieq(i)
 1190     format(' *error*makmat*1190* Unrecognised ',
     1     'condition for ieq(',I2,') = ',I5)
          go to 500
          endif
c
c   ity = 1 (valence sum rule); 
c   ity = 2 (fit bond valences)
c   ity = 3 (fit bond distances)
c   ity = 4 (fit contacts)
c
c   valence sums need to know the atomic environment
c   ------------------------------------------------
        if(ity.eq.1) then
          iat = isy
          itype = 3
          call expand(itype)
          call jyat(1,ky)
          endif
c
c    Debug check
c    -----------
        if(ndebug.ne.0) then
          write(ndebug,9150) ity,isy,ij,targ(i),eqwgt(i),diff(i)
 9150     format(' *debug*makmat*9150* ity =',I3,', isy =',I3,
     1       ', ij =',I3,', targ =',F6.3,', eqwgt =',F6.1,
     2       ', diff =',F6.3)
          endif
c
c    decompose im and calculate derivatives
c    --------------------------------------
        do 200 j=1,nm
          cwgt(j) = 0.0
          jat = im(j,1)/10
          ico = im(j,1) - 10*jat
c    Valence sums
c    ------------
          if(ity.eq.1) then
            dervsm = 0.0
            do 170 ii=2,nya
              call jyat(ii,ly)
              dervsm = dervsm + dervs(jat,ico,ky,ly)
  170         continue
            cwgt(j) =  dervsm
c    Bond valences
c    -------------
          else if(ity.eq.2) then
            if(ij.eq.0) then
              cwgt(j) = dervs(jat,ico,ibon(1,isy),jbon(1,isy))
            else
              cwgt(j) = dervs(jat,ico,jbon(1,isy),ibon(1,isy))
              endif
c     Bond lengths
c     ------------
          else if(ity.eq.3) then
            if(jat.eq.0) then
              cwgt(j) = derc(ico,ibon(1,isy),ibon(2,isy),ibon(3,isy),
     1                           jbon(1,isy),jbon(2,isy),jbon(3,isy))
            else
              if(jat.eq.ibon(1,isy)) then
                cwgt(j) = derx(ico,ibon(1,isy),ibon(2,isy),ibon(3,isy),
     1                           jbon(1,isy),jbon(2,isy),jbon(3,isy))
              else if(jat.eq.jbon(1,isy)) then
                cwgt(j) = derx(ico,jbon(1,isy),jbon(2,isy),jbon(3,isy),
     1                           ibon(1,isy),ibon(2,isy),ibon(3,isy))
                endif
              endif
c     Contacts
c     --------
          else if(ity.eq.4) then
            if(jat.eq.0) then
              cwgt(j) = derc(ico,icon(1,isy),icon(2,isy),icon(3,isy),
     1                           jcon(1,isy),jcon(2,isy),jcon(3,isy))
            else
              if(jat.eq.icon(1,isy)) then
                cwgt(j) = derx(ico,icon(1,isy),icon(2,isy),icon(3,isy),
     1                           jcon(1,isy),jcon(2,isy),jcon(3,isy))
              else if(jat.eq.jcon(1,isy)) then
                cwgt(j) = derx(ico,jcon(1,isy),jcon(2,isy),jcon(3,isy),
     1                           icon(1,isy),icon(2,isy),icon(3,isy))
                endif
              endif
            endif
  200     continue
c
c   debug check of derivatives
c   --------------------------
        if(ndebug.ne.0) then
          write(ndebug,9200) (cwgt(j),j=1,nm)
 9200     format(' *debug*makmat*9200* ',5(10F9.2,/,26X))
          endif
c
c   create vector and matrix elements
c   ---------------------------------
        do 350 k=1,nm
          weight = abs(eqwgt(i))
          if(ity.eq.4.and.eqwgt(i).lt.0.and.diff(i).lt.0) weight = 0.
          v(k) = v(k) + diff(i)*cwgt(k)*weight
          do 340 l=1,nm
            xm(k,l) = xm(k,l) + cwgt(k)*cwgt(l)*weight
  340       continue
  350     continue
  500   continue
      return
      end
c
c
      FUNCTION DERVS(jat,ico,k,l)
c     -------------------------------------------
c
c   written 91.7.31 by I.D.Brown
c
c   Calculates the derivative of the valence of the bond between k and l
c   wrt cell (iat = 0) or the icoth coordinate of atom jat.
c
c     dervs = - s*derx/B or -s*derc/B
c
c    calls CHEMLB/IBOST,BOST
c          DISTLB/dstnce
c
      implicit character*8 (g)
c
      common/files/nin,nout,ndebug
      include 'comatom.for'
      common/bostt/ nbost,mbost,ioxbost(2,30),ifbost(30),pbost(5,30)
      dimension k(5),l(5),xk(3),xl(3),dx(3)
c
      dervs = 0.
      if(ndebug.ne.0) then
        write(ndebug,9001) jat,k(1),l(1),gelem(k(1)),oxid(k(1)),
     1                                   gelem(l(1)),oxid(l(1))
 9001   format(' *debug*dervs*9001* ',3I3,2(2X,A2,F4.1))
        endif
      ival = ibost(gelem(k(1)),gelem(l(1)),oxid(k(1)),oxid(l(1)))
      b = 0.37
      if(ival.gt.0) then
        if(ifbost(ival).eq.2) b = pbost(2,ival)
        endif
      d = dstnce(k(1),k(2),k(3),l(1),l(2),l(3))
      call bost(k(1),l(1),str,estr,err,d)
      str = abs(str)
      if(jat.eq.0) dervs = -str*derc(ico,k(1),k(2),k(3),
     1                                   l(1),l(2),l(3))/b
      if(jat.eq.k(1)) dervs = -str*derx(ico,k(1),k(2),k(3),
     1                                   l(1),l(2),l(3))/b
      if(jat.eq.l(1)) dervs = -str*derx(ico,l(1),l(2),l(3),
     1                                   k(1),k(2),k(3))/b
      if(ndebug.ne.0) then
        write(ndebug,9000) ival,b,str,d,dervs
 9000   format(' *debug*dervs*9000* ',I3,4F10.3)
        endif
      return
      end
c
C
c
      FUNCTION DERC(ic,k,isk,itk,l,isl,itl)
c
c  Written 89.8.3 by I.D.Brown
c
c   Returns the derivative of the distance between atoms k and l
c   w.r.t. the ICth cell parameter
c
c   calls symopl, vminv, dist
c
      common/files/nin,nout,ndebug
      common/cell/ a,b,c,alpha,beta,agamma,vol,am(3,3)
      common/sym/nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      include 'comatom.for'
      dimension itk(3),itl(3),xk(3),xl(3),dx(3),xv(3,3),cell(3)
c
      cell(1) = a
      cell(2) = b
      cell(3) = c
      call symopl(xk,s(1,1,isk),t(1,isk),itk,xx(1,k))
      call symopl(xl,s(1,1,isl),t(1,isl),itl,xx(1,l))
      call vminv(dx,xl,xk)
      d = dist(dx,am)
      derc = 0.
      if(d.lt.0.0001) go to 200
      do 130 i=1,3
        do 130 j=1,3
          xv(i,j) = dx(j)*am(i,j)*dx(i)
  130     continue
      if(ic.ge.1.and.ic.le.3) then
        do 140 i=1,3
          derc = derc + xv(ic,i)/(cell(ic)*d)
  140     continue
      else if (ic.eq.4) then
        derc = -b*c*dx(2)*dx(3)*sin(alpha)/d
      else if (ic.eq.5) then
        derc = -c*a*dx(3)*dx(1)*sin(beta)/d
      else if (ic.eq.6) then
        derc = -a*b*dx(1)*dx(2)*sin(agamma)/d
        endif
  200 if(ndebug.ne.0) write(ndebug,9200) ic,k,isk,l,isl,
     +                 (dx(j),j=1,3),derc
 9200   format(' *debug*derc*9200* ',5I4,3F10.3,F9.2)
      return
      end
c
c
c
C
      FUNCTION DERX(I,K,ISK,ITK,L,ISL,ITL)
C
C   Written 85.2.25 by I.D.Brown
C
C   Returns the derivative of the distance between atoms K and L
C   w.r.t. the Ith (I=1,3) coordinate of atom K
C
C   Calls SYMOPL, VMINV, DIST
C
      implicit character*8 (g)
      implicit character*80 (h)
c
c     system common blocks
c
      COMMON /FILES/  NIN,NOUT,NDEBUG,nread,nwrite
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      common /sym/    nsym,msym,s(3,3,48),t(3,48),nlt,tu(3,4)
      include 'comatom.for'
C
      DIMENSION ITK(3),ITL(3),XK(3),XL(3),DX(3)
C
      CALL SYMOPL(XK,S(1,1,ISK),T(1,ISK),ITK,XX(1,K))
      CALL SYMOPL(XL,S(1,1,ISL),T(1,ISL),ITL,XX(1,L))
      CALL VMINV(DX,xl,xk)
      DERX=0.0
      DO 200 J=1,3
        DO 100 JJ=1,3
          DERX=DERX-(S(J,I,ISK)*DX(JJ)+S(JJ,I,ISK)*DX(J))*AM(J,JJ)
  100     CONTINUE
  200   CONTINUE
      d = dist(dx,am)
      if(abs(d).lt.0.001) then
        write(nout,1200) i,k,isk,itk,l,isl,itl
        write(nwrite,1200) i,k,isk,itk,l,isl,itl
 1200   format(' *error*derx*1200* 'I3,2X,5I3,2X,5I3)
        derx = 0.
        go to 300
        endif
      DERX=DERX/(2.*D)
  300 continue
      IF(NDEBUG.NE.0) WRITE(NDEBUG,9100) I,K,ISK,L,ISL,(DX(J),J=1,3),
     +   DERX
 9100 FORMAT(' *DEBUG*DERX*9100* ',5I4,3F10.3,F12.5)
      RETURN
      END
C
C
c
      SUBROUTINE NEWXYZ(DAMP,XSH)
c     ---------------------------
C
C   Written 85.2.26 by I.D.Brown as newpar in disls
C   Updated in April 85 by D.Altermatt
c   Moved to VISLS 91.8.9 by I.D.Brown
C
C   This program calculates the new parameters from the old
C
c   calls MAINLB:param
c
      implicit character*8 (g)
      implicit character*80 (h)
c
C DOUBLE PRECISION ARRAYS
      double precision xm(500,500),v(500)
      dimension wk(500)
      common/matrixx/ nm,mm,xm,v,im(500,500),bwgt(500),cwgt(500)
c
c     system common blocks
c
      common /files/  nin,nout,ndebug,nread,nwrite,nspgp
      common /cell/   a,b,c,alpha,beta,agamma,vol,am(3,3),icell
      include 'comatom.for'
      common /syma/   mult(48,48),it(3,48),nsp,msp,isptab(48,30),
     1                multsp(30),sp(3,3,30),tp(3,30)
      COMMON /GMAT/   GX(3,500),gcell(6)
      common /cellsym/ iax,iaxx,iayx
C
c         dimensions of dx are 3,MX
      DIMENSION DX(3,1000),DLAT(6),clat(6)
c
c   check dimensions
c   ----------------
      if(mx.gt.1000) then
        write(nout,1001) mx
 1001   format(/,' *ERROR*newxyz*1001* Increase the dimensions of',
     +   ' DX to ',I5,/,
     +   ' Calculation aborted')
        return
        endif
C
c   initialize
c   ----------
      SV=0.
      DAMP1=1.
      RAD=57.29577
      clat(1)=a
      clat(2)=b
      clat(3)=c
      clat(4)=alpha
      clat(5)=beta
      clat(6)=agamma
      DO 10 I=1,3
        DLAT(I)=0.0
        DLAT(I+3)=0.0
        DO 10 J=1,100
   10     DX(I,J)=0.0
C   ----------------------------
C   calculate the new parameters
C   ----------------------------
      DO 100 I=1,NM
  100   SV=SV+ABS(V(I))
      SU=SV/NM
      IF(SU.GT.0.1) DAMP=(0.1*DAMP)/SU
      DO 200 I=1,NM
        IAT=IM(I,1)/10
        ICO=IM(I,1)-10*IAT
c
c      lattice parameters
c      ------------------
        IF(IAT.EQ.0) THEN
          gcell(ico) = ' '
          DLAT(ICO)=V(I)*DAMP
          if(abs(dlat(ico)).gt.0.5) then
            dlat(ico) = 0.5*(dlat(ico)/abs(dlat(ico)))
            gcell(ico) = 'C'
            endif
          CLAT(ICO)=CLAT(ICO)+DLAT(ICO)
c
c      atomic coordinates
c      ------------------
        ELSE IF(IAT.GT.0.AND.IAT.LE.NX) THEN
          IALT=IALTNO(IAT)
          GX(ICO,IAT)=' '
          DO 150 J=1,3
            SHIFT=V(I)*DAMP
            IF(ABS(SHIFT).GT.(XSH/CLAT(J))) THEN
              SHIFT=(SHIFT*XSH)/(ABS(SHIFT)*CLAT(J))
              GX(ICO,IAT)='C'
              ENDIF
  150       DX(J,IAT)=DX(J,IAT)+SP(J,ICO,IALT)*SHIFT
          ENDIF
  200   CONTINUE
c
c   calculate new atomic coordinates
c   --------------------------------
      DO 210 I=1,NX
        DO 210 J=1,3
  210     XX(J,I)=XX(J,I)+DX(J,I)
c  -----------------
c  print the results
c  ------------------
      WRITE(NOUT,2000)
      WRITE(NOUT,2001) (CLAT(I),I=1,3),(RAD*CLAT(I),I=4,6),(DLAT(I),
     +  GCELL(I),I=1,3),(RAD*DLAT(I),GCELL(I),I=4,6)
 2000 FORMAT(//,'     New unit cell',/,
     +          '     -------------',/)
 2001 FORMAT('  a=',F7.3,',  b=',F7.3,',  c=',F7.3,',  alpha=',
     +  F6.2,',  beta=',F6.2,',  gamma=',F6.2,/,' da=',F6.3,A1,', db=',
     +  F6.3,A1,', dc=',F6.3,A1,', dalpha=',F5.2,A1,', dbeta=',F5.2,A1,
     +   ', dgamma=',F5.2,A1)
      WRITE(NOUT,2010)
 2010 FORMAT(//,'     New coordinates and damped shifts',/,
     +          '     ---------------------------------',/,
     +  4X,' Atom',9X,'X',5X,'shift',8X,'Y',5X,'shift',8X,'Z',5X,
     +  'shift',/)
      LINC=7
      DO 250 I=1,NX
        LINC=LINC+1
        WRITE(NOUT,2011) GELEM(I),GDENT(I),(XX(J,I),DX(J,I),GX(J,I)
     +    ,J=1,3)
 2011   FORMAT(5X,A2,1X,A6,3(F8.4,F8.4,2X,A1))
        IF(LINC.GE.20.OR.I.EQ.NX) THEN
          call wait
  300     IF(I.NE.NX) WRITE(NOUT,2010)
          LINC=0
          ENDIF
  250   continue
c   -----------------------------------------
c   reset symmetry related lattice parameters
c   -----------------------------------------
      a = clat(1)
      b = clat(2)
      c = clat(3)
      alpha = clat(4)
      beta  = clat(5)
      agamma= clat(6)
      do 120 j=1,6
        if(gcell(j).eq.'F') then
          if(j.eq.2) then
            if(iaxx.eq.1.and.iayx.ne.6) then
              b = c
            else
              b = a
              endif
          else if(j.eq.3) then
            if(iaxx.eq.1) then
              c = b
            else
              c = a
              endif
          else if(j.eq.5.or.j.eq.6) then
            if(iax.eq.3.and.iaxx.eq.6) then
              if(i.eq.5) beta = alpha
              if(i.eq.6) agamma = alpha
              endif
            endif
          endif
  120   continue
      call param
      RETURN
      END
C
c
