c          
c        File W77PLOT                    Written: March, 1993
c                                                 T. Stephen
c        ____________
c
c  This file contains the following low level Dos/W77 routines:
c
c  F  PLOTTEST: shows whether this or dudplot was installed
c  F  VIDEO: attempts to set the monitor to the proper video mode
c  F  SHADE: alters a 256+ colour palette to enable shading effects
c  F  CIRCLE: draws a circle with specified radius, center and color
c  F  LINE: draws a line with specified endpoints and colour
c  F  CHARPLO: puts up graphics text
C  F  GLINT: creates a glint for 3-D (sorta) effect
c  F  CLEARSCREEN: clears a graphics screen
c  F  BLUETORED: sets extended palette to a blue to red setup
c  F  BOX: prints a rectangle
c  F  SIXTEENC: sets a nicer short palette
c  F  BACKGROUND: sets the background colour
c
c  You must have the Watcom Fortran 77 graphics library to compile this!
c  If not, compile DUDPLOT.FOR instead of this file!!!
c  DO NOT include both this and DUDPLOT.OBJ in your linking list!!
c
c
      FUNCTION PLOTTEST(iswitch)
c
c Tells computer whether this plotting library has been installed
c input iswitch 0 do not print test line
c       iswitch 1 print test line
c output plottest 0 this was called from DUDPLOT.FOR
c        plottest 1 this was called from this file
c
      if (iswitch.eq.1) write(6,1000)
1000  format(9x,'*** W77 VGA and SVGA routines installed ***')
      plottest=1
      return
      end
c
      FUNCTION VIDEO(imode)
c
c  Input IVGA=desired video mode
c  0- Return to defult mode
c  1- 16 colour, 640X480 Resolution- For standard VGA
c  2- 256 colour, 640X480 Resolution- Unsure of constraints
c  3- 256 colour, 1024X768 Resolution- Super VGA
c  
c    uses graphics library _setvideomode
c    + graphics constants _vres16color, _vres256color,_xres256color
c                         _defaultmode
c
      include 'graph.fi'
      include 'graphapi.fi'
      ierror=0
      if (imode.eq.1) then
	ierror=_setvideomode(_vres16color)
      else if (imode.eq.2) then
	ierror=_setvideomode(_vres256color)
      else if (imode.eq.3) then
	ierror=_setvideomode(_xres256color)
      else
	ierror=_setvideomode(_defaultmode)
      endif
      video=ierror
      return
      end
c
      function SHADE(itnfl)
c
c         This mess assigns a fading array of colours where 256 colour
c         palettes are available.  #16-31 are shades of colour 1,
c         #32-47 are shades of colour 2, etc.
c
c   uses graphics library _remappalette
c
      include 'graph.fi'
      include 'graphapi.fi'
      dimension irainbow(0:15)
      data irainbow/0,63,5431,203063,467007,864319,1392447,
     +     1195824,999192,12800,1387008,2757888,4128768,4128810,
     +     4128831,4144959/
c
      do 4012 i=1,15
	do 4012 j=0,15
	  ii=irainbow(i)
	  ii1=mod(ii,256)
	  ii=ii-ii1
	  ii2=mod(ii,65536)
	  ii3=ii-ii2
	  if (ii1.gt.1) ii1=ii1-j
	  if (ii2.gt.256) ii2=ii2-256*j
	  if (ii3.gt.65536) ii3=ii3-65536*j
	  if (ii1.lt.8) ii1=8
	  if (ii2.lt.2048) ii2=2048
	  if (ii3.lt.524288) ii3=524288
c             ii=ii3-mod(ii3,65536)+ii2-mod(ii2,256)+ii1
	  ii=ii3+ii2+ii1
	  if (itnfl.eq.1) then
	    ii=_remappalette(16*i+j,ii)
	  else
	    ii=_remappalette(16*i+15-j,ii)
	  endif
4012  continue
      shade=itnfl
      return
      end

      FUNCTION CIRCLE(ifill,icox,icoy,irad,icol)
c
c This draws a circle with:
c   Centre: (icox,icoy) in pixel co-ordinates
c   Radius: irad pixels
c   Colour: icol, depends on palette
c   Filled/not filled: 1=fill, 0=don't
c
c Uses graphics library _setcolor, _ellipse 
c      and constants _gfillinterior, _gborder
c
      include 'graph.fi'
      include 'graphapi.fi'
      call _setcolor(icol)
      ii=0
      if (ifill.eq.1) then
	ii=_ellipse(_gfillinterior,icox-irad,icoy-irad,
     +   icox+irad,icoy+irad)
      else
	ii=_ellipse(_gborder,icox-irad,icoy-irad,icox+irad,icoy+irad)
      endif
      if (ii.eq.0) then
	circle=0
      else
	circle=1
      endif
      return
      end
c
      Function LINE(ix1,iy1,ix2,iy2,icol)
c
c     This function attempts to draw a line
c     From (ix1,iy1) to (ix2,iy2) in colour icol
c
c  uses graphics library _setcolor, _moveto, _lineto
c
      include 'graph.fi'
      include 'graphapi.fi'
      call _setcolor(icol)
      call _moveto(ix1,iy1)
      call _lineto(ix2,iy2)
      line=1
      return
      end

      Function CHARPLO(is1,is2,ix1,ix2,ilen,gstring,icol)
c
c     This function attempts to draw the graphics string hstring
c     of character length ilen
c     of size (is1,is2) in pixels (along (x,y))
c     with top left corner at (ix1,ix2)
c     in colour icol.
c
c uses graphics library _setcharsize, _setcolor, _grtext
c
      implicit character*8 (g)
      include 'graph.fi'
      include 'graphapi.fi'
      call _setcharsize(is1,is2)       
      call _setcolor(icol)
      charplo=_grtext(ix1,ix2,gstring(1:ilen)//char(0))
      return
      end

      Function GLINT(ixpt,iypt,ilng,icol)
c
c     creates the little whit spot that is supposed to make tinted
c     balls look 3-D
c
c  uses graphics library _setcolor, _pie
c
      include 'graph.fi'
      include 'graphapi.fi'
      i=_setcolor(icol)
      i=_pie(3,ixpt-ilng,iypt-ilng,ixpt+ilng,iypt+ilng,
     +  ixpt+3*ilng/4,iypt-ilng/4,ixpt+ilng/4,iypt-3*ilng/4)
      glint=i
      return
      end
c
      Function CLEARSCREEN(a)      
c
c     clears a graphics screen
c
c  uses graphics library _clearscreen & g.l. constant _gclearscreen
c
      include 'graph.fi'
      include 'graphapi.fi'
      call _clearscreen(_gclearscreen)
      clearscreen=a
      return
      end
c
      Function BLUETORED(a)
c
c     Sets extended palette to go from blue to red through white
c     Uses colours 16 through 143
c
c uses graphics library _remappalette
c
      include 'graph.fi'
      include 'graphapi.fi'
      do 10 i=0,63
	iblue=2*i-63
	ired=i
	igreen=4*i-189
	if (iblue.le.0) iblue=0
	if (igreen.le.0) igreen=0
	ii=_remappalette(i+16,256*256*iblue+256*igreen+ired)
 10   continue
      do 20 i=0,63
	iblue=63-i
	ired=63-2*i
	igreen=63-4*i
	if (ired.le.0) ired=0
	if (igreen.le.0) igreen=0
	ii=_remappalette(i+80,256*256*iblue+256*igreen+ired)
 20   continue
      bluetored=a
      return
      end

      Function BOX(ifill,ix1,iy1,ix2,iy2,icol)
c
c  This outputs a rectangle on a graphics screen with
c   upper left corner (ix1,iy1)
c   upper right corner (ix2,iy2)
c   colour icol
c   filled if ifill=1
c
c  uses graphics library _setcolor, _rectangle, _moveto, _lineto
c   and constants _gborder, _gfillinterior
c
      include 'graph.fi'
      include 'graphapi.fi'
c
      if (ifill.eq.1) then 
	ifill=_gfillinterior
      else
	ifill=_gborder
      endif
      k=_setcolor(icol)
      if (ix1.ne.ix2.and.iy1.ne.iy2) then
	box=_rectangle(ifill,ix1,iy1,ix2,iy2)
      else
	call _moveto(x1,y1)
	call _lineto(x2,y2)
	box=1
      end if
      return
      end
c
      Function SIXTEENC(a)
c
c     Sets the colours 0-13 as Black to White to Blue
c     leaves 14 yellow, makes 15 green/red
c
c  uses graphics library _remappalette
c
      include 'graph.fi'
      include 'graphapi.fi'
c
      do 10 i=0,6
	k=i*657930
10      ii=_remappalette(i,k)
      do 20 j=0,6
	k=4128768+2570*j
20      ii=_remappalette(13-j,k)
      ii=_remappalette(15,4144896)
      sixteenc=a
      return
      end
c
      Function BACKGROUND(icol)
c
c     Sets the background colour to icol
c
c     uses graphics library _setbkcolor,_settextcolor
c
      include 'graph.fi'
      include 'graphapi.fi'
      integer colours(0:15) /
     + _black,_blue,_green,_cyan,_red,_magenta,_brown,_white,_gray,
     + _lightblue,_lightgreen,_lightcyan,_lightred,_lightmagenta,
     + _yellow,_brightwhite/
c
      background=_setbkcolor(colours(icol))
      return
      end
